/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg__do_cmd_commit.c
 *
 * @details Handle 'vv commit'.
 *
 */

//////////////////////////////////////////////////////////////////

#include <sg.h>

#include "sg_typedefs.h"
#include "sg_prototypes.h"
#include "sg_cmd_util_prototypes.h"

//////////////////////////////////////////////////////////////////

void do_cmd_commit(SG_context * pCtx,
                   SG_uint32 count_args, const char** paszArgs,
                   const char* psz_comment,
                   const char* psz_user,
                   const char* psz_when,
                   SG_stringarray* psa_stamps,
                   SG_bool bRecursive,
                   const SG_file_spec* pFilespec,
                   const char* const* ppszAssocs, 
				   SG_uint32 nCountAssocs,
				   SG_bool bNoSubmodules,
				   void (*promptfn)(SG_context *, SG_string *,char **, const SG_varray *),
				   SG_string *boilerplate)

{
	SG_pendingtree * pPendingTree = NULL;
	SG_repo * pRepo = NULL;					// we do not own this
    SG_vhash * pvh_pile = NULL;
	char * psz_hid_cset = NULL;
    SG_int64 when = 0;

	if (psz_when)
    {
        SG_bool b_all_digits = SG_TRUE;

        const char* p = psz_when;
        while (*p)
        {
            if ((*p >= '0') && (*p <= '9'))
            {
                p++;
            }
            else
            {
                b_all_digits = SG_FALSE;
                break;
            }
        }

        if (b_all_digits)
        {
            SG_ERR_CHECK(SG_int64__parse__strict(pCtx, &when, psz_when));
        }
        else
        {
            SG_ERR_CHECK(  SG_time__parse__informal__local(pCtx, psz_when, &when, SG_TRUE)  );
        }
    }
    else
    {
        when = SG_AUDIT__WHEN__NOW;
    }

	SG_ERR_CHECK(  SG_vv_verbs__commit(pCtx,
									   NULL,	// assume CWD
									   psz_comment,
									   pFilespec,
									   bRecursive,
									   bNoSubmodules,
									   ((psz_user && *psz_user) ? psz_user : SG_AUDIT__WHO__FROM_SETTINGS),
                                       when,
									   count_args, paszArgs,
									   nCountAssocs, ppszAssocs,
									   psa_stamps,
									   promptfn, boilerplate,
									   &psz_hid_cset)  );

	SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC_FROM_CWD(pCtx, &pPendingTree)  );
    SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );
    SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
    SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, psz_hid_cset, pvh_pile, SG_FALSE, SG_TRUE)  );

fail:
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
    SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_NULLFREE(pCtx, psz_hid_cset);
}
