/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg__do_cmd_export.c
 *
 * @details Handle 'vv export'.
 *
 */

//////////////////////////////////////////////////////////////////

#include <sg.h>

#include "sg_typedefs.h"
#include "sg_prototypes.h"

//////////////////////////////////////////////////////////////////

void do_cmd_export(SG_context * pCtx,
				   const char * pszRepoName,
				   SG_rev_spec* pRevSpec,
				   const char * pszFolder,
				   SG_bool bNoSubmodules,
				   const SG_vector * pvec_use_submodules)
{
	char * psz_hid_cs = NULL;
	SG_vhash * pvhResults = NULL;
	SG_int64 status;

	// convert cset-prefix/rev-nr/tag/explicit-branch/default-branch
	// into the full CSET HID *and* maybe that CSET's current branch.

	SG_ERR_CHECK(  SG_vv_utils__rev_spec_to_csid__no_wd(pCtx, pszRepoName, pRevSpec, &psz_hid_cs, NULL)  );

	// Try to create the WD and populate it.

	SG_ERR_CHECK(  SG_vv_verbs__export(pCtx,
									   pszRepoName, psz_hid_cs, pszFolder,
									   bNoSubmodules,
									   pvec_use_submodules,
									   &pvhResults)  );
	SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pvhResults, "status", &status)  );
	switch (status)
	{
	default:
	case SG_VV_VERBS__STATUS__OK:
		break;

	case SG_VV_VERBS__STATUS__SUBMODULE_PROBLEM:
		{
			SG_varray * pvaMissingDetails;	// we don't own this

			SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "sub", &pvaMissingDetails)  );

			SG_ERR_IGNORE(  sg_report_submodule_missing_details(pCtx, pvaMissingDetails)  );
			// TODO 2011/02/01 Should we have a __NESTED_EXPORT_PROBLEM code?
			SG_ERR_THROW(  SG_ERR_SUBMODULE_NESTED_UPDATE_CHECKOUT_PROBLEM  ); // throw so that exe exits with non-zero status
		}
		break;
	}
	
fail:
	// TODO 2010/10/25 Decide which error message we should try to provide a better error message for.
	// TODO            Candidates include
	// TODO SG_ERR_INVALID_REPO_NAME
	// TODO SG_ERR_AMBIGUOUS_ID_PREFIX
	// TODO SG_ERR_MULTIPLE_HEADS_FROM_DAGNODE
	// TODO
	// TODO            But we should look and see what the lower layers are throwing now.  Some of
	// TODO            of the messages that were here are now being handled.

	SG_NULLFREE(pCtx, psz_hid_cs);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
}
