/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

//////////////////////////////////////////////////////////////////
// Some basic tests for doing arbitrary (unrelated) updates when
// there is ignorable dirt in the working directory.
//////////////////////////////////////////////////////////////////

function st_upate_dirty__W2237()
{
    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    //////////////////////////////////////////////////////////////////

    this.setUp = function()
    {
        this.workdir_root = sg.fs.getcwd();     // save this for later

        //////////////////////////////////////////////////////////////////
        // fetch the HID of the initial commit.

        var h = sg.pending_tree.history();
        this.csets.push( h[0].changeset_id );
        this.names.push( "*Initial Commit*" );

        //////////////////////////////////////////////////////////////////
	// create 3 csets:
	//
	//      0
	//     /
	//    1
	//   /
	//  2
	//

        this.do_fsobj_mkdir("dir_0");
        this.do_addremove_scan();
        this.do_commit("Simple ADDs 0");

        this.do_fsobj_mkdir("dir_1");
        this.do_fsobj_mkdir("dir_1/dir_2");
        this.do_fsobj_mkdir("dir_1/dir_2/dir_3");
        this.do_fsobj_mkdir("dir_1/dir_2/dir_3/dir_4");
        this.do_fsobj_mkdir("dir_1/dir_2/dir_3/dir_4/dir_5");
        this.do_fsobj_mkdir("dir_1/dir_2/dir_3/dir_4/dir_5/dir_6");
        this.do_fsobj_mkdir("dir_1/dir_2/dir_3/dir_4/dir_5/dir_6/dir_7");
	this.do_fsobj_create_file("dir_1/dir_2/dir_3/dir_4/dir_5/dir_6/dir_7/file.txt");
        this.do_addremove_scan();
        this.do_commit("Simple ADDs 1");

        this.do_fsobj_mkdir("dir_2");		// noise to get a new cset
        this.do_addremove_scan();
        this.do_commit("Simple ADDs 2");

	// these should be equivalent
	// this.rev_trunk_head = this.csets[ this.csets.length - 1 ];
	this.rev_trunk_head = sg.pending_tree.parents()[0];

	// update back to '1' and start a new branch '3':
	//      0
	//     /
	//    1
	//   / \
	//  2   3
	//       \
	//        4

	this.do_update_when_clean_by_name("Simple ADDs 1");

        this.do_fsobj_mkdir("dir_3");		// noise to get a new cset
        this.do_addremove_scan();
        this.do_commit_in_branch("Simple ADDs 3");

	this.do_vc_move("dir_1/dir_2/dir_3/dir_4", "dir_1");	// move dir_4 2 levels up
	this.do_addremove_scan();
	this.do_commit_in_branch("Simple ADDs 4");

	// these should be equivalent
	// this.rev_branch_head = this.csets_in_branch[ this.csets_in_branch.length - 1 ];
	this.rev_branch_head = sg.pending_tree.parents()[0];

	var o = sg.exec(vv, "heads");
	print("Heads are:");
	print(o.stdout);
    }

    //////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////

    this.test_1 = function()
    {
	// we are looking at [4] the value in this.rev_branch_head.
	// verify that we can update from [4 ==> 2] and [2 ==> 4].
	// these are unrelated peers (non ancestor/descendant).

	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("");
	print("Verify that we can do CLEAN UPDATE [4 ==> 2]");

	var r42 = sg.vv.update( [ "rev="+this.rev_trunk_head, "verbose" ] );
	testlib.ok( (r42.status == 0), ("Expected update status of 0, received " + r42.status) );
	if (r42.status != 0)
	    throw("Unexpected update status " + r42.status);
	dumpActionLog("UPDATE [4 ==> 2]", r42.log);

        var expect_test_42 = new Array;
        this.confirm_when_ignoring_ignores(expect_test_42);

	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("");
	print("Verify that we can do CLEAN UPDATE [2 ==> 4]");

	var r24 = sg.vv.update( [ "rev="+this.rev_branch_head, "verbose" ] );
	testlib.ok( (r24.status == 0), ("Expected update status of 0, received " + r24.status) );
	if (r24.status != 0)
	    throw("Unexpected update status " + r24.status);
	dumpActionLog("UPDATE [2 ==> 4]", r24.log);

        var expect_test_24 = new Array;
        this.confirm_when_ignoring_ignores(expect_test_24);

	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("");
	print("Create an UNCONTROLLED file in the moved directory.");

	this.do_fsobj_create_file("dir_1/dir_4/dir_5/dir_6/dir_7/uncontrolled.txt");

        var expect_test_4d = new Array;
	expect_test_4d["Found"] = [ "@/dir_1/dir_4/dir_5/dir_6/dir_7/uncontrolled.txt" ];
        this.confirm_when_ignoring_ignores(expect_test_4d);

	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("");
	print("Verify that we can do DIRTY UPDATE [4 ==> 2]");

	var r42 = sg.vv.update( [ "rev="+this.rev_trunk_head, "verbose" ] );
	testlib.ok( (r42.status == 0), ("Expected update status of 0, received " + r42.status) );
	if (r42.status != 0)
	    throw("Unexpected update status " + r42.status);
	dumpActionLog("UPDATE [4 ==> 2]", r42.log);

        var expect_test_42 = new Array;
        this.confirm_when_ignoring_ignores(expect_test_42);

    }
}
