/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

/////////////////////////////////////////////////////////////////////////////////////
// Tests involving merges between changesets with various patterns of path swapping.
// Two of these tests cause merge to hang.  The remainder of the tests pass.
/////////////////////////////////////////////////////////////////////////////////////

var globalThis;  // var to hold "this" pointers

function st_merge_paths() 
{
    // Helper functions are in update_helpers.js
    // There is a reference list at the top of the file.
    // After loading, you must call initialize_update_helpers(this)
    // before you can use the helpers.
    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    this.verbose = true; // add this wherever you want it.  It'll stick through the end of the stGroup.

    //////////////////////////////////////////////////////////////////

    this.setUp = function()
    {
        this.workdir_root = sg.fs.getcwd();     // save this for later

        // we begin with a clean WD in a fresh REPO.
        // and build a sequence of changesets to use

        this.workdir_root = sg.fs.getcwd();        // save this for later

        //////////////////////////////////////////////////////////////////
        // fetch the HID of the initial commit.

        var h = sg.pending_tree.history();
        this.csets.push( h[0].changeset_id );
        this.names.push( "*Initial Commit*" );

        //////////////////////////////////////////////////////////////////
        // simple (3-deep) path swapping setup for merge tests

        this.do_fsobj_mkdir("dir_merge_10");
        this.do_addremove_scan();
        this.do_commit("Merge STEM 10");

        this.do_fsobj_mkdir("dir_merge_10/a");
        this.do_fsobj_mkdir("dir_merge_10/a/b");
        this.do_fsobj_mkdir("dir_merge_10/a/b/c");
        this.do_addremove_scan();
        this.do_commit("Merge HUB 10");

        this.do_vc_move("dir_merge_10/a/b", "dir_merge_10");
        this.do_vc_move("dir_merge_10/a", "dir_merge_10/b");
        this.do_vc_move("dir_merge_10/b/c", "dir_merge_10/b/a");
        this.do_commit("Merge HEAD 1 b/a/c");

        if (!this.do_update_when_clean_by_name("Merge STEM 10"))
            return;
        if (!this.do_update_when_clean_by_name("Merge HUB 10"))
            return;
        this.do_vc_move("dir_merge_10/a/b", "dir_merge_10");
        this.do_vc_move("dir_merge_10/a", "dir_merge_10/b/c");
        this.do_commit("Merge HEAD 2 b/c/a");

        if (!this.do_update_when_clean_by_name("Merge STEM 10"))
            return;
        if (!this.do_update_when_clean_by_name("Merge HUB 10"))
            return;
        this.do_vc_move("dir_merge_10/a/b/c", "dir_merge_10/a");
        this.do_vc_move("dir_merge_10/a/b", "dir_merge_10/a/c");
        this.do_commit("Merge HEAD 3 a/c/b");

        if (!this.do_update_when_clean_by_name("Merge STEM 10"))
            return;
        if (!this.do_update_when_clean_by_name("Merge HUB 10"))
            return;
        this.do_vc_move("dir_merge_10/a/b/c", "dir_merge_10");
        this.do_vc_move("dir_merge_10/a", "dir_merge_10/c");
        this.do_commit("Merge HEAD 4 c/a/b");

        if (!this.do_update_when_clean_by_name("Merge STEM 10"))
            return;
        if (!this.do_update_when_clean_by_name("Merge HUB 10"))
            return;
        this.do_vc_move("dir_merge_10/a/b/c", "dir_merge_10");
        this.do_vc_move("dir_merge_10/a/b", "dir_merge_10/c");
        this.do_vc_move("dir_merge_10/a", "dir_merge_10/c/b");
        this.do_commit("Merge HEAD 5 c/b/a");

        this.list_csets("After setup (from update_helpers.generate_test_changesets())");
                 print("------------------------------------------------------");
    }

    this.do_exec_merge = function(rev, bExpectFail, msgExpected)
    {
	print("vv merge --rev=" + rev);
	var o = sg.exec(vv, "merge", "--rev="+rev, "--verbose");
	
	print("STDOUT:\n" + o.stdout);
	print("STDERR:\n" + o.stderr);

	if (bExpectFail)
	{
	    testlib.ok( o.exit_status == 1, "Exit status should be 1" );
	    testlib.ok( o.stderr.search(msgExpected) >= 0, "Expecting error message [" + msgExpected + "]. Received:\n" + o.stderr);
            return ((o.exit_status == 1) && (o.stderr.search(msgExpected) >= 0));
	}
	else
	{
	    testlib.ok( o.exit_status == 0, "Exit status should be 0" );
            return (o.exit_status == 0);
	}
    }

    this.do_exec_merge_by_name = function(name, bExpectFail, msgExpected)
    {
        var ndx = this.name2ndx[name];
        if (typeof ndx != undefined)
            return this.do_exec_merge(this.csets[ndx], bExpectFail, msgExpected);

        testlib.ok( (0), "Unknown changeset: [" + name + "]" );
        return false;
    }

////////////////////////////////////////////////////////////////////////////

    this.merge_sub = function(baseName, mergeName)
    {
        this.assert_clean_WD();
        this.do_update_when_clean_by_name("Merge STEM 10");
        this.do_update_when_clean_by_name(baseName);
        print('While at changeset "' + baseName + '", merge changeset "' + mergeName + '" ...');
        this.do_exec_merge_by_name(mergeName, 0, ""); 
        this.force_clean_WD();
    }
    
    this.test_merge_1_2 = function test_merge_1_2()
    {
        this.merge_sub("Merge HEAD 1 b/a/c", "Merge HEAD 2 b/c/a");
    }

    this.test_merge_1_3 = function test_merge_1_3()
    {
        this.merge_sub("Merge HEAD 1 b/a/c", "Merge HEAD 3 a/c/b");
    }

    this.test_merge_1_4 = function test_merge_1_4()
    {
        this.merge_sub("Merge HEAD 1 b/a/c", "Merge HEAD 4 c/a/b");
    }

    this.test_merge_1_5 = function test_merge_1_5()
    {
        this.merge_sub("Merge HEAD 1 b/a/c", "Merge HEAD 5 c/b/a");
    }

////////////////////////////////////////////////////////////////////////////

    this.test_merge_2_1 = function test_merge_2_1()
    {
        this.merge_sub("Merge HEAD 2 b/c/a", "Merge HEAD 1 b/a/c");
    }

    this.test_merge_2_3 = function test_merge_2_3()
    {
        this.merge_sub("Merge HEAD 2 b/c/a", "Merge HEAD 3 a/c/b");
    }

    this.test_merge_2_4 = function test_merge_2_4()
    {
        this.merge_sub("Merge HEAD 2 b/c/a", "Merge HEAD 4 c/a/b");
    }

    this.test_merge_2_5 = function test_merge_2_5()
    {
        this.merge_sub("Merge HEAD 2 b/c/a", "Merge HEAD 5 c/b/a");
    }

////////////////////////////////////////////////////////////////////////////

    this.test_merge_3_1 = function test_merge_3_1()
    {
        this.merge_sub("Merge HEAD 3 a/c/b", "Merge HEAD 1 b/a/c");
    }

/*************************************************
 ***  THIS TEST CAUSES MERGE TO HANG (1 of 2)  ***
 *************************************************/
    this.test_merge_3_2 = function test_merge_3_2()
    {
        print("##########################################");
        print("### test_merge_3_2 is expected to hang ...");
        print("##########################################");
        this.merge_sub("Merge HEAD 3 a/c/b", "Merge HEAD 2 b/c/a");
    }

/*************************************************
 ***  THIS TEST CAUSES MERGE TO HANG (2 of 2)  ***
 *************************************************/
    this.test_merge_3_4 = function test_merge_3_4()
    {
        print("##########################################");
        print("### test_merge_3_4 is expected to hang ...");
        print("##########################################");
        this.merge_sub("Merge HEAD 3 a/c/b", "Merge HEAD 4 c/a/b");
    }

    this.test_merge_3_5 = function test_merge_3_5()
    {
        this.merge_sub("Merge HEAD 3 a/c/b", "Merge HEAD 5 c/b/a");
    }

////////////////////////////////////////////////////////////////////////////

    this.test_merge_4_1 = function test_merge_4_1()
    {
        this.merge_sub("Merge HEAD 4 c/a/b", "Merge HEAD 1 b/a/c");
    }

    this.test_merge_4_2 = function test_merge_4_2()
    {
        this.merge_sub("Merge HEAD 4 c/a/b", "Merge HEAD 2 b/c/a");
    }

    this.test_merge_4_3 = function test_merge_4_3()
    {
        this.merge_sub("Merge HEAD 4 c/a/b", "Merge HEAD 3 a/c/b");
    }

    this.test_merge_4_5 = function test_merge_4_5()
    {
       this.merge_sub("Merge HEAD 4 c/a/b", "Merge HEAD 5 c/b/a");
    }

////////////////////////////////////////////////////////////////////////////

    this.test_merge_5_1 = function test_merge_5_1()
    {
        this.merge_sub("Merge HEAD 5 c/b/a", "Merge HEAD 1 b/a/c");
    }

    this.test_merge_5_2 = function test_merge_5_2()
    {
        this.merge_sub("Merge HEAD 5 c/b/a", "Merge HEAD 2 b/c/a");
    }

    this.test_merge_5_3 = function test_merge_5_3()
    {
        this.merge_sub("Merge HEAD 5 c/b/a", "Merge HEAD 3 a/c/b");
    }

    this.test_merge_5_4 = function test_merge_5_4()
    {
        this.merge_sub("Merge HEAD 5 c/b/a", "Merge HEAD 4 c/a/b");
    }

    this.tearDown = function() 
    {

    }
}
