/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

//////////////////////////////////////////////////////////////////
// Explore SG_file_spec and specifically the __load_reserved() stuff.
//////////////////////////////////////////////////////////////////

function st_file_spec_reserved()
{
    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    //////////////////////////////////////////////////////////////////

    this.bad_add__not_allowed = function(path)
    {
	print("");
	print("================");
	print("Trying to add: " + path);
	print("================");
	print("");

	var errmsgs = new Array();
	errmsgs["SG_ERR_NOT_ALLOWED"] = "Error 109 ";

	try
	{
	    this.do_add( path );
	    testlib.ok( (0), "Operation succeeded, but was expecting error on 'ADD " + path + "'" );
	}
	catch (e)
	{
	    print(dumpObj(e, "Expect hard error.", "", 0));
	    testlib.ok( (e.name == "Error"),
			"Expect not allowed." );
	    testlib.ok( (e.message.search(errmsgs["SG_ERR_NOT_ALLOWED"]) >= 0),
			"Expect not allowed." );
	    testlib.ok( (e.message.search(path) >= 0),
			"Expect not allowed." );
	}
    }

    this.bad_add__not_found = function(path)
    {
	print("");
	print("================");
	print("Trying to add: " + path);
	print("================");
	print("");

	var errmsgs = new Array();
	errmsgs["SG_ERR_NOT_FOUND"] = "Error 62 ";

	try
	{
	    this.do_add( path );
	    testlib.ok( (0), "Operation succeeded, but was expecting error on 'ADD " + path + "'" );
	}
	catch (e)
	{
	    print(dumpObj(e, "Expect hard error.", "", 0));
	    testlib.ok( (e.name == "Error"),
			"Expect not found." );
	    testlib.ok( (e.message.search(errmsgs["SG_ERR_NOT_FOUND"]) >= 0),
			"Expect not found." );
	}
    }

    this.bad_move__not_allowed = function(path, dest)
    {
	print("");
	print("================");
	print("Trying to move '" + path + "' to '" + dest + "'");
	print("================");
	print("");

	var errmsgs = new Array();
	errmsgs["SG_ERR_NOT_ALLOWED"] = "Error 109 ";

	try
	{
	    this.do_vc_move( path, dest );
	    testlib.ok( (0), "Operation succeeded, but was expecting error on 'MOVE " + path + " to " + dest + "'" );
	}
	catch (e)
	{
	    print(dumpObj(e, "Expect hard error.", "", 0));
	    testlib.ok( (e.name == "Error"),
			"Expect not allowed." );
	    testlib.ok( (e.message.search(errmsgs["SG_ERR_NOT_ALLOWED"]) >= 0),
			"Expect not allowed." );
	    testlib.ok( (e.message.search(path) >= 0),
			"Expect not allowed." );
	}
    }

    this.bad_move__not_found = function(path, dest)
    {
	print("");
	print("================");
	print("Trying to move '" + path + "' to '" + dest + "'");
	print("================");
	print("");

	var errmsgs = new Array();
	errmsgs["SG_ERR_NOT_FOUND"] = "Error 62 ";

	try
	{
	    this.do_vc_move( path, dest );
	    testlib.ok( (0), "Operation succeeded, but was expecting error on 'MOVE " + path + " to " + dest + "'" );
	}
	catch (e)
	{
	    print(dumpObj(e, "Expect hard error.", "", 0));
	    testlib.ok( (e.name == "Error"),
			"Expect not found." );
	    testlib.ok( (e.message.search(errmsgs["SG_ERR_NOT_FOUND"]) >= 0),
			"Expect not found." );
	}
    }

    //////////////////////////////////////////////////////////////////

    this.test_1 = function()
    {
	print("");
	print("================================================================");
	print("Begin test");
	print("================================================================");
	print("");

        this.workdir_root = sg.fs.getcwd();     // save this for later

        //////////////////////////////////////////////////////////////////
        // fetch the HID of the initial commit.

        var h = sg.pending_tree.history();
        this.csets.push( h[0].changeset_id );
        this.names.push( "*Initial Commit*" );

        //////////////////////////////////////////////////////////////////
	
        this.do_fsobj_mkdir("dir_1");
        this.do_fsobj_mkdir("dir_2");
        this.do_fsobj_mkdir("dir_3");
        this.do_fsobj_mkdir("dir_4");
        this.do_fsobj_create_file("dir_1/common.txt");
        this.do_fsobj_create_file("dir_2/common.txt");
        this.do_addremove_scan();

	this.do_fsobj_create_file("dir_1/another.txt");
	this.do_add("dir_1/another.txt");

	// test both ".sgdrawer" and ".sgdrawer/" as a target because
	// there is code somewhere (sg_getopt) to strip out an exact
	// match but does not catch the other.

	this.bad_add__not_allowed( ".sgdrawer" );
	this.bad_add__not_allowed( ".sgdrawer/" );
	this.bad_add__not_allowed( ".sgdrawer/branch.json" );

	this.bad_add__not_found( ".sgdrawer/foo" );

	this.bad_move__not_allowed( "dir_1", ".sgdrawer/" );
	this.do_fsobj_mkdir(".sgdrawer/test_temp_dir");
	this.bad_move__not_allowed( "dir_2", ".sgdrawer/test_temp_dir" );

	if (sg.platform() == "LINUX")
	{
	    // TODO 2011/07/15 Need a set of tests for Linux that tries to
	    // TODO            abuse the fact that .SGDRAWER could be created
	    // TODO            in root beside .sgdrawer.  (Such as adding .sgdrawer
	    // TODO            to the portability collider for top-level entrynames.)
	    // TODO            See W7461.
	}
	else
	{
	    // verify that the code to prevents .sgdrawer from being
	    // (explicitly or implicitly) added isn't fooled by case.

	    this.bad_add__not_found( ".SGDRAWER/" );
	    this.bad_add__not_found( ".SGDRAWER/foo" );

	    this.bad_move__not_found( "dir_3", ".SGDRAWER/" );
	}

	// cd into .sgdrawer and try to add things using relative paths
	// bypassing the various arg checks.

	this.do_fsobj_cd( ".sgdrawer" );
	this.bad_add__not_allowed( "branch.json" );
	this.bad_move__not_allowed( "../dir_4", "." );

    }
}
