/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

function st_jsglue()
{
	
	this.testOpenRepoSanityCheck = function()
	{
		var caught = false;
		try
		{
			var repo = sg.open_repo(null);
		}
		catch (e)
		{
			caught = true;
		}
		finally
		{
			testlib.testResult(caught, "Open repo throws JS exception");
		}
	};
	
	this.testCreateZingDBSanityCheck = function()
	{
		var caught = false;
		try
		{
			var db = new zingdb(null, sg.dagnum.WORK_ITEMS);
		}
		catch(e)
		{
			caught = true;
		}
		finally
		{
			testlib.testResult(caught, "zingdb constructor thorws JS exception on null repo");
		}
		
		var caught = false;
		try
		{
			var repo = sg.open_repo(repInfo.repoName);
			var db = new zingdb(repo, null);
		}
		catch(e)
		{
			caught = true;
		}
		finally
		{
			repo.close();
			testlib.testResult(caught, "zingdb constructor throws JS exception on null dagnum");
		}
		
	};
	
	this.testUpdateZingTemplateSanityCheck = function()
	{
		var caught = false;
		try
		{
			var repo = sg.open_repo(repInfo.repoName);
			var db = new zingdb(repo, sg.dagnum.WORK_ITEMS);
			db.set_template(null);
		}
		catch(e)
		{
			caught = true;
		}
		finally
		{
			repo.close();
			testlib.testResult(caught, "set_template throws JS exception on null template");
		}
	};
	
	this.testZingNewRecordSanityCheck = function()
	{
		var caught = false;
		try
		{
			var repo = sg.open_repo(repInfo.repoName);
			var db = new zingdb(repo, sg.dagnum.WORK_ITEMS);
			var tx = db.begin_tx();
			tx.new_record(null);
		}
		catch(e)
		{
			caught = true;
		}
		finally
		{
			tx.abort();
			repo.close();
			testlib.testResult(caught, "new_record throws JS exception on null rectype");
		}
		
		var caught = false;
		try
		{
			var repo = sg.open_repo(repInfo.repoName);
			var db = new zingdb(repo, sg.dagnum.WORK_ITEMS);
			var tx = db.begin_tx();
			tx.new_record("item", null);
		}
		catch(e)
		{
			caught = true;
		}
		finally
		{
			tx.abort();
			repo.close();
			testlib.testResult(caught, "new_record throws JS exception on null record");
		}
	};
	
	this.testSGHashSanityCheck = function()
	{
		var caught = false;
		
		try
		{
			sg.hash(null);
		}
		catch (e)
		{
			caught = true;
		}
		finally
		{
			testlib.testResult(caught, "sg.hash throws JS exception on null argument");
		}
	};
	
	this.testSGCloneAllFullSanityCheck = function()
	{
		var caught = false;
		
		try
		{
			sg.clone__all_full(null);
		}
		catch (e)
		{
			caught = true;
		}
		finally
		{
			testlib.testResult(caught, "sg.clone__all_full throws JS exception on null argument");
		}
	};
	
	this.testSGCloneAllZlibSanityCheck = function()
	{
		var caught = false;
		
		try
		{
			sg.clone__all_zlib(null);
		}
		catch (e)
		{
			caught = true;
		}
		finally
		{
			testlib.testResult(caught, "sg.clone__all_full throws JS exception on null argument");
		}
	}
}