/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

//////////////////////////////////////////////////////////////////

function st_portability_06()
{
    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    // this.verbose = true; // add this wherever you want it.  It'll stick through the end of the stGroup.

    //////////////////////////////////////////////////////////////////

    var errmsgs = new Array();
    errmsgs["SG_ERR_PENDINGTREE_PARTIAL_CHANGE_COLLISION"] = "Error 123 ";

    this.test_0 = function()
    {
	this.do_fsobj_mkdir("DIR");
	this.do_fsobj_mkdir("AAA");
	this.do_addremove_scan();
	this.do_commit("Added AAA");

	this.do_vc_move("AAA", "DIR");
	this.do_fsobj_mkdir("AAA");
	this.do_addremove_scan();

	var expect_test = new Array;
	expect_test["Moved"] = [ "@/DIR/AAA" ];
	expect_test["Added"] = [ "@/AAA" ];
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	// a PARTIAL REVERT on "DIR/AAA" ("DIR/AAA" ==> "AAA") will
	// collide with the new "AAA" on all platforms.

	print("");
	print("================================================================");
	print("Trying partial revert...");
	print("");
	try
	{
	    var r = sg.vv.revert( [ "verbose" ], "DIR/AAA" );
	    print(dumpObj(r, "Expect hard collisions.", "", 0));
	    testlib.ok( (0),
			"Expect hard collisions." );
	}
	catch (e)
	{
	    print(dumpObj(e, "Expect hard error.", "", 0));
	    testlib.ok( (e.name == "Error"),
			"Expect hard collisions." );
	    testlib.ok( (e.message.search(errmsgs["SG_ERR_PENDINGTREE_PARTIAL_CHANGE_COLLISION"]) >= 0),
			"Expect hard collisions." );
	}

	// nothing should have changed on disk

	print("Verify failed partial revert did not alter working directory...");
        this.confirm_when_ignoring_ignores(expect_test);

	print("");
	print("================================================================");
	print("Trying revert --all...");
	print("");
	var r = sg.vv.revert( [ "verbose" ] );
	print(dumpObj(r, "Full revert should succeed.", "", 0));
	testlib.ok( (r.status == 0), "Full revert should succeed." );

	var expect_test = new Array;
	expect_test["Found"] = [ "@/AAA~found00" ];
        this.confirm_when_ignoring_ignores(expect_test);
    }

    this.test_1 = function()
    {
	this.do_fsobj_mkdir("BBB");
	this.do_addremove_scan();
	this.do_commit("Added BBB");

	this.do_vc_move("BBB", "DIR");
	this.do_fsobj_mkdir("bbb");
	this.do_addremove_scan();

	var expect_test = new Array;
	expect_test["Moved"] = [ "@/DIR/BBB" ];
	expect_test["Added"] = [ "@/bbb" ];
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	// a PARTIAL REVERT on "DIR/BBB" ("DIR/BBB" ==> "BBB") will
	// collide with the new "bbb" on Mac/Windows.  so we
	// should get a portability warning on all platforms.

	print("");
	print("================================================================");
	print("Trying partial revert...");
	print("");
	var r = sg.vv.revert( [ "verbose" ], "DIR/BBB" );
	print(dumpObj(r, "Expect portability warning.", "", 0));
	var SG_VV_VERBS__STATUS__PORTABILITY_WARNING = 1;
	testlib.ok( (r.status == SG_VV_VERBS__STATUS__PORTABILITY_WARNING), "Expect portability warning." );
	testlib.ok( (r.port[0].type == "collision"), "Expect portability warning." );
	testlib.ok( (r.port[0].key == "bbb"), "Expect portability warning." );

	// nothing should have changed on disk
        this.confirm_when_ignoring_ignores(expect_test);

	print("");
	print("================================================================");
	print("Trying revert --all...");
	print("");
	var r = sg.vv.revert( [ "verbose" ] );
	print(dumpObj(r, "Full revert should succeed.", "", 0));
	testlib.ok( (r.status == 0), "Full revert should succeed." );

	var expect_test = new Array;
	expect_test["Found"] = [ "@/bbb~found00" ];
        this.confirm_when_ignoring_ignores(expect_test);

    }

}
