/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

//////////////////////////////////////////////////////////////////
// Tests REMVOE using --exclude
//////////////////////////////////////////////////////////////////

function st_remove_modified_force__K8955()
{
    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    load("update_submodule_helpers.js");
    initialize_update_submodule_helpers(this);

    //////////////////////////////////////////////////////////////////

    this.test_1 = function()
    {
	this.force_clean_WD();

        this.do_fsobj_create_file("file_1");
        this.do_fsobj_create_file("file_2");
        this.do_fsobj_create_file("file_3");
        this.do_fsobj_create_file("file_4");
        this.do_fsobj_create_file("file_5");
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	{
	    this.do_make_symlink("file_1", "link_1");
	}
	this.do_addremove_scan();
	this.do_commit("add initial files");
	var cset_base = sg.pending_tree.parents()[0];

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Make some dirt....");
	print("");

	this.do_fsobj_append_file("file_3", "Hello!");
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	{
	    // retarget the symlink
	    this.do_make_symlink("file_2", "link_1");
	}

        var expect_test = new Array;
	expect_test["Modified"] = [ "@/file_3" ];
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	    expect_test["Modified"].push( "@/link_1" );
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Verify that normal 'vv remove' fails with unsafe...");
	print("");

	var result = sg.exec(vv, "remove", "file_3");
	print(dumpObj(result, "sg.exec()", "", 0));
	testlib.ok( (result.exit_status != 0), "'vv remove file_3' should fail" );
	testlib.ok( (result.stderr.indexOf("The object cannot be safely removed") != -1), "Expect failure for file_3" );

	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	{
	    var result = sg.exec(vv, "remove", "link_1");
	    print(dumpObj(result, "sg.exec()", "", 0));
	    testlib.ok( (result.exit_status != 0), "'vv remove link_1' should fail" );
	    testlib.ok( (result.stderr.indexOf("The object cannot be safely removed") != -1), "Expect failure for link_1" );
	}

	// nothing should have changed

        var expect_test = new Array;
	expect_test["Modified"] = [ "@/file_3" ];
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	    expect_test["Modified"].push( "@/link_1" );
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Force the removes with -F...");
	print("");

	var result = sg.exec(vv, "remove", "-F", "file_3");
	print(dumpObj(result, "sg.exec()", "", 0));
	testlib.ok( (result.exit_status == 0), "'vv remove -F file_3' should work" );

	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	{
	    var result = sg.exec(vv, "remove", "-F", "link_1");
	    print(dumpObj(result, "sg.exec()", "", 0));
	    testlib.ok( (result.exit_status == 0), "'vv remove -F link_1' should work" );
	}

	// with V00115 'vv rm -F' creates backups for dirty files/symlinks

        var expect_test = new Array;
	expect_test["Removed"] = [ "@/file_3" ];
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	    expect_test["Removed"].push( "@/link_1" );
	expect_test["Found"] = [ "@/file_3~sg00~" ];
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	    expect_test["Found"].push( "@/link_1~sg00~" );
        this.confirm_when_ignoring_ignores(expect_test);

	// commit this so that we get a clean status for subsequent tests

	this.do_fsobj_remove_file("file_3~sg00~");
	if ((sg.platform() == "LINUX") || (sg.platform() == "MAC"))
	    this.do_fsobj_remove_file( "link_1~sg00~" );
	this.do_commit("commit changes to file_3");

        var expect_test = new Array;
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Verify that normal JS remove fails with unsafe...");
	print("");

	// make file_4 dirty and try to REMOVE it.

	this.do_fsobj_append_file("file_4", "Hello!");

	var expect_msg = "The object cannot be safely removed";
	try
	{
	    sg.pending_tree.remove( "file_4" );
	    testlib.ok( (false), "REMOVE should have thrown error: '" + expect_msg + "'" );
	}
	catch (e)
	{
	    this.my_verify_expected_throw(e, expect_msg);
	}

	// nothing should have changed

        var expect_test = new Array;
	expect_test["Modified"] = [ "@/file_4" ];
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Force the JS removes with -F...");
	print("");

	sg.pending_tree.remove( [ "force" ], "file_4" );

	// with V00115 'vv rm -F' creates backups for dirty files/symlinks

        var expect_test = new Array;
	expect_test["Removed"] = [ "@/file_4" ];
	expect_test["Found"] = [ "@/file_4~sg00~" ];
        this.confirm_when_ignoring_ignores(expect_test);

	// commit this so that we get a clean status for subsequent tests

	this.do_fsobj_remove_file("file_4~sg00~");
	this.do_commit("commit changes to file_4");

        var expect_test = new Array;
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Verify that normal JS remove fails with unsafe...");
	print("");

	// make file_5 dirty and try to REMOVE it.

	this.do_fsobj_append_file("file_5", "Hello!");

	var expect_msg = "The object cannot be safely removed";
	try
	{
	    sg.pending_tree.remove( "file_5" );
	    testlib.ok( (false), "REMOVE should have thrown error: '" + expect_msg + "'" );
	}
	catch (e)
	{
	    this.my_verify_expected_throw(e, expect_msg);
	}

	// nothing should have changed

        var expect_test = new Array;
	expect_test["Modified"] = [ "@/file_5" ];
        this.confirm_when_ignoring_ignores(expect_test);

	//////////////////////////////////////////////////////////////////
	print("");
	print("//////////////////////////////////////////////////////////////////");
	print("Force the JS removes with 'force' and 'no-backups' ...");
	print("");

	sg.pending_tree.remove( [ "force", "no-backups" ], "file_5" );

	// with V00115 'vv rm -F' creates backups for dirty files/symlinks
	// with W6069  'vv rm -F --no-backups' doesn't

        var expect_test = new Array;
	expect_test["Removed"] = [ "@/file_5" ];
        this.confirm_when_ignoring_ignores(expect_test);

    }

}
