/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

function st_revert_nobackup() {

    load("update_helpers.js");          // load the helper functions
    initialize_update_helpers(this);    // initialize helper functions

    //////////////////////////////////////////////////////////////////

    this.testEditFileRevert = function testEditFileRevert() {

        var base = 'testEditFileRevert';
        var folder = createFolderOnDisk(base);
        var file = createFileOnDisk(pathCombine(folder, "file1.txt"), 39);
        addRemoveAndCommit();
        testlib.inRepo(folder);
        testlib.inRepo(file);

        var tmpFile = createTempFile(file);

        insertInFile(file, 20);

	this.vv_revert_nobackup_verbose( base );

        var backup = file + "~sg00~";

        // testlib.statusEmpty();
        testlib.ok(compareFiles(tmpFile, file), "Files should be identical after the revert");
        testlib.notOnDisk(backup);
        sg.fs.remove(tmpFile);
        testlib.statusEmpty();

    }

    this.testEditMoveRevert = function testEditMoveRevert() {
        //edit a file and move the same file then revert.
        var base = 'testEditMoveRevert';
        var folder = createFolderOnDisk(base + "1");
        var folder2 = createFolderOnDisk(base + "2", 3);
        var file = createFileOnDisk(pathCombine(folder, "file1.txt"), 120);

        addRemoveAndCommit();

        testlib.inRepo(folder);
        testlib.inRepo(folder2);
        testlib.inRepo(file);

        var filev1 = createTempFile(file);

        doRandomEditToFile(file);
	testlib.ok((compareFiles(file,filev1) == false), "doRandomEditToFile modified file.");

        sg.pending_tree.move(file, folder2);
        testlib.notOnDisk(file);
        testlib.existsOnDisk(pathCombine(folder2, "file1.txt"));

	this.vv_revert_nobackup_verbose( base );

        var backup = file + "~sg00~";

        // testlib.statusEmpty();
        testlib.existsOnDisk(file);
        testlib.notOnDisk(pathCombine(folder2, "file1.txt"));
        testlib.ok(compareFiles(filev1, file), "Files should be identical after the revert");
        testlib.notOnDisk(backup);
        sg.fs.remove(filev1);
        testlib.statusEmpty();
    }

    this.testEditRenameRevert = function testEditRenameRevert() {
        //edit a file and move the same file then revert.
        var base = 'testEditRenameRevert';
        var folder = createFolderOnDisk(base);
        var file = createFileOnDisk(pathCombine(folder, "file1.txt"), 120);

        addRemoveAndCommit();

        testlib.inRepo(folder);
        testlib.inRepo(file);

        var filev1 = createTempFile(file);

        doRandomEditToFile(file);
	testlib.ok((compareFiles(file,filev1) == false), "doRandomEditToFile modified file.");

        sg.pending_tree.rename(file, "renamed_file1.txt");

        testlib.existsOnDisk(pathCombine(folder, "renamed_file1.txt"));
        testlib.notOnDisk(file);

	this.vv_revert_nobackup_verbose( base );

        var backup = file + "~sg00~";

        // testlib.statusEmpty();
        testlib.existsOnDisk(file);
        testlib.notOnDisk(pathCombine(folder, "renamed_file1.txt"));
        testlib.ok(compareFiles(filev1, file), "Files should be identical after the revert");
        testlib.notOnDisk(backup);
        sg.fs.remove(filev1);
        testlib.statusEmpty();
    }

    this.testDeleteEditRevert = function testDeleteEditRevert() {
        // delete a file and edit the same file then revert 
	//
	// TODO 2011/04/18 The above comment is a little misleading.
	// TODO            What we do here is pend the DELETE of a file
	// TODO            and then pend an ADD a new file with the same 
	// TODO            name and MODIFY it (unnecessary because it is
	// TODO            new).  And then REVERT.
	//
	// The revert finds a name collision -- the revert of the ADDED 
	// file should just leave it in the directory (with FOUND status)
	// and the revert of the DELETED file should re-create it using
	// the contents stored in the repo.
	//
	// Because of the name collision -- move the found one to a temporary
	// name and let the restored version have the file name so that the
	// WD is in sync.
	//
	// Because this type of temp name is not a backup of the original
	// file, I've changed the ~suffix~.

        var i = 0;
        var base = 'testDeleteEditRevert';
        var folder = createFolderOnDisk(base);
        var file = createFileOnDisk(pathCombine(folder, "file.txt"), 10);
        addRemoveAndCommit();

        var filev1 = createTempFile(file);
        sg.pending_tree.remove(file);
        testlib.notOnDisk(file);

        copyFileOnDisk(filev1, file);
        testlib.existsOnDisk(file);
        doRandomEditToFile(file);
	testlib.ok((compareFiles(file,filev1) == false), "doRandomEditToFile modified file.");
        var filev2 = createTempFile(file);
        reportTreeObjectStatus(file);
        sg.pending_tree.addremove();
        reportTreeObjectStatus(file);

	this.vv_revert_verbose( base );

        var backup = file + "~found00";
        testlib.existsOnDisk(file);
        testlib.ok(compareFiles(file, filev1), "Files should be identical after the revert");
        testlib.existsOnDisk(backup);
        testlib.ok(compareFiles(backup, filev2), "Backup file should match edited file");
        sg.fs.remove(filev1);
        sg.fs.remove(filev2);
        sg.fs.remove(backup);
        testlib.statusEmpty();
    }

}
