/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

// for JSON stringifying.  I know sglib has this, but this is the version
// used client-side, so it's available here for consistency
load("json2.js");

function suite_web_dispatch(baseUrl, serverHasWorkingFolder) {
	
	this._testRepos = [];
	
	this.suite_setUp = function() {
		{
			var repo = sg.open_repo(repInfo.repoName);
		
			{
				var zs = new zingdb(repo, sg.dagnum.USERS);
				var urec = zs.query('user', ['*'], "name == '" + repInfo.userName + "'");

				this.userId = urec[0].recid;
			}

			repo.set_user(repInfo.userName);

			repo.close();
			
			var repos = [
				"www.veracity.com",
				"repo.json",
				"veracity"
			];
			
			for (repo in this._testRepos)
			{
				rInfo = createNewRepo(this._testRepos[repo]);
				
				this._testRepos.push(rInfo.repoName);
			}
		}
	}
	
	this.verifyNot404 = function(path, addJsonHeader) {
		var url = baseUrl + path;
		var o;
		if (addJsonHeader) {
			o = sg.exec("curl", "-D", "headers.txt", "-H", "Accept: application/json", url);
		}
		else
		o = sg.exec("curl", "-D", "headers.txt", url);
		if (o.exit_status != 0) {
			testlib.fail("curl returned nonzero exit status, " + url);
			return;
		}
		var s = sg.file.read("headers.txt");
		if (s.search("404 Not Found") >= 0) {
			testlib.fail(url, "Something other than \"... 404 Not Found...\"", "\"" + this.firstLineOf(s) + "\"");
			return;
		}
		testlib.testResult(true);
	};

	this.firstLineOf = function(st) {
		var t = st.replace(/[\r\n]+/, "\n");
		var lines = t.split("\n");

		if (lines.length == 0)
			return ("");

		return (lines[0]);
	};

	this.checkStatus = function(expected, headerBlock, msg) {
		var s = headerBlock.replace(/[\r\n]+/g, "\n");
		var lines = s.split("\n");
		var got = "";

		var prefix = "";

		if (!!msg)
			prefix = msg + ": ";

		for (var i in lines) {
			var status = lines[i];

			var matches = status.match(/^HTTP\/[0-9\.]+[ \t]+(.+?)[ \t]*$/i);

			if (matches) {
				got = matches[1];

				if (got == expected)
					return testlib.testResult(true);
			}
		}

		var err = prefix + "expected '" + expected + "', got '" + got + "'";

		return (testlib.fail(err));
	};

	this.checkHeader = function(header, expected, headerBlock, msg)
	{
		var s = headerBlock.replace(/[\r\n]+/g, "\n");
		var lines = s.split("\n");
		var got = [];
		
		var prefix = "";

		if (!!msg)
			prefix = msg + ": ";
			
		var regex = new RegExp("^" + header + ":[ \t]+(.+?)[ \t]*$","i");
		
		for (var i in lines) 
		{
			var line = lines[i];
			
			if (line.indexOf(header) == 0)
			{
				
				var matches = line.match(regex);
				if (matches)
				{
					if (matches[1] == expected)
						return testlib.testResult(true);
					else
						got.push(matches[1]);
				}
			}
		}
		
		var err = "expected '" + expected + "', got '" + got.join("' and '") + "'";
		
		return (testlib.fail(err));
	};
	
	this.cookieWasSet = function(name, headerBlock, msg)
	{
		var s = headerBlock.replace(/[\r\n]+/g, "\n");
		var lines = s.split("\n");
		var got = [];
		
		var prefix = "";

		if (!!msg)
			prefix = msg + ": ";
			
		var regex = new RegExp("^Set-Cookie:[ \t]+(" + name + ")=.+[ \t]*$","i");
		
		for (var i in lines) 
		{
			var line = lines[i];
			
			var matches = line.match(regex);
			if (matches)
			{
				if (matches[1] == name)
					return testlib.testResult(true);
				else
					got.push(matches[1]);
			}
		}
		
		var err = "expected cookie name'" + name + "', got '" + got.join("' and '") + "'";
		
		return (testlib.fail(err));
	}
	
	this.getCookieValue = function(name, headerBlock)
	{
		var s = headerBlock.replace(/[\r\n]+/g, "\n");
		var lines = s.split("\n");
		
		var regex = new RegExp("^Set-Cookie:[ \t]+(" + name + ")=([^;]*).+[ \t]*$","i");
		
		for (var i in lines) 
		{
			var line = lines[i];
			
			var matches = line.match(regex);
			if (matches)
			{
				if (matches[1] == name)
					return matches[2]
			}
		}
		
		return null;
	};

	this.testContentTypeByBrowser = function() {
		var browsers = {
			"IE 7" : {
				"useragent" : "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)",
				"contenttype" : "text/html"
			},
			"IE 8" : {
				"useragent" : "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)",
				"contenttype" : "text/html"
			},
			"IE 8 Compat" : {
				"useragent" : "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/4.0)",
				"contenttype" : "text/html"
			},
			"IE 8 Win 7" : {
				"useragent" : "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0)",
				"contenttype" : "text/html"
			},
			"IE 8 64 bit" : {
				"useragent" : "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Win64; x64; Trident/4.0)",
				"contenttype" : "text/html"
			},
			"IE 9" : {
				"useragent" : "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)",
				"contenttype" : "application/xhtml+xml"
			},
			"Firefox" : {
				"useragent" : "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.3) Gecko/20100401 Firefox/3.6.3",
				"contenttype" : "application/xhtml+xml"
			},
			"Chrome" : {
				"useragent" : "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.1 (KHTML, like Gecko) Chrome/5.0.322.2 Safari/533.1",
				"contenttype" : "application/xhtml+xml"
			},
			"Safari Win" : {
				"useragent" : "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10",
				"contenttype" : "application/xhtml+xml"
			},
			"Safari" : {
				"useragent" : "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10",
				"contenttype" : "application/xhtml+xml"
			},
			"Opera" : {
				"useragent" : "Opera/9.80 (Windows NT 5.1; U; en) Presto/2.5.22 Version/10.50",
				"contenttype" : "application/xhtml+xml"
			}
		};

		for ( var desc in browsers )
		{
			var expected = browsers[desc]["contenttype"];
			var ua = browsers[desc]["useragent"];

			var o = sg.exec("curl",
				"-D", "headers.txt",
				"-H", "User-Agent: " + ua,
				encodeURI(baseUrl + "/test/responses/template-test.html"));

			if(!testlib.testResult(o.exit_status == 0, "curl as " + desc))
				continue;

			var headers = sg.file.read("headers.txt");
			if(! this.checkStatus("200 OK", sg.file.read("headers.txt"), "curl as " + desc))
				continue;

			this.checkHeader("Content-Type", expected + ";charset=UTF-8", headers, desc);
		}
	};

	this.testOKResponse = function()
	{
		var url = baseUrl + "/test/responses/ok";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	this.testOKResponse_Post = function()
	{
		var url = baseUrl + "/test/responses/ok";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	this.testOKResponse_Post = function()
	{
		var url = baseUrl + "/test/responses/ok";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	this.testOKResponse_Thrown = function()
	{
		var url = baseUrl + "/test/responses/ok/thrown";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	this.testOKResponse_Post_Thrown = function()
	{
		var url = baseUrl + "/test/responses/ok/thrown";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	
	this.testImplicitOKResponse = function()
	{
		var url = baseUrl + "/test/responses/implicit-200-ok";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
	};
	
	this.testJSONResponse = function()
	{
		var url = baseUrl + "/test/responses/json";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "application/json;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal('{"test":true}', sg.file.read("fetched"));
	}
	
	this.testTextResponse = function()
	{
		var url = baseUrl + "/test/responses/text";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Test", sg.file.read("fetched"));
	}
	
	this.testBadRequestResponse = function()
	{
		var url = baseUrl + "/test/responses/bad_request";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("400 Bad Request", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Bad request.", sg.file.read("fetched"));
	}
	this.testBadRequestResponse_Thown = function()
	{
		var url = baseUrl + "/test/responses/bad_request/thrown";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("400 Bad Request", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Bad request.", sg.file.read("fetched"));
	}
	
	this.testPreconditionFailedResponse = function()
	{
		var url = baseUrl + "/test/responses/precondition_fail";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("412 Precondition Failed", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("fail", sg.file.read("fetched"));
	}
	
	this.testMethodNotAllowedResponse = function()
	{
		var url = baseUrl + "/test/responses/method_not_allowed";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("405 Method Not Allowed", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("405 Method Not Allowed", sg.file.read("fetched"));
	}
	
	this.testMethodNotAllowedResponse_ForReal = function()
	{
		var url = baseUrl + "/test/responses/method_not_allowed";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("405 Method Not Allowed", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("405 Method Not Allowed", sg.file.read("fetched"));
	}
	
	this.testMethodNotAllowedResponse_ForReal2 = function()
	{
		var url = baseUrl + "/test/responses/method_not_allowed/except_on_post";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("405 Method Not Allowed", sg.file.read("headers.txt") )) return;
		this.checkHeader("Allow", "POST", sg.file.read("headers.txt"));
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
	}
	
	this.testNotFoundResponse = function()
	{
		var url = baseUrl + "/test/responses/not_found";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("404 Not Found", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Not found.", sg.file.read("fetched"));
	}
	this.testNotFoundResponse_ForReal = function()
	{
		var url = baseUrl + "/test/responses/not_found/really_really_not_found";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("404 Not Found", sg.file.read("headers.txt") )) return;
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
	}
	this.testNotFoundResponse_ForReal2 = function()
	{
		var url = baseUrl + "/test/responses/not_found/really_really_not_found.json";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("404 Not Found", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Not found.", sg.file.read("fetched"));
	}
	
	this.testUnauthorizedResponse = function()
	{
		var url = baseUrl + "/test/responses/unauthorized";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		if (!this.checkStatus("401 Unauthorized", sg.file.read("headers.txt") )) return;
		if (!this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"))) return;
		testlib.equal("Unauthorized.", sg.file.read("fetched"));
	}
	
	this.test500Response_exception_onDispatch = function()
	{
		var o;

		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/js-exception-in-ondispatch.json"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				encodeURI(baseUrl + "/test/responses/500/js-exception-in-ondispatch.html"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
	}
	this.test500Response_exception_onJsonReceived = function()
	{
		var o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "{\"test\":true}",
				encodeURI(baseUrl + "/test/responses/500/js-exception-in-onjsonreceived"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
	}
	this.test500Response_exception_onFileReceived = function()
	{
		var o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "File contents.",
				encodeURI(baseUrl + "/test/responses/500/js-exception-in-onfilereceived"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
	}
	
	this.testExceptionInOnChunk = function()
	{
		var o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/text/js-exception-in-onchunk"));
		testlib.equal(18, o.exit_status, "curl exit status 18: Partial file. Only a part of the file was transferred.");
		this.checkStatus("200 OK", sg.file.read("headers.txt"));
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
		testlib.equal("12345", sg.file.read("fetched"));
		testlib.testResult(o.stderr.indexOf("transfer closed with 7 bytes remaining")>=0);
	}

	this.test500Error_invalid_response_generated = function()
	{
		var o;
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/undefined"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/null"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/object"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/int"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/string"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "application/xhtml+xml;charset=UTF-8", sg.file.read("headers.txt"));
	}

	this.test500Error_invalid_response_generated__Post = function()
	{
		var o;
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/null"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/object"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/int"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
		
		o = sg.exec("curl",
				"-H", "From: " + this.userId,
				"-D", "headers.txt",
				"-o", "fetched",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-d", "data",
				encodeURI(baseUrl + "/test/responses/500/invalid-response/string"));
		if (!testlib.equal(0, o.exit_status, "curl exit status 0")) return;
		this.checkStatus("500 Internal Server Error", sg.file.read("headers.txt") );
		this.checkHeader("Content-Type", "text/plain;charset=UTF-8", sg.file.read("headers.txt"));
	}
	
	this.testSetBasicCookie = function()
	{
		var url = baseUrl + "/test/cookies/basic";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;

		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
		if (!this.checkHeader("Set-Cookie", "testCookie=testValue; Max-Age=10", sg.file.read("headers.txt"))) return;
	}
	
	this.testSessionCookie = function()
	{
		var url = baseUrl + "/test/cookies/basic";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;

		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		
		if (!this.checkHeader("Content-Length", "0", sg.file.read("headers.txt"))) return;
		if (!this.cookieWasSet("vvSession", sg.file.read("headers.txt"))) return;
	},
	
	this.testDistinctSessionCookie = function()
	{
		var url = baseUrl + "/test/cookies/basic";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;

		var headers = sg.file.read("headers.txt");
		if (!this.checkStatus("200 OK", headers )) return;
		
		if (!this.checkHeader("Content-Length", "0", headers)) return;
		var first_cookie = this.getCookieValue("vvSession", headers);
		
		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;

		var headers = sg.file.read("headers.txt");
		if (!this.checkStatus("200 OK", headers )) return;
		
		if (!this.checkHeader("Content-Length", "0", headers)) return;
		var second_cookie = this.getCookieValue("vvSession", headers);
		
		testlib.notEqual(first_cookie, second_cookie, "Recieved non-distinct session cookies");
	},
	
	this.testPassHeader = function()
	{
		var url = baseUrl + "/test/headers/x-powered-by";
		var username = this.userId;

		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));
				

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		
		var headers = sg.file.read("headers.txt");
		sg.fs.remove("headers.txt");
		
		if (!this.checkStatus("200 OK", headers )) return;
		
		if (!this.checkHeader("Content-Length", "0", headers)) return;
		if (!this.checkHeader("X-Powered-By", "veracity", headers)) return;
	}
	
	this.testSessionPersist = function()
	{
		var url = baseUrl + "/test/session/persist/test";
		var username = this.userId;
		var obj = { test: true };
		
		var json = JSON.stringify(obj);
		var o = sg.exec("curl",
				"-X", "POST",
				"-H", "From: " + username,
				"-d", json,
				"-D", "headers.txt",
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				encodeURI(url));

		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		
		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		sg.fs.remove("headers.txt");
		
		// Refetch the data
		var o = sg.exec("curl",
				"-H", "From: " + username,
				"-b", "cookies.txt",
				"-c", "cookies.txt",
				"-D", "headers.txt",
				encodeURI(url));
				
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;

		if (!this.checkStatus("200 OK", sg.file.read("headers.txt") )) return;
		sg.fs.remove("headers.txt");
		var sessionObj = JSON.parse(o.stdout);
		
		testlib.testResult(json == o.stdout, "Session data was persisted", json, o.stdout);
	}
	
	this.testRepoNamesWithDots = function()
	{
		for (repo in this._testRepos)
		{
			this.verifyNot404("/repos/" + this._testRepos[repo]);
			this.verifyNot404("/repos/" + this._testRepos[repo] + ".json");
		}
		
	};
}
