/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

// for JSON stringifying.  I know sglib has this, but this is the version
// used client-side, so it's available here for consistency
load("json2.js");

function suite_web_filter(baseUrl)
{
    this.userId = "stewie@sourcegear.com";
    this.userId2 = "bosley@sourcegear.com";
    this.sprintId = "";
    this.sprintId2 = "";

    this.createLotsOfWitRecords = function (repo)
    {
        for (var i = 0; i < 5; i++)
        {
            var rec = {
                title: i.toString() + "Dear stupid dog, I've gone to live with the children on jolly farm. Good bye forever. Stewie.",
                status: "open",
                assignee: repInfo.userid,
                verifier: this.userId,             
                priority: "High"              
            };

            newRecord("item", repo, rec);
        }
        for (var i = 0; i < 5; i++)
        {
            var rec = {
                title: i.toString() + " What the deuce?",
                description: "Stewie: There's always been a lot of tension between Lois and me. And it's not so much that I want to kill her, it's just, I want her not to be alive anymore. " + i.toString(),
                status: "open",
                assignee: repInfo.userid,
                verifier: this.userId2,
                priority: "Low"
               
            };
            newRecord("item", repo, rec);
        }        

    }
    this.suite_setUp = function ()
    {
        try
        {
           
            this.repo = sg.open_repo(repInfo.repoName);
            this.repo.create_user(this.userId);
          //  this.repo.create_user(this.userId2);

            this.repo.set_user(this.userId);           

           
            var serverFiles = sg.local_settings().machine.server.files;
            var core = serverFiles + "/core";
            load(serverFiles + "/dispatch.js");
            load(core + "/vv.js");

			var udb = new zingdb(this.repo, sg.dagnum.USERS);
			var urecs = udb.query('user', ['recid'], "name == '" + repInfo.userName + "'");

			repInfo.userid = urecs[0].recid;
         
            this.createLotsOfWitRecords(this.repo);
        }
        finally
        {
            if (this.repo)
                this.repo.close();
        }
    };

    this.checkStatus = function (expected, headerBlock, msg, stdout)
    {
        var s = headerBlock.replace(/[\r\n]+/g, "\n");
        var lines = s.split("\n");
        var got = "";

        var prefix = "";

        if (!!msg)
            prefix = msg + ": ";

        for (var i in lines)
        {
            var status = lines[i];

            var matches = status.match(/^HTTP\/[0-9\.]+[ \t]+(.+?)[ \t]*$/i);

            if (matches)
            {
                got = matches[1];

                if (got == expected)
                    return testlib.testResult(true, "===== " + expected + " =====");
            }
        }

        var err = prefix + "expected '" + expected + "', got '" + got + "'";

        if (stdout)
            testlib.log(stdout);

        return (testlib.fail(err));
    };
    this.testAddFilter = function ()
    {
        var filter = {
            "name": "testAddFilter",
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "assignee": this.userId, "priority": "High" }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recid = eval(o.stdout);
        print(recid);

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
       

        testlib.equal("testAddFilter", savedFilter.name);
        testlib.equal("priority", savedFilter.sort);
        testlib.equal("id,status,description", savedFilter.columns);
        testlib.equal("status", savedFilter.groupedby);
        testlib.equal(2, savedFilter.criteriatmp.length);
     

    };
    this.testUpdateFilter = function ()
    {
        //add a new filter
        var filter = {
            "name": "testUpdateFilter",
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "status": "open" }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recid = eval(o.stdout);
        print(recid);

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.equal("testUpdateFilter", savedFilter.name);
        testlib.equal("priority", savedFilter.sort);
        testlib.equal("id,status,description", savedFilter.columns);
        testlib.equal("status", savedFilter.groupedby);
        testlib.equal(1, savedFilter.criteriatmp.length);

        //add some new values 
        filter.criteria = { "status": "open,fixed", "priority": "High,Low" }
        filter.recid = recid;

        o = sg.exec("curl",
						"-H", "From: " + repInfo.userid,
						"-X", "PUT",
						"-d", JSON.stringify(filter),
						"-D", "headers.txt",
						encodeURI(baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid));

        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.equal(2, savedFilter.criteriatmp.length);
        for (var i = 0; i < savedFilter.criteriatmp.length; i++)
        {
            if (savedFilter.criteriatmp[i].fieldid == "status")
            {
                testlib.equal(savedFilter.criteriatmp[i].value, "open,fixed");
            }
            else if (savedFilter.criteriatmp[i].fieldid == "priority")
            {
                testlib.equal(savedFilter.criteriatmp[i].value, "High,Low");
            }
        }

        //remove a value in the same group
        filter.criteria = { "status": "fixed", "priority": "Low" }
        o = sg.exec("curl",
						"-H", "From: " + repInfo.userid,
						"-X", "PUT",
						"-d", JSON.stringify(filter),
						"-D", "headers.txt",
						encodeURI(baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid));

        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.equal(2, savedFilter.criteriatmp.length);
        for (var i = 0; i < savedFilter.criteriatmp.length; i++)
        {
            if (savedFilter.criteriatmp[i].fieldid == "status")
            {
                testlib.equal(savedFilter.criteriatmp[i].value, "fixed");
            }
            else if (savedFilter.criteriatmp[i].fieldid == "priority")
            {
                testlib.equal(savedFilter.criteriatmp[i].value, "Low");
            }
        }

        //remove all values in a group
        filter.criteria = { "status": "fixed" }
        o = sg.exec("curl",
						"-H", "From: " + repInfo.userid,
						"-X", "PUT",
						"-d", JSON.stringify(filter),
						"-D", "headers.txt",
						encodeURI(baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid));

        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.equal(1, savedFilter.criteriatmp.length);
        testlib.equal("fixed", savedFilter.criteriatmp[0].value);

        //remove all criteria
        filter.criteria = [];

        o = sg.exec("curl",
						"-H", "From: " + repInfo.userid,
						"-X", "PUT",
						"-d", JSON.stringify(filter),
						"-D", "headers.txt",
						encodeURI(baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid));

        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.ok(!savedFilter.criteriatmp);

        //change base filter values
        filter.sort = "status";
        filter.columns = "id,status,description,assignee";
        filter.groupedby = "assignee";
        
        o = sg.exec("curl",
						"-H", "From: " + repInfo.userid,
						"-X", "PUT",
						"-d", JSON.stringify(filter),
						"-D", "headers.txt",
						encodeURI(baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid));

        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var db = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var savedFilter = db.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);
        this.repo.close();
        print(sg.to_json(savedFilter));

        testlib.equal("status", savedFilter.sort);
        testlib.equal("id,status,description,assignee", savedFilter.columns);
        testlib.equal("assignee", savedFilter.groupedby);
       
    };
    this.testListFilters = function ()
    {
        var filter = {
            "name": "testListFilters",
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "priority": "High" }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recid = eval(o.stdout);

        url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json?max=1";

        o = sg.exec("curl", "-D", "headers.txt", "-H", "Accept: application/json", "-H", "From: " + repInfo.userid, url);
        if (!testlib.testResult(o.exit_status == 0)) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;
        var recs = eval('(' + o.stdout + ')');
        testlib.equal(1, recs.length);

        url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json?details";


        o = sg.exec("curl", "-D", "headers.txt", "-H", "Accept: application/json", "-H", "From: " + repInfo.userid, url);
        if (!testlib.testResult(o.exit_status == 0)) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;
        var recs = eval('(' + o.stdout + ')');
        testlib.ok(recs.length > 0);


    };
    this.testDeleteFilter = function ()
    {
        var filter = {
            "name": "testDeleteFilter",
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "priority": "Medium", "verifier": this.userId2 }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recid = eval(o.stdout);

        var url = baseUrl + "/repos/" + repInfo.repoName + "/filter/" + recid;
        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-X", "DELETE",
            "-d", recid,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        this.repo = sg.open_repo(repInfo.repoName);
        var zs = new zingdb(this.repo, sg.dagnum.WORK_ITEMS);
        var filters = zs.query("filter", ["*"], "recid == '" + recid + "'");
       
        testlib.equal(0, filters.length);

        var filtercrit = zs.query("filtercriteria", ["*"], "filter == '" + recid + "'");
        print(sg.to_json(filtercrit));
        testlib.equal(0, filtercrit.length);

        this.repo.close();

    };
    this.testFilterItems = function ()
    {
        var filter = {
            "name": "testFilterItemsHigh",           
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "priority": "High" }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recidHigh = eval(o.stdout);

        var filter = {
            "name": "testFilterItemsCurrent",
            "user": repInfo.userName,
            "sort": "priority",
            "columns": "id,status,description",
            "groupedby": "status",
            "criteria": { "assignee": "currentuser" }
        };
        var url = baseUrl + "/repos/" + repInfo.repoName + "/filters.json";
        var j = JSON.stringify(filter);

        var o = sg.exec("curl",
            "-H", "From: " + repInfo.userid,
            "-d", j,
            "-D", "headers.txt",
            encodeURI(url));
        if (!testlib.testResult(o.exit_status == 0, "failed curl")) return;
        if (!this.checkStatus("200 OK", sg.file.read("headers.txt"))) return;

        var recidCurrent = eval(o.stdout);

        url = baseUrl + "/repos/" + repInfo.repoName + "/filteredworkitems.json?filter=" + recidHigh + "&skip=0&max=100";

        o = sg.exec("curl", "-D", "headers.txt", "-H", "Accept: application/json", "-H", "From: " + repInfo.userid, url);
        var recs = eval('(' + o.stdout + ')');
        testlib.equal(5, recs.results.length);

        url = baseUrl + "/repos/" + repInfo.repoName + "/filteredworkitems.json?filter=" + recidCurrent + "&skip=0&max=100";
		testlib.log("url: " + url);

        o = sg.exec("curl", "-D", "headers.txt", "-H", "Accept: application/json", "-H", "From: " + repInfo.userid, url);
        var recs = eval('(' + o.stdout + ')');
        testlib.equal(10, recs.results.length);

        //todo current milestone
    };
}