/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");
load("vv_serve_process.js");

/* This file contains tests for HTTP push, pull, and clone. */

function suite_web_readonly(baseReadonlyUrl, baseNormalUrl)
{
	var sourceRepo;
	var sourceRepoHttpPath;
	
	var destRepo;
	var destRepoHttpPath;

	this.suite_setUp = function()
	{
		var repo;
		var bIdentical;
		var destName = "dest_" + sg.gid()
		var destPath = pathCombine(tempDir, destName);
		
		print(destPath);
		destPath = destPath.replace("\\\\","/", "g");
		destPath = destPath.replace("\\","/", "g");
		print(destPath);

		sourceRepo = repInfo;
		sourceRepoHttpPath = baseReadonlyUrl + "/repos/" + encodeURI(sourceRepo.repoName);
		
		print("Cloning from " + sourceRepoHttpPath + " to " + destName);
		sg.clone__exact(sourceRepoHttpPath, destName);

		print("Checking out to " + destPath);
		testlib.execVV("checkout", destName, destPath);
		
		destRepo = new repoInfo(destName, destPath, destName)
		destRepoHttpPath = baseReadonlyUrl + "/repos/" + encodeURI(destRepo.repoName);

		repo = sg.open_repo(sourceRepo.repoName);
		bIdentical = repo.compare(destRepo.repoName);
		testlib.testResult(bIdentical, "After HTTP clone, repos are identical");
		repo.close();
	}

	this.testSimplePull = function() // Copied from suite_web_sync.
	{
		var bIdentical;
		
		repInfo = sourceRepo;
		createFileOnDisk("file1.txt", 1);
		addRemoveAndCommit();
		
		var repo = sg.open_repo(sourceRepo.repoName);
		bIdentical = repo.compare(destRepo.repoName);
		testlib.testResult(!bIdentical, "After a change to sourceRepo, repos are different");
		
		testlib.execVV("pull", sourceRepoHttpPath, "--dest", destRepo.repoName);
		bIdentical = repo.compare(destRepo.repoName);
		testlib.testResult(bIdentical, "After HTTP pull, repos are identical");
		
		repo.close();
	}

	this.testSimplePush = function() // Copied from suite_web_sync, then modified.
	{
		var bIdentical;
		
		repInfo = sourceRepo;
		createFileOnDisk("file2.txt", 1);
		addRemoveAndCommit();
		
		var repo = sg.open_repo(sourceRepo.repoName);
		bIdentical = repo.compare(destRepo.repoName);
		testlib.testResult(!bIdentical, "After a change to sourceRepo, repos are different");
		
		testlib.execVV_expectFailure("push", destRepoHttpPath, "--src", sourceRepo.repoName);
		bIdentical = repo.compare(destRepo.repoName);
		testlib.testResult(!bIdentical, "After HTTP push, repos are NOT identical");
		
		repo.close();
	}

	this.checkStatus = function(expected, headerBlock, msg) // Copied from suite_web_dispatch.
	{
		var s = headerBlock.replace(/[\r\n]+/g, "\n");
		var lines = s.split("\n");
		var got = "";

		var prefix = "";

		if (!!msg)
			prefix = msg + ": ";

		for (var i in lines) {
			var status = lines[i];

			var matches = status.match(/^HTTP\/[0-9\.]+[ \t]+(.+?)[ \t]*$/i);

			if (matches) {
				got = matches[1];

				if (got == expected)
					return testlib.testResult(true);
			}
		}

		var err = prefix + "expected '" + expected + "', got '" + got + "'";

		return (testlib.fail(err));
	};

	this.testSimpleRoutes = function()
	{
		var o;

		// GETs against the normal server
		// (Everything should work ("200 OK") since urls should only ever be disabled when in readonly mode.)

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseNormalUrl + "/test/readonly/default.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseNormalUrl + "/test/readonly/explicitly-enabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseNormalUrl + "/test/readonly/explicitly-disabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		// POSTs against the normal server
		// (Everything should work ("200 OK") since urls should only ever be disabled when in readonly mode.)

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseNormalUrl + "/test/readonly/default.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseNormalUrl + "/test/readonly/explicitly-enabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseNormalUrl + "/test/readonly/explicitly-disabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		// GETs against the readonly server

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseReadonlyUrl + "/test/readonly/default.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseReadonlyUrl + "/test/readonly/explicitly-enabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", encodeURI(baseReadonlyUrl + "/test/readonly/explicitly-disabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("404 Not Found", sg.file.read("headers.txt"));

		// POSTs against the readonly server

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseReadonlyUrl + "/test/readonly/default.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("405 Method Not Allowed", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseReadonlyUrl + "/test/readonly/explicitly-enabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("200 OK", sg.file.read("headers.txt"));

		o = sg.exec("curl", "-D", "headers.txt", "-d", "data", encodeURI(baseReadonlyUrl + "/test/readonly/explicitly-disabled.txt"));
		if (!testlib.testResult(o.exit_status == 0, "Curl returned non-zero status")) return;
		this.checkStatus("405 Method Not Allowed", sg.file.read("headers.txt"));
	};

	this.testNewWorkitemButtonHidden = function()
	{
		var o;

		// Verify button is present in a "normal" server.
		o = sg.exec("curl", encodeURI(baseNormalUrl + "/" + repInfo.repoName));
		if(testlib.testResult(o.exit_status == 0, "curl exit status"))
			testlib.testResult(o.stdout.indexOf("new work item")>=0, "present in 'normal' server");

		// Verify button is absent in a readonly server.
		o = sg.exec("curl", encodeURI(baseReadonlyUrl + "/" + repInfo.repoName));
		if(testlib.testResult(o.exit_status == 0, "curl exit status"))
			testlib.testResult(o.stdout.indexOf("new work item")<0, "absent in readonly server");
	}
}

