/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

//////////////////////////////////////////////////////////////////
// update_submodules_helpers.js
// contains some common functions used to test nested-updates of submodules.
//////////////////////////////////////////////////////////////////

function initialize_update_submodule_helpers(obj)
{
    obj.my_verify_expected_throw = function(e, expected_pattern)
    {
	var emsg = e.toString();
	testlib.ok( (emsg.indexOf(expected_pattern) != -1), emsg );
    }

    obj.my_do_commit_array = function( a )
    {
	var m = "Commit:";
	for (var k in a)
	    m += "\n\t" + a[k];
        print(m);

        var csetid = sg.pending_tree.commit( a );

        print("Yields: --> [" + csetid + "]");
        print(" ");

	return csetid;
    }

    obj.my_do_update_confirmed__clean_clean = function(name,
						       outer_root, outer_csetid_target,
						       inner_root, inner_csetid_expected)
    {
	var msg = "my_do_update_confirmed__clean_clean: attempting '" + name + "' with target csetid '" + outer_csetid_target + "'";

	print("Attempting: " + msg);

	// do an UPDATE on the OUTER.

	this.do_fsobj_cd(outer_root);
	var r = sg.vv.update( [ "rev=" + outer_csetid_target, "verbose" ] );
	testlib.ok( (r.status == 0), ("Expected update status of 0, received " + r.status) );
	if (r.log != undefined)
	    dumpActionLog(msg, r.log);

	// make sure OUTER updated to what we asked for (and is clean).

	this.assert_current_baseline(outer_csetid_target);
	var expect_test_outer = new Array;
        this.confirm_when_ignoring_ignores(expect_test_outer);

	// verify that INNER also got updated (and is clean).

	this.do_fsobj_cd(inner_root);
	this.assert_current_baseline(inner_csetid_expected);
	var expect_test_inner = new Array;
	this.confirm_when_ignoring_ignores(expect_test_inner);
    }

    obj.my_do_update_confirmed__clean = function(name,
						 root, csetid_target)
    {
	// this can be used on the outer to do a nested update
	// or on the inner to do a single update.

	var msg = "my_do_update_confirmed__clean: attempting '" + name + "' with target csetid '" + csetid_target + "'";

	print("Attempting: " + msg);

	// do an UPDATE

	this.do_fsobj_cd(root);
	var r = sg.vv.update( [ "rev=" + csetid_target, "verbose" ] );
	testlib.ok( (r.status == 0), ("Expected update status of 0, received " + r.status) );
	if (r.log != undefined)
	    dumpActionLog(msg, r.log);

	// make WD updated to what we asked for (and is clean).

	this.assert_current_baseline(csetid_target);
	var expect_test = new Array;
        this.confirm_when_ignoring_ignores(expect_test);
    }

    obj.my_do_update_confirmed = function(name, 
					  outer_root, outer_csetid_target,
					  inner_root, inner_csetid_expected,
					  expect_test_outer, expect_test_inner)
    {
	var msg = "my_do_update_confirmed: attempting '" + name + "' with target csetid '" + outer_csetid_target + "'";

	print("Attempting: " + msg);

	// do an UPDATE on the OUTER.

	this.do_fsobj_cd(outer_root);
	var r = sg.vv.update( [ "rev=" + outer_csetid_target, "verbose" ] );

	testlib.ok( (r.status == 0), ("Expected update status of 0, received " + r.status) );
	if (r.log != undefined)
	    dumpActionLog(msg, r.log);

	// make sure OUTER updated to what we asked for (and has expected dirt).

	this.assert_current_baseline(outer_csetid_target);
        this.confirm_when_ignoring_ignores(expect_test_outer);

	// verify that INNER also got updated (and has expected dirt).

	this.do_fsobj_cd(inner_root);
	this.assert_current_baseline(inner_csetid_expected);
	this.confirm_when_ignoring_ignores(expect_test_inner);
    }

    obj.my_do_update__try = function(name, 
				     outer_root, outer_csetid_target,
				     expected_pattern)
    {
	var msg = "my_do_update_confirmed: attempting '" + name + "' with target csetid '" + outer_csetid_target + "'";

	print("Attempting: " + msg);

	// try to do an UPDATE on the OUTER.

	try
	{
	    this.do_fsobj_cd(outer_root);
	    var r = sg.vv.update( [ "rev=" + outer_csetid_target, "verbose" ] );
	    if (r.log != undefined)
		dumpActionLog(msg, r.log);

	    testlib.ok( (false), "Update succeeded; expected throw with '" + expected_pattern + "'");
	}
	catch (e)
	{
	    this.my_verify_expected_throw(e, expected_pattern);
	}
    }

    obj.my_do_update_with_deleted_sub_confirmed = function(name, 
							   outer_root, outer_csetid_target,
							   inner_root, 
							   expect_test_outer)
    {
	var msg = "my_do_update_with_deleted_sub_confirmed: attempting '" + name + "' with target csetid '" + outer_csetid_target + "'";

	print("Attempting: " + msg);

	this.do_fsobj_cd(outer_root);
	var r = sg.vv.update( [ "rev=" + outer_csetid_target, "verbose" ] );
	testlib.ok( (r.status == 0), ("Expected update status of 0, received " + r.status) );
	if (r.log != undefined)
	    dumpActionLog(msg, r.log);

	// make sure OUTER updated to what we asked for (and is clean).

	this.assert_current_baseline(outer_csetid_target);
        this.confirm_when_ignoring_ignores(expect_test_outer);

	// verify that INNER got deleted.

	var bSubExists = sg.fs.exists(inner_root);
	testlib.ok( (!bSubExists), msg );
    }

    obj.my_do_update_with_deleted_sub__try = function(name, 
						      outer_root, outer_csetid_target,
						      expected_pattern)
    {
	var msg = "my_do_update_with_deleted_sub__try: attempting '" + name + "' with target csetid '" + outer_csetid_target + "'";

	print("Attempting: " + msg);

	// try to do an UPDATE on the OUTER.

	try
	{
	    this.do_fsobj_cd(outer_root);
	    var r = sg.vv.update( [ "rev=" + outer_csetid_target, "verbose" ] );
	    if (r.log != undefined)
		dumpActionLog(msg, r.log);

	    testlib.ok( (false), "Update succeeded; expected throw with '" + expected_pattern + "'");
	}
	catch (e)
	{
	    this.my_verify_expected_throw(e, expected_pattern);
	}
    }

    obj.my_do_update__expect_non_zero_status = function(name,
							root, csetid_target,
							expected_status)
    {
	// do an update on outer when we expect to get a non-zero result status.
	// such as a portability warning or a submodule problem that prevents
	// the update but without throwing.

	var msg = "my_do_update__expect_non_zero_status: attempting '" + name + "' with target csetid '" + csetid_target + "'";

	print("Attempting: " + msg);

	// do an UPDATE

	this.do_fsobj_cd(root);
	var r = sg.vv.update( [ "rev=" + csetid_target, "verbose" ] );
	print(dumpObj(r, "Update Result", "", 0));
	testlib.ok( (r.status == expected_status), ("Expected update status of " + expected_status + ", received " + r.status) );

	// if the update fails, it's hard to say what state the WD is in.

	return r;
    }

    obj.my_do_checkout__expect_non_zero_status = function(name,
							  repo_name,
							  csetid_target,
							  wd_root,
							  expected_status)
    {
	// do an checkout on outer when we expect to get a non-zero result status.
	// such as a portability warning or a submodule problem that prevents
	// the checkout but without throwing.

	var msg = "my_do_checkout__expect_non_zero_status: attempting '" + name + "' with '(" + repo_name + ", " + csetid_target + ")' into '" + wd_root + "'";

	print("Attempting: " + msg);

	// do an CHECKOUT

	var r = sg.vv.checkout( [ "rev=" + csetid_target ], repo_name, wd_root );
	print(dumpObj(r, "Checkout Result", "", 0));
	testlib.ok( (r.status == expected_status), ("Expected CHECKOUT status of " + expected_status + ", received " + r.status) );

	// TODO think about verifying that the new wd_root directory is empty or something.

	return r;
    }

    obj.my_do_export = function(name,
				repo_name,
				csetid_target,
				temp_dir,
				expected_status)
    {
	// do an EXPORT and confirm the result status.
	// we don't verify the contents of the created temp directory
	// because we don't know what they should be (well, ok we could
	// also do a checkout and do a file by file diff, but i'm not).

	var msg = "my_do_export: attempting '" + name + "' with '(" + repo_name + ", " + csetid_target + ")' into '" + temp_dir + "'";
	print("Attempting: " + msg);

	// do EXPORT

	var r = sg.vv.export( [ "rev=" + csetid_target ], repo_name, temp_dir );
	print(dumpObj(r, "Export Result", "", 0));
	testlib.ok( (r.status == expected_status), ("Expected EXPORT status of " + expected_status + ", received " + r.status) );

	return r;
    }

    obj.my_do_export__option_array = function(name,
					      option_array,
					      repo_name,
					      csetid_target,
					      temp_dir,
					      expected_status)
    {
	// do an EXPORT and confirm the result status.
	// we don't verify the contents of the created temp directory
	// because we don't know what they should be (well, ok we could
	// also do a checkout and do a file by file diff, but i'm not).

	var local_array = new Array;
	local_array.push( "rev=" + csetid_target );
	for (var index in option_array)
	{
	    local_array.push( option_array[index] );
	}

	var msg = "my_do_export: attempting '" + name + "' with '(" + repo_name + ", " + csetid_target + ")' into '" + temp_dir + "'";
	print("Attempting: " + msg);

	// do EXPORT

	var r = sg.vv.export( local_array, repo_name, temp_dir );
	print(dumpObj(r, "Export Result", "", 0));
	testlib.ok( (r.status == expected_status), ("Expected EXPORT status of " + expected_status + ", received " + r.status) );

	return r;
    }

}
