/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "sg_fs3__private.h"

#define MY_SLEEP_MS      100
#define MY_TIMEOUT_MS  30000

static void sg_dbndx__collect_all_rectypes(
	SG_context* pCtx,
    SG_vector* pvec_templates,
    SG_vhash** ppvh
    )
{
    SG_uint32 count_templates = 0;
    SG_vhash* pvh = NULL;
    SG_uint32 i = 0;

    SG_ERR_CHECK(  SG_vector__length(pCtx, pvec_templates, &count_templates)  );
    SG_ARGCHECK(count_templates > 0, pvec_templates);

    SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh)  );
    for (i=0; i<count_templates; i++)
    {
        SG_zingtemplate* pzt = NULL;

        SG_ERR_CHECK(  SG_vector__get(pCtx, pvec_templates, i, (void**) &pzt)  );

        SG_ERR_CHECK(  SG_zingtemplate__list_rectypes__update_into_vhash(pCtx, pzt, pvh)  );
    }

    *ppvh = pvh;
    pvh = NULL;

fail:
    SG_VHASH_NULLFREE(pCtx, pvh);
}

void SG_dbndx__create_schema(
	SG_context* pCtx,
    SG_uint64 dagnum,
    SG_vector* pvec_templates,
    SG_vhash** ppvh
    )
{
    SG_vhash* pvh_rectypes = NULL;
    SG_vhash* pvh_result = NULL;
    SG_vhash* pvh_one_rectype = NULL;
    SG_uint32 count_rectypes = 0;
    SG_uint32 i = 0;

    SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_result)  );

    SG_ERR_CHECK(  sg_dbndx__collect_all_rectypes(pCtx, pvec_templates, &pvh_rectypes)  );
    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_rectypes, &count_rectypes)  );
    for (i=0; i<count_rectypes; i++)
    {
        const char* psz_rectype = NULL;

        SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_rectypes, i, &psz_rectype, NULL)  );
        SG_ERR_CHECK(  SG_zing__collect_fields_for_rectype(pCtx, dagnum, psz_rectype, pvec_templates, &pvh_one_rectype)  );
        SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvh_result, psz_rectype, &pvh_one_rectype)  );
    }

	SG_ERR_CHECK(  SG_vhash__sort(pCtx, pvh_result, SG_TRUE, SG_vhash_sort_callback__increasing)  );

    *ppvh = pvh_result;
    pvh_result = NULL;

fail:
    SG_VHASH_NULLFREE(pCtx, pvh_rectypes);
    SG_VHASH_NULLFREE(pCtx, pvh_result);
    SG_VHASH_NULLFREE(pCtx, pvh_one_rectype);
}

static void sg_dbndx__make_sql__create_record_table(
	SG_context* pCtx,
    const char* psz_table_prefix,
    const char* psz_rectype,
    SG_vhash* pvh_fields,
    SG_string** ppstr
    )
{
    SG_string* pstr_sql = NULL;
    SG_uint32 count_fields = 0;
    SG_uint32 i = 0;

    SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr_sql)  );

    SG_ERR_CHECK(  SG_string__sprintf(pCtx, pstr_sql, "CREATE TABLE \"%s_%s\" (hidrec VARCHAR PRIMARY KEY NOT NULL", psz_table_prefix, psz_rectype)  );

    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_fields, &count_fields)  );
    for (i=0; i<count_fields; i++)
    {
        SG_vhash* pvh_one_field = NULL;
        const char* psz_field_name = NULL;
        const char* psz_field_type = NULL;
        SG_bool b_integerish = SG_FALSE;
        SG_bool b_has = SG_FALSE;
        SG_bool b_sort_by_allowed = SG_FALSE;

        SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_fields, i, &psz_field_name, &pvh_one_field)  );
        SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_one_field, SG_DBNDX_SCHEMA__FIELD__TYPE, &psz_field_type)  );
        
        SG_ERR_CHECK(  SG_zing__field_type_is_integerish__sz(pCtx, psz_field_type, &b_integerish)  );

        SG_ERR_CHECK(  SG_string__append__format(
                    pCtx, 
                    pstr_sql, 
                    ", \"%s\" %s NULL",
                    psz_field_name,
                    b_integerish ? "INTEGER" : "VARCHAR"
                    )  );

        SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_one_field, SG_DBNDX_SCHEMA__FIELD__SORT_BY_ALLOWED, &b_has)  );
        if (b_has)
        {
            SG_ERR_CHECK(  SG_vhash__get__bool(pCtx, pvh_one_field, SG_DBNDX_SCHEMA__FIELD__SORT_BY_ALLOWED, &b_sort_by_allowed)  );
        }
        if (b_sort_by_allowed)
        {
            SG_ERR_CHECK(  SG_string__append__format(
                        pCtx, 
                        pstr_sql, 
                        ", \"ndx$_%s\" INTEGER NULL",
                        psz_field_name
                        )  );
        }
    }

    SG_ERR_CHECK(  SG_string__append__sz(pCtx, pstr_sql, ")")  );

    *ppstr = pstr_sql;
    pstr_sql = NULL;

fail:
    SG_STRING_NULLFREE(pCtx, pstr_sql);
}

static void sg_dbndx__store_schema(
	SG_context* pCtx,
    sqlite3* psql,
    const char* psz_schema_token,
    SG_vhash* pvh_schema
    )
{
    SG_string* pstr = NULL;
    sqlite3_stmt* pStmt = NULL;

    SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
    SG_ERR_CHECK(  SG_vhash__to_json(pCtx, pvh_schema, pstr)  );

	SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt, "INSERT INTO cur_schema (token, json) VALUES (?, ?)")  );
    SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 1, psz_schema_token)  );
    SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 2, SG_string__sz(pstr))  );
    SG_ERR_CHECK(  sg_sqlite__step(pCtx, pStmt, SQLITE_DONE)  );

fail:
    if (pStmt)
    {
        SG_ERR_CHECK(  sg_sqlite__finalize(pCtx, pStmt)  );
    }
    SG_STRING_NULLFREE(pCtx, pstr);
}

void SG_dbndx__update_schema_token(
	SG_context* pCtx,
    sqlite3* psql,
    const char* psz_schema_token
	)
{
    sqlite3_stmt* pStmt = NULL;

	SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt, "UPDATE cur_schema SET token = ?")  );
    SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 1, psz_schema_token)  );
    SG_ERR_CHECK(  sg_sqlite__step(pCtx, pStmt, SQLITE_DONE)  );
    SG_ERR_CHECK(  sg_sqlite__nullfinalize(pCtx, &pStmt)  );

fail:
    if (pStmt)
    {
        SG_ERR_CHECK(  sg_sqlite__finalize(pCtx, pStmt)  );
    }
}

void SG_dbndx__update_shadow_users(
	SG_context* pCtx,
    sqlite3* psql,
    const char* psz_shadow_users_csid,
    SG_varray* pva_users
	)
{
    sqlite3_stmt* pStmt = NULL;
    SG_uint32 count_users = 0;
    SG_uint32 i_user = 0;
    SG_bool b_in_transaction = SG_FALSE;

    SG_ERR_CHECK(  sg_sqlite__exec__retry(pCtx, psql, "BEGIN TRANSACTION", MY_SLEEP_MS, MY_TIMEOUT_MS)  );
    b_in_transaction = SG_TRUE;

	SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "DELETE FROM shadow_users")  );

	SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt, "INSERT INTO shadow_users (userid, name) VALUES (?, ?)")  );

    // clear the shadow table and stuff everything in t
    SG_ERR_CHECK(  SG_varray__count(pCtx, pva_users, &count_users)  );
    for (i_user=0; i_user<count_users; i_user++)
    {
        SG_vhash* pvh_user = NULL;
        const char* psz_name = NULL;
        const char* psz_userid = NULL;

        SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_users, i_user, &pvh_user)  );

        SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_user, "name", &psz_name)  );
        SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_user, "recid", &psz_userid)  );

        SG_ERR_CHECK(  sg_sqlite__reset(pCtx, pStmt)  );
        SG_ERR_CHECK(  sg_sqlite__clear_bindings(pCtx, pStmt)  );
        SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 1, psz_userid)  );
        SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 2, psz_name)  );
        SG_ERR_CHECK(  sg_sqlite__step(pCtx, pStmt, SQLITE_DONE)  );
    }
    SG_ERR_CHECK(  sg_sqlite__nullfinalize(pCtx, &pStmt)  );

    // fix the csid
    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "DELETE FROM cur_shadow")  );
	SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt, "INSERT INTO cur_shadow (csid) VALUES (?)")  );
    SG_ERR_CHECK(  sg_sqlite__bind_text(pCtx, pStmt, 1, psz_shadow_users_csid)  );
    SG_ERR_CHECK(  sg_sqlite__step(pCtx, pStmt, SQLITE_DONE)  );
    SG_ERR_CHECK(  sg_sqlite__nullfinalize(pCtx, &pStmt)  );

    SG_ERR_CHECK(  sg_sqlite__exec__retry(pCtx, psql, "COMMIT TRANSACTION", MY_SLEEP_MS, MY_TIMEOUT_MS)  );
    b_in_transaction = SG_FALSE;

fail:
    if (pStmt)
    {
        SG_ERR_IGNORE(  sg_sqlite__finalize(pCtx, pStmt)  );
    }
    if (b_in_transaction)
    {
        SG_ERR_IGNORE(  sg_sqlite__exec__retry(pCtx, psql, "ROLLBACK TRANSACTION", MY_SLEEP_MS, MY_TIMEOUT_MS)  );
    }
}

void SG_dbndx__get_shadow_users_csid(
	SG_context* pCtx,
    sqlite3* psql,
    char** ppsz_shadow_users_csid
	)
{
    int rc = -1;
    sqlite3_stmt* pStmt = NULL;
    char* psz_shadow_users_csid = NULL;

    SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt, "SELECT csid FROM cur_shadow")  );
    if ((rc = sqlite3_step(pStmt)) == SQLITE_ROW)
    {
        const char* psz_csid = (const char*) sqlite3_column_text(pStmt,0);
        SG_ERR_CHECK(  SG_STRDUP(pCtx, psz_csid, &psz_shadow_users_csid)  );
    }
    else if (SQLITE_DONE != rc)
    {
        SG_ERR_THROW(SG_ERR_SQLITE(rc));
    }
    SG_ERR_CHECK(  sg_sqlite__nullfinalize(pCtx, &pStmt)  );

    *ppsz_shadow_users_csid = psz_shadow_users_csid;
    psz_shadow_users_csid = NULL;

fail:
    SG_NULLFREE(pCtx, psz_shadow_users_csid);
}

void SG_dbndx__get_schema_info(
	SG_context* pCtx,
    sqlite3* psql,
    char** ppsz_schema_token,
    char** ppsz_schema
	)
{
    int rc = -1;
    sqlite3_stmt* pStmt_schema = NULL;
    char* psz_schema_token = NULL;
    char* psz_schema = NULL;

    SG_ERR_CHECK(  sg_sqlite__prepare(pCtx, psql, &pStmt_schema, "SELECT token, json FROM cur_schema")  );
    if ((rc = sqlite3_step(pStmt_schema)) == SQLITE_ROW)
    {
        const char* psz_token = (const char*) sqlite3_column_text(pStmt_schema,0);
        const char* psz_json = (const char*) sqlite3_column_text(pStmt_schema,1);

        SG_ERR_CHECK(  SG_STRDUP(pCtx, psz_token, &psz_schema_token)  );
        SG_ERR_CHECK(  SG_STRDUP(pCtx, psz_json, &psz_schema)  );
    }
    else if (SQLITE_DONE != rc)
    {
        SG_ERR_THROW(SG_ERR_SQLITE(rc));
    }
    SG_ERR_CHECK(  sg_sqlite__nullfinalize(pCtx, &pStmt_schema)  );
    
    *ppsz_schema_token = psz_schema_token;
    psz_schema_token = NULL;

    *ppsz_schema = psz_schema;
    psz_schema = NULL;

fail:
    SG_NULLFREE(pCtx, psz_schema_token);
    SG_NULLFREE(pCtx, psz_schema);
}

static void sg_dbndx__create_record_tables_from_schema(
	SG_context* pCtx,
    sqlite3* psql,
    SG_vhash* pvh_schema
    )
{
    SG_uint32 count_rectypes = 0;
    SG_uint32 i = 0;
    SG_string* pstr_sql = NULL;

    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_schema, &count_rectypes)  );
    for (i=0; i<count_rectypes; i++)
    {
        SG_vhash* pvh_one_rectype = NULL;
        const char* psz_rectype = NULL;

        SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_schema, i, &psz_rectype, &pvh_one_rectype)  );

        SG_ERR_CHECK(  sg_dbndx__make_sql__create_record_table(pCtx, SG_DBNDX_RECORD_TABLE_PREFIX, psz_rectype, pvh_one_rectype, &pstr_sql)  );

        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, SG_string__sz(pstr_sql))  );

        SG_STRING_NULLFREE(pCtx, pstr_sql);
    }

fail:
    SG_STRING_NULLFREE(pCtx, pstr_sql);
}

static void sg_dbndx__create_fts_tables_from_schema(
	SG_context* pCtx,
    sqlite3* psql,
    SG_vhash* pvh_schema
    )
{
    SG_uint32 count_rectypes = 0;
    SG_uint32 i_rectype = 0;

    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_schema, &count_rectypes)  );
    for (i_rectype=0; i_rectype<count_rectypes; i_rectype++)
    {
        SG_vhash* pvh_one_rectype = NULL;
        const char* psz_rectype = NULL;
        SG_uint32 count_fields = 0;
        SG_uint32 i_field = 0;

        SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_schema, i_rectype, &psz_rectype, &pvh_one_rectype)  );

        SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_one_rectype, &count_fields)  );
        for (i_field=0; i_field<count_fields; i_field++)
        {
            SG_vhash* pvh_one_field = NULL;
            const char* psz_field_name = NULL;
            SG_bool b_has = SG_FALSE;
            SG_bool b_full_text_search = SG_FALSE;

            SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_one_rectype, i_field, &psz_field_name, &pvh_one_field)  );
            
            SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_one_field, SG_DBNDX_SCHEMA__FIELD__FULL_TEXT_SEARCH, &b_has)  );
            if (b_has)
            {
                SG_ERR_CHECK(  SG_vhash__get__bool(pCtx, pvh_one_field, SG_DBNDX_SCHEMA__FIELD__FULL_TEXT_SEARCH, &b_full_text_search)  );
            }
            if (b_full_text_search)
            {
                SG_ERR_CHECK(  sg_sqlite__exec__va(pCtx, psql, "CREATE VIRTUAL TABLE \"%s_%s_%s\" USING FTS4 (text VARCHAR NOT NULL)", SG_DBNDX_FTS_TABLE_PREFIX, psz_rectype, psz_field_name)  );
                SG_ERR_CHECK(  sg_sqlite__exec__va(pCtx, psql, "CREATE TABLE \"%s_map_%s_%s\" (ftsrowid INTEGER PRIMARY KEY NOT NULL, hidrec VARCHAR NOT NULL)", SG_DBNDX_FTS_TABLE_PREFIX, psz_rectype, psz_field_name)  );
            }
        }
    }

fail:
    ;
}

static void sg_dbndx__create_db(SG_context* pCtx, sqlite3* psql, SG_uint64 dagnum)
{
    if (SG_DAGNUM__USERS != dagnum)
    {
        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE shadow_users (userid VARCHAR NOT NULL UNIQUE, name VARCHAR NOT NULL UNIQUE)")  );
        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE cur_shadow (csid VARCHAR NOT NULL)")  );
    }

	SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE db_audits (csid VARCHAR NOT NULL, userid VARCHAR NOT NULL, timestamp INTEGER NOT NULL, hidrec VARCHAR NOT NULL)")  );
    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_audits_timestamp ON db_audits (timestamp)")  );
    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_audits_csid ON db_audits (csid)")  );

    if (SG_DAGNUM__HAS_NO_RECID(dagnum))
    {
        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE db_history (hidrec VARCHAR NOT NULL, csid VARCHAR NOT NULL, generation INTEGER NOT NULL)")  );
    }
    else
    {
        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE db_history (recid VARCHAR NOT NULL, hidrec VARCHAR NOT NULL, csid VARCHAR NOT NULL, generation INTEGER NOT NULL)")  );
        SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_history_recid ON db_history (recid)")  );
    }

    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_history_hidrec ON db_history (hidrec)")  );
    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_history_csid ON db_history (csid)")  );
    SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE INDEX IF NOT EXISTS db_history_generation ON db_history (generation)")  );

	SG_ERR_CHECK(  sg_sqlite__exec(pCtx, psql, "CREATE TABLE cur_schema (token VARCHAR NOT NULL, json VARCHAR NOT NULL)")  );

fail:
    ;
}

void SG_dbndx__create_new(
	SG_context* pCtx,
    SG_uint64 dagnum,
    sqlite3* psql,
    const char* psz_schema_token,
    SG_vhash* pvh_schema
	)
{
    SG_NULLARGCHECK(psql);

    SG_ERR_CHECK(  sg_dbndx__create_db(pCtx, psql, dagnum)  );
    SG_ERR_CHECK(  sg_dbndx__create_record_tables_from_schema(pCtx, psql, pvh_schema)  );
    SG_ERR_CHECK(  sg_dbndx__create_fts_tables_from_schema(pCtx, psql, pvh_schema)  );
    SG_ERR_CHECK(  sg_dbndx__store_schema(pCtx, psql, psz_schema_token, pvh_schema)  );

fail:
    ;
}

