/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_dbndx_prototypes.h
 *
 */

#ifndef H_SG_DBNDX_H
#define H_SG_DBNDX_H

BEGIN_EXTERN_C

void SG_dbndx_query__open(
	SG_context*,

	SG_repo* pRepo,                         /**< You still own this, but don't
	                                          free it until the dbndx is
	                                          freed.  It will retain this
	                                          pointer. */

	SG_uint64 iDagNum,                      /**< Which dag in that repo are we
	                                          indexing? */

    sqlite3* psql,
	SG_pathname* pPath, // the path for setting up the filters

	SG_dbndx_query** ppNew
	);

void SG_dbndx_update__open(
	SG_context*,

	SG_repo* pRepo,                         /**< You still own this, but don't
	                                          free it until the dbndx is
	                                          freed.  It will retain this
	                                          pointer. */

	SG_uint64 iDagNum,                      /**< Which dag in that repo are we
	                                          indexing? */

    sqlite3* psql,

	SG_dbndx_update** ppNew
	);

void SG_dbndx_query__free(SG_context* pCtx, SG_dbndx_query* pdbc);
void SG_dbndx_update__free(SG_context* pCtx, SG_dbndx_update* pdbc);

void SG_dbndx_query__lookup_audits(
        SG_context* pCtx, 
        SG_dbndx_query* pdbc, 
        const char* psz_csid,
        SG_varray** ppva
        );

void SG_dbndx_query__query_record_history(
        SG_context* pCtx,
        SG_dbndx_query* pdbc,
        const char* psz_recid,
        SG_varray** ppva
        );

/**
 * Tell the dbndx to update itself with new dagnodes.
 *
 * Typical usage here is to call
 * this routine immediately after adding a changeset to the REPO.
 *
 */
void SG_dbndx_update__do(SG_context* pCtx, SG_dbndx_update* pdbc, SG_varray* pva_audits, SG_vhash* pvh_changesets);

void SG_dbndx__create_schema(
	SG_context* pCtx,
    SG_uint64 dagnum,
    SG_vector* pvec_templates,
    SG_vhash** ppvh
    );

void SG_dbndx__create_new(
	SG_context* pCtx,
    SG_uint64 dagnum,
    sqlite3* psql,
    const char* psz_schema_token,
    SG_vhash* pvh_schema
	);

void SG_dbndx__get_schema_info(
	SG_context* pCtx,
    sqlite3* psql,
    char** ppsz_schema_token,
    char** ppsz_schema
	);

void SG_dbndx__get_shadow_users_csid(
	SG_context* pCtx,
    sqlite3* psql,
    char** ppsz_shadow_users_csid
	);

void SG_dbndx__update_shadow_users(
	SG_context* pCtx,
    sqlite3* psql,
    const char* psz_shadow_users_csid,
    SG_varray* pva_users
	);

void SG_dbndx__update_schema_token(
	SG_context* pCtx,
    sqlite3* psql,
    const char* psz_schema_token
	);

// this function is used by the sqlite and fs2 repos to do
// ndx updates.  it needs callbacks to fetch the actual
// indexes.

typedef void (SG_get_dbndx_path_callback)(SG_context* pCtx, void * pVoidData, SG_uint64 iDagNum, SG_pathname** pp);
typedef void (SG_get_sql_callback)(SG_context* pCtx, void * pVoidData, SG_pathname* pPath, sqlite3** pp);
typedef void (SG_get_treendx_callback)(SG_context* pCtx, void * pVoidData, SG_uint64 iDagNum, SG_treendx** ppndx);

void SG_dbndx_query__query__recent(
	SG_context* pCtx,
	SG_dbndx_query* pndx,
    const char* psz_rectype,
	const SG_varray* pcrit,
	SG_uint32 iNumRecordsToReturn,
	SG_uint32 iNumRecordsToSkip,
    SG_varray* pva_fields,
    SG_varray** ppva
	);

void SG_dbndx_query__query(
	SG_context* pCtx,
	SG_dbndx_query* pndx,
	const char* pidState,
    const char* psz_rectype,
	const SG_varray* pcrit,
	const SG_varray* pSort,
	SG_uint32 iNumRecordsToReturn,
	SG_uint32 iNumRecordsToSkip,
    SG_varray* pva_fields,
    SG_varray** ppva
	);

void SG_dbndx_query__query__one(
	SG_context* pCtx,
	SG_dbndx_query* pndx,
	const char* pidState,
    const char* psz_rectype,
    const char* psz_field_name,
    const char* psz_field_value,
    SG_varray* pva_fields,
    SG_vhash** ppvh
	);

void SG_dbndx_query__fts(
	SG_context* pCtx,
	SG_dbndx_query* pndx,
	const char* pidState,
    const char* psz_rectype,
    const char* psz_keywords,
    const char* psz_field_name,
	SG_uint32 iNumRecordsToReturn,
	SG_uint32 iNumRecordsToSkip,
    SG_varray* pva_fields,
    SG_varray** ppva
	);

void sg_repo__update_ndx(
    SG_context * pCtx,
    SG_repo* pRepo,
    SG_rbtree* prb_new_dagnodes,
    SG_get_dbndx_path_callback* pfn_get_dbndx_path,
    void* p_arg_get_dbndx,
    SG_get_sql_callback* pfn_get_sql,
    void* p_arg_get_sql,
    SG_get_treendx_callback* pfn_get_treendx,
    void* p_arg_get_treendx,
    SG_vhash** ppvh_rebuild,
    SG_vhash* pvh_fresh
    );

void sg_repo__rebuild_dbndxes(
    SG_context * pCtx,
    SG_repo* pRepo,
    SG_rbtree* prb_new_dagnodes,
    SG_get_dbndx_path_callback* pfn_get_dbndx_path,
    void* p_arg_get_dbndx,
    SG_get_sql_callback* pfn_get_sql,
    void* p_arg_get_sql
    );

END_EXTERN_C

#endif

