/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_cbuffer_prototypes.h
 *
 */

#ifndef H_SG_CBUFFER_PROTOTYPES_H
#define H_SG_CBUFFER_PROTOTYPES_H

BEGIN_EXTERN_C

///////////////////////////////////////////////////////////////////////////////


// Create brand new buffer of a certain size to be passed to/through a layer of
// JavaScript by way of an SG_cbuffer.
void SG_cbuffer__alloc__new(
	SG_context * pCtx,
	SG_cbuffer ** ppCbuffer,
	SG_uint32 len
	);

// Create a cbuffer that points to memory that was allocated by
// JS_EncodeString(). We "take ownership" of the pointer, which is to say we
// take responsibility for freeing it. (... And thus null the caller's copy).
// 
// "The array returned by JS_EncodeString on success is allocated as though by
// a call to JS_malloc. The caller may modify it and is responsible for freeing
// it." -- https://developer.mozilla.org/en/SpiderMonkey/JSAPI_Reference/JS_GetStringBytes
void SG_cbuffer__alloc__js_sz(
	SG_context * pCtx,
	SG_cbuffer ** ppCbuffer,
	char ** ppSz // We take ownership of the string and null the callers copy.
	);

// Create a cbuffer out of an SG_string.
void SG_cbuffer__alloc__string(
	SG_context * pCtx,
	SG_cbuffer ** ppCbuffer,
	SG_string ** ppString // We take ownership of the string and null the callers copy.
	);

void SG_cbuffer__nullfree(
	SG_cbuffer ** ppCbuffer,
	JSContext * cx
	);


///////////////////////////////////////////////////////////////////////////////

END_EXTERN_C

#endif
