/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
 * This module defines a "filetool" object and associated functionality.
 * See the prototypes header for additional information.
 */

#ifndef H_SG_FILETOOL_TYPEDEFS_H
#define H_SG_FILETOOL_TYPEDEFS_H

BEGIN_EXTERN_C;

/**
 * Definition of a filetool.
 */
typedef struct _SG_filetool SG_filetool;

/**
 * Prototype of a function that invokes/implements a file tool.
 */
typedef void (SG_filetool__invoker)(
	SG_context*        pCtx,         //< [in] [out] Error and context info.
	const SG_filetool* pTool,        //< [in] The tool being invoked.
	const SG_vhash*    pTokenValues, //< [in] A list of name/value pairs to substitute into the tool's arguments.
	                                 //<      May be NULL if there are no values to substitute.
	SG_int32*          pExitCode     //< [out] The exit code returned by the tool.
	                                 //<       May be NULL if the exit code is unneeded.
	);

/**
 * Prototype of a function that looks up invokers for built-in file tools.
 */
typedef void (SG_filetool__plugin)(
	SG_context*            pCtx,      //< [in] [out] Error and context info.
	const char*            szType,    //< [in] The type of tool being looked up.
	const char*            szContext, //< [in] The context the tool is being looked up in.
	                                  //<      If NULL, then no specific context is being used.
	const char*            szName,    //< [in] The name of the tool being looked up.
	                                  //<      If NULL, a custom invoker for external tools is being requested.
	SG_filetool__invoker** ppInvoker  //< [out] The invoker that was found for the named tool.
	                                  //<       Set to NULL if no matching invoker was found.
	);

/**
 * List of built-in file classes.
 */
#define SG_FILETOOL__CLASS__BINARY SG_FILETOOLCONFIG__BINARY_CLASS
#define SG_FILETOOL__CLASS__TEXT   SG_FILETOOLCONFIG__TEXT_CLASS

/*
 * List of built-in file tools.
 */
#define SG_FILETOOL__INTERNAL__NULL ":null" //< SG_filetool__invoke__null

END_EXTERN_C;

#endif
