/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_inv_entry_typedefs.h
 *
 * @details
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_INV_ENTRY_TYPEDEFS_H
#define H_SG_INV_ENTRY_TYPEDEFS_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

typedef struct _inv_entry SG_inv_entry;

typedef SG_uint32 SG_inv_entry_flags;
#define SG_INV_ENTRY_FLAGS__ZERO				((SG_inv_entry_flags)0x0000)
#define SG_INV_ENTRY_FLAGS__SCANNED				((SG_inv_entry_flags)0x0001)	// SG_dir__read() found it

#define SG_INV_ENTRY_FLAGS__ACTIVE_SOURCE		((SG_inv_entry_flags)0x0010)
#define SG_INV_ENTRY_FLAGS__NON_ACTIVE_SOURCE	((SG_inv_entry_flags)0x0020)

#define SG_INV_ENTRY_FLAGS__ACTIVE_TARGET		((SG_inv_entry_flags)0x0100)
#define SG_INV_ENTRY_FLAGS__NON_ACTIVE_TARGET	((SG_inv_entry_flags)0x0200)

#define SG_INV_ENTRY_FLAGS__PARKED_FOR_SWAP		((SG_inv_entry_flags)0x1000)	// parked/swapped because of entryname used by 2 entries; need swap file
#define SG_INV_ENTRY_FLAGS__PARKED_FOR_CYCLE	((SG_inv_entry_flags)0x2000)	// parked because directory moves caused cycle in ancestor
#define SG_INV_ENTRY_FLAGS__PARKED__MASK		((SG_inv_entry_flags)0xf000)

// this bit indicates that this entry has some type of portability issue.
// this DOES NOT mean that the item needs to throw an error, just that
// there is an issue.
//
// there are (at least) 2 types of issues we might run into:
// [1] a real/final issue.  such as when an UPDATE, REVERT, or MERGE
//     needs to cause both "FOO" and "foo" to appear in the directory
//     in the final result.  this *could* succeed on Linux, but will
//     fail hard on Windows/Mac (and normally would only be detected
//     when we are actually populating the new version of the WD, but
//     hopefully we can catch it in the pre-compute phase).
#define SG_INV_ENTRY_FLAGS__PORT_FINAL				((SG_inv_entry_flags)0x10000)

// [2] a transient potential issue.  such as when UPDATE, REVERT, or
//     MERGE needs to rename "FOO" to "foo" -or- rename "FOO" to "BAR"
//     and create a different file called "foo".  Both cases should
//     succeed as the final result is well-defined on all platforms;
//     it is just that we may have to watch the order of filesystem
//     operations so that we don't introduce a collision on Windows/Mac
//     systems.
#define SG_INV_ENTRY_FLAGS__PORT_TRANSIENT			((SG_inv_entry_flags)0x20000)

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_INV_ENTRY_TYPEDEFS_H
