/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_jsglue_prototypes.h
 *
 */

#ifndef H_SG_JSGLUE_PROTOTYPES_H
#define H_SG_JSGLUE_PROTOTYPES_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

#define SG_OPT_NONRECURSIVE "nonrecursive";
#define SG_OPT_TEST "test";
#define SG_OPT_FORCE "force";
#define SG_OPT_STAMP "stamp";
#define SG_OPT_MESSGE "message";
#define SG_OPT_INCLUDE "include";
#define SG_OPT_EXCLUDE "exclude"

SG_context * SG_jsglue__get_clean_sg_context(JSContext * cx);

void SG_jsglue__set_sg_context(SG_context * pCtx, JSContext * cx);

JSBool SG_jsglue__context_callback(JSContext * cx, uintN contextOp);

void SG_jsglue__error_reporter(JSContext * cx, const char * pszMessage, JSErrorReport * pJSErrorReport);

void SG_jsglue__report_sg_error(SG_context * pCtx, JSContext * cx);

void SG_jsglue__install_scripting_api(SG_context * pCtx, JSContext *cx, JSObject *obj);

//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
// Utility functions used by methods and properties.
//These are also used by sg_zing_jsglue and the "web glue" (sg_uridispatch.c).
void sg_jsglue__copy_varray_into_jsobject(SG_context * pCtx, JSContext* cx, SG_varray* pva, JSObject* obj);
void sg_jsglue__copy_vhash_into_jsobject(SG_context * pCtx, JSContext* cx, SG_vhash* pvh, JSObject* obj);
void sg_jsglue__jsobject_to_vhash(SG_context * pCtx, JSContext* cx, JSObject* obj, SG_vhash** ppvh);
void sg_jsglue__jsobject_to_varray(SG_context * pCtx, JSContext* cx, JSObject* obj, SG_varray** ppva);
void sg_jsglue__jsval_to_json(SG_context * pCtx, JSContext* cx, jsval obj, SG_bool bPretty, SG_string ** ppstr);
SG_bool sg_jsglue_get_equals_flag(const char* psz_flag, const char* psz_match, const char** psz_value);
void sg_jsglue_get_pattern_array(SG_context * pCtx, const char* psz_patterns, SG_stringarray** psaPatterns);


/**
 * SG_JS_BOOL_CHECK() is used to wrap JS_ statements that fail by returning JS_FALSE.
 */
#define SG_JS_BOOL_CHECK(expr) SG_STATEMENT(\
	JSBool sg_js_bool_check_ok;\
	sg_js_bool_check_ok = (expr);\
	SG_ERR_CHECK_CURRENT;\
	if (!sg_js_bool_check_ok)\
	{\
		SG_context__err(pCtx,SG_ERR_JS,__FILE__,__LINE__,#expr);\
		goto fail;\
	}\
)

/**
 * SG_JS_NULL_CHECK() is used to wrap JS_ statements that fail by returning NULL.
 */
#define SG_JS_NULL_CHECK(expr) SG_STATEMENT(\
	void * sg_js_null_check_ok;\
	sg_js_null_check_ok = (void*)(expr);\
	SG_ERR_CHECK_CURRENT;\
	if (!sg_js_null_check_ok)\
	{\
		SG_context__err(pCtx,SG_ERR_JS,__FILE__,__LINE__,#expr);\
		goto fail;\
	}\
)

/**
 * JSVAL_FROM_SZ() effectively makes a call to JS_NewStringCopyZ(), wrapped in an SG_JS_NULL_CHECK().
 * Then, if successful, it uses STRING_TO_JSVAL() to store the result in a jsval.
 * 
 * NB: The first parameter "dest" is an lval.
 * 
 * Also note we use a hardcoded "cx" for the JSContext parameter to JS_StringCopyZ().
 */
#define JSVAL_FROM_SZ(_dest_, _src_) SG_STATEMENT(\
	JSString * js_coppied_string;\
	js_coppied_string = JS_NewStringCopyZ(cx, (_src_));\
	SG_ERR_CHECK_CURRENT;\
	if (!js_coppied_string)\
	{\
		SG_context__err(pCtx,SG_ERR_JS,__FILE__,__LINE__, "JS_NewStringCopyZ() failed.");\
		goto fail;\
	}\
	(_dest_) = STRING_TO_JSVAL(js_coppied_string);\
)




// "JSVAL_IS_OBJECT(v) returns true if v is either an object or JSVAL_NULL.
// This indicates that it is safe to call JSVAL_TO_OBJECT(v) to convert v to
// type JSObject *. Be sure to check for JSVAL_NULL separately if necessary.
// If you only want to determine whether a value v is an object but not null,
// use !JSVAL_IS_PRIMITIVE(v)."
// -- https://developer.mozilla.org/en/SpiderMonkey/JSAPI_Reference/JSVAL_IS_OBJECT
#define JSVAL_IS_OBJECT_FOR_REAL(x) (!JSVAL_IS_PRIMITIVE(x))


END_EXTERN_C;

#endif//H_SG_JSGLUE_PROTOTYPES_H

