/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
 * This module layers on top of "filetool" and adds functionality for calling merge tools.
 * See the prototypes header for additional information.
 */

#ifndef H_SG_MERGETOOL_TYPEDEFS_H
#define H_SG_MERGETOOL_TYPEDEFS_H

BEGIN_EXTERN_C;

// the filetool "type" for all merge tools
#define SG_MERGETOOL__TYPE "merge"

// merge tool contexts
#define SG_MERGETOOL__CONTEXT__MERGE   "merge"   // invoked during a "merge" command / API call
#define SG_MERGETOOL__CONTEXT__RESOLVE "resolve" // invoked during a "resolve" command / API call

/**
 * Possible status/results of a merge operation.
 */
#define SG_MERGETOOL__STATUS__SUCCESSFUL		((SG_int32) 0)
#define SG_MERGETOOL__STATUS__CONFLICT			((SG_int32) 1)
#define SG_MERGETOOL__STATUS__ERROR				((SG_int32) 2)
#define SG_MERGETOOL__STATUS__NOT_ATTEMPTED		((SG_int32) 3)
#define SG_MERGETOOL__STATUS__EXEC_ERROR		((SG_int32) 4)
#define SG_MERGETOOL__STATUS__NO_OUTPUT			((SG_int32) 5)
#define SG_MERGETOOL__STATUS__MUST_BE_LAST		((SG_int32) 6)	// must be last

// The names of internal merge tools are defined below.
// The ':' prefix is the current convention for internal tools,
// though nothing is preventing users from using that prefix in localsettings.
#define SG_MERGETOOL__INTERNAL__SKIP         ":skip"              // don't actually run a merge, just return NOT_ATTEMPTED
#define SG_MERGETOOL__INTERNAL__MERGE_STRICT ":merge_strict"      // use compiled-in C code in sg_diffcore with SG_TEXTFILEDIFF_OPTION__STRICT_EOL
#define SG_MERGETOOL__INTERNAL__MERGE        ":merge"             // use complied-in C code in sg_diffcore with SG_TEXTFILEDIFF_OPTION__REGULAR
#define SG_MERGETOOL__INTERNAL__BASELINE     ":baseline"          // pretend to merge, but just take my version
#define SG_MERGETOOL__INTERNAL__OTHER        ":other"             // pretend to merge, but just take the other version
#define SG_MERGETOOL__INTERNAL__DIFFMERGE    ":diffmerge"         // use compiled-in C code that knows how to exec diffmerge.

END_EXTERN_C;

#endif
