/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_sync_prototypes.h
 *
 */

#ifndef H_SG_SYNC_PROTOTYPES_H
#define H_SG_SYNC_PROTOTYPES_H

BEGIN_EXTERN_C

/**
 * A routine that should be used only for tests to compare repository DAGs.
 * It checks exhaustively, walking the DAG, so you can find out what the differences are in a debugger.
 * But that's not necessary to just determine DAG equality. So don't call this in production code. Tests only.
 */
void SG_sync_debug__compare_one_dag(
	SG_context* pCtx,
	SG_repo* pRepo1,
	SG_repo* pRepo2,
	SG_uint64 iDagNum,
	SG_bool* pbIdentical);

/**
 * A routine that should be used only for tests to compare repository DAGs.
 * It checks exhaustively, walking the DAG, so you can find out what the differences are in a debugger.
 * But that's not necessary to just determine DAG equality. So don't call this in production code. Tests only.
 */
void SG_sync_debug__compare_repo_dags(
	SG_context* pCtx,
	SG_repo* pRepo1,
	SG_repo* pRepo2,
	SG_bool* pbIdentical);

void SG_sync__compare_repo_blobs(SG_context* pCtx,
								 SG_repo* pRepo1,
								 SG_repo* pRepo2,
								 SG_bool* pbIdentical);

void SG_sync__build_best_guess_dagfrag(
	SG_context* pCtx,
	SG_repo* pRepo,
	SG_uint64 iDagNum,
	SG_rbtree* prbStartFromHids,
	SG_vhash* pvhConnectToHidsAndGens,
	SG_dagfrag** ppFrag);

void SG_sync__add_blobs_to_fragball(SG_context* pCtx,
									SG_repo* pRepo,
									SG_pathname* pPath_fragball,
									SG_vhash* pvh_missing_blobs);


/** Add a repo descriptor or URL to the localsettings array.  This is used by the verbs for both push and pull */

void SG_sync__remember_sync_target(SG_context* pCtx, const char * pszLocalRepoDescriptor, const char * pszSyncTarget);

END_EXTERN_C

#endif

