/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_treenode_submodule_prototypes.h
 *
 * @details 
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_TREENODE_SUBMODULE_PROTOTYPES_H
#define H_SG_TREENODE_SUBMODULE_PROTOTYPES_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__free(SG_context * pCtx, SG_treenode_submodule * pTSM);

void SG_treenode_submodule__save_to_repo(SG_context * pCtx,
										 SG_treenode_submodule * pTSM,
										 SG_repo * pRepo,
										 SG_repo_tx_handle * pRepoTx,
										 const char * pszgid,
										 SG_int64 i_objectid_timestamp,
										 SG_uint64 * piBlobFullLength);

void SG_treenode_submodule__load_from_repo(SG_context * pCtx,
										   SG_repo * pRepo,
										   const char * pszHid,
										   SG_treenode_submodule ** ppTSM);

#if defined(DEBUG)
#define SG_TREENODE_SUBMODULE__LOAD_FROM_REPO(pCtx, pRepo, pszHid, ppTSM)	SG_STATEMENT(	SG_treenode_submodule * _p = NULL; \
																							SG_treenode_submodule__load_from_repo(pCtx, pRepo, pszHid, &_p); \
																							_sg_mem__set_caller_data(_p, __FILE__, __LINE__, "SG_treenode_submodule"); \
																							*(ppTSM) = _p; )
#else
#define SG_TREENODE_SUBMODULE__LOAD_FROM_REPO(pCtx, pRepo, pszHid, ppTSM)	SG_treenode_submodule__load_from_repo(pCtx, pRepo, pszHid, ppTSM)
#endif

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__get_id_ref(SG_context * pCtx, const SG_treenode_submodule * pTSM, const char ** ppszHid);

void SG_treenode_submodule__get_version(SG_context * pCtx, const SG_treenode_submodule * pTSM, SG_treenode_version * pVer);

void SG_treenode_submodule__get_tech(SG_context * pCtx, const SG_treenode_submodule * pTSM, const char ** ppszTech);

//////////////////////////////////////////////////////////////////

/**
 * Sniff the directory to see if it is the root of a working directory
 * and return flag.
 * 
 * If it is, optionally return an allocated TSM describing it.
 */
void SG_treenode_submodule__sniff_directory(SG_context * pCtx,
											const SG_pathname * pPath,
											SG_repo * pRepoOuter,
											SG_bool * pbIsWD,
											SG_treenode_submodule ** ppTSM);

#if defined(DEBUG)
#define SG_TREENODE_SUBMODULE__SNIFF_DIRECTORY(pCtx, pPath, pRepoOuter, pbIsWD, ppTSM)	SG_STATEMENT(	SG_treenode_submodule * _p = NULL; \
																										SG_treenode_submodule__sniff_directory(pCtx, pPath, pRepoOuter, pbIsWD, &_p); \
																										_sg_mem__set_caller_data(_p, __FILE__, __LINE__, "SG_treenode_submodule"); \
																										*(ppTSM) = _p; )
#else
#define SG_TREENODE_SUBMODULE__SNIFF_DIRECTORY(pCtx, pPath, pRepoOuter, pbIsWD, ppTSM)	SG_treenode_submodule__sniff_directory(pCtx, pPath, pRepoOuter, pbIsWD, ppTSM)
#endif

//////////////////////////////////////////////////////////////////
// We currently don't have a plain __alloc().  Use __load_from_repo() or
// __sniff_directory() to create a properly allocated and initialized one.

/**
 * Alloc a copy of the given TSM.
 */
void SG_treenode_submodule__alloc__copy(SG_context * pCtx,
										SG_treenode_submodule ** ppTSM_new,
										const SG_treenode_submodule * pTSM_src);

#if defined(DEBUG)
#define SG_TREENODE_SUBMODULE__ALLOC__COPY(pCtx, ppTSM_new, pTSM_src)	SG_STATEMENT(	SG_treenode_submodule * _p = NULL; \
																						SG_treenode_submodule__alloc__copy(pCtx, &_p, (pTSM_src)); \
																						_sg_mem__set_caller_data(_p, __FILE__, __LINE__, "SG_treenode_submodule"); \
																						*(ppTSM_new) = _p; )
#else
#define SG_TREENODE_SUBMODULE__ALLOC__COPY(pCtx, ppTSM_new, pTSM_src)	SG_treenode_submodule__alloc__copy(pCtx, ppTSM_new, pTSM_src)
#endif

//////////////////////////////////////////////////////////////////

#if defined(DEBUG)
void SG_treenode_submodule_debug__dump_link__by_hid(SG_context * pCtx,
													const char * pszHid,
													SG_repo * pRepo,
													int indent,
													SG_string * pStringResult);

void SG_treenode_submodule_debug__dump_to_console(SG_context * pCtx,
												  const SG_treenode_submodule * pTSM,
												  const char * pszLabel);
#endif

/**
 * Return TRUE if both submodule links are comparable.
 * This means that they refer to a submodule using the
 * same technology and are bound to the same repo such
 * that one could CD to that submodule and do an UPDATE
 * to convert the WD from the CSET referenced in one to
 * the CSET referenced in the other.
 */
void SG_treenode_submodule__are_comparable(SG_context * pCtx,
										   const SG_treenode_submodule * pTSM1,
										   const SG_treenode_submodule * pTSM2,
										   SG_bool * pbSimilar);
/**
 * Return TRUE if both submodule links refer to the same HID.
 * This DOES NOT look at the live/dirty stuff.  (and we assume
 * comparable because we're only looking for matching HIDs.)
 */
void SG_treenode_submodule__are_same_hid(SG_context * pCtx,
										 const SG_treenode_submodule * pTSM1,
										 const SG_treenode_submodule * pTSM2,
										 SG_bool * pbSameHid);

/**
 * Return TRUE if this TSM refers to a live WD (as opposed
 * to something we just loaded from a blob in the repo).
 */
void SG_treenode_submodule__is_live(SG_context * pCtx,
									const SG_treenode_submodule * pTSM,
									SG_bool * pbIsLive);

/**
 * Return TRUE if there is dirt in the WD of this submodule.
 */
void SG_treenode_submodule__is_dirty(SG_context * pCtx,
									 SG_treenode_submodule * pTSM,
									 SG_bool * pbIsDirty);

/**
 * Return a brief description of the current reference.
 * Probably just the CSID.
 */
void SG_treenode_submodule__describe_for_status(SG_context * pCtx,
												const SG_treenode_submodule * pTSM,
												char ** ppszDescription);

/**
 * Do full (non-partial) nested commit for submodule.
 */
void SG_treenode_submodule__commit(SG_context * pCtx,
								   const SG_treenode_submodule * pTSM,
								   const char * pszComment,
								   SG_bool * pbDidSomething);

/**
 * Emit console progress message about *not* doing a nested commit.
 */
void SG_treenode_submodule__emit_skip_commit_message(SG_context * pCtx,
													 const SG_treenode_submodule * pTSM);

/**
 * Determine *if* the submodule in the live WD *could* be updated
 * to the goal CSET.
 *
 * Return details in pvaMissingDetails for subs that need a pull/clone/etc.
 *
 * TODO 2010/12/16 Decide if we want to pass bIgnoreWarnings across the boundary.
 * TODO            I don't think we do because it'll make it harder for supporting
 * TODO            third-party DVCSs.
 */
void SG_treenode_submodule__precompute_update(SG_context * pCtx,
											  const SG_pathname * pPathSubmodule,
											  const SG_treenode_submodule * pTSM_pend,
											  const SG_treenode_submodule * pTSM_goal,
											  const SG_vector * pvec_use_submodules,
											  SG_varray * pvaMissingDetails,
											  SG_treenode_submodule_instance_data_update ** ppTSIDU);

/**
 * Actually do the nested-update using the cached info from the precompute step.
 */
void SG_treenode_submodule__apply_update(SG_context * pCtx,
										 const SG_pathname * pPathSubmodule,
										 const SG_treenode_submodule * pTSM_pend,
										 const SG_treenode_submodule * pTSM_goal,
										 SG_treenode_submodule_instance_data_update * pTSIDU);

void SG_treenode_submodule_instance_data_update__free(SG_context * pCtx,
													  SG_treenode_submodule_instance_data_update * pTSIDU);

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__precompute_checkout(SG_context * pCtx,
												const SG_pathname * pPathSubmodule,
												const SG_treenode_submodule * pTSM_goal,
												SG_bool bNoSubmodules,
												const SG_vector * pvec_use_submodules,
												SG_varray * pvaMissingDetails,
												SG_vhash ** pvhCheckoutInstanceData);

void SG_treenode_submodule__apply_checkout(SG_context * pCtx,
										   const SG_pathname * pPathSubmodule,
										   const SG_treenode_submodule * pTSM_goal,
										   const SG_vhash * pvhCheckoutInstanceData);

void SG_treenode_submodule__apply_checkout2(SG_context * pCtx,
											const SG_pathname * pPathSubmodule,
											const SG_vhash * pvhCheckoutInstanceData);

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__precompute_revert(SG_context * pCtx,
											  const SG_pathname * pPathSubmodule,
											  const SG_treenode_submodule * pTSM,	// for vtable only
											  const SG_vector * pvec_use_submodules,
											  SG_bool bNoBackups,
											  SG_varray * pvaMissingDetails);

void SG_treenode_submodule__apply_revert(SG_context * pCtx,
										 const SG_pathname * pPathSubmodule,
										 const SG_treenode_submodule * pTSM,		// for vtable only
										 const SG_vector * pvec_use_submodules,
										 SG_bool bNoBackups);

//////////////////////////////////////////////////////////////////

/**
 * This DOES NOT belong here.  I only made it public so that
 * I could call it from vv_verbs so that I could treat the
 * outermost module as a sub and get most of the error checking
 * and/or setup that the vtable routines were already doing.
 */
void SG_treenode_submodule_veracity__could_do_checkout(SG_context * pCtx,
													   const SG_pathname * pPath_wd_root,		// for error messages only
													   const char * pszRepoDescriptorName,
													   const char * pszHid_cset_id,
													   SG_bool bNoSubmodules,
													   const SG_vector * pvec_use_submodules,
													   SG_varray * pvaMissingDetails,
													   SG_vhash ** ppvhCheckoutInstanceData);

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_TREENODE_SUBMODULE_PROTOTYPES_H
