/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_jsglue__private_sg_vv.h
 *
 * @details Routines implementing the sg_vv.
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_JSGLUE__PRIVATE_SG_VV_H
#define H_SG_JSGLUE__PRIVATE_SG_VV_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.init_new_repo( [option_array,] repo_name [, folder_name] )
 * 
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, init_new_repo)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	const char * pszRepoName = NULL;
	const char * pszFolder = NULL;
	const char * pszStorage = NULL;
	const char * pszHashMethod = NULL;
	const char * psz_shared_users = NULL;
	SG_bool bNoWD = SG_FALSE;
	SG_bool bSubmodule = SG_FALSE;

	JSObject * jso = NULL;
	SG_varray * pvaOptions = NULL;
	SG_uint32 countOptions = 0;
	SG_uint32 k;
	SG_uint32 ndx = 0;

    SG_UNUSED(obj);

	if ((argc < 1) || (argc > 3))
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "No arguments.  Usage: sg.vv.init_new_repo( [option_array,] repo_name [, folder_name] )")  );
	}
	
	// argv[0] may or may not contain the optional 'option_array'.

	jso = JSVAL_TO_OBJECT( argv[ndx] );
	if (JS_IsArrayObject(cx, jso))
	{
		SG_ERR_CHECK(  sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pvaOptions)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pvaOptions, &countOptions)  );
		for (k=0; k<countOptions; k++)
		{
			const char * pszOption_k;
			const char * pszValue_k;

			SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaOptions, k, &pszOption_k)  );

			if (*pszOption_k == 0)
			{
				// eat ""
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "storage", &pszValue_k))
				pszStorage = pszValue_k;
			else if (sg_jsglue_get_equals_flag(pszOption_k, "hash", &pszValue_k))
				pszHashMethod = pszValue_k;
			else if (strcmp(pszOption_k, "no-wc") == 0)
				bNoWD = SG_TRUE;
			else if (strcmp(pszOption_k, "submodule") == 0)
				bSubmodule = SG_TRUE;

			else
				SG_ERR_THROW2(  SG_ERR_INVALIDARG,
								(pCtx, "sg.vv.init_new_repo: unrecognized argument '%s'", pszOption_k)  );
		}

		ndx++;
	}

	// argv[0 or 1] is the required repo_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "repo_name must be string.  Usage: sg.vv.init_new_repo( [option_array,] repo_name [, folder_name] )")  );
		pszRepoName = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}
	else
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "repo_name is required.  Usage: sg.vv.init_new_repo( [option_array,] repo_name [, folder_name] )")  );
	}
	
	// argv[1 or 2] is the optional folder_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "folder_name must be string.  Usage: sg.vv.init_new_repo( [option_array,] repo_name [, folder_name] )")  );
		pszFolder = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}

	// All JS args were parsed successfully, use the new SG_vv_verbs to actually do the work.
	// The following SHOULD MATCH the business part of sg.c:do_cmd_init_new_repo().

    
	SG_ERR_CHECK(  SG_vv_verbs__init_new_repo(pCtx,
											  pszRepoName,
											  pszFolder,
											  pszStorage,
											  pszHashMethod,
											  bNoWD,
											  bSubmodule,
                                              psz_shared_users,// TODO nothing above sets psz_shared_users
											  NULL,
											  NULL)  );

	SG_VARRAY_NULLFREE(pCtx, pvaOptions);

	// TODO 2010/10/25 Return a vhash containing essential details about the new REPO and/or the new WD.
	// TODO            This might include the first cset hid, the repo-id, the repo-name, the wd-root-dir,
	// TODO            and etc.

	*rval = JSVAL_VOID;
	return JS_TRUE;

fail:
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
	SG_VARRAY_NULLFREE(pCtx, pvaOptions);
	return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.checkout( [option_array,] repo_name [, folder_name] )
 *
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, checkout)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	const char * pszRepoName = NULL;
	const char * pszFolder = NULL;
	SG_rev_spec* pRevSpec = NULL;
	SG_bool bSubmodule = SG_FALSE;
    char* psz_target_branch_name = NULL;
	SG_uint32 countRevSpecs = 0;
	SG_vector * pvec_use_submodules = NULL;
	char * pszHint = NULL;
	JSObject * jso = NULL;
	JSObject* jsoReturn = NULL;
	SG_vhash * pvhResults = NULL;
	SG_varray * pvaOptions = NULL;
	SG_uint32 countOptions = 0;
	SG_uint32 k;
	SG_uint32 ndx = 0;

	char * pszHidCSetFirst = NULL;

    SG_UNUSED(obj);

	if ((argc < 1) || (argc > 3))
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "No arguments.  Usage: sg.vv.checkout( [option_array,] repo_name [, folder_name] )")  );
	}

	SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pRevSpec)  );

	// argv[0] may or may not contain the optional 'option_array'.

	jso = JSVAL_TO_OBJECT( argv[ndx] );
	if (JS_IsArrayObject(cx, jso))
	{
		SG_ERR_CHECK(  sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pvaOptions)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pvaOptions, &countOptions)  );
		for (k=0; k<countOptions; k++)
		{
			const char * pszOption_k;
			const char * pszValue_k;

			SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaOptions, k, &pszOption_k)  );

			if (*pszOption_k == 0)
			{
				// eat ""
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "rev", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "tag", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_tag(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "branch", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "use-sub", &pszValue_k))
			{
				if (!pvec_use_submodules)
					SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pvec_use_submodules, countOptions)  );
				SG_ERR_CHECK(  SG_STRDUP(pCtx, pszValue_k, &pszHint)  );
				SG_ERR_CHECK(  SG_vector__append(pCtx, pvec_use_submodules, pszHint, NULL)  );
				pszHint = NULL;
			}

			else if (strcmp(pszOption_k, "submodule") == 0)
				bSubmodule = SG_TRUE;

			else
				SG_ERR_THROW2(  SG_ERR_INVALIDARG,
								(pCtx, "sg.vv.checkout: unrecognized argument '%s'", pszOption_k)  );
		}

		ndx++;
	}

	// we only allow 1 of rev, tag, branch.

	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
	if (countRevSpecs > 1)
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Only one revision specifier (e.g. rev, tag, branch) may be used.")  );
	}

	// argv[0 or 1] is the required repo_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "repo_name must be string.  Usage: sg.vv.checkout( [option_array,] repo_name [, folder_name] )")  );
		pszRepoName = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}
	else
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "repo_name is required.  Usage: sg.vv.checkout( [option_array,] repo_name [, folder_name] )")  );
	}
	
	// argv[1 or 2] is the optional folder_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "folder_name must be string.  Usage: sg.vv.checkout( [option_array,] repo_name [, folder_name] )")  );
		pszFolder = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}

	// All JS args were parsed successfully, use the new SG_vv_verbs to actually do the work.
	// The following SHOULD MATCH the business part of sg.c:do_cmd_checkout().

	SG_ERR_CHECK(  SG_vv_utils__rev_spec_to_csid__no_wd(pCtx, pszRepoName, pRevSpec, &pszHidCSetFirst, &psz_target_branch_name)  );
	SG_ERR_CHECK(  SG_vv_verbs__checkout(pCtx, pszRepoName, pszHidCSetFirst, pszFolder, SG_TRUE, bSubmodule, psz_target_branch_name, pvec_use_submodules, &pvhResults)  );

	if (pvhResults)
	{
        SG_JS_NULL_CHECK(  (jsoReturn = JS_NewObject(cx, NULL, NULL, NULL))  );
		*rval = OBJECT_TO_JSVAL(jsoReturn);
		SG_ERR_CHECK(  sg_jsglue__copy_vhash_into_jsobject(pCtx, cx, pvhResults, jsoReturn)  );
	}
	else
	{
		*rval = JSVAL_VOID;
	}

	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VARRAY_NULLFREE(pCtx, pvaOptions);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszHidCSetFirst);
	SG_NULLFREE(pCtx, psz_target_branch_name);
	SG_NULLFREE(pCtx, pszHint);
	return JS_TRUE;

fail:
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VARRAY_NULLFREE(pCtx, pvaOptions);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszHidCSetFirst);
	SG_NULLFREE(pCtx, psz_target_branch_name);
	SG_NULLFREE(pCtx, pszHint);
	return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.export( [option_array,] repo_name [, folder_name] )
 *
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, export)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	const char * pszRepoName = NULL;
	const char * pszFolder = NULL;
	SG_rev_spec* pRevSpec = NULL;
	SG_uint32 countRevSpecs = 0;
	SG_vector * pvec_use_submodules = NULL;
	char * pszHint = NULL;
	JSObject * jso = NULL;
	JSObject* jsoReturn = NULL;
	SG_varray * pvaOptions = NULL;
	SG_vhash * pvhResults = NULL;
	SG_uint32 countOptions = 0;
	SG_uint32 k;
	SG_uint32 ndx = 0;
	char * pszHidCSetFirst = NULL;
	SG_bool bNoSubmodules = SG_FALSE;

    SG_UNUSED(obj);

	if ((argc < 1) || (argc > 3))
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "No arguments.  Usage: sg.vv.export( [option_array,] repo_name [, folder_name] )")  );
	}

	SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pRevSpec)  );

	// argv[0] may or may not contain the optional 'option_array'.

	jso = JSVAL_TO_OBJECT( argv[ndx] );
	if (JS_IsArrayObject(cx, jso))
	{
		SG_ERR_CHECK(  sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pvaOptions)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pvaOptions, &countOptions)  );
		for (k=0; k<countOptions; k++)
		{
			const char * pszOption_k;
			const char * pszValue_k;

			SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaOptions, k, &pszOption_k)  );

			if (*pszOption_k == 0)
			{
				// eat ""
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "rev", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "tag", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_tag(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "branch", &pszValue_k))
			{
				SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pRevSpec, pszValue_k)  );
			}
			else if (strcmp(pszOption_k, "no-submodules") == 0)
			{
				bNoSubmodules = SG_TRUE;
			}
			else if (sg_jsglue_get_equals_flag(pszOption_k, "use-sub", &pszValue_k))
			{
				if (!pvec_use_submodules)
					SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pvec_use_submodules, countOptions)  );
				SG_ERR_CHECK(  SG_STRDUP(pCtx, pszValue_k, &pszHint)  );
				SG_ERR_CHECK(  SG_vector__append(pCtx, pvec_use_submodules, pszHint, NULL)  );
				pszHint = NULL;
			}

			else
				SG_ERR_THROW2(  SG_ERR_INVALIDARG,
								(pCtx, "sg.vv.export: unrecognized argument '%s'", pszOption_k)  );
		}

		ndx++;
	}

	// we only allow 1 of rev, tag, branch.
	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
	if (countRevSpecs > 1)
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Only one 'rev', 'tag', or 'branch' option may be used.")  );
	}

	// argv[0 or 1] is the required repo_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "repo_name must be string.  Usage: sg.vv.export( [option_array,] repo_name [, folder_name] )")  );
		pszRepoName = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}
	else
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "repo_name is required.  Usage: sg.vv.export( [option_array,] repo_name [, folder_name] )")  );
	}
	
	// argv[1 or 2] is the optional folder_name.

	if (ndx < argc)
	{
		if (! JSVAL_IS_STRING( argv[ndx] ))
			SG_ERR_THROW2(  SG_ERR_INVALIDARG,
							(pCtx, "folder_name must be string.  Usage: sg.vv.export( [option_array,] repo_name [, folder_name] )")  );
		pszFolder = JS_GetStringBytes( JSVAL_TO_STRING( argv[ndx] ) );
		ndx++;
	}

	if ((bNoSubmodules) && (pvec_use_submodules))
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Cannot combine 'no-submodules' and 'use-sub' options.")  );

	// All JS args were parsed successfully, use the new SG_vv_verbs to actually do the work.
	// The following SHOULD MATCH the business part of sg.c:do_cmd_export().

	SG_ERR_CHECK(  SG_vv_utils__rev_spec_to_csid__no_wd(pCtx, pszRepoName, pRevSpec, &pszHidCSetFirst, NULL)  );
	SG_ERR_CHECK(  SG_vv_verbs__export(pCtx, pszRepoName, pszHidCSetFirst, pszFolder, bNoSubmodules, pvec_use_submodules, &pvhResults)  );

	if (pvhResults)
	{
        SG_JS_NULL_CHECK(  (jsoReturn = JS_NewObject(cx, NULL, NULL, NULL))  );
		*rval = OBJECT_TO_JSVAL(jsoReturn);
		SG_ERR_CHECK(  sg_jsglue__copy_vhash_into_jsobject(pCtx, cx, pvhResults, jsoReturn)  );
	}
	else
	{
		*rval = JSVAL_VOID;
	}

	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VARRAY_NULLFREE(pCtx, pvaOptions);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszHidCSetFirst);
	SG_NULLFREE(pCtx, pszHint);
	return JS_TRUE;

fail:
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VARRAY_NULLFREE(pCtx, pvaOptions);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszHidCSetFirst);
	SG_NULLFREE(pCtx, pszHint);
	return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.update( [option_array] )
 *
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, update)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	SG_rev_spec* pRevSpec = NULL;
	SG_bool bTest = SG_FALSE;
	SG_bool bVerbose = SG_FALSE;
	JSObject* jso = NULL;
	JSObject* jsoReturn = NULL;
	SG_varray* pva_options = NULL;
	SG_vhash * pvhResults = NULL;
	SG_uint32 countRevSpecs = 0;
	SG_vector * pvec_use_submodules = NULL;
	char * pszHint = NULL;
    char * psz_target_branch_name = NULL;
	char * pszTargetChangeset = NULL;

	SG_UNUSED(obj);

	if (argc > 1)
	{
		SG_ERR_THROW2(	SG_ERR_INVALIDARG,
						(pCtx, "Too many arguments.	 Usage: sg.vv.update( [option_array] )")  );
	}
	
	SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pRevSpec)  );

	// argv[0] (if present) should contain an 'option_array'.

	if (argc > 0)
	{
		jso = JSVAL_TO_OBJECT( argv[0] );
		if (JS_IsArrayObject(cx, jso))
		{
			SG_uint32 count = 0;
			SG_uint32 k = 0;

			SG_ERR_CHECK(	sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pva_options)	);
			SG_ERR_CHECK(	SG_varray__count(pCtx, pva_options, &count)	 );
			for (k = 0; k < count; k++)
			{
				const char * pszOption_k;
				const char * pszValue_k;

				SG_ERR_CHECK( SG_varray__get__sz(pCtx, pva_options, k, &pszOption_k) );

				if (*pszOption_k == 0)
				{
					// eat ""
					// this is useful when JS code wants to do stuff like this:
					//     var r = sg.vv.update( [ (obj.verbose ? "verbose" : ""), "rev=" + obj.csets[ndxNew] ] );
				}
				else if (sg_jsglue_get_equals_flag (pszOption_k, "rev", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (sg_jsglue_get_equals_flag (pszOption_k, "tag", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_tag(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "branch", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (strcmp(pszOption_k, "test") == 0)
				{
					bTest = SG_TRUE;
				}
				else if (strcmp(pszOption_k, "verbose") == 0)
				{
					bVerbose = SG_TRUE;
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "use-sub", &pszValue_k))
				{
					if (!pvec_use_submodules)
						SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pvec_use_submodules, count)  );
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszValue_k, &pszHint)  );
					SG_ERR_CHECK(  SG_vector__append(pCtx, pvec_use_submodules, pszHint, NULL)  );
					pszHint = NULL;
				}

				// TODO else if "nobackup"

				else
					SG_ERR_THROW2(  SG_ERR_INVALIDARG,
									(pCtx, "sg.vv.update: unrecognized argument '%s'", pszOption_k)  );
			}
		}
	}

	// we only allow 1 of rev, tag, branch.

	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
	if (countRevSpecs > 1)
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Only one 'rev', 'tag', or 'branch' option may be used.")  );
	}

	// All JS args were parsed successfully, use the new SG_vv_verbs to actually do the work.
	// The following SHOULD MATCH the business part of sg.c:do_cmd_update().

	SG_ERR_CHECK(  SG_vv_utils__rev_spec_to_csid__wd(pCtx, NULL,
													pRevSpec,
													&pszTargetChangeset, &psz_target_branch_name)  );
	
	SG_ERR_CHECK(  SG_vv_verbs__update2(pCtx, NULL,
										pszTargetChangeset, psz_target_branch_name,
										bVerbose, bTest,
										pvec_use_submodules,
										&pvhResults)  );

	// returns success if we were successful, if we had portability warnings (with
	// data), or if we had submodule problems (with data).  see "status" field for
	// details in JS.

        SG_JS_NULL_CHECK(  (jsoReturn = JS_NewObject(cx, NULL, NULL, NULL))  );
		*rval = OBJECT_TO_JSVAL(jsoReturn);
		SG_ERR_CHECK(  sg_jsglue__copy_vhash_into_jsobject(pCtx, cx, pvhResults, jsoReturn)  );

	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszTargetChangeset);
	SG_NULLFREE(pCtx, psz_target_branch_name);
	SG_NULLFREE(pCtx, pszHint);
	return JS_TRUE;

fail:
	// only throw for hard errors that we couldn't put in "status" field.
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_NULLFREE(pCtx, pszTargetChangeset);
	SG_NULLFREE(pCtx, psz_target_branch_name);
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
	SG_NULLFREE(pCtx, pszHint);
	return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.merge( [option_array] )
 *
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, merge)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	SG_rev_spec* pRevSpec = NULL;
	SG_bool bTest = SG_FALSE;
	JSObject* jso = NULL;
	JSObject* jsoReturn = NULL;
	SG_varray* pva_options = NULL;
	SG_vhash * pvhResults = NULL;
	SG_vector * pvec_use_submodules = NULL;
	char * pszHint = NULL;
	char * pszTargetChangeset = NULL;
	const char * pszToolOverride = NULL;
	SG_uint32 sum_rev_spec_args = 0;
	SG_uint32 sum_tools = 0;
	SG_wd_plan * pPlan = NULL;

	SG_UNUSED(obj);

	if (argc > 1)
	{
		SG_ERR_THROW2(	SG_ERR_INVALIDARG,
						(pCtx, "Too many arguments.	 Usage: sg.vv.merge( [option_array] )")  );
	}

	// argv[0] (if present) should contain an 'option_array'.

	if (argc > 0)
	{
		SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pRevSpec)  );

		jso = JSVAL_TO_OBJECT( argv[0] );
		if (JS_IsArrayObject(cx, jso))
		{
			SG_uint32 count = 0;
			SG_uint32 k = 0;

			SG_ERR_CHECK(	sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pva_options)	);
			SG_ERR_CHECK(	SG_varray__count(pCtx, pva_options, &count)	 );
			for (k = 0; k < count; k++)
			{
				const char * pszOption_k;
				const char * pszValue_k;

				SG_ERR_CHECK( SG_varray__get__sz(pCtx, pva_options, k, &pszOption_k) );

				// We take at most ONE --rev, --tag, or --branch.

				if (*pszOption_k == 0)
				{
					// eat ""
					// this is useful when JS code wants to do stuff like this:
					//     var r = sg.vv.merge( [ (obj.verbose ? "verbose" : ""), "rev=" + obj.csets[ndxNew] ] );
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "rev", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "tag", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_tag(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "branch", &pszValue_k))
				{
					SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pRevSpec, pszValue_k)  );
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "tool", &pszValue_k))
				{
					pszToolOverride = pszValue_k;
					sum_tools++;
				}
				else if (strcmp(pszOption_k, "no-auto-merge") == 0)
				{
					pszToolOverride = SG_MERGETOOL__INTERNAL__SKIP;
					sum_tools++;
				}
				else if (strcmp(pszOption_k, "test") == 0)
				{
					bTest = SG_TRUE;
				}
				else if (strcmp(pszOption_k, "verbose") == 0)
				{
				    //	bVerbose = SG_TRUE;
				}
				else if (sg_jsglue_get_equals_flag(pszOption_k, "use-sub", &pszValue_k))
				{
					if (!pvec_use_submodules)
						SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pvec_use_submodules, count)  );
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszValue_k, &pszHint)  );
					SG_ERR_CHECK(  SG_vector__append(pCtx, pvec_use_submodules, pszHint, NULL)  );
					pszHint = NULL;
				}
				else
				{
					SG_ERR_THROW2(  SG_ERR_INVALIDARG,
									(pCtx, "sg.vv.merge: unrecognized argument '%s'", pszOption_k)  );
				}
			}
		}
	}
	
	
	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &sum_rev_spec_args)  );
	if (sum_rev_spec_args > 1)
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
			(pCtx, "Only one 'rev', 'tag', or 'branch' option may be used.")  );
	}

	if (sum_tools > 1)
	{
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Only one 'tool' or 'no-auto-merge' option may be used.")  );
	}

	// All JS args were parsed successfully, use the new SG_vv_verbs to actually do the work.
	// The following SHOULD MATCH the business part of sg.c:do_cmd_merge().

	SG_ERR_CHECK(  SG_vv_utils__rev_spec_to_csid_for_merge(pCtx, NULL,
														   pRevSpec,
														   &pszTargetChangeset)  );
	
	SG_ERR_CHECK(  SG_vv_verbs__merge(pCtx,
									  NULL,
									  pszTargetChangeset,
									  pszToolOverride,
									  bTest,
									  pvec_use_submodules,
									  &pvhResults,
									  &pPlan)  );

	// returns success if we were successful, if we had portability warnings (with
	// data), or if we had submodule problems (with data).  see "status" field for
	// details in JS.

	// TODO 2011/02/22 Do we really need --verbose ?

	SG_JS_NULL_CHECK(  jsoReturn = JS_NewObject(cx, NULL, NULL, NULL)  );
	*rval = OBJECT_TO_JSVAL(jsoReturn);
	SG_ERR_CHECK(  sg_jsglue__copy_vhash_into_jsobject(pCtx, cx, pvhResults, jsoReturn)  );

	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_WD_PLAN_NULLFREE(pCtx, pPlan);
	SG_NULLFREE(pCtx, pszTargetChangeset);
	SG_NULLFREE(pCtx, pszHint);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	return JS_TRUE;

fail:
	// only throw for hard errors that we couldn't put in "status" field.
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_WD_PLAN_NULLFREE(pCtx, pPlan);
	SG_NULLFREE(pCtx, pszTargetChangeset);
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
	SG_NULLFREE(pCtx, pszHint);
	SG_REV_SPEC_NULLFREE(pCtx, pRevSpec);
	return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

/**
 * sg.vv.revert( [option_array,] [files_or_folders] )
 *
 */
SG_JSGLUE_METHOD_PROTOTYPE(vv, revert)
{
	SG_context * pCtx = SG_jsglue__get_clean_sg_context(cx);
	SG_pathname* pPath = NULL;
	const char** newArgv = NULL;
	uint i;
	uint n = 0;
	JSObject* jso = NULL;
	JSObject* jsoReturn = NULL;
	SG_bool bTestOnly = SG_FALSE;
	SG_bool bVerbose = SG_FALSE;
	SG_bool bNoSubmodules = SG_FALSE;
	SG_bool bNoBackups = SG_FALSE;
	SG_uint32 nCountItems = 0;
	SG_varray* pva_options = NULL;
	SG_file_spec* pFilespec = NULL;
	SG_vector * pvec_use_submodules = NULL;
	SG_vhash * pvhResults = NULL;
	char * pszHint = NULL;

	SG_UNUSED(obj);

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPath)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPath)  );

	SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );

	if (argc > 0)
	{
		jsval arg0 = argv[0];
		jso = JSVAL_TO_OBJECT(arg0);

		if (JS_IsArrayObject(cx, jso))
		{
			 SG_uint32 count = 0;
			 SG_uint32 j = 0;
			 const char* pszOption;
			 const char * pszValue;

			 SG_ERR_CHECK(  sg_jsglue__jsobject_to_varray(pCtx, cx, jso, &pva_options)  );

			 SG_ERR_CHECK(  SG_varray__count(pCtx, pva_options, &count)  );

			 for (j = 0; j < count; j++)
			 {
				 SG_ERR_CHECK( SG_varray__get__sz(pCtx, pva_options, j, &pszOption) );

				 if ( strcmp(pszOption, "test")==0)
				 {
					bTestOnly = SG_TRUE;
				 }
				 else if ( strcmp(pszOption, "verbose") == 0 )
				 {
					 bVerbose = SG_TRUE;
				 }
				 else if (strcmp(pszOption, "no-submodules") == 0)
				 {
					 bNoSubmodules = SG_TRUE;
				 }
				 else if (strcmp(pszOption, "no-backups") == 0)
				 {
					 bNoBackups = SG_TRUE;
				 }
				 else if (sg_jsglue_get_equals_flag(pszOption, "use-sub", &pszValue))
				 {
					 if (!pvec_use_submodules)
						 SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pvec_use_submodules, count)  );
					 SG_ERR_CHECK(  SG_STRDUP(pCtx, pszValue, &pszHint)  );
					 SG_ERR_CHECK(  SG_vector__append(pCtx, pvec_use_submodules, pszHint, NULL)  );
					 pszHint = NULL;
				 }

				 // TODO else if "force"
				 // TODO "recursive" ?

				 else if ( sg_jsglue__is_filespec_arg(pszOption) )
				 {
					 SG_ERR_CHECK (  sg_jsglue__process_filespec_arg(pCtx, pszOption, pFilespec)  );
				 }
			 }
			 n++;
		}
	}
	//allocate the new string array that we will send down.
	SG_ERR_CHECK(  SG_alloc(pCtx, argc, sizeof(char*), &newArgv)  );
	for (i = n; i < argc; i++)
	{
		SG_JS_BOOL_CHECK(  JSVAL_IS_STRING(argv[i])  );
		newArgv[nCountItems] = JS_GetStringBytes(JSVAL_TO_STRING(argv[i]));
		nCountItems++;
	}

	SG_ERR_CHECK(  SG_vv_verbs__revert(pCtx,
									   NULL,	// use CWD
									   SG_TRUE,	// bRecursive
									   bNoSubmodules,
									   bVerbose,
									   bTestOnly,
									   bNoBackups,
									   pvec_use_submodules,
									   pFilespec,
									   nCountItems, newArgv,
									   &pvhResults)  );

	// returns success if we were successful, if we had portability warnings (with
	// data), or if we had submodule problems (with data).  see "status" field for
	// details in JS.

	// TODO 2011/04/04 Do we really need --verbose ?

	SG_JS_NULL_CHECK(  jsoReturn = JS_NewObject(cx, NULL, NULL, NULL)  );
	*rval = OBJECT_TO_JSVAL(jsoReturn);
	SG_ERR_CHECK(  sg_jsglue__copy_vhash_into_jsobject(pCtx, cx, pvhResults, jsoReturn)  );

	SG_FILE_SPEC_NULLFREE(pCtx, pFilespec);
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_NULLFREE(pCtx, newArgv);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	return JS_TRUE;

fail:
	// only throw for hard errors that we couldn't put in "status" field.
	SG_FILE_SPEC_NULLFREE(pCtx, pFilespec);
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_NULLFREE(pCtx, newArgv);
	SG_VARRAY_NULLFREE(pCtx, pva_options);
	SG_VECTOR_NULLFREE_WITH_ASSOC(pCtx, pvec_use_submodules, _my_sg_free);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
	SG_jsglue__report_sg_error(pCtx,cx);	// DO NOT SG_ERR_IGNORE() THIS
    /* TODO */
    return JS_FALSE;
}

//////////////////////////////////////////////////////////////////

static JSClass sg_vv__class = {
    "sg_vv",
    0,
    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
    JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
    JSCLASS_NO_OPTIONAL_MEMBERS
};

static JSFunctionSpec sg_vv__methods[] = {
	{ "init_new_repo",    SG_JSGLUE_METHOD_NAME(vv, init_new_repo),  1,0,0},
	{ "checkout",         SG_JSGLUE_METHOD_NAME(vv, checkout),       1,0,0},
	{ "export",           SG_JSGLUE_METHOD_NAME(vv, export),         1,0,0},
	{ "update",           SG_JSGLUE_METHOD_NAME(vv, update),         0,0,0},
	{ "merge",            SG_JSGLUE_METHOD_NAME(vv, merge),          0,0,0},
	{ "revert",           SG_JSGLUE_METHOD_NAME(vv, revert),         0,0,0},
	{ NULL, NULL, 0, 0, 0}
};

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_JSGLUE__PRIVATE_SG_VV_H
