/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_mrg__private_build_wd_issues.h
 *
 * @details
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_MRG__PRIVATE_BUILD_WD_ISSUES_H
#define H_SG_MRG__PRIVATE_BUILD_WD_ISSUES_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

#if 1 && defined(DEBUG)
#define DEBUG_INCLUDE_CONFLICT_REASONS
#endif

#if defined(DEBUG_INCLUDE_CONFLICT_REASONS)
/**
 * Build a VARRAY of conflict reason messages.
 * Returns NULL when there are no reasons.
 *
 * TODO do we want this to be debug only?
 */
static void _make_conflict_reasons(SG_context * pCtx, SG_varray ** ppvaConflictReasons, SG_mrg_cset_entry_conflict_flags flagsConflict)
{
	SG_varray * pva = NULL;

	if (flagsConflict)
	{
		SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva)  );

#define M(f,bit,msg)	SG_STATEMENT( if ((f) & (bit)) SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx,pva,(msg))  ); )

		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_MOVE,						"REMOVE-vs-MOVE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_RENAME,					"REMOVE-vs-RENAME" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_ATTRBITS,					"REMOVE-vs-ATTRBITS_CHANGE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_XATTR,						"REMOVE-vs-XATTR_CHANGE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_SYMLINK_EDIT,				"REMOVE-vs-SYMLINK_EDIT" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_FILE_EDIT,					"REMOVE-vs-FILE_EDIT" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_VS_SUBMODULE_EDIT,			"REMOVE-vs-SUBMODULE_EDIT" );

		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DELETE_CAUSED_ORPHAN,				"REMOVE-CAUSED-ORPHAN" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__MOVES_CAUSED_PATH_CYCLE,				"MOVES-CAUSED-PATH_CYCLE" );

		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_MOVE,						"DIVERGENT-MOVE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME,					"DIVERGENT-RENAME" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_ATTRBITS,					"DIVERGENT-ATTRBITS_CHANGE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_XATTR,						"DIVERGENT-XATTR_CHANGE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SYMLINK_EDIT,				"DIVERGENT-SYMLINK_EDIT" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SUBMODULE_EDIT,			"DIVERGENT-SUBMODULE_EDIT" );

		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__TBD,			"DIVERGENT-FILE_EDIT_TBD" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__NO_RULE,		"DIVERGENT-FILE_EDIT_NO_RULE" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__AUTO_CONFLICT,	"DIVERGENT-FILE_EDIT_AUTO_CONFLICT" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__AUTO_ERROR,		"DIVERGENT-FILE_EDIT_AUTO_ERROR" );
		M(flagsConflict, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__AUTO_OK,		"DIVERGENT-FILE_EDIT_AUTO_MERGED" );

#undef M
	}

	*ppvaConflictReasons = pva;
	return;

fail:
	SG_VARRAY_NULLFREE(pCtx, pva);
}
#endif

//////////////////////////////////////////////////////////////////

static void _make_undeletes(SG_context * pCtx,
							SG_varray ** ppvaConflictUndeletes,
							SG_mrg_cset_entry * pMrgCSetEntry_FinalResult)
{
	SG_varray * pva = NULL;
	SG_uint32 k, kLimit;
	const SG_vector * pVec = pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->pVec_MrgCSet_Deletes;

	if (pVec)
	{
		SG_ERR_CHECK(  SG_vector__length(pCtx, pVec, &kLimit)  );
		if (kLimit > 0)
		{
			SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva)  );

			for (k=0; k<kLimit; k++)
			{
				SG_mrg_cset * pMrgCSet_k;

				SG_ERR_CHECK(  SG_vector__get(pCtx, pVec,k, (void **)&pMrgCSet_k)  );
				SG_ASSERT(  (pMrgCSet_k->bufHid_CSet[0] != 0)  );		// hid is only set for actual branch/leaf csets; it is null for merge-results.
				SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva, pMrgCSet_k->bufHid_CSet)  );
			}
		}
	}

	*ppvaConflictUndeletes = pva;
	return;

fail:
	SG_VARRAY_NULLFREE(pCtx, pva);
}

//////////////////////////////////////////////////////////////////

/**
 * Used to display conflict details when the list of candidates are given
 * by keys in the given rbUnique *and* the associated values are a vector
 * of entries that have that key (whatever it may be).  This type of rbUnique
 * is used to support a many-to-one relationship.  For example, when we have
 * a divergent XAttrs and we want to list all of the HID-XATTRs (the keys)
 * that were present in the tree and we want to list the leaves/branches
 * where they occured -- keeping in mind that 2 or more leaves may have
 * changed the XAttrs to the same value.
 *
 * We build a VHASH[<key> --> VARRAY[]]
 *
 * For example, if branch L0 (with hid0) renames "foo" to "foo.0" and the
 * branches L1 (hid1) and L2 (hid2) rename "foo" to "foo.x", we need to
 * create (handwave):
 *     {
 *       "foo.x" : [ "hid1", "hid2" ],
 *       "foo.0" : [ "hid0" ],
 *     }
 */
static void _make_rbUnique(SG_context * pCtx,
						   SG_vhash ** ppvhDivergent,
						   SG_rbtree * prbUnique)
{
	SG_vhash * pvhDivergent = NULL;
	SG_varray * pvaDuplicates_k = NULL;
	SG_rbtree_iterator * pIter = NULL;

	if (prbUnique)
	{
		const char * pszKey_k;
		const SG_vector * pVec_k;
		SG_bool bFound;

		SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvhDivergent)  );

		SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pIter, prbUnique, &bFound, &pszKey_k, (void **)&pVec_k)  );
		while (bFound)
		{
			SG_uint32 nrDuplicates_k;
			const SG_mrg_cset_entry * pMrgCSetEntry_j;
			SG_uint32 j;

			SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pvaDuplicates_k)  );

			SG_ERR_CHECK(  SG_vector__length(pCtx, pVec_k, &nrDuplicates_k)  );
			for (j=0; j<nrDuplicates_k; j++)
			{
				SG_ERR_CHECK(  SG_vector__get(pCtx, pVec_k, j, (void **)&pMrgCSetEntry_j)  );
				SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pvaDuplicates_k, pMrgCSetEntry_j->pMrgCSet->bufHid_CSet)  );
			}

			SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhDivergent, pszKey_k, &pvaDuplicates_k)  );	// this steals our VARRAY

			SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pIter, &bFound, &pszKey_k, (void **)&pVec_k)  );
		}
	}

	*ppvhDivergent = pvhDivergent;
	pvhDivergent = NULL;

fail:
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
	SG_VARRAY_NULLFREE(pCtx, pvaDuplicates_k);
	SG_VHASH_NULLFREE(pCtx, pvhDivergent);
}

/**
 * Use to build a varray of GIDs when given a vector of SG_mrg_cset_entry's.
 */
static void _make_gid_array__vec_mrg_cset_entry(SG_context * pCtx,
												SG_varray ** ppva,
												SG_vector * pVec_MrgCSetEntry)
{
	SG_varray * pva = NULL;
	SG_uint32 k, count;

	SG_ERR_CHECK(  SG_vector__length(pCtx, pVec_MrgCSetEntry, &count)  );
	SG_ASSERT(  (count > 0)  );

	SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva)  );
	for (k=0; k<count; k++)
	{
		SG_mrg_cset_entry * pMrgCSetEntry_k;

		SG_ERR_CHECK(  SG_vector__get(pCtx, pVec_MrgCSetEntry, k, (void **)&pMrgCSetEntry_k)  );
		SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva, pMrgCSetEntry_k->bufGid_Entry)  );
	}

	*ppva = pva;
	return;

fail:
	SG_VARRAY_NULLFREE(pCtx, pva);
}

//////////////////////////////////////////////////////////////////

/**
 * Record the given temp-file pathname in the Issue.
 * We convert this absolute path (which points to a
 * file in <sgtemp>) to a "repo-path" because the user
 * could move their entire working directory between
 * the MERGE and RESOLVE and/or COMMIT.
 *
 */
static void _store_temp_path(SG_context * pCtx,
							 SG_vhash * pvhIssue,
							 SG_mrg * pMrg,
							 const char * pszKey,
							 SG_pathname * pPath)
{
	const SG_pathname * pPathWorkingDirTop;
	SG_string * pStringRepoPath = NULL;

	SG_ERR_CHECK(  SG_pendingtree__get_working_directory_top__ref(pCtx, pMrg->pPendingTree, &pPathWorkingDirTop)  );
	SG_ERR_CHECK(  SG_workingdir__wdpath_to_repopath(pCtx, pPathWorkingDirTop, pPath, SG_FALSE, &pStringRepoPath)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, pszKey, SG_string__sz(pStringRepoPath))  );

fail:
	SG_STRING_NULLFREE(pCtx, pStringRepoPath);
}

//////////////////////////////////////////////////////////////////

/**
 * Document the various fields of an entry AS THEY EXISTED IN A PARTICULAR CSET.
 */
static void _doc_version_detail(SG_context * pCtx,
								SG_mrg * pMrg,
								const SG_mrg_cset_entry * pMrgCSetEntry,
								SG_vhash ** ppvhDetail)
{
	SG_vhash * pvh = NULL;
	SG_string * pStringRepoPath = NULL;

	// Create:
	//    <details> ::=  { "repopath"   : "<repo_path>",
	//                     "hid"        : "<hid>",
	//                     "entryname"  : "<entryname>",
	//                     "gid_parent" : "<gid_parent>",
	//                     "attrbits"   : "<attrbits>",
	//                     "hid_xattrs" : "<hid_xattrs>",
	//                     "cset_label" : "<knickname_for_cset>",
	//                     }

	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh)  );

	SG_ERR_CHECK(  SG_mrg_cset__compute_repo_path_for_entry(pCtx,
															pMrgCSetEntry->pMrgCSet,
															pMrgCSetEntry->bufGid_Entry,
															SG_FALSE,
															&pStringRepoPath)  );

	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "repopath",   SG_string__sz(pStringRepoPath))  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "hid",        pMrgCSetEntry->bufHid_Blob)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "entryname",  SG_string__sz(pMrgCSetEntry->pStringEntryname))  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "gid_parent", pMrgCSetEntry->bufGid_Parent)  );
	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvh, "attrbits",   pMrgCSetEntry->attrBits)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "hid_xattrs", ((pMrgCSetEntry->bufHid_XAttr[0])
																	   ? pMrgCSetEntry->bufHid_XAttr
																	   : SG_MRG_TOKEN__NO_XATTR))  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh, "cset_label", SG_string__sz(pMrgCSetEntry->pMrgCSet->pStringName))  );

	SG_ERR_CHECK(  SG_pendingtree__hack__set_sticky_dir(pCtx,
														pMrg->pPendingTree,
														pMrgCSetEntry->bufGid_Parent)  );

	*ppvhDetail = pvh;
	pvh = NULL;

fail:
	SG_VHASH_NULLFREE(pCtx, pvh);
	SG_STRING_NULLFREE(pCtx, pStringRepoPath);
}

static void _doc_version(SG_context * pCtx,
						 SG_vhash * pvhInputs,
						 SG_mrg * pMrg,
						 const SG_mrg_cset_entry * pMrgCSetEntry)
{
	SG_vhash * pvhDetail = NULL;
	SG_string * pStringRepoPath = NULL;

	// Add the following to the caller's vhash:
	//    <input_k> ::= "<hid_cset_k>" : <details_in_cset_k>,
	// or:
	//    <input_k> ::= "L0"           : <details_in_cset_k>

	SG_ERR_CHECK(  _doc_version_detail(pCtx, pMrg, pMrgCSetEntry, &pvhDetail)  );
	
	// we want to key the input details using the official CSET HID.
	// we have these for actual input leaves (and ancestor).  However,
	// for internal merge results in an n-way merge, we may not have
	// a HID so substitute the pretty name.

	if (pMrgCSetEntry->pMrgCSet->bufHid_CSet[0])
		SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx,
											pvhInputs,
											pMrgCSetEntry->pMrgCSet->bufHid_CSet,
											&pvhDetail)  );
	else
		SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx,
											pvhInputs,
											SG_string__sz(pMrgCSetEntry->pMrgCSet->pStringName),
											&pvhDetail)  );

fail:
	SG_VHASH_NULLFREE(pCtx, pvhDetail);
	SG_STRING_NULLFREE(pCtx, pStringRepoPath);
}

/**
 * Document the various fields of the entry as they existed in the
 * ancestor and all of the leaves (where it wasn't deleted).
 *
 * That is, the value of ancestor/baseline/other* as it WOULD HAVE
 * BEEN SEEEN in that CSET.  This is the historical INPUT data.  It
 * may or may not reflect the state of the entry after MERGE does
 * what it needs to do to the entry.  This info should only be used
 * to print nice headers for the user when they are helping to resolve
 * a conflict.
 * 
 * We're doing it the hardway (by looking up the entry in each
 * CSET) rather than just using the info in a "conflict" structure
 * so that we can give RESOLVE the info for collisions too.
 */
static void _sg_mrg__prepare_issue__hardway__document_inputs(SG_context * pCtx,
															 SG_mrg * pMrg,
															 SG_mrg_cset_entry * pMrgCSetEntry,
															 SG_vhash * pvhInputs)
{
	const char * pszGid = pMrgCSetEntry->bufGid_Entry;
	const SG_mrg_cset_entry * pMrgCSetEntry_k;
	SG_rbtree_iterator * pIter = NULL;
	SG_bool bFound;

	// Add the following to the caller's vhash:
	//
	// <inputs> ::= <input_ancestor> <input_L0> <input_L1> ...

	if (pMrg->pMrgCSet_LCA && pMrg->pMrgCSet_LCA->prbEntries)
	{
		SG_ERR_CHECK(  SG_rbtree__find(pCtx, pMrg->pMrgCSet_LCA->prbEntries, pszGid, &bFound, (void **)&pMrgCSetEntry_k)  );
		if (bFound)
			SG_ERR_CHECK(  _doc_version(pCtx,
										pvhInputs,
										pMrg,
										pMrgCSetEntry_k)  );
	}

	if (pMrg->pMrgCSet_Baseline && pMrg->pMrgCSet_Baseline->prbEntries)
	{
		SG_ERR_CHECK(  SG_rbtree__find(pCtx, pMrg->pMrgCSet_Baseline->prbEntries, pszGid, &bFound, (void **)&pMrgCSetEntry_k)  );
		if (bFound)
			SG_ERR_CHECK(  _doc_version(pCtx,
										pvhInputs,
										pMrg,
										pMrgCSetEntry_k)  );
	}

	if (pMrg->prbCSets_OtherLeaves)
	{
		const char * pszKey_hid_cset;
		SG_bool bHaveAnotherLeaf;

		SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pIter, pMrg->prbCSets_OtherLeaves,
												  &bHaveAnotherLeaf, &pszKey_hid_cset, NULL)  );
		while (bHaveAnotherLeaf)
		{
			const SG_mrg_cset * pMrgCSet_k;

			SG_ERR_CHECK(  SG_rbtree__find(pCtx, pMrg->prbCSets, pszKey_hid_cset, &bFound, (void **)&pMrgCSet_k)  );
			if (bFound && pMrgCSet_k->prbEntries)
			{
				SG_ERR_CHECK(  SG_rbtree__find(pCtx, pMrgCSet_k->prbEntries, pszGid, &bFound, (void **)&pMrgCSetEntry_k)  );
				if (bFound)
					SG_ERR_CHECK(  _doc_version(pCtx,
												pvhInputs,
												pMrg,
												pMrgCSetEntry_k)  );
			}

			SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pIter,
													 &bHaveAnotherLeaf, &pszKey_hid_cset, NULL)  );
		}
	}

fail:
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
}

/**
 * Document the various fields as they existed in all of the (non-deleted)
 * inputs for this entry (without regard for whether or not we have a "conflict"
 * structure for the entry).
 */
static void _sg_mrg__prepare_issue__document_inputs(SG_context * pCtx,
													SG_mrg * pMrg,
													SG_mrg_cset_entry * pMrgCSetEntry_FinalResult,
													SG_vhash * pvhIssue)
{
	SG_vhash * pvhInputs = NULL;

	// Add the following to the issue:
	//
	// "input" : <inputs>

	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvhInputs)  );
	SG_ERR_CHECK(  _sg_mrg__prepare_issue__hardway__document_inputs(pCtx,
																	pMrg,
																	pMrgCSetEntry_FinalResult,
																	pvhInputs)  );
	SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "input", &pvhInputs)  );

fail:
	SG_VHASH_NULLFREE(pCtx, pvhInputs);
}

/**
 * Document the various fields as they existed in the entry in the computed
 * final result cset.  This is a summary of what we did to the entry and
 * the (arbitrary) choices that we made to get the WD populated with the
 * merge result.
 */
static void _sg_mrg__prepare_issue__document_result(SG_context * pCtx,
													SG_mrg * pMrg,
													SG_mrg_cset_entry * pMrgCSetEntry_FinalResult,
													SG_vhash * pvhIssue)
{
	SG_vhash * pvhDetail = NULL;

	// Add the following to the issue:
	//
	// "output" : <details_for_result>

	SG_ERR_CHECK(  _doc_version_detail(pCtx, pMrg, pMrgCSetEntry_FinalResult, &pvhDetail)  );
	SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "output", &pvhDetail)  );

fail:
	SG_VHASH_NULLFREE(pCtx, pvhDetail);
}

//////////////////////////////////////////////////////////////////

/**
 * We get called once for each entry in the final-result.
 * We get called in RANDOM GID order.
 *
 * If this entry has a conflict, write the details to the WD_ISSUES.
 */
static SG_rbtree_foreach_callback _sg_mrg__prepare_issues__cb;

static void _sg_mrg__prepare_issues__cb(SG_context * pCtx,
										const char * pszKey_GidEntry,
										void * pVoidValue_MrgCSetEntry,
										void * pVoidData_Mrg)
{
	SG_mrg_cset_entry * pMrgCSetEntry_FinalResult = (SG_mrg_cset_entry *)pVoidValue_MrgCSetEntry;
	SG_mrg * pMrg = (SG_mrg *)pVoidData_Mrg;
	SG_vhash * pvhIssue = NULL;
	SG_string * pStringRepoPath = NULL;
	SG_string * pStringRepoPathTempDirForFile = NULL;
	SG_varray * pva = NULL;
	SG_vhash * pvh = NULL;
	SG_rbtree_iterator * pIter = NULL;
	SG_mrg_cset_entry_conflict_flags flagsConflict;
	SG_portability_flags flagsPortability;
	SG_bool bHaveConflicts, bHaveCollision, bHavePortability, bHaveIssues, bOnlyAutoOK;
	SG_pendingtree_wd_issue_status status;

	flagsConflict = ((pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict)
					 ? pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->flags
					 : SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__ZERO);
	bHaveConflicts = (flagsConflict != SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__ZERO);

	bHaveCollision = (pMrgCSetEntry_FinalResult->pMrgCSetEntryCollision != NULL);

	flagsPortability = pMrgCSetEntry_FinalResult->portFlagsLogged;
	bHavePortability = (flagsPortability != SG_PORT_FLAGS__NONE);

	bHaveIssues = (bHaveConflicts || bHaveCollision || bHavePortability);
	if (!bHaveIssues)
		return;

	// if the ONLY problem was a divergent file edit and we successfully auto-merged
	// the content, we go ahead and generate the issue and mark the issue resolved.
	// this allow the user to see that we did the auto-merge (using --list-all) but
	// not require them to manually mark it resolved.

	bOnlyAutoOK = (   (flagsConflict == SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__AUTO_OK)
				   && !bHaveCollision
				   && !bHavePortability);
	status = ((bOnlyAutoOK)
			  ? SG_ISSUE_STATUS__MARKED_RESOLVED
			  : SG_ISSUE_STATUS__ZERO);

	// this is the repo-path as it should exist in the final-result and in the WD
	// at the conclusion of the merge.  once the user starts resolving issues and/or
	// making more dirt in the WD, all bets are off.  (so we include the gid-parent
	// in the issue.)
	//
	// TODO 2010/08/31 We need to remove this field from the issue so that
	// TODO            downstream uses (STATUS, RESOLVE, etc) are not tempted
	// TODO            to use it.  But we need to fix the find-repo-path-by-gid
	// TODO            stuff first.

	SG_ERR_CHECK(  SG_mrg_cset__compute_repo_path_for_entry(pCtx,
															pMrg->pMrgCSet_FinalResult,
															pszKey_GidEntry,
															SG_FALSE,
															&pStringRepoPath)  );

	// begin an ISSUE:
	// {
	//    "gid"                      : "<gid>",
	//    "tne_type"                 : <tne_type>,
	//    "status"                   : <status>,
	//
	//    "conflict_flags"           : <conflict_flags>,
	//    "collision_flags"          : <collision_flags>,
	//    "portability_flags"        : <portability_flags>,
	//
	//    "made_unique_entryname"    : <made_unique_entryname>,
	//    DEPRECATED "entryname"                : "<entryname>",
	//    DEPRECATED "gid_parent"               : "<gid_parent_directory>",
	//    DEPRECATED "repopath"                 : "<repopath>",
	//
	//    DEPRECATED "attrbits"                 : <attrBits>,
	//    DEPRECATED "hid_xattrs"               : "<hid_xattrs>",

	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvhIssue)  );

	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "gid",               pszKey_GidEntry)  );
	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvhIssue, "tne_type",          pMrgCSetEntry_FinalResult->tneType)  );
	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvhIssue, "status",            status)  );
	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvhIssue, "conflict_flags",    flagsConflict)  );	// don't include __AUTO_OK bit
	SG_ERR_CHECK(  SG_vhash__add__bool(      pCtx, pvhIssue, "collision_flags",   bHaveCollision)  );
	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvhIssue, "portability_flags", flagsPortability)  );

	// indicate if we had to artificially resolve the entryname (either because
	// of a divergent rename or a collision or portability issue).
	SG_ERR_CHECK(  SG_vhash__add__bool(      pCtx, pvhIssue, "made_unique_entryname", pMrgCSetEntry_FinalResult->bMadeUniqueEntryname)  );

	// TODO 2010/07/10 Yank the repo-path and entryname fields.
	// TODO            Anyone who needs them should compute them
	// TODO            dynamically using the pendingtree.

	// the final entryname chosen (either from the branches or artificially by us)
	// should be the same as the final component in the repo-path, but it might be
	// convenient to have it as a separate field.  also note that this entryname
	// field is currently correct as of the time that we write out these ISSUES.
	// *but* it is time-sensitive, since the user may rename it or otherwise
	// manipulate the WD after the MERGE and before they RESOLVE this ISSUE.  so
	// if we have a "--fix" option, it may need to disregard this field and use
	// the GIDs to lookup the entry in the pendingtree.

	/*DEPRECATE*/	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "entryname",         SG_string__sz(pMrgCSetEntry_FinalResult->pStringEntryname))  );

	// the parent directory chosen for the entry (either from the branches or
	// artificially by us) should be the same as that implied by the repo-path.
	// but note that both the gid_parent and the repo-path are time-sensitive.
	// if the user manipulates the WD and/or resolves other issues, this entry
	// may or may not be in this directory or have this repo-path.
	/*DEPRECATE*/	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "gid_parent",        pMrgCSetEntry_FinalResult->bufGid_Parent)  );

	// the repo-path for the entry that we computed at the time the merge was
	// completed.  as with the entryname and gid_parent, this is is time-senstive.
	/*DEPRECATE*/	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "repopath",          SG_string__sz(pStringRepoPath))  );

	/*DEPRECATE*/	SG_ERR_CHECK(  SG_vhash__add__int64(     pCtx, pvhIssue, "attrbits",          pMrgCSetEntry_FinalResult->attrBits)  );
	/*DEPRECATE*/	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_xattrs",        ((pMrgCSetEntry_FinalResult->bufHid_XAttr[0])
																				   ? pMrgCSetEntry_FinalResult->bufHid_XAttr
																				   : SG_MRG_TOKEN__NO_XATTR))  );

	// add "inputs" data to the issue.

	SG_ERR_CHECK(  _sg_mrg__prepare_issue__document_inputs(pCtx,
														   pMrg,
														   pMrgCSetEntry_FinalResult,
														   pvhIssue)  );

	// add an "output" vhash with all of the details we have for the final result
	// as MERGE computed and updated set them in the WD.
	// WARNING: these values are ONLY CURRENT AS OF THE END OF THE MERGE.  These
	// WARNING: may be useful for RESOLVE to display where we put something, but
	// WARNING: it doesn't mean that RESOLVE can rely on them because the user could
	// WARNING: have already changed something.

	SG_ERR_CHECK(  _sg_mrg__prepare_issue__document_result(pCtx,
														   pMrg,
														   pMrgCSetEntry_FinalResult,
														   pvhIssue)  );

	if (bHaveConflicts)
	{
#if defined(DEBUG_INCLUDE_CONFLICT_REASONS)
		// add an array of messages for each bit in conflict_flags.
		// this is mostly for display purposes; use conflict_flags in code.
		//
		//    "conflict_reasons" : [ "<name1>", "<name2>", ... ],

		SG_ERR_CHECK(  _make_conflict_reasons(pCtx, &pva, flagsConflict)  );
		SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhIssue, "conflict_reasons", &pva)  );
#endif

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__UNDELETE__MASK)
		{
			// entry was deleted in one or more branches and modified/needed in other branch(es).
			// we treat all the various way this can happen the same way since we don't really
			// care whether it was a __DELETE_VS_MOVE or a __DELETE_VS_RENAME.  all that really
			// matters (to the RESOLVER) is whether to keep the entry (as modified) or to delete
			// it from the final merge result.
			//
			// as a convenience to the user (and mostly for display purposes), we list the
			// branch(es) in which it was deleted.
			//
			// "conflict_deletes" : [ "<hid_cset_k>", ... ],

			SG_ERR_CHECK(  _make_undeletes(pCtx, &pva, pMrgCSetEntry_FinalResult)  );
			SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhIssue, "conflict_deletes", &pva)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__MOVES_CAUSED_PATH_CYCLE)
		{
			// Give some hints about the other dirs involved in the cycle.
			// The HINT string uses the entrynames as they WANTED TO APPEAR
			// in the merge result.  This is info-only (because MERGE had to
			// arbitrarily put each of the dirs somewhere (probably where it
			// was in the baseline).
			//
			// Use the GIDs in the OTHERS array if you want to determine where
			// they actually are on disk after the MERGE.
			//
			// "conflict_path_cycle_hint" : ".../<name_this>/.../<dir_other>/.../<name_this>"
			// "conflict_path_cycle_others" : [ "<gid_other_1>", "<gid_other_2>", ... ],

			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx,
													 pvhIssue,
													 "conflict_path_cycle_hint",
													 SG_string__sz(pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->pStringPathCycleHint))  );

			SG_ERR_CHECK(  _make_gid_array__vec_mrg_cset_entry(pCtx,
															   &pva,
															   pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->pVec_MrgCSetEntry_OtherDirsInCycle)  );
			SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhIssue, "conflict_path_cycle_others", &pva)  );	// this steals our varray
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_MOVE)
		{
			// we DO NOT bother to list the repo-paths of the various parent directory
			// choices since these may be invalid by the time the user is ready to
			// resolve this issue.  the resolver should use the gid-parent-x and the
			// pendingtree/WD to find the actual location of the parent directory at
			// the time of the resovlve.
			//
			// "conflict_divergent_moves" : { "<gid_parent_1>" : [ "<hid_cset_j>", ... ],
			//                                "<gid_parent_2>" : [ "<hid_cset_k>", ... ],
			//                                ... },

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_GidParent)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_moves", &pvh)  );
		}

		if (pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->flags & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME)
		{
			// "conflict_divergent_renames" : { "<name_1>" : [ "<hid_cset_j>", ... ],
			//                                  "<name_2>" : [ "<hid_cset_k>", ... ],
			//                                  ... },

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_Entryname)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_renames", &pvh)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_ATTRBITS)
		{
			// "conflict_divergent_attrbits" : { "<attrbits_1>" : [ "<hid_cset_j>", ... ],
			//                                   "<attrbits_2>" : [ "<hid_cset_k>", ... ],
			//                                   ... },

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_AttrBits)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_attrbits", &pvh)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_XATTR)
		{
			// "conflict_divergent_hid_xattrs" : { "<hid_xattrs_1>" : [ "<hid_cset_j>", ... ],
			//                                     "<hid_xattrs_2>" : [ "<hid_cset_k>", ... ],
			//                                     ... },
			//
			// TODO 2010/05/20 I'm putting the HIDs of the various XATTR choices in the ISSUE.
			// TODO            And above we put the HID of the XATTR in the final-result.
			// TODO            Would it be better/more useful to fetch them from the REPO and
			// TODO            include them in the ISSUE?  I'm not sure that it's safe (or
			// TODO            necessary) to express the full XATTR in JSON....

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_HidXAttr)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_hid_xattrs", &pvh)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SYMLINK_EDIT)
		{
			// "hid_symlink" : "<hid_blob>",
			// "conflict_divergent_hid_symlink" : { "<hid_blob_1>" : [ "<hid_cset_j>", ... ],
			//                                      "<hid_blob_2>" : [ "<hid_cset_k>", ... ],
			//                                      ... },
			//
			// TODO 2010/05/20 I'm putting the HIDs of the symlink content blob.  This is not
			// TODO            very display friendly.  Think about including the link target
			// TODO            in the ISSUE.

			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_symlink", pMrgCSetEntry_FinalResult->bufHid_Blob)  );

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_Symlink_HidBlob)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_hid_symlink", &pvh)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SUBMODULE_EDIT)
		{
			// "hid_submodule" : "<hid_blob>",
			// "conflict_divergent_hid_submodule" : { "<hid_blob_1>" : [ "<hid_cset_j>", ... ],
			//                                        "<hid_blob_2>" : [ "<hid_cset_k>", ... ],
			//                                        ... },
			//
			// TODO 2011/02/14 I'm putting the HIDs of the submodule content blob.  This is not
			// TODO            very display friendly.

			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_submodule", pMrgCSetEntry_FinalResult->bufHid_Blob)  );

			SG_ERR_CHECK(  _make_rbUnique(pCtx, &pvh, pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict->prbUnique_Submodule_HidBlob)  );
			SG_ERR_CHECK(  SG_vhash__add__vhash(pCtx, pvhIssue, "conflict_divergent_hid_submodule", &pvh)  );
		}

		if (flagsConflict & SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__MASK)
		{
			// We write the following info for file-content conflicts (any time the
			// file was modified in both branches whether auto-mergeable or not).
			// The info here is only sufficient for a 2-way merge.  I'll extent this
			// later.
			// 
			// "repopath_tempdir"            : "<repo_path_of_temp_dir_for_this_file>",
			// "repopath_tempfile_ancestor"  : "<repo_path_of_temp_file_for_ancestor>",
			// "repopath_tempfile_baseline"  : "<repo_path_of_temp_file_for_baseline>",
			// "repopath_tempfile_other"     : "<repo_path_of_temp_file_for_other>",
			// "repopath_tempfile_result"    : "<repo_path_of_temp_file_for_result>",
			// "label_ancestor"              : "<gui_friendly_label_for_ancestor_panel>",
			// "label_baseline"              : "<gui_friendly_label_for_baseline_panel>",
			// "label_other"                 : "<gui_friendly_label_for_other_panel>",
			// "hid_file_ancestor"           : "<hid_blob_ancestor>",
			// "hid_file_baseline"           : "<hid_blob_baseline>",
			// "hid_file_other"              : "<hid_blob_other>",
			// "automerge_mergetool"         : "<merge_tool_name>",         // may be omitted if auto-merge disabled or no rule.
			// "resolve_mergetool"           : "<merge_tool_name>",         // only set when resolve has to do something
			// "resolve_mergetool_status"    : <status>,

			SG_mrg_cset_entry_conflict * pMrgCSetEntryConflict = pMrgCSetEntry_FinalResult->pMrgCSetEntryConflict;
			const char * pszKey_k;
			SG_bool bFound;

			SG_ERR_CHECK(  _store_temp_path(pCtx, pvhIssue, pMrg, "repopath_tempdir",  pMrgCSetEntryConflict->pPathTempDirForFile)  );

			SG_ERR_CHECK(  _store_temp_path(pCtx, pvhIssue, pMrg, "repopath_tempfile_ancestor", pMrgCSetEntryConflict->pPathTempFile_Ancestor)  );
			SG_ERR_CHECK(  _store_temp_path(pCtx, pvhIssue, pMrg, "repopath_tempfile_baseline", pMrgCSetEntryConflict->pPathTempFile_Baseline)  );
			SG_ERR_CHECK(  _store_temp_path(pCtx, pvhIssue, pMrg, "repopath_tempfile_other",    pMrgCSetEntryConflict->pPathTempFile_Other)  );
			SG_ERR_CHECK(  _store_temp_path(pCtx, pvhIssue, pMrg, "repopath_tempfile_result",   pMrgCSetEntryConflict->pPathTempFile_Result)  );

			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_file_ancestor", pMrgCSetEntryConflict->pMrgCSetEntry_Ancestor->bufHid_Blob)  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_file_baseline", pMrgCSetEntryConflict->pMrgCSetEntry_Baseline->bufHid_Blob)  );

			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "label_ancestor", SG_string__sz(pMrgCSetEntryConflict->pStringAncestorLabel))  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "label_baseline", SG_string__sz(pMrgCSetEntryConflict->pStringBaselineLabel))  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "label_other",    SG_string__sz(pMrgCSetEntryConflict->pStringOtherLabel))  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "label_result",   SG_string__sz(pMrgCSetEntryConflict->pStringResultLabel))  );

			SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pIter, pMrgCSetEntryConflict->prbUnique_File_HidBlob, &bFound, &pszKey_k, NULL)  );
			while (bFound)
			{
				if (strcmp(pszKey_k, pMrgCSetEntryConflict->pMrgCSetEntry_Baseline->bufHid_Blob) != 0)
				{
					SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "hid_file_other", pszKey_k)  );
					break;
				}
				SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pIter, &bFound, &pszKey_k, NULL)  );
			}
			SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);

			if (pMrgCSetEntryConflict->pMergeTool)
			{
				const char* szName = NULL;

				SG_ERR_CHECK(  SG_filetool__get_name(pCtx, pMrgCSetEntryConflict->pMergeTool, &szName)  );
				SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvhIssue, "automerge_mergetool", szName)  );
			}
		}
	}

	if (bHaveCollision)
	{
		// list the set of entries (including this entry) fighting for this entryname in this directory.
		// we only list the GIDs of the entries; we assume that each of them will have their own ISSUE
		// reported (with another copy of this list).  we CANNOT give the CSET HID for the leaf which
		// made the change that caused the collision to happen because we do not know it (and it may be
		// the result of combination of factors).
		//
		//     "collisions" : [ "<gid_1>", "<gid_2>", ... ],

		SG_ASSERT(  (pMrgCSetEntry_FinalResult->pMrgCSetEntryCollision->pVec_MrgCSetEntry)  );
		SG_ERR_CHECK(  _make_gid_array__vec_mrg_cset_entry(pCtx,
														   &pva,
														   pMrgCSetEntry_FinalResult->pMrgCSetEntryCollision->pVec_MrgCSetEntry)  );
		SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhIssue, "collisions", &pva)  );	// this steals our varray
	}

	if (bHavePortability)
	{
		// add the entry's portability-log to the ISSUE.
		//
		//     "portability_details" : <see portability.c>
		//
		// TODO 2010/05/21 Is there any reason to try to clone the port-log and put that
		// TODO            in the ISSUE rather than just stealing this one?  Are there any
		// TODO            steps following this one that might need the log?

		SG_ASSERT(  (pMrgCSetEntry_FinalResult->pvaPortLog)  );
		SG_ERR_CHECK(  SG_vhash__add__varray(pCtx, pvhIssue, "portability_details", &pMrgCSetEntry_FinalResult->pvaPortLog)  );

		// TODO 2010/07/10 Shouldn't we NULL pMrgCSetEntry_FinalResult->pvaPortLog since we stole it?
	}

	if (!pMrg->pvaIssues)
		SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pMrg->pvaIssues)  );
	SG_ERR_CHECK(  SG_varray__append__vhash(pCtx, pMrg->pvaIssues, &pvhIssue)  );	// this steals our vhash

fail:
	SG_STRING_NULLFREE(pCtx, pStringRepoPath);
	SG_STRING_NULLFREE(pCtx, pStringRepoPathTempDirForFile);
	SG_VARRAY_NULLFREE(pCtx, pva);
	SG_VHASH_NULLFREE(pCtx, pvh);
	SG_VHASH_NULLFREE(pCtx, pvhIssue);
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
}

//////////////////////////////////////////////////////////////////

/**
 * Write an ISSUE to the array of ISSUES for each unresolved issue (conflict).
 * This is varary[vhash] and will be written to the wd.json so that later
 * commands can help the user RESOLVE them before they COMMIT.
 */
static void _sg_mrg__prepare_issues(SG_context * pCtx, SG_mrg * pMrg)
{
	// we only need to look at each entry in the final-result for conflicts.
	// unlike when building wd_plan, we do not need to look at the things
	// present in the baseline that were deleted from the final-result.

	SG_ERR_CHECK(  SG_rbtree__foreach(pCtx, pMrg->pMrgCSet_FinalResult->prbEntries,
									  _sg_mrg__prepare_issues__cb, pMrg)  );

	// TODO 2010/09/13 Consider adding a LEGEND to the ISSUES.
	// TODO            Something like "L0 --> <hid_cset_0>" or vice versa.

fail:
	return;
}

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_MRG__PRIVATE_BUILD_WD_ISSUES_H
