/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
 * sg_status_formatter.c
 *
 *  Created on: Jun 8, 2010
 *      Author: jeremy
 */


#include "sg.h"

static void _status_append_all_entries(
	SG_context * pCtx, 
	SG_uint32 nDiffSection, 
	SG_vhash * pVHashIn, 
	const char* szLinePrefix,
	const char* szEOL,
	SG_string * pstrOutput)
{
	SG_varray * pCurrentVarray = NULL;
	SG_vhash * pCurrentVhash = NULL;
	const char * pszSectionName = NULL;
	const char * pszPath = NULL;
	const char * pszType = NULL;
	SG_uint32 nEntriesCount = 0;
	SG_uint32 nEntriesIndex = 0;
	SG_int64 nMask = 0;
	SG_string * pstrFlavorText = NULL;
	SG_bool bFound = SG_FALSE;
	SG_bool bHasXAttrChange = SG_FALSE;
	SG_bool bHasAttrChange = SG_FALSE;
	SG_bool bIsDirectory = SG_FALSE;
	SG_bool bIsSubmodule = SG_FALSE;
	SG_uint32 sumChanges = 0;

	pszSectionName = SG_treediff__get_status_section_name(nDiffSection);
	SG_ERR_CHECK(  SG_vhash__has(pCtx, pVHashIn, pszSectionName, &bFound)  );
	if (bFound == SG_FALSE)
		return;
	SG_vhash__get__varray(pCtx, pVHashIn, pszSectionName, &pCurrentVarray);
	SG_ERR_CHECK(  SG_varray__count(pCtx, pCurrentVarray, &nEntriesCount)  );
	for (nEntriesIndex = 0; nEntriesIndex < nEntriesCount; nEntriesIndex++)
	{
		SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstrFlavorText)  );

		SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pCurrentVarray, nEntriesIndex, &pCurrentVhash)  );
		SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "path", &pszPath)  );
		SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pCurrentVhash, "flags", &nMask)  );
		SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "type", &pszType)  );
		bIsDirectory = (strcmp(pszType, "Directory") == 0);
		bIsSubmodule = (strcmp(pszType, "Submodule") == 0);

		if (bIsSubmodule)
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "\t(Submodule)")  );

#if 0 && defined(DEBUG)
		{
			// Add GID to the end of the line for debugging.
			const char * pszGid;
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "gid", &pszGid)  );
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "\t(gid %s)(flags 0x%x)",
													 pszGid, (SG_uint32)nMask)  );
		}
#endif
		if (nDiffSection == SG_DIFFSTATUS_FLAGS__MOVED)
		{
			const char * pszOldPath = NULL;
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "from", &pszOldPath)  );
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "%s%s\t\t# was at %s", szEOL, szLinePrefix, pszOldPath)  );
		}
		else if (nDiffSection == SG_DIFFSTATUS_FLAGS__RENAMED)
		{
			const char * pszOldName = NULL;
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "oldname", &pszOldName)  );
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "%s%s\t\t# was %s", szEOL, szLinePrefix, pszOldName)  );
		}

		SG_ERR_CHECK(  SG_vhash__has(pCtx, pCurrentVhash, "changed_attr", &bHasAttrChange)  );
		SG_ERR_CHECK(  SG_vhash__has(pCtx, pCurrentVhash, "changed_xattr", &bHasXAttrChange)  );
		if (bHasAttrChange || bHasXAttrChange)
		{
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "%s%s\t\t# changed attributes", szEOL, szLinePrefix)  );
		}

		if (bIsDirectory)
			nMask &= ~SG_DIFFSTATUS_FLAGS__MODIFIED;

		sumChanges = 0;
		if (nMask & SG_DIFFSTATUS_FLAGS__MASK_MUTUALLY_EXCLUSIVE)
			sumChanges++;
		if (nMask & SG_DIFFSTATUS_FLAGS__RENAMED)
			sumChanges++;
		if (nMask & SG_DIFFSTATUS_FLAGS__MOVED)
			sumChanges++;
		if (nMask & SG_DIFFSTATUS_FLAGS__CHANGED_XATTRS)
			sumChanges++;
		if (nMask & SG_DIFFSTATUS_FLAGS__CHANGED_ATTRBITS)
			sumChanges++;
		if (sumChanges > 1)
			SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "%s%s\t\t# There are multiple changes to this item", szEOL, szLinePrefix)  );

		if (bIsSubmodule)
		{
			if (nDiffSection & SG_DIFFSTATUS_FLAGS__MODIFIED)
			{
				const char * pszOriginalDescription;
				const char * pszCurrentDescription;
				SG_bool bIsDirty;

				SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "submodule_original", &pszOriginalDescription)  );
				SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "submodule_current", &pszCurrentDescription)  );
				SG_ERR_CHECK(  SG_vhash__get__bool(pCtx, pCurrentVhash, "submodule_is_dirty", &bIsDirty)  );

				if (strcmp(pszOriginalDescription, pszCurrentDescription) != 0)
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText,
															 ("%s%s"
															  "\t\t# Submodule baseline changed:%s"
															  "%s\t\t#      was %s%s"
															  "%s\t\t#      now %s"),
															  szEOL, szLinePrefix,
															  szEOL, szLinePrefix,
															 pszOriginalDescription,
															 szEOL, szLinePrefix,
															 pszCurrentDescription)  );
				if (bIsDirty)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, 
						"%s%s\t\t# The working copy in this submodule is dirty.",
						szEOL, szLinePrefix)  );
				}
			}
			else if (nDiffSection & (SG_DIFFSTATUS_FLAGS__ADDED | SG_DIFFSTATUS_FLAGS__FOUND))
			{
				const char * pszCurrentDescription;
				SG_bool bIsDirty;

				SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "submodule_current", &pszCurrentDescription)  );
				SG_ERR_CHECK(  SG_vhash__get__bool(pCtx, pCurrentVhash, "submodule_is_dirty", &bIsDirty)  );

				SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, "%s%s\t\t# Submodule baseline: %s", szEOL, szLinePrefix, pszCurrentDescription)  );
				if (bIsDirty)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, 
						"%s%s\t\t# The working copy in this submodule is dirty.", 
						szEOL, szLinePrefix)  );
				}
			}
			else if (nDiffSection & (SG_DIFFSTATUS_FLAGS__DELETED | SG_DIFFSTATUS_FLAGS__LOST))
			{
				if (nMask & SG_DIFFSTATUS_FLAGS__HACK__DELETED_ADDED)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, 
															 "%s%s\t\t# Submodule was added and then deleted",
															 szEOL, szLinePrefix)  );
				}
				else if (nMask & SG_DIFFSTATUS_FLAGS__HACK__LOST_ADDED)
				{
					// This case happens when we have an ADDED+LOST submodule.
					// Since it wasn't present in the baseline, we don't have
					// any "original" info.

					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, 
															 "%s%s\t\t# Submodule was added but not present in baseline",
															 szEOL, szLinePrefix)  );
				}
				else
				{
					const char * pszOriginalDescription;
					SG_bool bHasOriginal;

					SG_ERR_CHECK(  SG_vhash__has(pCtx, pCurrentVhash, "submodule_original", &bHasOriginal)  );
					if (bHasOriginal)
					{
						SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "submodule_original", &pszOriginalDescription)  );
						SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrFlavorText, 
																 "%s%s\t\t# Submodule baseline was: %s", 
																 szEOL, szLinePrefix, pszOriginalDescription)  );
					}
				}
			}
		}

		SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput, "%s%8s:  %s%s%s", 
			szLinePrefix, pszSectionName, pszPath, SG_string__sz(pstrFlavorText), szEOL)  );
		SG_STRING_NULLFREE(pCtx, pstrFlavorText);
	}


fail:
	SG_STRING_NULLFREE(pCtx, pstrFlavorText);
}

static void _status_append_all_unresolved_conflicts(
	SG_context * pCtx,
	const SG_varray * pvaConflicts,
	SG_pendingtree * pPendingTree,
	const char* szLinePrefix,
	const char* szEOL,
	SG_string * pstrOutput)
{
	SG_vhash * pCurrentVhash = NULL;
	SG_string * pStrRepoPath = NULL;
	const char * pszGid = NULL;
	SG_uint32 nEntriesCount = 0;
	SG_uint32 nUnresolvedEntriesCount = 0;
	SG_uint32 nEntriesIndex = 0;
	const char * pszFlavorText = NULL;
	SG_int64 conflictFlags = 0;
	SG_int64 status64 = 0;
	SG_int64 tneType64 = 0;
	SG_treenode_entry_type tneType;
	SG_pendingtree_wd_issue_status status;

	if (pvaConflicts == NULL)
		return;
	SG_ERR_CHECK(  SG_varray__count(pCtx, pvaConflicts, &nEntriesCount)  );
	if (nEntriesCount == 0)
		return;

	for (nEntriesIndex = 0; nEntriesIndex < nEntriesCount; nEntriesIndex++)
	{
		SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pvaConflicts, nEntriesIndex, &pCurrentVhash)  );

		SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pCurrentVhash, "status", &status64)  );
		status = (SG_pendingtree_wd_issue_status)status64;
		if (status & SG_ISSUE_STATUS__MARKED_RESOLVED)
			continue;

		SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "gid", &pszGid)  );

		SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pCurrentVhash, "tne_type", &tneType64)  );
		tneType = (SG_treenode_entry_type)tneType64;

		// Compute the live/current repo-path for the entry.
		// 
		// WARNING: There currently is a "repopath" field on the conflict, *BUT* it
		//          may be stale.  And I want to say that it is deprecated and going
		//          away.
		// 
		// It was current/valid at the time that the MERGE was performed and before the
		// user was given a chance to modify the WD (either manually or with RESOLVE).
		// That repo-path may still be valid, but then again they may have moved/renamed
		// the item and/or moved/renamed a parent directory.  So try to compute a live
		// repo-path for the item using information in the pendingtree.  This repo-path
		// should match what is currently on disk *and* should match the value printed
		// in the non-conflict portion of the status report.
		//
		// TODO 2010/08/19 We currently have a bug where sometimes __find_repo_path_by_gid()
		// TODO            fails.  This happens when the user has modified the entry in the
		// TODO            WD such that it appears CLEAN relative to the BASELINE (and so
		// TODO            all of the stuff in pendingtree to strip out clean ptnodes bites
		// TODO            us).  What we really need to do is "pin" these ptnodes so that
		// TODO            they won't get stripped out if there's a chance we (STATUS, RESOLVE,
		// TODO            and etc.), might need them.
		// TODO
		// TODO            For now, let this fail, catch it and substitute a pathname with
		// TODO            a warning in it.

		SG_pendingtree__find_repo_path_by_gid(pCtx, pPendingTree, pszGid, &pStrRepoPath);
		if (SG_context__has_err(pCtx))
		{
			const char * pszPath = NULL;

			// TODO 2011/07/21 Remove this once we confirm the fix for Y4182.

			if (!SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
				SG_ERR_RETHROW_RETURN;
			
			SG_context__err_reset(pCtx);

			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "repopath", &pszPath)  );
			SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pStrRepoPath)  );
			SG_ERR_CHECK(  SG_string__sprintf(pCtx, pStrRepoPath, "TODO Fix Path for Display: %s", pszPath)  );
		}

		// TODO 2010/08/18 Look at the "collision_flags" and the "portability_flags"
		// TODO            and see if we should include some stuff here.

		SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pCurrentVhash, "conflict_flags", &conflictFlags)  );
		nUnresolvedEntriesCount++;

		switch (tneType)
		{
		default:
			SG_ERR_THROW(  SG_ERR_NOTIMPLEMENTED  );
			break;

		case SG_TREENODEENTRY_TYPE_REGULAR_FILE:
			{
				SG_bool bContentConflict   = ((conflictFlags &  SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__MASK__NOT_OK) != 0);
				SG_bool bNamespaceConflict = ((conflictFlags & ~SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_FILE_EDIT__MASK__NOT_OK) != 0);

				if (bContentConflict && bNamespaceConflict)
					pszFlavorText = "\t\t# namespace and content conflict";
				else if (bContentConflict)
					pszFlavorText = "\t\t# content conflict";
				else if (bNamespaceConflict)
					pszFlavorText = "\t\t# namespace conflict";
				else
					pszFlavorText = NULL;	// i'm not sure this is possible or necessary

				if (pszFlavorText)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput, "%8s:  %s%s%s%s%s%s", "Conflict", 
						SG_string__sz(pStrRepoPath), szEOL, szLinePrefix, pszFlavorText, szEOL, szLinePrefix)  );
				}
			}
			break;

		case SG_TREENODEENTRY_TYPE_SYMLINK:
			{
				SG_bool bContentConflict   = ((conflictFlags &  SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SYMLINK_EDIT) != 0);
				SG_bool bNamespaceConflict = ((conflictFlags & ~SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SYMLINK_EDIT) != 0);

				if (bContentConflict && bNamespaceConflict)
					pszFlavorText = "\t\t# namespace and content conflict";
				else if (bContentConflict)
					pszFlavorText = "\t\t# content conflict";
				else if (bNamespaceConflict)
					pszFlavorText = "\t\t# namespace conflict";
				else
					pszFlavorText = NULL;	// i'm not sure this is possible or necessary

				if (pszFlavorText)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput, "%8s:  %s%s%s%s%s%s", 
						"Conflict", SG_string__sz(pStrRepoPath), szEOL, szLinePrefix, pszFlavorText, szEOL, szLinePrefix)  );
				}
			}
			break;

		case SG_TREENODEENTRY_TYPE_DIRECTORY:
			{
				SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput, "%8s:  %s%s%s\t\t# content conflict%s", 
					"Conflict", SG_string__sz(pStrRepoPath), szEOL, szLinePrefix, szEOL)  );
			}
			break;

		case SG_TREENODEENTRY_TYPE_SUBMODULE:
			{
				SG_bool bContentConflict   = ((conflictFlags &  SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SUBMODULE_EDIT) != 0);
				SG_bool bNamespaceConflict = ((conflictFlags & ~SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_SUBMODULE_EDIT) != 0);

				if (bContentConflict && bNamespaceConflict)
					pszFlavorText = "\t\t# namespace and content conflict";
				else if (bContentConflict)
					pszFlavorText = "\t\t# content conflict";
				else if (bNamespaceConflict)
					pszFlavorText = "\t\t# namespace conflict";
				else
					pszFlavorText = NULL;	// i'm not sure this is possible or necessary

				if (pszFlavorText)
				{
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput, "%8s:  %s%s%s%s%s%s", 
						"Conflict", SG_string__sz(pStrRepoPath), szEOL, szLinePrefix, pszFlavorText, szEOL, szLinePrefix)  );
				}
			}
			break;
		}
		
		SG_STRING_NULLFREE(pCtx, pStrRepoPath);
	}
	if (nUnresolvedEntriesCount > 0)
	{
		SG_ERR_CHECK(  SG_string__append__format(pCtx, pstrOutput,
											 ("%s%s"
											  "# Use the 'vv resolve' command to view more details%s%s"
											  "# about your conflicts or to resolve them.%s"), 
											  szEOL, szLinePrefix, szEOL, szLinePrefix, szEOL)  );
	}

fail:
	SG_STRING_NULLFREE(pCtx, pStrRepoPath);
}

//////////////////////////////////////////////////////////////////

void SG_status_formatter__vhash_to_string(
	SG_context * pCtx,
	SG_vhash * pvhTreediffs,
	const SG_varray * pvaConflicts,
	SG_pendingtree * pPendingTree,
	const char* szLinePrefix,
	const char* szEOL,
	SG_string ** ppstrOutput)
{
	SG_string * pStrOutputTemp = NULL;

	SG_NULLARGCHECK_RETURN(ppstrOutput);

	if (!szLinePrefix)
		szLinePrefix = "";
	if (!szEOL)
		szEOL = SG_PLATFORM_NATIVE_EOL_STR;

	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pStrOutputTemp)  );

	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__ADDED, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
			// TODO 2010/11/15 Revisit how we combine MODIFIED/ATTRBITS/XATTRS for STATUS.
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__MODIFIED, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__DELETED, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__RENAMED, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__MOVED, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__LOST, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );
	SG_ERR_CHECK(  _status_append_all_entries(pCtx, SG_DIFFSTATUS_FLAGS__FOUND, pvhTreediffs, szLinePrefix, szEOL, pStrOutputTemp)  );

	SG_ERR_CHECK(  _status_append_all_unresolved_conflicts(pCtx, pvaConflicts, pPendingTree, szLinePrefix, szEOL, pStrOutputTemp)  );

	SG_RETURN_AND_NULL(pStrOutputTemp, ppstrOutput);

	/* Fall through to common cleanup */
fail:
	SG_STRING_NULLFREE(pCtx, pStrOutputTemp);
}
