/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_treenode_submodule__private_ctors_dtors.h
 *
 * @details 
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_TREENODE_SUBMODULE__PRIVATE_CTORS_DTORS_H
#define H_SG_TREENODE_SUBMODULE__PRIVATE_CTORS_DTORS_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

static _sg_treenode_submodule_impl * _impl_table[] = {
	&_impl__veracity,
};

//////////////////////////////////////////////////////////////////

static void _sg_treenode_submodule__find_impl(SG_context * pCtx,
											  const char * pszTech,
											  const _sg_treenode_submodule_impl ** ppImpl)
{
	SG_uint32 k;

	for (k=0; k<SG_NrElements(_impl_table); k++)
	{
		if (strcmp(pszTech, _impl_table[k]->pszTech) == 0)
		{
			*ppImpl = _impl_table[k];
			return;
		}
	}

	SG_ERR_THROW2_RETURN(  SG_ERR_INVALIDARG,
						   (pCtx, "Unrecognized submodule technology '%s'.", pszTech)  );
}

static void _sg_treenode_submodule__find_and_set_impl(SG_context * pCtx,
													  SG_treenode_submodule * pTSM)
{
	const char * pszTech;

	SG_NULLARGCHECK_RETURN( pTSM );
	SG_ARGCHECK_RETURN( (pTSM->pImpl == NULL), pTsm->pImpl );

	SG_ERR_CHECK_RETURN(  SG_treenode_submodule__get_tech(pCtx, pTSM, &pszTech)  );
	SG_ERR_CHECK_RETURN(  _sg_treenode_submodule__find_impl(pCtx, pszTech, &pTSM->pImpl)  );
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__load_from_repo(SG_context * pCtx,
										   SG_repo * pRepo,
										   const char * pszHid,
										   SG_treenode_submodule ** ppTSM)
{
	SG_byte * pBuf = NULL;
	SG_treenode_submodule * pTSM = NULL;
    SG_uint64 len = 0;

	SG_NULLARGCHECK_RETURN( pRepo );
	SG_NULLARGCHECK_RETURN( pszHid );
	SG_NULLARGCHECK_RETURN( ppTSM );

	SG_ERR_CHECK(  SG_repo__fetch_blob_into_memory(pCtx, pRepo, pszHid, &pBuf, &len)  );

#if 0 && TRACE_SUB
	SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
							   "Submodule_LoadFromRepo: [hid %s][blob %s] into %p\n",
							   pszHid, pBuf, pTSM)  );
#endif

	SG_ERR_CHECK(  SG_alloc1(pCtx, pTSM)  );
	SG_ERR_CHECK(  SG_VHASH__ALLOC__FROM_JSON__BUFLEN(pCtx, &pTSM->pvh, (const char *)pBuf, (SG_uint32) len)  );
	SG_ERR_CHECK(  SG_STRDUP(pCtx, pszHid, &pTSM->pszHidFrozen)  );
	SG_ERR_CHECK(  _sg_treenode_submodule__find_and_set_impl(pCtx, pTSM)  );

	pTSM->pPathWD = NULL;
	pTSM->pVoidData = NULL;

	SG_ERR_CHECK(  _sg_treenode_submodule__validate(pCtx, pTSM)  );

	SG_NULLFREE(pCtx, pBuf);

#if 0 && TRACE_SUB
	SG_ERR_IGNORE(  SG_treenode_submodule_debug__dump_to_console(pCtx, pTSM, "Submodule Loaded")  );
#endif

	*ppTSM = pTSM;
	return;

fail:
	SG_NULLFREE(pCtx, pBuf);
	SG_TREENODE_SUBMODULE_NULLFREE(pCtx, pTSM);
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__sniff_directory(SG_context * pCtx,
											const SG_pathname * pPath,
											SG_repo * pRepoOuter,
											SG_bool * pbIsWD,
											SG_treenode_submodule ** ppTSM)
{
	SG_bool bIsWD = SG_FALSE;
	SG_uint32 k;
	
	SG_NULLARGCHECK_RETURN( pPath );
	SG_NULLARGCHECK_RETURN( pbIsWD );
	// ppTSM is optional

	// Give each supported DVCS a chance to claim the directory.

	for (k=0; k<SG_NrElements(_impl_table); k++)
	{
		SG_ERR_CHECK_RETURN(  (*_impl_table[k]->pfn_sniff_and_allocate)(pCtx,
																		_impl_table[k],
																		pPath,
																		pRepoOuter,
																		&bIsWD,
																		ppTSM)  );
		if (bIsWD)
		{
#if defined(DEBUG)
			if (ppTSM)
			{
				SG_ASSERT(  (*ppTSM)  );
				SG_ASSERT(  ((*ppTSM)->pImpl)  );
				SG_ASSERT(  ((*ppTSM)->pPathWD)  );
				// ((*ppTSM)->pVoidData) is optional
			}
#endif

#if 0 && TRACE_SUB
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
									   "TreenodeSubmodule_SniffDirectory: Found [tech '%s'] in '%s'.\n",
									   _impl_table[k]->pszTech,
									   SG_pathname__sz(pPath))  );
			if (ppTSM)
				SG_ERR_IGNORE(  SG_treenode_submodule_debug__dump_to_console(pCtx, *ppTSM, "Submodule Sniff")  );
#endif

			*pbIsWD = SG_TRUE;
			return;
		}
	}
	
	*pbIsWD = SG_FALSE;
	if (ppTSM)
		*ppTSM = NULL;
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__save_to_repo(SG_context * pCtx,
										 SG_treenode_submodule * pTSM,
										 SG_repo * pRepoOuter,
										 SG_repo_tx_handle * pRepoTx,
										 const char * pszgid,
										 SG_int64 i_objectid_timestamp,
										 SG_uint64 * piBlobFullLength)
{
	char * pszHid_new = NULL;
	SG_string * pStringJSON = NULL;

	SG_NULLARGCHECK_RETURN( pTSM );
	SG_NULLARGCHECK_RETURN( pRepoOuter );

	// TODO 2010/11/11 We froze the vhash during the alloc (either __sniff or __load)
	// TODO            and already have the HID, but we didn't save the JSON, so we
	// TODO            recompute it here and verify that nothing changed.  Think about
	// TODO            making the vhash read-only and preserving the JSON so that we
	// TODO            don't need to recompute it here.

	SG_ERR_CHECK(  _sg_treenode_submodule__to_json(pCtx, pTSM, &pStringJSON)  );

	// save the JSON to a blob.
 
	SG_repo__store_blob_from_memory(pCtx, pRepoOuter, pRepoTx, pszgid, i_objectid_timestamp, SG_TRUE,
									(const SG_byte *)SG_string__sz(pStringJSON),
									SG_string__length_in_bytes(pStringJSON),
									&pszHid_new);
	if (SG_context__has_err(pCtx) && !SG_context__err_equals(pCtx, SG_ERR_BLOBFILEALREADYEXISTS))
		SG_ERR_RETHROW;

	if (pTSM->pszHidFrozen)
	{
		SG_ASSERT(  (strcmp(pTSM->pszHidFrozen, pszHid_new) == 0)  );
	}
	else
	{
		pTSM->pszHidFrozen = pszHid_new;
		pszHid_new = NULL;
	}
	
	if (piBlobFullLength)
		*piBlobFullLength = SG_string__length_in_bytes(pStringJSON);
		
fail:
	SG_STRING_NULLFREE(pCtx, pStringJSON);
	SG_NULLFREE(pCtx, pszHid_new);
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__free(SG_context * pCtx, SG_treenode_submodule * pTSM)
{
	if (!pTSM)
		return;

#if 0 && TRACE_SUB
	SG_ERR_IGNORE(  SG_treenode_submodule_debug__dump_to_console(pCtx, pTSM, "Submodule Free")  );
#endif

	// let implementation free pVoidData if necessary.

	if (pTSM->pImpl)
	{
		SG_ERR_CHECK_RETURN(  (*pTSM->pImpl->pfn_dtor)(pCtx, pTSM)  );
		pTSM->pImpl = NULL;
	}

	SG_VHASH_NULLFREE(pCtx, pTSM->pvh);
	SG_NULLFREE(pCtx, pTSM->pszHidFrozen);
	SG_PATHNAME_NULLFREE(pCtx, pTSM->pPathWD);

	SG_NULLFREE(pCtx, pTSM);
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule__alloc__copy(SG_context * pCtx,
										SG_treenode_submodule ** ppTSM_new,
										const SG_treenode_submodule * pTSM_src)
{
	SG_treenode_submodule * pTSM_copy = NULL;

	SG_ERR_CHECK(  SG_alloc1(pCtx, pTSM_copy)  );
	SG_ERR_CHECK(  SG_VHASH__ALLOC__COPY(pCtx, &pTSM_copy->pvh, pTSM_src->pvh)  );
	SG_ERR_CHECK(  SG_STRDUP(pCtx, pTSM_src->pszHidFrozen, &pTSM_copy->pszHidFrozen)  );

	pTSM_copy->pImpl = pTSM_src->pImpl;

	if (pTSM_src->pPathWD)
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__COPY(pCtx, &pTSM_copy->pPathWD, pTSM_src->pPathWD)  );

	if (pTSM_src->pVoidData)
		SG_ERR_CHECK(  (*pTSM_src->pImpl->pfn_copy)(pCtx, pTSM_src, pTSM_copy)  );

#if 0 && TRACE_SUB
	SG_ERR_IGNORE(  SG_treenode_submodule_debug__dump_to_console(pCtx, pTSM_copy, "Submodule AllocCopy")  );
#endif

	*ppTSM_new = pTSM_copy;
	return;

fail:
	SG_TREENODE_SUBMODULE_NULLFREE(pCtx, pTSM_copy);
}

//////////////////////////////////////////////////////////////////

void SG_treenode_submodule_instance_data_update__free(SG_context * pCtx,
													  SG_treenode_submodule_instance_data_update * pTSIDU)
{
	if (!pTSIDU)
		return;

	SG_ERR_IGNORE(  (*pTSIDU->pfn_dtor)(pCtx, pTSIDU)  );
}

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_TREENODE_SUBMODULE__PRIVATE_CTORS_DTORS_H
