/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include <sg.h>

static void sg_vc_locks__add(
	SG_context* pCtx, 
	SG_repo* pRepo, 
	const char* psz_csid,
	const char* psz_branch,
	SG_vhash* pvh_gids,
	const SG_audit* pq
	)
{
	char* psz_hid_cs_leaf = NULL;
	SG_zingtx* pztx = NULL;
	SG_zingrecord* prec = NULL;
	SG_dagnode* pdn = NULL;
	SG_changeset* pcs = NULL;
	SG_zingtemplate* pzt = NULL;
	SG_zingfieldattributes* pzfa = NULL;
	SG_uint32 count_gids = 0;
	SG_uint32 i_gid = 0;

	SG_ERR_CHECK(  SG_zing__get_leaf(pCtx, pRepo, NULL, SG_DAGNUM__VC_LOCKS, &psz_hid_cs_leaf)  );

	/* start a changeset */
	SG_ERR_CHECK(  SG_zing__begin_tx(pCtx, pRepo, SG_DAGNUM__VC_LOCKS, pq->who_szUserId, psz_hid_cs_leaf, &pztx)  );
	SG_ERR_CHECK(  SG_zingtx__add_parent(pCtx, pztx, psz_hid_cs_leaf)  );
	SG_ERR_CHECK(  SG_zingtx__get_template(pCtx, pztx, &pzt)  );

	SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids, &count_gids)  );

	for (i_gid=0; i_gid<count_gids; i_gid++)
	{
		const char* psz_gid = NULL;

		SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_gids, i_gid, &psz_gid, NULL)  );

		SG_ERR_CHECK(  SG_zingtx__create_new_record(pCtx, pztx, "lock", &prec)  );

		SG_ERR_CHECK(  SG_zingtemplate__get_field_attributes(pCtx, pzt, "lock", "csid", &pzfa)  );
		SG_ERR_CHECK(  SG_zingrecord__set_field__string(pCtx, prec, pzfa, psz_csid) );

		SG_ERR_CHECK(  SG_zingtemplate__get_field_attributes(pCtx, pzt, "lock", "branch", &pzfa)  );
		SG_ERR_CHECK(  SG_zingrecord__set_field__string(pCtx, prec, pzfa, psz_branch) );
		SG_ERR_CHECK(  SG_zingtemplate__get_field_attributes(pCtx, pzt, "lock", "gid", &pzfa)  );
		SG_ERR_CHECK(  SG_zingrecord__set_field__string(pCtx, prec, pzfa, psz_gid) );
		SG_ERR_CHECK(  SG_zingtemplate__get_field_attributes(pCtx, pzt, "lock", "userid", &pzfa)  );
		SG_ERR_CHECK(  SG_zingrecord__set_field__userid(pCtx, prec, pzfa, pq->who_szUserId) );
	}

	/* commit the changes */
	SG_ERR_CHECK(  SG_zing__commit_tx(pCtx, pq->when_int64, &pztx, &pcs, &pdn, NULL)  );

	// fall thru

fail:
	if (pztx)
	{
		SG_ERR_IGNORE(  SG_zing__abort_tx(pCtx, &pztx)  );
	}
	SG_NULLFREE(pCtx, psz_hid_cs_leaf);
	SG_DAGNODE_NULLFREE(pCtx, pdn);
	SG_CHANGESET_NULLFREE(pCtx, pcs);
}

static void sg_vc_locks__delete(
	SG_context* pCtx, 
	SG_repo* pRepo, 
	SG_vhash* pvh_hids,
	const SG_audit* pq
	)
{
	SG_zingtx* pztx = NULL;
	SG_dagnode* pdn = NULL;
	SG_changeset* pcs = NULL;
	char* psz_hid_cs_leaf = NULL;
	SG_uint32 count = 0;
	SG_uint32 i = 0;

	SG_ERR_CHECK(  SG_zing__get_leaf(pCtx, pRepo, NULL, SG_DAGNUM__VC_LOCKS, &psz_hid_cs_leaf)  );

	SG_ERR_CHECK(  SG_zing__begin_tx(pCtx, pRepo, SG_DAGNUM__VC_LOCKS, pq->who_szUserId, psz_hid_cs_leaf, &pztx)  );
	SG_ERR_CHECK(  SG_zingtx__add_parent(pCtx, pztx, psz_hid_cs_leaf)  );

	SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_hids, &count)  );
	for (i=0; i<count; i++)
	{
		const char* psz_hidrec = NULL;

		SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_hids, i, &psz_hidrec, NULL)  );
		SG_ERR_CHECK(  SG_zingtx__delete_record__hid(pCtx, pztx, psz_hidrec)  );
	}

	/* commit the changes */
	SG_ERR_CHECK(  SG_zing__commit_tx(pCtx, pq->when_int64, &pztx, &pcs, &pdn, NULL)  );

	// fall thru

fail:
	if (pztx)
	{
		SG_ERR_IGNORE(  SG_zing__abort_tx(pCtx, &pztx)  );
	}
	SG_DAGNODE_NULLFREE(pCtx, pdn);
	SG_CHANGESET_NULLFREE(pCtx, pcs);
	SG_NULLFREE(pCtx, psz_hid_cs_leaf);
}

static void sg_vc_locks__list_all(
	SG_context* pCtx, 
	SG_repo* pRepo, 
	SG_varray** ppva
	)
{
	char* psz_hid_cs_leaf = NULL;
	SG_varray* pva_fields = NULL;
	SG_varray* pva = NULL;
	SG_vhash* pvh_username_join = NULL;

	SG_ERR_CHECK(  SG_zing__get_leaf(pCtx, pRepo, NULL, SG_DAGNUM__VC_LOCKS, &psz_hid_cs_leaf)  );

	SG_ASSERT(psz_hid_cs_leaf);

	SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva_fields)  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "branch")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "gid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "userid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "csid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, SG_ZING_FIELD__HIDREC)  );
	SG_ERR_CHECK(  SG_varray__appendnew__vhash(pCtx, pva_fields, &pvh_username_join)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "type", "username")  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "userid", SG_AUDIT__USERID)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "alias", SG_AUDIT__USERNAME)  );

	SG_ERR_CHECK(  SG_repo__dbndx__query(pCtx, pRepo, SG_DAGNUM__VC_LOCKS, psz_hid_cs_leaf, NULL, NULL, NULL, 0, 0, pva_fields, &pva)  );

	*ppva = pva;
	pva = NULL;

fail:
	SG_VARRAY_NULLFREE(pCtx, pva);
	SG_VARRAY_NULLFREE(pCtx, pva_fields);
	SG_NULLFREE(pCtx, psz_hid_cs_leaf);
}

static void sg_vc_locks__by_branch(
	SG_context* pCtx, 
	SG_repo* pRepo,
	SG_vhash** ppvh_by_branch
	)
{
	SG_varray* pva_locks = NULL;
	SG_vhash* pvh_by_branch = NULL;
	SG_uint32 count_locks = 0;

	// ----------------------------------------------------------------
	// build a list of all the locks, organized by branch
	// ----------------------------------------------------------------

	SG_ERR_CHECK(  sg_vc_locks__list_all(pCtx, pRepo, &pva_locks)  );
	SG_ERR_CHECK(  SG_varray__count(pCtx, pva_locks, &count_locks)  );

	if (count_locks > 0)
	{
		SG_uint32 i_lock = 0;

		SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_by_branch)  );
		for (i_lock=0; i_lock<count_locks; i_lock++)
		{
			SG_vhash* pvh_record = NULL;
			const char* psz_branch = NULL;
			const char* psz_csid = NULL;
			const char* psz_userid = NULL;
			const char* psz_gid = NULL;
			const char* psz_hidrec = NULL;
			SG_vhash* pvh_gids_for_one_branch = NULL;
			SG_vhash* pvh_locks_for_one_gid = NULL;

			// grab data for this record
			SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_locks, i_lock, &pvh_record)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "branch", &psz_branch)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "gid", &psz_gid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "userid", &psz_userid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "csid", &psz_csid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, SG_ZING_FIELD__HIDREC, &psz_hidrec)  );

			SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_by_branch, psz_branch, &pvh_gids_for_one_branch)  );
			if (!pvh_gids_for_one_branch)
			{
				SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_by_branch, psz_branch, &pvh_gids_for_one_branch)  );
			}

			SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_gids_for_one_branch, psz_gid, &pvh_locks_for_one_gid)  );
			if (!pvh_locks_for_one_gid)
			{
				SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_gids_for_one_branch, psz_gid, &pvh_locks_for_one_gid)  );
			}

			{
				SG_vhash* pvh_copy = NULL;

				SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_locks_for_one_gid, psz_hidrec, &pvh_copy)  );
				SG_ERR_CHECK(  SG_vhash__copy_some_items(
					pCtx, 
					pvh_record, 
					pvh_copy,
					"username",
					"userid",
					"csid",
					NULL
					)  );
			}
		}
	}

	if (ppvh_by_branch)
	{
		*ppvh_by_branch = pvh_by_branch;
		pvh_by_branch = NULL;
	}

fail:
	SG_VARRAY_NULLFREE(pCtx, pva_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_by_branch);
}

static void sg_vc_locks__check_for_problems(
	SG_context* pCtx, 
	SG_repo* pRepo,
	SG_vhash* pvh_pile,
	SG_vhash* pvh_locks_by_branch,
	SG_vhash** ppvh_duplicate_locks,
	SG_vhash** ppvh_violated_locks
	)
{
	SG_vhash* pvh_branches = NULL;
	SG_uint32 count_branches_with_locks = 0;
	SG_uint32 i_branch = 0;
	SG_vhash* pvh_duplicate_locks = NULL;
	SG_vhash* pvh_violated_locks = NULL;
    SG_dagnode* pdn_branch_csid = NULL;

	SG_NULLARGCHECK_RETURN(pvh_locks_by_branch);
	SG_NULLARGCHECK_RETURN(ppvh_duplicate_locks);
	SG_NULLARGCHECK_RETURN(ppvh_violated_locks);

	SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_branches)  );

	// ----------------------------------------------------------------
	// look for locks that have been violated
	// ----------------------------------------------------------------

	//SG_ERR_CHECK(  SG_vhash_debug__dump_to_console__named(pCtx, pvh_locks_by_branch, "Locks by branch")  );

	SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_by_branch, &count_branches_with_locks)  );
	for (i_branch=0; i_branch<count_branches_with_locks; i_branch++)
	{
		const char* psz_branch = NULL;
		SG_vhash* pvh_gids_for_one_branch = NULL;
		SG_vhash* pvh_branch_info = NULL;
		SG_vhash* pvh_branch_values = NULL;
		SG_uint32 count_branch_values = 0;
		SG_uint32 count_gids = 0;
		SG_uint32 i_gid = 0;

		SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_by_branch, i_branch, &psz_branch, &pvh_gids_for_one_branch)  );

		SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branches, psz_branch, &pvh_branch_info)  );
		SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_branch_values, &count_branch_values)  );

		// now go through the locks for this branch
		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
		for (i_gid=0; i_gid<count_gids; i_gid++)
		{
			const char* psz_gid = NULL;
			SG_vhash* pvh_locks_for_one_gid = NULL;
			SG_uint32 i_branch_value = 0;
			SG_uint32 count_locks = 0;
			SG_uint32 i_lock = 0;
			const char* psz_hidrec_lock = NULL;

			SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );

			SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
			for (i_lock=0; i_lock<count_locks; i_lock++)
			{
				SG_vhash* pvh_lock = NULL;
				const char* psz_lock_csid = NULL;
				const char* psz_lock_userid = NULL;

				SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hidrec_lock, &pvh_lock)  );
				SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "csid", &psz_lock_csid)  );
				SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "userid", &psz_lock_userid)  );

				for (i_branch_value=0; i_branch_value<count_branch_values; i_branch_value++)
				{
					const char* psz_branch_csid = NULL;

					SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_branch_values, i_branch_value, &psz_branch_csid, NULL)  );

					if (0 == strcmp(psz_branch_csid, psz_lock_csid))
					{
						// no problem here
					}
					else
					{
						SG_bool b_different = SG_FALSE;

                        SG_repo__fetch_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_branch_csid, &pdn_branch_csid);
                        if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
                        {
                            SG_ERR_DISCARD;

                            // I think if this branch record points to a version
                            // control changeset that we do not have, then it
                            // cannot be a path in which we violated the lock.
                        }
                        else
                        {
                            SG_ERR_CHECK_CURRENT;
                            SG_DAGNODE_NULLFREE(pCtx, pdn_branch_csid);

                            SG_ERR_CHECK(  SG_repo__tree__check_for_lock_violation(pCtx, pRepo, psz_gid, psz_lock_userid, psz_lock_csid, psz_branch_csid, &b_different)  );

                            if (b_different)
                            {
                                if (!pvh_violated_locks)
                                {
                                    SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_violated_locks)  );
                                }

                                {
                                    SG_vhash* pvh_copy = NULL;

                                    SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_violated_locks, psz_hidrec_lock, &pvh_copy)  );
                                    SG_ERR_CHECK(  SG_vhash__copy_items(pCtx, pvh_lock, pvh_copy)  );
                                    SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_copy, "gid", psz_gid)  );
                                    SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_copy, "branch", psz_branch)  );
                                }
                            }
                        }
					}
				}
			}
		}
	}

	// ----------------------------------------------------------------
	// look for duplicate locks
	// ----------------------------------------------------------------

	for (i_branch=0; i_branch<count_branches_with_locks; i_branch++)
	{
		const char* psz_branch = NULL;
		SG_vhash* pvh_gids_for_one_branch = NULL;
		SG_uint32 count_gids = 0;
		SG_uint32 i_gid = 0;

		SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_by_branch, i_branch, &psz_branch, &pvh_gids_for_one_branch)  );

		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
		for (i_gid=0; i_gid<count_gids; i_gid++)
		{
			SG_vhash* pvh_locks_for_one_gid = NULL;
			const char* psz_gid = NULL;
			SG_uint32 count_locks = 0;

			SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );
			SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
			if (count_locks > 1)
			{
				SG_vhash* pvh_dup_gids_for_one_branch = NULL;

				if (!pvh_duplicate_locks)
				{
					SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_duplicate_locks)  );
				}

				// add to pvh_duplicate_locks
				SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_duplicate_locks, psz_branch, &pvh_dup_gids_for_one_branch)  );
				if (!pvh_dup_gids_for_one_branch)
				{
					SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_duplicate_locks, psz_branch, &pvh_dup_gids_for_one_branch)  );
				}

				{
					SG_vhash* pvh_copy = NULL;

					SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_dup_gids_for_one_branch, psz_gid, &pvh_copy)  );
					SG_ERR_CHECK(  SG_vhash__copy_items(pCtx, pvh_locks_for_one_gid, pvh_copy)  );
				}
			}
		}
	}

	if (ppvh_violated_locks)
	{
		*ppvh_violated_locks = pvh_violated_locks;
		pvh_violated_locks = NULL;
	}

	if (ppvh_duplicate_locks)
	{
		*ppvh_duplicate_locks = pvh_duplicate_locks;
		pvh_duplicate_locks = NULL;
	}

fail:
    SG_DAGNODE_NULLFREE(pCtx, pdn_branch_csid);
	SG_VHASH_NULLFREE(pCtx, pvh_duplicate_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_violated_locks);
}

void SG_vc_locks__list_for_one_branch(
	SG_context* pCtx, 
	SG_repo* pRepo, 
	const char* psz_branch_name,
	SG_vhash** ppvh_locks
	)
{
	char* psz_hid_cs_leaf = NULL;
	SG_varray* pva_fields = NULL;
	SG_varray* pva = NULL;
	SG_varray* pva_crit = NULL;
	SG_vhash* pvh = NULL;
	SG_vhash* pvh_username_join = NULL;

	SG_ERR_CHECK(  SG_zing__get_leaf(pCtx, pRepo, NULL, SG_DAGNUM__VC_LOCKS, &psz_hid_cs_leaf)  );

	SG_ASSERT(psz_hid_cs_leaf);

	SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva_crit)  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_crit, "branch")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_crit, "==")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_crit, psz_branch_name)  );

	SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pva_fields)  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "gid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "userid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, "csid")  );
	SG_ERR_CHECK(  SG_varray__append__string__sz(pCtx, pva_fields, SG_ZING_FIELD__HIDREC)  );
	SG_ERR_CHECK(  SG_varray__appendnew__vhash(pCtx, pva_fields, &pvh_username_join)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "type", "username")  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "userid", SG_AUDIT__USERID)  );
	SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_username_join, "alias", SG_AUDIT__USERNAME)  );

	SG_ERR_CHECK(  SG_repo__dbndx__query(pCtx, pRepo, SG_DAGNUM__VC_LOCKS, psz_hid_cs_leaf, NULL, pva_crit, NULL, 0, 0, pva_fields, &pva)  );

	if (pva)
	{
		SG_uint32 count_records = 0;
		SG_uint32 i = 0;

		SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh)  );

		SG_ERR_CHECK(  SG_varray__count(pCtx, pva, &count_records)  );
		for (i=0; i<count_records; i++)
		{
			SG_vhash* pvh_record = NULL;
			const char* psz_csid = NULL;
			const char* psz_userid = NULL;
			const char* psz_username = NULL;
			const char* psz_gid = NULL;
			const char* psz_hidrec = NULL;
			SG_vhash* pvh_new = NULL;
			SG_vhash* pvh_locks_for_one_gid = NULL;

			// grab data for this record
			SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva, i, &pvh_record)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "gid", &psz_gid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "userid", &psz_userid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "username", &psz_username)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, "csid", &psz_csid)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_record, SG_ZING_FIELD__HIDREC, &psz_hidrec)  );

			SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh, psz_gid, &pvh_locks_for_one_gid)  );
			if (!pvh_locks_for_one_gid)
			{
				SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh, psz_gid, &pvh_locks_for_one_gid)  );
			}

			SG_ERR_CHECK(  SG_vhash__addnew__vhash(pCtx, pvh_locks_for_one_gid, psz_hidrec, &pvh_new)  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_new, "username", psz_username)  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_new, "userid", psz_userid)  );
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_new, "csid", psz_csid)  );
		}
	}

	*ppvh_locks = pvh;
	pvh = NULL;

fail:
	SG_VHASH_NULLFREE(pCtx, pvh);
	SG_VARRAY_NULLFREE(pCtx, pva_crit);
	SG_VARRAY_NULLFREE(pCtx, pva);
	SG_VARRAY_NULLFREE(pCtx, pva_fields);
	SG_NULLFREE(pCtx, psz_hid_cs_leaf);
}

void SG_vc_locks__unlock(
	SG_context* pCtx, 
	const char* psz_repo_mine,
	const char* psz_branch,
	SG_vhash* pvh_gids,
	SG_bool b_force,
	const SG_audit* pq
	)
{
	SG_repo* pRepo = NULL;
	SG_vhash* pvh_existing_locks = NULL;
	SG_vhash* pvh_hids = NULL;

	SG_NULLARGCHECK_RETURN(pvh_gids);
	SG_NULLARGCHECK_RETURN(psz_branch);
	SG_NULLARGCHECK_RETURN(psz_repo_mine);
	SG_NULLARGCHECK_RETURN(pq);

	SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_repo_mine, &pRepo)  );

	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_hids)  );

	// now find the locks to be deleted
	SG_ERR_CHECK(  SG_vc_locks__list_for_one_branch(pCtx, pRepo, psz_branch, &pvh_existing_locks)  );
	if (pvh_existing_locks)
	{
		SG_uint32 count_gids = 0;
		SG_uint32 i_gid = 0;

		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids, &count_gids)  );
		for (i_gid=0; i_gid<count_gids; i_gid++)
		{
			const char* psz_gid = NULL;
			SG_vhash* pvh_locks_for_one_gid = NULL;
			SG_uint32 count_to_be_deleted_for_this_gid = 0;

			SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_gids, i_gid, &psz_gid, NULL)  );
			SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_existing_locks, psz_gid, &pvh_locks_for_one_gid)  );
			if (pvh_locks_for_one_gid)
			{
				SG_uint32 count_locks = 0;
				SG_uint32 i_lock = 0;

				SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
				for (i_lock=0; i_lock<count_locks; i_lock++)
				{
					const char* psz_hidrec = NULL;
					SG_vhash* pvh_lock = NULL;
					SG_bool b_delete = SG_FALSE;

					SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hidrec, &pvh_lock)  );
					if (b_force)
					{
						b_delete = SG_TRUE;
					}
					else
					{
						const char* psz_userid = NULL;

						SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "userid", &psz_userid)  );
						if (0 == strcmp(psz_userid, pq->who_szUserId))
						{
							b_delete = SG_TRUE;
						}
					}

					if (b_delete)
					{
						count_to_be_deleted_for_this_gid++;
						SG_ERR_CHECK(  SG_vhash__add__null(pCtx, pvh_hids, psz_hidrec)  );
					}
				}
			}

			if (0 == count_to_be_deleted_for_this_gid)
			{
				SG_ERR_THROW(  SG_ERR_VC_LOCK_NOT_FOUND  );
			}
		}
	}

	// then delete the lock records
	SG_ERR_CHECK(  sg_vc_locks__delete(pCtx, pRepo, pvh_hids, pq)  );

fail:
	SG_VHASH_NULLFREE(pCtx, pvh_existing_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_hids);
	SG_REPO_NULLFREE(pCtx, pRepo);
}

void SG_vc_locks__request(
	SG_context* pCtx, 
	const char* psz_repo_mine,
	const char* psz_repo_upstream,
	const char* psz_csid,
	const char* psz_branch,
	SG_vhash* pvh_gids,
	const SG_audit* pq
	)
{
	SG_vhash* pvh_pile = NULL;
	SG_repo* pRepo = NULL;
	SG_vhash* pvh_branches = NULL;
	SG_vhash* pvh_existing_locks = NULL;
	SG_bool b_has = SG_FALSE;

	SG_NULLARGCHECK_RETURN(pvh_gids);
	SG_NULLARGCHECK_RETURN(psz_branch);
	SG_NULLARGCHECK_RETURN(psz_csid);
	SG_NULLARGCHECK_RETURN(psz_repo_upstream);
	SG_NULLARGCHECK_RETURN(psz_repo_mine);
	SG_NULLARGCHECK_RETURN(pq);

	// first we need to pull the branches and locks from the other side

	SG_ERR_CHECK(  SG_vc_locks__pull_locks_and_branches(pCtx, psz_repo_upstream, psz_repo_mine)  );

	SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_repo_mine, &pRepo)  );

	// do the usual check for branch/lock problems.  at the end of this
	// operation, we're going to push the branch/locks dags back up.  We
	// can't do that if any of the branches need to be merged or if any
	// locks have been violated.

	SG_ERR_CHECK(  SG_vc_locks__ensure__okay_to_push(
		pCtx, 
		pRepo, 
		&pvh_pile,
		NULL // TODO could get locks_by_branch and save a call below
		)  );

	// now make sure the branch exists and the WD is up-to-date with it
	SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_branches)  );
	SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_branches, psz_branch, &b_has)  );
	if (b_has)
	{
		const char* psz_branch_csid = NULL;
		SG_vhash* pvh_branch_info = NULL;
		SG_vhash* pvh_branch_values = NULL;

		SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branches, psz_branch, &pvh_branch_info)  );
		SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
		SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_branch_values, 0, &psz_branch_csid, NULL)  );
		if (0 != strcmp(psz_branch_csid, psz_csid))
		{
			SG_ERR_THROW(  SG_ERR_VC_LOCK_BRANCH_MUST_BE_CURRENT  );
		}
	}
	else
	{
		SG_ERR_THROW(  SG_ERR_BRANCH_NOT_FOUND  );
	}

	// now check to see if the requested locks conflict with
	// any existing locks
	SG_ERR_CHECK(  SG_vc_locks__list_for_one_branch(pCtx, pRepo, psz_branch, &pvh_existing_locks)  );
	if (pvh_existing_locks)
	{
		SG_uint32 count_gids = 0;
		SG_uint32 i_gid = 0;

		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids, &count_gids)  );
		for (i_gid=0; i_gid<count_gids; i_gid++)
		{
			const char* psz_gid = NULL;
			SG_bool b_already = SG_FALSE;

			SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_gids, i_gid, &psz_gid, NULL)  );
			SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_existing_locks, psz_gid, &b_already)  );
			if (b_already)
			{
				// TODO we want to be able to include the path here
				// but we already converted everything to gids
				SG_ERR_THROW(  SG_ERR_VC_LOCK_ALREADY_LOCKED  );
			}
		}
	}

	// then create the lock records
	SG_ERR_CHECK(  sg_vc_locks__add(pCtx, pRepo, psz_csid, psz_branch, pvh_gids, pq)  );

	// then push the branches/locks dag back up
	SG_ERR_CHECK(  SG_vc_locks__push_locks_and_branches(pCtx, pRepo, psz_repo_upstream)  );

	// TODO then pull it back and see if a race condition occurred?

fail:
	SG_VHASH_NULLFREE(pCtx, pvh_existing_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_REPO_NULLFREE(pCtx, pRepo);
}

void SG_vc_locks__push_locks_and_branches(
		SG_context * pCtx,
		SG_repo* pRepo,
		const char * pszDestRepoDescriptorName
		)
	{
		SG_push* pPush = NULL;

		SG_ERR_CHECK(  SG_push__begin(pCtx, pRepo, pszDestRepoDescriptorName, &pPush)  );

		SG_ERR_CHECK(  SG_push__add(pCtx, pPush, SG_DAGNUM__VC_BRANCHES, NULL)  );
		SG_ERR_CHECK(  SG_push__add(pCtx, pPush, SG_DAGNUM__BASE_TO_AUDIT(SG_DAGNUM__VC_BRANCHES), NULL)  );

		SG_ERR_CHECK(  SG_push__add(pCtx, pPush, SG_DAGNUM__VC_LOCKS, NULL)  );
		SG_ERR_CHECK(  SG_push__add(pCtx, pPush, SG_DAGNUM__BASE_TO_AUDIT(SG_DAGNUM__VC_LOCKS), NULL)  );

		SG_ERR_CHECK(  SG_push__commit(pCtx, &pPush, SG_FALSE, NULL)  );

		/* fall through */
fail:
		if (pPush)
			SG_ERR_IGNORE(  SG_push__abort(pCtx, &pPush)  );
}

void SG_vc_locks__pull_locks_and_branches(
	SG_context * pCtx,
	const char* pszSrcRepoSpec,
	const char * pszDestRepoDescriptorName
	)
{
	SG_pull* pPull = NULL;

	SG_ERR_CHECK(  SG_pull__begin(pCtx, pszDestRepoDescriptorName, pszSrcRepoSpec, &pPull)  );

	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__VC_BRANCHES)  );
	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__BASE_TO_AUDIT(SG_DAGNUM__VC_BRANCHES))  );

	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__VC_LOCKS)  );
	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__BASE_TO_AUDIT(SG_DAGNUM__VC_LOCKS))  );

	SG_ERR_CHECK(  SG_pull__commit(pCtx, &pPull, NULL)  );

	/* fall through */
fail:
	if (pPull)
		SG_ERR_IGNORE(  SG_pull__abort(pCtx, &pPull)  );
}

void SG_vc_locks__pull_locks(
	SG_context * pCtx,
	const char* pszSrcRepoSpec,
	const char * pszDestRepoDescriptorName
	)
{
	SG_pull* pPull = NULL;

	SG_ERR_CHECK(  SG_pull__begin(pCtx, pszDestRepoDescriptorName, pszSrcRepoSpec, &pPull)  );

	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__VC_LOCKS)  );
	SG_ERR_CHECK(  SG_pull__add__dagnum(pCtx, pPull, SG_DAGNUM__BASE_TO_AUDIT(SG_DAGNUM__VC_LOCKS))  );

	SG_ERR_CHECK(  SG_pull__commit(pCtx, &pPull, NULL)  );

	/* fall through */
fail:
	if (pPull)
		SG_ERR_IGNORE(  SG_pull__abort(pCtx, &pPull)  );
}

void SG_vc_locks__check__okay_to_push(
	SG_context* pCtx, 
	SG_repo* pRepo,
	SG_vhash** ppvh_pile,
	SG_vhash** ppvh_locks_by_branch,
	SG_vhash** ppvh_duplicate_locks,
	SG_vhash** ppvh_violated_locks
	)
{
	SG_vhash* pvh_pile = NULL;
	SG_vhash* pvh_locks_by_branch = NULL;
	SG_vhash* pvh_duplicate_locks = NULL;
	SG_vhash* pvh_violated_locks = NULL;

	SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );

	SG_ERR_CHECK(  sg_vc_locks__by_branch(pCtx, pRepo, &pvh_locks_by_branch)  );

	if (pvh_locks_by_branch)
	{
		SG_ERR_CHECK(  sg_vc_locks__check_for_problems(pCtx, pRepo, pvh_pile, pvh_locks_by_branch, &pvh_duplicate_locks, &pvh_violated_locks)  );
	}

	if (ppvh_pile)
	{
		*ppvh_pile = pvh_pile;
		pvh_pile = NULL;
	}

	if (ppvh_locks_by_branch)
	{
		*ppvh_locks_by_branch = pvh_locks_by_branch;
		pvh_locks_by_branch = NULL;
	}

	if (ppvh_duplicate_locks)
	{
		*ppvh_duplicate_locks = pvh_duplicate_locks;
		pvh_duplicate_locks = NULL;
	}

	if (ppvh_violated_locks)
	{
		*ppvh_violated_locks = pvh_violated_locks;
		pvh_violated_locks = NULL;
	}

fail:
	SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_VHASH_NULLFREE(pCtx, pvh_locks_by_branch);
	SG_VHASH_NULLFREE(pCtx, pvh_duplicate_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_violated_locks);
}

void SG_vc_locks__pre_commit(
	SG_context* pCtx, 
	SG_repo* pRepo,
    const char* psz_userid,
    const char* psz_tied_branch_name,
    SG_vhash** ppvh_result
    )
{
	SG_vhash* pvh_locks_by_branch = NULL;
    SG_vhash* pvh_gids_for_one_branch = NULL;
    SG_vhash* pvh_result = NULL;

	SG_NULLARGCHECK_RETURN(psz_userid);
	SG_NULLARGCHECK_RETURN(psz_tied_branch_name);
	SG_NULLARGCHECK_RETURN(ppvh_result);

    // TODO could be faster perhaps if we passed in the tied branch name
    // and the user id and used it in the query
	SG_ERR_CHECK(  sg_vc_locks__by_branch(pCtx, pRepo, &pvh_locks_by_branch)  );
    if (pvh_locks_by_branch)
    {
        SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_locks_by_branch, psz_tied_branch_name, &pvh_gids_for_one_branch)  );
        if (pvh_gids_for_one_branch)
        {
            SG_uint32 count_gids = 0;
            SG_uint32 i_gid = 0;

            SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
            for (i_gid=0; i_gid<count_gids; i_gid++)
            {
                const char* psz_gid = NULL;
                SG_vhash* pvh_locks_for_one_gid = NULL;
                SG_uint32 count_locks = 0;
                SG_uint32 i_lock = 0;
                const char* psz_hidrec_lock = NULL;

                SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );

                SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
                for (i_lock=0; i_lock<count_locks; i_lock++)
                {
                    SG_vhash* pvh_lock = NULL;
                    const char* psz_lock_userid = NULL;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hidrec_lock, &pvh_lock)  );
                    SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "userid", &psz_lock_userid)  );
                    if (0 != strcmp(psz_userid, psz_lock_userid))
                    {
                        if (!pvh_result)
                        {
                            SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_result)  );
                        }
                        SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_result, psz_gid, psz_lock_userid)  );
                    }
                }
            }
        }
    }

    *ppvh_result = pvh_result;
    pvh_result = NULL;

fail:
	SG_VHASH_NULLFREE(pCtx, pvh_result);
	SG_VHASH_NULLFREE(pCtx, pvh_locks_by_branch);
}

void SG_vc_locks__ensure__okay_to_push(
	SG_context* pCtx, 
	SG_repo* pRepo,
	SG_vhash** ppvh_pile,
	SG_vhash** ppvh_locks_by_branch
	)
{
	SG_vhash* pvh_pile = NULL;
	SG_vhash* pvh_locks_by_branch = NULL;
	SG_vhash* pvh_branches_needing_merge = NULL;
	SG_vhash* pvh_duplicate_locks = NULL;
	SG_vhash* pvh_violated_locks = NULL;

	SG_ERR_CHECK(  SG_vc_locks__check__okay_to_push(
		pCtx, 
		pRepo, 
		&pvh_pile,
		&pvh_locks_by_branch,
		&pvh_duplicate_locks,
		&pvh_violated_locks
		)  );

	if (pvh_branches_needing_merge)
	{
		SG_ERR_THROW(  SG_ERR_BRANCH_NEEDS_MERGE  );
	}
	if (pvh_duplicate_locks)
	{
		SG_ERR_THROW(  SG_ERR_VC_LOCK_DUPLICATE  );
	}
	if (pvh_violated_locks)
	{
		SG_ERR_THROW(  SG_ERR_VC_LOCK_VIOLATION  );
	}

	if (ppvh_pile)
	{
		*ppvh_pile = pvh_pile;
		pvh_pile = NULL;
	}

	if (ppvh_locks_by_branch)
	{
		*ppvh_locks_by_branch = pvh_locks_by_branch;
		pvh_locks_by_branch = NULL;
	}

fail:
	SG_VHASH_NULLFREE(pCtx, pvh_branches_needing_merge);
	SG_VHASH_NULLFREE(pCtx, pvh_violated_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_duplicate_locks);
	SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_VHASH_NULLFREE(pCtx, pvh_locks_by_branch);
}
