/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file sg_vv_verbs__status.h
 *
 * @details Routines to perform the common guts 'vv status'.
 *
 */

//////////////////////////////////////////////////////////////////

#ifndef H_SG_VV_VERBS__STATUS_H
#define H_SG_VV_VERBS__STATUS_H

BEGIN_EXTERN_C;

//////////////////////////////////////////////////////////////////

void SG_vv_verbs__status(
	SG_context * pCtx,
	SG_pendingtree* pPendingTree,
	const char* pszFromRev,
	const char* pszToRev,
	SG_uint32 count_file_args,
	const char** paszFileArgs,
	const SG_file_spec* pFilespec,
	SG_bool bRecursive,
	SG_bool bVerbose,
	const SG_varray** ppvaMergeIssues,
	SG_vhash** ppvhStatus)
{
	SG_treediff2* pTreeDiff = NULL;
	SG_bool bHaveIssuesIn;
	const SG_varray * pvaMergeIssues = NULL;
	SG_vhash* pvhStatus = NULL;

	SG_ERR_CHECK(  SG_pendingtree__diff_or_status(pCtx, pPendingTree, pszFromRev, pszToRev, &pTreeDiff)   );
	SG_ERR_CHECK(  SG_treediff2__file_spec_filter(pCtx, pTreeDiff, count_file_args, paszFileArgs, bRecursive, pFilespec)  );

	// __diff_or_status() freed pPendingTree->pSuperRoot as part of that
	// one operation per pendingtree issue.  reload it here so that
	// _status_formatter__ can compute live repo-paths for things.
	SG_ERR_CHECK(  SG_pendingtree__reload_pendingtree_from_vfile(pCtx, pPendingTree)  );

	if (ppvaMergeIssues)
		SG_ERR_CHECK(  SG_pendingtree__get_wd_issues__ref(pCtx, pPendingTree, &bHaveIssuesIn, &pvaMergeIssues)  );

	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvhStatus)  );
	SG_ERR_CHECK(  SG_treediff2__report_status_to_vhash(pCtx, pTreeDiff, pvhStatus, bVerbose)  );

	SG_RETURN_AND_NULL(pvhStatus, ppvhStatus);
	SG_RETURN_AND_NULL(pvaMergeIssues, ppvaMergeIssues);

	/* Fall through to common cleanup */
fail:
	SG_VHASH_NULLFREE(pCtx, pvhStatus);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
}

//////////////////////////////////////////////////////////////////

END_EXTERN_C;

#endif//H_SG_VV_VERBS__STATUS_H
