/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

load("vscript_test_lib.js");

function st_ssjs_wiki()
{
	this.setUp = function()
	{
		this.repo = sg.open_repo(repInfo.repoName);

		var serverFiles = sg.local_settings().machine.server.files;
		var core = serverFiles + "/core/";
		var wiki = serverFiles + "/modules/wiki/";
		load(serverFiles + "/dispatch.js");
		load(core + "vv.js");
		load(core + "textCache.js");
		load(wiki + "wiki.js");
		load(core + "session.js");
		
		this.username1 = "wiki1@sourcegear.com";
		this.username2 = "wiki2@sourcegear.com";
		
		this.repo.create_user(this.username1);
		this.repo.create_user(this.username2);
		
		{
			var zs = new zingdb(this.repo, sg.dagnum.USERS);
			var urec = zs.query('user', ['*'], "name == '" + this.username1 + "'");

			this.user1 = urec[0];
			urec = zs.query('user', ['*'], "name == '" + this.username2 + "'");

			this.user2 = urec[0];
			
			urec = zs.query('user', ['*'], "name == '" + repInfo.userName + "'");

			this.userId = urec[0].recid;
		}
	};
	
	this.tearDown = function()
	{
		if (this.repo)
			this.repo.close();
	};

	this.testConflictDescription = function()
	{
		var db = new zingdb(this.repo, sg.dagnum.WIKI);
		var ztx = null;

		try
		{
			ztx = db.begin_tx();
			var rec = ztx.new_record('page');
			var recid = rec.recid;
			rec.title = "testConflictDescription";
			rec.text = "line1\nline2\n";
			var ct = vv.txcommit(ztx);
			var csid = ct.csid;
			ztx = null;

			ztx = db.begin_tx(csid);
			rec = ztx.open_record('page', recid);
			rec.text = "line1\nline5\nline2\n";
			vv.txcommit(ztx);
			ztx = null;

			ztx = db.begin_tx(csid);
			rec = ztx.open_record('page', recid);
			rec.text = "line1\nline7\nline2\n";
			vv.txcommit(ztx);
			ztx = null;

			rec = db.get_record('page', recid);
			testlib.log("record after merge: " + sg.to_json__pretty_print(rec));

			var request = {
				'repo': this.repo,
				'pagename': "testConflictDescription"
			};

			var p = vvWikiModule.getPage(request);

			var ptext = p.text;

			testlib.ok(ptext.match(/^# Conflicting Edits Detected/), "conflict header expected");

			testlib.ok(! ptext.match(/ZING_MERGE_CONCAT/), "raw zing separator unexpected");
			testlib.ok(ptext.match(/# Conflicts with:/), "conflict separator expected");

			testlib.log(ptext);
		}
		finally
		{
			if (ztx)
				ztx.abort();
		}
	};
}