/*
Copyright 20102011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

var wikiActivitySource = {
	name: function()
	{
		return("wikiActivitySource");
	},

	dagsUsed: function()
	{
		return( [sg.dagnum.WIKI, sg.dagnum.USERS] );
	},

	getActivity: function(request, max, userid)
	{
		var repo = request.repo;
		var db = new zingdb(repo, sg.dagnum.WIKI);

		var recentMods = db.query__recent('page', ['*'],
										  null,
										  max);

		var recentRecords = {};

		for ( var i = 0; i < recentMods.length; ++i )
			recentRecords[recentMods[i].recid] = true;

		var results = [];

		for ( var recid in recentRecords )
		{
			var lastpage = null;
			var skipFirst = false;

			var history = db.query__recent('page', ['*'], vv.where( { 'recid' : recid } ), max + 1);

			if (history.length > max)
				skipFirst = true;

			var title = history[0].title;

			var first = true;

			var ends = history.length - 1;

			for ( var j = ends; j >= 0; --j )
			{
				var thispage = history[j];

				var record = {
					what: thispage.title,
					title: thispage.title,
					who: thispage.userid,
					when: thispage.timestamp,
					description: thispage.title
				};

				if (first)
					record.action = "Created Wiki page";
				else
					record.action = "Edited Wiki page";

				if (lastpage)
				{
					if (lastpage.title != thispage.title)
					{
						record.action = "Renamed Wiki page";
						record.what += " (was " + lastpage.title + ")";
					}
				}

				lastpage = thispage;

				record.link = '/wiki.html?page=' + encodeURIComponent(title);

				if (! (first && skipFirst))
					results.push(record);

				first = false;
			}
		}

		return(results);
	}
};

vv.log("wiki activity");
activityStreams.addSource(wikiActivitySource);
