var vvWikiModule = {
	defaultHomePage: function()
	{
		var page = { 'title' : 'Home' };

		page.text = "This is the stock Veracity wiki home page.  Feel free to edit it to suit your needs - " +
			"include links to your most-visited pages, team-specific information and links, cookie recipes, " +
			"whatever you'd like.\n\n" +
			"The sidebar to the right lists your existing pages; use the search box to quickly filter that list.\n\n" +
			"You may also, of course, create a [[new page]] at any time.\n";

		return(page);
	},
	
	getPage: function(request)
	{
		var db = new zingdb(request.repo, sg.dagnum.WIKI);

		var leaves = db.get_leaves();

		if (leaves.length != 1)
			return (preconditionFailedResponse("Wiki DAG has multiple heads"));

		var csid = leaves[0];

		var w = vv.where( { "title": request.pagename } );

		var recs = db.query('page', ['text','title','recid'], w);

		if ((! recs) || (recs.length == 0))
		{
			if (request.pagename == 'Home')
				return(this.defaultHomePage());
			else
				return(null);
		}

		var rec = recs[0];

		var separator = "\n== ZING_MERGE_CONCAT ==\n";

		if (rec.text.match(separator))
		{
			var rtext = "# Conflicting Edits Detected\n\n" + 
				"Look below for any \"Conflicts with:\" headings.  Below each of these is a version of this wiki page " +
				"that could not be auto-merged.  Please choose the combination of text you'd like to keep, and delete the rest " +
				"(including this heading and paragraph)\n\n" +
				rec.text;  // todo: elaborate
			rtext = rtext.replace(/== ZING_MERGE_CONCAT ==/g, "# Conflicts with:");

			rec.text = rtext;
		}

		rec._csid = csid;

		return(rec);
	}
};


registerRoutes({
	"/repos/<repoName>/wiki/page/<pagename>.json": {
		"GET": {
			onDispatch: function (request) {
				var cc = new textCache(request, [sg.dagnum.WIKI]);

				if (! cc.isCached())
				{
					var rec = vvWikiModule.getPage(request);
					if (! rec)
						return(notFoundResponse());

					cc.cache(sg.to_json(rec));
				}

				return( cc.response() );
			}
		}
	},

	"/repos/<repoName>/wiki/page": {
		"POST": {
			onJsonReceived: function (request, newrec) {

				if (!request.headers.From)
					return (unauthorizedResponse());

				var db = new zingdb(request.repo, sg.dagnum.WIKI);
				var ztx = null;
				var rec = null;
				
				try
				{
					var csid = newrec._csid || null;
					delete newrec._csid;

					ztx = db.begin_tx(csid, request.headers.From);
					if (newrec.recid)
						rec = ztx.open_record('page', newrec.recid);
					else
					{
						rec = ztx.new_record('page');
						newrec.recid = rec.recid;
					}

					rec.title = newrec.title;
					rec.text = newrec.text;

					vv.txcommit(ztx);
					ztx = null;
				}
				finally
				{
					if (ztx)
						ztx.abort();
				}

				return( textResponse(newrec.recid) );
			}
		}
	},

   "/repos/<repoName>/wiki/<pagename>": {
	   "DELETE" : {
			onDispatch: function(request) {
				if (!request.headers.From)
					return (unauthorizedResponse());

				var p = vvWikiModule.getPage(request);

				if (! p)
					return(notFoundResponse());

				var recid = p.recid;

				var db = new zingdb(request.repo, sg.dagnum.WIKI);
				var ztx = null;
				
				try
				{
					ztx = db.begin_tx(null, request.headers.From);
					
					ztx.delete_record('page', recid);
					vv.txcommit(ztx);
					ztx = null;
				}
				finally
				{
					if (ztx)
						ztx.abort();
				}

				return(OK);
			}
	   }
   },


   "/repos/<repoName>/wiki/pages.json": {
	   "GET" : {
		   onDispatch: function(request) {
			   var cc = new textCache(request, [sg.dagnum.WIKI]);

			   if (! cc.isCached())
			   {
				   var db = new zingdb(request.repo, sg.dagnum.WIKI);

				   var term = vv.getQueryParam('q', request.queryString);
				   var recs = null;
				   
				   if (term)
				   {
					   term = term.replace(/'/g, '"');
					   term = term.ltrim();

					   if (term.indexOf('"') != 0 && term.indexOf(" ") < 0)
						   term += "*";
					   recs = db.query__fts('page', 'title', term, ['*']);
				   }
				   else
					   recs = db.query('page',  ['title','last_timestamp'], null, 'last_timestamp #DESC');

				   if (! recs)
					   recs = [];

				   cc.cache(sg.to_json(recs));
			   }

			   return( cc.response() );
		   }
	   }
   }
});

