/*
Copyright 20102011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

var vvWiki = {
	oldPage: '',
	knownGoodPages: {},
	knownBadPages: {},
	pageName: null,

	setGoodPage: function(ln, pageName)
	{
		this.knownGoodPages[pageName] = true;
		ln.addClass('validWikiUrl');
		ln.removeClass('invalidWikiUrl');
		ln.attr('title', "Wiki: \"" + pageName + "\"");
	},

	setBadPage: function(ln, pageName)
	{
		this.knownBadPages[pageName] = true;
		ln.addClass('invalidWikiUrl');
		ln.removeClass('validWikiUrl');
		ln.attr('title', "No wiki page defined for \"" + pageName + "\"");
	},

	validateLink: function(ln)
	{
		var href = ln.attr('href') || '';

		var re = new RegExp(sgCurrentRepoUrl + "/wiki.html\\?page=([^\\?&]+)");

		var matches = href.match(re);
		var win = window;

		if (matches)
		{
			var pageName = matches[1];

			if (this.knownGoodPages[pageName])
				this.setGoodPage(ln, pageName);
			else if (this.knownBadPages[pageName])
				this.setBadPage(ln, pageName);
			else
			{
				var purl = sgCurrentRepoUrl + "/wiki/page/" + pageName + ".json";

				vCore.ajax(
					{
						url: purl,
						dataType: 'json',
						reportErrors: false,
						success: function(data) {
							vvWiki.setGoodPage(ln, pageName);
						},
						error: function() {
							vvWiki.setBadPage(ln, pageName);
						}
					});
			}
		}
	},

	validateLinks: function()
	{
		var lns = $('#wmd-preview').find('a');

		$.each(	lns, function(i,v) { vvWiki.validateLink($(v)); } );
	},

	showEdits: function(e, hideCancel)
	{
		$.each(vvWiki.edits, function(i,v) { $(v).show(); } );
		vvWiki.edit.hide();
		vvWiki.editor.run();

		if (hideCancel)
		{
			vvWiki.cancel.hide();
			vvWiki.deletebutton.hide();
		}
	},

	deletePage: function()
	{
	//	vvWiki.deletebutton
		var db = vvWiki.deletebutton;
		var pn = vvWiki.pageName;

		var nb = $('<strong></strong>').text('"' + pn + '"');
		var sp = $('<span>Are you sure you want to delete the </span>').
			append(nb).
			append(' page?');

		confirmDialog(
			sp, vvWiki.deletebutton, 
			function(args) { 
				var durl = sgCurrentRepoUrl + "/wiki/" + args.pageName;

				vCore.ajax(
					{
						'url': durl,
						'type': 'DELETE',
						'errorMessage': "Unable to delete this page. The error has been logged.",
						success: function()
						{
							var d = $('<div><p>Deleted this Wiki page.</p></div>');
							var pop = new modalPopup(
								null, null,
								{
									onClose: function() {
										window.document.location.href = sgCurrentRepoUrl + "/wiki.html";
									}
								}
							);

							var buttonOK = new vButton("OK").appendTo(d);

							buttonOK.click(
								function() {
									pop.close();
								}
							);

							$('body').prepend(pop.create('Deleted', d).addClass('info'));
							pop.show();
						}
					}
				);
			}, 
			{ "pageName": pn } 
		);
	},

	grabPage: function(pageName) {
		pageName = pageName || 'Home';
		
		if (pageName == 'new page')
		{
			vvWiki.showEdits(null, true);
			return;
		}

		this.pageName = pageName;

		var purl = sgCurrentRepoUrl + "/wiki/page/" + encodeURIComponent(pageName) + ".json";
		var win = window;

		vCore.ajax(
			{
				url: purl,
				dataType: 'json',
				reportErrors: false,
				success: function(data) {
					$('#wmd-preview').html(vvWiki.converter.makeHtml(data.text));

					data.rectype = 'page';
					vvWiki.form.setRecord(data);
					vvWiki.form.ready();
				},
				error: function(xhr, status, errorthrown) {
					if (xhr && (xhr.status == 404))
					{
						var newRec = {
							title: pageName,
							text: '',
							rectype: 'page'
						};
						vvWiki.form.setRecord(newRec);
						vvWiki.showEdits(null, true);
						vvWiki.form.ready();
					}
					else
					{
						reportError("Unable to retrieve the requested page", xhr, status, errorthrown);
					}
				}
			}
		);
	},

	addEditor: function(roTitle) {
		var hideEdits = function()
		{
			$.each(vvWiki.edits, function(i,v) { $(v).hide(); } );
			vvWiki.edit.show();
		};

		var vfOptions = {
			rectype: "page",
			url: sgCurrentRepoUrl + "/wiki/page",
			method: "POST",
			dataType: "text",
			contentType: "application/json",

			afterSubmit: function(vform, record) {
				var newPage = record.title;

				// always reload, a merge may have happened
				document.location.href = sgCurrentRepoUrl + "/wiki.html?page=" + encodeURIComponent(vvWiki.titleedit.val());
				
				$('#wikititle').text(newPage);
				hideEdits();
			}
		};

		this.form = new vForm(vfOptions);

		var bb = $('#wmd-button-bar');
		var editButtons = $('<div id="topbuttons" class="clearfix" />').insertBefore(bb);
		var reqVal = { validators: { required: true } };

		this.titleedit = this.form.textField('title', reqVal).
			insertBefore(bb).
			hide().
			addClass('wiki').
			attr('placeholder', 'Page Title');

		if (roTitle)
			this.titleedit.
				attr('readonly',true).
				attr('disabled',true).
				addClass('disabled');

		this.textedit = this.form.textArea('text', reqVal).
			insertAfter(bb).
			addClass('wiki').
			css('display', 'block').
			hide().
			attr('id', 'wmd-input').
			addClass('wmd-input').
			attr('placeholder', 'Wiki Text');
		this.save = this.form.submitButton('Save').appendTo(editButtons).hide();
		this.cancel = this.form.button('Cancel').appendTo(editButtons).hide();
		this.rid = this.form.hiddenField('recid').insertAfter(this.save);
		this.csid = this.form.hiddenField('_csid').insertAfter(this.rid);
		var previewLabel = $("<h4>Preview:</h4>").
			insertBefore('#wikititle').
			hide();
		this.edits = [ this.titleedit, this.textedit, this.save, this.cancel, bb, previewLabel ];

		this.edit = this.form.button('Edit').appendTo(editButtons).
			click( vvWiki.showEdits );
		this.deletebutton = this.form.button('Delete').appendTo(editButtons).
			click( vvWiki.deletePage );

		this.cancel.click( hideEdits );
		this.cancel.click(
			function() {
				hideEdits();
				vvWiki.form.reset();
				updateTitle();
				vvWiki.editor.refreshPreview();
			}
		);

		this.textedit.focus(
			function() {
				$('#wikilist li').addClass('active');
				$('#wikilist a').removeClass('disabled');
				$('#wikilist a').removeAttr('disabled');
			}
		);
		this.textedit.blur(
			function() {
				$('#wikilist li').removeClass('active');
				$('#wikilist a.adder').addClass('disabled');
				$('#wikilist a.adder').attr('disabled', true);
			}
		);

		this.converter = Markdown.getSanitizingConverter();

		var wikiUrl = sgCurrentRepoUrl + "/wiki.html";

		this.converter.hooks.chain
		(
			"preConversion", 
			function(text) 
			{
				return( text.replace(/\[\[([^\[\]]+)\]\]/g, 
									 function(str, p1) {
										 var p = encodeURIComponent(p1);
										 var res = "[" + p1 + "](" + wikiUrl + "?page=" + p + ")";
										 return( res );
									 }
									)
					  );
			}
		);

		this.editor = new Markdown.Editor(
			this.converter, null, 
			{
				'handler': function()
				{
					$('#wikihelp').show();
				}
			}
		);

		$('#wikihelpclose').click(
			function() { 
				$('#wikihelp').hide(); 
				return(false);
			} 
		);

		this.editor.hooks.chain("onPreviewRefresh",
								function()
								{
									vvWiki.validateLinks();
								}
							   );

		var updateTitle = function ()
		{
			$('#wikititle').text(vvWiki.titleedit.val());
		};

		this.titleedit.change( updateTitle );
		this.titleedit.keyup( updateTitle );
	},

	updatePageList: function()
	{
		var requrl = sgCurrentRepoUrl + "/wiki/pages.json?q=" + $('#wikisearch').val();
		var pn = getQueryStringVal('page') || '';

		var insertLink = function(pageName) {
			var ta = $('#wmd-input');
			var before = ta.val();
			var len = before.length;

			var newLink = "[[" + pageName + "]]";

			var start = ta[0].selectionStart || 0;
			if (start > len)
				start = len;
			else if (start < 0)
				start = 0;
			
			var after = before.substring(0, start) + newLink + before.substring(start);

			ta.val(after);
			vvWiki.editor.refreshPreview();

			ta[0].selectionStart = start + newLink.length;
			ta[0].selectionEnd = start + newLink.length;
			ta.focus();
		};

		vCore.ajax(
			{
				"url": requrl,
				"dataType": 'json',
				reportErrors: false,
				success: function(data) {
					vvWiki.wil.empty();

					data = data || [];

					for ( var i = 0; i < data.length; ++i )
					{
						var p = data[i].title;

						if (p == pn)
							continue;

						var li = $('<li></li>');

						$('<a></a>').
							attr('href', 'wiki.html?page=' + encodeURIComponent(p)).
							text(p).
							appendTo(li);

						var addlink = $('<a></a>').
							attr('disabled', true).
							addClass('disabled').
							addClass('adder').
							prependTo(li).
							text(' ');

						$('<img />').
							attr('src', sgStaticLinkRoot + "/modules/wiki/trans.png").
							attr('title', "Insert a link to this page").
							prependTo(addlink);

						var pp = "" + p;

						addlink.click(
							function() {
								insertLink(pp);
							}
						);

						vvWiki.wil.append(li);
					}
				}
			}
		);
	},

	showWikiPages: function(container)
	{
		vvWiki.wil = $('<ul id="wikilist"></ul>').appendTo(container);

		this.updatePageList();

	}
};

$(document).ready(
	function()
	{
		var pn = getQueryStringVal('page') || 'Home';

		vvWiki.addEditor(pn == 'Home');

		document.title = (pn || "wiki") + " (" + startRepo + ")";
		vvWiki.oldPage = pn;
		$('#wikititle').text(pn);

		var pu = function() { vvWiki.updatePageList(); };

		var rsb = $('#rightSidebar');

		$("<h2>Wiki Pages:</h2>").appendTo(rsb);
		var ws = $("<input id='wikisearch' class='searchinput' />'").
			appendTo(rsb).
			keyup( pu ).
			change( pu );


		var holder = $('<div></div>').
			css('overflow', 'auto').
			appendTo(rsb);
		vCore.layout.afterResizeMainContent = function()
		{
			var sbheight = $(window).height() - $('#rightSidebar').position().top;
			var ctop = holder.position().top;
			holder.height(sbheight - ctop - 5);
		};


		vvWiki.showWikiPages(holder);

		vCore.showSidebar();
		vCore.resizeMainContent();

		vvWiki.grabPage(pn);
	}
);
	 

