/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

registerRoutes({
	"/repos/<repoName>/history.json": {
		"GET":
		{
			enableSessions: true,
			onDispatch: function (request)
			{
				var results = null;

				try
				{
					if (vv.queryStringHas(request.queryString, "more"))
					{
						results = vvHistory.fetchMore(request);
					}
					else
					{
						results = vvHistory.getHistory(request);
					}

					if (results)
					{
						return jsonResponse(results);
					}
					return jsonResponse({ error: "The history search returned no results." });
				}
				catch (ex)
				{
					if (ex.toString().indexOf("not be found") >= 0)
					{
						return jsonResponse({ error: "The requested object could not be found. Verify that the path exists in the repository." });
					}
					else if (ex.toString().indexOf("The stamp was not found") >= 0)
					{
						return jsonResponse({ error: "The requested stamp was not found" });
					}
					else
					{
						throw (ex);
					}
				}
			}
		}
	}
});

var vvHistory =
{
    getHistory: function (request)
    {
        var results = null;
        var session = vvSession.get();
        var partTmp = vv.getQueryParams(request.queryString) || {};
        var parts = {};
        var max = null

        vv.each(partTmp, function (i, v)
        {
            if (i == 'path')
            {
                parts[i] = v;
            }
            else
            {
                parts[i] = v.replace(/'/g, "''");
            }
        });

        if (parts['max'] !== undefined)
            max = parseInt(parts['max'], 10);

        var path = null;
        if (parts['path'])
        {
            path = parts['path'];
            path.ltrim("/");
            if (path.indexOf("@") != 0)
            {
                path = "@/" + path;
            }
        }
        var merges = null;
        if (parts['hidemerges'] && path && path.length)
        {
            var b = parts['hidemerges'];
            if (b == "true")
                merges = true;
            else
                merges = false;
        }

        results = request.repo.history(max,
								   parts['user'] || null,
								   parts['stamp'] || null,
								   parts['from'] || null,
								   parts['to'] || null,
                                   parts['branch'] || null,
                                   path || null,
                                   merges || false
								  );

        if (results)
        {
            if (results.next_token)
            {
                session.history_token = results.next_token;
            }
            else
            {
                delete session.history_token;
            }
            return ({ items: results.items, more: results.next_token != null });
        }
        else
        {
            delete session.history_token;
        }

    },
    fetchMore: function (request)
    {
        var max = vv.getQueryParam('max', request.queryString)
        var session = vvSession.get();
        var token = session.history_token;

        if (!token)
        {
            return null;
        }

        if (max)
            max = parseInt(max, 10);
        if (session.history_token)
        {
            var results = request.repo.history_fetch_more(session.history_token, max);
            if (results)
            {
                if (results.next_token)
                {
                    session.history_token = results.next_token;
                }
                else
                {
                    delete session.history_token;
                }
                return ({ items: results.items, more: results.next_token != null });
            }
            else
            {
                delete session.history_token;
            }
        }

    }

};
