/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

var vvModules = [];
var knownDags = false;
var checkedRepos = {};
var cmdInProgress = false;

function registerModule(def)
{
	vvModules.push(def);
}


function initModules()
{
	for ( var i = 0; i < vvModules.length; ++i )
	{
		var def = vvModules[i];
			
		for ( var dagname in def.dagnums )
		{
			var details = def.dagnums[dagname];
				
			var newdag = {
				type: "db",
				vendor: def.vendor,
				grouping: def.grouping,
				id: details.dagid
			};
				
			var dagnum = sg.make_dagnum(newdag);

			sg.dagnum[dagname] = dagnum;
		}
	}
}

function checkModuleDags(reponame, modulesPath, uid)
{
	var descriptors = sg.list_descriptors();

	var repoid = reponame;

	if (descriptors[reponame])
		repoid = descriptors[reponame].repo_id;

	if (checkedRepos[repoid])
		return;
	if (cmdInProgress)
		return;

	uid = uid || null;

	cmdInProgress = true;

	try
	{
		initModules();

		var repo = false;

		try
		{
			repo = sg.open_repo(reponame);
		}
		catch (e)
		{
			sg.log("error opening repo in checkModuleDags:" + e.toString());
			repo = false;
		}

		if (! repo)
			return;

		try
		{
			for ( var i = 0; i < vvModules.length; ++i )
			{
				var def = vvModules[i];
				
				for ( var dagname in def.dagnums )
				{
					var details = def.dagnums[dagname];
					
					var newdag = {
						type: "db",
						vendor: def.vendor,
						grouping: def.grouping,
						id: details.dagid
					};

					var dagnum = sg.make_dagnum(newdag);
					
					var a = false;

					try
					{
						a = areaExists(repo, dagnum);
					}
					catch(e)
					{
						repo.close();
						repo = null;
						return;
					}

					if (! a)
					{
						if (! makeArea(repo, def.area, dagnum, uid))
						{
							repo.close();
							repo = null;
							return;
						}
					}

					if (! dagnumExists(repo, dagnum))
					{
						var tfn = modulesPath + "/" + def.area + "/zing/" + details.template;

						var tf = sg.file.read(tfn);
						var t;
						eval('t = ' + tf);

						if (! setTemplate(repo, dagnum, t, uid))
						{
							repo.close();
							repo = null;
							return;
						}
					}
				}

				// any hooks?
				var hooks = def.hooks || {};

				for ( var iface in hooks )
				{
					var hook = hooks[iface];

					var needed = true;

					var installed = repo.vc_hooks_installed(iface);
					
					for ( var j = 0; j < installed.length; ++j )
					{
						var oldHook = installed[j];
						
						if ((oldHook.version >= def.hook_code_sync_rev) && (oldHook.module == def.area))
						{
							needed = false;
							break;
						}
					}

					if (needed)
					{
						var tfn = modulesPath + "/" + def.area + "/scripts/" + hook.file;
						var tf = sg.file.read(tfn);

						var installRequest = {
								"interface": iface,
								"js": tf,
								"module": def.area,
								"version": def.hook_code_sync_rev,
								"replace": hook.replace || false,
							};

						if (uid)
							installRequest.userid = uid;

						repo.install_vc_hook( installRequest );
					}
				}
			}

			checkedRepos[repoid] = true;
		}
		catch(e)
		{
			print(e.toString());
			throw(e);
		}
		finally
		{
			if (repo)
				repo.close();
		}
	}
	finally
	{
		cmdInProgress = false;
	}
}


function areaExists(repo, dagnum)
{
	var areas = repo.list_areas();
	var aid = sg.get_area_id(dagnum);

	for ( var an in areas )
		if (areas[an] == aid)
			return(true);

	return(false);  
}


function makeArea(repo, areaname, dagnum, uid)
{
	var aid = sg.get_area_id(dagnum);

	try
	{
		repo.area_add(areaname, aid, uid || null);
		return(true);
	}
	catch(e)
	{
		// throws if we have no user, or no default user, yet
		return(false);
	}
}


function dagnumExists(repo, dagnum)
{
	if (! knownDags)
	{
		knownDags = {};
		var d = repo.list_dags();

		for ( var i = 0; i < d.length; ++i )
			knownDags[d[i]] = true;
	}

	return( !! knownDags[dagnum] );
}

function setTemplate(repo, dagnum, template, uid)
{
	var ztx = false;

	try
	{
		var db = new zingdb(repo, dagnum);
		ztx = db.begin_tx(null, uid || null);
		ztx.set_template(template);

        var ct = ztx.commit();

        if (ct.errors)
        {
            var msg = sg.to_json(ct);

            throw (msg);
        }

		ztx = false;

		return(true);
	}
	catch (e)
	{
		// throws if no user set
		return(false);
	}
	finally
	{
		if (ztx)
			ztx.abort();
	}
}

