/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/



registerRoutes({
	"/": {
		"GET": {
			onDispatch: function (request)
			{
				var response;
				var defaultRepoName = getDefaultRepoName(request);
				if (defaultRepoName)
				{
					var url = request.linkRoot + "/repos/" + defaultRepoName;
					response = errorResponse(
						"307 Temporary Redirect",
						request,
						"Your browser should redirect you to <a href=\"" + url + "\">" + url + "</a>. (If it doesn't follow the link.)",
						true);
					response.headers["Location"] = url;
				}
				else
				{
					response = errorResponse(STATUS_CODE__OK, request, "There are no repositories available on this server.");
				}
				return response;
			}
		}
	},
	"/repos.html": {
		"GET": {
			onDispatch: function (request)
			{
				var vvt = new vvUiTemplateResponse(request, "core", "repositories");

				return (vvt);
			}
		}
	},
	"/repos.json": {
		"GET": {
			onDispatch: function (request)
			{
				var descriptors = sg.list_descriptors();
				var responseObj;
				if (vv.queryStringHas(request.queryString, "details"))
				{
					responseObj = {};
					for (var descriptorName in descriptors)
					{
						var repo = sg.open_repo(descriptorName);
						if (repo) try
						{
							var nbranches = repo.named_branches();
							var rhids = repo.fetch_dag_leaves(sg.dagnum.VERSION_CONTROL);

							responseObj[descriptorName] = { hids: rhids, branches: nbranches };
						}
						finally
						{
							repo.close();
						}
					}

				}
				else
				{
					responseObj = [];
					for (var descriptorName in descriptors)
						responseObj.push(descriptorName);
				}
				return jsonResponse(responseObj);
			}
		}
	},
	"/repos/<repoName>/users.json": {
		"GET": {
			onDispatch: function (request) {
				var cc = new textCache(request, [sg.dagnum.USERS]);

				if (! cc.isCached())
				{
					var db = new zingdb(request.repo, sg.dagnum.USERS);
					var users = db.query("user", ["*"]);
					cc.cache(sg.to_json(users));
				}

				return( cc.response() );
			}
		}
	},
	"/repos/<repoName>/fileclasses.json": {
		"GET": {
			onDispatch: function (request)
			{
				var fileClasses = request.repo.file_classes();
				if (fileClasses)
				{
					return jsonResponse(fileClasses);
				}
				return (notFoundResponse());
			}
		}
	},
	"/repos/<repoName>.$": {
		"GET": {
			onDispatch: function (request)
			{
				var vvt = new vvUiTemplateResponse(request, "core", "home", null, "home.html");

				return (vvt);
			}
		}
	},
	"/repos/<repoName>.json": {
		"GET": {
			onDispatch: function (request)
			{
				var cc = new textCache(request, [sg.dagnum.VERSION_CONTROL, sg.dagnum.VC_COMMENTS, sg.dagnum.USERS, sg.dagnum.VC_BRANCHES]);

				if (!cc.isCached())
				{
					vv.log("! cached");
					
					var responseObj = {};
					responseObj.RepoID = request.repo.repo_id;
					responseObj.AdminID = request.repo.admin_id;
					responseObj.HashMethod = request.repo.hash_method;
					
					if (vv.queryStringHas(request.queryString, "details"))
					{
						var leaves = request.repo.fetch_dag_leaves(sg.dagnum.VERSION_CONTROL);
						var namedbranches = request.repo.named_branches();

						//make sure we retrieve all the branch heads info even
						//if the head isn't a dag leaf
						if (namedbranches)
						{
							for (var branch in namedbranches.values)
							{
								var needed = true;
								for (var i = 0; i < leaves.length; i++)
								{
									if (leaves[i] == branch)
									{
										needed = false;
										break;
									}
								}
								if (needed)
									leaves.push(branch);
							}
						}
						var hidHash = {};
						vv.each(leaves, function (i, v)
						{
							try
							{
								hidHash[v] = request.repo.get_changeset_description(v);
							}
							catch(ex)
							{
								hidHash[v] = {"changeset_id" : v};
							}
						});

						for (var hid in hidHash)
						{
							if (hidHash[hid].audits)
							{
								for (var j = 0; j < hidHash[hid].audits.length; ++j)
								{
									var un = vv.lookupUser(hidHash[hid].audits[j].userid, request.repo);

									if (un)
										hidHash[hid].audits[j].username = un;
								}
							}
						}

						responseObj.hids = hidHash;
						responseObj.branches = namedbranches;
					}

					cc.cache(sg.to_json(responseObj));
				}

				return (cc.response());
			}
		}
	}

});
