/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

var vclinkFields = {
	"workitems": {
		isMultiple: true,
		isFromMe: false,
		linkType: 'vc_changeset',
		linkSource: 'item',
		linkTarget: 'commit',
		otherType: 'item'
	}
};


registerRoutes({
	"/repos/<repoName>/changesets/<csid>/comments.json": {
		"GET": {
			onDispatch: function (request)
			{
				var comments = request.repo.fetch_vc_comments(request.csid);
				return jsonResponse(comments);
			}
		},
		"POST": {
			onDispatch: function (request)
			{
				if (!request.headers["From"])
					return unauthorizedResponse();
				request.audit = request.repo.create_audit(request.headers["From"]);
			},
			onRequestAborted: function (request)
			{
				request.audit.fin();
			},
			onJsonReceived: function (request, data)
			{
				request.repo.add_vc_comment(request.csid, data.text, request.audit);
				request.audit.fin();

				var comments = request.repo.fetch_vc_comments(request.csid);
				return jsonResponse(comments);
			}
		}
	},

	"/repos/<repoName>/history.html": {
		"GET": {
			onDispatch: function (request)
			{
				var vvt = new vvUiTemplateResponse(
				request, "core", "history"
			);

				return (vvt);
			}
		}
	},


	"/repos/<repoName>/tags.json": {
		"GET": {
			onDispatch: function (request)
			{
				var tags = request.repo.fetch_vc_tags();
				return jsonResponse(tags);
			}
		}

	},

	"/repos/<repoName>/changesets/<csid>/tags.json": {
		"GET": {
			onDispatch: function (request)
			{
				var tags = request.repo.fetch_vc_tags(request.csid);
				return jsonResponse(tags);
			}
		},

		"POST": {
			onDispatch: function (request)
			{
				if (!request.headers["From"])
					return unauthorizedResponse();
			},
			onJsonReceived: function (request, data)
			{
				var zs = new zingdb(request.repo, sg.dagnum.VC_TAGS);
				var existingCSIDs = zs.query("item", ["csid"], vv.where({ "tag": data.text }));
				//should only be one if any
				if (existingCSIDs && existingCSIDs.length)
				{
					//if the tag is on the existing changeset return that changeset                                                              
					if (existingCSIDs[0].csid != request.csid)
					{
						var csdetails = request.repo.get_changeset_description(request.csid);
						return jsonResponse({ description: csdetails });
					}
				}
				else
				{
					var audit = request.repo.create_audit(request.headers["From"]);
					request.repo.add_vc_tag(request.csid, data.text, audit);
					audit.fin();
				}

				var tags = request.repo.fetch_vc_tags(request.csid);

				return jsonResponse(tags);
			}
		}
	},
	"/repos/<repoName>/changesets/<csid>/tags/<tag>.$":
	{
		"DELETE": {
			onDispatch: function (request)
			{
				if (!request.headers["From"])
					return unauthorizedResponse();
				var audit = request.repo.create_audit(request.headers["From"]);

				request.repo.delete_vc_tag(request.csid, request.tag, audit);
				audit.fin();

				var tags = request.repo.fetch_vc_tags(request.csid);

				return jsonResponse(tags);
			}
		}

	},
	"/repos/<repoName>/stamps.json": {
		"GET": {
			onDispatch: function (request)
			{
				var stamps = request.repo.fetch_vc_stamps();
				return jsonResponse(stamps);
			}
		}

	},

	"/repos/<repoName>/changesets/<csid>/stamps/<stamp>.$":
	{
		"DELETE": {
			onDispatch: function (request)
			{
				if (!request.headers["From"])
					return unauthorizedResponse();
				var audit = request.repo.create_audit(request.headers["From"]);

				request.repo.delete_vc_stamp(request.csid, request.stamp, audit);
				audit.fin();

				var stamps = request.repo.fetch_vc_stamps(request.csid);

				return jsonResponse(stamps);
			}
		}

	},
	"/repos/<repoName>/changesets/<csid>/stamps.json": {
		"GET": {
			onDispatch: function (request)
			{
				var stamps = request.repo.fetch_vc_stamps(request.csid);
				return jsonResponse(stamps);
			}
		},
		"POST": {
			onDispatch: function (request)
			{
				if (!request.headers["From"])
					return unauthorizedResponse();
				request.audit = request.repo.create_audit(request.headers["From"]);
			},
			onRequestAborted: function (request)
			{
				request.audit.fin();
			},
			onJsonReceived: function (request, data)
			{
				request.repo.add_vc_stamp(request.csid, data.text, request.audit);
				request.audit.fin();

				var stamps = request.repo.fetch_vc_stamps(request.csid);

				return jsonResponse(stamps);
			}
		}
	},
	"/repos/<repoName>/stamps.json": {
		"GET": {
			onDispatch: function (request)
			{
				var stamps = request.repo.fetch_vc_stamps();
				return jsonResponse(stamps);
			}
		}

	},
	"/repos/<repoName>/diff.json": {
		"GET":
			{
				onDispatch: function (request)
				{
					if (!vv.queryStringHas(request.queryString, "hid1") || !vv.queryStringHas(request.queryString, "hid2"))
					{
						return (badRequestResponse());
					}
					var hid1 = vv.getQueryParam("hid1", request.queryString);
					var hid2 = vv.getQueryParam("hid2", request.queryString);
					var filename = "file";
					if (vv.queryStringHas(request.queryString, "file"))
					{
						filename = vv.getQueryParam("file", request.queryString);
					}

					var diff = request.repo.diff_file(hid1, hid2, filename);
					return textResponse(diff);
				}
			}
	},
	"/repos/<repoName>/diff_local.json": {
		"GET":
			{
				onDispatch: function (request)
				{
					if (!vv.queryStringHas(request.queryString, "hid1") || !vv.queryStringHas(request.queryString, "path"))
					{
						return (badRequestResponse());
					}
					var hid1 = vv.getQueryParam("hid1", request.queryString);
					var path = vv.getQueryParam("path", request.queryString);
					path = path.rtrim("/");

					var diff = request.repo.diff_file_local(hid1, path);
					return textResponse(diff);
				}
			}
	},
	"/repos/<repoName>/zip/<csid>": {
		"GET": {
			onDispatch: function (request)
			{
				var tmp = sg.fs.tmpdir();
				var path = tmp + "/" + request.csid + ".zip";
				sg.zip(request.repoName, request.csid, path);
				return dispatchFileDownload(path, true, request.csid + ".zip", true);
			}
		}
	},
	"/repos/<repoName>/branches.json": {
		"GET": {
			onDispatch: function (request)
			{
				var branches = request.repo.named_branches();
				return jsonResponse(branches);
			}
		}

	},

	"/repos/<repoName>/branch-names.json": {
		"GET": {
			onDispatch: function (request)
			{
				var branches = get_branches(request.repo, (vv.queryStringHas(request.queryString, "all") ? true : false));

				var response = jsonResponse(branches);
				return response;
			}
		}

	},

	"/repos/<repoName>/changesets/<recid>/browsecs.json": {
		"GET": {
			onDispatch: function (request)
			{
				var gid = vv.getQueryParam("gid", request.queryString);
				var path = vv.getQueryParam("path", request.queryString);
				var tne = null;

				if (gid)
				{
					tne = request.repo.get_treenode_info_by_gid(request.recid, gid);
				}
				else
				{
					if (!path)
					{
						path = "@";
					}
					path = path.rtrim("/");
					tne = request.repo.get_treenode_info_by_path(request.recid, path);
				}
				if (tne)
				{
					return jsonResponse(tne);
				}
				return (badRequestResponse());
			}
		}

	},
	"/repos/<repoName>/changesets/<recid>.json": {
		"GET": {
			onDispatch: function (request)
			{
				var desc = request.repo.get_changeset_description(request.recid);
				if (!desc || !desc.changeset_id)
				{
					return (notFoundResponse());
				}

				var changeset = {};
				for (var i = 0; i < desc.audits.length; i++)
				{
					var un = vv.lookupUser(desc.audits[i].userid, request.repo);

					if (un)
						desc.audits[i].username = un;
				}

				changeset.description = desc;
				if (vv.queryStringHas(request.queryString, "details"))
				{
					vv.each(desc.parents, function (i, v)
					{
						var diff = request.repo.diff_changesets(i, request.recid);

						changeset[i] = diff;
					});

					var db = new zingdb(request.repo, sg.dagnum.WORK_ITEMS);

					changeset.description.associations = db.query(
						'vc_changeset',
						[
							{ 'type' : 'from_me', 'ref': 'item', 'alias': 'id', 'field': 'id' },
							{ 'type' : 'from_me', 'ref': 'item', 'alias': 'status', 'field': 'status' },
							{ 'type' : 'from_me', 'ref': 'item', 'alias': 'title', 'field': 'title' },
							{ 'type' : 'from_me', 'ref': 'item', 'alias': 'recid', 'field': 'recid' }
						],
						vv.where( { 'commit' : request.recid } )
					);
				}

				return jsonResponse(changeset);
			}
		}

	},

	"/repos/<repoName>/changeset/<recid>/<linkname>.json": {
		"GET": {
			onDispatch: function (request)
			{
				var wir = new witRequest(request, vclinkFields);

				return (wir.retrieveLink('vc_changeset'));
			}
		}
	},


	"/repos/<repoName>/changeset/<recid>/<linkname>": {
		"PUT": {
			onJsonReceived: function (request, data)
			{
				if (!request.headers.From)
					return (unauthorizedResponse());

				var wir = new witRequest(request, vclinkFields);

				return (wir.updateLinks('changeset', data));

			}
		}
	}

});
