/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/



function getDefaultFilter()
{
	var f = { sort: "priority", groupedby: "status", columns : "id,title,reporter,priority,logged_time,remaining_time" };
	f.criteria = { "assignee": "currentuser", "milestone": "currentsprint" };
	
	return f;

}

function witFilterRequest(request)
{    
	this.filter = {};
	this.maxrows = 0;
	this.skip = 0;
	this.recid = vv.getQueryParam("filter", request.queryString);
	this.request = request;
 
	var self = this;


	function buildWhereFromFilter(filtervals, db)
	{
	    var query = [];

	    for (var c in filtervals.criteria)
	    {
	        var criteria = filtervals.criteria[c];
	       
	        if (criteria && criteria.length > 0)
	        {
	            var wheres = [];

	            if (c == "last_timestamp" || c == "created_date")
	            {
	                if (criteria != "any")
	                {
	                    var date = new Date();
	                    date.setHours(0);
	                    date.setMinutes(0);
	                    date.setSeconds(0);
	                    date.setMilliseconds(0);
	                    var interval = parseInt(criteria);
	                    date.setDate(date.getDate() - interval);

	                    w = c + " >= " + date.getTime();
	                    wheres.push(w);
	                }
	            }
	            else if (c == "stamp")
	            {
	                var vals = criteria.split(",");
	                var recids = null;

	                var res = db.query("stamp", ["item"], vv.where({ name: vals }));

	                var recids = res.map(function (v) { return v.item; });

	                if (recids && recids.length > 0)
	                {
	                    var w = vv.where({ recid: recids });
	                    wheres.push(w);
	                }
	                else
	                {
	                    // If there are no stamps with this tag add a where clause
	                    // that will match no records
	                    wheres.push(vv.where({ recid: "" }));
	                }
	            }
	            else
	            {
	                var vals = criteria.split(",");
	                for (var i = 0; i < vals.length; i++)
	                {
	                    if (vals[i] == "currentuser")
	                    {
	                        vals[i] = self.request.headers.From;
	                    }
	                    if (vals[i] == "currentsprint")
	                    {
	                        var options = db.query("config", ["*"], vv.where({ key: "current_sprint" }), null, 1);

	                        var cs = "";
	                        if (options.length > 0)
	                        {
	                            vals[i] = options[0].value;
	                        }
	                    }
	                    var w = "";
	                    if (vals[i] == "(none)")
	                    {
	                        w = "(" + c + "== #NULL) || (" + c + " == '')";
	                    }
	                    else
	                    {
	                        var addquot = isNaN(vals[i]);
	                        w = c + " == " + (addquot ? "'" : "") + vals[i] + (addquot ? "'" : "");
	                    }
	                    wheres.push(w);
	                }
	            }

	            var strWhere = "";
	            for (var i = 0; i < wheres.length; i++)
	            {
	                var needwrap = (strWhere.length > 0);
	                if (needwrap)
	                {
	                    strWhere = "(" + strWhere + ") || (";
	                }
	                strWhere += wheres[i];
	                if (needwrap)
	                {
	                    strWhere += ")";
	                }
	            }
	            query.push(strWhere);
	        }
	    }
	    var where = "";
	    for (var i = 0; i < query.length; i++)
	    {
	        var needwrap = (where.length > 0);
	        if (needwrap)
	            where = "(" + where + ") && (";
	        // todo SQL quote

	        var val = query[i];
	        var isEmpty = (!val);
	        var qs;

	        qs = query[i];

	        where += qs;

	        if (needwrap)
	            where += ")";
	    }
	  
	    return where;
	}


	this.getFilter = function (repo, zd, id)
	{
	    var recid = "";

	    if (this.recid)
	        recid = this.recid;

	    if (id)
	        recid = id;

	    if (recid == "default")
	    {
	        this.filter = getDefaultFilter();
	    }
	    else if (recid)
	    {
	        var filter = zd.get_record('filter', recid, ["recid", "name", "user", "groupedby", "sort", "columns",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"] }
            ]);

	        if (!filter || !filter.recid)
	            return false;

	        if (filter.criteriatmp && filter.criteriatmp.length)
	        {
	            var criteria = {};
	            vv.each(filter.criteriatmp, function (i, crit)
	            {
	                if (crit.fieldid && crit.value)
	                {
	                    criteria[crit.fieldid] = crit.value;	                   
	                }

	            });
	            filter["criteria"] = criteria;
	        }
	        delete filter.criteriatmp;
	        this.filter = filter;	      

	    }
	    var params = vv.getQueryParams(self.request.queryString);

	    if (params["maxrows"])
	    {
	        this.maxrows = parseInt(params["maxrows"]);
	        delete params.maxrows;
	    }

	    if (params["skip"])
	    {
	        this.skip = parseInt(params["skip"]);
	        delete params.skip;
	    }

	    if (params["groupedby"])
	    {
	        this.filter.groupedby = params["groupedby"];
	        delete params.groupedby;
	    }
	    else
	    {
	        if (!this.filter.groupedby)
	        {
	            this.filter.groupedby = "status";
	        }
	    }

	    if (params["sort"])
	    {
	        this.filter.sort = params["sort"];

	        delete params.sort;
	    }
	    else
	    {
	        if (!this.filter.sort)
	        {
	            this.filter.sort = "priority";
	        }
	    }

	    if (params["columns"])
	    {
	        this.filter.columns = params["columns"];
	        delete params.columns;
	    }
	    else
	    {
	        if (!this.filter.columns)
	        {
	            this.filter.columns = "id,title,reporter,priority";
	        }
	    }

	    if (!recid)
	    {
	        this.filter.criteria = params;
	    }
	    return true;
	}


	this.addCriteria = function (zrec, filtercriteria, ztx)
	{
		for (var c in filtercriteria)
		{		    
			var criteria = filtercriteria[c];
			if (criteria.length > 0)
			{
				var zcrit = ztx.new_record("filtercriteria");
				zcrit.name = zrec.name;
				zcrit.fieldid = c;
				zcrit.value = criteria;
				zcrit.filter = zrec.recid;
			}
		}
   
		return zrec;
	}

	this.deleteFilter = function (id, user)
	{
		var db = new zingdb(this.request.repo, sg.dagnum.WORK_ITEMS);

		var rec = db.get_record('filter', id, ["*"]);         

		if (!rec)
			return (notFoundResponse());

		if (rec.user != user)
		{
			return (unauthorizedResponse());
		}

		var ztx = db.begin_tx(null, user);

		var crits = db.query('filtercriteria', ['recid'], vv.where({ 'filter' : id }));

		vv.each(crits, function (i, c)
		{
			if (c)
			{
				ztx.delete_record('filtercriteria', c.recid);
			}
		});

		ztx.delete_record('filter', id);

		vv.txcommit(ztx);
		ztx = false;
		db.merge(user);

		return (OK);
	}

	this.updateCriteria = function (zrec, newcriteria, db, ztx)
	{
	    var toDelete = [];
	    
	    if (zrec.criteriatmp)
	    {
	        vv.each(zrec.criteriatmp, function (i, v)
	        {
	            if (v && v.recid)
	            {
	                var rec = db.get_record('filtercriteria', v.recid);

	                //if the saved criteria field is in the
	                //new criteria update it
	                sg.log(rec.fieldid);
	                if (newcriteria && newcriteria[rec.fieldid])
	                {
	                    if (newcriteria[rec.fiedid] != rec.value)
	                    {
	                        var open = ztx.open_record('filtercriteria', rec.recid);
	                        open.value = newcriteria[rec.fieldid];
	                    }
	                }
	                //if the saved criteria field isn't in the new criteria
	                //delete it
	                else
	                {
	                    toDelete.push(v.recid);
	                    ztx.delete_record('filtercriteria', v.recid);
	                }
	                delete newcriteria[rec.fieldid];
	            }
	        });


	    }
	    zrec.criteria = [];
	    vv.each(zrec.criteriatmp, function (i, v)
	    {
	        if (vv.inArray(toDelete, v.recid) >= 0)
	            zrec.criteria.push(v);

	    });

	    delete zrec.criteriatmp;
	
	    //add the left over criteria
	    zrec = self.addCriteria(zrec, newcriteria, ztx);
	    //return the record leftover criteria to add later
	    return zrec;
	}

	this.updateFilter = function (request, filterdata)
	{
	    var self = this;
	    var filter = null;

	    var db;
	    var ztx;

	    try
	    {
	        db = new zingdb(this.request.repo, sg.dagnum.WORK_ITEMS);
	        ztx = db.begin_tx(null, this.request.headers.From);
	        var zrec = ztx.open_record('filter', filterdata.recid);
	        if (!zrec)
	        {
	            ztx.abort();
	            ztx = false;

	            return (notFoundResponse());
	        }
	        //remove any values not to be copied
	        var myrecid = filterdata.recid;
	        delete filterdata.recid;
	        delete filterdata.rectype;

	        //copy over the rest
	        for (var f in filterdata)
	        {
	            if (f != "criteria" && f != "criteriaIds")
	            {
	                zrec[f] = filterdata[f];
	            }
	        }

	        vv.txcommit(ztx);
	        ztx = false;

	        db.merge(this.request.headers.From);

	        ztx = db.begin_tx(null, this.request.headers.From);
	        zrec = db.get_record('filter', myrecid, ["recid", "name",
             { "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["recid", "fieldid", "value"]}]);


	        zrec = self.updateCriteria(zrec, filterdata.criteria, db, ztx);
	       
	        vv.txcommit(ztx);
	        ztx = false;

	        db.merge(this.request.headers.From);
	        ztx = false;

	    }
	    finally
	    {
	        if (ztx)
	            ztx.abort();
	    }
	    return OK;
	};

    this.getRealSortValue = function (val)
    {
        switch (val)
        {
            case "milestone":
            case "sprint": //depreciated
                {
                   return "milestonestart,milestonename";
                }
               
            case "assignee":
                {
                    return "assigneename";
                }
                
            case "verifier":
                {
                    return "verifiername";
                }
               
            case "reporter":
                {
                    return "reportername";
                }
            default:
                {
                    return val;
                }
        }

    };
    this.getResults = function ()
    {
        var records = null;
        var zs = new zingdb(this.request.repo, sg.dagnum.WORK_ITEMS);
        if (!self.getFilter(this.request.repo, zs))
        {
            return ({ error: notFoundResponse() });
        }

        var where = buildWhereFromFilter(self.filter, zs);

        var sort = self.filter.sort || null;

        if (sort)
        {

            var sortParts = sort.split(" ");
            if (sortParts[0] == "time_logged" || sortParts[0] == "remaining_time")
            {
                sort = null;
            }
            else
            {
                sortParts[0] = self.getRealSortValue(sortParts[0]);
                sort = sortParts.join(" ");
            }
        }
        if (self.filter.groupedby != "none"
			&& self.filter.groupedby != "remaining_time"
            && self.filter.groupedby != "time_logged")
        {
            var gb = self.getRealSortValue(self.filter.groupedby);

            if (sort)
            {
                sort = gb + ", " + sort;
            }
            else
            {
                sort = gb;
            }
        }

        //sort by grouped first so that the table groups are in the right order

        records = zs.query("item", ["recid", "id", "assignee", "reporter", "verifier", "title", "description", "status", "estimate", "priority", "milestone", "last_timestamp", "created_date",
            { "type": "to_me", "rectype": "work", "ref": "item", "alias": "work", "fields": ["amount", "when"] },
			{ "type": "to_me", "rectype": "stamp", "ref": "item", "alias": "stamp", "fields": ["name"] },
            { "type": "from_me", "ref": "milestone", "alias": "milestonename", "field": "name" },
            { "type": "from_me", "ref": "milestone", "alias": "milestonestart", "field": "startdate" },
            { "type": "username", "userid": "assignee", "alias": "assigneename" },
            { "type": "username", "userid": "reporter", "alias": "reportername" },
            { "type": "username", "userid": "verifier", "alias": "verifiername"}],

            where, sort, self.maxrows, self.skip);

        //store the results in the session
        var session = vvSession.get();
        //if skip is 0 or not defined reset the recids in stored in the session			          
        if (self.skip == 0 || !self.skip)
        {
            session.filterResults = [];
        }
        var resultIds = session.filterResults;
        vv.each(records, function (i, v)
        {
            session.filterResults.push(v.recid);
        });
        session.filterResults = resultIds;

        return records;

    }
  
}

registerRoutes({
    "/repos/<repoName>/filteredworkitems.json":
		{
		    "GET": {
		        enableSessions: true,
		        onDispatch: function (request)
		        {
		            var uservals = null;
		            var sprintvals = null;
		            var templatevals = null;
		            var wfr = new witFilterRequest(request);
		            var results = wfr.getResults();
		            if (results.error)
		                return (results.error);

		            var zs = new zingdb(request.repo, sg.dagnum.USERS);
		            uservals = zs.query("user", ["*"]);
		            var zs = new zingdb(request.repo, sg.dagnum.WORK_ITEMS);
		            sprintvals = zs.query("sprint", ["*"]);
		            templatevals = zs.get_template();

		            var result = { filter: wfr.filter, results: results, users: uservals, sprints: sprintvals, template: templatevals };
		            return jsonResponse(result);
		        }
		    }

		},
    "/repos/<repoName>/filtervalues.json":
		{
		    "GET": {
		        onDispatch: function (request)
		        {
		            var zs = new zingdb(request.repo, sg.dagnum.USERS);
		            var uservals = zs.query("user", ["*"]);
		            zs = new zingdb(request.repo, sg.dagnum.WORK_ITEMS);

		            var sprintvals = zs.query("sprint", ["*"], null, "startdate");

		            var templatevals = zs.get_template();

		            var options = zs.query("config", ["*"], vv.where({ key: "current_sprint" }), null, 1);

		            var cs = "";
		            if (options.length > 0)
		            {
		                cs = options[0].value;
		            }

		            var response = jsonResponse({
		                users: uservals,
		                sprints: sprintvals,
		                template: templatevals,
		                currentsprint: cs
		            });
		            return response;
		        }
		    }
		},
    "/repos/<repoName>/filteredworkitems/session": {

        "POST": {
            enableSessions: true,
            onJsonReceived: function (request, data)
            {
                if (!request.headers.From)
                    return (unauthorizedResponse());

                if (!data.recid)
                    return (badRequestResponse());

                var session = vvSession.get();
                var ids = session.filterResults || [];
                ids.push(data.recid);
                session.filterResults = ids;
                return jsonResponse(OK);
            }
        }
    },
    "/repos/<repoName>/filter/<recid>":
		{
		    "PUT":
			{
			    onJsonReceived: function (request, data)
			    {
			        if (!request.headers.From)
			            return (unauthorizedResponse());

			        if ((data.rectype) && (data.rectype != 'filter'))
			            return (badRequestResponse());

			        var wfr = new witFilterRequest(request);
			        var ret = wfr.updateFilter(request.recid, data);
			        return (jsonResponse(ret));
			    }

			},
		    "DELETE":
			{
			    onDispatch: function (request)
			    {
			        if (!request.headers.From)
			            return (unauthorizedResponse());

			        var wfr = new witFilterRequest(request);

			        return (wfr.deleteFilter(request.recid, request.headers.From));
			    }

			}
		},

    "/repos/<repoName>/filters.json":
		{
		    "GET": {
		        onDispatch: function (request)
		        {
		            var max = 0;
		            if (vv.queryStringHas(request.queryString, "max"))
		            {
		                max = parseInt(vv.getQueryParam("max", request.queryString));
		            }

		            var zs = new zingdb(request.repo, sg.dagnum.WORK_ITEMS);
		            var from = request.headers["From"];
		            var filtervals = {};

		            if (vv.queryStringHas(request.queryString, "details"))
		            {
		                filtervals = zs.query("filter", ["name", "user", "groupedby", "sort", "columns", "recid",
						{ "type": "to_me", "rectype": "filtercriteria", "ref": "filter", "alias": "criteriatmp", "fields": ["fieldid", "value"]}], "user == '" + from + "'", 'last_timestamp #DESC');

		                vv.each(filtervals, function (i, v)
		                {
		                    if (v.criteriatmp)
		                    {
		                        var criteria = {};
		                        vv.each(v.criteriatmp, function (index, crit)
		                        {
		                            if (crit.fieldid && crit.value)
		                            {
		                                criteria[crit.fieldid] = crit.value;
		                            }
		                        });
		                        v["criteria"] = criteria;
		                    }
		                    delete v.criteriatmp;
		                });

		            }
		            else
		            {
		                if (max)
		                {
		                    filtervals = zs.query("filter", ["*"], "user == '" + from + "'", 'last_timestamp #DESC', max);
		                }
		                else
		                {
                            filtervals = zs.query("filter", ["*"], "user == '" + from + "'", 'last_timestamp #DESC')
		                }
		            }
		            var response = jsonResponse(filtervals);

		            return response;
		        }
		    },
		    "POST": {
		        onDispatch: function (request)
		        {
		            if (!request.headers["From"])
		                return unauthorizedResponse();
		            request.audit = request.repo.create_audit(request.headers["From"]);

		        },
		        onRequestAborted: function (request)
		        {
		            request.audit.fin();
		        },
		        onJsonReceived: function (request, data)
		        {
		            var response = false;
		            try
		            {
		                // var wfr = new witFilterRequest(request);
		                var zs = new zingdb(request.repo, sg.dagnum.WORK_ITEMS);
		                var ztx = zs.begin_tx(null, request.headers.From);
		                var zrec = ztx.new_record("filter");
		                var id = zrec.recid;
		                zrec.name = data.name;
		                zrec.columns = data.columns; //todo get this from page
		                zrec.sort = data.sort; //todo get this from page
		                zrec.groupedby = data.groupedby;

		                for (var c in data.criteria)
		                {
		                    var criteria = data.criteria[c];
		                    if (criteria.length > 0)
		                    {
		                        var zcrit = ztx.new_record("filtercriteria");
		                        zcrit.name = data.name;
		                        zcrit.fieldid = c;
		                        zcrit.value = criteria;
		                        zcrit.filter = id;
		                    }
		                }

		                vv.txcommit(ztx);

		                zs.merge(request.headers.From);
		                response = jsonResponse(id);

		            }

		            finally
		            {
		                request.audit.fin();
		            }
		            if (!response)
		                response = badRequestResponse()
		            return response;
		        }
		    }

		}
});
