/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

var statusRecs;
var envRecs;
var loadingBuilds = true;
var allRetrieved = false;
var pageNum = 0;
var seriesRecId = getQueryStringVal("s");

var branchname = getQueryStringVal("b");

function getBuildSummaryCell(buildRec)
{
	/* Each cell looks like this:
	 * <td>
	 *   <table>
	 *     <tr>
	 *       <td>Status Icon</td>
	 *       <td>Status Text</td>
	 *     </tr>
	 *     <tr>
	 *       <td>Started/Finished:</td>
	 *       <td>datetime</td>
	 *     </tr>
	 *     <tr>
	 *       <td>ETA/Duration:</td>
	 *       <td>datetime</td>
	 *     </tr>
	 *   </table>
	 * </td>
	 */

	var statusRec = getRec(statusRecs, buildRec.status_ref);

	var cell = $(document.createElement("td"));
	cell.addClass("buildSummary");

	cell.attr("bgcolor", getBuildColor(statusRec.color));

	var subTable = $(document.createElement("table"));
	var tr = $(document.createElement("tr"));

	var td = $(document.createElement("td"));
	td.attr("rowspan","3");
	td.attr("valign", "top");
	var a = $(document.createElement("a"));
	a.attr("href", "build.html?recid=" + buildRec.recid);
	if (statusRec.icon != undefined)
	{
		var img = $(document.createElement("img"));
		img.attr("src", sgStaticLinkRoot + "/img/builds/" + statusRec.icon + "16.png");
		a.append(img);
		td.append(a);
	}
	tr.append(td);

	td = $(document.createElement("td"));
	td.attr("class", "buildStatus");
	a = $(document.createElement("a"));
	a.attr("href", "build.html?recid=" + buildRec.recid);
	a.text(statusRec.name);
	td.append(a);
	tr.append(td);

	subTable.append(tr);

	tr = $(document.createElement("tr"));
	td = $(document.createElement("td"));

	var div = $(document.createElement("div"));
	div.attr("class", "smallFieldLabel");
	div.text(getStartedFinishedLabel(buildRec, statusRec));
	td.append(div);
	td.append(getStartedFinishedValue(buildRec, statusRec));
	tr.append(td);
	subTable.append(tr);

	tr = $(document.createElement("tr"));
	td = $(document.createElement("td"));
	div = $(document.createElement("div"));
	div.attr("class", "smallFieldLabel");
	div.text(getEtaDurationLabel(buildRec, statusRec));
	td.append(div);

	var etaOrDuration = getEtaDurationValue(buildRec, statusRec);
	if (etaOrDuration != null)
	{
		td.append(etaOrDuration);
		tr.append(td);
		subTable.append(tr);
	}

	cell.append(subTable);

	return cell;
}

function buildHeaderRow(summaryTable)
{
	// Build the header row, with a cell for each environment.
	var tr = $(document.createElement("tr"));
	tr.append($(document.createElement("th")));
	$.each(envRecs, function(i, env)
		   {
			   var th = $(document.createElement("th"));
			   th.text(env.name);
			   th.addClass("col" + i);
			   tr.append(th);
		   });
	summaryTable.append(tr);
}

function addBuildSummaryRow(table, buildNameRec, buildRecs)
{
	var tr, td;

	tr = $(document.createElement("tr"));

	td = $(document.createElement("td"));
	td.text(buildNameRec.name);
	td.addClass("buildName");
	tr.append(td);

	$.each(envRecs, function(j, envRec)
	{
		var i;
		var buildRec = null;

		for (i = 0; i < buildRecs.length; i++)
		{
			if (buildNameRec.recid == buildRecs[i].buildname_ref && envRec.recid == buildRecs[i].environment_ref)
			{
				buildRec = buildRecs[i];
				break;
			}
		}
		if (buildRec != null)
		{
			td = getBuildSummaryCell(buildRec, statusRecs);
			td.addClass("col" + j);
			tr.append(td);
			envRec.gotOne = true;
		}
		else
		{
			td = $(document.createElement("td"));
			td.addClass("col" + j);
			tr.append(td);
		}
	});

	table.append(tr);
}

function showHideColumns()
{
	if (envRecs === undefined || envRecs === null)
		return;

	$.each(envRecs, function(i, envRec)
	{
		if (envRec.gotOne === undefined || envRec.gotOne === null || envRec.gotOne === false)
			$(".col" + i).hide();
		else
			$(".col" + i).show();
	});
}

function addBranches(branches)
{
	var branchList = $("#branchList");
	var gotCurrent = false;

	function addBranch(bn)
	{
		var li = $(document.createElement("li"));
		var a = $(document.createElement("a"));

		a.text(bn);
		a.data("branchname", bn);
		a.attr("href", "?s=" + seriesRecId + "&b=" + escape(bn));

		a.click({"li":li, "bn":bn},
				  function(event)
				  {
					  if (vCore.whichClicked(event.which) != 1)
						  return;

					  event.preventDefault(); // Don't follow the href.

					  var li = event.data.li;
					  var bn = event.data.bn;

					  $("#branchList").children(".current").removeClass("current");
					  li.addClass("current");

					  $("#buildSummaryTable").empty();
					  $.each(envRecs, function(i, envRec)	{envRec.gotOne = false; });

					  if (bn == "(none)")
						  branchname = null;
					  else
						  branchname = bn;

					  /* Alter series links so their middle/right clicks link to the newly selected branch. */
					  $("#seriesList a").each(function(i, v)
											  {
												  var a = $(v);
												  a.attr("href", "?s=" + a.data("seriesRec") + "&b=" + escape(bn));
											  });

					  pageNum = -1;
					  getMoreBuilds(true);
				  });

		li.append(a);
		branchList.prepend(li);
		return li;
	}

	for (var i = 0; i < branches.length; i++)
	{
		var bn = branches[i];
		var li = addBranch(bn);

		if (!gotCurrent && bn == branchname)
		{
			li.addClass("current");
			gotCurrent = true;
		}
	}

	if (!gotCurrent && branchname != null)
	{
		li = addBranch(branchname);
		li.addClass("current");
		gotCurrent = true;
	}

	li = addBranch("(none)");
	if (!gotCurrent && branchname == null)
		li.addClass("current");
}

function fetchAllBranches()
{
	// TODO show spinner
	$("#branchList").empty();

	vCore.ajax(
	{
		url : vCore.getOption("currentRepoUrl") + "/builds/all-branches.json",
		dataType : "json",

		success : function(data, status, xhr)
		{
			addBranches(data);
		},

		complete : function(xhr, status)
		{
			// TODO hide spinner
		}
	});
}

function getBuildSummary()
{
	var summaryTable, summaryDiv, summarySpinner;

	summaryDiv = $("#buildSummary");
	summarySpinner = $("#buildSummaryLoading");

	summaryDiv.empty();
	summarySpinner.show();

	loadingBuilds = true;

	if (branchname == null)
		branchname = "master";

	vCore.ajax(
	{
		url : vCore.getOption("currentRepoUrl") + "/builds/summary.json",
		dataType : "json",
		data :
		{
			series: seriesRecId,
			branchname: branchname
		},

		success : function(data, status, xhr)
		{
			// vCore.consoleLog(JSON.stringify(data, null, 4));

			if (data.seriesRecId != null && data.environments != null && data.series != null)
			{
				var seriesList = $("#seriesList");
				seriesRecId = data.seriesRecId;

				$.each(data.series, function(i, seriesRec)
				{
					var li = $(document.createElement("li"));
					var a = $(document.createElement("a"));

					a.text(seriesRec.name);
					a.data("seriesRec", seriesRec.recid);
					a.attr("href", "?s=" + seriesRec.recid + "&b=" + escape(branchname));

					a.click({"li":li, "seriesRecId":seriesRec.recid},
							  function(event)
							  {
								  if (vCore.whichClicked(event.which) != 1)
									  return;

								  event.preventDefault(); // Don't follow the href.

								  seriesRecId = event.data.seriesRecId;
								  var li = event.data.li;

								  $("#seriesList").children(".current").removeClass("current");
								  li.addClass("current");

								  $("#buildSummaryTable").empty();
								  $.each(envRecs, function(i, envRec)	{envRec.gotOne = false; });

								  /* Alter branch links so their middle/right clicks link to the newly selected series. */
								  $("#branchList a").each(function(i, v)
														  {
															  var a = $(v);
															  if (a.attr("href") != null) // Don't alter the button that fetches all branches
																  a.attr("href", "?s=" + event.data.seriesRecId + "&b=" + escape(a.data("branchname")));
														  });

								  pageNum = -1;
								  getMoreBuilds(true);
							  });

					li.append(a);
					seriesList.append(li);

					if (seriesRec.recid == seriesRecId)
						li.addClass("current");
				});

				branchname = data.branchname;
				addBranches(data.branches);

				$("#buildFilters").show();

				summaryTable = $(document.createElement("table"));
				summaryTable.attr("id", "buildSummaryTable");
				summaryTable.addClass("buildSummary");

				// Save lookup tables
				envRecs = data.environments;
				statusRecs = data.statuses;

				buildHeaderRow(summaryTable);
				$.each(data.buildnames, function(i, buildname) {addBuildSummaryRow(summaryTable, buildname, data.builds, envRecs, statusRecs);});

				summaryDiv.append(summaryTable);

				if (data != null && data.buildnames != null && data.buildnames.length > 0)
					$("#center").hide();
			}

			if (data.buildnames == null || data.buildnames.length < 10)
				allRetrieved = true;

			if (data.buildnames == null || data.buildnames.length == 0)
			{
				if (summaryTable)
					summaryTable.hide();
				$("#noBuilds").show();
			}

		},

		error : function(xhr, status, err)
		{
			allRetrieved = true;
		},

		complete : function(xhr, status)
		{
			showHideColumns();

			if (!allRetrieved && !isVerticalScrollRequired())
				getMoreBuilds(true);

			summarySpinner.hide();
			loadingBuilds = false;
		}
	});
}

function getMoreBuilds(fetchMoreUntilScrollNecessary)
{
	var spinner;

	loadingBuilds = true;
	allRetrieved = false;
	pageNum++;

	if (pageNum == 0)
	{
		$("#noBuilds").hide();
		spinner = $("#buildSummaryLoading");
	}
	else
		spinner = $("#moreBuildsLoading");

	$("#center").show();
	loadingBuilds = true;
	spinner.show();

	vCore.ajax(
	{
		url : vCore.getOption("currentRepoUrl") + "/builds/builds.json",
		dataType : "json",
		data :
		{
			series: seriesRecId,
			branchname: branchname,
			page: pageNum
		},

		success : function(data, status, xhr)
		{
			var summaryTable = $("#buildSummaryTable");

			// vCore.consoleLog(JSON.stringify(data, null, 4));

			if (pageNum == 0)
			{
				if (data.buildnames.length == 0)
				{
					summaryTable.hide();
					$("#noBuilds").show();
				}
				else
				{
					summaryTable.show();
					buildHeaderRow(summaryTable);
				}
			}

			$.each(data.buildnames, function(i, buildname) {addBuildSummaryRow(summaryTable, buildname, data.builds, envRecs, statusRecs);});

			if (data.buildnames == null || data.buildnames.length < 10)
				allRetrieved = true;

			if (data != null && data.buildnames != null && data.buildnames.length > 0)
				$("#center").hide();
		},

		error : function(xhr, status, err)
		{
			allRetrieved = true;
		},

		complete : function(xhr, status)
		{
			showHideColumns();

			if (fetchMoreUntilScrollNecessary && !allRetrieved && !isVerticalScrollRequired())
			 	getMoreBuilds(true);

			spinner.hide();
			loadingBuilds = false;
		}
	});
}

function sgKickoff()
{
	getBuildSummary();

    $(window).scroll(function()
    {
        // Chrome 5.0.x will call this twice:
        // http://code.google.com/p/chromium/issues/detail?id=43958
        // The loadingHistory flag is here to work around that.

        // scrollPosition = $(window).scrollTop();
        if (!loadingBuilds && isScrollBottom() && !allRetrieved)
        {
            getMoreBuilds(false);
        }
    });

	$(window).resize(function()
	{
		if (!loadingBuilds && !allRetrieved && !isVerticalScrollRequired())
			getMoreBuilds(true);
	});
}
