/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

var _changesetHover = null;
var _wiHover = null;
var stamps = [];

function setComments(comments)
{
	if (comments && comments.length > 0)
	{
		$('#comment').empty();

		var cmt = formatComments(comments);
		$('#comment').append(cmt);
	}
}

function showChangedFiles(diff, div)
{
    var ul = $('<ul>').addClass("cslist");

    $.each(diff, function (index, value)
    {
        value.sort(function (a, b)
        {
            return (b.path.toLowerCase().compare(a.path.toLowerCase()));
        });
        $.each(value, function (ii, vv)
        {
            var isDirectory = (vv.type == "Directory");
            var li = $('<li>');
            var sp = $('<span>');
            var toggle = $('<span>');
            var attrs = " ";
            if (vv.changed_attr)
            {
                attrs += " (Attributes) ";
            }
            if (vv.changed_xattr)
            {
                attrs += " (XAttrs) ";
            }
            sp.text(index + attrs);
            var plus = $('<a>').text("+").attr(
			{
			    href: "#"
			}).addClass("plus");
            var info = $('<div>').addClass("cs_info");

            plus.toggle(function ()
            {
                plus.text("-");
                info.css(
				{
				    "padding-top": "5px",
				    "padding-bottom": "5px"
				});
                info.show();
            }, function ()
            {
                plus.text("+");
                info.hide();
            });

            toggle.append(plus);
            if (index != "Added" && index != "Removed" && index != "Modified")
            {
                li.append(toggle);
                li.addClass("toggle");
            }

            li.append(sp);


            if (isDirectory && index == "Removed")
            {
                li.text("Removed " + vv.path);
            }
            else
            {
                var a = $('<a>').text(vv.path);
                if (isDirectory)
                {
                    a.attr(
				        {
				            href: sgCurrentRepoUrl + "/browsecs.html?recid=" + getQueryStringVal("recid") + "&gid=" + vv.gid,
				            'title': "display folder"
				        });

                }
                else
                {
                    a.attr(
				        {
				            href: "#",
				            'title': "display file"
				        }).click(function ()
				        {
				            displayFile(vv.hid, vv.path.ltrim("@/"));
				        });
                }

                li.append(a);
            }
            if (vv.from)
            {
                info.append(" moved from " + vv.from).hide();

            }
            if (vv.oldname)
            {
                info.append(" original name " + vv.oldname).hide();

            }

            var linkSp = $("<span>").addClass("linkSpan");

            if (isDirectory == false && index != "Removed")
            {
                var downloadURL = sgCurrentRepoUrl + "/blobs-download/" + vv.hid + "/"
						+ lastURLSeg(vv.path);
                var img = $('<img>').attr("src",
						sgStaticLinkRoot + "/img/downloadgreen2.png");
                var a_download = $('<a>').html(img).attr(
				{
				    href: downloadURL,
				    'title': "download file"
				}).addClass("small_link");

                if (index != "Added" && index != "Moved")
                {
                    var diff_div = $('<div>');
                    var diffURL = sgCurrentRepoUrl + "/diff.json?hid1=" + vv.old_hid + "&hid2=" + vv.hid + "&file="
							+ lastURLSeg(vv.path);
                    var imgd = $('<img>').attr("src",
							sgStaticLinkRoot + "/img/diff.png");

                    var a_diff = $('<a>').html(imgd).attr(
					{
					    href: "#",
					    'title': "diff against previous version"
					}).addClass("download_link").click(function (e)
					{
					    e.preventDefault();
					    e.stopPropagation();
					    showDiff(diffURL, diff_div, true);

					});
                    linkSp.append(a_diff);
                }

                linkSp.append(a_download);
            }

            li.append(linkSp);

            ul.prepend(li);
            li.append(info);
        });
    });
    if (div)
    {
        div.append(ul);
    }
    else
    {
        $('#changes').append(ul);
    }

}

function setChangedFiles(data, parents)
{
    $('#browse').attr("href", sgCurrentRepoUrl + "/browsecs.html?recid=" + getQueryStringVal("recid"));

	$.each(parents, function(i, v)
	{
		$('#changes').append(spanParent);
		var link = makePrettyRevIdLink(v.revno, i, false, null, null, sgCurrentRepoUrl);
		var spanParent = $('<span>');
		spanParent.append("<b>from parent</b> ");
		spanParent.append(link);

		if (v.comments && v.comments.length > 0)
		{
			var pdiv = formatComments(v.comments, null, spanParent);

			$('#changes').append(pdiv);
		}
		else
		{
			$('#changes').append(spanParent);
		}

		var ul = $('<ul>');
		var li = $('<li>');
		var div = $('<div>').attr({ id : i });
		showChangedFiles(data[i], div);
		div.append($('<br/>'));

		li.append(div);
		ul.append(li);

		$('#changes').append(ul);
	});

}

function setParents(parents)
{
	if (parents)
	{
		$.each(parents, function(i, v)
		{
		    var a = makePrettyRevIdLink(v.revno, i, false, " <", null, sgCurrentRepoUrl);
		    _changesetHover.addHover(a, i, 20);
			$('#parent_nav').append(a);
		});
	}

}

function setChildren(children)
{
	if (children && children.length > 0)
	{
	    $.each(children, function (i, v)
	    {
	        var a = makePrettyRevIdLink(v.revno, v.changeset_id, false, null,
							">", sgCurrentRepoUrl);

	        _changesetHover.addHover(a, v.changeset_id, 20);

	        $('#child_nav').append(a);

	    });
	}
	else
		$('#children').hide();

}

function deleteStamp(stamp)
{

    var url = sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") + "/stamps/" + encodeURIComponent(stamp);
	vCore.ajax(
	{
		type : "DELETE",
		url : url,		
		success : function(data, status, xhr)
		{
			var stamps = [];
			$.each(data, function(i, v)
			{
				stamps.push(v.stamp);
			});
			setStamps(stamps);

		}
	});

}

function setStamps(stamps)
{
	$('#stamps span').remove();
	if (stamps != null && stamps.length > 0)
	{

		var div = $('#stamps');
		div.addClass("greenblock");

		$.each(stamps, function (i, v)
		{
		    var a = $("<a>").attr(
			{
			    "title": "view changesets with this stamp",
			    href: sgCurrentRepoUrl + "/history.html?stamp=" + encodeURIComponent(v)
			});
			a.text(v);
		    var del = $('<a>').text("x").attr(
			{
			    "title": "delete this stamp",
			    href: "#"
			});
		  
		    var cont = $('<span>').addClass("cont");
		    var span = $('<span>').append(a).addClass("stamp");
		    cont.append(span);
		    cont.append(del);

		    div.append(cont);
		    del.click(function (e)
		    {
		        e.preventDefault;
		        e.stopPropegation;
		        confirmDialog($("<span>").text("Delete " + v + "?"), del, deleteStamp, v);

		    });

		    if ((i > 1) && (i % 5 == 0))
		    {
		        div.append($('<p>'));
		    }
		});
	}

}

function getAllStamps()
{
    var url = sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") +"/stamps.json";

	var allstamps = [];
	vCore.ajax(
	{
		url : url,
		dataType : 'json',
		error : function(xhr, textStatus, errorThrown)
		{
			$('#maincontent').removeClass('spinning');
			reportError(null, xhr, textStatus, errorThrown);
		},
		success : function(data, status, xhr)
		{

			if ((status != "success") || (!data))
			{
				return;
			}

			$.each(data, function(i, v)
			{
				allstamps.push(v.stamp);
			});			
		}

	});

}


function postComment()
{

	var url = sgCurrentRepoUrl + "/changesets/" + pageCurrentCsid + "/comments.json";

	var text = $('#inputcomment').val().trim();

	if (!text)
	{
		reportError("comment can not be empty", null, "error");
		return;
	}

	$('#maincontent').addClass('spinning');

	vCore.ajax(
	{
		type : "POST",
		url : url,
		data : JSON.stringify({text: text}),
		success : function(data, status, xhr)
		{
			setComments(data);
			$('#maincontent').removeClass('spinning');
		},
		error : function(xhr, textStatus, errorThrown)
		{
			$('#maincontent').removeClass('spinning');
		}

	});

	$('#inputcomment').val('');
}

function getStamps()
{

	$('#maincontent').css("height", "300px");

	var str = "";

	var url = sgCurrentRepoUrl + "/changesets/stamps";
	stamps = [];
	vCore.ajax(
	{
		url : url,
		dataType : 'json',

		success : function(data, status, xhr)
		{
			$('#maincontent').css("height", "");
			if ((status != "success") || (!data))
			{
				return;
			}
			$.each(data, function(i, v)
			{
				stamps.push(v.stamp);
			});
		}

	});

}

function postStamp()
{
	var url = sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") + "/stamps.json";
	var text = $('#inputstamp').val().trim();

	if (!text)
	{

		reportError("stamp can not be empty", null, "error");

		return;
	}
	
	if (!validateStamp(text))
	{
		reportError("Stamps cannot contain '/', '\\', commas, carriage returns, or line feeds");
		return;
	}
	
	vCore.ajax(
	{
		type : "POST",
		url : url,
		data: JSON.stringify({ text: text }),
		success : function(data, status, xhr)
		{
			var stamps = [];
			$.each(data, function(i, v)
			{
				stamps.push(v.stamp);
			});
			setStamps(stamps);
			$('#inputstamp').val("");
			getAllStamps();
		},
		error : function(xhr, textStatus, errorThrown)
		{
			$('#comments').removeClass('spinning');
		}

	});

	$('#inputcomment').val('');
}

function validateStamp(stamp)
{
	var noInvalidChars = /^[^,\\\/\r\n]*$/;
	
	if (stamp == "")
		return false;
		
	if (stamp.match(noInvalidChars))
		return true;
	
	return false;
}

function moveTagToThis(oldCsid)
{
	var text = $('#inputtag').val();
	var url = sgCurrentRepoUrl + "/changesets/" + oldCsid + "/tags/" + encodeURIComponent(text);
	deleteTag(text, url, postTag);

}

function deleteTag(tag, delurl, onSuccess)
{

    var url = delurl || sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") +"/tags/" + encodeURIComponent(tag);
	vCore.ajax(
	{
		type : "DELETE",
		url : url,		
		success : function(data, status, xhr)
		{
			if (onSuccess)
			{
				onSuccess();
			}
			else
			{
				var tags = [];
				$.each(data, function(i, v)
				{
					tags.push(v.tag);
				});
				setTags(tags);
			}

		}

	});

}

function setTags(tags)
{
	$('#tags span').remove();
	if (tags != null && tags.length > 0)
	{

		var div = $('#tags');
		div.addClass("greenblock");
		$.each(tags, function(i, v)
		{

			var span = $('<span>').text(v).addClass("vvtag");
			var del = $('<a>').text("x").attr(
			{
				"title" : "delete this tag",
				href : "#"
			});

			var cont = $('<span>').addClass("cont");
			
			del.click(function(e)
			{
				e.preventDefault;
				e.stopPropegation;

				confirmDialog($("<span>").text("Delete " + v + "?"), del, deleteTag, v);
			});
			cont.append(span);
			cont.append(del);
			div.append(cont);
			if ((i > 1) && (i % 5 == 0))
			{
				div.append($('<p>'));
			}
		});

	}
}


function deleteWit(data)
{
	var wits = data[0];
	var recid = data[1];
	var csid = getQueryStringVal("recid");

	vCore.ajax(
		{
			type: "DELETE",
			url: sgCurrentRepoUrl + "/workitem/" + recid + "/commit/" + csid,
			success: function() {

				for ( var i = wits.length - 1; i >= 0; --i )
					if (wits[i].recid == recid)
						wits.splice(i, 1);
				setWits(wits);
			}
		}
	);
}

function setWits(wits)
{
	$('#wits ul').remove();
	if (wits && (wits.length > 0))
	{
		var div = $('#wits');
		var ul = $('<ul />').appendTo(div);

		$.each(wits, function(i, v)
		{
			var link = $('<a />').text(v.id).addClass("wit").
				attr('href', sgCurrentRepoUrl + "/workitem.html?recid=" + v.recid);
			_wiHover.addHover(link, v.recid);
			var del = $('<a />').text("x").attr(
			{
				"title" : "delete this association",
				href : "#"
			}).
				addClass('greenblock');

			del.click(function(e)
			{
				e.preventDefault();
				e.stopPropagation();
                               
				confirmDialog("Delete link with " + v.id + "?", del, deleteWit, [ wits, v.recid ]);
			});
			
			$('<li />').
				append(link).
				append(": " + v.title + " ").
				append(del).
				appendTo(ul);
		});

	}
}


function addWit(id, wits)
{
	var posturl = sgCurrentRepoUrl + "/workitem/" + id + "/commit/" + getQueryStringVal("recid");

	vCore.ajax(
		{
			url: posturl,
			type: "POST",
			success: function() 
			{
				var u = sgCurrentRepoUrl + "/workitem/" + id + ".json?_fields=status,title,id,recid";

				vCore.ajax(
					{
						url: u,
						dataType: 'json',
						success: function(data) {
							wits.push(data);
							setWits(wits);
						}
					}
				);
			}
		}
	);

}


function initWitComplete(wits)
{
	var searchfield = $('#inputwit');

	searchfield.keypress(
		function (e)
		{
			var key = (e.which || e.keyCode);

			if (key == 13)
			{
				e.preventDefault();
				e.stopPropagation();
				return(false);
			}
	
			return(true);
		}
	);

	var loadZero = function() {
		if (searchfield.data('fired') || ! searchfield.data('waiting'))
			return;

		if (searchfield.val())
			return;

		searchfield.autocomplete('search', '');
	};

	searchfield.blur(
		function() {
			searchfield.data('waiting', false);
		}
	);

	searchfield.focus(
		function(e) {
			searchfield.data('fired', false);
			var v = searchfield.val();

			if (! v)
			{
				searchfield.data('waiting', true);
				window.setTimeout(loadZero, 3000);
			}
			else
			{
				window.setTimeout(
					function() {
						searchfield.autocomplete('search', v);
					}, 100);
			}
		}
	);

	searchfield.autocomplete(
		{
			minLength: 0,
			delay: 400,

			source: function(req, callback)
			{
				searchfield.data('fired', true);
				var term = req.term;
				var results = [];

				// todo: already-check

				var u = sgCurrentRepoUrl + "/workitem-fts.json?term=" + encodeURIComponent(term || '');

				if (! term)
				{
					var vlist = _visitedItems() || [];

					var ids = [];
					$.each
					(	
						vlist, 
						function (i, v)
						{
							var nv = v.split(":");
							ids.push(nv[0]);
						}
					);

					u = sgCurrentRepoUrl + "/workitem-details.json?_ids=" + encodeURIComponent(ids.join(','));
				}
				else if (term.length < 3)
				{
					callback([]);
					return;
				}

				vCore.ajax(
					{
						url: u,
						dataType: 'json',
						errorMessage: 'Unable to retrieve work items at this time.',

						success: function(data)
						{
							var already = {};
							results = [];

							for ( var i = 0; i < wits.length; ++i )
								already[ wits[i].recid ] = true;

							for ( i = 0; i < data.length; ++i )
								if (! already[ data[i].value ])
								{
									results.push(data[i]);
									already[ data[i].value ] = true;
								}
						},

						complete: function()
						{
							callback(results);
						}
					}
				);

			},
			
			select: function(event, ui) {
				event.stopPropagation();
				event.preventDefault();

				addWit(ui.item.value, wits);
				searchfield.val('');
				searchfield.blur();

				return(false);
			}
		}
	);

	var el = searchfield.data('autocomplete').element;
	el.oldVal = el.val;

	el.val = function(v) {
		return( el.oldVal() );
	};


}

function postTag()
{

    var url = sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") + "/tags.json";

	var text = $('#inputtag').val().trim();

	if (!text)
	{

		reportError("tag can not be empty", null, "error");
		return;
	}

	vCore.ajax(
		{
			type: "POST",
			url: url,
			data: JSON.stringify({ text: text }),
			success: function (data, status, xhr)
			{
				if (data.description)
				{
					var div = $('<div>').text(
							"The tag exists on this changesent, move tag?");
					div.append($('<br/><br/>'));
					var a = ($('<a>')).attr(
							{
							    href: sgCurrentRepoUrl + "/changeset.html?recid=" + data.description.changeset_id,
								"title": "view changeset",
								"target": "_blank"
							}).text(data.description.changeset_id);
					div.append(a);
					div.append($('<br/>'));
					setTimeStamps(data.description.audits, div);
					div.append($('<br/>'));
					div.append($('<br/>'));
					if (data.description.comments
							&& data.description.comments.length > 0)
					{
						data.description.comments
								.sort(function (a, b)
								{
									return (firstHistory(a).audits[0].timestamp - firstHistory(b).audits[0].timestamp);
								});
						div.append(data.description.comments[0].text);
					}
					confirmDialog(div, $('#inputtag'), moveTagToThis,
							data.description.changeset_id);
				}
				else
				{

					var tags = [];
					$.each(data, function (i, v)
					{
						tags.push(v.tag);
					});
					setTags(tags);
					$('#inputtag').val("");
				}
			}
		});

	$('#inputcomment').val('');
}

function bindObservers()
{
	if (!isTouchDevice())
	{
		$("#changes .cslist li").each(function(index)
		{
			$(this).find("span.linkSpan").first().hide()
			$(this).hover(function()
			{
				$(this).find("span.linkSpan").first().show();
			}, function()
			{
				$(this).find("span.linkSpan").first().hide();
			});
		});
	}
}

function getChangeSet()
{

    var listUrl = sgCurrentRepoUrl + "/changesets/" + getQueryStringVal("recid") + ".json?details";

    $('#csLoading .statusText').text("Loading Changeset...");
    $('#csLoading').show();

    vCore.ajax(
	{
	    url: listUrl,
	    dataType: 'json',

	    error: function (xhr, textStatus, errorThrown)
	    {
	        $('#csLoading').hide();
	    },
	    success: function (data, status, xhr)
	    {
	        var revno = data.description.revno + ":"
					+ data.description.changeset_id.slice(0, 10);
	        pageCurrentCsid = data.description.changeset_id;

	        $('#revid').text(data.description.revno + ":" + data.description.changeset_id);
	        $('#revid_short').text(revno);
	        $('#current_nav').text(revno);
	        vCore.setPageTitle(revno);
	        $('#zip_repo').attr(
			{
			    'title': 'download zip of repo at this version',
			    href: sgCurrentRepoUrl + "/zip/" + data.description.changeset_id
			});

	        setTimeStamps(data.description.audits);
	        setComments(data.description.comments);
	        setParents(data.description.parents);
	        setChangedFiles(data, data.description.parents);
	        setChildren(data.description.children);
	        setTags(data.description.tags);
	        setStamps(data.description.stamps);
	        setWits(data.description.associations);
	        bindObservers();

	        initWitComplete(data.description.associations);
	    },
	    complete: function ()
	    {
	        var url = sgCurrentRepoUrl + "/branches.json";
	        vCore.ajax({
	            url: url,
	            dataType: 'json',

	            error: function (xhr, textStatus, errorThrown)
	            {
	                $('#csLoading').hide();

	            },
	            success: function (data, status, xhr)
	            {
	                $('#csLoading').hide();

	                if (data && data.values && data.values[pageCurrentCsid])
	                {
	                    var name = [];
	                    for (var n in data.values[pageCurrentCsid])
	                    {
	                        if (data.closed[n])
	                            name.push('<a class="closedBranch" title="' + n + ' is closed.">(' + n + ')</a>');
	                        else
	                            name.push(n);
	                    }
	                    $("#libranchname").html("<b>branch: " + name.join(", ") + "</b>");

	                }
	            }
	        });
	    }
	});
}

function addBuildsToTable(table, builds)
{
	var lastname;
	$.each(builds, function(i, b)
		   {
			   if (b.name != lastname)
			   {
				   lastname = b.name;
				   var tr = $(document.createElement("tr"));
				   var td = $(document.createElement("td"));
				   td.attr("colspan", "2");

				   var buildtext = b.name;
				   if (b.branchname != null)
					   buildtext += " (" + b.branchname + ")";
				   td.text(buildtext);
				   tr.append(td);
				   table.append(tr);
			   }

			   tr = $(document.createElement("tr"));
			   td = $(document.createElement("td"));

			   if (b.icon != null)
			   {
				   var a = $(document.createElement("a"));
				   a.attr("href", "build.html?recid=" + b.recid);
				   a.attr("title", b.statusname);
				   var img = $(document.createElement("img"));
				   img.attr("src", sgStaticLinkRoot + "/img/builds/" + b.icon + "16.png");
				   a.append(img);
				   td.append(a);
			   }
			   tr.append(td);

			   td = $(document.createElement("td"));
			   a = $(document.createElement("a"));
			   a.attr("href", "build.html?recid=" + b.recid);
			   a.attr("title", b.statusname);
			   a.text(b.envname);
			   td.append(a);
			   tr.append(td);

			   table.append(tr);
		   });
}

function getBuildRequest()
{
	vCore.ajax(
	{
		url : vCore.getOption("currentRepoUrl") + "/builds/changeset-page.json",
		dataType : "json",
		data: {
			csid: getQueryStringVal("recid")
		},

		success : function(data, status, xhr)
		{
			//vCore.consoleLog(JSON.stringify(data, null, 4));

			if (data.builds != null && data.builds.length > 0)
			{
				var table = $(document.createElement("table"));
				table.attr("id", "buildsTable");
				addBuildsToTable(table, data.builds);
				$("#builds").append(table);
				$("#buildSection").show();
			}
			else
				$("#builds").append("No builds of this changeset.");

			if (data.environments != null && data.environments.length > 0)
			{
				if (data.config != null && data.config.length > 0)
				{
					/* Populate the environments list box */
					var envList = $("#buildEnvs");
					$.each(data.environments, function(i, env)
						   {
							   var opt = $(document.createElement("option"));
							   opt.text(env.name);
							   opt.attr("value", env.recid);
							   envList.append(opt);
						   });

					if (data.environments.length == 1)
						envList.attr("size", 1);
					else if (data.environments.length < 6)
					envList.attr("size", data.environments.length);
					else
						envList.attr("size", data.environments.length / 2);

					$("#buildSection").show();
					$("#buildReq").show();
				}
			}
		}
	});

}

function QBuild()
{
	var buildEnvs = $("#buildEnvs option:selected");

	// Make sure at least one environment is selected
	if( buildEnvs.length == 0 )
	{
		alert("You must select at least one environment");
		return;
	}

	$("#newBuildForm select").attr("disabled", "1");
	$("#buildBtn").hide();
	$("#newBuildSpinner").show();

	var newBuildEnvs = new Array(buildEnvs.length);
	$.each(buildEnvs, function(i, opt)
		   {
			   newBuildEnvs[i] = $(opt).attr("value");
		   });

	var data = new Object();
	data.csid = getQueryStringVal("recid");
	data.priority = parseInt($($("#buildPriority option:selected")[0]).attr("value"));
	data.envs = newBuildEnvs;

	vCore.ajax(
	{
		type: "POST",
		dataType : "json",
		contentType : "application/json",
		url : vCore.getOption("currentRepoUrl") + "/builds",
		data : JSON.stringify(data),

		success : function(data, status, xhr)
		{
			//vCore.consoleLog(JSON.stringify(data, null, 4));

			var newTable = false;
			if ($("#buildsTable").length == 0)
			{
				newTable = true;
				var table = $(document.createElement("table"));
				table.attr("id", "buildsTable");
			}
			else
				table = $("#buildsTable");

			addBuildsToTable(table, data);

			if (newTable)
			{
				$("#builds").empty();
				$("#builds").append(table);
				$("#builds").show();
			}
		},

		complete : function(xhr, status)
		{
			$("#newBuildSpinner").hide();
			$("#buildBtn").show();
			$("#newBuildForm select").removeAttr("disabled");
		}
	});
}

function sgKickoff()
{
    _changesetHover = new hoverControl(fetchChangesetHover);
	_wiHover = new hoverControl(fetchWorkItemHover);
    $('#inputstamp').keyup(function (e1)
    {
        if (e1.which == 13)
        {
            e1.preventDefault();
            e1.stopPropagation();
            postStamp();
        }
    });

    $('#inputtag').keyup(function (e1)
    {
        if (e1.which == 13)
        {
            e1.preventDefault();
            e1.stopPropagation();
            postTag();
        }
    });
	getChangeSet();
	getAllStamps();
	getBuildRequest();
}
