/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */


var svg;
var lastHeight = 0;
var svgGroup;
var BLUE_FILL = 'rgb(230,248,251)';
var YELLOW_FILL = 'rgb(255, 255, 234)';
var GREEN_FILL = 'rgb(228, 240, 206)';
var GREEN_STROKE = 'rgb(99, 132, 00)';
var YELLOW_STROKE = 'rgb(209, 169, 46)';
var BLUE_STROKE = 'rgb(0, 88, 104)';

function graphNode(dag, div, index, branch)
{
	this.Dag = dag || null;
	this.Div = div || null;
	this.Index = index || 0;
    
	this.Bottom = 0;
	this.Radius = 5;
	this.Left = 0;
	this.Top = 0;
	this.ColumnID = 0;
	this.Children = [];
	this.Parents = [];

	this.SvgParentLineFamily = [];
	this.SvgChildLineFamily = [];
	this.Circle = null;	
	this.positionCalculated = false;
	this.branchname = branch;
	this.fillColor = BLUE_FILL;
	this.strokeColor = BLUE_STROKE;

	this.reset = function ()
	{
		this.Bottom = 0;
		this.Top = 0;
		this.ColumnID = 0;
		this.Left = 0;
		this.Children = [];
		this.Parents = [];
		this.SvgParentCircleFamily = [];
		this.SvgChildCircleFamily = [];
		this.SvgParentLineFamily = [];
		this.SvgChildLineFamily = [];		
		this.Circle = null;
		this.positionCalculated = false;

	}
	this.calcPosition = function ()
	{
	    if (this.positionCalculated)
	        return;

	    // TODO this is a bit of a hack.  The node needs to know
	    // its parent's parent's top position (the table)
	    var parentTop = this.Div.parent().parent().position().top;
	    var height = this.Div.height();
	    var top = this.Div.position().top;

	    this.Top = top + (height + this.Radius) / 2 - parentTop;
	    this.Bottom = this.Top + this.Radius;
	    this.positionCalculated = true;
	}

}

function historyGraph()
{
    var FLAG_PARENT_HIGHLIGHT = 1;
    var FLAG_CHILD_HIGHLIGHT = 2;

    var currentLeaves = [];
    var graphNodes = {};
    var indexedNodes = [];
    var lineIndexes = {};
    var maxCols = 0;
    var columns;
    var hItems = [];
    var columnNum = 0;
    var lastNode = {};
    var colSpacer = 20;
    var filtered = false;
    var branchNodes = [];
    var visitedParentHighlights = {};
    var visitedChildHighlights = {};
    var lineMap = {};

    var currentHighlight = null;
    //Todo this is probably overkill now that
    //we don't sort
    function graphColumn(id)
    {
        this.Id = id;
        this.NodeCount = 0;
        this.LeftOffset = (this.Id * colSpacer) + colSpacer;
    }

    function getColumn(id)
    {
        for (var i = 0; i < columns.length; i++)
        {
            if (columns[i].Id == id)
            {
                return columns[i];
            }
        }
    }
    function getBestLine(node, pnode, checkval, nodeLeft, pnodeLeft)
    {
        var c = checkval;
        var column = getColumn(c);
        var hasCollision = false;
        var usedLines = [];
        //check for node on same line
        for (var i = node.Index + 1; i < pnode.Index; i++)
        {
            var cgn = indexedNodes[i];
            usedLines.push(cgn.ColumnID * 20);
            if (cgn.ColumnID == c)
            {
                hasCollision = true;              
            }
        }
        //check for any lines on the same line
        for (var i = node.Index; i < pnode.Index; i++)
        {
            usedLines = usedLines.concat(lineIndexes[i]);
            if (lineIndexes[i])
            {
                if ($.inArray(column.LeftOffset, lineIndexes[i]) >= 0)
                {
                    hasCollision = true;                
                }
            }
        }

        if (hasCollision)
        {
            var firstTry = column.LeftOffset + 10;
            while ($.inArray(firstTry, usedLines) >= 0)
                firstTry += 10;

            return { change: true, line: Math.abs(nodeLeft - firstTry) };
        }

        return { change: false, line: Math.abs(nodeLeft - pnodeLeft) };

    }

    function getBestColumn(node, pnode)
    {
        var maxColumn = -1;
        var column = -1;
        var clear = true;
        var columnsInUse = [];

        for (var i = node.Index + 1; i < pnode.Index; i++)
        {
            var cgn = indexedNodes[i];
            columnsInUse.push(cgn.ColumnID);
            maxColumn = Math.max(maxColumn, cgn.ColumnID);
        }

        $.each(currentLeaves, function (index, pid)
        {
            var cgn = graphNodes[pid];
            columnsInUse.push(cgn.ColumnID);
            maxColumn = Math.max(maxColumn, cgn.ColumnID);
        });

        for (var j = pnode.ColumnID; j < maxColumn; j++)
        {
            if ($.inArray(j, columnsInUse) < 0)
            {
                column = j;
                break;
            }
        }

        if (column < 0)
        {
            column = maxColumn + 1;
        }

        return column;
    }

    function positionNode(cgn)
    {
        var minLeafColumn = maxCols;
        var minAvailParent = 100;
        var parentsInArray = [];

        var countParentsFetched = 0;

        for (pid in cgn.Dag.parents)
        {
            var pnode = graphNodes[pid];
            if (pnode)
            {
                cgn.Parents.push(pnode);
                pnode.Children.push(cgn);
                countParentsFetched++;
                if (currentLeaves.length > 0)
                {
                    var idx = $.inArray(pid, currentLeaves);
                    if (idx >= 0)
                    {
                        currentLeaves.splice(idx, 1);
                        var curColumn = pnode.ColumnID;
                        minLeafColumn = Math.min(minLeafColumn, curColumn);
                        parentsInArray.push(pnode);
                    }
                    else
                    {
                        var c = getBestColumn(cgn, pnode);
                        columnNum = c;
                    }
                }
            }
            else
            {
                //if filtered by user or stamp or path don't
                //change the column or the graph could get crazy huge
                if (!(filterflags & FLAG_NAME_FILTER) && !(filterflags & FLAG_STAMP_FILTER) && !(filterflags & FLAG_PATH_FILTER))
                {
                    var c = getBestColumn(cgn, lastNode);
                    columnNum = c;
                }
            }

        }

        if (parentsInArray.length > 0)
        {
            if (parentsInArray.length == cgn.Dag.parents.length || countParentsFetched == 1)
            {
                columnNum = cgn.ColumnID = minLeafColumn;
            }
            else
            {
                columnNum = cgn.ColumnID = Math.min(columnNum, minLeafColumn);
            }
        }
        else
        {
            cgn.ColumnID = columnNum;
        }
        maxCols = Math.max(maxCols, cgn.ColumnID);

        var column = getColumn(cgn.ColumnID);
        if (!column)
        {
            column = new graphColumn(cgn.ColumnID);
            columns.push(column);
        }

        //TODO we don't sort now, so this probably isn't needed
        //column.NodeCount++;
        currentLeaves.push(cgn.Dag.changeset_id);
    }

    function calculatePositions()
    {
        columns = [];
        columns.length = 0;
        columnNum = 0;
        currentLeaves = [];

        for (var i = indexedNodes.length - 1; i >= 0; i--)
        {
            var cgn = indexedNodes[i];

            if (i == indexedNodes.length - 1)
            {
                lastNode = cgn;
            }

            if (cgn)
            {
                cgn.reset();

                positionNode(cgn);
            }
        }
    }

    function formatFamilyBlockFromServer(v, div, childrenOnly)
    {
        var url = sgCurrentRepoUrl + "/changesets/" + v + ".json";
        div.addClass('smallProgress').css({ "height": 200, "width": 200 });

        vCore.ajax(
			{
			    url: url,
			    dataType: 'json',

			    error: function (xhr, textStatus, errorThrown)
			    {
			        div.removeClass('spinning');
			    },
			    success: function (data)
			    {
			        div.removeClass('smallProgress').css({ "height": "", "width": "" });
			        if (data)
			        {
			            if (data.description.children.length > 0)
			            {
			                div.append("<b>Children</b>");

			                $.each(data.description.children, function (i, child)
			                {
			                    var pChild = $('<p>').addClass("commentlist");
			                    pChild.append("<b>" + child.revno + ":" + child.changeset_id.slice(0, 10) + "</b>").append($('<br/>'));
			                    setTimeStamps(child.audits, pChild);
			                    pChild.append($('<br/>'));
			                    if (child.comments.length > 0)
			                    {
			                        var vc = new vvComment(child.comments[0].text);
			                        pChild.append(vc.summary);
			                    }
			                    div.append(pChild);
			                });
			            }
			            if (!childrenOnly)
			            {
			                div.append("<b>Parents</b>");

			                $.each(data.description.parents, function (i, parent)
			                {
			                    var pParent = $('<p>').addClass("commentlist");
			                    pParent.append("<b>" + parent.revno + ":" + i.slice(0, 10) + "</b>").append($('<br/>'));

			                    setTimeStamps(parent.audits, pParent);
			                    pParent.append($('<br/>'));
			                    if (parent.comments.length > 0)
			                    {
			                        var vc = new vvComment(parent.comments[0].text);
			                        pParent.append(vc.summary);
			                    }
			                    div.append(pParent);
			                });
			            }
			        }

			    }
			});
    }

    function addFamilyPopupInfo(nodes, div, title)
    {
        div.append("<b>" + title + "</b>");

        $.each(nodes, function (i, v)
        {
            var p = $('<p>').addClass("commentlist");
            var c = graphNodes[v.Dag.changeset_id];

            if (c)
            {
                p.append("<b>" + c.Dag.revno + ":" + c.Dag.changeset_id.slice(0, 10) + "</b>").append($('<br/>'));
                setTimeStamps(c.Dag.audits, p);
                p.append($('<br/>'));
                if (c.Dag.comments.length > 0)
                {
                    var vc = new vvComment(c.Dag.comments[0].text);
                    p.append(vc.summary);
                }
            }

            div.append(p);
        });
    }


    function addMouseEvents(circle, node)
    {
        var pop = new modalPopup("", true);

        circle.onmouseover = function (e)
        {
            var gotKids = false;

            svg.change(circle, { r: node.Radius + 2 });
            highlightCloseFamilyNodes(node, YELLOW_FILL, YELLOW_STROKE, false, FLAG_CHILD_HIGHLIGHT | FLAG_PARENT_HIGHLIGHT);

            var divParents = $('<div>');
            var divChildren = $('<div>');
            var div = $('<div>');
            if (node.branchname && node.branchname.length)
            {
                div.html("<b>Branch: " + node.branchname.join(", ") + "</b><br/><br/>");
            }
            var parents = [];
            $.each(node.Dag.parents, function (i, p)
            {
                parents.push(p);
            });
            if (parents.length > 0)
            {
                if (node.Parents.length == parents.length)
                {
                    addFamilyPopupInfo(node.Parents, divParents, "Parents");
                }
                else
                {
                    formatFamilyBlockFromServer(node.Dag.changeset_id, div, false, node.branchname);
                    //if we have to get any of the parents from the server, we will have
                    //the child info so set this so we don't hit the server twice
                    gotKids = true;

                }
            }

            if (!gotKids)
            {
                if ((filterflags & FLAG_NAME_FILTER) || (filterflags & FLAG_STAMP_FILTER) || (filterflags & FLAG_TO_FILTER))
                {
                    //if the history results have been filtered by name, stamp or date to
                    //we need to get the changeset info to make sure all the kids are accounted for
                    formatFamilyBlockFromServer(node.Dag.changeset_id, divChildren, true);

                }
                //We will always have all the kids that exist if the query is not filtered by name, stamp or date to
                else if (node.Children.length > 0)
                {
                    addFamilyPopupInfo(node.Children, divChildren, "Children");
                }

            }
            div.append(divChildren);
            div.append(divParents);

            $('#maincontent').prepend(pop.create("", div));
            pop.show(null, 0, node.Div.offset(), 0, maxCols * colSpacer + 30);

        };
        circle.onmouseout = function (e)
        {
            highlightCloseFamilyNodes(node);
            if (currentHighlight == null || node.Dag.revno != currentHighlight.Dag.revno)
            {
                svg.change(circle, { r: node.Radius });
            }
            pop.close();
        };
        circle.onclick = function (e)
        {
            visitedParentHighlights = {};
            visitedChildHighlights = {};
            if (currentHighlight)
            {
                svg.change(currentHighlight.Circle, { r: node.Radius });
                highlightAllParentNodes(currentHighlight, BLUE_FILL, BLUE_STROKE, true);
                highlightAllChildNodes(currentHighlight, BLUE_FILL, BLUE_STROKE, true);
                visitedParentHighlights = {};
                visitedChildHighlights = {};

                if (currentHighlight.Dag.revno == node.Dag.revno)
                {
                    currentHighlight = null;

                    return;
                }
            }

            currentHighlight = node;
            svg.change(circle, { r: node.Radius + 2 });
            highlightAllParentNodes(node, GREEN_FILL, GREEN_STROKE, true);
            highlightAllChildNodes(node, GREEN_FILL, GREEN_STROKE, true);

            // window.location = sgCurrentRepoUrl + "/changeset.html?recid=" + node.Dag.changeset_id;
        };
    }

    this.addToGraph = function (dag, div, index, branchname)
    {
        var gn = new graphNode(dag, div, index, branchname);
        graphNodes[dag.changeset_id] = gn;
        indexedNodes[index] = gn;
        if (branchname && branchname.length)
        {
            branchNodes.push(gn);
        }
    };

    function highlightAllParentNodes(node, fillColor, strokeColor, stick)
    {
        highlightCloseFamilyNodes(node, fillColor, strokeColor, stick, FLAG_PARENT_HIGHLIGHT);
        visitedParentHighlights[node.Dag.revno] = node;
        $.each(node.Parents, function (i, n)
        {
            if (!visitedParentHighlights[n.Dag.revno])
            {
                highlightAllParentNodes(n, fillColor, strokeColor, stick);
            }
        });
    };

    function highlightAllChildNodes(node, fillColor, strokeColor, stick)
    {
        highlightCloseFamilyNodes(node, fillColor, strokeColor, stick, FLAG_CHILD_HIGHLIGHT);
        visitedChildHighlights[node.Dag.revno] = node;

        $.each(node.Children, function (i, n)
        {
            if (!visitedChildHighlights[n.Dag.revno])
            {
                highlightAllChildNodes(n, fillColor, strokeColor, stick);
            }
        });
    };


    function highlightCloseFamilyNodes(node, fillColor, strokeColor, stick, flgs)
    {
        var circle = node.Circle;

        var strWidth = 10;
        var flags = flgs || (FLAG_CHILD_HIGHLIGHT | FLAG_PARENT_HIGHLIGHT);


        if (node)
        {
            if (flags & FLAG_PARENT_HIGHLIGHT)
            {
                $.each(node.SvgParentCircleFamily, function (i, n)
                {
                    var cnode = graphNodes[n];
                    if (cnode)
                    {
                        if (stick)
                        {
                            cnode.fillColor = fillColor;
                            cnode.strokeColor = strokeColor;
                        }
                        svg.change(cnode.Circle, { fill: fillColor || cnode.fillColor, stroke: strokeColor || cnode.strokeColor });
                    }
                });

                $.each(node.SvgParentLineFamily, function (i, n)
                {
                    var line = lineMap[n];
                    if (line)
                    {
                        if (stick)
                        {
                            line.strokeColor = strokeColor;
                        }
                        svg.change(line.line, { fill: 'none', stroke: strokeColor || line.strokeColor, strokeWidth: strWidth });
                    }
                });
            }
            if (flags & FLAG_CHILD_HIGHLIGHT)
            {
                $.each(node.SvgChildCircleFamily, function (i, n)
                {
                    var cnode = graphNodes[n];
                    if (cnode)
                    {
                        if (stick)
                        {
                            cnode.fillColor = fillColor;
                            cnode.strokeColor = strokeColor;
                        }
                        svg.change(cnode.Circle, { fill: fillColor || cnode.fillColor, stroke: strokeColor || cnode.strokeColor });
                    }
                });

                $.each(node.SvgChildLineFamily, function (i, n)
                {
                    var line = lineMap[n];
                    if (line)
                    {
                        if (stick)
                        {
                            line.strokeColor = strokeColor;
                        }
                        svg.change(line.line, { fill: 'none', stroke: strokeColor || line.strokeColor, strokeWidth: strWidth });
                    }
                });
            }
        }

    };



    this.clearGraph = function ()
    {
        if (svgGroup)
        {
            try
            {
                svg.remove(svgGroup);
            }
            catch (e)
            {

            }
        }
        indexedNodes = [];
        lineIndexes = {};
        currentLeaves = [];
        branchNodes = [];
        graphNodes = {};
        lineMap = {};
        maxCols = -1;
        columns = [];
        hItems = [];
        columnNum = 0;
        visitedParentHighlights = {};
        visitedChildHighlights = {};
    };

    this.update = function (s)
    {
    
        visitedParentHighlights = {};
        visitedChildHighlights = {};
        lineMap = {};
        if (!svg)
            svg = s;

        if (!svg)
            return;
        $('#graphdiv').show();
        maxCols = -1;
        if ($('.commentcell').html() == null)
            return;

        calculatePositions();

        var left = $('td.commentcell').first().position().left;
        var top = $('td.commentcell').first().position().top;

        var canvasWidth = (maxCols * colSpacer) + (2 * colSpacer);
        //make room on canvas for branch names
        $.each(branchNodes, function (b, bnode)
        {
            for (var ib = 0; ib < bnode.branchname.length; ib++)
            {
                var len = bnode.branchname[ib].length;
                var off = (len * 5) + getColumn(bnode.ColumnID).LeftOffset;
                canvasWidth = Math.max(canvasWidth, off);
            }

        });
        var _windowHeight = windowHeight();
        var height = Math.max($('#tablehistory').height() + 60, _windowHeight);

        $("td.commentcell").css({ "padding-left": canvasWidth + 25 });
        $("th.comment").css({ "padding-left": canvasWidth + 25 });
        //set the approx height of the div.
        $('#graphdiv').css({ "height": height, "width": canvasWidth + 20, "position": "absolute", "top": top, "left": left });

        if (svgGroup)
        {
            try
            {
                svg.remove(svgGroup);
            }
            catch (e)
            {

            }
        }

        svgGroup = svg.group();

        var gLines = svg.group(svgGroup, { fill: 'none', stroke: 'rgb(0, 88, 104)', strokeWidth: 1 });
        var c;
        lineIndexes = {};

        $.each(graphNodes, function (index, node)
        {
            node.calcPosition();
            var leftCol = getColumn(node.ColumnID);

            if (leftCol)
                left = leftCol.LeftOffset;

            c = svg.circle(svgGroup, left, node.Top, node.Radius, { id: "c" + node.Index, fill: 'rgb(230,248,251)', stroke: 'rgb(0, 88, 104)', strokeWidth: 1 });

            node.Circle = c;
            node.Left = left;
            node.SvgParentCircleFamily.push(node.Dag.changeset_id);
            node.SvgChildCircleFamily.push(node.Dag.changeset_id);
            $.each(node.Children, function (i, v)
            {
                v.SvgParentCircleFamily.push(node.Dag.changeset_id);
            });

            if (node.branchname && node.branchname.length)
            {
                var topMult = 3;
                var startTop = (node.branchname.length > 1 ? (node.Top - (node.branchname.length / 2) * 10) : node.Top);
                $.each(node.branchname, function (b, bname)
                {
                    var btext = svg.text(svgGroup, node.Left + 8, startTop + topMult, bname, { fill: 'rgb(0, 88, 104)', fontSize: '10' })
                    topMult += 10;
                });
            }
            addMouseEvents(c, node);
            //$.each(node.Dag.parents, function (i, pid)
            var pGraphNodes = [];
            for (var pid in node.Dag.parents)
            {
                var pnode = graphNodes[pid];
                if (pnode)
                {
                    pGraphNodes.push(pnode);
                }
            }
            pGraphNodes.sort(function (a, b)
            {
                return (b.Index - a.Index);
            });
            $.each(pGraphNodes, function (i, pnode)
            {

                if (pnode)
                {
                    pnode.calcPosition();
                    pnode.SvgChildCircleFamily.push(node.Dag.changeset_id);
                    var l;
                    var pleft = colSpacer;
                    var pCol = getColumn(pnode.ColumnID);
                    if (pCol)
                        pleft = pCol.LeftOffset;
                    var finalLinePos = -1;
                    if (Math.abs(pnode.Index - node.Index) == 1)
                    {
                        l = svg.polyline(gLines, [[left, node.Bottom - 5], [pleft, pnode.Top]]);
                    }
                    else
                    {
                        if (left < pleft)
                        {
                            var lineLength = getBestLine(node, pnode, pnode.ColumnID, left, pleft);
                            finalLinePos = left + lineLength.line;
                            if (lineLength.change)
                            {
                                l = svg.polyline(gLines, [[left, node.Bottom - 5], [finalLinePos, node.Top + 15], [left + lineLength.line, pnode.Top - 15], [pleft, pnode.Top]]);
                            }
                            else
                            {
                                l = svg.polyline(gLines, [[left, node.Bottom - 5], [finalLinePos, node.Bottom + 15], [pleft, pnode.Top]]);
                            }
                        }
                        else
                        {
                            var lineLength = getBestLine(node, pnode, node.ColumnID, left, pleft);

                            if (lineLength.change)
                            {
                                finalLinePos = left + lineLength.line;
                                l = svg.polyline(gLines, [[left, node.Bottom - 5], [finalLinePos, node.Top + 15], [finalLinePos, pnode.Top - 15], [pleft, pnode.Top]]);
                            }
                            else
                            {
                                finalLinePos = left;
                                l = svg.polyline(gLines, [[left, node.Bottom - 5], [left, pnode.Top - 15], [pleft, pnode.Top]]);

                            }
                        }
                    }
                    var lineKey = node.Dag.revno.toString() + pnode.Dag.revno.toString();
                    lineMap[lineKey] = { line: l, strokeColor: node.strokeColor };
                    node.SvgParentLineFamily.push(lineKey);
                    pnode.SvgChildLineFamily.push(lineKey);

                    for (var line = node.Index; line < pnode.Index; line++)
                    {
                        if (line >= 0)
                        {
                            if (!lineIndexes[line])
                            {
                                lineIndexes[line] = [];
                            }
                            lineIndexes[line].push(finalLinePos);
                        }
                    }

                }
            });
        });

        //recalculate the height and set the view port accordingly
        height = Math.max($('#tablehistory').height() + 60, _windowHeight);
        svg.change(svg.root(), { height: height, width: canvasWidth + 20 });
        if (currentHighlight)
        {
            svg.change(currentHighlight.Circle, { r: currentHighlight.Radius + 2 });
            highlightAllParentNodes(currentHighlight, GREEN_FILL, GREEN_STROKE, true);
            highlightAllChildNodes(currentHighlight, GREEN_FILL, GREEN_STROKE, true);
        }
    };

    this.draw = function ()
    {
        try
        {
            if (!svg)
            {
                $('#graphdiv').svg(
					{
					    onLoad: this.update,
					    settings: { overflow: "visible" },
					    initPath: sgStaticLinkRoot + "/lib/"
					});
                if ($.browser.msie)
                {
                    $('embed').attr("wmode", "transparent");
                }
            }
            else
                this.update();
        }
        catch (e)
        {
            $('#graphdiv').hide();
        }

        $(document).mouseup($(this), function (e)
        {
            e.stopPropagation();
            if (currentHighlight)
            {
                if (!$(e.target).is("circle"))
                {
                    svg.change(currentHighlight.Circle, { r: currentHighlight.Radius });

                    visitedParentHighlights = {};
                    visitedChildHighlights = {};
                    highlightAllParentNodes(currentHighlight, BLUE_FILL, BLUE_STROKE, true);
                    highlightAllChildNodes(currentHighlight, BLUE_FILL, BLUE_STROKE, true);

                    currentHighlight = null;
                }

            }

        });
    };
   
}


