/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/


function sprintFormController()
{
	this.treeController = null;
	this.loading = null;
	this.form = null;
	this.pop = null;
	this.currentSprint = sgCurrentSprint;
	this.sprints = [];
	
	this.releaseSprint = null;
	this.sprintItems = null;
	
	var self = this;

	this.init = function()
	{
		var self = this;
		var header = $("<h1>");
		header.text("Milestones");
		$('#rightSidebar').append(header);

		var newSprint = $("<a>");
		newSprint.attr({
			href: "#",
			id: "newSprint"
		});

		newSprint.text("New Milestone").css( "float", "right");
		header.append(newSprint);

		$("#newSprint").click(function (event) { self.newSprintClicked(); });

		var cont = $("<div>");
		cont.attr("id", "treeControl");
		$('#rightSidebar').append(cont);

		$("#sprintForm").text("Select a milestone or 'New Milestone' in the sidebar");
		
		this.treeController = new SprintTreeControl($('#treeControl'), { 
			selectCurrent: true,
			beforeSprintSelect: function() { return self.sprintWillChange(); },
			onSelect: function() { self.sprintSelected(); }
		});

		this._fetchCurrent();
	};

	this._fetchCurrent = function(after)
	{
		vCore.ajax( {
			url : vCore.repoRoot() + "/config/current_sprint",
			dataType: 'json',
			success: function(data, xhr, status) {
				if (data)
					self.currentSprint = data.value;
				if (after)
					after(data.value);
			}
		});
	};

	this._formOptions = {
		rectype: "sprint",
		errorFormatter: bubbleErrorFormatter,
		beforeFilter: function(form, rec) {
			var startdate = form.getRecordValue("startdate");
			if (!rec.startdate)
			{
				if (startdate != null || startdate != undefined)
				{
					rec.startdate = null;
				}
				else
				{
					delete rec.startdate;
				}
			}
			var enddate = form.getRecordValue("startdate");
			if (!rec.enddate)
			{
				if (enddate != null || enddate != undefined)
				{
					rec.enddate = null;
				}
				else
				{
					delete rec.enddate;
				}
			}
				
			return rec;
		},
		customValidator: function(form) { return self.validate(form); },
		afterSubmit: function(form) {
			self.treeController.refresh(form.getRecordValue("recid"));
		}
	};

	this.newSprintClicked = function()
	{
		if (this.form)
		{
			if (this.form.willBeDestroyed() === false)
			return;

			delete this.form;
			this.form = null;
		}

		this.treeController.clearSelected();

		$("#sprintForm").empty();
		var vFormOptions = jQuery.extend(true, 
			{
				url: vCore.repoRoot() + "/milestones",
				method: "POST"
			}, this._formOptions);

		this.form = new vForm(vFormOptions);

		this.renderForm(this.form);
	};

	this.sprintSelected = function(recid)
	{
		$("#sprintForm").empty();
		var sprint = this.treeController.getSelectedSprint();

		if (!sprint)
		{
			$("#sprintForm").text("Select a milestone or 'New Milestone' in the sidebar");
			return;
		}
		
		var vFormOptions = jQuery.extend(true, {
			url: vCore.repoRoot() + "/milestone/" + sprint.recid,
			method: "PUT"
		}, this._formOptions);

		this.form = new vForm(vFormOptions);
		this.form.setRecord(sprint);
		this.renderForm(this.form);
	};

	this.sprintWillChange = function()
	{
		if (this.form)
		{
			if (this.form.willBeDestroyed() === false)
				return false;

			delete this.form;
			this.form = null;
		}

		return true;
	};

	this.renderForm = function(form)
	{

		if (!form)
			return;

		var invalidParents = [];
		var sprints = this.treeController.getSprints();
		var recid = null;
		
		if (form.record)
		{
			recid = form.record.recid;
		}
		
		invalidParents = this._buildInvalidParents(sprints, form.record, invalidParents);
		
		// Name
		var namelabel = form.label("name", "Name:");
		form.field($("#sprintForm"), "text", "name", {
			label: "Name",
			validators: { required: true }
		});

		// Description
		form.field($("#sprintForm"), "textarea", "description", {
			label: "Description",
			rows: 3
		});

		// Parent Sprint
		form.field($("#sprintForm"), "sprintSelect", "parent", {
			label: "Parent:",
			nested: true,
			filter: function(value)
			{
				if (value)
					value = value.recid;
					
				if ($.inArray(value, invalidParents) > -1)
					return false;

				return true;
			},
		});

		var dateDiv = $(document.createElement("div"));
		dateDiv.attr({
			id: "dates"
		}).addClass("clearfix");

		// Start Date
		var startDiv = $(document.createElement("div"));
		form.field(startDiv, "dateSelect", "startdate", {
			label: "Start Date:",
			onchange: function(input) { self.sprintDatesDidChange(input); }
		});
		dateDiv.append(startDiv);

		// End Date
		var endDiv = $(document.createElement("div"));
		form.field(endDiv, "dateSelect", "enddate", {
			label: "End Date:",
			onchange: function(input) { self.sprintDatesDidChange(input); }
		});

		dateDiv.append(endDiv);

		$("#sprintForm").append(dateDiv);

		var workingDays = $(document.createElement("div"));
		workingDays.attr("id", "workingDays");
		$("#sprintForm").append(workingDays);

		var buttons = $(document.createElement("div"));
		buttons.attr("id", "buttons").addClass("clearfix");
		var right = $(document.createElement("div")).css("float", "right");
		var left = $(document.createElement("div")).css("float", "left");

		// Submit and Cancel
		var releasedate = form.getRecordValue("releasedate");
		if (releasedate)
		{
			var date = new Date(parseInt(releasedate));
			var fdate = vCore.formatDate(date);
			var rdiv = $("<div>").text("Released: " + fdate);
			left.append(rdiv);
		}
		else
		{
			if (form.record)
			{
				if (this.currentSprint != form.getRecordValue("recid"))
				{
					var current = form.button("Set Current");
					current.click(function ()
					{
						$("#sprintForm").empty();
						
						var progress = new smallProgressIndicator("releaseSprogress", "Setting Current Sprint");
						$("#sprintForm").append(progress.draw());
						
						vCore.setConfig("current_sprint", form.getRecordValue("recid"), {
							success: function ()
							{
								self._fetchCurrent(function(current) {
									self.treeController.currentSprint = current;
									self.treeController.refresh();
								});
								
							}
						});
					});
					left.append(current);

				}
				else
				{
					var cur = $("<span>").text("Current Milestone");
					left.append(cur);
				}

				var release = form.button("Release");
				release.click(function () { self.releaseSprintClicked(); });
				left.append(release);
			}
		}

		right.append(form.submitButton("Ok"));

		var cancel = form.button("Cancel");
		cancel.click(function () { form.reset(); });
		right.append(cancel);
		buttons.append(right).append(left);
		$("#sprintForm").append(buttons);

		if (releasedate)
		{
			form.disable();
		}

		form.ready();
	};

	this.sprintDatesDidChange = function(input)
	{
		
		if (input.attr)
			var name = input.attr("name");
			
		var startId = this.form._generateId("startdate");
		var endId = this.form._generateId("enddate");
		var startDate = $("#" + startId).datepicker('getDate');
		var endDate = $("#" + endId).datepicker('getDate');
		
		var sdfield = this.form.getField("startdate");
		var edfield = this.form.getField("enddate");
		
		this.form.hideErrors();
		
		if (!startDate && !endDate)
		{
			$("#workingDays").text("");
			return;
		}
		
		if (name == "startdate" && startDate && !endDate)
		{
			$("#" + endId).datepicker("setDate", startDate);
			endDate = startDate;
		}
		
		if (name == "enddate" && endDate && !startDate)
		{
			$("#" + startId).datepicker("setDate", endDate);
			startDate = endDate;
		}
		
		if (!this.form.validate())
		{
			var nameInp = this.form.getField("name");
			this.form.displayErrors();
		}
		else
		{
			var days = vCore.workingDays(startDate, endDate);

			$("#workingDays").text(days + " Working Days" );
		}
	};

	this.validate = function(form)
	{
		var record = form.record;
		var newRec = form.getFieldValues();

		// Get the field we might need to add errors to
		var sdfield = form.getField("startdate");
		var edfield = form.getField("enddate");
		var nameField = form.getField("name");
		
		var valid = true;

		if (newRec.startdate && ! newRec.enddate)
		{
			valid = false;

			form.addError(edfield, 'An end date is needed when a start date is given')
		}
		else if (newRec.enddate && ! newRec.startdate)
		{
			valid = false;
			form.addError(sdfield, 'A start date is needed when an end date is given');
		}
		else if (newRec.enddate < newRec.startdate)
		{
			valid = false;
			//form.addError(sdfield, "Milestone start date can't be later than the end date");
			form.addError(edfield, "Milestone end date can't be earlier than the start date")
		}

		if (newRec.name.toLowerCase() == "product backlog")
		{
			valid = false;
			form.addError(nameField, "\"" + newRec.name + "\" in not a valid name");
		}
		
		var precid = null;
		if (newRec.parent)
			precid = newRec.parent;
			
		var parent = this.treeController.milestoneTree.getSubTree(precid);
		
		$.each(parent.root.children, function(i, node) {
			if (node.data.recid == form.getRecordValue("recid"))
				return;
				
			if (node.data.name == newRec.name)
			{
				valid = false;
				form.addError(nameField, "The milestone name is not unique for parent " + parent.root.title + ", please pick a new name");
			}
		});
		
		var parentId = this.form._generateId("parent");
		var parent_recid = $("#" + parentId).val();
		var parent = this.treeController.getSprint(parent_recid);

		if (parent && parent.children && record)
		{
			$.each(parent.children, function(i,node) {
				var sprint = node.data;
				if (!sprint)
					return;

				if (sprint.recid == record.recid)
					return;

				var _sstart = new Date(parseInt(sprint.startdate));
				var _send = new Date(parseInt(sprint.enddate));

				if ((newRec.startdate >= _sstart.getTime() && 
					newRec.startdate <= _send.getTime()) ||
					(newRec.enddate >= _sstart.getTime() &&
					newRec.enddate <= _send.getTime())
					)
				{
					valid = valid && confirm("Milestone dates overlap with milestone " + sprint.name + "\n Save anyway?");
					return false; // break loop
				}	

			});
		}
		return(valid);
	};

	this.releaseSprintClicked = function()
	{
		var self = this;
		var recid = this.form.getRecordValue("recid");
		this.pop = new modalPopup("#backgroundPopup", null, {
			onClose: function() {
				// Remove the release form as a sprint form subform
				self.form.subForms = [];
				// Clear the form callback
				self.form.options.afterValidation = null;
			}
		});
		
		var div = $("<div>");
		
		var toReleaseTree = this.treeController.milestoneTree.getSubTree(recid);

		unreleased = [];
		for (var i = 0; i < toReleaseTree.root.children.length; i++)
		{
			var node = toReleaseTree.root.children[i];
			if (!node.data.releasedate)
				unreleased.push(node.data);
		}
		
		if (unreleased.length > 0)
		{
			var p = $("<p>").text("Unable to release milestone.  The following milestones are still open.");
			div.append(p);
			
			var names = unreleased.map(function(v) { return v.name; });
			var list = $("<p>").html(names.join("<br/>"));
			div.append(list);
			
			var close = $("<input type='button' value='Close'/>");
			close.click(function() { 
				self.pop.close(); 
			});
			close.css("float", "right");
			div.append(close);
		}
		else
		{
			this.loading = new smallProgressIndicator("releaseLoading", "Loading milestone...");
			div.append(this.loading.draw());
		}
		
		$('#maincontent').prepend(this.pop.create("Release Milestone", div).addClass('info'));
		this.pop.show();
		if (unreleased.length > 0)
			return;
			
		var queue = new ajaxQueue({
			onSuccess: function()
			{
				self._displaySprintReleaseForm();
			}
		});
		
		queue.addQueueRequest({
			url : vCore.getOption("currentRepoUrl") + "/milestone/" + recid + ".json",
			dataType : 'json',
			success : function(data) 
			{
				self.releaseSprint = data;
			}
		});
		
		queue.addQueueRequest({
			url : vCore.getOption("currentRepoUrl") + "/milestone/" + recid + "/items.json",
			dataType : 'json',
			success : function(data) 
			{
				self.sprintItems = data;
			}
		});
		
		queue.execute();
	};

	this._displaySprintReleaseForm = function()
	{
		if (!this.releaseSprint)
		{
			return;
		}
		
		var sprint = this.releaseSprint;
		var items = [];
		
		if (this.sprintItems)
		{
			items = this.sprintItems;
		}

		
		// Add a call back to the master form so we can blank the form on submit
		this.form.options.afterValidation = function(form, valid) { return self._formWillSubmit(form, valid); }

		var options = {
			rectype: "release",
			forceDirty: true,
			url: vCore.repoRoot() + "/milestone/release/" + sprint.recid,
			method: "PUT",
			afterSubmit: function() { self._sprintReleased(); }
		};

		var rform = new vForm(options);
		var rec = {
			rectype: "release",
			move: "true"
		};
		rform.setRecord(rec);

		var openCount = 0;
		$.each(items, function(i,v) {
			if (v.status == "open")
				openCount++;
		});

		var div = $(document.createElement("div"));
		div.attr({
			"id": "sprintReleaseForm"
		});

		if (openCount != 0)
		{
			var open = $("<span>");
			var openLink = $("<a>").text(openCount + " incomplete work items were found.");
			openLink.attr({
				href: vCore.repoRoot() + "/workitems.html?milestone=" + sprint.recid + "&status=open&sort=priority&groupedby=status&columns=id%2Ctitle%2Cassignee%2Creporter%2Cpriority%2Cmilestone&maxrows=100?skip=0",
				target: "_new"
			});
			open.append(openLink);
			div.append(open);

			var move = rform.radio("move", "true").attr("id", "sprint_release_move");
			var ignore = rform.radio("move", "false").attr("id", "sprint_release_ignore");

			var table = $("<table>");

			// first row
			var row = $("<tr>");
			row.append($("<td>").attr({valign: "top"}).append(ignore));
			row.append($("<td>").append(rform.label("move", "Ignore them").attr("for", ignore.attr("id"))));
			table.append(row);

			// second row
			row = $("<tr>");
			var td = $("<td>").attr( { "valign": "top"}).append(move);
			row.append(td);
			var td = $("<td>");
			td.append(rform.label("move", "Move them into:").attr("for", move.attr("id"))).append("<br/>");

			td.append(rform.sprintSelect("new_sprint", {
				//allowEmptySelection: true,
				emptySelectionText: "(Product Backlog)",
				nested: true,
				filter: function(sprint) {
					var selected = self.treeController.selectedRecid;

					if (!sprint || (!sprint.releasedate && (sprint.recid != selected)))
						return true;
				}
			}));

			td.append($("<br/>"));
			td.append(rform.booleanCheckbox("set_current").attr('disabled', true));
			td.append(rform.label("set_current", "Mark Current"));

			row.append(td);
			table.append(row)

			div.append(table);
		}
		else
		{
			var confirm = $("<span>").text("No incomplete items.");
			div.append(confirm);
			div.append($("<br/>"));
		}

		var buttons = $("<div>").attr("id", "release_buttons");
		
		var cancel = rform.button("Cancel");
		cancel.click(function() { 
			// Remove the release form as a sprint form subform
			self.form.subForms = [];
			// Clear the form callback
			self.form.options.afterValidation = null;
			self.pop.close(); 
		});
		cancel.css("float", "right");

		var release = this.form.submitButton("Release");
		release.css("float", "right");

		buttons.append(release);
		buttons.append(cancel);
		div.append(buttons);

		this.loading.hide();
		$("#popupContent").empty();
		$("#popupTitle").text("Release " + sprint.name);
		$("#popupContent").append(div);

		if ($("input[name=move]"))
		{
			$("input[name=move]").change(function(event) {
				var ele = event.target;
				if ($(ele).val() == "true")
				{
					$("#release_new_sprint").removeAttr('disabled');
					if ($("#release_new_sprint").val() != "")
						$("#release_set_current").removeAttr('disabled');
				}
				else
				{
					$("#release_new_sprint").attr('disabled', true);
					$("#release_set_current").attr('disabled', true);
				}
			});
		}
		
		if ($("#release_new_sprint"))
		{
			$("#release_new_sprint").change(function(event) {
				var ele = event.target;
				if ($(ele).val() == "")
				{
					$("#release_set_current").attr('disabled', true);
				}
				else
				{
					$("#release_set_current").removeAttr('disabled');
				}
			});
		}

		rform.ready();
		this.form.addSubForm(rform);
	};

	this._formWillSubmit = function(form, valid)
	{
		
		if (valid)
		{
			$("#sprintReleaseForm").hide();
			var progress = new smallProgressIndicator("releaseSprogress", "Releasing Sprint");
			$("#popupContent").append(progress.draw());
		}
		
		return true;
	};
	
	this._sprintReleased = function()
	{
		var self = this;
		
		this._fetchCurrent(function(current) {
			self.treeController.currentSprint = current;
			self.pop.close();
			$("#sprintForm").empty();
			
			self.treeController.refresh(self.form.getRecordValue("recid"));
		});
	};

	this._buildInvalidParents = function(sprints, sprint, arr)
	{
		if (sprint && sprint.recid && $.inArray(sprint.recid, arr) <= 0 )
			arr.push(sprint.recid);
		
		children = [];
		// Find child sprints
		for (var i = 0; i < sprints.length; i++)
		{
			var chksprint = sprints[i];
			
			if (chksprint.releasedate && (!sprint || sprint.parent != chksprint.recid) && $.inArray(chksprint.recid, arr) <= 0)
				arr.push(chksprint.recid);
				
			if (sprint && chksprint.parent == sprint.recid)
				arr = this._buildInvalidParents(sprints, chksprint, arr);
		}
		
		return arr;
	};
	
	// Start up the controller
	this.init();
}

function sgKickoff() {
	vCore.showSidebar();
	var controller = new sprintFormController();
}
