/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

Array.isArray = Array.isArray || function(o) { return Object.prototype.toString.call(o) === '[object Array]'; };


var _wiusers = {};
var _sprints = {};
var _comments = {};


function _disableButton(button)
{
	$(button).addClass('disabled');
	$(button).attr('disabled', true);
}

function _enableButton(button)
{
	$(button).removeClass('disabled');
	$(button).removeAttr('disabled');
}


function _niceUser(uid)
{
	if (_wiusers[uid])
		return(_wiusers[uid]);
	return(uid);
}
function _niceSprint(sid)
{
	if (_sprints[sid])
		return(_sprints[sid]);
	return(sid);
}

function _addAdder(addpage)
{
	var adder = $('#newwi');
	var addurl = sgCurrentRepoUrl + "/" + addpage;

	adder.attr('href', addurl);
}


function _watchContainerChanges(container, buttonlist)
{
	var inputs = $(container).find('input');
	var selects = $(container).find('select');
	var areas = $(container).find('textarea');

	var enabler = function (el) 
    {
		if (el.currentTarget)
			el = el.currentTarget;
            
		$.each($(el).data('buttonlist'),
			function(i, button)
			{
				_enableButton(button);
			}
		);
	};

	var addButtons = function(i, el) {
		var bl = [];

		if ($(el).data('buttonlist'))
			bl = $(el).data('buttonlist');

		bl = bl.concat(buttonlist);
		
		$(el).data('buttonlist', bl);
	};


	$.each(
		[inputs, selects, areas],
		function(i, ellist)
		{
			ellist.each(addButtons);
		}
	);

	selects.change( enabler );

	var checkOne = function(i, el) {
		if ((! $(el).attr('disabled')) &&
				$(el).is(':visible') &&
				($(el).val() || $(el).data('zflastval'))
		)
		{
			if ($(el).val() != $(el).data('zflastval'))
			{
				$(el).data('zflastval', $(el).val());
				enabler($(el));
			}
		}
	};

	var textChecker = function() {
		inputs.each( checkOne );
		areas.each( checkOne );

		window.setTimeout(textChecker, 250);
	};

	window.setTimeout(textChecker, 250);
}

function _watchChanges(containers, buttonlist)
{
	for ( var i in containers )  
		_watchContainerChanges(containers[i], buttonlist);
}

function _findCookie(cname, defval)
{
	var un = cname + "=";
	var v = defval || false;

	if (document.cookie.length > 0)
	{
		var cookies = document.cookie.split(';');

		for (var i = 0; i < cookies.length; ++i)
		{
			var cstr = jQuery.trim(cookies[i]);

			if (cstr.substring(0, un.length) == un)
			{
				// don't just return; in corner cases, multiple versions
				// of the cookie may exist. we want the last one.
				v = decodeURIComponent(cstr.substring(un.length));
			}
		}
	}

	return (v);
}

function _setCookie(cname, cval, exp)
{
	var exdate = new Date();
	exdate.setDate(exdate.getDate() + exp);

	var cstr = cname + "=" + encodeURIComponent(cval) +
	"; path=/" +
	"; expires=" + exdate.toUTCString();

	document.cookie = cstr;
}


// JQuery's innerHeight method is broken when you call is
// for the window i.e. $(window).innerHeight
// This is a workaround.
function windowHeight()
{
	// JQuery return a height of zero in safari for innerHeight
	// the window.innerHeight value is correct
	if (typeof window.innerHeight != 'undefined')
	{
		return window.innerHeight;
	}

	// in IE innerHeight return NaN
	if (isNaN($(window).innerHeight()))
	{
		return $(window).height();
	}

	return $(window).innerHeight();
}

function stretch(t)
{
	if (t < 10)
	{
		t = "0" + t;
	}

	return (t);
}


// TODO remove this date formatting code and use strftime.js
var actMonths = ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
                 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

function shortDateTime(d)
{

	var result = shortDate(d) + " at " + shortTime(d);

	return (result);
}

function shortDate(d)
{
	var result = actMonths[d.getMonth()] + " " + d.getDate();
	return (result);
}

function shortTime(d)
{
	var result = stretch(d.getHours()) + ":" + stretch(d.getMinutes());
	return (result);
}

function _shortWho(longwho)
{
	var rewho = /^(.+?)[ \t]*<.+>[ \t]*$/;

	var matches = longwho.match(rewho);

	if (matches)
		return (matches[1]);

	return (longwho);
}

function setTimeStamps(ts, div)
{
	var users = [];
	var dates = [];
	$.each(ts, function (i, v)
	{

		var d = new Date(parseInt(v.timestamp));
		var who = v.username || v.name || v.userid || '';
		if (div)
		{
			var csig = $('<span>').text(who + ",  ");
		
			csig.append(shortDateTime(d));
			div.append(csig);
			div.addClass("sig");
		}
		else
		{
			$('#who').append(who);
			$('#when').append(shortDateTime(d));
		}
	});


}


function vvComment(text)
{
    if (!text)
        return "";

    this.details = "";
    this.rest = "";
    this.summary = "";
    this.moreText = false;
   
    var formattedText = vCore.createLinks(text);
    formattedText = formattedText.replace(/\r\n/gm, "<br/>");
    formattedText = formattedText.replace(/[\r|\n]/gm, "<br/>");

    this.details = formattedText;

    var splits = formattedText.split("<br/>");
    this.summary = splits[0];
    this.moreText = splits.length > 1;
    if (this.moreText)
    {
        this.rest = "<br/>" + splits.slice(1).join("<br/>");
    }
   
}

function firstHistory(rec)
{
	var oldest = (rec.history.length > 0) ? rec.history[0] : null;

	for ( var i = 1; i < rec.history.length; ++i )
		if (rec.history[i].generation < oldest.generation)
			oldest = rec.history[i];

	return(oldest);
}

// This function assumes the comments have a history. Don't use for history
// items right now
// since comment history is turned off
function formatComments(comments, numDisplayed, extraTop, extraBottom)
{
	var div = $('<div>');
	var num = numDisplayed || Number.MAX_VALUE;
	comments.sort(
			function (a, b)
			{
				return( firstHistory(a).audits[0].timestamp - firstHistory(b).audits[0].timestamp );
			} );
			$.each(comments, function (index, comment)
			{
			    if (index < num)
			    {

			        var cp = $('<p>').addClass("commentlist");

			        var span = $('<span>');
			        setTimeStamps(firstHistory(comment).audits, span);
			        cp.append(span);
			        cp.append($('<br/>'));
			        var cdiv = $('<div>');
			        var vc = new vvComment(comment.text);			      
			        cdiv.html(vc.details);

			        cp.append(cdiv);
			        if (extraTop)
			        {
			            cp.prepend($('<p>'));
			            cp.prepend(extraTop);
			        }
			        if (extraBottom)
			        {
			            cp.append($('<p>'));
			            cp.append(extraBottom);
			        }
			    }
			    div.append(cp);
			});

	return div;
}

function findBaseURL(lastSeg)
{

	var url = window.location.pathname;
	if (url.indexOf(lastSeg) > 0)
	{
		url = url.substring(0, url.indexOf(lastSeg));
	}

	return url.rtrim("/");
}

function makePrettyRevIdLink(revno, id, expandable, prepend, append, urlBase)
{

    var base = urlBase || sgCurrentRepoUrl;
    var myURL = base + "/changeset.html?recid=" + id;

	var span = $('<span>');

	var csa = $('<a>').text((prepend ? prepend : "") + (revno ? (revno + ":") : "") + id.slice(0, 10) + (append ? append : ""))
	.attr({ href: myURL}).addClass("small_link");

	span.append(csa);

	if (expandable)
	{
		var a = $('<a>').text("...").attr({ href: "#" });

		span.append(a);

		a.click(function (e)
			{
				e.preventDefault();

				var info = $('<span>').text(id);

				var pop = new modalPopup("none");

				$('#maincontent').prepend(pop.create("", info));

				pop.show(null, null, a.position());
			});
	}
	return span;

}

function makeId(childurl)
{
	var idre = /[^a-zA-Z0-9]/g;

	var url = childurl.replace(/\/[^\/]*\/?$/, '');

	var theid = url.replace(idre, "_");

	return (theid);
}

function boolFromZingVal(zingVal)
{
	if (zingVal == "1")
		return true;
	if (zingVal == "0")
		return false;
	return Boolean(zingVal);
}

function lastURLSeg(url)
{

	var p = url || window.location.pathname;
	p = p.rtrim("/");

	var ar = p.split('/');

	return (ar[ar.length - 1]);

}

function getParentPath(path)
{

	var parent = path.rtrim("/");

	var index = parent.lastIndexOf("/");

	if (index < 0)
	{

		return "@";
	}

	parent = parent.slice(0, index);

	return parent;

}

function cwd()
{
	var p = window.location.pathname;
	if (p[p.length - 1] == '/')
		return p;
	else
		return p + '/';
}

function isScrollBottom()
{
	var documentHeight = $(document).height();
	var _windowHeight = windowHeight();

	var scrollP = _windowHeight + $(window).scrollTop();

	return (scrollP + 5 >= documentHeight);
}

function isVerticalScrollRequired()
{
	var docHeight = $(document).height() - 5; // Extraordinarily lame hack to deal with IE's weird height();
	return docHeight > windowHeight();
}


function vButton(text)
{
	var span = $('<span>').addClass("button").text(text);

	return span;
}

// message can be object (span, div etc)
// optional posObject used to position the dialog
// optional onOKfunction is called when the user clicks yes
// optional args for the onOKfunction
function confirmDialog(message, posObject, onOKfunction, okargs, onCancelfunction, cancelargs, offset)
{
    this.ok = false;
    this.offset = offset || [0, 0];

	var div = $('<div>').append(message);
	var buttonOK = new vButton("yes");
	var buttonCancel = new vButton("no");
	var span = $('<span>').addClass("dialogButtons");

	buttonOK.click(function ()
	{
	    this.ok = true;
	    pop.close();
        if (onOKfunction)
	        onOKfunction(okargs);
	  
	});

	buttonCancel.click(function ()
	{
	    this.ok = false;
	    pop.close();
	    if (onCancelfunction)
	        onCancelfunction(cancelargs);
	    
	});

	div.append($('<br/><br/>'));
	span.append(buttonOK);
	span.append(buttonCancel);
	div.append(span);
	var pop = new modalPopup("none");

	$('#maincontent').prepend(pop.create("confirm action", div).addClass('info'));

	pop.show(null, null, posObject.position(), this.offset[0], this.offset[1]);

}

function browseFolderControl()
{

	hideIndex = 0;

	this.creatFolderDivLink = function (container, name, cid, url, browseFunction, fromFS)
	{
		var a = $('<a>').text(name);
		a.attr({ href: "#", 'title': "view folder" });

		a.click(function (e)
				{
			container.find('li').removeClass("selected");
			$(this).parent().addClass("selected");

			e.preventDefault();
			e.stopPropagation();
			var windowWidth = $(window).width() || window.innerWidth;
			var newCtr = $(document.getElementById(cid));

			if (newCtr.length == 0)
			{
				newCtr = $('<div>');
				newCtr.attr({ id: cid });
				newCtr.addClass('dir');

				if ((container.position().left + container.width() * 3) > windowWidth)
				{
					$('div.dir').eq(hideIndex).hide();
					hideIndex++;
				}
				container.after(newCtr);

				if (!fromFS)
				{
					browseFunction(url, newCtr);
				}
			}

			if (fromFS)
			{
				browseFunction(url, newCtr);
			}

			newCtr.nextAll().remove();

			if (hideIndex > 0)
			{

				if ((newCtr.position().left + newCtr.width() * 3) < windowWidth)
				{
					$('div.dir').eq(hideIndex - 1).show();
					hideIndex--;
				}

			}

				});

		return a;
	};

	this.creatFileDivLink = function (container, hid, name)
	{
		var a = $('<a>').text(name);
		a.attr({ href: "#", 'title': "view file" });
		a.click(function (e)
			{
				e.preventDefault();
				e.stopPropagation();
				displayFile(hid, null, name);
			});
		return a;
	};
}

function getQueryStringVals(url)
{
    var vars = ""
    if (url)
        vars = $.deparam.querystring(url);
    else
        vars = $.deparam.querystring();


	return vars;
}

function getQueryStringVal(name, url)
{
	var queryhash = getQueryStringVals(url);
	return queryhash[name];
}

String.prototype.trim = function (chars)
{
	return this.rtrim(chars).ltrim(chars);
};
String.prototype.ltrim = function (chars)
{
	chars = chars || "\\s";
	return this.replace(new RegExp("^[" + chars + "]+", "g"), "");
};

String.prototype.rtrim = function (chars)
{
	chars = chars || "\\s";
	return this.replace(new RegExp("[" + chars + "]+$", "g"), "");
};

String.prototype.compare = function (b)
{
	var a = this + '';
	return (a === b) ? 0 : (a > b) ? 1 : -1;
};


function getViolations(text)
{
	var v = false;

	try
	{
		eval('v = ' + text);

		if (v && ! v.constraint_violations)
		{
			v = false;
		}
	}
	catch(e)
	{
		v = false;
	}

	if (v)
	{
		var errs = [];
		var viols = v.constraint_violations;

		for ( var i = 0; i < viols.length; ++i )
		{
			var viol = viols[i];
			var desc = "violates a template constraint";
			var val = viol.field_value;

			if (viol.type == "unique")
				desc = "is not unique";
			else if (viol.type == "maxlength")
			{
				if (val.length > 1024)
				{
					val = val.substring(0, 1024) + "...";
				}
				desc = "is too long";
			}

			else if (viol.type == "minlength")
				desc = "is too short";
			else if (viol.type == "prohibited")
				desc = "is prohibited";
			else if (viol.type == "allowed")
				desc = "is not an allowed value";
			else if (viol.type == "min")
				desc = "is too low";
			else if (viol.type == "max")
				desc = "is too high";

			errs.push("field '" + viol.field_name + "' " + desc + ":\r\n" + val + "\r\n");
		}

		return(errs.join("\r\n"));
	}
	else
		return(text);
}


function reportError(message, xhr, textStatus, errorThrown)
{
	var msg = message || "";

	msg += "\n";

	if (errorThrown)
		msg += "\n" + errorThrown + "\n";

	if (xhr && xhr.responseText)
		msg += "\n" + getViolations(xhr.responseText);

	msg = msg.replace(/^[\r\n]+/, '');
	msg = msg.replace(/[\r\n]+$/, '');

	var lines = msg.split(/\r?\n/);

	var div = $('<div />');

	for ( var i = 0; i < lines.length; ++i )
	{
		if (i > 0)
			div.append($('<br />'));
		div.append(document.createTextNode(lines[i]));
	}

	var pop = new modalPopup();

	$('body').prepend(pop.create(textStatus, div).addClass('info'));
	pop.show();

}


// For IE
function parseJSONforIE(xml)
{

	var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
	xmlDoc.loadXML(xml);
	xml = xmlDoc;

	return xml;
}


function sleep(delay)
{

	var start = new Date().getTime();
	while (new Date().getTime() < start + delay);
}


function fitToContent(text, maxHeight)
{
	var pn = $(text.parent());
	var l = text.position().left;
	var pw = pn.innerWidth();

	text.css('width', (pw - l - 50) + "px");

	var adjustedHeight = text.height();
	var relative_error = parseInt(text.attr('relative_error'));
	if (!maxHeight || maxHeight > adjustedHeight)
	{
		adjustedHeight = Math.max(text[0].scrollHeight, adjustedHeight);
		if (maxHeight)
			adjustedHeight = Math.min(maxHeight, adjustedHeight);
		if ((adjustedHeight - relative_error) > text.height())
		{
			text.css('height', (adjustedHeight - relative_error) + "px");
			// chrome fix
			if (text[0].scrollHeight != adjustedHeight)
			{
				var relative = text[0].scrollHeight - adjustedHeight;
				if (relative_error != relative)
				{
					text.attr('relative_error', relative + relative_error);
				}
			}
		}
	}
}

function _findElement(idOrEl)
{
	if (typeof (idOrEl) == 'string')
	{
		if (idOrEl.substring(0, 1) == '#')
			return ($(idOrEl));
		else
			return ($('#' + idOrEl));
	}

	return ($(idOrEl));
}

function _spinUp(id, fillContainer)
{
	var el = _findElement(id);

	if (fillContainer)
	{
		var windowHeight = windowHeight();
		var elTop = el.offset().top;
		var elHeight = el.height();
		var fillHeight = windowHeight - elTop;

		if (elHeight < fillHeight)
			el.height(fillHeight);
	}

	$(el).addClass('spinning');
}

function _spinDown(id, fillContainer)
{
	var el = _findElement(id);

	if (fillContainer)
		el.height('auto');

	$(el).removeClass('spinning');
}

function _inputLoading(input)
{
    var el = $("<div>").addClass("smallProgress").addClass("inputLoading")
    el.css({ "float": "left"});

	input.after(el);
}

function _stopInputLoading(input)
{
	$(input).siblings(".inputLoading").first().remove();
}

function autoResizeText(ta, maxHeight)
{
	var resize = function ()
	{
		fitToContent($(this), maxHeight);
	};
	$(ta).attr('relative_error', 0);
	$(ta).each(resize);
	$(ta).keyup(resize).keydown(resize);
}


// AJAX common calls
function showDiff(url, container, showAsPopup)
{

	container.find('pre').remove();

	vCore.ajax({
		url: url,
		success: function (data, status, xhr)
		{

			$('#maincontent').removeClass('spinning');
			var pre = $('<pre>').addClass("prettyprint");

			var lines = data.split("\n");
			var diffText = "";

			$.each(lines, function (line, val)
				{
					if (line > 1)
					{

						var span = $('<span>');

						if (val.indexOf("@@") == 0)
						{
							span.addClass("diff_header");
						}
						else if (val.indexOf("+") == 0)
						{
							span.addClass("diff_added");
						}
						else if (val.indexOf("-") == 0)
						{
							span.addClass("diff_removed");
						}

						val = prettyPrintOne((val + "\n").replace(/[<]/g, "&lt;")
								.replace(/[>]/g, "&gt;"));
						span.html(val);
						pre.append(span);
					}
					else
						pre.append($('<span>').addClass("diff_header").html(val));
				});

			container.append(pre);

			if (showAsPopup)
			{
				var pop = new modalPopup("#backgroundPopup");

				$('body').prepend(pop.create((getQueryStringVal("file", url) || "file"), container));
				pop.show();
			}
		},
		error: function (d, s, x)
		{
			$('#maincontent').removeClass('spinning');
		}
	});

}

function filePopup(data, path, url)
{
	var pre = $('<pre>').addClass("prettyprint");
	var txt = data.replace(/[<]/g, "&lt;")
			.replace(/[>]/g, "&gt;");
	var content = prettyPrintOne(txt);

	try
	{
		pre.html(content);
	}
	catch (e)
	{
		pre.text(txt);
	}
	var div = $('<div>');
	var span = $('<span>').attr({ id: "newwindow" });
	var rv = $('<a>').text("raw view").attr({ href: url, target: "new" });
	span.append(rv);
	// div.append(span);
	div.append(pre);

	var pop = new modalPopup("#backgroundPopup");

	$('body').prepend(pop.create((path ? path : file), div, span));
	pop.show();


}


function checkFileTypeAndDisplay(onTestComplete, params)
{
    var file = params.file || lastURLSeg(params.path);
    params.file = file;
    var segs = file.split(".");
  
    var isText = false;
    vCore.ajax({
        url: sgCurrentRepoUrl + "/fileclasses.json",
        success: function (data, status, xhr)
        {
            if (data && data[":text"])
            {
                var patterns = data[":text"].patterns;
                var test1 = "*." + segs[segs.length - 1];
                var test2 = segs[segs.length - 1];
                isText = $.inArray(test1, patterns) >= 0 || $.inArray(test2, patterns) >= 0;
            }
        },
        complete: function ()
        {
            onTestComplete(params, isText);
        }
    });
    
    return isText;
}

function _displayLocalFile(params, isText)
{
    var url = sgLinkRoot + "/local/file?path=" + params.path;
    if (isText)
    {
        $('#maincontent').addClass('spinning');
        var pre = $('<pre>').addClass("prettyprint");

        vCore.ajax({
            url: url,

            success: function (data, status, xhr)
            {
                $('#maincontent').removeClass('spinning');
                if (xhr && xhr.responseText)
                {
                    filePopup(xhr.responseText, params.path, url);
                }
            },
            error: function (d, s, x)
            {
                $('#maincontent').removeClass('spinning');
            }
        });
    }
    else
    {
        window.open(url);
    }


}

function displayLocalFile(path)
{   
    checkFileTypeAndDisplay( _displayLocalFile, {"path": path});	
}

function _displayFile(params, isText)
{
	var url = sgCurrentRepoUrl + "/blobs/" + params.hid + "/" + params.file;
    if (isText)
	{
		$('#maincontent').addClass('spinning');
		var pre = $('<pre>').addClass("prettyprint");

		vCore.ajax({
		    url: url,

		    success: function (data, status, xhr)
		    {
		        $('#maincontent').removeClass('spinning');

		        if (xhr && xhr.responseText)
		        {
		            var txt = xhr.responseText.replace(/[<]/g, "&lt;").replace(/[>]/g, "&gt;");
		            var content = prettyPrintOne(txt);

		            try
		            {
		                pre.html(content);
		            }
		            catch (e)
		            {
		                pre.text(txt);
		            }
		            var div = $('<div>');
		            var span = $('<span>').attr({ id: "newwindow" });
		            var rv = $('<a>').text("raw view").attr({ href: url, target: "new" });
		            span.append(rv);
		            // div.append(span);
		            div.append(pre);

		            var pop = new modalPopup("#backgroundPopup");

		            $('body').prepend(pop.create((params.path ? params.path : params.file), div, span));
		            pop.show();
		        }
		    },
		    error: function (d, s, x)
		    {
		        $('#maincontent').removeClass('spinning');

		    }
		});


	}
	else
	{
	    window.open(url);
	}


}
function displayFile(hid, path, file)
{   
	 checkFileTypeAndDisplay(_displayFile, { "hid": hid, "path": path, "file": file});
}

function getRepos(afterSuccessFunction, bDetails)
{
	var url = sgLinkRoot + "/repos.json";
	if (bDetails)
	{
		url += "?details";
	}
	vCore.ajax({
			url: url,
			dataType: 'json',

			success: function (data, status, xhr)
			{
				if ((status != "success") || (!data))
				{
					return;
				}

				afterSuccessFunction(data);
			}

		});
}

function isTouchDevice()
{
	var types = ["iphone", "ipod", "ipad", "android", "s60", "windows ce", "blackberry", "palm"];
	var found = false;
	var ua = navigator.userAgent.toLowerCase();

	for (var i = 0; i < types.length; i++)
	{
		found |= (ua.search(types[i]) > -1);
	}

	return found;
}

/*
 * Compares two values
 */
function strcmp(a, b) {
	return ( ( a == b ) ? 0 : ( ( a > b ) ? 1 : -1 ) );
}

/*
 * Test to see if a javascript object is empty (i.e. {})
 */
function isEmpty(obj) {
	for(var prop in obj) {
		if(obj.hasOwnProperty(prop))
			return false;
	}

	return true;
}

/**
 * Creates a clickable image that hides/shows other elements
 *
 * @param openSrc URL of the image used to show the elements
 * @param closeSrc URL of the image used to hide the elements
 * @param elements array of elements to be hidden/shown
 * @param showAtStart if true, show the elements initially. otherwise hide.
 */
function _toggler(openSrc, closeSrc, elements, showAtStart)
{
	var i = $(document.createElement('img'));
	i.attr('src', openSrc);

	var doToggle = function(show)
	{
		if (show)
			i.attr('src', closeSrc);
		else
			i.attr('src', openSrc);

		$.each(elements,
			function (j, el) {
				if (show)
					$(el).show();
				else
					$(el).hide();
			}
		);

		vCore.resizeMainContent();
	};

	doToggle(showAtStart);

    i.css('cursor', 'pointer');

	i.data('open', showAtStart);

	i.click(
		function(e) {
			i.data('open', ! i.data('open'));
			doToggle(i.data('open'));
		}
	);

	return(i);
}

function _addInputPlaceholder(input, msg)
{
	if ('placeholder' in input[0])
	{
		input.attr('placeholder', msg);
		return;
	}

	input.data('placeholderval', msg);

	var hidePlaceholder = function(e)
	{
		if (input.val() == msg)
			input.val('');

		input.removeClass('placeholder');
	};

	var showPlaceholder = function(e)
	{
		if (input.val() == '')
			input.val(msg);

		if (input.val() == msg)
		{
			input.addClass('placeholder');
		}
	};

	showPlaceholder();

	input.click(hidePlaceholder);
	input.blur(showPlaceholder);
	input.focus(hidePlaceholder);
}

/*
Quick way to define prototypes that take up less space and result in
smaller file size; much less verbose than standard
foobar.prototype.someFunc = function() lists.

@param f Function object/constructor to add to.
@param addMe Object literal that contains the properties/methods to
  add to f's prototype.
*/
function extend(f, addMe) {
	for (var i in addMe) {
		f.prototype[i] = addMe[i];
	}
}

/*
 * Resize text to fill a container.
 * From http://stackoverflow.com/questions/687998/auto-size-dynamic-text-to-fill-fixed-size-container
 *
 * Set up your container like this:
 * <div class="jtextfill" style="width:150px;height:40px;">
 *   <span id="textInHere"></span>
 * </div>
 *
 * Then do something like this:
 * $(".jtextfill").textfill({ maxFontPixels: 36 });
 */
;(function($)
{
    $.fn.textfill = function(options)
	{
        var fontSize = options.maxFontPixels;
        var ourText = $('span:visible:first', this);
        var maxHeight = $(this).height();
        var maxWidth = $(this).width();
        var textHeight;
        var textWidth;
        do
		{
            ourText.css('font-size', fontSize);
            textHeight = ourText.height();
            textWidth = ourText.width();
            fontSize = fontSize - 1;
        }
		while (textHeight > maxHeight || textWidth > maxWidth && fontSize > 3);
		return this;
    };
})(jQuery);
