/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// VeracityContextMenu.cpp : Implementation of CVeracityContextMenu

#include "stdafx.h"
#include "VeracityContextMenu.h"
#include "ICommandBase.h"
#include "Commands.h"
#include <cstringt.h>
#include "dllmain.h"
#include "resource.h"
#include "GlobalHelpers.h"

extern "C"{
#define BEGIN_EXTERN_C
#define END_EXTERN_C
#include "sg.h"
#include "sg_workingdir_prototypes.h"

}

// CVeracityContextMenu
/////////////////////////////////////////////////////////////////////////////
// CVeracityContextMenu

STDMETHODIMP CVeracityContextMenu::Initialize (
    LPCITEMIDLIST pidlFolder, LPDATAOBJECT pDataObj, HKEY hProgID )
{
FORMATETC fmt = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
STGMEDIUM stg = { TYMED_HGLOBAL };
HDROP     hDrop;
	pDataObj;
	hProgID;
	pidlFolder;
    // Look for CF_HDROP data in the data object.
	TCHAR path [MAX_PATH];
	SG_error err;
	SG_CONTEXT_NULLFREE(m_pCtx);
	err = SG_context__alloc(&m_pCtx);
	SG_context * pCtx = m_pCtx;
	SG_string * pstr_selectedFile = NULL;
	SG_STRINGARRAY_NULLFREE(pCtx, mp_selectedPaths);
	SG_RBTREE__ALLOC(pCtx, &mp_contextMenuItems);
	SG_ERR_CHECK(  SG_STRINGARRAY__ALLOC(pCtx, &mp_selectedPaths, 5)  );
	if (pDataObj == NULL)
	{
		if (SHGetPathFromIDList(pidlFolder, path))
		{
			SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr_selectedFile)  );
			SG_ERR_CHECK(  SG_UTF8__INTERN_FROM_OS_BUFFER(pCtx, pstr_selectedFile, path)  );
			SG_ERR_CHECK(  SG_stringarray__add(pCtx, mp_selectedPaths, SG_string__sz(pstr_selectedFile))  );
			SG_ERR_CHECK(  SG_STRING_NULLFREE(pCtx, pstr_selectedFile)  );
			return S_OK;
		}
		else
			return E_INVALIDARG;
	}
    if ( FAILED( pDataObj->GetData ( &fmt, &stg ) ))
        {
        // Nope! Return an "invalid argument" error back to Explorer.
        return E_INVALIDARG;
        }
    // Get a pointer to the actual data.
    hDrop = (HDROP) GlobalLock ( stg.hGlobal );

	// Make sure it worked.
    if ( NULL == hDrop )
	{
        return E_INVALIDARG;
	}

    // Sanity check - make sure there is at least one filename.
UINT uNumFiles = DragQueryFile ( hDrop, 0xFFFFFFFF, NULL, 0 );
HRESULT hr = S_OK;

    if ( 0 == uNumFiles )
        {
        GlobalUnlock ( stg.hGlobal );
        ReleaseStgMedium ( &stg );
        return E_INVALIDARG;
        }
	UINT index = 0;
	for (index = 0; index < uNumFiles; index++)
	{
		// Get the name of the first file and store it in our member variable m_szFile.
		if ( 0 == DragQueryFile ( hDrop, index, path, MAX_PATH ) )
			hr = E_INVALIDARG;
		SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr_selectedFile)  );
		SG_ERR_CHECK(  SG_UTF8__INTERN_FROM_OS_BUFFER(pCtx, pstr_selectedFile, path)  );
		SG_ERR_CHECK(  SG_stringarray__add(pCtx, mp_selectedPaths, SG_string__sz(pstr_selectedFile))  );
		SG_ERR_CHECK(  SG_STRING_NULLFREE(pCtx, pstr_selectedFile)  );
	}

    GlobalUnlock ( stg.hGlobal );
    ReleaseStgMedium ( &stg );
    return hr;
fail:
	SG_ERR_CHECK(  SG_STRING_NULLFREE(pCtx, pstr_selectedFile)  );
	SG_STRINGARRAY_NULLFREE(pCtx, mp_selectedPaths);
	SG_RBTREE_NULLFREE(pCtx, mp_contextMenuItems);
	SG_CONTEXT_NULLFREE(pCtx);
	return E_UNEXPECTED;
}
STDMETHODIMP CVeracityContextMenu::QueryContextMenu (
    HMENU hmenu, UINT uMenuIndex, UINT uidFirstCmd,
    UINT uidLastCmd, UINT uFlags )
{
	
//	int number_of_commands =0;
	SG_context* pCtx = m_pCtx;
	SG_pathname * pPathCwd = NULL;
	SG_bool bInWF = SG_TRUE;
	uidLastCmd;
    // If the flags include CMF_DEFAULTONLY then we shouldn't do anything.
    if ( uFlags & CMF_DEFAULTONLY )
        return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, 0 );
	HMENU hSubmenu = CreatePopupMenu();
	MENUITEMINFO miiSubMenu = { sizeof(MENUITEMINFO) };
    UINT uID = uidFirstCmd;
//	MENUINFO menuInfoParent = {};
//	MENUINFO menuInfoNew = {};
	//SG_ERR_CHECK_CURRENT;
	//SG_lib__global_initialize(pCtx);
	
#define COMMAND_COUNT 22
	ICommandBase * commandsArray[COMMAND_COUNT];
	commandsArray[0] = new CreateWFCommand();
	commandsArray[1] = new SyncCommand();
	commandsArray[2] = new CommitCommand();
	commandsArray[3] = new SeparatorCommand();
	commandsArray[4] = new StatusCommand();
	commandsArray[5] = new DiffCommand();
	commandsArray[6] = new RevertCommand();
	commandsArray[7] = new SeparatorCommand();
	commandsArray[8] = new AddCommand();
	commandsArray[9] = new MoveCommand();
	commandsArray[10] = new DeleteCommand();
	commandsArray[11] = new RenameCommand();
	commandsArray[12] = new SeparatorCommand();
	commandsArray[13] = new HistoryCommand();
	commandsArray[14] = new MergeCommand();
	commandsArray[15] = new ResolveCommand();
	commandsArray[16] = new UpdateCommand();
	commandsArray[17] = new SeparatorCommand();
	commandsArray[18] = new DagCommand();
	commandsArray[19] = new SettingsCommand();
	commandsArray[20] = new ManageReposCommand();
	commandsArray[21] = new HelpCommand();

	bool haveInspectedPaths = false;
	bool allFiles = false;
	bool allFolders = false;
	bool mixed = false;
	SG_uint32 selectionCount = 0;
	const char * pszCurrentPath = NULL;
	SG_pathname * pCurrentPath = NULL;
	SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, mp_selectedPaths, 0, &pszCurrentPath)  );
	SG_ERR_CHECK(  SG_stringarray__count(pCtx, mp_selectedPaths, &selectionCount)  );
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPathCwd, pszCurrentPath)  );
	
	SG_string * pstrRepoDescriptorName = NULL;
	SG_workingdir__find_mapping(pCtx, pPathCwd, NULL, &pstrRepoDescriptorName, NULL);
	
	if (SG_context__err_equals(pCtx, SG_ERR_NOT_A_WORKING_COPY) || SG_context__err_equals(pCtx, SG_ERR_NOTAREPOSITORY))
	{
		bInWF = SG_FALSE;
		SG_context__err_reset(pCtx);
	}
	else
		SG_ERR_CHECK_CURRENT;
	
	int commandIndex = 0;
	SG_uint32 subMenuIndex = 0;
	//InsertMenu(hmenu, uMenuIndex++, MF_SEPARATOR|MF_BYPOSITION, 0, NULL);
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pCurrentPath)  );
	bool bLastWasSeparator = true;
	bool bWFHasOneHead = false;
	SG_int_to_string_buffer buf;
	HINSTANCE hInst= GetModuleHandle(_T("VeracityTortoise.dll") );

	if (bInWF == SG_TRUE)
	{
		int headCount = 1;
		SG_ERR_CHECK(  headCount = GlobalHelpers::GetHeadCount(pCtx, pszCurrentPath)  );
		bWFHasOneHead = headCount == 1;
	}
	for (commandIndex =0; commandIndex < COMMAND_COUNT; commandIndex++)
	{
		ICommandBase * CurrentCommand = commandsArray[commandIndex];
		if (CurrentCommand->IsSeparator())
		{
			if (!bLastWasSeparator)
				InsertMenu(hSubmenu, subMenuIndex++, MF_SEPARATOR|MF_BYPOSITION, 0, NULL);
			bLastWasSeparator = true;
			continue;
		}

		if (!CurrentCommand->EnableWhenInWF(bInWF == SG_TRUE))
			continue;
		if (!bWFHasOneHead && CurrentCommand->DisableWhenThereIsOneHead())
			continue;
		if (!CurrentCommand->EnableWhenSelectionCountIs(selectionCount)  )
			continue;
		SG_uint32 selectionIndex = 0;
		if (haveInspectedPaths == false)
		{
			//Loop over the strings to identify if it's all files, all folders or a mix.
			for (selectionIndex = 0; selectionIndex < selectionCount; selectionIndex++)
			{
				SG_fsobj_stat statResults;
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, mp_selectedPaths, selectionIndex, &pszCurrentPath)  );
				SG_ERR_CHECK(  SG_pathname__set__from_sz(pCtx, pCurrentPath, pszCurrentPath)  );
				SG_ERR_CHECK(  SG_fsobj__stat__pathname(pCtx, pCurrentPath, &statResults)  );
				if (statResults.type == SG_FSOBJ_TYPE__DIRECTORY)
				{
					if (allFiles)
					{
						//Once we determine that we're mixed, we can quit.
						allFiles = false;
						mixed = true;
						SG_PATHNAME_NULLFREE(pCtx,  pCurrentPath);
						break;
					}
					else if (!allFolders)
						allFolders = true;
				}
				else if(statResults.type == SG_FSOBJ_TYPE__REGULAR)
				{
					if (allFolders)
					{
						//Once we determine that we're mixed, we can quit.
						allFolders = false;
						mixed = true;
						SG_PATHNAME_NULLFREE(pCtx,  pCurrentPath);
						break;
					}
					else if (!allFiles)
						allFiles = true;
				}
				else
				{
					mixed = true;
					SG_PATHNAME_NULLFREE(pCtx,  pCurrentPath);
					break;
				}
			}
			haveInspectedPaths = true;
		}
		if (mixed && !CurrentCommand->EnableWhenSelectionIsFilesAndFolders()  )
			continue;
		if (allFiles && !CurrentCommand->EnableWhenSelectionIsAllFiles()  )
			continue;
		if (allFolders && !CurrentCommand->EnableWhenSelectionIsAllFolders()  )
			continue;
		bool bSkipThisCommand = false;
		if (bInWF)
		{
			//Skip enable/disable by status.
		/*	for (selectionIndex = 0; selectionIndex < selectionCount; selectionIndex++)
			{
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, mp_selectedPaths, selectionIndex, &pszCurrentPath)  );
				SG_diffstatus_flags status = CurrentCommand->GetStatus(pCtx, pszCurrentPath);
				if (!CurrentCommand->EnableWhenStatusIs(status))
				{
					bSkipThisCommand = true;
					break;
				}
			}*/
			if (!bSkipThisCommand)
			{
				for (selectionIndex = 0; selectionIndex < selectionCount; selectionIndex++)
				{
					SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, mp_selectedPaths, selectionIndex, &pszCurrentPath)  );
					bool isConflict = GlobalHelpers::IsConflict(pCtx, pszCurrentPath);
					if (!isConflict && !CurrentCommand->EnableWhenThereIsNoConflict())
					{
						bSkipThisCommand = true;
						break;
					}
				}
			}
		}
		if (bSkipThisCommand)
			continue;
		//From diffmerge
	
		
		int iconID = CurrentCommand->GetIconIdentifier();
		if (iconID != 0)
		{
			//HBITMAP paHbm = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(iconID), IMAGE_BITMAP, 0, 0, LR_LOADTRANSPARENT); 
			HBITMAP paHbm = (HBITMAP)LoadBitmap(hInst, MAKEINTRESOURCE(iconID)); 
			//HDC hdc_dest = CreateCompatibleDC(NULL);
			//HDC hdc_src = CreateCompatibleDC(NULL);

			//HBITMAP paHbmNew = CreateCompatibleBitmap(hdc_dest, 15, 15);
			/*HBITMAP old_bmp =*/ //(HBITMAP)SelectObject(hdc_dest, paHbmNew);
			//(HBITMAP)SelectObject(hdc_src, paHbm);
			//COLORREF clrTopLeft = GetPixel(hdc_src,0,0);
			//SetBkColor(hdc_dest,clrTopLeft);
			//TransparentBlt(hdc_dest, 0, 0, 15, 15,hdc_src, 0,0,15,15,clrTopLeft);

//			HICON pa_icon = (HICON)LoadIcon(hInst, MAKEINTRESOURCE(iconID)); 
//			HBITMAP paHbm = GetBitm
			//BITMAP thebitmap;
			//GetObject(paHbm,sizeof(bm),&bm);

			//HBITMAP paHbm_transparent = CreateBitmap(bm.bmWidth,bm.bmHeight,1,1,NULL);	
			MENUITEMINFO mii; 
			memset(&mii,0,sizeof(mii));
			mii.cbSize = sizeof(MENUITEMINFO);
			mii.fMask = MIIM_ID | MIIM_FTYPE | MIIM_STRING | MIIM_STATE /*| MIIM_BITMAP */| MIIM_CHECKMARKS ;
			mii.fType = MFT_STRING;
			mii.wID = uID++;
			mii.dwTypeData = CurrentCommand->GetCommandString();
			mii.cch = (UINT)wcslen(mii.dwTypeData);
			mii.fState = MFS_ENABLED;
			mii.hbmpChecked = paHbm;
			mii.hbmpUnchecked = paHbm;
			//mii.hbmpItem = paHbm;
			InsertMenuItem( hSubmenu, subMenuIndex++, false, &mii);
//			InsertMenuItem( hmenu, uID, false, &mii);
			
			SG_ERR_CHECK(  SG_rbtree__add__with_assoc(pCtx, mp_contextMenuItems, SG_int64_to_sz(uID - 1, buf), CurrentCommand)  );
			SG_ERR_CHECK(  SG_rbtree__add__with_assoc(pCtx, mp_contextMenuItems, SG_int64_to_sz(uID - uidFirstCmd - 1, buf), CurrentCommand)  );
			//InsertMenu ( hSubmenu, subMenuIndex++, MF_BYPOSITION, uID++, CurrentCommand->GetCommandString());
		}
		else
		{
			InsertMenu ( hSubmenu, subMenuIndex++, MF_BYPOSITION, uID++, CurrentCommand->GetCommandString());
			SG_ERR_CHECK(  SG_rbtree__add__with_assoc(pCtx, mp_contextMenuItems, SG_int64_to_sz(uID - 1, buf), CurrentCommand)  );
			SG_ERR_CHECK(  SG_rbtree__add__with_assoc(pCtx, mp_contextMenuItems, SG_int64_to_sz(uID - uidFirstCmd - 1, buf), CurrentCommand)  );
		}
		bLastWasSeparator = false;
	}

	/*GetMenuInfo(hmenu, &menuInfoParent);
	GetMenuInfo(hSubmenu, &menuInfoNew);
	menuInfoNew.cbSize = sizeof(MENUINFO);
	menuInfoNew.fMask = MIM_STYLE | MIM_APPLYTOSUBMENUS;
	menuInfoNew.dwStyle = MNS_CHECKORBMP;
	SetMenuInfo(hSubmenu, &menuInfoNew);*/
	HBITMAP paHbmSubMenu = (HBITMAP)LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP_COMMIT)); 
	if ((uID - uidFirstCmd) > 0)
	{
		miiSubMenu.fMask = MIIM_SUBMENU | MIIM_STRING | MIIM_ID | MIIM_STATE | MIIM_CHECKMARKS;
		miiSubMenu.wID = uID++;
		miiSubMenu.hSubMenu = hSubmenu;
		miiSubMenu.dwTypeData = _T("Veracity");
		miiSubMenu.hbmpChecked = paHbmSubMenu;
		miiSubMenu.hbmpUnchecked = paHbmSubMenu;

		InsertMenuItem ( hmenu, uMenuIndex, TRUE, &miiSubMenu );
	}
	//Try to disable it.
	//EnableMenuItem ( hmenu, uidFirstCmd, MF_GRAYED );
	//SG_lib__global_cleanup(pCtx);
    //SG_CONTEXT_NULLFREE(pCtx);
	SG_PATHNAME_NULLFREE(pCtx,  pCurrentPath);
	return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, uID - uidFirstCmd);
fail:
	NotifyError(pCtx);
	//SG_lib__global_cleanup(pCtx);
	//SG_CONTEXT_NULLFREE(pCtx);
	SG_PATHNAME_NULLFREE(pCtx,  pCurrentPath);
	return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, uID - uidFirstCmd );
}

void CVeracityContextMenu::NotifyError(SG_context * pCtx)
{
	SG_UNUSED_PARAM(pCtx);

	
#if  DEBUG
	wchar_t * pBuf = NULL;
	SG_string * errorMessage = NULL;
	SG_context * pSubContext = NULL;
	SG_context__alloc(&pSubContext);
	SG_context__err_to_string(pCtx, SG_TRUE, &errorMessage);
	SG_utf8__extern_to_os_buffer__wchar(pSubContext, SG_string__sz(errorMessage), &pBuf, NULL);
	MessageBox(NULL,(const TCHAR *)pBuf,NULL,MB_OK|MB_ICONERROR);
	SG_CONTEXT_NULLFREE(pSubContext);
	free(pBuf);
#else
	SG_log__report_error__current_error(pCtx);
#endif
}

STDMETHODIMP CVeracityContextMenu::GetCommandString (
    UINT_PTR idCmd, UINT uFlags, UINT* pwReserved, LPSTR pszName, UINT cchMax )
{
USES_CONVERSION;
pwReserved;
    // If Explorer is asking for a help string, copy our string into the
    // supplied buffer.
    if ( uFlags & GCS_HELPTEXT )
    {
		SG_context * pCtx = m_pCtx;
		LPCTSTR szText = _T("help string");
		//if (idCmd == 0)
		//	command = new AddCommand();
		//else 
		ICommandBase * CurrentCommand = NULL;
		SG_bool bFound = SG_FALSE;
		SG_int_to_string_buffer buf;
		SG_ERR_CHECK(  SG_rbtree__find(m_pCtx, mp_contextMenuItems, SG_int64_to_sz(idCmd, buf), &bFound, (void**)&CurrentCommand)  );
		if (!bFound)
			return E_INVALIDARG;
		szText = CurrentCommand->GetHelpString();
		
        if ( uFlags & GCS_UNICODE )
            {
            // We need to cast pszName to a Unicode string, and then use the
            // Unicode string copy API.
            lstrcpynW ( (LPWSTR) pszName, T2CW(szText), cchMax );
            }
        else
            {
            // Use the ANSI string copy API to return the help string.
            lstrcpynA ( pszName, T2CA(szText), cchMax );
            }

        return S_OK;
        }
fail:
    return E_INVALIDARG;
}

STDMETHODIMP CVeracityContextMenu::InvokeCommand ( LPCMINVOKECOMMANDINFO pCmdInfo )
{
    // If lpVerb really points to a string, ignore this function call and bail out.
    if ( 0 != HIWORD( pCmdInfo->lpVerb ) )
        return E_INVALIDARG;
	SG_context * pCtx = m_pCtx;
	UINT_PTR idCmd = LOWORD( pCmdInfo->lpVerb);
	SG_int_to_string_buffer buf;

	ICommandBase * CurrentCommand = NULL;
	SG_bool bFound = SG_FALSE;
	SG_ERR_CHECK(  SG_rbtree__find(pCtx, mp_contextMenuItems, SG_int64_to_sz(idCmd, buf), &bFound, (void**)&CurrentCommand)  );
	if (!bFound)
		return E_INVALIDARG;
	SG_ERR_CHECK(  CurrentCommand->RunCommand(m_pCtx, mp_selectedPaths)  );
	return S_OK;
fail:
	NotifyError(pCtx);
	return S_OK;
}
