#include "sg_treediff_cache.h"

sg_treediff_cache::sg_treediff_cache()
{
	m_pCtx = NULL;
	m_pTreeDiff = NULL;
	m_lastStatusTime = 0;
}
sg_treediff_cache::sg_treediff_cache(SG_context * pCtx)
{
	m_pCtx = pCtx;
	m_pTreeDiff = NULL;
	m_lastStatusTime = 0;
}
sg_treediff_cache::~sg_treediff_cache()
{
	
}
void sg_treediff_cache::clearCache()
{
	if (m_pCtx && m_pTreeDiff)
		SG_TREEDIFF2_NULLFREE(m_pCtx, m_pTreeDiff);
}
void sg_treediff_cache::GetStatusForPath(SG_context * pCtx, const char * pszPath, SG_diffstatus_flags * pStatusFlags)
{
	if (m_pCtx == NULL)
		m_pCtx = pCtx;
	//Get the latest treediff.
	//Filter it down to a specific filespec.
	//Return the status of the first thing in the treediff.
	SG_file_spec * pFileSpec = NULL;
	SG_pathname * pPathName = NULL;
	SG_bool bOK = SG_FALSE;
	const SG_treediff2_ObjectData * pOD_opaque_k = NULL;
	char * psz_repoPath = NULL;
	SG_diffstatus_flags statusFlags = SG_DIFFSTATUS_FLAGS__ZERO;
	//SG_treediff2_iterator * pIter = NULL;

	SG_ERR_CHECK( SG_FILE_SPEC__ALLOC(pCtx, &pFileSpec)  );
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPathName, pszPath)  );
	
	SG_ERR_CHECK(  GetLatestTreediff(pCtx, pPathName, &psz_repoPath)  );

	if (psz_repoPath != NULL)
	{
		SG_ERR_CHECK(  SG_log__set_value__sz(pCtx, "repo path", psz_repoPath, SG_LOG__FLAG__NONE)  );
		SG_ERR_CHECK(  SG_treediff2__find_by_repo_path(pCtx, m_pTreeDiff, (const char *)psz_repoPath, &bOK, NULL, &pOD_opaque_k)  );
		//SG_ERR_CHECK(  SG_treediff2__file_spec_filter(pCtx, m_pTreeDiff,
			//										1, &pszPath, SG_FALSE,
				//									pFileSpec)  );
		//SG_ERR_CHECK(  SG_treediff2__iterator__first(pCtx, m_pTreeDiff,&bOK,NULL,&pOD_opaque_k,&pIter)  );
		SG_ERR_CHECK(  SG_log__set_value__bool(pCtx, "diff value", bOK, SG_LOG__FLAG__NONE)  );
		if (bOK == SG_TRUE)
			SG_ERR_CHECK(  SG_treediff2__ObjectData__get_dsFlags(pCtx, pOD_opaque_k, &statusFlags)  );
	}
	else
	{
		SG_ERR_CHECK(  SG_log__report_verbose(pCtx, "repo path was null")  );
		statusFlags = SG_DIFFSTATUS_FLAGS__FOUND;
		//statusFlags = statusFlags;
	}
	*pStatusFlags = statusFlags;
fail:
	SG_NULLFREE(pCtx, psz_repoPath);
	SG_FILE_SPEC_NULLFREE(pCtx, pFileSpec);
	SG_PATHNAME_NULLFREE(pCtx, pPathName);
}
void sg_treediff_cache::GetLatestTreediff(SG_context * pCtx, SG_pathname * pPathName, char **ppszRepoPath)
{
	if (m_pCtx == NULL)
		m_pCtx = pCtx;
	SG_pendingtree * pPendingTree = NULL;
	SG_int64 currentTime = 0;
	//SG_stringarray * psa_gids = NULL;
	//SG_bool bRoot = SG_FALSE;
	SG_string * pszRepoPathTmp = NULL;
	SG_pathname * pPathNameTop = NULL;
	SG_bool bRefreshed = SG_FALSE;

	SG_ERR_CHECK(  SG_time__get_milliseconds_since_1970_utc(pCtx, &currentTime)  );

	//SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathName, &pPendingTree)  );
	SG_ERR_CHECK(  SG_workingdir__find_mapping2(pCtx, pPathName, SG_TRUE, &pPathNameTop, NULL, NULL)  );
	//SG_ERR_CHECK(  SG_pendingtree__get_working_directory_top(pCtx, pPendingTree, &pPathNameTop)  );
	SG_ERR_CHECK(  SG_workingdir__wdpath_to_repopath(pCtx, pPathNameTop, pPathName, SG_FALSE, &pszRepoPathTmp)  );
	SG_ERR_CHECK(  SG_STRDUP(pCtx, SG_string__sz(pszRepoPathTmp), ppszRepoPath)  );

	if (currentTime > m_lastStatusTime + (5 * 1000))
	{
		if (m_pTreeDiff != NULL)
			SG_TREEDIFF2_NULLFREE(pCtx, m_pTreeDiff);
		bRefreshed = SG_TRUE;
		SG_ERR_CHECK(  SG_log__push_operation(pCtx, "refreshing cached tree diff", SG_LOG__FLAG__NONE)  );
		SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathName, &pPendingTree)  );
		//SG_ERR_CHECK(  SG_pendingtree__reload_pendingtree_from_vfile(pCtx, pPendingTree)  );
		SG_ERR_CHECK(  SG_pendingtree__diff_or_status(pCtx, pPendingTree, NULL, NULL, &m_pTreeDiff)  );
		//Refresh the time, so that we don't waste our "valid time" reloading the diff.
		SG_ERR_CHECK(  SG_time__get_milliseconds_since_1970_utc(pCtx, &currentTime)  );
		m_lastStatusTime = currentTime;
	}
	
	
fail:
	//SG_STRINGARRAY_NULLFREE(pCtx, psa_gids);
	SG_PATHNAME_NULLFREE(pCtx, pPathNameTop);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_STRING_NULLFREE(pCtx, pszRepoPathTmp);
	if (bRefreshed)
		SG_log__pop_operation(pCtx);
}
