#include "stdafx.h"
#include "sg_treediff_cache.h"

#include "windows.h"
extern "C"{
#define BEGIN_EXTERN_C
#define END_EXTERN_C
#include "sg.h"
#include "sg_workingdir_prototypes.h"

}

//Name given to the pipe
#define g_szPipeName2 _T("\\\\.\\Pipe\\VeracityCacheNamedPipe")
#define BUFFER_SIZE 1024 //1k
#define	WHATSMYNAME	"veracity_cache"

SG_int64 m_lastConflictTime;
SG_bool m_bHaveIssuesIn = SG_FALSE;
SG_varray * m_pvaIssuesIn = NULL;
SG_vhash * m_gidCache = NULL;
SG_vhash * m_foundFilesCache = NULL;

SG_context * GetAContext()
{
	SG_context* pCtx = NULL;
	SG_error err;
	err = SG_context__alloc(&pCtx);
	return pCtx;
}

void handle_message__conflict(SG_context * pCtx, SG_vhash * pvh_message, sg_treediff_cache * cache, SG_vhash ** ppvh_result)
{
	cache;
	SG_vhash * pvh_result = NULL;
	SG_pendingtree * pPendingTree = NULL;
	
	char * pszgid = NULL;
	bool returnVal = false;
	SG_uint32 nEntriesCount = 0, nEntriesIndex = 0;
	SG_int64 status64 = 0;
	SG_pendingtree_wd_issue_status status;
	SG_vhash * pCurrentVhash = NULL;
	const char * pszConflictGid = NULL;
	
	const char * pszPath = NULL;
	SG_pathname * pPathName = NULL;
	SG_int64 currentTime = 0;
	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_result)  );
	SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_message, "path", &pszPath)  );
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPathName, pszPath)  );
	SG_ERR_CHECK(  SG_time__get_milliseconds_since_1970_utc(pCtx, &currentTime)  );

	if (m_gidCache)
	{
		SG_bool bInCache = SG_FALSE;
		SG_bool bFoundFile = SG_FALSE;
		SG_ERR_CHECK(  SG_vhash__has(pCtx, m_gidCache, pszPath, &bInCache)  );
		SG_ERR_CHECK(  SG_vhash__has(pCtx, m_foundFilesCache, pszPath, &bFoundFile)  );
		if (bFoundFile)
		{
			SG_ERR_CHECK(  SG_log__report_verbose(pCtx, "we've encountered this found file before.")  );
			if (currentTime > m_lastConflictTime + (30 * 1000))
			{
				SG_ERR_CHECK(  SG_vhash__remove(pCtx, m_foundFilesCache, pszPath)  );
				bFoundFile = SG_FALSE;
			}
			else
			{
				SG_ERR_CHECK(  SG_vhash__add__bool(pCtx, pvh_result, "isConflict", SG_FALSE)  );
				goto fail;
			}
		}
		if (bInCache == SG_FALSE)
		{
			SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathName, &pPendingTree)  );
			SG_pendingtree__get_gid_from_local_path(pCtx, pPendingTree, pPathName, &pszgid);
			if (SG_context__err_equals(pCtx, SG_ERR_ITEM_NOT_UNDER_VERSION_CONTROL))
			{
				SG_context__err_reset(pCtx);
				SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, m_foundFilesCache, pszPath, "found")  );
				SG_ERR_CHECK(  SG_vhash__add__bool(pCtx, pvh_result, "isConflict", SG_FALSE)  );
				goto fail;
			}
			else
			{
				SG_ERR_CHECK_CURRENT;
			}
			SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, m_gidCache, pszPath, pszgid)  );
		}
		else
		{
			const char * pszTemp = NULL;
			SG_ERR_CHECK(  SG_log__report_verbose(pCtx, "found a cached GID.")  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, m_gidCache, pszPath, &pszTemp)  );
			SG_ERR_CHECK(  SG_STRDUP(pCtx, pszTemp, &pszgid)  );
		}
	}

	if (pszgid != NULL && currentTime > m_lastConflictTime + (5 * 1000))
	{
		SG_ERR_CHECK(  SG_log__report_verbose(pCtx, "reloading conflicts.")  );
		SG_bool bHaveIssuesIn = SG_FALSE;
		const SG_varray * pvaIssuesIn = NULL;
		if (pPendingTree == NULL)
			SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathName, &pPendingTree)  );
		SG_VARRAY_NULLFREE(pCtx, m_pvaIssuesIn);
		SG_ERR_CHECK(  SG_pendingtree__get_wd_issues__ref(pCtx, pPendingTree, &bHaveIssuesIn, &pvaIssuesIn)  );
		if (bHaveIssuesIn && m_pvaIssuesIn)
			SG_ERR_CHECK(  SG_VARRAY__ALLOC__COPY(pCtx, &m_pvaIssuesIn, pvaIssuesIn)  );
		m_bHaveIssuesIn = bHaveIssuesIn;
		m_lastConflictTime = currentTime;
	}
	if (pszgid != NULL && m_bHaveIssuesIn == SG_TRUE && m_pvaIssuesIn != NULL)
	{
		SG_ERR_CHECK(  SG_varray__count(pCtx, m_pvaIssuesIn, &nEntriesCount)  );
		if (nEntriesCount == 0)
		{
			SG_ERR_CHECK(  SG_vhash__add__bool(pCtx, pvh_result, "isConflict", SG_FALSE)  );
			goto fail;
		}

		for (nEntriesIndex = 0; nEntriesIndex < nEntriesCount; nEntriesIndex++)
		{
			//Compare the GID on the conflict to the gid of the local item.
			SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, m_pvaIssuesIn, nEntriesIndex, &pCurrentVhash)  );
			SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pCurrentVhash, "status", &status64)  );
			status = (SG_pendingtree_wd_issue_status)status64;
			if (status & SG_ISSUE_STATUS__MARKED_RESOLVED)
				continue;

			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pCurrentVhash, "gid", &pszConflictGid)  );
			if (strcmp(pszConflictGid, pszgid) == 0)
			{
				returnVal = true;
				break;
			}
		}

	}
	SG_ERR_CHECK(  SG_vhash__add__bool(pCtx, pvh_result, "isConflict", returnVal)  );
fail:
	SG_RETURN_AND_NULL(pvh_result, ppvh_result);
	SG_NULLFREE(pCtx, pszgid);
	SG_PATHNAME_NULLFREE(pCtx, pPathName);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void handle_message__headCount(SG_context * pCtx, SG_vhash * pvh_message, sg_treediff_cache * cache, SG_vhash ** ppvh_result)
{
	cache;
	SG_vhash * pvh_result = NULL;
	SG_pendingtree * pPendingTree = NULL;
	SG_file_spec * pFileSpec = NULL;
	SG_treediff2 * pTreeDiff = NULL;
	SG_pathname * pPathName = NULL;
	SG_repo * pRepo = NULL;
	SG_rbtree * prbLeaves = NULL;
	SG_uint32 count = 0;
	SG_diffstatus_flags statusFlags = SG_DIFFSTATUS_FLAGS__ZERO;

	const char * pszPath = NULL;
	
	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_result)  );
	SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_message, "path", &pszPath)  );

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPathName, pszPath)  );
	SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathName, &pPendingTree)  );
	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );
	SG_ERR_CHECK(  SG_repo__fetch_dag_leaves(pCtx,pRepo,SG_DAGNUM__VERSION_CONTROL,&prbLeaves)  );
	SG_ERR_CHECK(  SG_rbtree__count(pCtx, prbLeaves, &count)  );
	////
	SG_ERR_CHECK(  SG_vhash__add__int64(pCtx, pvh_result, "headCount", statusFlags)  );
	SG_RETURN_AND_NULL(pvh_result, ppvh_result);

	SG_RBTREE_NULLFREE(pCtx, prbLeaves);
    SG_PATHNAME_NULLFREE(pCtx, pPathName);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_FILE_SPEC_NULLFREE(pCtx, pFileSpec);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
	return;
fail:
	SG_RBTREE_NULLFREE(pCtx, prbLeaves);
	SG_VHASH_NULLFREE(pCtx, pvh_result);
	SG_PATHNAME_NULLFREE(pCtx, pPathName);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_FILE_SPEC_NULLFREE(pCtx, pFileSpec);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
}

void handle_message__status(SG_context * pCtx, SG_vhash * pvh_message, sg_treediff_cache * cache, SG_vhash ** ppvh_result)
{
	SG_vhash * pvh_result = NULL;
	SG_pendingtree * pPendingTree = NULL;
	SG_file_spec * pFileSpec = NULL;
	SG_treediff2 * pTreeDiff = NULL;
	SG_pathname * pPathName = NULL;
	SG_diffstatus_flags statusFlags = SG_DIFFSTATUS_FLAGS__ZERO;

	const char * pszPath = NULL;
	
	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pvh_result)  );
	SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_message, "path", &pszPath)  );
	SG_ERR_CHECK(  SG_log__push_operation(pCtx, "getting status", SG_LOG__FLAG__NONE)  );
	SG_ERR_CHECK(  SG_log__set_value__sz(pCtx, "path", pszPath, SG_LOG__FLAG__NONE)  );
	cache->GetStatusForPath(pCtx, pszPath, &statusFlags);
	SG_ERR_CHECK(  SG_log__set_value__int(pCtx, "raw status returned", statusFlags, SG_LOG__FLAG__NONE)  );
	if (statusFlags != 0 && statusFlags != 4)
		statusFlags = statusFlags;
	else
		SG_ERR_CHECK(  SG_log__report_verbose(pCtx, "ignoring this status")  );
	////
	SG_ERR_CHECK(  SG_vhash__add__int64(pCtx, pvh_result, "status", statusFlags)  );
	SG_RETURN_AND_NULL(pvh_result, ppvh_result);
fail:
	SG_log__pop_operation(pCtx);
	SG_VHASH_NULLFREE(pCtx, pvh_result);
	SG_PATHNAME_NULLFREE(pCtx, pPathName);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_FILE_SPEC_NULLFREE(pCtx, pFileSpec);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
}


void NotifyError(SG_context * pCtx)
{
#if  DEBUG
	wchar_t * pBuf = NULL;
	SG_string * errorMessage = NULL;
	SG_context * pSubContext = NULL;
	SG_context__alloc(&pSubContext);
	SG_context__err_to_string(pCtx, SG_TRUE, &errorMessage);
	printf("\n%s\n", SG_string__sz(errorMessage));
	SG_utf8__extern_to_os_buffer__wchar(pSubContext, SG_string__sz(errorMessage), &pBuf, NULL);
	//MessageBox(NULL,(const TCHAR *)pBuf,NULL,MB_OK|MB_ICONERROR);
	SG_CONTEXT_NULLFREE(pSubContext);
	free(pBuf);
#endif
	SG_log__report_error__current_error(pCtx);
}

int main(int argc, char* argv[])
{
	argc;
	argv;
	

	SG_context * pCtx = GetAContext();
	SG_lib__global_initialize(pCtx);
	SG_log__init_context(pCtx);
	SG_vhash * pvhMessage = NULL;
	SG_vhash * pvhResult = NULL;
	const char * commandString = NULL;
	char * previousCommand = NULL;
	char * previousResult = NULL;
	SG_int64 previousCommandTime = 0, currentTime = 0;
	SG_string * sgstrReturnMessage = NULL;
	const char * pResult = NULL;
	char * pCachedResult = NULL;
	sg_treediff_cache cache;
	 HANDLE hPipe;
     
	 //Log initialization:
	 SG_pathname*                          pLogPath           = NULL;
	char*                                 szLogPath          = NULL;
	char*                                 szLogLevel         = NULL;
	SG_log_console__data                  cLogStdData;
	SG_log_text__data                     cLogFileData;
	SG_log_text__writer__daily_path__data cLogFileWriterData;
	SG_uint32							  logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__ALL;

	SG_zero(cLogStdData);
	SG_zero(cLogFileData);
	SG_zero(cLogFileWriterData);
	m_lastConflictTime = 0;
	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &m_gidCache)  );
	SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &m_foundFilesCache)  );
	// find the appropriate log path
	SG_ERR_CHECK(  SG_localsettings__get__sz(pCtx, SG_LOCALSETTING__LOG_PATH, NULL, &szLogPath, NULL)  );
	if (szLogPath != NULL)
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pLogPath, szLogPath)  );
	}
	else
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__LOG_DIRECTORY(pCtx, &pLogPath)  );
	}

	// get the configured log level
	SG_ERR_CHECK(  SG_localsettings__get__sz(pCtx, SG_LOCALSETTING__LOG_LEVEL, NULL, &szLogLevel, NULL)  );

	// register the stdout console logger
	SG_ERR_CHECK(  SG_log_console__set_defaults(pCtx, &cLogStdData)  );
	SG_ERR_CHECK(  SG_log_console__register(pCtx, &cLogStdData, SG_LOG__FLAG__HANDLER_TYPE__ALL)  );

	// register the file logger
	SG_ERR_CHECK(  SG_log_text__set_defaults(pCtx, &cLogFileData)  );
	cLogFileData.fWriter             = SG_log_text__writer__daily_path;
	cLogFileData.pWriterData         = &cLogFileWriterData;
	cLogFileData.szRegisterMessage   = NULL;
	cLogFileData.szUnregisterMessage = NULL;
	if (szLogLevel != NULL)
	{
		if (SG_stricmp(szLogLevel, "quiet") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__QUIET;
			cLogFileData.bLogVerboseOperations = SG_FALSE;
			cLogFileData.bLogVerboseValues     = SG_FALSE;
		}
		else if (SG_stricmp(szLogLevel, "normal") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__NORMAL;
			cLogFileData.bLogVerboseOperations = SG_FALSE;
			cLogFileData.bLogVerboseValues     = SG_FALSE;
		}
		else if (SG_stricmp(szLogLevel, "verbose") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__ALL;
			cLogFileData.szRegisterMessage   = "---- " WHATSMYNAME " started logging ----";
			cLogFileData.szUnregisterMessage = "---- " WHATSMYNAME " stopped logging ----";
		}
	}
	SG_ERR_CHECK(  SG_log_text__writer__daily_path__set_defaults(pCtx, &cLogFileWriterData)  );
	cLogFileWriterData.bReopen          = SG_FALSE;
	cLogFileWriterData.pBasePath        = pLogPath;
	cLogFileWriterData.szFilenameFormat = "veracity_cache-%d-%02d-%02d.log";
	SG_ERR_CHECK(  SG_log_text__register(pCtx, &cLogFileData, logFileFlags)  );


     hPipe = CreateNamedPipe( 
          g_szPipeName2,             // pipe name 
          PIPE_ACCESS_DUPLEX,       // read/write access 
          PIPE_TYPE_MESSAGE |       // message type pipe 
          PIPE_READMODE_MESSAGE |   // message-read mode 
          PIPE_WAIT,                // blocking mode 
          PIPE_UNLIMITED_INSTANCES, // max. instances  
          BUFFER_SIZE,              // output buffer size 
          BUFFER_SIZE,              // input buffer size 
          25000, // client time-out 
          NULL);                    // default security attribute 

     
     if (INVALID_HANDLE_VALUE == hPipe) 
     {
		 SG_log__report_error(pCtx, "Error occurred while " 
                 "creating the pipe: %d", GetLastError()); 
          return 1;  //Error

     }
     else
     {
		 SG_log__report_verbose(pCtx, "CreateNamedPipe() was successful.");
     }
     
	 SG_log__report_info(pCtx, "veracity_cache statup successful.  Entering message loop"); 
	 bool bExit = false;
     while (!bExit)
	 {
     
   
		 SG_log__report_verbose(pCtx, "Waiting for client connection...");
		 //Wait for the client to connect
		 BOOL bClientConnected = ConnectNamedPipe(hPipe, NULL);
     
		 if (FALSE == bClientConnected)
		 {
			 continue;

		 }
		 else
		 {
			  SG_log__report_verbose(pCtx, "ConnectNamedPipe() was successful.");
		 }
     
		 char szBuffer[BUFFER_SIZE];
		 DWORD cbBytes;
     
		 //We are connected to the client.

		 //To communicate with the client 
		 //we will use ReadFile()/WriteFile() 
		 //on the pipe handle - hPipe
		 SG_ERR_CHECK(  SG_log__push_operation(pCtx, "got client message", SG_LOG__FLAG__NONE)  );
		 //Read client message
		 BOOL bResult = ReadFile( 
			  hPipe,                // handle to pipe 
			  szBuffer,             // buffer to receive data 
			  sizeof(szBuffer),     // size of buffer 
			  &cbBytes,             // number of bytes read 
			  NULL);                // not overlapped I/O 

     
		 if ( (!bResult) || (0 == cbBytes)) 
		 {
	  //        printf("\nError occurred while reading " 
		//             "from the client: %d", GetLastError()); 
			  continue;
		 }
		 else
		 {
			  SG_log__report_verbose(pCtx, "ReadFile() was successful.");
		 }
     
		 SG_ERR_CHECK(  SG_time__get_milliseconds_since_1970_utc(pCtx, &currentTime)  );

		 SG_log__report_verbose(pCtx, "Client sent the following message: %s", szBuffer);
		 pCachedResult = NULL;
		 
		 if (currentTime < previousCommandTime + (5 * 1000))
		 {
			 //Check to see if windows is asking the same question it _just_ asked.
			 if (szBuffer != NULL && previousCommand != NULL && strcmp(szBuffer, previousCommand) == 0)
			 {
				 SG_log__report_verbose(pCtx, "reusing previous result");
				 pCachedResult = previousResult; 
			 }
		 }
		 if (pCachedResult == NULL)
		 {
			 SG_ERR_CHECK(  SG_VHASH__ALLOC__FROM_JSON__SZ(pCtx, &pvhMessage, szBuffer)  );
			 SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvhMessage, "command", &commandString)  );

			 if (commandString != NULL)
			 {
				 if (strcmp(commandString, "status") == 0)
				 {
					 SG_ERR_CHECK(  handle_message__status(pCtx, pvhMessage, &cache, &pvhResult)  );
				 }
				 else if (strcmp(commandString, "conflict") == 0)
				 {
					 SG_ERR_CHECK(  handle_message__conflict(pCtx, pvhMessage, &cache, &pvhResult)  );
				 }
				 else if (strcmp(commandString, "getHeadCount") == 0)
				 {
					 SG_ERR_CHECK(  handle_message__headCount(pCtx, pvhMessage, &cache, &pvhResult)  );
				 }
				 else if (strcmp(commandString, "exit") == 0)
				 {
					 bExit = true;
					 goto finishloop;
				 }
			 }
		 

		 
			 SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &sgstrReturnMessage)  );
			 SG_ERR_CHECK(  SG_vhash__to_json(pCtx, pvhResult, sgstrReturnMessage)  );
			 SG_NULLFREE(pCtx, previousResult);
			 SG_ERR_CHECK(  SG_STRDUP(pCtx, SG_string__sz(sgstrReturnMessage), &previousResult )  );
			 pResult = SG_string__sz(sgstrReturnMessage);
			 SG_NULLFREE(  pCtx, previousCommand);
			 SG_ERR_CHECK(  SG_STRDUP(pCtx, szBuffer, &previousCommand)  );
		}
		 else
		 {
			 pResult = previousResult;
		 }
		 SG_log__report_verbose(pCtx, "Result is: %s", pResult);
		 //Reply to client
		 bResult = WriteFile( 
			  hPipe,                // handle to pipe 
			  pResult,             // buffer to write from 
			  static_cast<DWORD>(strlen(pResult)+1),   // number of bytes to write, include the NULL 
			  &cbBytes,             // number of bytes written 
			  NULL);                // not overlapped I/O 
		 
		 if ( (!bResult) || (strlen(pResult)+1 != cbBytes))
		 {
			 SG_log__report_error(pCtx, "Error occurred while writing" 
					 " to the client: %d", GetLastError()); 
			  CloseHandle(hPipe);
			  return 1;  //Error

		 }
		 else
		 {
			  if (FlushFileBuffers(hPipe))
			  {
				  DisconnectNamedPipe(hPipe);
			  }
		 }
finishloop:
		 SG_VHASH_NULLFREE(pCtx, pvhMessage);
		 SG_VHASH_NULLFREE(pCtx, pvhResult);
		 SG_STRING_NULLFREE(pCtx, sgstrReturnMessage);
		 SG_ERR_CHECK(  SG_log__pop_operation(pCtx)  );
		 previousCommandTime = currentTime;
	 }
	 SG_log__report_info(pCtx, "exiting successfully"); 
	 cache.clearCache();
     CloseHandle(hPipe);
	 SG_ERR_IGNORE(  SG_log_text__unregister(pCtx, &cLogFileData)  );
	 SG_ERR_IGNORE(  SG_log_console__unregister(pCtx, &cLogStdData)  );
	 SG_NULLFREE(pCtx, szLogLevel);
	 SG_NULLFREE(pCtx, szLogPath);
	 SG_PATHNAME_NULLFREE(pCtx, pLogPath); // must come after cLogFileData is unregistered, because it uses pLogPath
	 SG_lib__global_cleanup(pCtx);
	 SG_CONTEXT_NULLFREE(pCtx);
	 SG_NULLFREE(pCtx, previousResult);
	 SG_NULLFREE(pCtx, previousCommand);

	 return 0; //Success
fail:
	 NotifyError(pCtx);
	 cache.clearCache();
	 SG_VHASH_NULLFREE(pCtx, pvhMessage);
	 SG_VHASH_NULLFREE(pCtx, pvhResult);
	 SG_STRING_NULLFREE(pCtx, sgstrReturnMessage);
	 SG_ERR_IGNORE(  SG_log_text__unregister(pCtx, &cLogFileData)  );
	 SG_ERR_IGNORE(  SG_log_console__unregister(pCtx, &cLogStdData)  );
	 SG_NULLFREE(pCtx, szLogLevel);
	 SG_NULLFREE(pCtx, szLogPath);
	 SG_PATHNAME_NULLFREE(pCtx, pLogPath); // must come after cLogFileData is unregistered, because it uses pLogPath
	 SG_lib__global_cleanup(pCtx);
	 SG_CONTEXT_NULLFREE(pCtx);
	 return 1;
}



/*
	SG_TREEDIFF2_ITERATOR_NULLFREE(pCtx, pIter);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	return statusFlags;
fail:
	//NotifyError(pCtx);
	SG_TREEDIFF2_ITERATOR_NULLFREE(pCtx, pIter);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	return SG_DIFFSTATUS_FLAGS__ZERO;
	*/
