/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvAddCommand.h"

#include "vvContext.h"
#include "vvVerbs.h"


/*
**
** Globals
**
*/

/**
 * Global instance of vvAddCommand.
 */
static const vvAddCommand gcAddCommand;

/*
 * Command specification.
 * These need to be char* instead of wxString, because they're used by a static constructor.
 * And we can't be sure the wxStrings will be constructed before the static instance that uses them.
 */
static const char*        gszCommandName        = "add";
static const unsigned int guCommandFlags        = vvCommand::FLAG_NO_DIALOG;
static const char*        gszCommandDescription = "Add a repository object.";


/*
**
** Public Functions
**
*/

vvAddCommand::vvAddCommand()
	: vvCommand(gszCommandName, guCommandFlags, gszCommandDescription)
{
}

bool vvAddCommand::ParseArguments(
	const wxArrayString& cArguments
	)
{
	// get the directory name from the first argument
	if (cArguments.Count() < 1u)
	{
		wxLogError("No argument was passed in to Add.");
		return false;
	}

	// store the given paths
	for (wxArrayString::const_iterator it = cArguments.begin(); it != cArguments.end(); ++it)
	{
		this->mcPaths.Add(*it);
	}
	// success
	return true;
}

vvDialog* vvAddCommand::CreateDialog(
	wxWindow* wParent
	)
{
	wParent;
	return NULL;
}

bool vvAddCommand::Execute()
{
	vvVerbs*   pVerbs   = this->GetVerbs();
	vvContext* pContext = this->GetContext();

	wxASSERT(pVerbs != NULL);
	wxASSERT(pContext != NULL);
	if (this->mcPaths.Count() > 1)
		pContext->Log_PushOperation("Adding objects", vvContext::LOG_FLAG_NONE);
	else
		pContext->Log_PushOperation("Adding object", vvContext::LOG_FLAG_NONE);
	pContext->Log_SetSteps(2u);
	wxScopeGuard cPopOperation = wxMakeGuard(&vvContext::Log_PopOperation_Static, pContext);
	wxUnusedVar(cPopOperation);

	pVerbs->Add(*pContext, this->mcPaths);

	pContext->Log_FinishStep();
	return !pContext->Error_Check();
}
