/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvCommand.h"


/*
**
** Globals
**
*/

vvCommand* vvCommand::spFirst = NULL;


/*
**
** Public Functions
**
*/

vvCommand* vvCommand::GetFirst()
{
	return vvCommand::spFirst;
}

vvCommand* vvCommand::FindByName(
	const wxString& sName
	)
{
	vvCommand* pCurrent = vvCommand::GetFirst();
	while (pCurrent != NULL)
	{
		if (pCurrent->GetName().compare(sName) == 0)
		{
			return pCurrent;
		}
		pCurrent = pCurrent->GetNext();
	}
	return NULL;
}

vvCommand::vvCommand(
	const wxString& sName,
	unsigned int    uFlags,
	const wxString& sUsageDescription
	)
	: vvHasFlags(uFlags)
	, mpNext(vvCommand::spFirst) // we'll point to the current first instance
	, msName(sName)
	, msUsageDescription(sUsageDescription)
	, mpVerbs(NULL)
	, mpContext(NULL)
{
	// set ourselves as the new first instance in the global list
	vvCommand::spFirst = this;
}

const wxString& vvCommand::GetName() const
{
	return this->msName;
}

const wxString& vvCommand::GetUsageDescription() const
{
	return this->msUsageDescription;
}

vvCommand* vvCommand::GetNext() const
{
	return this->mpNext;
}

void vvCommand::SetVerbs(
	class vvVerbs* pVerbs
	)
{
	this->mpVerbs = pVerbs;
}

class vvVerbs* vvCommand::GetVerbs() const
{
	return this->mpVerbs;
}

void vvCommand::SetContext(
	class vvContext* pContext
	)
{
	this->mpContext = pContext;
}

class vvContext* vvCommand::GetContext() const
{
	return this->mpContext;
}

bool vvCommand::ParseArguments(
	const wxArrayString& WXUNUSED(cArguments)
	)
{
	return true;
}

bool vvCommand::Init()
{
	return true;
}

class vvDialog* vvCommand::CreateDialog(
	wxWindow* WXUNUSED(wParent)
	)
{
	return NULL;
}

bool vvCommand::Execute()
{
	return true;
}
