/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvInOutCommand.h"

#include "vvContext.h"
#include "vvVerbs.h"


/*
**
** Globals
**
*/

/*
 * Command specification.
 * These need to be char* instead of wxString, because they're used by a static constructor.
 * And we can't be sure the wxStrings will be constructed before the static instance that uses them.
 */
static const char*        gszCommandName        = "inout";
static const unsigned int guCommandFlags        = 0;
static const char*        gszCommandDescription = "This command is only used internally.";


/*
**
** Public Functions
**
*/

vvInOutCommand::vvInOutCommand()
	: vvCommand(gszCommandName, guCommandFlags, gszCommandDescription)
{
}

vvInOutCommand::vvInOutCommand(bool bIncoming, wxString sLocalRepo, wxString sOtherRepo)
	: vvCommand(gszCommandName, guCommandFlags, gszCommandDescription)
{
	mbIncoming = bIncoming;
	msLocalRepo = sLocalRepo;
	msOtherRepo = sOtherRepo;
}
bool vvInOutCommand::Execute()
{
	vvVerbs*   pVerbs   = this->GetVerbs();
	vvContext* pCtx = this->GetContext();

	wxASSERT(pVerbs != NULL);
	wxASSERT(pCtx != NULL);
	if (this->mbIncoming)
	{
		pVerbs->Incoming(*pCtx, this->msOtherRepo, this->msLocalRepo, mResults);
	}
	else
	{
		pVerbs->Outgoing(*pCtx, this->msOtherRepo, this->msLocalRepo, mResults);
	}
	return !pCtx->Error_Check();
}

vvVerbs::HistoryData vvInOutCommand::GetResults()
{
	return mResults;
}
