/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvRenameCommand.h"

#include "vvContext.h"
#include "vvRenameDialog.h"
#include "vvVerbs.h"


/*
**
** Globals
**
*/

/**
 * Global instance of vvRenameCommand.
 */
static const vvRenameCommand gcRenameCommand;

/*
 * Command specification.
 * These need to be char* instead of wxString, because they're used by a static constructor.
 * And we can't be sure the wxStrings will be constructed before the static instance that uses them.
 */
static const char*        gszCommandName        = "rename";
static const unsigned int guCommandFlags        = 0;
static const char*        gszCommandDescription = "Rename a repository object.";


/*
**
** Public Functions
**
*/

vvRenameCommand::vvRenameCommand()
	: vvCommand(gszCommandName, guCommandFlags, gszCommandDescription)
	, mwDialog(NULL)
{
}

bool vvRenameCommand::ParseArguments(
	const wxArrayString& cArguments
	)
{
	// get the directory name from the first argument
	if (cArguments.Count() < 1u)
	{
		wxLogError("No argument was passed in to rename.");
		return false;
	}
	wxString sItem = cArguments[0];

	// make sure this is actually an existing directory
	if ( ! (wxFileExists(sItem) || wxDirExists(sItem)) )
	{
		wxLogError("Specified item doesn't exist: %s", sItem);
		return false;
	}

	// success
	this->msDirectory = sItem;
	return true;
}

vvDialog* vvRenameCommand::CreateDialog(
	wxWindow* wParent
	)
{
	vvVerbs*   pVerbs   = this->GetVerbs();
	vvContext* pContext = this->GetContext();

	wxASSERT(pVerbs != NULL);
	wxASSERT(pContext != NULL);
	wParent;
	this->mwDialog = new vvRenameDialog();
	if (!this->mwDialog->Create(wParent, this->msDirectory, *pVerbs, *pContext))
	{
		delete this->mwDialog;
		return NULL;
	}
	return this->mwDialog;
}

bool vvRenameCommand::Execute()
{
	vvVerbs*   pVerbs   = this->GetVerbs();
	vvContext* pContext = this->GetContext();

	wxASSERT(pVerbs != NULL);
	wxASSERT(pContext != NULL);

	pContext->Log_PushOperation("Renaming object", vvContext::LOG_FLAG_NONE);
	pContext->Log_SetSteps(2u);
	wxScopeGuard cPopOperation = wxMakeGuard(&vvContext::Log_PopOperation_Static, pContext);
	wxUnusedVar(cPopOperation);

	if (((const char *)this->mwDialog->GetNewName().c_str()) == NULL || ((const char *)this->mwDialog->GetNewName().c_str())[0] == NULL)
	{
		pContext->Log_ReportError("The new name is empty.");
		pContext->Log_FinishStep();
		return false;
	}
	else
		pVerbs->Rename(*pContext, this->msDirectory.c_str(), this->mwDialog->GetNewName().c_str());

	pContext->Log_FinishStep();
	return !pContext->Error_Check();
}
