/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#ifndef VV_STATUS_DIALOG_H
#define VV_STATUS_DIALOG_H

#include "precompiled.h"
#include "vvDialog.h"


/**
 * A dialog box used to display the status of a working folder.
 */
class vvStatusDialog : public vvDialog
{
// construction
public:
	/**
	 * Default constructor.
	 */
	vvStatusDialog();

	/**
	 * Destructor.
	 */
	~vvStatusDialog();

// functionality
public:
	/**
	 * Creates the actual dialog window.
	 */
	virtual bool Create(
		wxWindow*        pParent,    //< [in] The new dialog's parent window, or NULL if it won't have one.
		const wxString&  sDirectory, //< [in] The directory of the working folder to display the status of.
		class vvVerbs&   cVerbs,     //< [in] The vvVerbs implementation to retrieve status data from.
		class vvContext& cContext    //< [in] Context to use with pVerbs.
		);

// implementation
public:
	/**
	 * Transfers our internal data to our widgets.
	 */
	virtual bool TransferDataToWindow();

// internal functionality
protected:
	/**
	 * Update the visible/enabled state of all controls, based on the current values.
	 */
	void UpdateDisplay();

// event handlers
protected:
	void OnNotebookPageChanged(wxBookCtrlEvent& cEvent);

// private types
private:
	typedef std::map<unsigned int, wxCheckBox*> stlValueCheckboxMap;

// private data
private:
	// internal data
	wxString msDirectory; //< The directory of the working folder to display the status of.

	// widgets
	class vvStatusControl* mwStatus;                    //< The control to display the actual status data in.
	stlValueCheckboxMap    mcStatusFilters;             //< Checkboxes controlling which statuses to display, indexed by vvStatusControl::Status.
	stlValueCheckboxMap    mcTypeFilters;               //< Checkboxes controlling which types to display, indexed by vvStatusControl::ItemType.
	stlValueCheckboxMap    mcColumnFilters;             //< Checkboxes controlling which columns to display, indexed by vvStatusControl::Column.
	wxCheckBox*            mwOptions_Misc_CollapseMode; //< Whether or not we're using MODE_COLLAPSE (vs MODE_FOLDER).

// macro declarations
private:
	DECLARE_DYNAMIC_CLASS(vvStatusDialog);
	DECLARE_EVENT_TABLE();
};


#endif
