/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvUpdateDialog.h"

#include "vvVerbs.h"
#include "vvValidatorEmptyConstraint.h"
#include "vvValidatorRepoExistsConstraint.h"
#include "vvHistoryTabbedControl.h"
#include "vvDialogHeaderControl.h"


/*
**
** Globals
**
*/

/*
**
** Public Functions
**
*/

bool vvUpdateDialog::Create(
	wxWindow*            pParent,
	const wxString&      sInitialValue,
	vvVerbs&             cVerbs,
	vvContext&           cContext
	)
{
	// create the dialog window
	if (!vvDialog::Create(pParent, &cVerbs, &cContext, this->GetClassInfo()->GetClassName()))
	{
		return false;
	}

	wxXmlResource::Get()->AttachUnknownControl("wHeader", new vvDialogHeaderControl(this, wxID_ANY, sInitialValue, cVerbs, cContext));

	wxString psRepoDescriptorName;
	wxArrayString baselineHIDs;
	cVerbs.FindWorkingFolder(cContext, sInitialValue, NULL, &psRepoDescriptorName, &baselineHIDs);
	m_sRepoName = wxString(psRepoDescriptorName);
	m_revisionsControl = new vvHistoryTabbedControl(cVerbs, cContext, m_sRepoName, this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, baselineHIDs[0]);
	wxXmlResource::Get()->AttachUnknownControl("wRevisions", m_revisionsControl, this);
	
	m_ShowAll = XRCCTRL(*this, "wCheckBox_ShowAll",                 wxCheckBox);
	m_BranchesCombo = XRCCTRL(*this, "wBranchCombo",                 wxComboBox);

	m_filterOptions.nResultsAtOneTime = 25;
	vvHistoryColumnPreferences columnPrefs;
	cVerbs.GetColumnPreferences(cContext, columnPrefs);
	m_revisionsControl->Enable(false);
	this->Fit();
	// success
	return true;
}

void vvUpdateDialog::FillBranchesCombo()
{
	wxArrayString cBranches;
	this->GetVerbs()->GetBranches(*this->GetContext(), m_sRepoName, this->m_ShowAll->GetValue(), cBranches);
	m_BranchesCombo->Clear();
	m_BranchesCombo->Append(cBranches);
	
}
bool vvUpdateDialog::TransferDataToWindow()
{ 
	m_revisionsControl->ApplyDefaultOptions(m_filterOptions);
	m_revisionsControl->LoadData(m_filterOptions);
	FillBranchesCombo();
	
	return true;
}

bool vvUpdateDialog::TransferDataFromWindow()
{
	m_revisionsControl->SaveSettings();
	if (XRCCTRL(*this, "wSpecificRevision", wxRadioButton)->GetValue())
	{
		wxArrayString mcChangesets = this->m_revisionsControl->GetSelection();
		if (mcChangesets.size() == 0)
			return false;
		m_sSelection = mcChangesets[0];
		m_CheckoutType = vvVerbs::CHECKOUT_CHANGESET;
	}
	else
	{
		m_sSelection = this->m_BranchesCombo->GetValue();
		m_CheckoutType = vvVerbs::CHECKOUT_BRANCH;
	}
	
	// success
	return true;
}

wxString vvUpdateDialog::GetTargetBranchOrRev()
{
		return m_sSelection;
}

vvVerbs::CheckoutType vvUpdateDialog::GetCheckoutType()
{
	return m_CheckoutType;
}

void vvUpdateDialog::OnControlValueChange(
	wxCommandEvent& cEvent
	)
{
	cEvent;
	if (XRCCTRL(*this, "wSpecificRevision", wxRadioButton)->GetValue())
	{
		m_revisionsControl->Enable(true);
	}
	else
		m_revisionsControl->Enable(false);
}

void vvUpdateDialog::OnCheckBoxChanged(
	wxCommandEvent& cEvent)
{
	cEvent;
	FillBranchesCombo();
}

IMPLEMENT_DYNAMIC_CLASS(vvUpdateDialog, vvDialog);
BEGIN_EVENT_TABLE(vvUpdateDialog, vvDialog)
	EVT_RADIOBUTTON(XRCID("wBranch"),      vvUpdateDialog::OnControlValueChange)
	EVT_RADIOBUTTON(XRCID("wSpecificRevision"),    vvUpdateDialog::OnControlValueChange)
	EVT_CHECKBOX(XRCID("wCheckBox_ShowAll"), vvUpdateDialog::OnCheckBoxChanged)
END_EVENT_TABLE()
