/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvVerbs.h"

#include "vvContext.h"


/*
**
** Globals
**
*/

const char* vvVerbs::sszRepoRoot = "@";


/*
**
** Public Functions
**
*/

vvVerbs::~vvVerbs()
{
}

bool vvVerbs::FindWorkingFolder(
	class vvContext&     cContext,
	const wxArrayString& cPaths,
	wxString*            pFolder,
	wxString*            pRepo,
	wxArrayString*        pBaselineHIDs
	)
{
	wxString sFolder = wxEmptyString;
	wxString sRepo   = wxEmptyString;

	// run through each specified path
	for (wxArrayString::const_iterator it = cPaths.begin(); it != cPaths.end(); ++it)
	{
		// find a working folder from this path
		wxString sCurrentFolder;
		wxString sCurrentRepo;
		if (!this->FindWorkingFolder(cContext, *it, &sCurrentFolder, &sCurrentRepo, pBaselineHIDs))
		{
			return false;
		}

		// if we already found a working folder, make sure this one matches
		if (!sFolder.IsEmpty() && sFolder != sCurrentFolder)
		{
			return false;
		}

		// store this working folder and repo
		if (sFolder.IsEmpty())
		{
			sFolder = sCurrentFolder;
			sRepo   = sCurrentRepo;
		}
	}

	// if we got this far, we found exactly one working folder
	if (pFolder != NULL)
	{
		*pFolder = sFolder;
	}
	if (pRepo != NULL)
	{
		*pRepo = sRepo;
	}
	return true;
}

bool vvVerbs::GetRepoPaths(
	class vvContext&     cContext,
	const wxArrayString& cAbsolutePaths,
	const wxString&      sWorkingFolder,
	wxArrayString&       cRepoPaths
	)
{
	for (wxArrayString::const_iterator it = cAbsolutePaths.begin(); it != cAbsolutePaths.end(); ++it)
	{
		wxString sRepoPath;
		if (!this->GetRepoPath(cContext, *it, sWorkingFolder, sRepoPath))
		{
			return false;
		}
		cRepoPaths.Add(sRepoPath);
	}
	return true;
}

bool vvVerbs::GetRelativePath(
	class vvContext& cContext,
	const wxString&  sRepoPath,
	wxString&        sRelativePath
	)
{
	wxString sRepoRoot   = vvVerbs::sszRepoRoot;
	wxString sSeparators = wxFileName::GetPathSeparators();
	for (wxString::const_iterator it = sSeparators.begin(); it != sSeparators.end(); ++it)
	{
		wxString sPrefix = sRepoRoot + *it;
		if (sRepoPath.StartsWith(sPrefix, &sRelativePath))
		{
			return true;
		}
	}

	cContext.Log_ReportError("Given path isn't a repository path: %s", sRepoPath);
	return false;
}

bool vvVerbs::GetRelativePaths(
	class vvContext&     cContext,
	const wxArrayString& cRepoPaths,
	wxArrayString&       cRelativePaths
	)
{
	for (wxArrayString::const_iterator it = cRepoPaths.begin(); it != cRepoPaths.end(); ++it)
	{
		wxString sRelativePath;
		if (!this->GetRelativePath(cContext, *it, sRelativePath))
		{
			return false;
		}
		cRelativePaths.Add(sRelativePath);
	}
	return true;
}

bool vvVerbs::GetDiskPaths(
	class vvContext&     cContext,
	const wxString&		 cDiskRootFolder,
	const wxArrayString& cRepoPaths,
	wxArrayString&       cRelativePaths
	)
{
	wxString separator = "";
	if (! cDiskRootFolder.EndsWith("\\"))
		separator = "\\";
	for (wxArrayString::const_iterator it = cRepoPaths.begin(); it != cRepoPaths.end(); ++it)
	{
		wxString sRelativePath;
		if (!this->GetRelativePath(cContext, *it, sRelativePath))
		{
			return false;
		}
		cRelativePaths.Add(cDiskRootFolder + separator + sRelativePath);
	}
	return true;
}