/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvVerbsDebug.h"

#include "vvContext.h"
#include "vvCppUtilities.h"
#include "vvSgHelpers.h"


/*
**
** Macros
**
*/


/**
 * Gets the value of a string, or a default value if the string is empty.
 */
#define DEFAULT_STRING(sString, szDefault) (sString.IsEmpty() ? wxString(szDefault) : sString)


/*
**
** Globals
**
*/

/**
 * Test list of local repos.
 */
static const char* gaLocalRepos[] =
{
	"My First Repo",
	"Hello World",
	"Veracity",
	"Sandbox",
};

/**
 * Test list of remote repos.
 */
static const char* gaRemoteRepos[] =
{
	"http://www.sourcegear.com/veracity/repo",
	"http://www.whatever.com/repo",
	"http://kibble.ad.sourcegear.com/repos/veracity",
};

/**
 * Test list of branches.
 */
static const char* gaBranches[] =
{
	"master",
	"test",
	"release",
};

/**
 * Test list of users.
 */
static vvVerbs::UserData gaUsers[] =
{
	{ "g1234567890123456789012345678901234567890123456789012345678900000", "me@sourcegear.com",    "M" },
	{ "g1234567890123456789012345678901234567890123456789012345678900000", "debug@sourcegear.com", "D" },
	{ "g1234567890123456789012345678901234567890123456789012345678900000", "foo@sourcegear.com",   "F" },
	{ "g1234567890123456789012345678901234567890123456789012345678900000", "test@sourcegear.com",  "T" },
};

/**
 * Test current user.
 */
static const char* gszCurrentUser = "foo@sourcegear.com";

/**
 * Test list of stamps.
 */
static vvVerbs::StampData gaStamps[] =
{
	{ "MyStamp",       5 },
	{ "YourStamp",    10 },
	{ "FooStamp",  12345 },
	{ "TestStamp",     1 },
};

/**
 * Test list of tags.
 */
static vvVerbs::TagData gaTags[] =
{
	{ "MyTag",       "0194" },
	{ "YourTag",    "c164" },
	{ "FooTag",  "a123" },
	{ "TestTag",     "d432" },
};

/**
 * Test status, invented data.
 *//*
static vvVerbs::DiffEntry gaStatus[] =
{
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900000", "@/file/a/1.txt",          vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900001", "@/file/a/2.txt",          vvVerbs::DiffEntry::CHANGE_DELETED,                                        false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900002", "@/file/a/3.txt",          vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900003", "@/file/a/b/1.txt",        vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900004", "@/file/a/b/2.txt",        vvVerbs::DiffEntry::CHANGE_FOUND,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900005", "@/file/a/b/3.txt",        vvVerbs::DiffEntry::CHANGE_RENAMED,                                        false, "", "", "",              "4.txt", "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900006", "@/file/a/c/1.txt",        vvVerbs::DiffEntry::CHANGE_MOVED,                                          false, "", "", "@/a/b/c/1.txt", "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900007", "@/file/a/c/2.txt",        vvVerbs::DiffEntry::CHANGE_XATTRS,                                         false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900008", "@/file/a/c/3.txt",        vvVerbs::DiffEntry::CHANGE_ATTRS,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900009", "@/file/a/d/e/f/g/1.txt",  vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED,  false, "", "", "",              "4.txt", "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900010", "@/file/a/d/e/f/g/2.txt",  vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,    false, "", "", "@/a/2.txt",     "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900011", "@/folder/a",              vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900012", "@/folder/b",              vvVerbs::DiffEntry::CHANGE_DELETED,                                        false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900013", "@/folder/a/1",            vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900014", "@/folder/a/2",            vvVerbs::DiffEntry::CHANGE_FOUND,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900015", "@/folder/b/1",            vvVerbs::DiffEntry::CHANGE_RENAMED,                                        false, "", "", "",              "3",     "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900016", "@/folder/b/2",            vvVerbs::DiffEntry::CHANGE_MOVED,                                          false, "", "", "@/c/2",         "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900017", "@/folder/c/1/A",          vvVerbs::DiffEntry::CHANGE_XATTRS,                                         false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900018", "@/folder/c/1/B",          vvVerbs::DiffEntry::CHANGE_ATTRS,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900019", "@/folder/d/2/A",          vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED,  false, "", "", "",              "C",     "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900020", "@/folder/d/2/B",          vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,    false, "", "", "@/a/3/B",       "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900021", "@/symlink/a/1.txt",       vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900022", "@/symlink/a/2.txt",       vvVerbs::DiffEntry::CHANGE_DELETED,                                        false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900023", "@/symlink/a/3.txt",       vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900024", "@/symlink/a/b/1.txt",     vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900025", "@/symlink/a/b/2.txt",     vvVerbs::DiffEntry::CHANGE_FOUND,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900026", "@/symlink/a/b/3.txt",     vvVerbs::DiffEntry::CHANGE_RENAMED,                                        false, "", "", "",              "4.txt", "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900027", "@/symlink/a/c/1.txt",     vvVerbs::DiffEntry::CHANGE_MOVED,                                          false, "", "", "@/a/b/c/1.txt", "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900028", "@/symlink/a/c/2.txt",     vvVerbs::DiffEntry::CHANGE_XATTRS,                                         false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900029", "@/symlink/a/c/3.txt",     vvVerbs::DiffEntry::CHANGE_ATTRS,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900030", "@/symlink/a/d/e/1.txt",   vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED,  false, "", "", "",              "4.txt", "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SYMLINK,   "g1234567890123456789012345678901234567890123456789012345678900031", "@/symlink/a/f/g/2.txt",   vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,    false, "", "", "@/a/2.txt",     "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900032", "@/submodule/clean/a",     vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900033", "@/submodule/clean/b",     vvVerbs::DiffEntry::CHANGE_DELETED,                                        false, "", "", "",              "",      "a", "",  false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900034", "@/submodule/clean/a/1",   vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "a", "b", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900035", "@/submodule/clean/a/2",   vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "a", "",  false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900036", "@/submodule/clean/b/1",   vvVerbs::DiffEntry::CHANGE_FOUND,                                          false, "", "", "",              "",      "",  "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900037", "@/submodule/clean/b/2",   vvVerbs::DiffEntry::CHANGE_RENAMED,                                        false, "", "", "",              "3",     "a", "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900038", "@/submodule/clean/c/1/A", vvVerbs::DiffEntry::CHANGE_MOVED,                                          false, "", "", "@/c/1/A",       "",      "a", "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900039", "@/submodule/clean/c/1/B", vvVerbs::DiffEntry::CHANGE_XATTRS,                                         false, "", "", "",              "",      "a", "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900040", "@/submodule/clean/d/2/A", vvVerbs::DiffEntry::CHANGE_ATTRS,                                          false, "", "", "",              "",      "a", "a", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900041", "@/submodule/clean/d/2/B", vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED,  false, "", "", "",              "C",     "a", "b", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900042", "@/submodule/clean/e/1/A", vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,    false, "", "", "@/c/1/A",       "",      "a", "b", false },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900043", "@/submodule/dirty/a",     vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900044", "@/submodule/dirty/b",     vvVerbs::DiffEntry::CHANGE_DELETED,                                        false, "", "", "",              "",      "a", "",  true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900045", "@/submodule/dirty/a/1",   vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "a", "b", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900046", "@/submodule/dirty/a/2",   vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "a", "",  true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900047", "@/submodule/dirty/b/1",   vvVerbs::DiffEntry::CHANGE_FOUND,                                          false, "", "", "",              "",      "",  "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900048", "@/submodule/dirty/b/2",   vvVerbs::DiffEntry::CHANGE_RENAMED,                                        false, "", "", "",              "3",     "a", "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900049", "@/submodule/dirty/c/1/A", vvVerbs::DiffEntry::CHANGE_MOVED,                                          false, "", "", "@/c/1/A",       "",      "a", "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900050", "@/submodule/dirty/c/1/B", vvVerbs::DiffEntry::CHANGE_XATTRS,                                         false, "", "", "",              "",      "a", "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900051", "@/submodule/dirty/d/2/A", vvVerbs::DiffEntry::CHANGE_ATTRS,                                          false, "", "", "",              "",      "a", "a", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900052", "@/submodule/dirty/d/2/B", vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED,  false, "", "", "",              "C",     "a", "b", true  },
	{ vvVerbs::DiffEntry::ITEM_SUBMODULE, "g1234567890123456789012345678901234567890123456789012345678900053", "@/submodule/dirty/e/1/A", vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,    false, "", "", "@/c/1/A",       "",      "a", "b", true  },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900054", "@/a/1.txt",               vvVerbs::DiffEntry::CHANGE_ADDED,                                          false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900055", "@/b/c/2.txt",             vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900056", "@/b/d/e/3.txt",           vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900057", "@/b/d/f/g/4.txt",         vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900058", "@/b/d/f/h/i/5.txt",       vvVerbs::DiffEntry::CHANGE_MODIFIED,                                       false, "", "", "",              "",      "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900059", "@/b/d/f/h/j/6.txt",       vvVerbs::DiffEntry::CHANGE_LOST,                                           false, "", "", "",              "",      "",  "",  false },
};*/

/**
 * Test status, realistic data.
 */
static vvVerbs::DiffEntry gaStatus[] =
{
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900000", "@/added.txt",                                        vvVerbs::DiffEntry::CHANGE_ADDED,                                         false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900001", "@/conflict.txt",                                     vvVerbs::DiffEntry::CHANGE_MODIFIED,                                      false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900002", "@/modified.txt",                                     vvVerbs::DiffEntry::CHANGE_MODIFIED,                                      false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900003", "@/resolved.txt",                                     vvVerbs::DiffEntry::CHANGE_MODIFIED,                                      false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900004", "@/deleted.txt",                                      vvVerbs::DiffEntry::CHANGE_DELETED,                                       false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900005", "@/renamed.txt",                                      vvVerbs::DiffEntry::CHANGE_RENAMED,                                       false, "", "", "",   "rename.txt",              "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900006", "@/lost.txt",                                         vvVerbs::DiffEntry::CHANGE_LOST,                                          false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900007", "@/found.txt",                                        vvVerbs::DiffEntry::CHANGE_FOUND,                                         false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900008", "@/dir1/dir2/dir3/moved.txt",                         vvVerbs::DiffEntry::CHANGE_MOVED,                                         false, "", "", "@/", "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900009", "@/ignored.txt~",                                     vvVerbs::DiffEntry::CHANGE_FOUND,                                         true,  "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900010", "@/modified_renamed.txt",                             vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_RENAMED, false, "", "", "",   "modified_rename.txt",     "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FILE,      "g1234567890123456789012345678901234567890123456789012345678900011", "@/dir1/dir2/dir3/modified_moved.txt",                vvVerbs::DiffEntry::CHANGE_MODIFIED | vvVerbs::DiffEntry::CHANGE_MOVED,   false, "", "", "@/", "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900012", "@/added",                                            vvVerbs::DiffEntry::CHANGE_ADDED,                                         false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900013", "@/deleted",                                          vvVerbs::DiffEntry::CHANGE_DELETED,                                       false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900014", "@/conflict_delete_renamed~L0~g123456~",              vvVerbs::DiffEntry::CHANGE_RENAMED,                                       false, "", "", "",   "conflict_delete_renamed", "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900015", "@/dir1/dir2/dir3/conflict_delete_moved~L0~g123456~", vvVerbs::DiffEntry::CHANGE_RENAMED,                                       false, "", "", "",   "conflict_delete_moved",   "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900016", "@/renamed",                                          vvVerbs::DiffEntry::CHANGE_RENAMED,                                       false, "", "", "",   "rename",                  "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900017", "@/dir1/dir2/dir3/conflict_moved_renamed",            vvVerbs::DiffEntry::CHANGE_MOVED,                                         false, "", "", "@/", "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900018", "@/dir1/dir2/dir3/moved",                             vvVerbs::DiffEntry::CHANGE_MOVED,                                         false, "", "", "@/", "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900019", "@/lost",                                             vvVerbs::DiffEntry::CHANGE_LOST,                                          false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900020", "@/found",                                            vvVerbs::DiffEntry::CHANGE_FOUND,                                         false, "", "", "",   "",                        "",  "",  false },
	{ vvVerbs::DiffEntry::ITEM_FOLDER,    "g1234567890123456789012345678901234567890123456789012345678900021", "@/ignored~",                                         vvVerbs::DiffEntry::CHANGE_FOUND,                                         true,  "", "", "",   "",                        "",  "",  false },
};

/**
 * Test conflicts.
 */
static vvVerbs::ConflictEntry gaConflicts[] =
{
	{ "g1234567890123456789012345678901234567890123456789012345678900001", false },
	{ "g1234567890123456789012345678901234567890123456789012345678900003", true  },
	{ "g1234567890123456789012345678901234567890123456789012345678900012", false },
	{ "g1234567890123456789012345678901234567890123456789012345678900013", false },
};


/*
**
** Internal Functions
**
*/

/**
 * Finds the deepest folder that contains all paths in a given set.
 * Returns wxEmptyString if no shared parent could be found.
 * That should only occur if the paths are on different volumes (or if no paths were given).
 */
wxString _GetSharedParentPath(
	const wxArrayString& cPaths //< [in] The set of paths to find the largest shared parent of.
	)
{
	typedef std::vector<wxFileName> stlFileNameVector;

	// make sure they gave us some paths
	if (cPaths.IsEmpty())
	{
		return wxEmptyString;
	}

	// convert all of the paths into wxFileName instances
	// along the way:
	// 1) remember the one with the fewest number of directories
	// 2) make sure they're all on the same volume
	stlFileNameVector cFilenames(cPaths.size());
	wxString     sVolume        = wxEmptyString;
	unsigned int uShortestIndex = 0u;
	unsigned int uShortestCount = std::numeric_limits<unsigned int>::max();
	for (unsigned int uIndex = 0u; uIndex < cPaths.size(); ++uIndex)
	{
		wxFileName cFilename = wxFileName::DirName(cPaths[uIndex]);

		// check this path's volume
		if (sVolume.IsEmpty())
		{
			sVolume = cFilename.GetVolume();
		}
		else if (sVolume != cFilename.GetVolume())
		{
			return wxEmptyString;
		}

		// if the path has a filename but no extension
		// then treat the filename as another directory
		if (!cFilename.GetName().IsEmpty() && cFilename.GetExt().IsEmpty())
		{
			cFilename.AppendDir(cFilename.GetName());
			cFilename.SetName(wxEmptyString);
		}

		// check this path's directory count
		unsigned int uCount = cFilename.GetDirCount();
		if (uCount < uShortestCount)
		{
			uShortestIndex = uIndex;
			uShortestCount = uCount;
		}

		// place the filename in the list
		cFilenames[uIndex].Assign(cFilename);
	}

	// the directories of the path that had the fewest is the largest possible shared path
	// the actual shared path is the subset of those directories that are shared by all the paths
	// loop through that set of directories building up a shared path
	// until we either reach the end or find a directory that isn't actually shared
	wxString sPath = wxFileName::GetPathSeparator();
	for (unsigned int uDirIndex = 0u; uDirIndex < cFilenames[uShortestIndex].GetDirCount(); ++uDirIndex)
	{
		wxString sDir   = cFilenames[uShortestIndex].GetDirs()[uDirIndex];
		bool     bEqual = true;

		// loop through all of the paths to check if the dir matches
		for (unsigned int uPathIndex = 0u; uPathIndex < cFilenames.size(); ++uPathIndex)
		{
			// don't bother comparing this dir against itself
			if (uPathIndex == uShortestIndex)
			{
				wxASSERT(cFilenames[uPathIndex].GetDirs()[uDirIndex] == sDir);
				continue;
			}

			// check if this path has the same directory
			if (cFilenames[uPathIndex].GetDirs()[uDirIndex] != sDir)
			{
				bEqual = false;
				break;
			}
		}

		// if this dir was shared among all of the paths, add it to our shared path
		// otherwise, we're done building the shared path
		if (bEqual)
		{
			sPath += sDir;
			sPath += wxFileName::GetPathSeparator();
		}
		else
		{
			break;
		}
	}

	// construct the wxFileName of the shared path and return it
	return wxFileName(sVolume, sPath, wxEmptyString, wxEmptyString).GetFullPath();
}


/*
**
** Public Functions
**
*/

bool vvVerbsDebug::Initialize(
	vvContext& WXUNUSED(pCtx)
	)
{
	return true;
}

void vvVerbsDebug::Shutdown(
	vvContext& WXUNUSED(pCtx)
	)
{
}

bool vvVerbsDebug::GetLocalRepos(
	vvContext&     pCtx,
	wxArrayString& cLocalRepos
	)
{
	pCtx.Log_PushOperation("Retrieving local repositories", vvContext::LOG_FLAG_VERBOSE);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaLocalRepos));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaLocalRepos); ++uIndex)
	{
		pCtx.Log_SetStep(gaLocalRepos[uIndex]);
		cLocalRepos.Add(gaLocalRepos[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::LocalRepoExists(
	vvContext&      pCtx,
	const wxString& sName
	)
{
	bool bExists = false;

	pCtx.Log_PushOperation("Checking if repository exists", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sName.c_str(), vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaLocalRepos));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaLocalRepos); ++uIndex)
	{
		pCtx.Log_SetStep(gaLocalRepos[uIndex]);
		if (sName == gaLocalRepos[uIndex])
		{
			bExists = true;
			break;
		}
		pCtx.Log_FinishStep();
	}

	pCtx.Log_SetValue("Exists", bExists, vvContext::LOG_FLAG_OUTPUT);
	pCtx.Log_PopOperation();

	return bExists;
}

bool vvVerbsDebug::GetRemoteRepos(
	vvContext&     pCtx,
	wxString& sRepoDescriptorName,
	wxArrayString& cRemoteRepos,
	wxArrayString& cRemoteRepoAliases
	)
{
	sRepoDescriptorName;
	pCtx.Log_PushOperation("Retrieving remote repositories", vvContext::LOG_FLAG_VERBOSE);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaRemoteRepos));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaRemoteRepos); ++uIndex)
	{
		pCtx.Log_SetStep(gaRemoteRepos[uIndex]);
		cRemoteRepos.Add(gaRemoteRepos[uIndex]);
		if (uIndex == 0)
			cRemoteRepoAliases.Add("default");
		else
			cRemoteRepoAliases.Add("");
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::FindWorkingFolder(
	class vvContext& pCtx,
	const wxString&  sPath,
	wxString*        pFolder,
	wxString*        pRepo,
	wxArrayString*        pBaselineHIDs
	)
{
	pCtx.Log_PushOperation("Finding working folder", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Path", sPath, vvContext::LOG_FLAG_INPUT);

	if (pFolder != NULL)
	{
		pFolder->assign(sPath);
		pCtx.Log_SetValue("Working folder", *pFolder, vvContext::LOG_FLAG_OUTPUT);
	}

	if (pRepo != NULL)
	{
		wxASSERT(vvARRAY_COUNT(gaLocalRepos) > 0);
		pRepo->assign(gaLocalRepos[0]);
		pCtx.Log_SetValue("Repository", *pRepo, vvContext::LOG_FLAG_OUTPUT);
	}
	if (pBaselineHIDs != NULL)
	{
		pBaselineHIDs->Add("abc123");
		pBaselineHIDs->Add("123abc");
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::FindWorkingFolder(
	class vvContext&     pCtx,
	const wxArrayString& cPaths,
	wxString*            pFolder,
	wxString*            pRepo,
	wxArrayString*        pBaselineHIDs
	)
{
	pCtx.Log_PushOperation("Finding working folder", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Paths", wxJoin(cPaths, ','), vvContext::LOG_FLAG_INPUT);

	if (pFolder != NULL)
	{
		pFolder->assign(_GetSharedParentPath(cPaths));
		pCtx.Log_SetValue("Working folder", *pFolder, vvContext::LOG_FLAG_OUTPUT);
	}

	if (pRepo != NULL)
	{
		wxASSERT(vvARRAY_COUNT(gaLocalRepos) > 0);
		pRepo->assign(gaLocalRepos[0]);
		pCtx.Log_SetValue("Repository", *pRepo, vvContext::LOG_FLAG_OUTPUT);
	}
	if (pBaselineHIDs != NULL)
	{
		pBaselineHIDs->Add("abc123");
		pBaselineHIDs->Add("123abc");
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::GetRepoPath(
	vvContext&      pCtx,
	const wxString& sAbsolutePath,
	const wxString& sWorkingFolder,
	wxString&       sRepoPath
	)
{
	wxFileName cPath = wxFileName::DirName(sAbsolutePath);
	wxASSERT(cPath.IsOk());
	wxASSERT(cPath.IsAbsolute());

	pCtx.Log_PushOperation("Converting to repo path", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Absolute Path", sAbsolutePath, vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Working Folder", sWorkingFolder, vvContext::LOG_FLAG_INPUT);

	if (!cPath.MakeRelativeTo(sWorkingFolder))
	{
		return false;
	}
	wxASSERT(cPath.IsRelative());
	cPath.PrependDir(vvVerbs::sszRepoRoot);
	sRepoPath.assign(cPath.GetFullPath(wxPATH_UNIX));
	pCtx.Log_SetValue("Repository Path", sRepoPath, vvContext::LOG_FLAG_OUTPUT);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::GetBranches(
	vvContext&      pCtx,
	const wxString& sRepo,
	bool bIncludeClosed,
	wxArrayString&  cBranches
	)
{
	bIncludeClosed;
	pCtx.Log_PushOperation("Retrieving branches", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sRepo, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaBranches));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaBranches); ++uIndex)
	{
		pCtx.Log_SetStep(gaBranches[uIndex]);
		cBranches.Add(gaBranches[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::GetUsers(
	vvContext&       pCtx,
	const wxString&  sRepo,
	stlUserDataList& cUsers
	)
{
	pCtx.Log_PushOperation("Retrieving users", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sRepo, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaUsers));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaUsers); ++uIndex)
	{
		pCtx.Log_SetStep(gaUsers[uIndex].sName);
		cUsers.push_back(gaUsers[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::GetCurrentUser(
	class vvContext& pCtx,
	const wxString&  sRepo,
	wxString&        sUser
	)
{
	pCtx.Log_PushOperation("Retrieving current user", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sRepo, vvContext::LOG_FLAG_INPUT);

	sUser = gszCurrentUser;
	pCtx.Log_SetValue("User", sUser, vvContext::LOG_FLAG_OUTPUT);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::GetStamps(
	vvContext&        pCtx,
	const wxString&   sRepo,
	stlStampDataList& cStamps
	)
{
	pCtx.Log_PushOperation("Retrieving stamps", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sRepo, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaStamps));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaStamps); ++uIndex)
	{
		pCtx.Log_SetStep(gaStamps[uIndex].sName);
		cStamps.push_back(gaStamps[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}


bool vvVerbsDebug::GetTags(
	vvContext&        pCtx,
	const wxString&   sRepo,
	stlTagDataList& cTags
	)
{
	pCtx.Log_PushOperation("Retrieving tags", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Repository", sRepo, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaTags));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaTags); ++uIndex)
	{
		pCtx.Log_SetStep(gaTags[uIndex].sName);
		cTags.push_back(gaTags[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}


bool vvVerbsDebug::GetBranch(
	vvContext&      pCtx,
	const wxString& sFolder,
	wxString&       sBranch
	)
{
	pCtx.Log_PushOperation("Retrieving tied branch", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Folder", sFolder, vvContext::LOG_FLAG_INPUT);

	sBranch = gaBranches[0];
	pCtx.Log_SetValue("Branch", sBranch, vvContext::LOG_FLAG_OUTPUT);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::New(
	vvContext&      pCtx,
	const wxString& sName,
	const wxString& sFolder,
	const wxString& sStorageDriver,
	const wxString& sHashMethod
	)
{
	static const unsigned int uSteps = 2u;

	pCtx.Log_PushOperation("Creating new repository", vvContext::LOG_FLAG_NONE);
	pCtx.Log_SetValue("Repository",      sName,                                       vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Checkout folder", DEFAULT_STRING(sFolder,        "(none)"),    vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Storage driver",  DEFAULT_STRING(sStorageDriver, "(default)"), vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Hash method",     DEFAULT_STRING(sHashMethod,    "(default)"), vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(uSteps);
	this->SimulateWork(pCtx, 2000u, uSteps);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::Clone(
	vvContext&      pCtx,
	const wxString& sExistingRepo,
	const wxString& sNewRepoName
	)
{
	static const unsigned int uSteps = 4;

	pCtx.Log_PushOperation("Cloning repository", vvContext::LOG_FLAG_NONE);
	pCtx.Log_SetValue("Source", sExistingRepo, vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Target", sNewRepoName,  vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(uSteps);
	this->SimulateWork(pCtx, 8000u, uSteps);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}
bool vvVerbsDebug::Update(
	vvContext&      pCtx,
	const wxString& sFolder,
	CheckoutType    eType,
	const wxString& sTarget
	)
{
	pCtx;
	sFolder;
	eType;
	sTarget;

	return true;
}
bool vvVerbsDebug::Checkout(
	vvContext&      pCtx,
	const wxString& sName,
	const wxString& sFolder,
	CheckoutType    eType,
	const wxString& sTarget
	)
{
	static const unsigned int uSteps = 10u;

	pCtx.Log_PushOperation("Checking out repository", vvContext::LOG_FLAG_NONE);
	pCtx.Log_SetValue("Repository",    sName,   vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Target folder", sFolder, vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Target type",   eType,   vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Target name",   sTarget, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(uSteps);
	this->SimulateWork(pCtx, 5000u, uSteps);

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::Status(
	vvContext&      pCtx,
	const wxString& sFolder,
	const wxString& sRevision,
	DiffData&       cDiff
	)
{
	pCtx.Log_PushOperation("Retrieving status", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Folder", sFolder, vvContext::LOG_FLAG_INPUT);
	if (sRevision != wxEmptyString)
	{
		pCtx.Log_SetValue("Revision", sRevision, vvContext::LOG_FLAG_INPUT);
	}

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaStatus));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaStatus); ++uIndex)
	{
		pCtx.Log_SetStep(gaStatus[uIndex].sItemGid);
		cDiff.push_back(gaStatus[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::Conflicts(
	vvContext&      pCtx,
	const wxString& sFolder,
	ConflictData&   cConflicts
	)
{
	pCtx.Log_PushOperation("Retrieving conflicts", vvContext::LOG_FLAG_VERBOSE);
	pCtx.Log_SetValue("Folder", sFolder, vvContext::LOG_FLAG_INPUT);

	pCtx.Log_SetSteps(vvARRAY_COUNT(gaConflicts));
	for (unsigned int uIndex = 0u; uIndex < vvARRAY_COUNT(gaConflicts); ++uIndex)
	{
		pCtx.Log_SetStep(gaConflicts[uIndex].sItemGid);
		cConflicts.push_back(gaConflicts[uIndex]);
		pCtx.Log_FinishStep();
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::Commit(
	vvContext&           pCtx,
	const wxString&      sFolder,
	const wxString&      sComment,
	const wxArrayString* pPaths,
	const wxArrayString* pStamps,
	const wxString*      pUser,
	vvFlags              cFlags,
	wxString*            pHid
	)
{
	static wxString sHid = "1234567890123456789012345678901234567890";

	pCtx.Log_PushOperation("Committing", 0u);
	pCtx.Log_SetValue("Folder", sFolder, vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Comment", sComment, vvContext::LOG_FLAG_INPUT);
	pCtx.Log_SetValue("Flags", cFlags, vvContext::LOG_FLAG_INPUT);
	if (pPaths != NULL)
	{
		pCtx.Log_SetValue("Items", wxJoin(*pPaths, ','), vvContext::LOG_FLAG_INPUT);
	}
	if (pStamps != NULL)
	{
		pCtx.Log_SetValue("Stamps", wxJoin(*pStamps, ','), vvContext::LOG_FLAG_INPUT);
	}
	if (pUser != NULL)
	{
		pCtx.Log_SetValue("User", *pUser, vvContext::LOG_FLAG_INPUT);
	}

	this->SimulateWork(pCtx, 5000u);

	pCtx.Log_SetValue("Hid", sHid, vvContext::LOG_FLAG_OUTPUT);
	if (pHid != NULL)
	{
		pHid->assign(sHid);
	}

	pCtx.Log_PopOperation();
	return !pCtx.Error_Check();
}

void vvVerbsDebug::SimulateWork(
	vvContext&   pCtx,
	unsigned int uBase,
	unsigned int uSteps,
	float        dVariation
	)
{
	wxASSERT(dVariation >= 0.0f);
	wxASSERT(dVariation < 1.0f);

	unsigned int uVariation = uBase * dVariation;
	unsigned int uMinimum   = uBase - uVariation; // can never go negative because 0.0f <= dVariation < 1.0f
	unsigned int uMaximum   = uBase + uVariation;
	unsigned int uRange     = uMaximum - uMinimum;
	unsigned int uRandom    = (((unsigned int)rand()) % uRange) + uMinimum;

	if (uSteps == 0u)
	{
		Sleep(uRandom);
	}
	else
	{
		unsigned int uPerStep = uRandom / uSteps;
		for (unsigned int uIndex = 0; uIndex < uSteps; ++uIndex)
		{
			Sleep(uPerStep);
			pCtx.Log_FinishStep();
		}
	}

	return;
}

bool vvVerbsDebug::Rename(
	vvContext&      pCtx,
	const wxString& sFullPath_objectToRename,
	const wxString& sNewName
	)
{
	pCtx;
	sFullPath_objectToRename;
	sNewName;
	return true;
}

bool vvVerbsDebug::AddComment(
		class vvContext& pCtx, 
		const wxString&  sRepoName,    
		const wxString&  sChangesetHID,  
		const wxString&  sNewComment	
		)
{
	pCtx;
	sRepoName;
	sChangesetHID;
	sNewComment;
	return true;
}

bool vvVerbsDebug::AddStamp(
		class vvContext& pCtx, 
		const wxString&  sRepoName,    
		const wxString&  sChangesetHID,  
		const wxString&  sNewComment	
		)
{
	pCtx;
	sRepoName;
	sChangesetHID;
	sNewComment;
	return true;
}

bool vvVerbsDebug::AddTag(
		class vvContext& pCtx, 
		const wxString&  sRepoName,    
		const wxString&  sChangesetHID,  
		const wxString&  sNewComment	
		)
{
	pCtx;
	sRepoName;
	sChangesetHID;
	sNewComment;
	return true;
}
bool vvVerbsDebug::DeleteStamp(
		class vvContext& pCtx, 
		const wxString&  sRepoName,    
		const wxString&  sChangesetHID,  
		const wxString&  sRemoveMe	
		)
{
	sRepoName;
	sChangesetHID;
	sRemoveMe;
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::DeleteTag(
		class vvContext& pCtx, 
		const wxString&  sRepoName,    
		const wxString&  sRemoveMe	
		)
{
	sRepoName;
	sRemoveMe;
	return !pCtx.Error_Check();
}

bool vvVerbsDebug::Move(
	vvContext&      pCtx,
	const wxArrayString& sFullPath_objectToMove,
	const wxString& sNewParentPath
	)
{
	pCtx;
	sFullPath_objectToMove;
	sNewParentPath;
	return true;
}

bool vvVerbsDebug::Add(
	vvContext&      pCtx,
	const wxArrayString& sFullPath_objectToAdd
	)
{
	pCtx;
	sFullPath_objectToAdd;
	return true;
}

bool vvVerbsDebug::Remove(
	vvContext&      pCtx,
	const wxArrayString& sFullPath_objectToRemove
	)
{
	pCtx;
	sFullPath_objectToRemove;
	return true;
}


bool vvVerbsDebug::Pull(
		class vvContext& cContext,						
		const wxString&  sSrcLocalRepoNameOrRemoteURL,
		const wxString&  sDestLocalRepoName,				
		const wxArrayString& saAreas,										
		const wxArrayString& saTags,					
		const wxArrayString& saDagnodes,
		const wxArrayString& saBranchNames
		)
{
	cContext;
	sDestLocalRepoName;
	sSrcLocalRepoNameOrRemoteURL;
	saAreas;
	saTags;
	saDagnodes;
	saBranchNames;
	return true;
}


bool vvVerbsDebug::Push(
		class vvContext& cContext,						
		const wxString&  sDestLocalRepoNameOrRemoteURL,	
		const wxString&  sSrcLocalRepoName,				
		const wxArrayString& saAreas,										
		const wxArrayString& saTags,					
		const wxArrayString& saDagnodes,
		const wxArrayString& saBranchNames
		)
{
	cContext;
	sSrcLocalRepoName;
	sDestLocalRepoNameOrRemoteURL;
	saAreas;
	saTags;
	saDagnodes;
	saBranchNames;
	return true;
}

bool vvVerbsDebug::Incoming(
		class vvContext& cContext,						//< [in] [out] Error and context info.
		const wxString&  sSrcLocalRepoNameOrRemoteURL,	//< [in] The local repo name, or remote repo URL.  Changes will be pulled from that repo.
		const wxString&  sDestLocalRepoName,			//< [in] The name of the local repo to pull changes into.
		HistoryData& cHistoryData
		)
{
	cContext;
	sDestLocalRepoName;
	sSrcLocalRepoNameOrRemoteURL;
	cHistoryData;
	return true;
}

bool vvVerbsDebug::Outgoing(
		class vvContext& cContext,						//< [in] [out] Error and context info.
		const wxString&  sDestLocalRepoNameOrRemoteURL,	//< [in] The local repo name, or remote repo URL.  Changes will be pulled from that repo.
		const wxString&  sSrcLocalRepoName,			//< [in] The name of the local repo to pull changes into.
		HistoryData& cHistoryData
		)
{
	cContext;
	sDestLocalRepoNameOrRemoteURL;
	sSrcLocalRepoName;
	cHistoryData;
	return true;
}

bool vvVerbsDebug::History(
		class vvContext& cContext,						//< [in] [out] Error and context info.
		const wxString&  sLocalRepoName,				//< [in] The name of the local repo to pull changes into.
		const vvHistoryFilterOptions filterOptions,		//< [in] History Options.
		HistoryData& cHistoryData,
		void ** ppNewHistoryToken
		)
{
	cContext;
	filterOptions;
	sLocalRepoName;
	cHistoryData;
	ppNewHistoryToken;
	return true;
}


bool vvVerbsDebug::History__Fetch_More(
		class vvContext& pCtx,							//< [in] [out] Error and context info.
		const wxString&  sLocalRepoName,				//< [in] The name of the local repo to pull changes into.
		unsigned int nNumResultsDesired,				//< [in] The number of additional results desired.
		void * pHistoryToken,
		HistoryData& cHistoryData,
		void ** ppNewHistoryToken
		)
{
	pCtx;
	sLocalRepoName;
	cHistoryData;
	nNumResultsDesired;
	pHistoryToken;
	ppNewHistoryToken;
	return true;
}


bool vvVerbsDebug::History__Free_Token(
		class vvContext& pCtx,							//< [in] [out] Error and context info.
		void * pHistoryToken)
{
	pCtx;
	pHistoryToken;
	return true;
}



bool vvVerbsDebug::GetColumnPreferences(
	vvContext&     pCtx,
	vvHistoryColumnPreferences& cColumnPrefs
	)
{
	pCtx;
	cColumnPrefs;
	return true;
}

bool vvVerbsDebug::SaveColumnPreferences(
	vvContext&     pCtx,
	vvHistoryColumnPreferences& cColumnPrefs
	)
{
	pCtx;
	cColumnPrefs;
	return true;
}

bool vvVerbsDebug::GetHistoryFilterDefaults(
	vvContext&     pCtx,
	wxString& sRepoName,
	vvHistoryFilterOptions& cColumnPrefs
	)
{
	sRepoName;
	pCtx;
	cColumnPrefs;
	return true;
}

bool vvVerbsDebug::Repo_Item_Get_Children(
		class vvContext& pCtx,
		wxString sRepoName,
		wxString sChangesetHID,
		wxString sItemPath,
		vvVerbs::RepoItemList& cChildren)
{
	sRepoName;
	cChildren;
	sChangesetHID;
	pCtx;
	sItemPath;
	return true;
}

bool vvVerbsDebug::Get_Revision_Details(
	class vvContext& pCtx,							//< [in] [out] Error and context info.
	const wxString&  sLocalRepoName,				//< [in] The name of the local repo to pull changes into.
	const wxString& sRevID,							//< [in] The revision to load
	vvVerbs::HistoryEntry& cHistoryEntry,						//< [out] The details.
	vvVerbs::RevisionList& children						//< [out] A vector of the children of the revision.
	)
{
	pCtx;
	sLocalRepoName;
	sRevID;
	cHistoryEntry;
	children;
	return true;
}

bool vvVerbsDebug::Export_Revision(
	class vvContext& pCtx,							//< [in] [out] Error and context info.
	const wxString&  sLocalRepoName,				//< [in] The name of the local repo to pull changes into.
	const wxString& sRevID,							//< [in] The revision to load
	wxString sLocalOutputPath						//< [in] The local path to output to.
)
{
	pCtx;
	sLocalRepoName;
	sRevID;
	sLocalOutputPath;
	return true;
}