#!/bin/bash
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Copyright 2010-2011 SourceGear, LLC
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## usage: c_test.sh c_test_name ${CMAKE_CURRENT_SOURCE_DIR}
set -o xtrace

export SGCLOSET="$HOME/.sgcloset-test"
##echo SGCLOSET is $SGCLOSET

export TEST=$1
##echo TEST is $TEST

export SRCDIR=$2
##echo SRCDIR is $SRCDIR

export BINDIR=`pwd`
##echo BINDIR is $BINDIR

## We need vv to be in our path.
export PATH=$BINDIR/../src/cmd:$PATH
##echo PATH is $PATH

## Turn on core dumps.
ulimit -c unlimited
ulimit -a

## Dump the localsettings environment.
vv config

echo "////////////////////////////////////////////////////////////////"
echo "Starting test $TEST..."
echo "////////////////////////////////////////////////////////////////"
echo ""

./$TEST "${SRCDIR}"
RESULT_TEST=$?

## If the test crashed (SIGSEGV), try to get a stacktrace.
if [ $RESULT_TEST -eq 139 ]; then
    echo ""
    echo "////////////////////////////////////////////////////////////////"
    echo "Getting stacktrace..."
    echo "////////////////////////////////////////////////////////////////"
    echo ""
    ## the most recent coredump should be ours
    COREFILE=`ls -t /cores | head -1`
    GDBINPUT="$BINDIR"/gdbinput.txt
    echo "thread apply all bt full" >  $GDBINPUT
    echo "quit"                     >> $GDBINPUT
    /usr/bin/gdb -batch -x $GDBINPUT ./$TEST /cores/$COREFILE
    rm $GDBINPUT
fi


exit $RESULT_TEST;
