/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 *
 * @file u0084_pathname.c
 *
 * @details 
 *
 */

//////////////////////////////////////////////////////////////////

#include <sg.h>

#include "unittests.h"

//////////////////////////////////////////////////////////////////

void u0084_pathname__test_make_relative(SG_context * pCtx)
{
	SG_pathname * pPath1 = NULL;
	SG_pathname * pPath2 = NULL;
	SG_string * pStringDelta = NULL;

	struct table_entry
	{
		const char * pszInput1;
		const char * pszInput2;
		const char * pszExpectedResult;
	};

	static struct table_entry table[] =
		{ 
			{ "/a/b/c/d",         "/a/b/e/f",       "../../e/f" },
			{ "/a/b/c/d",         "/a/b/c/d",       ""          },
			{ "/a/b/c/d/",        "/a/b/c/d/",      ""          },
		};

	SG_uint32 kLimit = SG_NrElements(table);
	SG_uint32 k;

	for (k=0; k<kLimit; k++)
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath1, table[k].pszInput1)  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath2, table[k].pszInput2)  );

		VERIFY_ERR_CHECK(  SG_pathname__make_relative(pCtx, pPath1, pPath2, &pStringDelta)  );
		VERIFYP_COND("test_make_relative", (strcmp(SG_string__sz(pStringDelta),table[k].pszExpectedResult)==0),
					 ("Row[%d] '%s' vs '%s' expected '%s' received '%s'",
					  k,
					  table[k].pszInput1,
					  table[k].pszInput2,
					  table[k].pszExpectedResult,
					  SG_string__sz(pStringDelta)));

		SG_PATHNAME_NULLFREE(pCtx, pPath1);
		SG_PATHNAME_NULLFREE(pCtx, pPath2);
		SG_STRING_NULLFREE(pCtx, pStringDelta);
	}

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPath1);
	SG_PATHNAME_NULLFREE(pCtx, pPath2);
	SG_STRING_NULLFREE(pCtx, pStringDelta);
}

TEST_MAIN(u0084_pathname)
{
	TEMPLATE_MAIN_START;

#if 0	// waiting for bug: S00043
	BEGIN_TEST(  u0084_pathname__test_make_relative(pCtx)  );
#endif

	TEMPLATE_MAIN_END;
}

