/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
  vv

  The Sprawl command-line application
*/

#include <sg.h>
#include "sg_typedefs.h"
#include "sg_prototypes.h"
#include "sg_cmd_util_prototypes.h"
#include "sg_help_source.h"

/* For every command, provide a forward declaration of the cmd func
 * here, so that its function pointer can be included in the lookup
 * table. */

DECLARE_CMD_FUNC(add);
DECLARE_CMD_FUNC(addremove);
DECLARE_CMD_FUNC(lock);
DECLARE_CMD_FUNC(unlock);
DECLARE_CMD_FUNC(locks);
DECLARE_CMD_FUNC(branch);
DECLARE_CMD_FUNC(cat);
DECLARE_CMD_FUNC(checkout);
DECLARE_CMD_FUNC(export);
DECLARE_CMD_FUNC(fast_import);
DECLARE_CMD_FUNC(fast_export);
DECLARE_CMD_FUNC(zip);
DECLARE_CMD_FUNC(clone);
DECLARE_CMD_FUNC(user);
#if 0
DECLARE_CMD_FUNC(creategroup);
DECLARE_CMD_FUNC(groups);
DECLARE_CMD_FUNC(group);
#endif
DECLARE_CMD_FUNC(log);
DECLARE_CMD_FUNC(comment);
DECLARE_CMD_FUNC(commit);
DECLARE_CMD_FUNC(diff);
DECLARE_CMD_FUNC(diffmerge);
DECLARE_CMD_FUNC(heads);
DECLARE_CMD_FUNC(leaves);
DECLARE_CMD_FUNC(help);
DECLARE_CMD_FUNC(history);
DECLARE_CMD_FUNC(incoming);
DECLARE_CMD_FUNC(localsetting);
DECLARE_CMD_FUNC(localsettings);
DECLARE_CMD_FUNC(merge);
DECLARE_CMD_FUNC(move);
DECLARE_CMD_FUNC(init_new_repo);
DECLARE_CMD_FUNC(outgoing);
DECLARE_CMD_FUNC(parents);
DECLARE_CMD_FUNC(pull);
DECLARE_CMD_FUNC(push);
DECLARE_CMD_FUNC(reindex);
DECLARE_CMD_FUNC(remove);
DECLARE_CMD_FUNC(rename);
DECLARE_CMD_FUNC(repo);
DECLARE_CMD_FUNC(resolve);
DECLARE_CMD_FUNC(resolve2);
DECLARE_CMD_FUNC(revert);
DECLARE_CMD_FUNC(scan);
DECLARE_CMD_FUNC(serve);
#if 0
DECLARE_CMD_FUNC(slurp);
#endif
DECLARE_CMD_FUNC(stamp);
DECLARE_CMD_FUNC(status);
DECLARE_CMD_FUNC(tag);
DECLARE_CMD_FUNC(update);
DECLARE_CMD_FUNC(version);
DECLARE_CMD_FUNC(whoami);
#if 0
DECLARE_CMD_FUNC(fragball);
#endif

//advanced commands
DECLARE_CMD_FUNC(zingmerge);
DECLARE_CMD_FUNC(blobcount);
DECLARE_CMD_FUNC(dump_json);
DECLARE_CMD_FUNC(walk_fulltree);
DECLARE_CMD_FUNC(manually_set_parents);
DECLARE_CMD_FUNC(graph_history);
DECLARE_CMD_FUNC(dump_lca);
#ifdef DEBUG
DECLARE_CMD_FUNC(dump_treenode);
#endif
DECLARE_CMD_FUNC(hid);
DECLARE_CMD_FUNC(vcdiff);
DECLARE_CMD_FUNC(check_integrity);

static void _sg_option_state__free(SG_context* pCtx, SG_option_state* pThis);

#if 1 && defined(DEBUG)
#define HAVE_TEST_CRASH
DECLARE_CMD_FUNC(crash);
#endif

struct sg_getopt_option sg_cl_options[] =
{                                                                                           //break point for 80-column display
	// word-wrapping is done automatically now for command and option descriptions, so hard tabs and newlines should be avoided.
	{"all",          opt_all,      0, "Include all files and folders recursively"},
	{"all-but",      opt_all_but,  0, "Include all items except for those listed."},
	{"all-commands", opt_show_all, 0, "Include normally hidden commands used primarily for debugging"},
	{"all-full",     opt_all_full, 0, "Change all blobs to full encoding"},
	{"all-zlib",     opt_all_zlib, 0, "Change all blobs to zlib encoding"},
	{"area",         opt_area,     1, "Specify which area of repo data to push or pull"},
	{"assoc",        'a',          1, "Associate work item"},
	{"attach",       opt_attach,   1, "Attach the working copy"},
	{"branch",       'b',          1, "Use the changeset identified by the head of this branch"},
	{"cert",         opt_cert,     1, "Full or relative path of certificate (.PEM) file to use in SSL encryption. Defaults to ./ssl_cert.pem"},
	{"clean",        'C',          0, "Replace the working copy with the baseline repository version. "},
	{"create",       opt_create,   0, "Add the user if it does not exist"},
	{"dest",         opt_dest,     1, "Pull into the specified repository"},
	{"detached",     opt_detached, 0, "Allow the commit even though the working copy is not attached to a branch"},
	/* See: Y2908 {"exclude",      'X',          1, "Exclude files matching the given (extended glob) pattern. May be used multiple times."},
	{"exclude-from", opt_exclude_from, 1, ""}, */
	{"force",        'F',          0, "Force removal of modified items"},
	{"from",         opt_from,     1, "Earliest date included in the history query"},
	{"global",       opt_global,   0, "Apply change to the Machine Default of the setting (in addition to the current working copy, if applicable)."},
	{"hash",         opt_hash,     1, "Hash algorithm to use for the new repository; valid values are: SHA1/160 (default), SHA2/256, SHA2/384, SHA2/512, SKEIN/256, and SKEIN/512"},
	{"hide-merges",  opt_hide_merges,   0, "Do not include merge changesets where a file or folder is identical to the version in one of the parent changesets; requires a file or folder argument"},
	/* See: Y2908 {"include",      'I',          1, "Include only files matching the given (extended glob) pattern. May be used multiple times."},
	{"include-from", opt_include_from, 1, ""}, */
	{"interactive" , 'i',          0, "Run the command in interactive console mode"},
	{"label",        opt_label,    1, "Use the specified label"},
	{"leaves",       opt_leaves,   0, "Include history starting with all leaf nodes in the DAG"},
	{"list",         opt_list,     0, "List one or more issues or all unresolved issues."},
	{"list-all",     opt_list_all, 0, "List all resolved and unresolved issues."},
	{"mark",         opt_mark,     0, "Manually mark one or more issues as resolved."},
	{"mark-all",     opt_mark_all, 0, "Manually mark all issues as resolved."},
	{"max",          opt_max,      1, "Maximum number of results to include in the history query"},
	{"message",      'm',          1, "Add changeset comment"},
	{"no-auto-merge",opt_no_auto_merge, 0, "Do not auto-merge file content"},
	{"no-backups",   opt_no_backups, 0, "Do not create backups of dirty files"},
	{"no-ignores",   opt_no_ignores, 0, "Include files that would otherwise be ignored"},
	{"no-plain",     opt_no_plain, 0, "Serve encrypted content only."},
	{"no-prompt",    opt_no_prompt,0, "Bypass confirmation prompt."},
	/* See: W6233 {"no-submodules",opt_no_submodules, 0, "Do not recursively perform command on nested submodules."}, */
	/* {"submodule",    opt_submodule,0, "Create the working copy as a submodule of another working copy"}, */
	/* {"use-sub",      opt_use_submodule, 1, "Suggest the repository name to use for a submodule."}, */
	{"nonrecursive", 'N',          0, "Do not recurse subdirectories"},
	{"overwrite",    opt_overwrite, 0, "Overwrite something that already exists."},
	{"pack",         opt_pack,     0, "Compress the repository using vcdiff and zlib where appropritate"},
	{"port",         opt_port,     1, "Port to use for unencrypted content; defaults to 8080"},
	{"public",       'P',          0, "Allow remote connections"},
	{"quiet",        opt_quiet,    0, "Show less detailed output" },
	{"pull",         opt_pull,     0, "Refresh the list of locks from the upstream repository instance"},
	{"remerge",      opt_remerge,  0, "Remerge the contents of the file."},
	{"remove",       opt_remove,   0, "Remove, rather than add."},
	{"repo",         opt_repo,     1, "The repository descriptor name"},
	{"rev",          'r',          1, "Use the changeset identified by this revision identifier"},
	{"reverse",		 opt_reverse,  0, "Reverse the order of the history output (this does not change which revisions are returned, only the order in which they are displayed)"},
	{"server",       opt_server,   1, "Specify the repository instance to use when synchronizing locks"},
	{"shared-users", opt_shared_users, 1, "New repository should share user accounts with the given existing repository instance"},
	{"sport",        opt_sport,    1, "The port to use for encrypted content. Must not be the same as the implicit or explicit value of --port."},
	{"src",          opt_src,      1, "Push from the specified repository"},
	{"stamp",        opt_stamp,    1, "List only changesets identified with the given stamp"},
	{"storage",      opt_storage,  1, "The storage technology to use for the new repository" },
	{"subgroups",    opt_subgroups,0, "The members to be added are names of subgroups rather than names of users"},
	{"tag",          opt_tag,      1, "Use the changeset identified by this tag"},
	{"test",         'T',          0, "Do not pend any actions; display details about actions that would be pended"},
	{"to",           opt_to,       1, "Latest date included in the history query"},
	{"tool",         opt_tool,     1, "Specify a tool to use"},
	{"trace",        opt_trace,    0, "Print verboser-than-verbose output for debugging and diagnostics. (debug)"},
	{"unified",      opt_unified,  1, "number of lines of context to print around diffs"},
	{"update",       'u',          0, "Update the working copy after the pull"},
	{"user",         'U',          1, "List only changesets committed by this user"},
	{"verbose",      opt_verbose,  0, "Include full text of multi-line comments"},
	{"when",         opt_when,     1, "Specify when these changes happened."},

#if defined(DEBUG)
	{"dagnum",       opt_dagnum,   1, "Request a specific DAG (debug)"},
#endif

	{0,               0, 0, 0},
};

/* Now the lookup table. */

struct cmdinfo commands[] =
{
	{
		"add", NULL, NULL, NULL, cmd_add,
		"Pend the addition of a file or directory to the current repository", "file-or-folder [file-or-folder ...]", NULL,
		SG_FALSE, 
		{ {'N',0,0}, {'T',0,0}, /* See: Y2908 {'I',0,0}, {'X',0,0}, */ {opt_no_ignores,0,0}, /* See: Y2908 {opt_include_from,0,0}, {opt_exclude_from,0,0} */}, 
		{ {'T', "Do not pend any adds; display details about adds that would be pended"} }
	},

	{
		"addremove", NULL, NULL, NULL, cmd_addremove,
		"Pend additions for all Found files and removals for all Lost files", "file-or-folder [file-or-folder ...]", NULL,
		SG_FALSE, 
		{ {'N',0,0}, {'T',0,0}, /* See: Y2908  {'I',0,0}, {'X',0,0}, */ {opt_no_ignores,0,0}, /* See: Y2908 {opt_include_from,0,0}, {opt_exclude_from,0,0} */}, 
		{ {0, ""} }
	},

	{
		"blobcount", NULL, NULL, NULL, cmd_blobcount,
		"Show the number of blobs in a repo", "repo_name", NULL,
		SG_TRUE, 
		{ {0,0,0} }, 
		{ {0, ""} }
	},

	{
		"branch", "branches", NULL, NULL, cmd_branch,
		"Create, delete, manipulate or list named branches", 
		"[subcommand [branchname]] [OPTIONS] ", &vv_help_branch,
		SG_FALSE, { {opt_all,0,1}, {OPT_REV_SPEC,0,0}, {opt_repo,0,1} }, 
		{
			{opt_all, "Include all heads when moving; include closed branches when listing"},
			{'b', "Use the changeset(s) identified by the head(s) of this branch"}
		}
	},

	{
		"cat", "type", NULL, NULL, cmd_cat,
		"Unix-style cat on a repository file", "[--repo=REPOSITORY] [--rev=CHANGESETID | --tag=TAG]  file [...]", NULL,
		SG_FALSE, 
		{ {opt_repo,0,1}, {OPT_REV_SPEC,0,1} }, 
		{ {'b', "Use the changeset identified by the head of this branch; it is an error if the branch has multiple heads"} }
	},

	{
		"verify", "check_integrity", NULL, NULL, cmd_check_integrity,
		"Check repository integrity", "repo_name", NULL,
		SG_TRUE, 
		{ {0,0,0} }, 
		{ {0, ""} }
	},

	{
		"checkout", NULL, NULL, NULL, cmd_checkout,
		"Create a new working copy of a repository version", "OPTIONS REPOSITORY [path]", NULL,
		SG_FALSE, 
		{ {OPT_REV_SPEC,0,1}, {opt_attach,0,1}/* See: W6233 , {opt_submodule,0,0}, {opt_use_submodule,0,0} */ }, 
		{ {opt_attach, "Attach the working copy to the specified branch"} }
	},

	{
		"fast-import", NULL, NULL, NULL, cmd_fast_import,
		"Import from a fast-import stream", "OPTIONS import_file.fi REPOSITORY", NULL,
		SG_FALSE, 
		{ {opt_shared_users,0,1}, {opt_hash,0,1} },
		{ {0, ""} }
	},

	{
		"fast-export", NULL, NULL, NULL, cmd_fast_export,
		"Export a repository as a fast-import stream", "OPTIONS REPOSITORY export_file.fi", NULL,
		SG_FALSE, 
		{ {0,0,0} }, 
		{ {0, ""} }
	},

	{
		"export", NULL, NULL, NULL, cmd_export,
		"Export a repository version to a non-version-controlled directory", "OPTIONS REPOSITORY [path]", NULL,
		SG_FALSE, 
		{ {OPT_REV_SPEC,0,1}/* See: W6233 , {opt_no_submodules,0,0}, {opt_use_submodule,0,0} */}, 
		{ {0, ""} }
	},

	{
		"clone", NULL, NULL, NULL, cmd_clone,
		"Create a new instance of an existing repository", "existing_repo_name new_repo_name", NULL,
		SG_FALSE, 
		{
			{opt_all_full,0,0}, {opt_all_zlib,0,0}, {opt_pack,0,0} 
		},
		{ {0, ""} }
	},

	{
		"config", "localsettings", "localsetting", NULL, cmd_localsettings,
		"View and manipulate configuration settings", "[subcommand [...]]", &vv_help_config,
		SG_FALSE, 
		{ {'F',0,0}, {opt_verbose,0,0} },
		{
			{'F', "Allow specification of settings in unknown scopes"},
			{opt_verbose, "Include factory default settings in output"}
		}
	},

	{
		"comment", NULL, NULL, NULL, cmd_comment,
		"Add a comment to a changeset", "(--rev=CHANGESET | --tag=TAG) [--message=message]", NULL,
		SG_FALSE, 
		{ {OPT_REV_SPEC,1,1}, {'m',0,1} }, 
		{ {0, ""} }
	},

	{
		"commit", NULL, NULL, NULL, cmd_commit,
		"Commit the pending changeset to the repository", "[local paths of things to be committed]", &vv_help_commit,
		SG_FALSE, 
		{ 
			/*{'U',0,1}, */{'N',0,0}, {'m',0,1}, /* See: Y2908 {'I',0,0}, {'X',0,0}, */ {opt_detached,0,0}, /*{opt_when,0,1}, */
			/* See: Y2908 {opt_include_from,0,0}, {opt_exclude_from,0,0}, */{opt_stamp,0,0}, {'a',0,0}/* See: W6233 , {opt_no_submodules,0,0} */
		},
		{
			{opt_stamp, "Stamp(s) to be added"},
			//{'U', "Specify the user committing these changes. Without this parameter, " WHATSMYNAME " will fall back on the default user"},
		}
	},

	{
		"diff", NULL, NULL, NULL, cmd_diff,
		"Show differences between versions of files or folders", "", NULL,
		SG_FALSE, 
		{
			{OPT_REV_SPEC,0,2}, {opt_tool,0,1}, {opt_unified,0,1}, {'N',0,0}, /* See: Y2908 {'I',0,0}, {'X',0,0}, 
			{opt_include_from,0,0}, {opt_exclude_from,0,0}, */{opt_no_ignores,0,0}
		},
		{ {opt_tool, "Specify a tool to use to perform the diff"} }
	},

	{
		"diffmerge", "sgdm", "dm", NULL, cmd_diffmerge,
		"Diff from root using SourceGear DiffMerge", "[(--rev=FROM_REV | --tag=FROM_TAG) [(--rev=TO_REV | --tag=TO_TAG)]]", NULL,
		SG_FALSE, 
		{ {OPT_REV_SPEC,0,2}/* See: W6233 , {opt_no_submodules,0,0}, {opt_use_submodule,0,0}*/ },
		{ {0, ""} }
	},

	{
		"dump_json", NULL, NULL, NULL, cmd_dump_json,
		"Write a repository blob to stdout in JSON format", "blobid", NULL,
		SG_TRUE, 
		{ {opt_repo,0,1} },
		{ {0, ""} }
	},

	{
		"debug_walk_fulltree", NULL, NULL, NULL, cmd_walk_fulltree,
		"walk the entire tree, without printing anything.", NULL, NULL,
		SG_TRUE,
		{ {opt_repo,0,1} }, 
		{ {0, ""} }
	},
#ifdef debug
	{
		"debug_manually_set_parents", NULL, NULL, NULL, cmd_manually_set_parents,
		"explicitly set the parents that will be used as the baseline.  This is mostly useful for importers.", NULL, NULL,
		SG_TRUE, 
		{ {OPT_REV_SPEC,1,0} }, 
		{ {0, ""} }
	},
#endif
	{
		"zingmerge", NULL, NULL, NULL, cmd_zingmerge,
		"Attempt automatic merge of all zing dags", "[--repo=REPOSITORY]", NULL,
		SG_TRUE, 
		{ {opt_repo,0,1} }, 
		{ {0, ""} }
	},

	{
		"reindex", NULL, NULL, NULL, cmd_reindex,
		"Rebuild indexes", "[--repo=REPOSITORY]", NULL,
		SG_TRUE, 
		{ {opt_repo,0,1} }, 
		{ {0, ""} }
	},

	{
		"dump_lca", NULL, NULL, NULL, cmd_dump_lca,
		"Write the DAG LCA to stdout in graphviz dot format or optionally as a simple list of nodes", "", NULL,
		SG_TRUE, 
		{ {OPT_REV_SPEC,0,0}, {opt_list,0,0}, {opt_verbose,0,0}, {opt_repo,0,1}
#if defined(DEBUG)
				  , {opt_dagnum,0,1}
#endif
		},
		{ {opt_list, "Output a simple list of nodes rather than using graphviz format"} }
	},

#ifdef DEBUG
	{
		"dump_treenode", NULL, NULL, NULL, cmd_dump_treenode,
		"Dump the contents of the treenode for the given changeset", "[changesetid]", NULL,
		SG_TRUE, 
		{ {opt_repo,0,1} },
		{ {0, ""} }
	},
#endif

#if 0
	{
		"creategroup", NULL, NULL, NULL, cmd_creategroup,
		"Create a new user group", "[--repo=REPOSITORY] group_name", NULL,
		SG_FALSE, 
		{ {opt_repo,0,1} },
		{ {0, ""} }
	},

	{
		"group", NULL, NULL, NULL, cmd_group,
		"Manage membership of a group", "group_name [ addusers|addsubgroups|removeusers|removesubgroups|list|listdeep] member_name+ ]", NULL,
		SG_FALSE,
		{ {opt_repo,0,1} },
		{ {0, ""} }
	},

	{
		"groups", NULL, NULL, NULL, cmd_groups,
		"List groups", "[--repo=REPOSITORY]", NULL,
		SG_FALSE, 
		{ {opt_repo,0,1} }, 
		{ {0, ""} }
	},
#endif

	{
		"heads", "tips", NULL, NULL, cmd_heads,
		"List the heads of all active named branches", "[--repo DESCRIPTORNAME]", NULL,
		SG_FALSE, 
		{ {opt_repo,0,1}, {opt_all,0,0}, {'b',0,0}, {opt_verbose,0,1} },
		{
			{opt_all, "Include closed branch heads"},
			{'b', "Show heads for only the specified branch(es)"},
			{opt_verbose, "Include full text of multi-line comments"},
			{0, ""}
		}
	},

	{
		"help", NULL, NULL, NULL, cmd_help,
		"List supported commands", "[command name]", NULL,
		SG_FALSE, 
		{ {opt_show_all,0,0}, {opt_quiet,0,0} }, 
		{ {0, ""} }
	},

	{
		"hid", NULL, NULL, NULL, cmd_hid,
		"Print the hash of a file", "filename", NULL,
		SG_TRUE , 
		{ {0,0,0} }, 
		{ {0, ""} }
	},

	{
		"history", "log", NULL, NULL, cmd_history,
		"Show the history of a repository, directory, or file", "[file-or-folder]", NULL,
		SG_FALSE, 
		{
			{opt_max, 0, 1},
			{opt_repo, 0, 1}, 
			{OPT_REV_SPEC, 0, 0}, 
			{'U',0,1}, {opt_from,0,1}, {opt_to,0,1}, {opt_leaves,0,0}, {opt_reverse,0,0}, {opt_hide_merges,0,0}, {opt_stamp,0,0},
			{opt_verbose, 0, 1}
		},
		{ {0, ""} }
	},

	{
		"incoming", NULL, NULL, NULL, cmd_incoming,
		"List changesets that would be pulled", "[--dest DESCRIPTORNAME] [SOURCEREPO]", NULL,
		SG_FALSE,
		{
			{opt_dest,0,1} 
#if defined(DEBUG)
			,{opt_trace,0,0}
#endif
			,{opt_verbose,0,0}
		}, 
		{ 
			{opt_dest, "Print results for a pull into the specified repository"}
		}
	},

	{
		"graph_history", NULL, NULL, NULL, cmd_graph_history,
		"Same as the history command, except output in graphviz dot format", "", NULL,
		SG_TRUE, 
		{
			{opt_repo,0,1}, {'b',0,0}, {'r',0,0}, {opt_tag,0,0}, {'U',0,1}, {opt_max,0,1}, {opt_from,0,1}, 
			{opt_to,0,1}, {opt_leaves,0,0}, {opt_reverse,0,0}, {opt_hide_merges,0,0}, {opt_stamp,0,0}}, 
		{
			{opt_stamp, "List only changesets identified with this stamp."},
		}
	},

	{
		"leaves", NULL, NULL, NULL, cmd_leaves,
		"List all leaves of a repository", "[--repo DESCRIPTORNAME]", NULL,
		SG_FALSE,
		{
			{opt_repo,0,1}
#if defined(DEBUG)
			, {opt_dagnum,0,1}
#endif
			, {opt_verbose, 0, 1}
		},
		{ {0, ""} }
	},

	{
		"lock", NULL, NULL, NULL, cmd_lock,
		"Lock a file", "filename [filename [...]] [--server REPOINSTANCE]", NULL,
		SG_FALSE, 
		{ {opt_server,0,1} }, 
		{ {0, ""} }
	},

	{
		"unlock", NULL, NULL, NULL, cmd_unlock,
		"Unlock a file", "filename", NULL,
		SG_FALSE, 
		{ {'F',0,0} }, 
		{ {0, ""} }
	},

	{
		"locks", NULL, NULL, NULL, cmd_locks,
		"Show the status of all locks on the current branch", " [--pull] [--server REPOINSTANCE]", NULL,
		SG_FALSE, 
		{ {opt_pull,0,0}, {opt_server,0,1} },
		{ {0, ""} }
	},

	{
		"merge", NULL, NULL, NULL, cmd_merge,
		"Merge branches", "", NULL,
		SG_FALSE,
		{
			{OPT_REV_SPEC,0,1}, {'T',0,0}, {opt_verbose,0,0}, 
			{opt_tool,0,1}, {opt_no_auto_merge,0,0}/* See: W6233 , {opt_use_submodule,0,0}*/
		},
		{
			{'T', "Do not pend the merge result; display details about the results of the merge"},
			{opt_verbose, "Show extra, low-level status details"},
			{opt_tool, "Select a different (non-default) merge tool"}
		}
	},

	{
		"move", "mv", NULL, NULL, cmd_move,
		"Pend the moving of a file or directory", "item_to_be_moved ... new_directory", NULL,
		SG_FALSE, 
		{ {0,0,0} },
		{ {0, ""} }
	},

	{
		// NOTE: opt_storage has been removed.  It should be added back when we have more than one storage technology available.
		"init", "new", "init_new_repo", NULL, cmd_init_new_repo,
		"Create a new repository and optionally create a working copy", "OPTIONS repo_name folder", NULL,
		SG_FALSE,
		{ {opt_shared_users,0,1}, {opt_hash,0,1}/* See: W6233 , {opt_submodule,0,0} */},
		{ {0, ""} }
	},

	{
		"outgoing", NULL, NULL, NULL, cmd_outgoing,
		"List changesets that would be pushed", "[--src DESCRIPTORNAME] [DESTREPO]", NULL,
		SG_FALSE,
		{ 
			{opt_src,0,1}
#if defined(DEBUG)
			,{opt_trace,0,1}
#endif
			, {opt_verbose, 0, 1}
		}, 
		{ 
			{opt_src, "Print results for a push from the specified repository"}
		}
	},

	{
		"parents", NULL, NULL, NULL, cmd_parents,
		"List the parent changesets of the working copy", "", NULL,
		SG_FALSE,
		{ {OPT_REV_SPEC,0,0}, {opt_verbose,0,1} },
		{ {'b', "Use the changeset(s) identified by the head of this branch"} }
	},

	{
		"pull", NULL, NULL, NULL, cmd_pull,
		"Pull committed changes from another repository instance", "[--update] [--rev CHANGESETID] [--tag TAGID]\n"
		"               [--area AREARG] [--dest DESCRIPTORNAME] [SOURCEREPO]", NULL,
		// TODO 2011/01/27 I added --use-sub to UPDATE to hint for nested checkouts.  Do we need this here too?
		SG_FALSE,
		{ 
			{'u',0,0}, {OPT_REV_SPEC,0,0}, {opt_dest,0,1}, {opt_area,0,0} 
#if defined(DEBUG)
			,{opt_trace,0,0}
#endif
		},
		{
			{'r', "Pull only the specified changeset and its ancestors; may be used more than once"},
			{opt_tag, "Pull only the tagged changeset and its ancestors; may be used more than once"},
			{'b',"Pull only the changeset(s) that are part of this branch"},
			{opt_dest, "Pull into the specified repository"},
			{opt_area, "Pull from an area of repository data; may be specified more than once"}
		}
	},

	{
		"push", NULL, NULL, NULL, cmd_push,
		"Push committed changes to another repository instance", "[--force] [--rev CHANGESETID] [--tag TAGID]\n"
		"               [--area AREARG] [--src DESCRIPTORNAME] [DESTREPO]", NULL,
		SG_FALSE, 
		{ 
			{'F',0,0}, {OPT_REV_SPEC,0,0}, {opt_src,0,1}, {opt_area,0,0} 
#if defined(DEBUG)
			,{opt_trace,0,0}
#endif
		},
		{
			{'F', "Perform the push even if a branch needs to be merged"},
			{'r', "Push only the specified changeset and its ancestors; may be used more than once"},
			{opt_tag, "Push only the tagged changeset and its ancestors; may be used more than once"},
			{'b',"Push only the changeset(s) that are part of this branch"},
			{opt_src, "Push from the specified repository"},
			{opt_area, "Push from an area of repository data; may be specified more than once"}
		}
	},

	{
		"remove", "rm", "delete", "del", cmd_remove,
		"Pend the removal of an item from the current repository", "[local path of things to be removed]", &vv_help_remove,
		SG_FALSE, 
		{ {'T',0,0}, /* See: Y2908 {'I',0,0}, {'X',0,0},  {opt_include_from,0,0}, {opt_exclude_from,0,0}, */{'F',0,0}, {opt_no_backups,0,0} },
		{
			{'T', "Do not pend any removals; display details about removals that would be pended"}
		}
	},

	{
		"rename", "ren", NULL, NULL, cmd_rename,
		"Pend the renaming of an item in the current repository", "item_to_be_renamed new_name", NULL,
		SG_FALSE, { {0,0,0} }, { {0, ""} }
	},

	{
		"repo", "repos", "repository", "repositories", cmd_repo,
		"Create, list, delete, rename, or show details about repositories",
		"[SUBCOMMAND] [...]", &vv_help_repo,
		SG_FALSE,
		{
			{opt_all_but,0,1},
			{opt_no_prompt,0,1},
			{opt_verbose,0,1},
			{opt_shared_users,0,1},
			{opt_hash,0,1}
			// NOTE: opt_storage has been removed.  It should be added back when we have more than one storage technology available.
		}, 
		{ {opt_no_prompt, "Do not prompt to confirm delete (only applicable when using '--all-but') or rename."}, {0, ""} }
	},

	{
		"resolve", NULL, NULL, NULL, cmd_resolve2,
		"Resolve merge conflicts", "[subcommand [...]]", &vv_help_resolve2,
		SG_FALSE,
		{
			{'i',           0, 1},
			{opt_all,       0, 1},
			{opt_overwrite, 0, 1},
			{opt_tool,      0, 1},
			{opt_label,     0, 1},
		},
		{
			{opt_all, "Include all conflicts that meet any provided criteria"},
			{opt_overwrite, "Overwrite previous work"}
		}
	},

	{
		"revert", NULL, NULL, NULL, cmd_revert,
		"Revert changes in the pending changeset", "[local paths of things to be reverted]", NULL,
		SG_FALSE, 
		{
			{'N',0,0}, {'T',0,0}, /* See: Y2908 {'I',0,0}, {'X',0,0}, {opt_include_from,0,0}, {opt_exclude_from,0,0}, */
			{opt_all,0,0}, {opt_verbose,0,0},/* See: W6233 {opt_no_submodules,0,0}, {opt_use_submodule,0,0}, */{opt_no_backups,0,0}
		},
		{
			{'T', "Do not revert pending changes; display details about changes that would be reverted"},
			{opt_verbose,"Show extra, low-level status details"}
		}
	},

	{
		"scan", NULL, NULL, NULL, cmd_scan,
		"Scan the working copy for changed files", "", NULL,
		SG_TRUE,
		{ {0,0,0} },
		{ {0, ""} }
	},

	{
		"serve", "server", NULL, NULL, cmd_serve,
		"Launch veracity in server mode", "", NULL,
		SG_FALSE, 
		{ {'P',0,0}, {opt_port,0,1}/*, {opt_sport,0,1}, {opt_no_plain,0,0}, {opt_cert,0,1}*/ },
		{ {0, ""} }
	},

#if 0
	{
		"slurp", NULL, NULL, NULL, cmd_slurp,
		"Import a fragball file into a repo", "", NULL,
		SG_FALSE, {0}, { {0, ""} }
	},
#endif

	{
		"stamp", "stamps", NULL, NULL, cmd_stamp,
		"Stamp a changeset with a user-specified label", 
		"SUBCOMMAND [STAMPNAME] [--all | {--REVSPEC [...]}] [--repo=REPOSITORY]", &vv_help_stamp,
		SG_FALSE, { {opt_all,0,1}, {opt_repo, 0, 1}, {OPT_REV_SPEC,0,0} }, 
		{
			{opt_all, "Used to remove a stamp from all changesets where it is applied"},
			{'r', "Changeset to which stamp should apply"},
			{'b', "Apply stamp to the head(s) of this branch"},
			{opt_tag, "Existing tag identifying a changeset to which stamp should apply"}
		}
	},

	{
		"status", "st", NULL, NULL, cmd_status,
		"Show the status of the working copy", "", &vv_help_status,
		SG_FALSE, 
		{
			{OPT_REV_SPEC,0,2}, 
			{'N',0,0},
			/* See: Y2908 {'I',0,0},
			{'X',0,0},
			{opt_include_from,0,0},
			{opt_exclude_from,0,0}, */
			{opt_verbose,0,0},
			{opt_no_ignores,0,0}
		}, 
		{ {opt_verbose, "Show extra, low-level status details"} }
	},

	{
		"tag", "tags", "label", "labels", cmd_tag,
		"Tag the specified changeset with a unique, user-specified label",
		"SUBCOMMAND [TAGNAME] [--REVSPEC] [--repo=REPOSITORY]", &vv_help_tag,
		SG_FALSE, 
		{ {opt_repo,0,1}, {OPT_REV_SPEC,0,1} }, 
		{
			{opt_tag, "Existing tag identifying a changeset to which the new tag will apply"},
			{'r', "Changeset to which this tag should apply"},
			{'b', "Apply tag to the head of the specified branch"}
		}
	},

	{
		"update", NULL, NULL, NULL, cmd_update,
		"Update the working copy contents to match a repository version", "", NULL,
		SG_FALSE, 
		{
			{OPT_REV_SPEC,0,1}, {opt_attach,0,1}, {'T',0,0},
			{opt_verbose,0,0}/* See: W6233 , {opt_use_submodule ,0,0}*/
		},
		{
			{'T', "Do not perform the update; display details about changes that would be applied"},
			{opt_verbose, "Show extra, low-level status details"}
		}
	},

	{
		"user", "users", NULL, NULL, cmd_user,
		"Create, rename, or list user accounts", "SUBCOMMAND [...] [--repo=REPOSITORY]", &vv_help_user,
		SG_FALSE,
		{ {opt_repo,0,1} },
		{ {opt_repo, "Manage user accounts belonging to this repository"} }
	},

	{
		"vcdiff", NULL, NULL, NULL, cmd_vcdiff,
		"Encode or decode binary file deltas", "deltify|undeltify basefile targetfile deltafile", NULL,
		SG_TRUE,
		{ {0,0,0} },
		{ {0, ""} }
	},

	{
		"version", NULL, NULL, NULL, cmd_version,
		"Report the current version.", "[server_url]", NULL,
		SG_TRUE,
		{ {0,0,0} },
		{ {0, ""} }
	},

	{
		"whoami", NULL, NULL, NULL, cmd_whoami,
		"Set or print current user account", "[--repo=REPOSITORY] [--create] username", NULL,
		SG_FALSE,
		{ {opt_repo,0,1}, {opt_create, 0, 0} },
		{ {0, ""} }
	},

#if 0
	{
		"fragball", NULL, NULL, NULL, cmd_fragball,
		"Package a changeset as a fragball file", "", NULL,
		SG_FALSE,
		{ {0,0,0} },
		{ {0, ""} }
	},
#endif

	{
		"zip", NULL, NULL, NULL, cmd_zip,
		"Export a repository version to a zip file", 
		"filename.zip", NULL,
		SG_FALSE,
		{ {opt_repo,0,1}, {OPT_REV_SPEC,0,0} },
		{ {0, ""} }
	},

#if defined(HAVE_TEST_CRASH)
	{
		"crash", NULL, NULL, NULL, cmd_crash,
		"Cause a crash", "", NULL,
		SG_TRUE,
		{ {0,0,0} },
		{ {0, ""} }
	},
#endif

};

const SG_uint32 VERBCOUNT = (sizeof(commands) / sizeof(struct cmdinfo));


subcmdinfo subcommands[] =
{
	{
		"config", "export",
		"Export veracity configuration as a JSON object",
		"[filename]",
		NULL,
		{ {opt_verbose,0,0} },
		{
			{0, NULL}
		}
	},
	{
		"config", "import",
		"Import veracity configuration from a JSON document",
		"[filename]",
		NULL,
		{ {0,0,0} },
		{ {0, NULL} }
	},

	{
		"config", "set", 
		"Set a configuration value",
		"(setting) (value)",
		NULL,
		{ {'F',0,0}, {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "reset",
		"Reset a configuration setting to its default value",
		"(setting)",
		NULL,
		{ {'F',0,0}, {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "empty",
		"Remove the contents of a configuration setting",
		"(setting)",
		NULL,
		{ {'F',0,0}, {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "add-to",
		"Add a value to a config setting list",
		"(setting) (value) (value) ...",
		&vv_help_config_add_to,
		{ {'F',0,0}, {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "remove-from",
		"Remove a value from a config setting list",
		"(setting) (value) (value) ...",
		NULL,
		{ {'F',0,0}, {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "show",
		"Print all configuration values at and below the named setting, or all values if setting is omitted",
		"[setting]",
		NULL,
		{ {0,0,0} },
		{ {0, NULL} }
	},
	{
		"config", "defaults",
		"Print default configuration values",
		"",
		NULL,
		{ {0,0,0} },
		{ {0, NULL} }
	},
	{
		"stamp", "add",
		"Add a stamp to a changeset",
		"STAMPNAME [--REVSPEC [...]] [--repo REPOSITORY]",
		&vv_help_stamp_add,
		{ {opt_repo, 0, 1}, {OPT_REV_SPEC,0,0} }, 
		{
			{'r', "Changeset to which this stamp should be added"},
			{'b', "Add the stamp to the head(s) of this branch"},
			{opt_tag, "Existing tag identifying a changeset to which this stamp should be added"}
		}
	},
	{
		"stamp", "remove",
		"Remove a stamp from a changeset",
		"STAMPNAME [--all | {--REVSPEC [...]}] [--repo REPOSITORY]",
		&vv_help_stamp_remove,
		{ {opt_all,0,1}, {opt_repo, 0, 1}, {OPT_REV_SPEC,0,0} }, 
		{
			{opt_all, "Remove stamp from all changesets where it has been applied"},
			{'r', "Changeset from which this stamp should be removed"},
			{'b', "Remove the stamp from the heads(s) of this branch"},
			{opt_tag, "Existing tag identifying a changeset from which this stamp should be removed"}
		}
	},
	{
		"stamp", "list",
		"Print a summary of stamp usage",
		"[STAMPNAME] [--repo REPOSITORY]",
		&vv_help_stamp_list,
		{ {opt_repo, 0, 1} }, 
		{ {0, NULL} }
	},
	{
		"tag", "add",
		"Add a tag to a changeset",
		"NEWTAG [--REVSPEC] [--repo REPOSITORY]",
		&vv_help_tag_add,
		{ {opt_repo, 0, 1}, {OPT_REV_SPEC,0,0} }, 
		{
			{'r', "Changeset to which this tag should be added"},
			{'b', "Add tag to the head of this branch"},
			{opt_tag, "Existing tag identifying a changeset to which this tag should be added"}
		}
	},
	{
		"tag", "move",
		"Move a tag to a different changeset",
		"EXISTINGTAG [--REVSPEC] [--repo REPOSITORY]",
		&vv_help_tag_move,
		{ {opt_repo, 0, 1}, {OPT_REV_SPEC,0,0} }, 
		{
			{'r', "Changeset to which this tag should be moved"},
			{'b', "Move tag to the head of this branch"},
			{opt_tag, "Existing tagged changeset to which this tag should be moved"}
		}
	},
	{
		"tag", "remove",
		"Remove a tag from its associated changeset",
		"EXISTINGTAG [--repo REPOSITORY]",
		&vv_help_tag_remove,
		{ {opt_repo, 0, 1} }, 
		{ {0, NULL} }
	},
	{
		"tag", "list",
		"Print a list of tags and their associated changesets",
		"[--repo REPOSITORY]",
		&vv_help_tag_list,
		{ {opt_repo, 0, 1} }, 
		{ {0, NULL} }
	},
	{
		"resolve", "list",
		"List conflicts in the working copy",
		"[FILENAME]",
		&vv_help_resolve2_list,
		{
			{ opt_all,  0, 1 },
			{ 'i',      0, 1 },
			{ opt_tool, 0, 1 },
		},
		{
			{ opt_all,  "Include already resolved conflicts in the list" },
			{ 'i',      "Interactively prompt for an action to take for each listed conflict" },
			{ opt_tool, "Default mergetool to use for interactive 'merge' commands" },
		}
	},
	{
		"resolve", "next",
		"List the next unresolved conflict in the working copy",
		"[FILENAME]",
		&vv_help_resolve2_next,
		{
			{ 'i',      0, 1 },
			{ opt_tool, 0, 1 },
		},
		{
			{ 'i', "Interactively prompt for an action to take for the listed conflict" },
			{ opt_tool, "Default mergetool to use for interactive 'merge' commands" },
		}
	},
	{
		"resolve", "accept",
		"Resolve a conflict or conflicts by accepting potential resolution values",
		"VALUE [FILENAME] [CONFLICT]",
		&vv_help_resolve2_accept,
		{
			{ opt_all,       0, 1 },
			{ opt_overwrite, 0, 1 },
		},
		{
			{ opt_all,       "Accept the resolution value on all conflicts that meet any provided criteria" },
			{ opt_overwrite, "Allow resolved conflicts to be re-resolved with a different resolution value" },
		}
	},
/*	{
		"resolve", "view",
		"View a value from a conflict in a viewer tool.",
		"VALUE FILENAME [CONFLICT]",
		&vv_help_resolve2_view,
		{
			{ opt_tool, 0, 1 },
		},
		{
			{ opt_tool, "Use a specific viewer tool." },
		}
	},
*/	{
		"resolve", "diff",
		"Diff two resolution values from a conflict",
		"VALUE1 VALUE2 FILENAME [CONFLICT]",
		&vv_help_resolve2_diff,
		{
			{ opt_tool, 0, 1 },
		},
		{
			{ opt_tool, "Use a specific diff tool" },
		}
	},
	{
		"resolve", "merge",
		"Merge two resolution values from a conflict into a new value",
		"[VALUE1 VALUE2] FILENAME [CONFLICT]",
		&vv_help_resolve2_merge,
		{
			{ opt_tool,      0, 1 },
			{ opt_label,     0, 1 },
			{ opt_overwrite, 0, 1 },
		},
		{
			{ opt_tool,      "Use a specific merge tool" },
			{ opt_label,     "Specify a label for the new value instead of using an automatically generated one" },
			{ opt_overwrite, "Allow an established resolution value to be overwritten by a new one; only necessary when specifying a label that already exists" },
		}
	},
	{
		"resolve", "FILENAME",
		"Shortcut for: list --interactive FILENAME",
		"",
		NULL,
		{
			{ opt_all,  0, 1 },
			{ opt_tool, 0, 1 },
		},
		{
			{ opt_all,  "Include already resolved conflicts in the list" },
			{ opt_tool, "Default mergetool to use for interactive 'merge' commands" },
		}
	},
	{
		"repo", "new",
		"Create a new, empty repository (without associating any directory as a\n"
		"        working copy)",
		"NAME [--shared-users=REPOSITORY | --hash=HASHMETHOD]",
		NULL,
		{
			{opt_shared_users,0,1},
			{opt_hash,0,1},
			// NOTE: opt_storage has been removed.  It should be added back when we have more than one storage technology available.
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"repo", "delete",
		"Delete one or more repositories",
		"[--all-but] REPOSITORY [REPOSITORY2 ...]",
		NULL,
		{
			{ opt_all_but, 0, 1 },
			{ opt_no_prompt, 0, 1 },
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"repo", "rename",
		"Rename a repository",
		"OLDNAME NEWNAME",
		NULL,
		{
			{ opt_no_prompt, 0, 1 },
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"repo", "info",
		"Show information about a repository",
		"[REPONAME | DISKPATH]",
		NULL,
		{
			{ opt_verbose, 0, 1 },
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"repo", "list",
		"List available veracity repositories",
		"",
		NULL,
		{
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"user", "create",
		"Create a new user account",
		"USERNAME [--repo REPOSITORY]",
		NULL,
		{
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"user", "rename",
		"Rename an existing user account",
		"OLDUSERNAME NEWUSERNAME [--repo REPOSITORY]",
		NULL,
		{
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	},
	{
		"user", "list",
		"Print a list of existing user accounts",
		"[--repo REPOSITORY]",
		NULL,
		{
			{ 0, 0, 0 }
		},
		{
			{ 0, NULL }
		}
	}
};

const SG_uint32 SUBCMDCOUNT = (sizeof(subcommands) / sizeof(struct subcmdinfo));


static cmdhelpinfo vvBonusHelp[] = 
{
	{ "filetool", "Specify tools used to compare and merge different file types", &vv_help_filetool }
};

const SG_uint32 BONUSHELPCOUNT = (sizeof(vvBonusHelp) / sizeof(cmdhelpinfo));


/* ----------------*/
/* Helper routines */
/* ----------------*/

static void _get_canonical_command(const char *cmd_name,
								   const cmdinfo** pCmdInfo)
{
	SG_uint32 i = 0;
	const cmdinfo *ci = NULL;
	SG_uint32 len = 0;

	if (cmd_name == NULL)
		return;

	for ( i = 0; i < VERBCOUNT; ++i )
	{
		if ((strcmp(cmd_name, commands[i].name) == 0) ||
			(commands[i].alias1 && (strcmp(cmd_name, commands[i].alias1) == 0)) ||
			(commands[i].alias2 && (strcmp(cmd_name, commands[i].alias2) == 0)) ||
			(commands[i].alias3 && (strcmp(cmd_name, commands[i].alias3) == 0))
		   )
		{
			*pCmdInfo = commands + i;
			return;
		}
	}

	len = SG_STRLEN(cmd_name);

	if (len == 0)
		return;

	// look for *exactly one* substring match
	for ( i = 0; i < VERBCOUNT; ++i )
	{
		if ((strncmp(cmd_name, commands[i].name, len) == 0) ||
			(commands[i].alias1 && (strncmp(cmd_name, commands[i].alias1, len) == 0)) ||
			(commands[i].alias2 && (strncmp(cmd_name, commands[i].alias2, len) == 0)) ||
			(commands[i].alias3 && (strncmp(cmd_name, commands[i].alias3, len) == 0))
		   )
		{
			if (ci != NULL)   // multiple matches, bail
				return;

			ci = commands + i;
		}
	}

	if (ci)
		*pCmdInfo = ci;

	// TODO Review: so what happens here?  We return nothing or NULL?
	// TODO         is this case even possible? (have we already gone
	// TODO         a parser that did validate the name?)

  /* If we get here, there was no matching command name or alias. */
	return;
}

static void _parse_and_count_args(
	SG_context *pCtx,
	SG_getopt* os,
	const char*** ppaszArgs,
	SG_uint32 expectedCount,
	SG_bool *pbUsageError)
{
	SG_uint32	count_args = 0;

	SG_ERR_CHECK_RETURN(  SG_getopt__parse_all_args(pCtx, os, ppaszArgs, &count_args)  );

	if (count_args != expectedCount)
		*pbUsageError = SG_TRUE;
}

// callback for SG_qsort
int _command_compare(SG_context * pCtx,
					const void * pVoid_pIndex1,
					const void * pVoid_pIndex2,
					SG_UNUSED_PARAM(void * pVoidCallerData))
{
	const SG_uint32* pIndex1 = (const SG_uint32 *) pVoid_pIndex1;
	const SG_uint32* pIndex2 = (const SG_uint32 *) pVoid_pIndex2;
	SG_UNUSED(pCtx);
	SG_UNUSED(pVoidCallerData);

	return strcmp( commands[*pIndex1].name, commands[*pIndex2].name );
}

static void do_cmd_help__list_bonus_help(SG_context *pCtx, SG_console_stream cs, const char *topic, SG_bool *pbListed)
{
	SG_uint32 i;

	SG_NULLARGCHECK_RETURN(pbListed);
	SG_NULLARGCHECK_RETURN(topic);

	*pbListed = SG_FALSE;

	for ( i = 0; i < BONUSHELPCOUNT; ++i )
	{
		cmdhelpinfo *help = &vvBonusHelp[i];

		if (strcmp(help->name, topic) == 0)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, cs, "%s\n\n%s", help->desc, *help->helptext)  );

			*pbListed = SG_TRUE;
			break;
		}
	}
}


static void do_cmd_help__list_all_commands(SG_context * pCtx, SG_console_stream cs, const char *pszAppName, SG_option_state * pOptSt)
{
	SG_uint32 commandIndex[(sizeof(commands) / sizeof(struct cmdinfo))];
	SG_uint32 i;
	SG_uint32 commandWidth = 0;
	SG_uint32 consoleWidth;
	SG_bool bVerbose = SG_FALSE;
	SG_bool bQuiet = SG_FALSE;
	
	consoleWidth = SG_console__get_width();
	
	if (pOptSt)
	{
		bVerbose = pOptSt->bShowAll;
		bQuiet = pOptSt->bQuiet;
	}
	
	if (!bQuiet)
		SG_ERR_IGNORE(  SG_console(pCtx, cs, "Usage: %s command [options]\n\nCommands:\n\n", pszAppName)  );
	for (i=0; i<VERBCOUNT; i++)
	{
		commandIndex[i] = i;
		if ((bVerbose || (! commands[i].advanced)) && (SG_STRLEN(commands[i].name) > commandWidth))
		{
			commandWidth = SG_STRLEN(commands[i].name);
		}
	}

	if (bVerbose)
		for ( i = 0; i < BONUSHELPCOUNT; ++i )
			if (SG_STRLEN(vvBonusHelp[i].name) > commandWidth)
				commandWidth = SG_STRLEN(vvBonusHelp[i].name);

	SG_qsort(pCtx, commandIndex, VERBCOUNT, sizeof(commandIndex[0]),
			 _command_compare, NULL);

	for (i=0; i<VERBCOUNT; i++)
	{
		if (bVerbose || (! commands[commandIndex[i]].advanced))
		{
			SG_getopt__pretty_print(pCtx, cs, commands[commandIndex[i]].name, " ", commands[commandIndex[i]].desc, commandWidth, consoleWidth);
		}
	}

	if (bVerbose)
	{
		if (!bQuiet)
			SG_getopt__pretty_print(pCtx, cs, "", "", "\nAdditional Help topics:\n", 0, consoleWidth);

		for ( i = 0; i < BONUSHELPCOUNT; ++i )
			SG_getopt__pretty_print(pCtx, cs, vvBonusHelp[i].name, " ", vvBonusHelp[i].desc, commandWidth, consoleWidth);
	}
	
	if (!bQuiet)
		SG_getopt__pretty_print(pCtx, cs, "", "", "\ninput \"" WHATSMYNAME " help command\" for details on a particular command or topic\n", 0, consoleWidth);
}

static void _print_option_help(SG_context* pCtx, const cmdinfo* thiscmd, SG_int32 optch, SG_console_stream cs) 
{
	const char* dest_override = NULL;
	SG_uint32 idx;
	SG_uint32 k;

	for (k = 0; k<(sizeof(sg_cl_options) / sizeof(struct sg_getopt_option)); k++)
	{
		if (optch == sg_cl_options[k].optch)
		{
			SG_getopt_option* pOpt = &sg_cl_options[k];
			for (idx = 0; (idx < SG_CMDINFO_OPTION_LIMIT) && (thiscmd->desc_overrides[idx]).optch; idx++)
			{
				if ((thiscmd->desc_overrides[idx]).optch == pOpt->optch)
				{
					dest_override = (thiscmd->desc_overrides[idx]).desc;
					break;
				}
			}
			SG_ERR_IGNORE(  SG_getopt__print_option(pCtx, cs, pOpt, dest_override)  );
			break;
		}
	}
}


static void _print_suboption_help(SG_context* pCtx, const cmdinfo* thiscmd, const subcmdinfo *thissub, SG_int32 optch, SG_console_stream cs) 
{
	const char* dest_override = NULL;
	SG_uint32 idx;
	SG_uint32 k;

	for (k = 0; k<(sizeof(sg_cl_options) / sizeof(struct sg_getopt_option)); k++)
	{
		if (optch == sg_cl_options[k].optch)
		{
			SG_getopt_option* pOpt = &sg_cl_options[k];
			for (idx = 0; (idx < SG_CMDINFO_OPTION_LIMIT) && (thissub->desc_overrides[idx]).optch; idx++)
			{
				if ((thissub->desc_overrides[idx]).optch == pOpt->optch)
				{
					dest_override = (thissub->desc_overrides[idx]).desc;
					break;
				}
			}

			if (dest_override)
				SG_ERR_IGNORE(  SG_getopt__print_option(pCtx, cs, pOpt, dest_override)  );
			else
				SG_ERR_IGNORE(  _print_option_help(pCtx, thiscmd, optch, cs)  );

			break;
		}
	}
}


static SG_bool _anySubcommands(const cmdinfo *cmd, const char *subname)
{
	SG_uint32 i;

	for ( i = 0; i < SUBCMDCOUNT; ++i )
		if (strcmp(cmd->name, subcommands[i].cmdname) == 0)
			if ((! subname) || (strcmp(subcommands[i].name, subname) == 0))
				return(SG_TRUE);

	return(SG_FALSE);
}

static void _list_subcommands(
	SG_context *pCtx, 
	SG_console_stream cs,
	const cmdinfo *cmd)
{
	if (_anySubcommands(cmd, NULL))
	{
		SG_uint32 i;

		SG_ERR_IGNORE(  SG_console(pCtx, cs, "\nSubcommands:\n------------\n")  );

		for ( i = 0; i < SUBCMDCOUNT; ++i )
			if (strcmp(cmd->name, subcommands[i].cmdname) == 0)
			{
				const subcmdinfo *sci = &subcommands[i];

				SG_ERR_IGNORE(  SG_console(pCtx, cs, "    %s %s\n        %s\n", sci->name, sci->extraHelp, sci->desc)  );
			}
	}
}

static void do_cmd_help__list_subcommand_help(
	SG_context * pCtx, 
	SG_console_stream cs,
	const char * szAppName,
	const cmdinfo *cmd,
	const char * szSubcommandName)
{
	SG_uint32 i = 0;
	SG_uint32 consoleWidth;
	const subcmdinfo *thissub = NULL;

	consoleWidth = SG_console__get_width();

	for ( i = 0; i < SUBCMDCOUNT; ++i )
		if ((strcmp(cmd->name, subcommands[i].cmdname) == 0) && (strcmp(subcommands[i].name, szSubcommandName) == 0))
		{
			thissub = &subcommands[i];
			break;
		}

	if (thissub)
	{
		SG_uint32 j = 0;

		SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n")  );

		SG_getopt__pretty_print(pCtx, cs, "", "", thissub->desc, 0, consoleWidth);

		SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n")  );

		SG_ERR_IGNORE(  SG_console(pCtx, cs, "Usage: %s %s %s %s\n", szAppName, cmd->name, thissub->name, thissub->extraHelp)  );

		if (thissub->valid_options[0].option)
			SG_ERR_IGNORE(  SG_console(pCtx, cs, "\nPossible options:\n")  );
		while(thissub->valid_options[j].option)
		{
			if (thissub->valid_options[j].option == OPT_REV_SPEC)
			{
				SG_ERR_IGNORE(  _print_suboption_help(pCtx, cmd, thissub, 'r', cs)  );
				SG_ERR_IGNORE(  _print_suboption_help(pCtx, cmd, thissub, opt_tag, cs)  );
				SG_ERR_IGNORE(  _print_suboption_help(pCtx, cmd, thissub, 'b', cs)  );
			}
			SG_ERR_IGNORE(  _print_suboption_help(pCtx, cmd, thissub, thissub->valid_options[j].option, cs)  );
			j++;
		}

		if (thissub->extendedHelp && *(thissub->extendedHelp))
			SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n%s", *(thissub->extendedHelp))  );
	}
	else
	{
		SG_ERR_IGNORE(  SG_console(pCtx, cs, "Unknown subcommand: %s %s\n\n", cmd->name, szSubcommandName)  );
		SG_ERR_THROW_RETURN(  SG_ERR_USAGE  );
	}
}


static void do_cmd_help__list_command_help(
	SG_context * pCtx, 
	SG_console_stream cs,
	const char * szAppName, 
	const char * szCommandName, 
	const char * szSubcommandName,
	SG_bool showCommandName)
{
	SG_uint32 j = 0;
	const cmdinfo *thiscmd = NULL;
	SG_uint32 consoleWidth;

	_get_canonical_command(szCommandName, &thiscmd);

	if (szSubcommandName && _anySubcommands(thiscmd, szSubcommandName))
	{
		do_cmd_help__list_subcommand_help(pCtx, cs, szAppName, thiscmd, szSubcommandName);
		return;
	}

	consoleWidth = SG_console__get_width();

	if (thiscmd)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n")  );

		if (showCommandName)
		{
			SG_getopt__pretty_print(pCtx, cs, thiscmd->name, ": ", thiscmd->desc, 0, consoleWidth);
		}
		else
		{
			SG_getopt__pretty_print(pCtx, cs, "", "", thiscmd->desc, 0, consoleWidth);
		}
		SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n")  );

		SG_ERR_IGNORE(  SG_console(pCtx, cs, "Usage: %s %s %s\n", szAppName, thiscmd->name, thiscmd->extraHelp)  );

		if (thiscmd->valid_options[0].option)
			SG_ERR_IGNORE(  SG_console(pCtx, cs, "\nPossible options:\n")  );
		while(thiscmd->valid_options[j].option)
		{
			if (thiscmd->valid_options[j].option == OPT_REV_SPEC)
			{
				SG_ERR_IGNORE(  _print_option_help(pCtx, thiscmd, 'r', cs)  );
				SG_ERR_IGNORE(  _print_option_help(pCtx, thiscmd, opt_tag, cs)  );
				SG_ERR_IGNORE(  _print_option_help(pCtx, thiscmd, 'b', cs)  );
			}
			SG_ERR_IGNORE(  _print_option_help(pCtx, thiscmd, thiscmd->valid_options[j].option, cs)  );
			j++;
		}

		SG_ERR_IGNORE(  _list_subcommands(pCtx, cs, thiscmd)  );

		if (thiscmd->extendedHelp && *(thiscmd->extendedHelp))
			SG_ERR_IGNORE(  SG_console(pCtx, cs, "\n%s", *(thiscmd->extendedHelp))  );
	}
	else
	{
		SG_ERR_IGNORE(  SG_console(pCtx, cs, "Unknown command: %s\n\n", szCommandName)  );
		SG_ERR_THROW_RETURN(  SG_ERR_USAGE  );
	}
}

//////////////////////////////////////////////////////////////////

void sg_report_portability_warnings2(SG_context * pCtx, const SG_varray * pva)
{
    SG_vhash* pvh = NULL;
    SG_string* pstr = NULL;
    SG_uint32 count;
    SG_uint32 i;

	if (!pva)
		return;

    SG_ERR_CHECK(  SG_varray__count(pCtx, pva, &count)  );

    /* TODO dumping these warnings as serialized JSON is a little unfriendly,
     * but it'll do for now. */

    SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
    for (i=0; i<count; i++)
    {
        SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva, i, &pvh)  );
        SG_ERR_CHECK(  SG_string__clear(pCtx, pstr)  );
        SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
        SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR ,"%s\n", SG_string__sz(pstr))  );
    }
    SG_STRING_NULLFREE(pCtx, pstr);

    return;

fail:
    SG_STRING_NULLFREE(pCtx, pstr);
    return;
}

void sg_report_portability_warnings(SG_context * pCtx, SG_pendingtree* pPendingTree)
{
    SG_varray* pva = NULL;		// we do not own this

    SG_ERR_CHECK_RETURN(  SG_pendingtree__get_warnings(pCtx, pPendingTree, &pva)  );
	if (pva)
		SG_ERR_CHECK_RETURN(  sg_report_portability_warnings2(pCtx, pva)  );
}

//////////////////////////////////////////////////////////////////

/**
 * Dump the "ActionLog" to the console.  This is for things that
 * requested "--test" or "--verbose".
 *
 * TODO Pretty this up; dumping JSON to the terminal is stupid.
 */
void sg_report_action_log2(SG_context * pCtx, const SG_varray * pva)
{
    SG_vhash* pvh = NULL;				// we DO NOT own this
    SG_string* pstr = NULL;
    SG_uint32 count;
    SG_uint32 i;

	if (pva)
	{
		SG_ERR_CHECK(  SG_varray__count(pCtx, pva, &count)  );

		SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
		for (i=0; i<count; i++)
		{
			SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva, i, &pvh)  );
			SG_ERR_CHECK(  SG_string__clear(pCtx, pstr)  );
			SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT ,"%s\n", SG_string__sz(pstr))  );
		}
	}

fail:
    SG_STRING_NULLFREE(pCtx, pstr);
}

void sg_report_action_log(SG_context * pCtx, SG_pendingtree* pPendingTree)
{
    SG_varray* pva = NULL;				// we DO NOT own this

    SG_ERR_CHECK_RETURN(  SG_pendingtree__get_action_log(pCtx, pPendingTree, &pva)  );
	if (pva)
		SG_ERR_CHECK_RETURN(  sg_report_action_log2(pCtx, pva)  );
}
	
//////////////////////////////////////////////////////////////////

/**
 * Dump the "missing details" array from an UPDATE or CHECKOUT
 * that couldn't start because of a submodule problem.
 *
 * TODO Pretty this up; dumping JSON to the terminal is stupid.
 */
void sg_report_submodule_missing_details(SG_context * pCtx, const SG_varray * pvaMissingDetails)
{
    SG_vhash* pvh = NULL;				// we DO NOT own this
    SG_string* pstr = NULL;
    SG_uint32 count;
    SG_uint32 i;

	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );

	SG_ERR_CHECK(  SG_varray__count(pCtx, pvaMissingDetails, &count)  );
	for (i=0; i<count; i++)
	{
		SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pvaMissingDetails, i, &pvh)  );
		SG_ERR_CHECK(  SG_string__clear(pCtx, pstr)  );
		SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT ,"%s\n", SG_string__sz(pstr))  );
	}

fail:
    SG_STRING_NULLFREE(pCtx, pstr);
}

//////////////////////////////////////////////////////////////////

/* TODO we should ask ourselves if we need portability warnings on tag strings.
 * For now, I'm going to assume no. */


static SG_bool _get_option_index(const cmdinfo* pCmdInfo, SG_int32 iOptionCode, SG_uint32* piIdx)
{
	SG_uint32 i;

	for (i = 0; i < SG_NrElements(pCmdInfo->valid_options); i++)
	{
		if (pCmdInfo->valid_options[i].option == iOptionCode)
		{
			if (piIdx)
				*piIdx = i;
			return SG_TRUE;
		}
		else if (pCmdInfo->valid_options[i].option == 0) // no more valid options
			break;
	}

	return SG_FALSE;
}

static void _print_invalid_option(SG_context* pCtx, const char* pszAppName, const char* pszCommandName, SG_int32 iInvalidOpt)
{
	// TODO fix this to print a nicer message.
	SG_console(pCtx, SG_CS_STDERR, "\nInvalid option for '%s %s': ", pszAppName, pszCommandName);
	SG_getopt__print_invalid_option(pCtx, iInvalidOpt, sg_cl_options);
	SG_console(pCtx, SG_CS_STDERR, "\n");
}

static void _print_too_many(SG_context* pCtx, SG_int32 iInvalidOpt, SG_uint32 min, SG_uint32 max)
{
	char* pszOptionAsProvided = NULL;
	char* pszRefOptionDesc = NULL;

	if (iInvalidOpt != OPT_REV_SPEC)
	{
		SG_ERR_CHECK(  SG_getopt__get_option_as_provided(pCtx, iInvalidOpt, sg_cl_options, &pszOptionAsProvided)  );
		pszRefOptionDesc = pszOptionAsProvided;
	}
	else
	{
		pszRefOptionDesc = OPT_REV_SPEC_DESCRIPTION;
	}

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "\n")  );

	if (max == 1)
	{
		if (min)
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' must be specified.\n", pszRefOptionDesc)  );
		else
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' can only be specified once.\n", pszRefOptionDesc)  );
	}
	else
	{
		if (!min)
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' can be specified up to %u times.\n", pszRefOptionDesc, max)  );
		else
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' must be specified between %u and %u times.\n", pszRefOptionDesc, min, max)  );
	}

	/* common cleanup */
fail:
	SG_NULLFREE(pCtx, pszOptionAsProvided);
}

static void _print_too_few(SG_context* pCtx, SG_int32 iInvalidOpt, SG_uint32 min, SG_uint32 max)
{
	char* pszOptionAsProvided = NULL;
	const char* pszRefOptionDesc = NULL;

	if (iInvalidOpt != OPT_REV_SPEC)
	{
		SG_ERR_CHECK(  SG_getopt__get_option_as_provided(pCtx, iInvalidOpt, sg_cl_options, &pszOptionAsProvided)  );
		pszRefOptionDesc = pszOptionAsProvided;
	}
	else
	{
		pszRefOptionDesc = OPT_REV_SPEC_DESCRIPTION;
	}

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "\n")  );

	if (max == 0)
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' must be specified at least %u times.\n", pszRefOptionDesc, min)  );
	else if (max == 1)
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' must be specified.\n", pszRefOptionDesc)  );
	else 
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' must be specified between %u and %u times.\n", pszRefOptionDesc, min, max)  );

	/* common cleanup */
fail:
	SG_NULLFREE(pCtx, pszOptionAsProvided);
}

static void _print_invalid_option_combination(SG_context * pCtx, const char * pszAppName, const char * pszCommandName,
											  SG_int32 iInvalidOpt_1,
											  SG_int32 iInvalidOpt_2)
{
	// TODO fix this to print a nicer message.
	SG_console(pCtx, SG_CS_STDERR, "\nInvalid option combination for '%s %s':\n", pszAppName, pszCommandName);
	SG_getopt__print_invalid_option(pCtx, iInvalidOpt_1, sg_cl_options);
	SG_getopt__print_invalid_option(pCtx, iInvalidOpt_2, sg_cl_options);
}


/**
 * Look at all of the -f or --flags given on the command line
 * and verify each is defined for this command.
 *
 * Print error message for each invalid flag and return bUsageError.
 * (We don't throw this because we are only part of the parse and
 * validation.  We want the caller to be able to use the existing
 * INVOKE() macro and be consistent with the other CMD_FUNCs (at
 * least for now).
 *
 * TODO convert all CMD_FUNCs that have a loop like the one in this
 * TODO function to use this function instead.
 */
static void _inspect_all_options(SG_context * pCtx,
								 const char * pszAppName,
								 const char * pszCommandName,
								 SG_option_state * pOptSt,
								 SG_bool * pbUsageError)
{
	const cmdinfo * pThisCmd = NULL;
	SG_bool bUsageError = SG_FALSE;
	SG_uint32 k;
	SG_varray* pvaHandled = NULL;

	_get_canonical_command(pszCommandName, &pThisCmd);
	SG_ASSERT(  (pThisCmd)  );

	SG_VARRAY__ALLOC__PARAMS(pCtx, &pvaHandled, 10, NULL, NULL);

	/* Handle OPT_REV_SPEC, if this command takes one. */
	for (k=0; k < SG_CMDINFO_OPTION_LIMIT; k++) /* Looping over all options this command supports. */
	{
		cmdopt current_option = pThisCmd->valid_options[k];
		if (current_option.option == OPT_REV_SPEC)
		{
			/* If the command supports OPT_REV_SPEC, we handle rev/tag/branch arguments and prevent them from being checked below. */
			
			SG_uint32 numRevSpecsGiven = 0;

			if (pOptSt->psa_revs)
				SG_STRINGARRAY_NULLFREE(pCtx, pOptSt->psa_revs);
			if (pOptSt->psa_tags)
				SG_STRINGARRAY_NULLFREE(pCtx, pOptSt->psa_tags);
			if (pOptSt->psa_branch_names)
				SG_STRINGARRAY_NULLFREE(pCtx, pOptSt->psa_branch_names);

			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &numRevSpecsGiven)  );
			
			if (current_option.min && numRevSpecsGiven < current_option.min)
			{
				bUsageError = SG_TRUE;
				SG_ERR_IGNORE(  _print_too_few(pCtx, current_option.option, current_option.min, current_option.max)  );
			}
			if (current_option.max && numRevSpecsGiven > current_option.max)
			{
				bUsageError = SG_TRUE;
				SG_ERR_IGNORE(  _print_too_many(pCtx, current_option.option, current_option.min, current_option.max)  );
			}

			SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, OPT_REV_SPEC)  );
			SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, 'r')  );
			SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, opt_tag)  );
			SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, 'b')  );

			break;
		}
		if (current_option.option == 0)
			break;
	}

	/* Loop through all provided options */
	for (k=0; k < pOptSt->count_options; k++)
	{
		SG_uint32 idx;
		SG_int32 current_option = pOptSt->paRecvdOpts[k];
		SG_bool bHandled = SG_FALSE;

		SG_ERR_CHECK(  SG_varray__find__int64(pCtx, pvaHandled, current_option, &bHandled, NULL)  );

		if (bHandled)
			continue;

		/* If an option that's not valid for this command is specified, say so. */
		if ( !_get_option_index(pThisCmd, current_option, &idx) )
		{
			SG_ERR_IGNORE(  _print_invalid_option(pCtx, pszAppName, pszCommandName, current_option)  );
			bUsageError = SG_TRUE;
		}
		else
		{
			/* If a non-zero max is specified for the command, verify that the option wasn't specified more than that. */
			if (pThisCmd->valid_options[idx].max && (pOptSt->optCountArray[current_option] > pThisCmd->valid_options[idx].max))
			{
				SG_bool found = SG_FALSE;
				SG_ERR_CHECK(  SG_varray__find__int64(pCtx, pvaHandled, current_option, &found, NULL)  );
				if (!found)
				{
					SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, current_option)  );
					SG_ERR_IGNORE(  _print_too_many(pCtx, current_option, 
						pThisCmd->valid_options[idx].min, pThisCmd->valid_options[idx].max)  );
					bUsageError = SG_TRUE;
				}
			}
		}
	}

	/* Loop through valid options for this command */
	for (k=0; k < SG_CMDINFO_OPTION_LIMIT; k++)
	{
		cmdopt current_option = pThisCmd->valid_options[k];
		if (current_option.option == 0)
			break;

		/* If a non-zero max is specified for the command, verify that the option was specified at least that many times. */
		if ( current_option.min && (pOptSt->optCountArray[current_option.option] < current_option.min) )
		{
			SG_bool found = SG_FALSE;
			SG_ERR_CHECK(  SG_varray__find__int64(pCtx, pvaHandled, current_option.option, &found, NULL)  );
			if (!found)
			{
				SG_uint32 idx;
				SG_ERR_CHECK(  SG_varray__append__int64(pCtx, pvaHandled, current_option.option)  );
				if ( _get_option_index(pThisCmd, current_option.option, &idx) )
				{
					SG_ERR_IGNORE(  _print_too_few(pCtx, current_option.option, 
						pThisCmd->valid_options[idx].min, pThisCmd->valid_options[idx].max)  );
				}
				else
				{
					SG_ASSERT(SG_FALSE);
				}
				bUsageError = SG_TRUE;
			}
		}
	}

	*pbUsageError = bUsageError;

	/* common cleanup */
fail:
	SG_VARRAY_NULLFREE(pCtx, pvaHandled);
}

/**
 * Helper routine for verbs that are valid when zero, one, or two revisions are specified.
 */
static void _get_zero_one_or_two_revision_specifiers(SG_context* pCtx, const char* pszRepoName, SG_rev_spec* pRevSpec, char** ppszHid1, char** ppszHid2)
{
	SG_stringarray* psaFullHids = NULL;

	SG_NULLARGCHECK_RETURN(ppszHid1);
	SG_NULLARGCHECK_RETURN(ppszHid2);

	if (pRevSpec)
	{
		SG_ERR_CHECK(  SG_rev_spec__get_all(pCtx, pszRepoName, pRevSpec, SG_FALSE, &psaFullHids)  );
		if (psaFullHids)
		{
			SG_uint32 count, i;

			SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaFullHids, &count)  );
			for (i = 0; i < count; i++)
			{
				const char* pszRefHid;
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaFullHids, i, &pszRefHid)  );
				if (i == 0)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszRefHid, ppszHid1)  );
				else if (i==1)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszRefHid, ppszHid2)  );
			}
		}
	}

	/* common cleanup */
fail:
	SG_STRINGARRAY_NULLFREE(pCtx, psaFullHids);
}

void my_wrap_text(
        SG_context* pCtx,
        SG_string* pstr,
        SG_uint32 width,
        SG_string** ppstr
        )
{
	char** words = NULL;
    SG_uint32 count_words = 0;
    SG_string* pstr_result = NULL;
    SG_uint32 len_cur_line = 0;
    SG_uint32 i = 0;

	SG_NULLARGCHECK_RETURN(pstr);
	SG_ARGCHECK_RETURN(width > 8, width);

    // TODO 1000 for max substrings?
	SG_ERR_CHECK(  SG_string__split__asciichar(pCtx, pstr, ' ', 1000, &words, &count_words) );

    SG_ERR_CHECK(  SG_string__alloc(pCtx, &pstr_result)  );
    len_cur_line = 0;
    for (i=0; i<count_words; i++)
    {
        SG_uint32 len_cur_word = SG_STRLEN(words[i]);

        if ((len_cur_line + len_cur_word + 1) <= width)
        {
            if (len_cur_line)
            {
                SG_ERR_CHECK(  SG_string__append__format(pCtx, pstr_result, " %s", words[i])  );
            }
            else
            {
                SG_ERR_CHECK(  SG_string__append__format(pCtx, pstr_result, "%s", words[i])  );
            }
            len_cur_line += (1 + len_cur_word);
        }
        else
        {
            SG_ERR_CHECK(  SG_string__append__format(pCtx, pstr_result, "\n%s", words[i])  );
            len_cur_line = (1 + len_cur_word);
        }
    }

    *ppstr = pstr_result;
    pstr_result = NULL;

fail:
    SG_STRING_NULLFREE(pCtx, pstr_result);
	if (words && count_words)
    {
		SG_ERR_IGNORE(  SG_freeStringList(pCtx, (const char ***)&words, count_words)  );
    }
}

void do_cmd_history(
        SG_context * pCtx, 
        SG_bool b_graph,
		const char* pszRepo,
        SG_int32 iCountArgs, 
        const char ** paszArgs, 
		SG_rev_spec* pRevSpec,
        const char* pszUser, 
        const char * pszStamp, 
        const char* pszFromDate, 
        const char* pszToDate, 
        SG_bool bLeaves, 
        SG_uint32 iMaxResults,
		SG_bool bHideMerges,
		SG_bool bReverseOutput,
		SG_bool bVerbose
        )
{
	SG_history_result * pHistResult = NULL;
    SG_int64 nFromDate = 0;
    SG_int64 nToDate = SG_INT64_MAX;
    SG_string* pstr_comment = NULL;
    SG_string* pstr_wrapped = NULL;
	SG_uint32 nResultsSoFar = 0;
	SG_history_token * pHistoryToken = NULL;
	SG_history_token * pNewHistoryToken = NULL;
	SG_vhash* pvhBranchPile = NULL;

	//Jeremy took out the paging of history results, since
	//it wasn't a really big performance win.
#ifdef SG_PAGE_HISTORY_RESULTS_CLC
	SG_uint32 nNumberOfResultsAtATime = 1000;
#endif

	if (pszToDate != NULL)
		SG_ERR_CHECK(  SG_time__parse__informal__local(pCtx, pszToDate, &nToDate, SG_TRUE)  );
	if (pszFromDate != NULL)
		SG_ERR_CHECK(  SG_time__parse__informal__local(pCtx, pszFromDate, &nFromDate, SG_FALSE)  );

#ifdef SG_PAGE_HISTORY_RESULTS_CLC
	if (iMaxResults < nNumberOfResultsAtATime)
		nNumberOfResultsAtATime = iMaxResults;
#endif

	if (pszRepo != NULL)
		SG_ERR_CHECK(  SG_vv_verbs__history__repo(pCtx, pszRepo, iCountArgs,paszArgs, pRevSpec, pszUser, pszStamp, iMaxResults /*nNumberOfResultsAtATime*/, bHideMerges, nFromDate, nToDate, NULL, &pvhBranchPile, &pHistResult, NULL)  );
	else
		SG_ERR_CHECK(  SG_vv_verbs__history__workingFolder(pCtx, NULL, iCountArgs, paszArgs, pRevSpec, pszUser, pszStamp, !bLeaves, iMaxResults /*nNumberOfResultsAtATime*/, bHideMerges, nFromDate, nToDate, NULL, &pvhBranchPile, &pHistResult, NULL /*&pHistoryToken*/)  );

	if (pHistResult != NULL)
	{
#ifdef SG_PAGE_HISTORY_RESULTS_CLC
		if (iMaxResults == -1)
			iMaxResults = SG_INT32_MAX;
#endif
		if (bReverseOutput)
		{
			SG_ERR_CHECK(  SG_history_result__reverse(pCtx, pHistResult)  );
		}
		while(nResultsSoFar < iMaxResults)
		{
			SG_uint32 nHistItemsReturned = 0;
			SG_ERR_CHECK(  SG_history_result__count(  pCtx, pHistResult, &nHistItemsReturned)  );
			nResultsSoFar += nHistItemsReturned;
#ifdef SG_PAGE_HISTORY_RESULTS_CLC
			iMaxResults -= nHistItemsReturned;
#endif
			if (b_graph)
			{
				// this code dumps history in the form of a graphviz "dot" file instead
				// of the usual way.
				//print the information for each
				SG_bool bFound = SG_TRUE;
				const char* psz_cs_hid = NULL;
				SG_uint32 count_parents = 0;
				SG_uint32 nCount = 0;
				SG_uint32 nIndex = 0;
				const char* psz_comment = NULL;
				const char* psz_who = NULL;
				char buf_time_formatted[256];

        SG_console(pCtx, SG_CS_STDOUT, "digraph %s\n", "history");
        SG_console(pCtx, SG_CS_STDOUT, "{\n");
        SG_console(pCtx, SG_CS_STDOUT, "rankdir=BT;\n");

			while (bFound)
			{
				SG_ERR_CHECK(  SG_history_result__get_cshid(pCtx, pHistResult, &psz_cs_hid)  );
				SG_ERR_CHECK(  SG_history_result__get_parent__count(pCtx, pHistResult, &count_parents)  );

				SG_ERR_CHECK(  SG_history_result__get_audit__count(pCtx, pHistResult, &nCount)  );
				if (nCount)
				{
					SG_int64 itime = -1;
					SG_time tmLocal;

					SG_ERR_CHECK(  SG_history_result__get_audit__who(pCtx, pHistResult, nCount-1, &psz_who)  );

					SG_ERR_CHECK(  SG_history_result__get_audit__when(pCtx, pHistResult, nCount-1, &itime)  );
					SG_ERR_CHECK(  SG_time__decode__local(pCtx, itime, &tmLocal)  );
					SG_ERR_CHECK(  SG_sprintf(pCtx, 
								buf_time_formatted, 
								sizeof(buf_time_formatted),
								"%4d-%02d-%02d", 
								tmLocal.year,
								tmLocal.month,
								tmLocal.mday
								)  );
				}

				SG_ERR_CHECK(  SG_history_result__get_first_comment__text(pCtx, pHistResult, &psz_comment)  );

				SG_ERR_CHECK(  SG_string__alloc__sz(pCtx, &pstr_comment, psz_comment)  );
				SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pstr_comment,
					(const SG_byte*)"\n", 1,
					(const SG_byte*)" ", 1,
					SG_UINT32_MAX, SG_TRUE, NULL)  );
				SG_ERR_CHECK(  my_wrap_text(pCtx, pstr_comment, 40, &pstr_wrapped)  );
				SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pstr_wrapped,
					(const SG_byte*)"\n", 1,
					(const SG_byte*)"\\n", 2,
					SG_UINT32_MAX, SG_TRUE, NULL)  );
				SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pstr_wrapped,
					(const SG_byte*)"\\\"", 2,
					(const SG_byte*)"\"", 1,
					SG_UINT32_MAX, SG_TRUE, NULL)  );
				SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pstr_wrapped,
					(const SG_byte*)"\"", 1,
					(const SG_byte*)"\\\"", 2,
					SG_UINT32_MAX, SG_TRUE, NULL)  );
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "_%s [%s label=\"%s\\n%s\\n%s\"];\n", 
							psz_cs_hid, 
							(count_parents > 1) ? "style=filled, fillcolor=yellow," : "",
							psz_who,
							buf_time_formatted,
							SG_string__sz(pstr_wrapped)
							)  );
				SG_STRING_NULLFREE(pCtx, pstr_comment);
				SG_STRING_NULLFREE(pCtx, pstr_wrapped);

				for (nIndex = 0; nIndex < count_parents; nIndex++)
				{
					const char * pszParent = NULL;
					SG_ERR_CHECK(  SG_history_result__get_parent(pCtx, pHistResult, nIndex, &pszParent, NULL)  );
					SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "_%s -> _%s\n", psz_cs_hid, pszParent)  );
				}

					SG_ERR_CHECK(  SG_history_result__next(pCtx, pHistResult, &bFound)  );
				}
				SG_console(pCtx, SG_CS_STDOUT, "}\n");
			}
			else
			{
				SG_ERR_CHECK(  SG_cmd_util__dump_history_results(pCtx, SG_CS_STDOUT, pHistResult, pvhBranchPile, SG_FALSE, bVerbose, SG_FALSE) );
			}
#ifdef SG_PAGE_HISTORY_RESULTS_CLC
			if (nResultsSoFar < iMaxResults && pHistoryToken != NULL)
			{
				if (iMaxResults < nNumberOfResultsAtATime)
					nNumberOfResultsAtATime = iMaxResults;
				SG_HISTORY_RESULT_NULLFREE(pCtx, pHistResult);
				if (pszRepo != NULL)
					SG_ERR_CHECK(  SG_vv_verbs__history__fetch_more__repo(pCtx, pszRepo, pHistoryToken, nNumberOfResultsAtATime, &pHistResult, &pHistoryToken)  );
				else
					SG_ERR_CHECK(  SG_vv_verbs__history__fetch_more__working_folder(pCtx, pszRepo, pHistoryToken, nNumberOfResultsAtATime, &pHistResult, &pNewHistoryToken)  );
				SG_HISTORY_TOKEN_NULLFREE(pCtx, pHistoryToken);
				pHistoryToken = pNewHistoryToken;
				pNewHistoryToken = NULL;
			}
			else
#endif
				break;
		}

	}
fail:
	SG_STRING_NULLFREE(pCtx, pstr_wrapped);
	SG_STRING_NULLFREE(pCtx, pstr_comment);
	SG_VHASH_NULLFREE(pCtx, pvhBranchPile);
	SG_HISTORY_RESULT_NULLFREE(pCtx, pHistResult);
	SG_HISTORY_TOKEN_NULLFREE(pCtx, pHistoryToken);
	SG_HISTORY_TOKEN_NULLFREE(pCtx, pNewHistoryToken);
}

void _vv__delete_repo(SG_context * pCtx, const char * szRepoName)
{
    SG_repo * pRepo = NULL;

	SG_ASSERT(pCtx!=NULL);
	SG_NULLARGCHECK_RETURN(szRepoName);

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Deleting %s...", szRepoName)  );

    SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepoName, &pRepo)  );
    SG_ERR_CHECK(  SG_repo__delete_repo_instance(pCtx, &pRepo)  );

    SG_ERR_CHECK(  SG_closet__descriptors__remove(pCtx, szRepoName)  );

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, " Done.\n")  );

	return;
fail:
    SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, " An error occurred deleting this repo.\n")  );

	SG_REPO_NULLFREE(pCtx, pRepo);
}

void do_cmd_blobcount(SG_context * pCtx, const char* psz_descriptor_name);

void do_cmd_repo(
	SG_context * pCtx,
	SG_bool plural,
	SG_uint32 count_args, const char ** paszArgs,
	SG_bool allBut, SG_bool noPrompt,
	SG_bool verbose,
	const char * pszStorage, const char * pszHashMethod, const char * psz_shared_users,
	int * pExitStatus)
{
	SG_uint32 i;
	SG_stringarray * pHitList = NULL;
	SG_vhash * pDescriptors = NULL;
	SG_string * pStr = NULL;
	SG_string * pDescriptorName = NULL;
	SG_pathname * pPath = NULL;
	SG_pathname * pTopPath = NULL;

	SG_vhash* pvhDescriptor = NULL;
	SG_varray* pva = NULL;

	SG_repo * pRepo = NULL;
	char * sz = NULL;

	SG_ASSERT(pCtx!=NULL);

	if(count_args>=2 && strcmp(paszArgs[0],"delete")==0)
	{
		SG_bool proceed = SG_TRUE;

		if(verbose || pszStorage || pszHashMethod || psz_shared_users)
			SG_ERR_THROW(SG_ERR_USAGE);

		count_args-=1;
		paszArgs+=1;

		SG_ERR_CHECK(  SG_STRINGARRAY__ALLOC(pCtx, &pHitList, 1)  );

		// Validate all repo names
		for(i=0; i<count_args; ++i)
		{
			SG_vhash * pTemp = NULL;
			SG_ERR_CHECK(  SG_closet__descriptors__get(pCtx, paszArgs[i], &pTemp)  );
			SG_VHASH_NULLFREE(pCtx, pTemp);
		}

		if(!allBut)
		{
			for(i=0; i<count_args; ++i)
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pHitList, paszArgs[i])  );
		}
		else
		{
			SG_uint32 repo_count;
			SG_uint32 delete_count = 0;

			if(!noPrompt)
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Repositories:\n")  );

			SG_ERR_CHECK(  SG_closet__descriptors__list(pCtx, &pDescriptors)  );
			SG_ERR_CHECK(  SG_vhash__count(pCtx, pDescriptors, &repo_count)  );
			for (i=0; i<repo_count; ++i) {
				SG_uint32 j;
				const char * szRepoName = NULL;
				SG_bool found_in_exception_list = SG_FALSE;
				SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pDescriptors, i, &szRepoName, NULL)  );
				for (j=0; (j<count_args)&&(!found_in_exception_list); ++j)
					found_in_exception_list = (strcmp(szRepoName, paszArgs[j])==0);
				if(!found_in_exception_list)
				{
					SG_ERR_CHECK(  SG_stringarray__add(pCtx, pHitList, szRepoName)  );
					++delete_count;
					if(!noPrompt)
						SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "     %s\n", szRepoName)  );
				}
			}
			SG_VHASH_NULLFREE(pCtx, pDescriptors);

			if(!noPrompt && delete_count>0)
			{
				if(delete_count==1)
					SG_ERR_CHECK(  SG_console__ask_question__yn(pCtx, "Delete this repository?", SG_QA_DEFAULT_NO, &proceed)  );
				else
				{
					SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pStr)  );
					SG_ERR_CHECK(  SG_string__append__format(pCtx, pStr, "Delete these %d repositories?", delete_count)  );
					SG_ERR_CHECK(  SG_console__ask_question__yn(pCtx, SG_string__sz(pStr), SG_QA_DEFAULT_NO, &proceed)  );
					SG_STRING_NULLFREE(pCtx, pStr);
				}
			}
			else if(!noPrompt)
			{
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "     (None)\n")  );
			}
		}

		if(proceed)
		{
			SG_uint32 count;
			const char * const * ppHitList = NULL;
			SG_ERR_CHECK(  SG_stringarray__sz_array_and_count(pCtx, pHitList, &ppHitList, &count)  );
			for (i=0; i<count; ++i)
			{
				_vv__delete_repo(pCtx, ppHitList[i]);
				if(SG_context__has_err(pCtx))
				{
					if(count>1)
					{
						SG_log__report_error__current_error(pCtx);
						SG_context__err_reset(pCtx);
					}
					else
						SG_ERR_RETHROW;
				}
			}
		}
		else
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Aborted. No repositories have been deleted.\n")  );
		}

		SG_STRINGARRAY_NULLFREE(pCtx, pHitList);
	}
	else if((count_args>=1 && strcmp(paszArgs[0],"list")==0) || (count_args==0 && plural))
	{
		SG_uint32 count = 0;

		if(count_args>1 || allBut || noPrompt || verbose || pszStorage || pszHashMethod || psz_shared_users)
			SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "The 'list' subcommand does not take any additional arguments or options."));

		SG_ERR_CHECK(  SG_closet__descriptors__list(pCtx, &pDescriptors)  );
		SG_ERR_CHECK(  SG_vhash__get_keys(pCtx, pDescriptors, &pva)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pva, &count)  );

		for (i=0; i<count; i++)
		{
			const char* pRepoName = NULL;
			SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pva, i, &pRepoName)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", pRepoName)  );
		}

		SG_VARRAY_NULLFREE(pCtx, pva);
		SG_VHASH_NULLFREE(pCtx, pDescriptors);
	}
	else if(count_args==2 && strcmp(paszArgs[0],"new")==0)
	{
		if(allBut || noPrompt || verbose)
			SG_ERR_THROW(SG_ERR_USAGE);

		if(pszHashMethod && psz_shared_users)
			SG_ERR_THROW(SG_ERR_USAGE);

		SG_ERR_CHECK(  SG_vv_verbs__init_new_repo(pCtx, paszArgs[1], NULL, pszStorage, pszHashMethod, SG_TRUE, SG_FALSE, psz_shared_users, NULL, NULL)  );
	}
	else if((count_args>=1 && strcmp(paszArgs[0],"info")==0) || count_args==0)
	{
		const char * pszDescriptorName = NULL;

		if(count_args>2 || allBut || noPrompt || pszStorage || pszHashMethod || psz_shared_users)
			SG_ERR_THROW(SG_ERR_USAGE);

		if(count_args<2)
		{
			SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPath);
			if(SG_context__err_equals(pCtx, SG_ERR_NOT_A_WORKING_COPY))
			{
				SG_context__err_reset(pCtx);
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "You are not currently inside a working copy.\n")  );
				*pExitStatus = 1;
				goto fail;
			}
			SG_ERR_CHECK_CURRENT;

			SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath, &pTopPath, &pDescriptorName, NULL)  );
			pszDescriptorName = SG_string__sz(pDescriptorName);

			SG_PATHNAME_NULLFREE(pCtx, pPath);

			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Current repository:   %s\n", pszDescriptorName)  );
		}
		else
		{
			SG_bool isDescriptor=SG_FALSE;
			SG_ERR_CHECK(  SG_closet__descriptors__exists(pCtx, paszArgs[1], &isDescriptor)  );

			if(isDescriptor)
			{
				pszDescriptorName=paszArgs[1];
				SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pszDescriptorName, &pRepo)  );
			}
			else
			{
				SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath, paszArgs[1])  );
				SG_fsobj__verify_directory_exists_on_disk__pathname(pCtx, pPath);
				if(SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND) || SG_context__err_equals(pCtx, SG_ERR_NOT_A_DIRECTORY))
				{
					SG_context__err_reset(pCtx);
					SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "'%s' is not a valid repository name or directory path on disk.\n", paszArgs[1])  );
					*pExitStatus = 1;
					goto fail;
				}
				SG_ERR_CHECK_CURRENT;

				SG_workingdir__find_mapping(pCtx, pPath, &pTopPath, &pDescriptorName, NULL);
				if(SG_context__err_equals(pCtx, SG_ERR_NOT_A_WORKING_COPY))
				{
					SG_context__err_reset(pCtx);
					SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "The directory %s is not a working copy.\n", SG_pathname__sz(pPath))  );
					*pExitStatus = 1;
					goto fail;
				}
				SG_ERR_CHECK_CURRENT;

				pszDescriptorName = SG_string__sz(pDescriptorName);
				SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pszDescriptorName, &pRepo)  );

				SG_PATHNAME_NULLFREE(pCtx, pPath);
			}

			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Repository:           %s\n", pszDescriptorName)  );
		}

		if(pTopPath!=NULL)
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Root of Working Copy: %s\n", SG_pathname__sz(pTopPath))  );
			SG_PATHNAME_NULLFREE(pCtx, pTopPath);
		}

		SG_localsettings__get__sz(pCtx, SG_LOCALSETTING__PATHS_DEFAULT, pRepo, &sz, NULL);
		if(!SG_context__has_err(pCtx) && sz && *sz!='\0')
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Default push/pull:    %s\n", sz)  );
		}
		else
			SG_context__err_reset(pCtx);
		SG_NULLFREE(pCtx, sz);

		if(verbose)
		{
			SG_ERR_CHECK(  SG_closet__descriptors__get(pCtx, pszDescriptorName, &pvhDescriptor)  );
			SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pStr)  );
			SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvhDescriptor, pStr)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\nDescriptor:\n%s", SG_string__sz(pStr))  );
			SG_VHASH_NULLFREE(pCtx, pvhDescriptor);
			SG_STRING_NULLFREE(pCtx, pStr);

			SG_ERR_CHECK(  SG_repo__get_hash_method(pCtx, pRepo, &sz)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\nHash method: %s\n", sz)  );
			SG_NULLFREE(pCtx, sz);

			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\nBlob Count:\n")  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT,   "==========================\n")  );
			SG_ERR_CHECK(  do_cmd_blobcount(pCtx, pszDescriptorName)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT,   "==========================\n")  );
		}

		SG_STRING_NULLFREE(pCtx, pDescriptorName);
		SG_REPO_NULLFREE(pCtx, pRepo);
	}
	else if(count_args==3 && strcmp(paszArgs[0],"rename")==0)
	{
		SG_bool bExists = SG_FALSE;
		SG_bool proceed = SG_TRUE;

		if(allBut || verbose || pszStorage || pszHashMethod || psz_shared_users)
			SG_ERR_THROW(SG_ERR_USAGE);

		SG_ERR_CHECK(  SG_closet__descriptors__exists(pCtx, paszArgs[2], &bExists)  );
		if(bExists)
			SG_ERR_THROW2(SG_ERR_REPO_ALREADY_EXISTS, (pCtx, "%s", paszArgs[2]));

		SG_ERR_CHECK(  SG_closet__descriptors__get(pCtx, paszArgs[1], &pvhDescriptor)  );

		if(!noPrompt)
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, 
				"Renaming a repository breaks all associations with current working copies of that repository."
				" To create a working copy for the new name, perform a checkout.\n"
				"\n"
				"Additionally, no config settings (such as the default push/pull location) will be migrated to the new name."
				" Any desired repository-specific settings must be set manually under the new name."
				" To see the old settings, run:\n"
				"vv config show /instance/%s\n", paszArgs[1])  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\n")  );
			SG_ERR_CHECK(  SG_console__ask_question__yn(pCtx, "Are you sure you want to rename this repository?", SG_QA_DEFAULT_NO, &proceed)  );
		}

		if(proceed)
		{
			SG_ERR_CHECK(  SG_closet__descriptors__add(pCtx, paszArgs[2], pvhDescriptor)  );
			SG_ERR_CHECK(  SG_closet__descriptors__remove(pCtx, paszArgs[1])  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Rename complete.\n")  );
		}
		else
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Aborted. The repository was not renamed.\n")  );
		}

		SG_VHASH_NULLFREE(pCtx, pvhDescriptor);
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}

	return;
fail:
	SG_STRINGARRAY_NULLFREE(pCtx, pHitList);
	SG_VHASH_NULLFREE(pCtx, pDescriptors);
	SG_STRING_NULLFREE(pCtx, pStr);
	SG_STRING_NULLFREE(pCtx, pDescriptorName);
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_PATHNAME_NULLFREE(pCtx, pTopPath);
	SG_VARRAY_NULLFREE(pCtx, pva);
	SG_VHASH_NULLFREE(pCtx, pvhDescriptor);
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_NULLFREE(pCtx, sz);
}

// Forward declaration, because this and my_localsettings__join call each other.
static void my_localsettings__variant_to_string(
	SG_context* pCtx,
	const SG_variant* pValue,
	SG_int32 iArrayIndent,
	SG_string* pOutput
	);

/**
 * Structure used by my_localsettings__join as pCallerData.
 */
typedef struct
{
	SG_int32   iArrayIndent; //< Passed to my_localsettings__variant_to_string.
	SG_string* pGlue;        //< The glue string to join to join the values together.
	SG_string* pResult;      //< The joined result string.
}
my_localsettings__join__data;

/**
 * Joins all the values in an array together into one string, formatted for localsettings output.
 * Intended for use as a SG_varray_foreach_callback.
 */
static void my_localsettings__join(
	SG_context*       pCtx,        //< Error and context information.
	void*             pCallerData, //< An allocated instance of my_localsettings__join__data.
	                               //< Pass the same instance to all related calls to this function.
	const SG_varray*  pArray,      //< The array containing values being joined.
	SG_uint32         uIndex,      //< The index of the current value in the array.
	const SG_variant* pValue       //< The current value being joined to the others provided so far.
	)
{
	SG_string* pValueString = NULL;
	my_localsettings__join__data* pJoinData = NULL;

	SG_NULLARGCHECK(pCallerData);
	SG_UNUSED(pArray);
	SG_NULLARGCHECK(pValue);

	pJoinData = (my_localsettings__join__data*) pCallerData;

	if (uIndex > 0)
	{
		SG_ERR_CHECK(  SG_string__append__string(pCtx, pJoinData->pResult, pJoinData->pGlue)  );
	}

	SG_ERR_CHECK(  SG_string__alloc(pCtx, &pValueString)  );
	SG_ERR_CHECK(  my_localsettings__variant_to_string(pCtx, pValue, pJoinData->iArrayIndent, pValueString)  );
	SG_ERR_CHECK(  SG_string__append__string(pCtx, pJoinData->pResult, pValueString)  );

fail:
	SG_STRING_NULLFREE(pCtx, pValueString);
}

/**
 * Validates and resolves an input setting name into a fully-qualified one.
 * Names that start with '/' are assumed to be fully-qualified already.
 * They will just be checked to ensure that they're in a valid scope.
 * Other names are placed in the MACHINE scope.
 */
static void my_localsettings__resolve_name(
	SG_context* pCtx,   //< Error and context information.
	const char* szName, //< The input name to resolve.
	SG_bool     bForce, //< True if names outside of the known scopes should be allowed.
	SG_string*  pOutput //< The resolved name string.
	)
{
	SG_NULLARGCHECK(szName);
	SG_NULLARGCHECK(pOutput);

	// check if the given name is global
	if (szName[0] == '/')
	{
		// only check that it's in a known scope if the force flag wasn't used
		if (!bForce)
		{
			SG_int32 iSlashCount = 0;
			SG_uint32 uCurrent = 0;

			// make sure the global name starts with a known scope
			if (0 == strncmp(szName, SG_LOCALSETTING__SCOPE__INSTANCE, strlen(SG_LOCALSETTING__SCOPE__INSTANCE)))
			{
				iSlashCount = 3;
			}
			else if (0 == strncmp(szName, SG_LOCALSETTING__SCOPE__REPO, strlen(SG_LOCALSETTING__SCOPE__REPO)))
			{
				iSlashCount = 3;
			}
			else if (0 == strncmp(szName, SG_LOCALSETTING__SCOPE__ADMIN, strlen(SG_LOCALSETTING__SCOPE__ADMIN)))
			{
				iSlashCount = 3;
			}
			else if (0 == strncmp(szName, SG_LOCALSETTING__SCOPE__MACHINE, strlen(SG_LOCALSETTING__SCOPE__MACHINE)))
			{
				iSlashCount = 2;
			}
			else
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting %s is in an unknown scope.", szName)  );
			}

			// make sure the name contains enough slashes for the scope
			// this ensures that "/instance/<name>" isn't used by itself
			while (szName[uCurrent] != '\0')
			{
				if (szName[uCurrent] == '/')
				{
					iSlashCount = iSlashCount - 1;
				}
				uCurrent = uCurrent + 1;
			}
			if (iSlashCount > 0)
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting %s isn't fully qualified.", szName)  );
			}
		}

		// the name looks good, copy it to the output
		SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, szName)  );
	}
	else
	{
		// prefix the local name with a default scope (machine, for now)
		SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, SG_LOCALSETTING__SCOPE__MACHINE)  );
		SG_ERR_CHECK(  SG_string__append__sz(pCtx, pOutput, "/")  );
		SG_ERR_CHECK(  SG_string__append__sz(pCtx, pOutput, szName)  );
	}

fail:
	return;
}

/**
 * Converts a settings variant value into a string suitable for display.
 */
static void my_localsettings__variant_to_string(
	SG_context*       pCtx,         //< Error and context information.
	const SG_variant* pValue,       //< The setting value to convert.
	                                //< Should not be a VHASH, recurse into them before calling this.
	SG_int32          iArrayIndent, //< Indicates how to handle the multiple values in an array.
	                                //< If non-negative, then values will be on their own line indented by this number of spaces.
	                                //< If negative, then values will just be separated with a comma.
	                                //< Nested arrays aren't really handled and haven't been tested.
	SG_string*        pOutput       //< The string to place the converted value into.
	)
{
	SG_string* pJoinGlue = NULL;

	SG_NULLARGCHECK(pOutput);

	if (pValue == NULL)
	{
		SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, "{UNSET}")  );
		return;
	}
	else
	{
		switch (pValue->type)
		{
			case SG_VARIANT_TYPE_NULL:
			{
				SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, "{NULL}")  );
				break;
			}
			case SG_VARIANT_TYPE_INT64:
			{
				SG_int64 value;
				SG_int_to_string_buffer szConvertBuffer;
				SG_ERR_CHECK(  SG_variant__get__int64(pCtx, pValue, &value)  );
				SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, SG_int64_to_sz(value, szConvertBuffer))  );
				break;
			}
			case SG_VARIANT_TYPE_DOUBLE:
			{
				double value;
				SG_ERR_CHECK(  SG_variant__get__double(pCtx, pValue, &value)  );
				SG_ERR_CHECK(  SG_string__sprintf(pCtx, pOutput, "%g", value)  );
				break;
			}
			case SG_VARIANT_TYPE_BOOL:
			{
				SG_bool value;
				SG_ERR_CHECK(  SG_variant__get__bool(pCtx, pValue, &value)  );
				SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, value ? "True" : "False")  );
				break;
			}
			case SG_VARIANT_TYPE_SZ:
			{
				SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, pValue->v.val_sz)  );
				break;
			}
			case SG_VARIANT_TYPE_VARRAY:
			{
				my_localsettings__join__data JoinData;
				SG_uint32 uCount = 0u;

				// check the length of the array
				SG_ERR_CHECK(  SG_varray__count(pCtx, pValue->v.val_varray, &uCount)  );
				if (uCount == 0u)
				{
					// empty, that's easy
					SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, "{EMPTY LIST}")  );
				}
				else
				{
					// build the glue to join all the values together
					if (iArrayIndent < 0)
					{
						// the glue should just be a comma
						SG_ERR_CHECK(  SG_string__set__sz(pCtx, pJoinGlue, ", ")  );

						// use the next array indent if it goes recursive (this array contains arrays)
						JoinData.iArrayIndent = iArrayIndent - 1;
					}
					else
					{
						SG_int32 i = 0;

						// the glue should be a newline and the given indent
						SG_ERR_CHECK(  SG_string__alloc(pCtx, &pJoinGlue)  );
						SG_ERR_CHECK(  SG_string__set__sz(pCtx, pJoinGlue, "\n")  );
						for (i=0; i<iArrayIndent; ++i)
						{
							SG_ERR_CHECK(  SG_string__append__sz(pCtx, pJoinGlue, " ")  );
						}

						// use the next array indent if it goes recursive (this array contains arrays)
						JoinData.iArrayIndent = iArrayIndent + 4;
					}

					// join the array values together using the glue to give us our final display value
					JoinData.pGlue = pJoinGlue;
					JoinData.pResult = pOutput;
					SG_ERR_CHECK(  SG_varray__foreach(pCtx, pValue->v.val_varray, my_localsettings__join, &JoinData)  );
				}

				break;
			}
			case SG_VARIANT_TYPE_VHASH:
			{
				SG_uint32 uCount = 0u;

				// we should only ever end up seeing empty hash values
				// non-empty hashes should be recursed into instead of provided directly
				SG_ERR_CHECK(  SG_vhash__count(pCtx, pValue->v.val_vhash, &uCount)  );
				if (uCount > 0u)
				{
					SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, "{ERROR: Non-empty vhash}")  );
				}
				else
				{
					SG_ERR_CHECK(  SG_string__set__sz(pCtx, pOutput, "{EMPTY HASH}")  );
				}
				break;
			}
		}
	}

fail:
	SG_STRING_NULLFREE(pCtx, pJoinGlue);
}

/**
 * Structure used by my_localsettings__print_value as pCallerData.
 */
typedef struct
{
	SG_string* pLastScope; //< Stores the name of the last scope seen by the callback.
}
my_localsettings__print_value__data;

/**
 * Prints a single setting value.
 * Generally intended for use as a SG_localsettings_foreach_callback.
 */
static void my_localsettings__print_value(
	SG_context*       pCtx,       //< Error and context information.
	const char*       szFullName, //< The fully qualified name of the setting to print.
	const char*       szScope,    //< Just the scope of the setting being printed (including admin/repo ID or descriptor name, where relevant).
	const char*       szName,     //< The name of the setting without the scope.
	const SG_variant* pValue,     //< The value of the setting, or NULL if it has no value.
	void*             pCallerData //< NULL or an allocated instance of my_localsettings__print_value__data.
	                              //< If an instance is given and reused across several calls,
	                              //< then back-to-back settings in the same scope will be grouped together.
	)
{
	static const SG_uint32 iIndent = 4;
	SG_string* pValueString = NULL;
	my_localsettings__print_value__data* pData = NULL;
	SG_uint32 i = 0u;

	SG_UNUSED(szFullName);
	SG_NULLARGCHECK(szScope);
	SG_NULLARGCHECK(szName);

	pData = (my_localsettings__print_value__data*) pCallerData;
	SG_ERR_CHECK(  SG_string__alloc(pCtx, &pValueString)  );

	// convert the value to a string
	// the extra +2 indent is for the colon and space in the output
	SG_ERR_CHECK(  my_localsettings__variant_to_string(pCtx, pValue, SG_STRLEN(szName) + iIndent + 2, pValueString)  );

	// if the name is empty, then the scope IS the full name
	// this is a weird forced setting where the top-level key doesn't have a hash value
	// but we still have to deal with it somehow
	if (szName[0] == '\0')
	{
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s: %s\n", szScope, SG_string__sz(pValueString))  );
	}
	else
	{
		// if this is a different scope than last time, print it out and then remember it for next time
		if (pData == NULL || strcmp(SG_string__sz(pData->pLastScope), szScope) != 0)
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s/\n", szScope)  );
			SG_ERR_CHECK(  SG_string__set__sz(pCtx, pData->pLastScope, szScope)  );
		}

		// print out the indent
		for (i=0u; i<iIndent; ++i)
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, " ")  );
		}

		// print out the value
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s: %s\n", szName, SG_string__sz(pValueString))  );
	}

fail:
	SG_STRING_NULLFREE(pCtx, pValueString);
}

/**
 * Prints the values of all settings whose names match a pattern.
 */
static void my_localsettings__print_values(
	SG_context* pCtx,            //< Error and context information.
	const char* szPattern,       //< The pattern to match setting names against, or NULL to not restrict by pattern.
	                             //< Patterns that start with "/" will only match the beginning of fully qualified names.
	                             //< Other patterns will match anywhere in non-qualified names.
	SG_bool     bIncludeDefaults //< If true, then factory default values will also be included.
	)
{
	my_localsettings__print_value__data PrintValueData = {NULL};

	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &(PrintValueData.pLastScope))  );
	SG_ERR_CHECK(  SG_localsettings__foreach(pCtx, szPattern, bIncludeDefaults, my_localsettings__print_value, &PrintValueData)  );

fail:
	SG_STRING_NULLFREE(pCtx, PrintValueData.pLastScope);
}

/**
 * Prints the old and new values of a single setting.
 * Use this to display the results of manipulating a setting.
 */
static void my_localsettings__print_changed_value(
	SG_context* pCtx,       //< Error and context information.
	const char* szFullName, //< The fully qualified name of the changed setting.
	SG_variant* pOldValue,  //< The setting's value before it was changed.
	SG_variant* pNewValue   //< The setting's value after it was changed.
	)
{
	SG_string* pScopeName = NULL;
	SG_string* pSettingName = NULL;
	SG_string* pName = NULL;
	SG_uint32 uLength = 0u;
	SG_uint32 i = 0u;
	my_localsettings__print_value__data PrintValueData = {NULL};

	// allocate data
	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &(PrintValueData.pLastScope))  );

	// split the full name into scope and setting names
	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pScopeName)  );
	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pSettingName)  );
	SG_ERR_CHECK(  SG_localsettings__split_full_name(pCtx, szFullName, NULL, pScopeName, pSettingName)  );

	// print the old value
	SG_ERR_CHECK(  my_localsettings__print_value(pCtx, szFullName, SG_string__sz(pScopeName), SG_string__sz(pSettingName), pOldValue, &PrintValueData)  );

	// clear the setting name and build a new one that's just a right-aligned arrow: ->
	// if the pSettingName is empty, then the pScopeName is the full name we're displaying (and replacing)
	// (it's a weird forced edged case in localsettings where a top-level key doesn't have a vhash value)
	pName = pSettingName;
	uLength = SG_string__length_in_bytes(pName);
	if (uLength == 0u)
	{
		pName = pScopeName;
		uLength = SG_string__length_in_bytes(pName);
	}
	if (uLength >= 2u)
	{
		uLength = uLength - 2;
	}
	else if (uLength == 1u)
	{
		uLength = uLength - 1;
	}
	SG_ERR_CHECK(  SG_string__clear(pCtx, pName)  );
	for (i=0u; i<uLength; ++i)
	{
		SG_ERR_CHECK(  SG_string__append__sz(pCtx, pName, " ")  );
	}
	SG_ERR_CHECK(  SG_string__append__sz(pCtx, pName, "->")  );

	// print the new value
	SG_ERR_CHECK(  my_localsettings__print_value(pCtx, szFullName, SG_string__sz(pScopeName), SG_string__sz(pSettingName), pNewValue, &PrintValueData)  );

fail:
	SG_STRING_NULLFREE(pCtx, PrintValueData.pLastScope);
	SG_STRING_NULLFREE(pCtx, pScopeName);
	SG_STRING_NULLFREE(pCtx, pSettingName);
}

/**
 * Imports localsettings from a JSON file, replacing all existing settings.
 */
static void my_localsettings__import(
	SG_context* pCtx,     //< Error and context information.
	SG_bool     bVerbose, //< True if imported settings should be printed out.
	const char* psz_path  //< The path/filename to read the settings from, or NULL to read them from STDIN.
	)
{
	SG_uint32 len32 = 0;
	SG_uint64 len64 = 0;
	SG_vhash* pvh = NULL;
	SG_pathname* pPath = NULL;
	SG_file* pFile = NULL;
	SG_byte* p = NULL;
	SG_string* s = NULL;

	// check where to read the settings from
	if (psz_path == NULL) {
		// read from stdin
		SG_ERR_CHECK(  SG_string__alloc(pCtx, &s)  );
		SG_ERR_CHECK(  SG_console__read_stdin(pCtx, &s)  );
		SG_ERR_CHECK(  SG_string__sizzle(pCtx, &s, &p, NULL)  );
	} else {
		// read from the specified file

		// determine file length
		SG_ERR_CHECK(  SG_pathname__alloc__sz(pCtx, &pPath, psz_path)  );
		SG_ERR_CHECK(  SG_fsobj__length__pathname(pCtx, pPath, &len64, NULL)  );
		if (!SG_uint64__fits_in_uint32(len64))
		{
			SG_ERR_THROW(  SG_ERR_UNSPECIFIED  ); // TODO
		}
		len32 = (SG_uint32)len64;

		// read the file into a buffer
		SG_ERR_CHECK(  SG_file__open__pathname(pCtx, pPath, SG_FILE_RDONLY|SG_FILE_OPEN_EXISTING, SG_FSOBJ_PERMS__UNUSED, &pFile)  );
		SG_ERR_CHECK(  SG_alloc(pCtx, 1,len32+1,&p)  );
		SG_ERR_CHECK(  SG_file__read(pCtx, pFile, len32, p, NULL)  );
		p[len32] = 0;
	}

	// parse the json data in the buffer to a vhash
	SG_ERR_CHECK(  SG_VHASH__ALLOC__FROM_JSON__SZ(pCtx, &pvh, (const char*) p)  );
	SG_NULLFREE(pCtx, p);

	// import settings from the vhash
	SG_ERR_CHECK(  SG_localsettings__import(pCtx, pvh)  );

	// print out the results, if requested
	if (bVerbose == SG_TRUE)
	{
		SG_ERR_CHECK(  my_localsettings__print_values(pCtx, NULL, SG_FALSE)  );
	}

fail:
	SG_VHASH_NULLFREE(pCtx, pvh);
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_FILE_NULLCLOSE(pCtx, pFile);
	SG_NULLFREE(pCtx, p);
	SG_STRING_NULLFREE(pCtx, s);
}

/**
 * Exports localsettings to a JSON file.
 */
static void my_localsettings__export(
	SG_context* pCtx,    //< Error and context information.
	const char* psz_path //< The path/filename to write the settings to, or NULL to write them to STDOUT.
	)
{
	SG_vhash* pvh = NULL;
	SG_string* pstr = NULL;
	SG_pathname* pPath = NULL;
	SG_file* pFile = NULL;

	// dump settings to a vhash and then to a json string
	SG_ERR_CHECK(  SG_localsettings__list__vhash(pCtx,  &pvh )  );
	SG_ERR_CHECK(  SG_string__alloc(pCtx, &pstr)  );
	SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
	SG_VHASH_NULLFREE(pCtx, pvh );

	// check where to dump the string
	if (psz_path == NULL) {
		// dump it to stdout
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", SG_string__sz(pstr))  );
	} else {
		// dump it to the specified file
		SG_ERR_CHECK(  SG_pathname__alloc__sz(pCtx, &pPath, psz_path)  );
		SG_ERR_CHECK(  SG_file__open__pathname(pCtx, pPath, SG_FILE_WRONLY | SG_FILE_OPEN_OR_CREATE | SG_FILE_TRUNC, 0777, &pFile)  );
		SG_ERR_CHECK(  SG_file__write__string(pCtx, pFile, pstr)  );
		SG_ERR_CHECK(  SG_file__close(pCtx, &pFile)  );
	}

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_FILE_NULLCLOSE(pCtx, pFile);
	SG_STRING_NULLFREE(pCtx, pstr);
	SG_VHASH_NULLFREE(pCtx, pvh );
}

/**
 * print details for "list-ignores".
 *
 */
static void my_localsettings__list_ignores(SG_context * pCtx)
{
	SG_pendingtree * pPendingTree = NULL;
	SG_file_spec * pFilespec = NULL;
	SG_repo * pRepo;
	const SG_pathname * pPathWorkingDirTop = NULL;
	SG_uint32 k, count;

	SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC_FROM_CWD(pCtx, &pPendingTree)  );
	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );
	SG_ERR_CHECK(  SG_pendingtree__get_working_directory_top__ref(pCtx, pPendingTree, &pPathWorkingDirTop)  );
	SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
	SG_ERR_CHECK(  SG_file_spec__load_ignores(pCtx, pFilespec, pRepo, pPathWorkingDirTop, 0u)  );

	SG_ERR_CHECK(  SG_file_spec__count_patterns(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__IGNORE, &count)  );
	for (k=0; k<count; k++)
	{
		const char * psz_k;

		SG_ERR_CHECK(  SG_file_spec__get_nth_pattern(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__IGNORE, k, &psz_k, NULL)  );
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz_k)  );
	}

fail:
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_FILE_SPEC_NULLFREE(pCtx, pFilespec);
}

/**
 * Handles the "localsettings" command.
 */
void do_cmd_localsettings(
	SG_context*  pCtx,      //< Error and context information.
	SG_bool      bVerbose,  //< Whether or not the global verbose flag was used.
	SG_bool      bForce,    //< Whether or not the global force flag was used.
	SG_uint32    uArgCount, //< The number of arguments to the command.
	const char** pszArgs    //< The arguments to the command.
	)
{
	SG_string* pName = NULL;
	SG_variant* pOldValue = NULL;
	SG_variant* pNewValue = NULL;

	// if we have any arguments, then pszArgs[0] is the subcommand.
	if (uArgCount == 0u)
	{
		SG_ERR_CHECK(  my_localsettings__print_values(pCtx, NULL, bVerbose)  );
	}
	else if (0 == strcmp(pszArgs[0], "export"))
	{
		if (uArgCount > 2)
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}
		SG_ERR_CHECK(  my_localsettings__export(pCtx, uArgCount == 1 ? NULL : pszArgs[1])  );
	}
	else if (0 == strcmp(pszArgs[0], "import"))
	{
		if (uArgCount > 2)
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}
		SG_ERR_CHECK(  my_localsettings__import(pCtx, bVerbose, uArgCount == 1 ? NULL : pszArgs[1])  );
	}
	else if (0 == strcmp(pszArgs[0], "defaults"))
	{
		if (uArgCount > 1)
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}
		SG_ERR_CHECK(  my_localsettings__print_values(pCtx, SG_LOCALSETTING__SCOPE__DEFAULT, SG_TRUE)  );
	}
	else if (0 == strcmp(pszArgs[0], "show"))
	{
		if (uArgCount > 2)
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}
		SG_ERR_CHECK(  my_localsettings__print_values(pCtx, uArgCount > 1 ? pszArgs[1] : NULL, bVerbose)  );
	}
	else if (0 == strcmp(pszArgs[0], "list-ignores"))
	{
		// TODO 2011/07/16 I'm not sure that this command belongs here
		// TODO            now that I've added the .sgignores stuff.
		// TODO            It was originally here to show the net result
		// TODO            from a SG_localsettings__get__collapsed_varray()
		// TODO            but I removed that function.

		if (uArgCount > 2)
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}

		SG_ERR_CHECK(  my_localsettings__list_ignores(pCtx)  );
	}
	else
	{
		SG_int64	intValue	= 0;
		// for all of the other valid subcommands
		// paszArgs[1] must be a setting path that we're going to manipulate
		if (uArgCount < 2 || ((0 != strcmp(pszArgs[0], "set"))  
							  && (0 != strcmp(pszArgs[0], "reset"))  
							  && (0 != strcmp(pszArgs[0], "empty"))  
							  && (0 != strcmp(pszArgs[0], "add-to"))
							  && (0 != strcmp(pszArgs[0], "remove-from"))  )  )
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}


		// resolve the setting path to something that we know is usable
		SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pName)  );
		SG_ERR_CHECK(  my_localsettings__resolve_name(pCtx, pszArgs[1], bForce, pName)  );

		// retrieve the setting's current value
		SG_ERR_CHECK(  SG_localsettings__get__variant(pCtx, SG_string__sz(pName), NULL, &pOldValue, NULL)  );

		// manipulate the setting according to the command used
		if (0 == strcmp(pszArgs[0], "set"))
		{
			if (uArgCount != 3)
			{
				SG_ERR_THROW(  SG_ERR_USAGE  );
			}
			if (pOldValue != NULL  && pOldValue->type == SG_VARIANT_TYPE_SZ)
			{
				SG_ERR_CHECK(  SG_localsettings__update__sz(pCtx, SG_string__sz(pName), pszArgs[2])  );
			}
			else if (pOldValue != NULL  && pOldValue->type == SG_VARIANT_TYPE_INT64)
			{
				SG_ERR_CHECK(  SG_int64__parse__strict(pCtx, &intValue, pszArgs[2])  );
				SG_ERR_CHECK(  SG_localsettings__update__int64(pCtx, SG_string__sz(pName), intValue)  );
			}
			else if (pOldValue == NULL)
			{
				//assume string
				SG_ERR_CHECK(  SG_localsettings__update__sz(pCtx, SG_string__sz(pName), pszArgs[2])  );
			}
			else
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting type is not a string or an integer")  );
			}
			
		}
		else if (0 == strcmp(pszArgs[0], "reset")) // TODO delete?  remove?
		{
			if (uArgCount != 2)
			{
				SG_ERR_THROW(  SG_ERR_USAGE  );
			}
			SG_ERR_CHECK(  SG_localsettings__reset(pCtx, SG_string__sz(pName))  );
		}
		else if (0 == strcmp(pszArgs[0], "empty"))
		{
			if (uArgCount != 2)
			{
				SG_ERR_THROW(  SG_ERR_USAGE  );
			}
			if (pOldValue != NULL && pOldValue->type != SG_VARIANT_TYPE_VARRAY)
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting type is not an array")  );
			}
			SG_ERR_CHECK(  SG_localsettings__varray__empty(pCtx, SG_string__sz(pName))  );
		}
		else if (0 == strcmp(pszArgs[0], "add-to"))
		{
			SG_uint32 i;
			if (uArgCount < 3)
			{
				SG_ERR_THROW(  SG_ERR_USAGE  );
			}
			if (pOldValue != NULL && pOldValue->type != SG_VARIANT_TYPE_VARRAY)
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting type is not an array")  );
			}
			for(i=2;i<uArgCount;++i)
			{
				SG_ERR_CHECK(  SG_localsettings__varray__append(pCtx, SG_string__sz(pName), pszArgs[i])  );
			}
		}
		else if (0 == strcmp(pszArgs[0], "remove-from"))
		{
			SG_uint32 i;
			if (uArgCount < 3)
			{
				SG_ERR_THROW(  SG_ERR_USAGE  );
			}
			if (pOldValue != NULL && pOldValue->type != SG_VARIANT_TYPE_VARRAY)
			{
				SG_ERR_THROW2(  SG_ERR_INVALIDARG, (pCtx, "Setting type is not an array")  );
			}
			for(i=2;i<uArgCount;++i)
			{
				SG_ERR_CHECK(  SG_localsettings__varray__remove_first_match(pCtx, SG_string__sz(pName), pszArgs[i])  );
			}
		}
		else
		{
			SG_ERR_THROW(  SG_ERR_USAGE  );
		}

		// get the setting's new value
		SG_ERR_CHECK(  SG_localsettings__get__variant(pCtx, SG_string__sz(pName), NULL, &pNewValue, NULL)  );

		// print out the old and new values to show the results
		SG_ERR_CHECK(  my_localsettings__print_changed_value(pCtx, SG_string__sz(pName), pOldValue, pNewValue)  );
	}

fail:
	SG_STRING_NULLFREE(pCtx, pName);
	SG_VARIANT_NULLFREE(pCtx, pOldValue);
	SG_VARIANT_NULLFREE(pCtx, pNewValue);
}

void do_cmd_stamp(SG_context * pCtx, const char * szRepo, SG_bool plural, SG_uint32 argc, const char ** argv, SG_rev_spec* pRevSpec, SG_bool all)
{
	SG_repo* pRepo = NULL;
	SG_audit q;
	SG_uint32 countRevSpecs = 0;
	SG_stringarray* psaHidChangesets = NULL;
	SG_varray * pva_results = NULL;
	SG_uint32 count, i;
	SG_vhash * pvh_pile = NULL;

	if(argc==2 && strcmp(argv[0], "add")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		if (pRevSpec != NULL)
			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
		if (countRevSpecs <= 0)
		{
			if(szRepo==NULL)
				SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, NULL, NULL, &psaHidChangesets)  );
			else
				SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "When using the '--repo' option you must specify which specific changeset(s) to apply the stamp to."));
		}
		else
			SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pRevSpec, SG_TRUE, &psaHidChangesets)  );

		if (psaHidChangesets)
		{
			SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaHidChangesets, &count)  );
			for (i = 0; i < count; i++)
			{
				const char* pszRefHid;
				SG_bool redundant = SG_FALSE;
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, i, &pszRefHid)  );
				SG_ERR_CHECK(  SG_vc_stamps__add(pCtx, pRepo, pszRefHid, argv[1], &q, &redundant)  );
				if(!redundant)
					SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "Stamp \"%s\" added to changeset %s\n", argv[1], pszRefHid)  );
				else
					SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "(Changeset %s was already stamped \"%s\".)\n", pszRefHid, argv[1])  );
			}
		}
	}
	else if(argc==2 && strcmp(argv[0], "remove")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		if (pRevSpec != NULL)
			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
		if (countRevSpecs <= 0)
		{
			if(all)
				SG_ERR_CHECK(  SG_vc_stamps__lookup_by_stamp(pCtx, pRepo, &q, argv[1], &psaHidChangesets)  );
			else if(szRepo==NULL)
				SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, NULL, NULL, &psaHidChangesets)  );
			else
				SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "When using the '--repo' option you must specify which specific changeset(s) to remove the stamp from."));
		}
		else
		{
			if(all)
				SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "'--all' option should not be used together with specific revisions"));
			else
				SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pRevSpec, SG_TRUE, &psaHidChangesets)  );
		}

		count = 0;
		if (psaHidChangesets)
		{
			SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaHidChangesets, &count)  );
			if(count>0)
			{
				for (i = 0; i < count; i++)
				{
					const char* pszRefHid;
					SG_bool bStampIsThere = SG_FALSE;
					SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, i, &pszRefHid)  );
					SG_ERR_CHECK(  SG_vc_stamps__is_stamp_already_applied(pCtx, pRepo, pszRefHid, argv[1], &bStampIsThere)  );
					if(!bStampIsThere)
						SG_ERR_THROW2(SG_ERR_STAMP_NOT_FOUND, (pCtx, "%s in changeset %s", argv[1], pszRefHid));
				}

				for (i = 0; i < count; i++)
				{
					const char* pszRefHid;
					SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, i, &pszRefHid)  );
					SG_ERR_CHECK(  SG_vc_stamps__remove(pCtx, pRepo, &q, pszRefHid, 1, &argv[1])  );
					SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "Stamp \"%s\" removed from changeset %s\n", argv[1], pszRefHid)  );
				}
			}
		}
		if(count==0)
			SG_ERR_THROW2(SG_ERR_STAMP_NOT_FOUND, (pCtx, "%s", argv[1]));
	}
	else if((argc==1 && strcmp(argv[0], "list")==0) || (argc==0 && plural))
	{
		SG_uint32 count_results = 0;
		SG_uint32 index_results = 0;
		const char * psz_thisStamp = NULL;
		SG_int64 thisCount = 0;

		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(   SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

		SG_ERR_CHECK( SG_vc_stamps__list_all_stamps(pCtx, pRepo, &pva_results)  );

		SG_ERR_CHECK(  SG_varray__count(pCtx, pva_results, &count_results)  );
		for (index_results = 0; index_results < count_results; index_results++)
		{
			SG_vhash * pvhThisResult = NULL;
			SG_ERR_CHECK( SG_varray__get__vhash(pCtx, pva_results, index_results, &pvhThisResult)  );
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvhThisResult, "stamp", &psz_thisStamp)  );
			SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pvhThisResult, "count", &thisCount)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s:\t%d\n", psz_thisStamp, (int)thisCount)  );
		}
	}
	else if(argc==2 && strcmp(argv[0], "list")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(   SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		SG_ERR_CHECK(  SG_vc_stamps__lookup_by_stamp(pCtx, pRepo, &q, argv[1], &psaHidChangesets)  );
		if (psaHidChangesets)
		{
			SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
			SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaHidChangesets, &count)  );
			for (i = 0; i < count; i++)
			{
				const char* pszRefHid;
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, i, &pszRefHid)  );
				SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, pszRefHid, pvh_pile, SG_TRUE, SG_FALSE)  );
			}
		}
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}

	/* fall through */
fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_STRINGARRAY_NULLFREE(pCtx, psaHidChangesets);
	SG_VARRAY_NULLFREE(pCtx, pva_results);
	SG_VHASH_NULLFREE(pCtx, pvh_pile);
}

#if 0
/* TODO the "UI" (cmdline params) for this needs work.  What we have right now
 * is insufficient, but serves as a proof of concept.  We need a way to
 * specify how much history should be included.  Perhaps by number of revisions
 * back, or by date.  And we need a way to pick up the
 * tags/stamps/comments/etc. */
void do_cmd_fragball(SG_context * pCtx, const char* psz_filename, const char* psz_spec_cs)
{
	SG_pathname* pPathCwd = NULL;
    SG_pathname* pPath_fragball = NULL;
    SG_string* pstrRepoDescriptorName = NULL;
    SG_vhash* pvhDescriptor = NULL;
    SG_repo* pRepo = NULL;
    char* psz_hid_cs = NULL;
    SG_dagfrag* pFrag = NULL;
    SG_rbtree* prb_frags = NULL;
    char buf_dagnum[32];

	/* find the repo  - TODO: use SG_cmd_util__get_repo_from_cwd */
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );
	SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPathCwd, NULL, &pstrRepoDescriptorName, NULL)  );

    SG_ERR_CHECK(  SG_closet__descriptors__get(pCtx, SG_string__sz(pstrRepoDescriptorName), &pvhDescriptor)  );
    SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, &pRepo, pvhDescriptor)  );

    /* lookup the changeset id */
    SG_ERR_CHECK(  SG_hidlookup__changeset(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_spec_cs, &psz_hid_cs)  );

    /* allocate a frag */
    SG_ERR_CHECK(  SG_dagfrag__alloc(pCtx, &pFrag, SG_DAGNUM__VERSION_CONTROL, my_fetch_dagnodes_from_repo, pRepo)  );
    SG_ERR_CHECK(  SG_dagfrag__add_leaf(pCtx, pFrag, psz_hid_cs, 2)  );

    /* write the fragball file */
    SG_ERR_CHECK(  SG_RBTREE__ALLOC(pCtx, &prb_frags)  );
    SG_ERR_CHECK(  SG_dagnum__to_string(pCtx, SG_DAGNUM__VERSION_CONTROL, buf_dagnum, sizeof(buf_dagnum))  );
    SG_ERR_CHECK(  SG_rbtree__add__with_assoc(pCtx, prb_frags, buf_dagnum, pFrag)  );
    SG_ERR_CHECK(  SG_PATHNAME__ALLOC__PATHNAME_SZ(pCtx, &pPath_fragball, pPathCwd, psz_filename)  );
    SG_ERR_CHECK(  SG_fragball__write(pCtx, pRepo, pPath_fragball, prb_frags)  );

    /* fall through to common cleanup */

fail:
    SG_RBTREE_NULLFREE(pCtx, prb_frags);
    SG_DAGFRAG_NULLFREE(pCtx, pFrag);
    SG_NULLFREE(pCtx, psz_hid_cs);
    SG_VHASH_NULLFREE(pCtx, pvhDescriptor);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_STRING_NULLFREE(pCtx, pstrRepoDescriptorName);
	SG_PATHNAME_NULLFREE(pCtx, pPath_fragball);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
}
#endif

#if 0
void do_cmd_groups(SG_context * pCtx, const char* psz_descriptor_name)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
    SG_varray* pva_groups = NULL;
    SG_uint32 i = 0;
    SG_uint32 count = 0;

    if (psz_descriptor_name)
    {
        SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
    }

    SG_ERR_CHECK(  SG_group__list(pCtx, pRepo, &pva_groups)  );
    SG_ERR_CHECK(  SG_varray__count(pCtx, pva_groups, &count)  );
    for (i=0; i<count; i++)
    {
        SG_vhash* pvh = NULL;
        const char* psz_name = NULL;

        SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_groups, i, &pvh)  );

        SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh, "name", &psz_name)  );
        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz_name)  );
    }

    /* fall through */

fail:
    SG_VARRAY_NULLFREE(pCtx, pva_groups);
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
}

void do_cmd_creategroup(SG_context * pCtx, const char* psz_name, const char* psz_descriptor_name)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;

    if (psz_descriptor_name)
    {
        SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
    }

    SG_ERR_CHECK(  SG_group__create(pCtx, pRepo, psz_name)  );

    /* fall through */

fail:
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
}

#define MY_CMD_GROUP_OP__LIST_SHALLOW      (1)
#define MY_CMD_GROUP_OP__LIST_DEEP         (2)
#define MY_CMD_GROUP_OP__ADD_USERS         (3)
#define MY_CMD_GROUP_OP__REMOVE_USERS      (4)
#define MY_CMD_GROUP_OP__ADD_SUBGROUPS     (5)
#define MY_CMD_GROUP_OP__REMOVE_SUBGROUPS  (6)

void do_cmd_group(SG_context * pCtx, const char* psz_group_name, const char* psz_descriptor_name, SG_uint32 op, const char** paszNames, SG_uint32 count_names)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
    SG_varray* pva = NULL;
    SG_vhash* pvh = NULL;
    SG_vhash* pvh_group = NULL;

    if (psz_descriptor_name)
    {
        SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
    }

    switch (op)
    {
        case MY_CMD_GROUP_OP__LIST_DEEP:
            {
                SG_uint32 count_members = 0;
                SG_uint32 i = 0;

                if (count_names)
                {
                    // TODO error
                }

                SG_ERR_CHECK(  SG_group__list_all_users(pCtx, pRepo, psz_group_name, &pvh)  );
                SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh, &count_members)  );
                for (i=0; i<count_members; i++)
                {
                    const char* psz_userid = NULL;
                    SG_vhash* pvh_user = NULL;
                    const SG_variant* pv = NULL;
                    const char* psz_username = NULL;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh, i, &psz_userid, &pv)  );
                    SG_ERR_CHECK(  SG_variant__get__vhash(pCtx, pv, &pvh_user)  );

                    SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_user, "name", &psz_username)  );
                    SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz_username)  );
                }
            }
            break;

        case MY_CMD_GROUP_OP__LIST_SHALLOW:
            {
                SG_uint32 count_members = 0;
                SG_uint32 i = 0;

                if (count_names)
                {
                    // TODO error
                }

                SG_ERR_CHECK(  SG_group__get_by_name(pCtx, pRepo, psz_group_name, &pvh_group)  );
                if (pvh_group)
                {
                    SG_varray* pva_members = NULL;

                    SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvh_group, "users", &pva_members)  );
                    if (pva_members)
                    {
                        SG_ERR_CHECK(  SG_varray__count(pCtx, pva_members, &count_members)  );
                        for (i=0; i<count_members; i++)
                        {
                            SG_vhash* pvh_member = NULL;
                            const char* psz_member_name = NULL;

                            SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_members, i, &pvh_member)  );
                            SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_member, "name", &psz_member_name)  );
                            SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s: %s\n", "user", psz_member_name)  );
                        }
                    }

                    SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvh_group, "subgroups", &pva_members)  );
                    if (pva_members)
                    {
                        SG_ERR_CHECK(  SG_varray__count(pCtx, pva_members, &count_members)  );
                        for (i=0; i<count_members; i++)
                        {
                            SG_vhash* pvh_member = NULL;
                            const char* psz_member_name = NULL;

                            SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_members, i, &pvh_member)  );
                            SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_member, "name", &psz_member_name)  );
                            SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s: %s\n", "subgroup", psz_member_name)  );
                        }
                    }
                    SG_VHASH_NULLFREE(pCtx, pvh_group);
                }
                else
                {
                }
            }
            break;

        case MY_CMD_GROUP_OP__ADD_USERS:
            {
                SG_ERR_CHECK(  SG_group__add_users(pCtx, pRepo, psz_group_name, paszNames, count_names)  );
            }
            break;

        case MY_CMD_GROUP_OP__REMOVE_USERS:
            {
                SG_ERR_CHECK(  SG_group__remove_users(pCtx, pRepo, psz_group_name, paszNames, count_names)  );
            }
            break;

        case MY_CMD_GROUP_OP__ADD_SUBGROUPS:
            {
                SG_ERR_CHECK(  SG_group__add_subgroups(pCtx, pRepo, psz_group_name, paszNames, count_names)  );
            }
            break;

        case MY_CMD_GROUP_OP__REMOVE_SUBGROUPS:
            {
                SG_ERR_CHECK(  SG_group__remove_subgroups(pCtx, pRepo, psz_group_name, paszNames, count_names)  );
            }
            break;

        default:
            {
                SG_ERR_THROW(  SG_ERR_NOTIMPLEMENTED  );
            }
            break;
    }

    /* fall through */

fail:
    SG_VHASH_NULLFREE(pCtx, pvh_group);
    SG_VARRAY_NULLFREE(pCtx, pva);
    SG_VHASH_NULLFREE(pCtx, pvh);
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
}
#endif

void do_cmd_set_whoami(SG_context * pCtx, const char* psz_username, const char* psz_descriptor_name, SG_bool bCreate)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
    char* psz_admin_id = NULL;
	char * psz_userid = NULL;

    if (psz_descriptor_name)
    {
        SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
		SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL);
		if (SG_context__err_equals(pCtx, SG_ERR_NOT_A_WORKING_COPY))
		{
			SG_context__err_reset(pCtx);
            pRepo = NULL;
		}

		SG_ERR_CHECK_CURRENT;
    }

	if (bCreate)
	{
		if (pRepo == NULL)
			SG_ERR_THROW2(  SG_ERR_NOT_A_WORKING_COPY,
								   (pCtx, "Could not determine which repo should get the new user.  Maybe you should use the --repo option.")  );

		SG_user__create(pCtx, pRepo, psz_username, &psz_userid);
		if (  SG_context__err_equals(pCtx, SG_ERR_ZING_CONSTRAINT)  )
		{
			SG_context__err_reset(pCtx);
		}
		else if (  SG_context__has_err(pCtx)  )
			SG_ERR_RETHROW;

		
	}
    SG_ERR_CHECK(  SG_user__set_user__repo(pCtx, pRepo, psz_username)  );

    /* fall through */

fail:
    SG_NULLFREE(pCtx, psz_admin_id);
	SG_NULLFREE(pCtx, psz_userid);
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
}

void do_cmd_user(SG_context * pCtx, SG_bool plural, const char* psz_descriptor_name, SG_uint32 argc, const char ** argv)
{
	SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_varray* pva_users = NULL;
	SG_uint32 i = 0;
	SG_uint32 count = 0;
	SG_vhash * pvh_user = NULL;

	if(argc==2 && !strcmp(argv[0],"create"))
	{
		if (psz_descriptor_name)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );

        SG_ERR_CHECK(  SG_user__lookup_by_name(pCtx, pRepo, argv[1], &pvh_user)  );
        if (pvh_user)
        {
            SG_VHASH_NULLFREE(pCtx, pvh_user);
            SG_ERR_THROW(SG_ERR_USER_ALREADY_EXISTS);
        }
		SG_ERR_CHECK(  SG_user__create(pCtx, pRepo, argv[1], NULL)  );
	}
	else if(argc==3 && !strcmp(argv[0],"rename"))
	{
		const char * szUserid = NULL;
		const char * szOldUsername = NULL;

		if (psz_descriptor_name)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );

		SG_user__lookup_by_userid(pCtx, pRepo, argv[1], &pvh_user);
		if(!SG_CONTEXT__HAS_ERR(pCtx) && pvh_user!=NULL)
		{
			szUserid = argv[1];
		}
		else
		{
			SG_ERR_CHECK_CURRENT_DISREGARD(SG_ERR_USERNOTFOUND);
			SG_ERR_CHECK(  SG_user__lookup_by_name(pCtx, pRepo, argv[1], &pvh_user)  );
			if(pvh_user==NULL){SG_ERR_THROW2(SG_ERR_USERNOTFOUND,(pCtx,"%s",argv[1]));}
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_user, "recid", &szUserid)  );
		}
		SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_user, "name", &szOldUsername)  );

		if(strcmp(szOldUsername, argv[2]))
			SG_ERR_CHECK(  SG_user__rename(pCtx, pRepo, szUserid, argv[2])  );
	}
	else if((argc==1 && !strcmp(argv[0],"list")) || (argc==0 && plural))
	{
		if (psz_descriptor_name)
		{
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
		}
		else
		{
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
		}

		SG_ERR_CHECK(  SG_user__list_all(pCtx, pRepo, &pva_users)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pva_users, &count)  );
		for (i=0; i<count; i++)
		{
			SG_vhash* pvh = NULL;
			const char* psz = NULL;

			SG_ERR_CHECK(  SG_varray__get__vhash(pCtx, pva_users, i, &pvh)  );

			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh, "name", &psz)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz)  );
#if 1
			SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh, "recid", &psz)  );
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "    %s\n", psz)  );
#endif
		}
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}

	/* fall through */

fail:
	SG_VARRAY_NULLFREE(pCtx, pva_users);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_VHASH_NULLFREE(pCtx, pvh_user);
}

void do_cmd_version__remote(SG_context * pCtx, const char * szUrl)
{
	SG_string * pUrl = NULL;
	SG_curl* pCurl = NULL;
	SG_string* pstrResponse = NULL;

	SG_ERR_CHECK(  SG_STRING__ALLOC__SZ(pCtx, &pUrl, szUrl)  );

	if(SG_string__sz(pUrl)[SG_string__length_in_bytes(pUrl)-1]!='/')
		SG_ERR_CHECK(  SG_string__append__sz(pCtx, pUrl, "/")  );
	SG_ERR_CHECK(  SG_string__append__sz(pCtx, pUrl, "version.txt")  );

	SG_ERR_CHECK_RETURN(  SG_curl__alloc(pCtx, &pCurl)  );
	SG_ERR_CHECK(  SG_curl__setopt__sz(pCtx, pCurl, CURLOPT_URL, SG_string__sz(pUrl))  );

	SG_ERR_CHECK(  SG_string__alloc(pCtx, &pstrResponse)  );
	SG_ERR_CHECK(  SG_curl__set__write_string(pCtx, pCurl, pstrResponse)  );

	SG_ERR_CHECK(  SG_curl__perform(pCtx, pCurl)  );

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s - %s", szUrl, SG_string__sz(pstrResponse))  );
	
	SG_STRING_NULLFREE(pCtx, pstrResponse);
	SG_CURL_NULLFREE(pCtx, pCurl);
	SG_STRING_NULLFREE(pCtx, pUrl);

	return;
fail:
	SG_STRING_NULLFREE(pCtx, pUrl);
	SG_CURL_NULLFREE(pCtx, pCurl);
	SG_STRING_NULLFREE(pCtx, pstrResponse);
}

void do_cmd_check_integrity(SG_context * pCtx, const char* psz_descriptor_name)
{
	SG_repo* pRepo = NULL;
	SG_vhash* pvh = NULL;
	SG_string* pstr = NULL;

	SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );

    SG_ERR_CHECK(  SG_repo__check_integrity(pCtx, pRepo, NULL, &pvh )  );
    if (pvh)
    {
        SG_STRING__ALLOC(pCtx, &pstr);
        SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
        SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, SG_string__sz(pstr))  );
    }

    // TODO do more stuff here.  fetch every blob (hid check)
    // TODO make sure every blob referenced in every changeset is present.

	/* fall through */
fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_VHASH_NULLFREE(pCtx, pvh);
	SG_STRING_NULLFREE(pCtx, pstr);
}

void do_cmd_blobcount(SG_context * pCtx, const char* psz_descriptor_name)
{
    SG_repo* pRepo = NULL;

    SG_uint32 count_blobs__all = 0;
    SG_uint64 len_encoded__all = 0;
    SG_uint64 len_full__all = 0;
    SG_uint64 max_len_encoded__all = 0;
    SG_uint64 max_len_full__all = 0;

    SG_uint32 count_blobs__full = 0;
    SG_uint64 len_encoded__full = 0;
    SG_uint64 len_full__full = 0;
    SG_uint64 max_len_encoded__full = 0;
    SG_uint64 max_len_full__full = 0;

    SG_uint32 count_blobs__alwaysfull = 0;
    SG_uint64 len_encoded__alwaysfull = 0;
    SG_uint64 len_full__alwaysfull = 0;
    SG_uint64 max_len_encoded__alwaysfull = 0;
    SG_uint64 max_len_full__alwaysfull = 0;

    SG_uint32 count_blobs__zlib = 0;
    SG_uint64 len_encoded__zlib = 0;
    SG_uint64 len_full__zlib = 0;
    SG_uint64 max_len_encoded__zlib = 0;
    SG_uint64 max_len_full__zlib = 0;

    SG_uint32 count_blobs__vcdiff = 0;
    SG_uint64 len_encoded__vcdiff = 0;
    SG_uint64 len_full__vcdiff = 0;
    SG_uint64 max_len_encoded__vcdiff = 0;
    SG_uint64 max_len_full__vcdiff = 0;

    SG_int_to_string_buffer buf;
    SG_blobset* pbs = NULL;

	SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );

    SG_ERR_CHECK(  SG_repo__list_blobs(pCtx, pRepo, 0, 0, 0, &pbs)  );

    SG_ERR_CHECK(  SG_blobset__get_stats(
                pCtx,
                pbs,
                0,
                &count_blobs__all,
                &len_encoded__all,
                &len_full__all,
                &max_len_encoded__all,
                &max_len_full__all
                )  );

    SG_ERR_CHECK(  SG_blobset__get_stats(
                pCtx,
                pbs,
                SG_BLOBENCODING__FULL,
                &count_blobs__full,
                &len_encoded__full,
                &len_full__full,
                &max_len_encoded__full,
                &max_len_full__full
                )  );

    SG_ERR_CHECK(  SG_blobset__get_stats(
                pCtx,
                pbs,
                SG_BLOBENCODING__ALWAYSFULL,
                &count_blobs__alwaysfull,
                &len_encoded__alwaysfull,
                &len_full__alwaysfull,
                &max_len_encoded__alwaysfull,
                &max_len_full__alwaysfull
                )  );

    SG_ERR_CHECK(  SG_blobset__get_stats(
                pCtx,
                pbs,
                SG_BLOBENCODING__ZLIB,
                &count_blobs__zlib,
                &len_encoded__zlib,
                &len_full__zlib,
                &max_len_encoded__zlib,
                &max_len_full__zlib
                )  );

    SG_ERR_CHECK(  SG_blobset__get_stats(
                pCtx,
                pbs,
                SG_BLOBENCODING__VCDIFF,
                &count_blobs__vcdiff,
                &len_encoded__vcdiff,
                &len_full__vcdiff,
                &max_len_encoded__vcdiff,
                &max_len_full__vcdiff
                )  );

    SG_BLOBSET_NULLFREE(pCtx, pbs);

    printf("full\n");
    printf("%12s  %d\n", "count", count_blobs__full);
    printf("%12s  %12s\n", "total", SG_int64_to_sz(len_encoded__full, buf));

    printf("alwaysfull\n");
    printf("%12s  %d\n", "count", count_blobs__alwaysfull);
    printf("%12s  %12s\n", "total", SG_int64_to_sz(len_encoded__alwaysfull, buf));

    printf("zlib\n");
    printf("%12s  %d\n", "count", count_blobs__zlib);
    printf("%12s  %12s\n", "full", SG_int64_to_sz(len_full__zlib,buf));
    printf("%12s  %12s\n", "encoded", SG_int64_to_sz(len_encoded__zlib,buf));
    if (len_full__zlib)
    {
        printf("%12s  %d%%\n", "saved", (int) ((len_full__zlib - len_encoded__zlib) / (double) len_full__zlib * 100.0));
    }

    printf("vcdiff\n");
    printf("%12s  %d\n", "count", count_blobs__vcdiff);
    printf("%12s  %12s\n", "full", SG_int64_to_sz(len_full__vcdiff,buf));
    printf("%12s  %12s\n", "encoded", SG_int64_to_sz(len_encoded__vcdiff,buf));
    if (len_full__vcdiff)
    {
        printf("%12s  %d%%\n", "saved", (int) ((len_full__vcdiff - len_encoded__vcdiff) / (double) len_full__vcdiff * 100.0));
    }

    printf("all\n");
    printf("%12s  %d\n", "count", count_blobs__all);
    printf("%12s  %12s\n", "full", SG_int64_to_sz(len_full__all,buf));
    printf("%12s  %12s\n", "encoded", SG_int64_to_sz(len_encoded__all,buf));
    printf("%12s  %d%%\n", "saved", (int) ((len_full__all - len_encoded__all) / (double) len_full__all * 100.0));

    /* fall through */

fail:
    SG_REPO_NULLFREE(pCtx, pRepo);
}

#if 0
void do_cmd_slurp(SG_context * pCtx, const char* psz_descriptor_name, const char* psz_fragball_name)
{
    SG_pathname* pPath_fragball = NULL;
    SG_repo* pRepo = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath_fragball, psz_fragball_name)  );

	SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );

    SG_ERR_CHECK(  SG_fragball__slurp(pCtx, pRepo, pPath_fragball)  );

    /* fall through */

fail:
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_PATHNAME_NULLFREE(pCtx, pPath_fragball);
}
#endif

void do_cmd_zip(SG_context * pCtx, SG_option_state* pOptSt, const char* psz_path)
{
	SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
	char* pszHidCs = NULL;
	const char* pszRefHidCs;
	SG_pendingtree* pPendingTree = NULL;

	SG_ERR_CHECK(  SG_cmd_util__get_repo_from_options_or_cwd(pCtx, pOptSt, &pRepo, &pPathCwd)  );

	{
		SG_uint32 countRevSpecs;
		SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &countRevSpecs)  );
		if (countRevSpecs)
		{
			SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pOptSt->pRevSpec, &pszHidCs, NULL)  );
			pszRefHidCs = pszHidCs;
		}
		else
		{
			/* No rev specified, try to use working folder. */
			if (!pPathCwd)
			{
				SG_ERR_THROW2(SG_ERR_NOT_A_WORKING_COPY, (pCtx, "you must specify a revision to zip."));
			}
			else
			{
				const SG_varray* pvaRefWdParents;
				SG_uint32 countParents;
				SG_ERR_CHECK(  SG_pendingtree__alloc_from_cwd(pCtx, &pPendingTree)  );
				SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pvaRefWdParents)  );
				SG_ERR_CHECK(  SG_varray__count(pCtx, pvaRefWdParents, &countParents)  );
				if (countParents != 1)
					SG_ERR_THROW2(SG_ERR_CANNOT_DO_WHILE_UNCOMMITTED_MERGE, (pCtx, "you must specify a revision to zip."));
				SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaRefWdParents, 0, &pszRefHidCs)  );
			}
		}
	}

    SG_ERR_CHECK_RETURN(  SG_repo__zip(pCtx, pRepo, pszRefHidCs, psz_path)  );

	/* Common cleanup */
fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_NULLFREE(pCtx, pszHidCs);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void do_cmd_clone(
        SG_context * pCtx, 
        const char* psz_existing, 
        const char* psz_new,
        SG_option_state * pOptSt
        )
{
    SG_vhash* pvh_changes = NULL;
    SG_clone* pClone = NULL;
    struct SG_clone__params__all_to_something ats;
    struct SG_clone__demands demands;

    memset(&ats, 0, sizeof(ats));
    ats.low_pass = 0;
    ats.high_pass = 0;

    SG_ERR_CHECK(  SG_clone__init_demands(pCtx, &demands)  );
    demands.zlib_savings_over_full = -1;
    demands.vcdiff_savings_over_full = -1;
    demands.vcdiff_savings_over_zlib = -1;

    if (pOptSt->b_all_full)
    {
        ats.new_encoding = SG_BLOBENCODING__KEEPFULLFORNOW;
        SG_ERR_CHECK(  SG_clone__all_to_something(
                    pCtx, 
                    psz_existing, 
                    psz_new, 
                    &ats,
                    &demands
                    )  );
    }
    else if (pOptSt->b_all_zlib)
    {
        ats.new_encoding = SG_BLOBENCODING__ZLIB;
        SG_ERR_CHECK(  SG_clone__all_to_something(
                    pCtx, 
                    psz_existing, 
                    psz_new, 
                    &ats,
                    &demands
                    )  );
    }
    else if (pOptSt->bPack)
    {
        SG_ERR_CHECK(  SG_clone__pack(
                    pCtx, 
                    psz_existing, 
                    psz_new, 
                    SG_PACK_DEFAULT__KEYFRAMEDENSITY, 
                    SG_PACK_DEFAULT__REVISIONSTOLEAVEFULL, 
                    SG_PACK_DEFAULT__MINREVISIONS,
                    0,
                    0,
                    &demands
                    )  );
    }
    else
    {
        SG_ERR_CHECK(  SG_clone__exact(pCtx, psz_existing, psz_new)  );
    }

	// Save default push/pull location for the new repository instance.
	SG_ERR_CHECK(  SG_localsettings__descriptor__update__sz(pCtx, psz_new, SG_LOCALSETTING__PATHS_DEFAULT, psz_existing)  );

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Use 'vv checkout %s <path>' to get a working copy.\n", psz_new)  );

	/* fall through*/
fail:
    SG_VHASH_NULLFREE(pCtx, pvh_changes);
    SG_ERR_IGNORE(  SG_clone__cleanup(pCtx, &pClone)  );
}

void do_cmd_comment(SG_context * pCtx, const char* psz_hid_cs, const char* psz_comment)
{
    SG_repo* pRepo = NULL;
    SG_audit q;

	// try to assoc one with the current wd

	SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL);
	if (SG_context__err_equals(pCtx, SG_ERR_NOT_A_WORKING_COPY))
	{
		SG_context__err_reset(pCtx);
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "There is no repository associated with the current directory.  Please provide a repository.\n")  );
		return;
	}

	SG_ERR_CHECK_CURRENT;


    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );
    SG_ERR_CHECK(  SG_vc_comments__add(pCtx, pRepo, psz_hid_cs, psz_comment, &q)  );

    SG_REPO_NULLFREE(pCtx, pRepo);

	return;

fail:
    SG_REPO_NULLFREE(pCtx, pRepo);
}

void do_cmd_tag(SG_context * pCtx, const char* szRepo, SG_bool plural, SG_uint32 argc, const char ** argv, SG_rev_spec* pRevSpec)
{
	SG_repo* pRepo = NULL;
	SG_audit q;
	SG_uint32 countRevSpecs = 0;
	SG_stringarray* psaHidChangesets = NULL;
	SG_uint32 count;
	char * pszMyHid = NULL;
	const char * pszHid = NULL;
	SG_rbtree * prbTags = NULL;
	SG_rbtree_iterator * pIter = NULL;
	SG_dagnode* pDagnode = NULL;

	if(argc==2 && strcmp(argv[0],"add")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		if (pRevSpec != NULL)
			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
		if (countRevSpecs <= 0)
		{
			if(szRepo==NULL)
			{
				SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, NULL, NULL, &psaHidChangesets)  );
				SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaHidChangesets, &count)  );
				if(count>1)
					SG_ERR_THROW2(SG_ERR_TAG_CONFLICT, (pCtx, "Cannot apply tag to multiple parent changesets."));
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, 0, &pszHid)  );
			}
			else
				SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "When using the '--repo' option you must specify which specific changeset to apply the tag to."));
		}
		else
		{
			SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pRevSpec, &pszMyHid, NULL)  );
			pszHid = pszMyHid;
		}

		SG_ERR_CHECK(  SG_vc_tags__add(pCtx, pRepo, pszHid, argv[1], &q)  );
	}
	else if(argc==2 && strcmp(argv[0],"move")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		if (pRevSpec != NULL)
			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
		if (countRevSpecs <= 0)
		{
			if(szRepo==NULL)
			{
				SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, NULL, NULL, &psaHidChangesets)  );
				SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaHidChangesets, &count)  );
				if(count>1)
					SG_ERR_THROW2(SG_ERR_TAG_CONFLICT, (pCtx, "Cannot apply tag to multiple parent changesets."));
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaHidChangesets, 0, &pszHid)  );
			}
			else
				SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "When using the '--repo' option you must specify which specific changeset to apply the tag to."));
		}
		else
		{
			SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pRevSpec, &pszMyHid, NULL)  );
			pszHid = pszMyHid;
		}

		SG_ERR_CHECK(  SG_vc_tags__remove(pCtx, pRepo, &q, argc-1, argv+1)  );
		SG_ERR_CHECK(  SG_vc_tags__add(pCtx, pRepo, pszHid, argv[1], &q)  );
	}
	else if(argc==2 && strcmp(argv[0],"remove")==0)
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
		SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

		if (pRevSpec != NULL)
			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
		if (countRevSpecs > 0)
			SG_ERR_THROW2(SG_ERR_USAGE, (pCtx, "Do not specify a revision when removing a tag."));

		SG_ERR_CHECK(  SG_vc_tags__remove(pCtx, pRepo, &q, argc-1, argv+1)  );
	}
	else if((argc==1 && strcmp(argv[0],"list")==0) || (argc==0 && plural))
	{
		if(szRepo)
			SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, szRepo, &pRepo)  );
		else
			SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

		SG_ERR_CHECK(  SG_vc_tags__list(pCtx, pRepo, &prbTags)  );

		if (prbTags)
		{
			SG_bool ok;
			const char* psz_tag = NULL;
			const char* psz_csid = NULL;
			SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pIter, prbTags, &ok, &psz_tag, (void**)&psz_csid)  );
			while(ok)
			{
				SG_uint32 revno;

				SG_ERR_CHECK(  SG_repo__fetch_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_csid, &pDagnode)  );
				SG_ERR_CHECK(  SG_dagnode__get_revno(pCtx, pDagnode, &revno)  );
				SG_DAGNODE_NULLFREE(pCtx, pDagnode);
		
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%20s: %6u:%s\n", psz_tag, revno, psz_csid)  );

				SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pIter, &ok, &psz_tag, (void**)&psz_csid)  );
			}
			SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
			SG_RBTREE_NULLFREE(pCtx, prbTags);
		}
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}


fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_STRINGARRAY_NULLFREE(pCtx, psaHidChangesets);
	SG_NULLFREE(pCtx, pszMyHid);
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
	SG_RBTREE_NULLFREE(pCtx, prbTags);
	SG_DAGNODE_NULLFREE(pCtx, pDagnode);
}

void do_cmd_add(SG_context * pCtx, SG_bool bTest,
				SG_uint32 count_args, const char** paszArgs,
				SG_bool bRecursive,
				const SG_file_spec* pFilespec)
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__add(pCtx, pPendingTree, count_args, paszArgs, bRecursive, pFilespec, bTest)  );

	if (bTest == SG_TRUE)
		sg_report_action_log(pCtx, pPendingTree);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);

	return;

fail:
	if (pPendingTree && SG_context__err_equals(pCtx,SG_ERR_PORTABILITY_WARNINGS))
		SG_ERR_IGNORE(  sg_report_portability_warnings(pCtx,pPendingTree)  );
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void do_zingmerge(SG_context * pCtx, const char* psz_descriptor_name)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
    char* psz_admin_id = NULL;
    SG_varray* pva_log = NULL;

    if (psz_descriptor_name)
    {
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
    }

    SG_ERR_CHECK(  SG_zing__auto_merge_all_dags(pCtx, pRepo)  );

    /* fall through */

fail:
    SG_NULLFREE(pCtx, psz_admin_id);
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
	SG_VARRAY_NULLFREE(pCtx, pva_log);
}

void do_reindex(SG_context * pCtx, const char* psz_descriptor_name)
{
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;

    if (psz_descriptor_name)
    {
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_descriptor_name, &pRepo)  );
    }
    else
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );
    }

    SG_ERR_CHECK(  SG_repo__rebuild_indexes(pCtx, pRepo)  );

    /* fall through */

fail:
    SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
}

void do_dump_json(SG_context * pCtx, const char* psz_repo, const char* psz_spec_hid)
{
    SG_repo* pRepo = NULL;
    SG_vhash* pvh = NULL;
    SG_string* pstr = NULL;
    char* psz_hid = NULL;

    /* open the repo */
	if (psz_repo)
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_repo, &pRepo)  );
	else
		SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

    /* lookup the blob id */
    SG_ERR_CHECK(  SG_repo__hidlookup__blob(pCtx, pRepo, psz_spec_hid, &psz_hid)  );

    /* fetch the blob */
    SG_ERR_CHECK(  SG_repo__fetch_vhash(pCtx, pRepo, psz_hid, &pvh)  );
    SG_REPO_NULLFREE(pCtx, pRepo);

    /* print it out.  fix the CRLF */
    SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
    SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh, pstr)  );
    SG_VHASH_NULLFREE(pCtx, pvh);

    SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pstr,
                                          (SG_byte *)"\r\n",2,
                                          (SG_byte *)"\n",1,
                                          SG_UINT32_MAX,SG_TRUE,NULL)  );

    SG_console(pCtx, SG_CS_STDOUT, "%s", SG_string__sz(pstr));

    SG_NULLFREE(pCtx, psz_hid);
    SG_STRING_NULLFREE(pCtx, pstr);

	return;

fail:
    SG_NULLFREE(pCtx, psz_hid);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_STRING_NULLFREE(pCtx, pstr);
    SG_VHASH_NULLFREE(pCtx, pvh);
}

void do_cmd_dump_json(SG_context * pCtx, const char* psz_repo, SG_uint32 count_args, const char** paszArgs)
{
    SG_uint32 i;

    for (i=0; i<count_args; i++)
    {
        SG_ERR_CHECK_RETURN(  do_dump_json(pCtx, psz_repo, paszArgs[i])  );
    }
}

void do_cmd_walk_fulltree(SG_context * pCtx)
{
	SG_pendingtree * pPendingTree = NULL;
	SG_ERR_CHECK(  SG_pendingtree__alloc_from_cwd(pCtx, &pPendingTree)  );
	SG_ERR_CHECK(  SG_pendingtree__walk_tree(pCtx, pPendingTree)  );
fail:
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void do_manually_set_parents(SG_context * pCtx, SG_rev_spec* pRevSpec)
{
	SG_pendingtree * pPendingTree = NULL;
	SG_stringarray * psaHidChangesets = NULL;
	SG_varray * pvaHidChangesets = NULL;
	SG_repo * pRepo = NULL;
	
	SG_ERR_CHECK(  SG_pendingtree__alloc_from_cwd(pCtx, &pPendingTree)  );
	SG_ERR_CHECK(  SG_VARRAY__ALLOC(pCtx, &pvaHidChangesets)  );
	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );

	SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pRevSpec, SG_FALSE, &psaHidChangesets)  );
	SG_ERR_CHECK(  SG_varray__alloc__stringarray(pCtx, &pvaHidChangesets, psaHidChangesets)  );

	SG_ERR_CHECK(  SG_pendingtree__set_wd_parents(pCtx,  pPendingTree, &pvaHidChangesets)  );
	SG_ERR_CHECK(  SG_pendingtree__save(pCtx, pPendingTree)  );

	/* common cleanup */
fail:
	SG_STRINGARRAY_NULLFREE(pCtx, psaHidChangesets);
	SG_VARRAY_NULLFREE(pCtx, pvaHidChangesets);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

#ifdef DEBUG
void do_cmd_dump_treenode(SG_context * pCtx, const char * psz_repo, const char * psz_spec_cs)
{
	SG_pathname * pPathCwd = NULL;
    SG_repo* pRepo = NULL;
	SG_pendingtree* pPendingTree = NULL;
	char * psz_hid_cs = NULL;
	char * szHidSuperRoot = NULL;
	SG_string * pStrDump = NULL;

	if (psz_repo)
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_repo, &pRepo)  );
	else
		SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPathCwd)  );

	if (psz_spec_cs && *psz_spec_cs)
	{
		SG_ERR_CHECK(  SG_repo__hidlookup__dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_spec_cs, &psz_hid_cs)  );
	}
	else
	{
		// assume the current baseline.  NOTE that our dump will not show the dirt in the WD.
		//
		// when we have an uncomitted merge, we will have more than one parent.
		// what does this command mean then?  It feels like we we should throw
		// an error and say that you have to commit first.

		const SG_varray * pva_wd_parents;		// we do not own this
		const char * psz_hid_parent_0;			// we do not own this
		SG_uint32 nrParents;

		SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPathCwd, &pPendingTree)  );
		SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pva_wd_parents)  );
		SG_ERR_CHECK(  SG_varray__count(pCtx, pva_wd_parents, &nrParents)  );
		if (nrParents > 1)
			SG_ERR_THROW(  SG_ERR_CANNOT_DO_WHILE_UNCOMMITTED_MERGE  );

		SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pva_wd_parents, 0, &psz_hid_parent_0)  );
		SG_ERR_CHECK(  SG_STRDUP(pCtx, psz_hid_parent_0, &psz_hid_cs)  );
	}

	// load the contents of the desired CSET and get the super-root treenode.

    SG_ERR_CHECK(  SG_changeset__tree__find_root_hid(pCtx, pRepo, psz_hid_cs, &szHidSuperRoot)  );

	// let the treenode dump routine populate our string.
	// TODO we may want to crack this open and dump the user's actual root.

	SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pStrDump)  );
	SG_ERR_CHECK(  SG_treenode_debug__dump__by_hid(pCtx,szHidSuperRoot,pRepo,4,SG_TRUE,pStrDump)  );

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "CSet[%s]:\n%s\n", psz_hid_cs, SG_string__sz(pStrDump))  );

    /* fall through to common cleanup */

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
    SG_NULLFREE(pCtx, psz_hid_cs);
	SG_NULLFREE(pCtx, szHidSuperRoot);
	SG_STRING_NULLFREE(pCtx, pStrDump);
}
#endif


void do_cmd_hid(SG_context * pCtx, const char* pszPath)
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
    SG_repo* pRepo = NULL;
    SG_pathname* pPath = NULL;
    SG_file* pFile = NULL;
    char* pszHid = NULL;

	// we need a pRepo because HIDs are now repo-specific.

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );
    SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );

    SG_PATHNAME__ALLOC__SZ(pCtx, &pPath, pszPath);
    SG_ERR_CHECK(  SG_file__open__pathname(pCtx, pPath, SG_FILE_RDONLY | SG_FILE_OPEN_EXISTING, SG_FSOBJ_PERMS__UNUSED, &pFile)  );
    SG_ERR_CHECK(  SG_repo__alloc_compute_hash__from_file(pCtx, pRepo, pFile, &pszHid)  );

    SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", pszHid)  );

fail:
	SG_FILE_NULLCLOSE(pCtx, pFile);
    SG_NULLFREE(pCtx, pszHid);
    SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

static void x_try_to_find_path(
        SG_context * pCtx, 
        SG_repo* pRepo,
        const char* psz_gid,
        const char* psz_csid,
        const char* psz_branch_name,
        SG_string** ppstr)
{
    SG_string* pstr = NULL;

    SG_UNUSED(psz_branch_name);

    SG_repo__treendx__get_path_in_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_gid, psz_csid, &pstr, NULL);
    if (SG_context__err_equals(pCtx, SG_ERR_CHANGESET_BLOB_NOT_FOUND))
    {
        SG_ERR_DISCARD;
        // TODO look elsewhere
    }
    else
    {
		SG_ERR_CHECK_CURRENT;
        // TODO use it
    }

    *ppstr = pstr;
    pstr = NULL;

fail:
    SG_STRING_NULLFREE(pCtx, pstr);
}

void do_cmd_locks(
        SG_context * pCtx, 
        SG_option_state * pOptSt
        )
{
	SG_string* pstr_descriptor = NULL;
	SG_string* pstr_repo_path = NULL;
	SG_pathname* pPath_cwd = NULL;
	SG_pathname* pPath_top = NULL;
    char* psz_tied_branch_name = NULL;
    char* psz_repo_upstream = NULL;
    char* psz_csid_parent = NULL;
    char* psz_userid = NULL;
    SG_repo* pRepo = NULL;
    SG_audit q;
    SG_vhash* pvh_locks_by_branch = NULL;
    SG_vhash* pvh_duplicate_locks = NULL;
    SG_vhash* pvh_violated_locks = NULL;
    //SG_string* pstr = NULL;

    // first we need to get the descriptor name from the working dir
    // if this is not a working dir, just fail
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPath_cwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPath_cwd)  );

	SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath_cwd, &pPath_top, &pstr_descriptor, NULL)  );

    // now we need to know what branch we are tied to.
    // if we're not tied, fail
    SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );
    if (!psz_tied_branch_name)
    {
        SG_ERR_THROW(  SG_ERR_NOT_TIED  );
    }

    // now we need the current userid
    SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, SG_string__sz(pstr_descriptor), &pRepo)  );
    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

    if (pOptSt->bPull)
    {
        if (pOptSt->psz_server)
        {
            SG_ERR_CHECK(  SG_STRDUP(pCtx, pOptSt->psz_server, &psz_repo_upstream)  );
        }
        else
        {
            SG_ERR_CHECK(  SG_localsettings__descriptor__get__sz(pCtx, SG_string__sz(pstr_descriptor), "paths/default", &psz_repo_upstream)  );
        }

        SG_ERR_CHECK(  SG_vc_locks__pull_locks_and_branches(pCtx, psz_repo_upstream, SG_string__sz(pstr_descriptor))  );
    }

    SG_ERR_CHECK(  SG_vc_locks__check__okay_to_push(
                pCtx, 
                pRepo, 
                NULL,
                &pvh_locks_by_branch,
                &pvh_duplicate_locks,
                &pvh_violated_locks
                )  );

    if (pvh_duplicate_locks)
    {
        SG_uint32 count_branches_with_duplicate_locks = 0;
        SG_uint32 i_branch = 0;

        SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_duplicate_locks, &count_branches_with_duplicate_locks)  );
        for (i_branch=0; i_branch<count_branches_with_duplicate_locks; i_branch++)
        {
            const char* psz_branch = NULL;
            SG_vhash* pvh_gids_for_one_branch = NULL;

            SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_duplicate_locks, i_branch, &psz_branch, &pvh_gids_for_one_branch)  );

            if (pvh_gids_for_one_branch)
            {
                SG_uint32 count_gids = 0;
                SG_uint32 i_gid = 0;

                SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
                for (i_gid=0; i_gid<count_gids; i_gid++)
                {
                    SG_vhash* pvh_locks_for_one_gid = NULL;
                    SG_uint32 count_locks = 0;
                    SG_uint32 i_lock = 0;
                    const char* psz_gid = NULL;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );

                    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
                    for (i_lock=0; i_lock<count_locks; i_lock++)
                    {
                        const char* psz_hidrec = NULL;
                        SG_vhash* pvh_lock = NULL;
                        const char* psz_csid = NULL;

                        SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hidrec, &pvh_lock)  );
                        SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "csid", &psz_csid)  );
                        // TODO psz_csid isn't necessarily the one we want to use here.
                        SG_ERR_CHECK(  SG_repo__treendx__get_path_in_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_gid, psz_csid, &pstr_repo_path, NULL)  );

                        SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_lock, "path", SG_string__sz(pstr_repo_path))  );
                        SG_STRING_NULLFREE(pCtx, pstr_repo_path);
                    }
                }
            }
        }
    }

    if (pvh_violated_locks)
    {
        SG_uint32 count = 0;
        SG_uint32 i = 0;

        SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_violated_locks, &count)  );
        for (i=0; i<count; i++)
        {
            const char* psz_hidrec = NULL;
            SG_vhash* pvh_lock = NULL;
            const char* psz_gid = NULL;
            const char* psz_csid = NULL;

            SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_violated_locks, i, &psz_hidrec, &pvh_lock)  );
            SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "gid", &psz_gid)  );
            SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "csid", &psz_csid)  );
            // TODO psz_csid isn't necessarily the one we want to use here.
            SG_ERR_CHECK(  SG_repo__treendx__get_path_in_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, psz_gid, psz_csid, &pstr_repo_path, NULL)  );
            SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_lock, "path", SG_string__sz(pstr_repo_path))  );
            SG_STRING_NULLFREE(pCtx, pstr_repo_path);
        }
    }

    if (pvh_locks_by_branch)
    {
        SG_vhash* pvh_gids_for_one_branch = NULL;

        SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_locks_by_branch, psz_tied_branch_name, &pvh_gids_for_one_branch)  );
        if (pvh_gids_for_one_branch)
        {
            SG_uint32 count_gids = 0;
            SG_uint32 i_gid = 0;

            SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
            for (i_gid=0; i_gid<count_gids; i_gid++)
            {
                SG_vhash* pvh_locks_for_one_gid = NULL;
                SG_uint32 count_locks = 0;
                SG_uint32 i_lock = 0;
                const char* psz_gid = NULL;

                SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );

                SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
                for (i_lock=0; i_lock<count_locks; i_lock++)
                {
                    const char* psz_hidrec = NULL;
                    SG_vhash* pvh_lock = NULL;
                    const char* psz_csid = NULL;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hidrec, &pvh_lock)  );
                    SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "csid", &psz_csid)  );
                    SG_ERR_CHECK(  x_try_to_find_path(pCtx, pRepo, psz_gid, psz_csid, psz_tied_branch_name, &pstr_repo_path)  );

                    if (pstr_repo_path)
                    {
                        SG_ERR_CHECK(  SG_vhash__add__string__sz(pCtx, pvh_lock, "path", SG_string__sz(pstr_repo_path))  );
                        SG_STRING_NULLFREE(pCtx, pstr_repo_path);
                    }
                }
            }
        }
    }

#if 0
    if (pvh_duplicate_locks)
    {
        // TODO make this output friendlier
        SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
        SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh_duplicate_locks, pstr)  );
        printf("Duplicate locks:\n");
        printf("%s\n", SG_string__sz(pstr));
        SG_STRING_NULLFREE(pCtx, pstr);
    }
#endif

#if 0
    if (pvh_violated_locks)
    {
        // TODO make this output friendlier
        SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
        SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh_violated_locks, pstr)  );
        printf("Violated locks:\n");
        printf("%s\n", SG_string__sz(pstr));
        SG_STRING_NULLFREE(pCtx, pstr);
    }
#endif

    if (pvh_locks_by_branch)
    {
        SG_vhash* pvh_gids_for_one_branch = NULL;

        SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_locks_by_branch, psz_tied_branch_name, &pvh_gids_for_one_branch)  );
        if (pvh_gids_for_one_branch)
        {
#if 0
            // TODO make this output friendlier
            SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstr)  );
            SG_ERR_CHECK(  SG_vhash__to_json__pretty_print_NOT_for_storage(pCtx, pvh_gids_for_one_branch, pstr)  );
            printf("Locks on this branch (%s):\n", psz_tied_branch_name);
            printf("%s\n", SG_string__sz(pstr));
            SG_STRING_NULLFREE(pCtx, pstr);
#else
            SG_uint32 count_gids = 0;
            SG_uint32 i_gid = 0;

            SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Locks on this branch (%s):\n", psz_tied_branch_name)  );
            SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_gids_for_one_branch, &count_gids)  );
            for (i_gid=0; i_gid<count_gids; i_gid++)
            {
                SG_vhash* pvh_locks_for_one_gid = NULL;
                SG_uint32 count_locks = 0;
                SG_uint32 i_lock = 0;
                const char* psz_gid = NULL;

                SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_gids_for_one_branch, i_gid, &psz_gid, &pvh_locks_for_one_gid)  );

                SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_locks_for_one_gid, &count_locks)  );
                for (i_lock=0; i_lock<count_locks; i_lock++)
                {
                    const char* psz_hid_lock = NULL;
                    SG_vhash* pvh_lock = NULL;
                    const char* psz_username = NULL;
                    const char* psz_path = NULL;
                    SG_bool b_violated = SG_FALSE;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair__vhash(pCtx, pvh_locks_for_one_gid, i_lock, &psz_hid_lock, &pvh_lock)  );

                    if (pvh_violated_locks)
                    {
                        SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_violated_locks, psz_hid_lock, &b_violated)  );
                    }

                    SG_ERR_CHECK(  SG_vhash__get__sz(pCtx, pvh_lock, "username", &psz_username)  );
                    SG_ERR_CHECK(  SG_vhash__check__sz(pCtx, pvh_lock, "path", &psz_path)  );
                    if (psz_path)
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%8s %s: %s\n", (b_violated ? "VIOLATED" : ""), psz_username, psz_path)  );
                    }
                    else
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%8s %s: (object %s)\n", (b_violated ? "VIOLATED" : ""), psz_username, psz_gid)  );
                    }
                }

            }
#endif
        }
    }

fail:
    SG_STRING_NULLFREE(pCtx, pstr_repo_path);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_NULLFREE(pCtx, psz_repo_upstream);
    SG_STRING_NULLFREE(pCtx, pstr_descriptor);
	SG_NULLFREE(pCtx, psz_userid);
	SG_NULLFREE(pCtx, psz_csid_parent);
	SG_NULLFREE(pCtx, psz_tied_branch_name);
    SG_PATHNAME_NULLFREE(pCtx, pPath_cwd);
    SG_PATHNAME_NULLFREE(pCtx, pPath_top);
    SG_VHASH_NULLFREE(pCtx, pvh_locks_by_branch);
    SG_VHASH_NULLFREE(pCtx, pvh_violated_locks);
    SG_VHASH_NULLFREE(pCtx, pvh_duplicate_locks);
}

void do_cmd_unlock(
    SG_context * pCtx,
    SG_bool b_force,
    SG_uint32 count_args, 
    const char** paszArgs
    )
{
	SG_string* pstr_descriptor = NULL;
	SG_pathname* pPath_cwd = NULL;
	SG_pathname* pPath_top = NULL;
	SG_pendingtree* pPendingTree = NULL;
    char* psz_tied_branch_name = NULL;
    char* psz_userid = NULL;
    SG_repo* pRepo = NULL;
    SG_audit q;
    SG_vhash* pvh_gids = NULL;

    if (0 == count_args)
    {
        SG_ERR_THROW2(  SG_ERR_INVALIDARG,
                       (pCtx, "Nothing to unlock")  );
    }

    // first we need to get the descriptor name from the working dir
    // if this is not a working dir, just fail
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPath_cwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPath_cwd)  );

	SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath_cwd, &pPath_top, &pstr_descriptor, NULL)  );

    // now we need to know what branch we are tied to.
    // if we're not tied, fail
    SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );
    if (!psz_tied_branch_name)
    {
        SG_ERR_THROW(  SG_ERR_NOT_TIED  );
    }

    // and the gids of all the objects to be unlocked
    SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPath_cwd, &pPendingTree)  );
    SG_ERR_CHECK(  SG_pendingtree__find_gids_for_locking__must_be_files(pCtx, pPendingTree, pPath_cwd, count_args, paszArgs, &pvh_gids)  );
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

    // now we need the current userid
    SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, SG_string__sz(pstr_descriptor), &pRepo)  );
    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );
    SG_REPO_NULLFREE(pCtx, pRepo);

    // OK, we have all the pieces.  Time to call the unlock code
    SG_ERR_CHECK(  SG_vc_locks__unlock(
                pCtx,
                SG_string__sz(pstr_descriptor),
                psz_tied_branch_name,
                pvh_gids,
                b_force,
                &q
                )  );

fail:
    SG_VHASH_NULLFREE(pCtx, pvh_gids);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_STRING_NULLFREE(pCtx, pstr_descriptor);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_NULLFREE(pCtx, psz_userid);
	SG_NULLFREE(pCtx, psz_tied_branch_name);
    SG_PATHNAME_NULLFREE(pCtx, pPath_cwd);
    SG_PATHNAME_NULLFREE(pCtx, pPath_top);
}

void do_cmd_lock(
    SG_context * pCtx,
    SG_option_state* pOptSt,
    SG_uint32 count_args, 
    const char** paszArgs
    )
{
	SG_string* pstr_descriptor = NULL;
	SG_pathname* pPath_cwd = NULL;
	SG_pathname* pPath_top = NULL;
	SG_pendingtree* pPendingTree = NULL;
    char* psz_tied_branch_name = NULL;
    const SG_varray* pva_wd_parents = NULL;
    SG_uint32 count_parents = 0;
    const char* psz_csid_parent_ref = NULL;
    char* psz_repo_upstream = NULL;
    char* psz_csid_parent = NULL;
    char* psz_userid = NULL;
    SG_repo* pRepo = NULL;
    SG_audit q;
    SG_vhash* pvh_gids = NULL;

    if (0 == count_args)
    {
        SG_ERR_THROW2(  SG_ERR_INVALIDARG,
                       (pCtx, "Nothing to lock")  );
    }

    // first we need to get the descriptor name from the working dir
    // if this is not a working dir, just fail
	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPath_cwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPath_cwd)  );

	SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath_cwd, &pPath_top, &pstr_descriptor, NULL)  );

    // now we need to know what branch we are tied to.
    // if we're not tied, fail
    SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );
    if (!psz_tied_branch_name)
    {
        SG_ERR_THROW(  SG_ERR_NOT_TIED  );
    }

    // now we need the single parent csid of this working dir
    // if more than one, fail
    SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC(pCtx, pPath_cwd, &pPendingTree)  );
    SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pva_wd_parents)  );
    SG_ERR_CHECK(  SG_varray__count(pCtx, pva_wd_parents, &count_parents)  );
    if (count_parents > 1)
    {
        SG_ERR_THROW(  SG_ERR_CANNOT_DO_WHILE_UNCOMMITTED_MERGE  );
    }

    SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pva_wd_parents, 0, &psz_csid_parent_ref)  );
    SG_ERR_CHECK(  SG_STRDUP(pCtx, psz_csid_parent_ref, &psz_csid_parent)  );

    // and the gids of all the objects to be locked
    SG_ERR_CHECK(  SG_pendingtree__find_gids_for_locking__must_be_files(pCtx, pPendingTree, pPath_cwd, count_args, paszArgs, &pvh_gids)  );
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

    // now we need the current userid
    SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, SG_string__sz(pstr_descriptor), &pRepo)  );
    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );
    SG_REPO_NULLFREE(pCtx, pRepo);

    // we need the repo spec for the other side
    if (pOptSt->psz_server)
    {
        SG_ERR_CHECK(  SG_STRDUP(pCtx, pOptSt->psz_server, &psz_repo_upstream)  );
    }
    else
    {
		SG_localsettings__descriptor__get__sz(pCtx, SG_string__sz(pstr_descriptor), "paths/default", &psz_repo_upstream);
		if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
		{
			SG_ERR_DISCARD;
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nNo default path and no server specified.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		SG_ERR_CHECK_CURRENT;
    }

    // OK, we have all the pieces.  Time to request the lock
    SG_ERR_CHECK(  SG_vc_locks__request(
                pCtx,
                SG_string__sz(pstr_descriptor),
                psz_repo_upstream,
                psz_csid_parent,
                psz_tied_branch_name,
                pvh_gids,
                &q
                )  );

fail:
    SG_VHASH_NULLFREE(pCtx, pvh_gids);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_NULLFREE(pCtx, psz_repo_upstream);
    SG_STRING_NULLFREE(pCtx, pstr_descriptor);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_NULLFREE(pCtx, psz_userid);
	SG_NULLFREE(pCtx, psz_csid_parent);
	SG_NULLFREE(pCtx, psz_tied_branch_name);
    SG_PATHNAME_NULLFREE(pCtx, pPath_cwd);
    SG_PATHNAME_NULLFREE(pCtx, pPath_top);
}

void do_cmd_remove(SG_context * pCtx,
				   SG_uint32 count_args, const char** paszArgs,
				   const SG_file_spec* pFilespec,
				   SG_bool bTest, SG_bool bForce, SG_bool bNoBackups)
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__remove(pCtx, pPendingTree, count_args, paszArgs, pFilespec, bTest, bForce, bNoBackups)  );

	if (bTest == SG_TRUE)
			sg_report_action_log(pCtx, pPendingTree);

	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);

	return;

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void do_cmd_rename(SG_context * pCtx, const char* pszPath, const char* pszNewName)
{
	SG_pathname* pPath = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	if (pszPath[0] == '@')
	{
		SG_ERR_CHECK(  SG_workingdir__construct_absolute_path_from_repo_path(pCtx, pPathCwd, pszPath, &pPath)  );
	}
	else
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath, pszPath)  );

	SG_ERR_CHECK(  SG_pendingtree__rename(pCtx, pPendingTree, pPath, pszNewName, SG_TRUE)  );

	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	return;

fail:
	if (pPendingTree && SG_context__err_equals(pCtx,SG_ERR_PORTABILITY_WARNINGS))
		SG_ERR_IGNORE(  sg_report_portability_warnings(pCtx,pPendingTree)  );
	SG_PATHNAME_NULLFREE(pCtx, pPath);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

void do_cmd_move(SG_context * pCtx, const char** paszPathsToMove, SG_uint32 nCountMoves, const char* pszPathMoveTo)
{
	SG_pathname* pPathnameTemp = NULL;
	SG_pathname* pPathMoveTo = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	SG_pathname ** newArgv = NULL;
	SG_uint32 i=0, n=0;
	SG_fsobj_type t = 0;
	SG_bool bExists = SG_FALSE;
	SG_uint32 errorCount = 0;
	SG_error nLastError = 0;

	SG_ERR_CHECK(  SG_alloc(pCtx, nCountMoves, sizeof(SG_pathname*), &newArgv)  );

	for (i = n; i < nCountMoves; i++)
	{
		SG_ERR_CHECK( SG_PATHNAME__ALLOC__SZ(pCtx, &pPathnameTemp, paszPathsToMove[i])  );
		newArgv[i] = pPathnameTemp;
	}

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPathMoveTo, pszPathMoveTo)  );

	/* TODO check to see if this exists before going down into the pending tree code? */

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	/* First make sure the paths exist */
	SG_ERR_CHECK(  SG_fsobj__exists__pathname(pCtx, pPathMoveTo, &bExists, &t, NULL)  );

	if (!bExists || t != SG_FSOBJ_TYPE__DIRECTORY)
	{
		SG_ERR_THROW2(  SG_ERR_NOT_A_DIRECTORY,
						(pCtx, "%s", SG_pathname__sz(pPathMoveTo))  );
	}
	for (i = 0; i < nCountMoves; i ++)
	{
		SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );
		SG_pendingtree__move(pCtx, pPendingTree, (const SG_pathname **)&newArgv[i], 1, pPathMoveTo);
		if (SG_context__has_err(pCtx))
		{
			if (i == nCountMoves - 1)
				SG_ERR_CHECK_CURRENT;  //We're on the last item to move, just throw the error.
			errorCount++;
			SG_context__get_err(pCtx, &nLastError);
			SG_context__err_to_console(pCtx, SG_CS_STDERR);
			SG_context__err_reset(pCtx);
		}
		SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	}

	if (errorCount == 1)
		SG_ERR_THROW2( nLastError, (pCtx, "1 error encountered." ) );
	if (errorCount > 1)
		SG_ERR_THROW2( nLastError, (pCtx, "%d errors encountered.", errorCount) );
	SG_PATHNAME_NULLFREE(pCtx, pPathMoveTo);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	for (i = n; i < nCountMoves; i++)
	{
		SG_PATHNAME_NULLFREE(pCtx, newArgv[i]);
	}
	SG_NULLFREE(pCtx, newArgv);
	return;

fail:
	if (pPendingTree && SG_context__err_equals(pCtx,SG_ERR_PORTABILITY_WARNINGS))
		SG_ERR_IGNORE(  sg_report_portability_warnings(pCtx,pPendingTree)  );
	for (i = n; i < nCountMoves; i++)
	{
		SG_PATHNAME_NULLFREE(pCtx, newArgv[i]);
	}
	SG_NULLFREE(pCtx, newArgv);
	SG_PATHNAME_NULLFREE(pCtx, pPathMoveTo);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
}

/**
 * List the heads/tips/leaves.
 */
void do_cmd_leaves(
        SG_context * pCtx, 
        SG_option_state * pOptSt
        )
{
	SG_rbtree * prbLeaves = NULL;
	SG_rbtree_iterator * pIter = NULL;
	SG_repo * pRepo = NULL;
	const char * pszHid_k;
    SG_vhash* pvh_pile = NULL;
	SG_bool bFound = SG_FALSE;

	if (pOptSt->psz_repo)
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pOptSt->psz_repo, &pRepo)  );
	else
		SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

#if defined(DEBUG)
	if ((pOptSt->bGaveDagNum) && (pOptSt->ui64DagNum != SG_DAGNUM__VERSION_CONTROL))
	{
		// can't use the history code to do fancy printout of non-vc dags...

		SG_uint32 count;
		
		SG_ERR_CHECK(  SG_repo__fetch_dag_leaves(pCtx,pRepo,pOptSt->ui64DagNum,&prbLeaves)  );
		SG_ERR_CHECK(  SG_rbtree__count(pCtx, prbLeaves, &count)  );
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "Found %d leaves:\n", count)  );
		SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx,&pIter,prbLeaves,&bFound,&pszHid_k,NULL)  );
		while (bFound)
		{
			SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", pszHid_k)  );

			SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx,pIter,&bFound,&pszHid_k,NULL)  );
		}

		goto fail;	// regular cleanup
	}
#endif

	SG_ERR_CHECK(  SG_repo__fetch_dag_leaves(pCtx,pRepo,SG_DAGNUM__VERSION_CONTROL,&prbLeaves)  );
    // we still ask for the branch pile so we can show the branch name
    // in the output
    SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );

	SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx,&pIter,prbLeaves,&bFound,&pszHid_k,NULL)  );
	while (bFound)
	{
        SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx,SG_CS_STDOUT,pRepo,pszHid_k,pvh_pile, SG_FALSE, pOptSt->bVerbose)  );

		SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx,pIter,&bFound,&pszHid_k,NULL)  );
	}

	// fall thru to common cleanup.

fail:
    SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pIter);
	SG_RBTREE_NULLFREE(pCtx, prbLeaves);
}

void do_cmd_heads(
        SG_context * pCtx, 
        const char* psz_repo,
		const SG_stringarray* psaRequestedBranchNames,
		SG_bool bAll,
		SG_bool bVerbose
        )
{
	SG_repo* pRepo = NULL;
	SG_rbtree* prbLeaves = NULL;
	SG_vhash* pvhPileBranches = NULL;
	SG_rbtree* prbRequestedBranchNames = NULL;
	SG_vhash* pvhRefClosedBranches = NULL;

	SG_rbtree* prbLeafHidsToOutput = NULL;
	SG_rbtree* prbNonLeafHidsToOutput = NULL;
	SG_rbtree_iterator* pit = NULL;

	if (psz_repo)
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, psz_repo, &pRepo)  );
	else
		SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

	SG_ERR_CHECK(  SG_repo__fetch_dag_leaves(pCtx,pRepo,SG_DAGNUM__VERSION_CONTROL,&prbLeaves)  );

	if (psaRequestedBranchNames)
	{
		/* GetOpt puts these in a stringarray. But we'll look them up in a loop below, so we copy them to an rbtree. */
		SG_ERR_CHECK(  SG_stringarray__to_rbtree_keys(pCtx, psaRequestedBranchNames, &prbRequestedBranchNames)  );
	}
    
	SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvhPileBranches)  );

	{
		SG_bool bHasClosedBranches;
		SG_ERR_CHECK(  SG_vhash__has(pCtx, pvhPileBranches, "closed", &bHasClosedBranches)  );
		if (bHasClosedBranches)
			SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvhPileBranches, "closed", &pvhRefClosedBranches)  );
	}

	/* Collect the HIDs of changesets we want to output. 
	 * We keep leaf and non-leaf changesets in separate lists because we want to display leaves first.
	 * We keep the lists in rbtrees to easily prevent duplicates, otherwise a changeset belonging to two branches could appear twice. */
	{
		SG_uint32 i, countAllBranches;
		SG_vhash* pvhRefBranches = NULL;
		
		SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvhPileBranches, "branches", &pvhRefBranches)  );
		SG_ERR_CHECK(  SG_vhash__count(pCtx, pvhRefBranches, &countAllBranches)  );
		SG_ERR_CHECK(  SG_RBTREE__ALLOC__PARAMS(pCtx, &prbLeafHidsToOutput, countAllBranches, NULL)  );
		SG_ERR_CHECK(  SG_RBTREE__ALLOC__PARAMS(pCtx, &prbNonLeafHidsToOutput, countAllBranches, NULL)  );

		for (i = 0; i < countAllBranches; i++)
		{
			const SG_variant* pv = NULL;
			const char* pszRefBranchName = NULL;
			SG_vhash* pvh_branch_info = NULL;
			SG_vhash* pvh_branch_values = NULL;
			SG_uint32 count_branch_values = 0;
			SG_uint32 j = 0;
			SG_bool bBranchIsClosed = SG_FALSE;
			SG_bool bBranchRequested = SG_TRUE;

			SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvhRefBranches, i, &pszRefBranchName, &pv)  );

			SG_ERR_CHECK(  SG_vhash__has(pCtx, pvhRefClosedBranches, pszRefBranchName, &bBranchIsClosed)  );
			if (prbRequestedBranchNames)
				SG_ERR_CHECK(  SG_rbtree__find(pCtx, prbRequestedBranchNames, pszRefBranchName, &bBranchRequested, NULL)  );
			
			if (bBranchRequested && (bAll || !bBranchIsClosed))
			{
				SG_ERR_CHECK(  SG_variant__get__vhash(pCtx, pv, &pvh_branch_info)  );
				SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
				SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_branch_values, &count_branch_values)  );

				for (j=0; j<count_branch_values; j++)
				{
					const char* pszRefHid = NULL;
					SG_bool bIsLeaf;
					SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_branch_values, j, &pszRefHid, NULL)  );
					SG_ERR_CHECK(  SG_rbtree__find(pCtx, prbLeaves, pszRefHid, &bIsLeaf, NULL)  );
					if (bIsLeaf)
						SG_ERR_CHECK(  SG_rbtree__update(pCtx, prbLeafHidsToOutput, pszRefHid)  );
					else
						SG_ERR_CHECK(  SG_rbtree__update(pCtx, prbNonLeafHidsToOutput, pszRefHid)  );
				}
			}
		}
	}

	/* Output the results. */
	{
		SG_bool bOk = SG_FALSE;
		const char* pszRefHid;

		/* Output leaves at the top of the list. */
		SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pit, prbLeafHidsToOutput, &bOk, &pszRefHid, NULL)  );
		while (bOk)
		{
			SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, pszRefHid, pvhPileBranches, !bAll, bVerbose)  );
			SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pit, &bOk, &pszRefHid, NULL)  );
		}
		SG_RBTREE_ITERATOR_NULLFREE(pCtx, pit);

		/* Then the non-leaves */
		SG_ERR_CHECK(  SG_rbtree__iterator__first(pCtx, &pit, prbNonLeafHidsToOutput, &bOk, &pszRefHid, NULL)  );
		while (bOk)
		{
			SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, pszRefHid, pvhPileBranches, !bAll, bVerbose)  );
			SG_ERR_CHECK(  SG_rbtree__iterator__next(pCtx, pit, &bOk, &pszRefHid, NULL)  );
		}
	}

	// fall thru to common cleanup.

fail:
    SG_VHASH_NULLFREE(pCtx, pvhPileBranches);
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_RBTREE_NULLFREE(pCtx, prbLeaves);
	SG_RBTREE_NULLFREE(pCtx, prbRequestedBranchNames);
	SG_RBTREE_NULLFREE(pCtx, prbLeafHidsToOutput);
	SG_RBTREE_NULLFREE(pCtx, prbNonLeafHidsToOutput);
	SG_RBTREE_ITERATOR_NULLFREE(pCtx, pit);
}

#if 0 // TODO doesn't seem like we need this anymore
static void _get_baseline(SG_context * pCtx, SG_pendingtree * pPendingTree, char ** ppszBaseline)
{
	const SG_varray * pvaParents;
	const char * pszBaseline;
	char * pszAllocated = NULL;

	// get the HID of the BASELINE (aka PARENT[0] from the pendingtree).

	SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pvaParents)  );
	SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaParents, 0, &pszBaseline)  );

	SG_ERR_CHECK(  SG_STRDUP(pCtx, pszBaseline, &pszAllocated)  );

	*ppszBaseline = pszAllocated;
	return;

fail:
	SG_NULLFREE(pCtx, pszAllocated);
}
#endif

static void my_get_tied_branch(
        SG_context* pCtx,
        char** ppsz_tied_branch
        )
{
    SG_pathname* pPath_cwd = NULL;
    SG_pathname* pPath_top = NULL;
    char* psz_tied_branch_name = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPath_cwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPath_cwd)  );

	SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath_cwd, &pPath_top, NULL, NULL)  );
    SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );

    *ppsz_tied_branch = psz_tied_branch_name;
    psz_tied_branch_name = NULL;

fail:
    SG_NULLFREE(pCtx, psz_tied_branch_name);
    SG_PATHNAME_NULLFREE(pCtx, pPath_top);
    SG_PATHNAME_NULLFREE(pCtx, pPath_cwd);
}

static void do_cmd_branch(
        SG_context* pCtx,
        SG_bool plural,
        SG_uint32 count_args, 
        const char** paszArgs,
        SG_option_state * pOptSt
        )
{
    SG_repo* pRepo = NULL;
    SG_vhash* pvh_pile = NULL;
    SG_pathname* pPath_cwd = NULL;
    SG_pathname* pPath_top = NULL;
    char* psz_tied_branch_name = NULL;
    char* psz_rev_1 = NULL;
    char* psz_rev_2 = NULL;
	SG_stringarray* psaFullHids = NULL;
	
    if (
        (0 == count_args && !plural)
        || (0 < count_args && 0 == strcmp(paszArgs[0], "new"))
        || (0 < count_args && 0 == strcmp(paszArgs[0], "attach"))
        || (0 < count_args && 0 == strcmp(paszArgs[0], "detach"))
       )
    {
        SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, &pPath_cwd)  );

        SG_ERR_CHECK(  SG_workingdir__find_mapping(pCtx, pPath_cwd, &pPath_top, NULL, NULL)  );

        if (0 == count_args)
        {
            SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );
            if (psz_tied_branch_name)
            {
                SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz_tied_branch_name)  );
            }
            else
            {
                // TODO print there is no current named branch
            }
        }
        else if (0 == strcmp(paszArgs[0], "new"))
        {
            SG_bool b_has = SG_FALSE;
            SG_vhash* pvh_pile_branches = NULL;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
            SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_pile_branches, paszArgs[1], &b_has)  );
            if (b_has)
            {
                SG_ERR_THROW(  SG_ERR_BRANCH_ALREADY_EXISTS  );
            }

            SG_ERR_CHECK(  SG_workingdir__set_branch_info(pCtx, pPath_top, paszArgs[1])  );
            SG_console(pCtx, SG_CS_STDOUT, "Working copy attached to %s. A new head will be created with the next commit.\n", paszArgs[1]);
        }
        else if (0 == strcmp(paszArgs[0], "attach"))
        {
            SG_bool b_has = SG_FALSE;
            SG_vhash* pvh_pile_branches = NULL;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
            SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_pile_branches, paszArgs[1], &b_has)  );
            if (!b_has)
            {
                SG_console(pCtx, SG_CS_STDERR, "Branch %s does not exist. Use 'vv branch new' to create it.\n", paszArgs[1]);
                SG_ERR_THROW(  SG_ERR_BRANCH_NOT_FOUND  );
            }

            SG_ERR_CHECK(  SG_workingdir__set_branch_info(pCtx, pPath_top, paszArgs[1])  );
            SG_console(pCtx, SG_CS_STDOUT, "Working copy attached to %s. A new head will be created with the next commit.\n", paszArgs[1]);
        }
        else if (0 == strcmp(paszArgs[0], "detach"))
        {
            SG_ERR_CHECK(  SG_workingdir__get_branch_info(pCtx, pPath_top, &psz_tied_branch_name)  );
            if (psz_tied_branch_name)
            {
                SG_ERR_CHECK(  SG_workingdir__remove_branch_info(pCtx, pPath_top)  );
                SG_console(pCtx, SG_CS_STDOUT, "Working copy detached from any branch.  The next commit will start an anonymous branch and will require the '--detached' option.\n");
            }
            else
            {
                SG_console(pCtx, SG_CS_STDOUT, "Working copy was already detached.\n");
            }
        }
    }
    else
    {
        const char * LIST = "list";
        if(0 == count_args)
        {
            paszArgs=&LIST;
            count_args=1;
        }

        if (pOptSt->psz_repo)
        {
            SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pOptSt->psz_repo, &pRepo)  );
        }
        else
        {
            SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
        }

        if (0 == strcmp(paszArgs[0], "add-head"))
        {
            SG_audit q;
            SG_vhash* pvh_pile_branches = NULL;
            SG_vhash* pvh_branch_info = NULL;
            SG_vhash* pvh_branch_values = NULL;
            SG_bool b_has = SG_FALSE;
			SG_uint32 countRevSpecs = 0;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
            SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_pile_branches, paszArgs[1], &pvh_branch_info)  );
            if (pvh_branch_info)
            {
                SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
            }

			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &countRevSpecs)  );
            if (1 != countRevSpecs)
            {
                SG_console(pCtx, SG_CS_STDERR, "%s accepts one revision, which may be specified with --rev, --tag or --branch\n", "add-head");

                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

			SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pOptSt->pRevSpec, &psz_rev_1, NULL)  );

            if (pvh_branch_values)
            {
                SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_branch_values, psz_rev_1, &b_has)  );
                if (b_has)
                {
                    SG_console(pCtx, SG_CS_STDERR, "%s is already a head of branch %s\n", psz_rev_1, paszArgs[1]);
                    SG_ERR_THROW(  SG_ERR_USAGE  );
                }
            }

            // TODO check for attempts to add ancestor/descendant

            SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

            SG_ERR_CHECK(  SG_vc_branches__add_head(pCtx, pRepo, paszArgs[1], psz_rev_1, NULL, SG_TRUE, &q)  );
            if (pvh_branch_info)
            {
                SG_console(pCtx, SG_CS_STDOUT, "%s is now a head of branch %s\n", psz_rev_1, paszArgs[1]);
            }
            else
            {
                SG_console(pCtx, SG_CS_STDOUT, "%s is now a head of newly created branch %s\n", psz_rev_1, paszArgs[1]);
            }
        }
        else if (0 == strcmp(paszArgs[0], "remove-head"))
        {
            SG_audit q;
            SG_uint32 i = 0;
			SG_uint32 countRevSpecs;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            if (pOptSt->bAll)
            {
                SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );
                SG_ERR_CHECK(  SG_vc_branches__remove_head(pCtx, pRepo, paszArgs[1], NULL, &q)  );
                SG_console(pCtx, SG_CS_STDOUT, "Branch %s no longer exists.\n", paszArgs[1]);
            }
            else
            {
				SG_uint32 countResolvedHids = 0;
                SG_vhash* pvh_pile_branches = NULL;
                SG_vhash* pvh_branch_info = NULL;

				SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &countRevSpecs)  );

                if (0 == countRevSpecs)
                {
                    SG_console(pCtx, SG_CS_STDERR, "%s requires at least one revision, which may be specified by with --rev, --tag or --branch\n", "add-head");

                    SG_ERR_THROW(  SG_ERR_USAGE  );
                }

                SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );
				
				SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pOptSt->pRevSpec, SG_TRUE, &psaFullHids)  );
				SG_ERR_CHECK(  SG_stringarray__count(pCtx, psaFullHids, &countResolvedHids)  );
                for (i = 0; i < countResolvedHids; i++)
                {
					const char* pszRefHid;
					SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaFullHids, i, &pszRefHid)  );
                    SG_ERR_CHECK(  SG_vc_branches__remove_head(pCtx, pRepo, paszArgs[1], pszRefHid, &q)  );
                    SG_console(pCtx, SG_CS_STDOUT, "%s is no longer a head of %s\n", pszRefHid, paszArgs[1]);
                }

                // get pile and see if the branch exists, and if not, say so
                SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
                SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
                SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_pile_branches, paszArgs[1], &pvh_branch_info)  );
                if (!pvh_branch_info)
                {
                    SG_console(pCtx, SG_CS_STDOUT, "Branch %s no longer exists.\n", paszArgs[1]);
                }
            }
        }
        else if (0 == strcmp(paszArgs[0], "move-head"))
        {
            SG_vhash* pvh_pile_branches = NULL;
            SG_vhash* pvh_branch_info = NULL;
            SG_vhash* pvh_branch_values = NULL;
            SG_uint32 count_branch_values = 0;
			SG_uint32 countRevSpecs = 0;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
            SG_ERR_CHECK(  SG_vhash__check__vhash(pCtx, pvh_pile_branches, paszArgs[1], &pvh_branch_info)  );
            if (!pvh_branch_info)
            {
                SG_ERR_THROW(  SG_ERR_BRANCH_NOT_FOUND  );
            }
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
            SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_branch_values, &count_branch_values)  );

			SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &countRevSpecs)  );

            if (1 == countRevSpecs)
            {
				SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pOptSt->pRevSpec, &psz_rev_1, NULL)  );

                if (pOptSt->bAll)
                {
                    SG_audit q;

                    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

                    SG_ERR_CHECK(  SG_vc_branches__move_head__all(pCtx, pRepo, paszArgs[1], psz_rev_1, &q)  );
                    SG_console(pCtx, SG_CS_STDOUT, "%s is now the only head of %s\n", psz_rev_1, paszArgs[1]);
                }
                else if (1 == count_branch_values)
                {
                    const char* psz_cur_rev = NULL;
                    SG_audit q;

                    SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_branch_values, 0, &psz_cur_rev, NULL)  );
                    SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

                    SG_ERR_CHECK(  SG_vc_branches__move_head(pCtx, pRepo, paszArgs[1], psz_cur_rev, psz_rev_1, &q)  );
                }
                else
                {
                    SG_console(pCtx, SG_CS_STDERR, "%s has more than one head.  Unable to determine the source changeset.\n", paszArgs[1]);
                    SG_ERR_THROW(  SG_ERR_USAGE  );
                }
            }
            else if (2 == countRevSpecs)
            {
				const char* pszRefHid1 = NULL;
				const char* pszRefHid2 = NULL;

				SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pOptSt->pRevSpec, SG_FALSE, &psaFullHids)  );
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaFullHids, 0, &pszRefHid1)  );
				SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, psaFullHids, 1, &pszRefHid2)  );

                {
                    const char* psz_from = NULL;
                    const char* psz_to = NULL;
                    SG_audit q;
                    SG_bool b_has_1 = SG_FALSE;
                    SG_bool b_has_2 = SG_FALSE;

                    SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_branch_values, pszRefHid1, &b_has_1)  );
                    SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_branch_values, pszRefHid2, &b_has_2)  );

                    if (b_has_1 && !b_has_2)
                    {
                        psz_from = pszRefHid1;
                        psz_to = pszRefHid2;
                    }
                    else if (!b_has_1 && b_has_2)
                    {
                        psz_from = pszRefHid2;
                        psz_to = pszRefHid1;
                    }
                    else if (b_has_1 && b_has_2)
                    {
                        SG_console(pCtx, SG_CS_STDERR, "%s already exists at both changesets.\n", paszArgs[1]);
                    }
                    else if (!b_has_1 && !b_has_2)
                    {
                        SG_console(pCtx, SG_CS_STDERR, "%s does not exist at either changeset.\n", paszArgs[1]);
                    }

                    if (psz_from && psz_to)
                    {
                        SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

                        SG_ERR_CHECK(  SG_vc_branches__move_head(pCtx, pRepo, paszArgs[1], psz_from, psz_to, &q)  );
                    }
                }
            }
            else
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }
        }
        else if (0 == strcmp(paszArgs[0], "close"))
        {
            SG_audit q;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

            SG_ERR_CHECK(  SG_vc_branches__close(pCtx, pRepo, paszArgs[1], &q)  );
            SG_console(pCtx, SG_CS_STDOUT, "Branch %s is now closed.\n", paszArgs[1]);
        }
        else if (0 == strcmp(paszArgs[0], "reopen"))
        {
            SG_audit q;

            if (count_args != 2)
            {
                SG_ERR_THROW(  SG_ERR_USAGE  );
            }

            SG_ERR_CHECK(  SG_audit__init(pCtx,&q,pRepo,SG_AUDIT__WHEN__NOW,SG_AUDIT__WHO__FROM_SETTINGS)  );

            SG_ERR_CHECK(  SG_vc_branches__reopen(pCtx, pRepo, paszArgs[1], &q)  );
            SG_console(pCtx, SG_CS_STDOUT, "Branch %s has been reopened.\n", paszArgs[1]);
        }
        else if (0 == strcmp(paszArgs[0], "list"))
        {
            SG_uint32 count = 0;
            SG_uint32 i = 0;
            SG_vhash* pvh_pile_branches = NULL;
            SG_vhash* pvh_pile_closed = NULL;

            SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvh_pile)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "branches", &pvh_pile_branches)  );
            SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_pile, "closed", &pvh_pile_closed)  );

            // TODO deal with closed and --all

            SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_pile_branches, &count)  );

            for (i=0; i<count; i++)
            {
                SG_bool b_include_this_one = SG_FALSE;
                SG_bool b_closed = SG_FALSE;
                const SG_variant* pv = NULL;
                const char* psz_name = NULL;
                SG_vhash* pvh_branch_info = NULL;
                SG_vhash* pvh_branch_values = NULL;
                SG_uint32 count_branch_values = 0;

                SG_ERR_CHECK(  SG_vhash__get_nth_pair(pCtx, pvh_pile_branches, i, &psz_name, &pv)  );
                SG_ERR_CHECK(  SG_vhash__has(pCtx, pvh_pile_closed, psz_name, &b_closed)  );
                if (pOptSt->bAll)
                {
                    b_include_this_one = SG_TRUE;
                }
                else
                {
                    if (b_closed)
                    {
                        b_include_this_one = SG_FALSE;
                    }
                    else
                    {
                        b_include_this_one = SG_TRUE;
                    }
                }

                if (b_include_this_one)
                {
                    SG_ERR_CHECK(  SG_variant__get__vhash(pCtx, pv, &pvh_branch_info)  );
                    SG_ERR_CHECK(  SG_vhash__get__vhash(pCtx, pvh_branch_info, "values", &pvh_branch_values)  );
                    SG_ERR_CHECK(  SG_vhash__count(pCtx, pvh_branch_values, &count_branch_values)  );

                    if (count_branch_values > 1)
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s (needs merge)", psz_name)  );
                    }
                    else
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s", psz_name)  );
                    }

                    if (b_closed)
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, " (closed)\n")  );
                    }
                    else
                    {
                        SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\n")  );
                    }
                }
            }
        }
        else
        {
            SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "Unknown subcommand: %s\n", paszArgs[0])  );
            SG_ERR_THROW(  SG_ERR_USAGE  );
        }
    }

fail:
	SG_NULLFREE(pCtx, psz_rev_1);
	SG_NULLFREE(pCtx, psz_rev_2);
    SG_NULLFREE(pCtx, psz_tied_branch_name);
    SG_PATHNAME_NULLFREE(pCtx, pPath_top);
    SG_PATHNAME_NULLFREE(pCtx, pPath_cwd);
    SG_REPO_NULLFREE(pCtx, pRepo);
    SG_VHASH_NULLFREE(pCtx, pvh_pile);
	SG_STRINGARRAY_NULLFREE(pCtx, psaFullHids);
}

void do_cmd_vcdiff(SG_context * pCtx, SG_uint32 count_args, const char** paszArgs)
{
	SG_pathname* pPath1 = NULL;
	SG_pathname* pPath2 = NULL;
	SG_pathname* pPath3 = NULL;

	if (
		(count_args == 4)
		&& (0 == strcmp(paszArgs[0], "deltify"))
		)
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath1, paszArgs[1])  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath2, paszArgs[2])  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath3, paszArgs[3])  );
		SG_ERR_CHECK(  SG_vcdiff__deltify__files(pCtx, pPath1, pPath2, pPath3)  );
	}
	else if (
		(count_args == 4)
		&& (0 == strcmp(paszArgs[0], "undeltify"))
		)
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath1, paszArgs[1])  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath2, paszArgs[2])  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pPath3, paszArgs[3])  );
		SG_ERR_CHECK(  SG_vcdiff__undeltify__files(pCtx, pPath1, pPath2, pPath3)  );
	}
	else
	{
		SG_ERR_THROW_RETURN(  SG_ERR_USAGE  );
	}

	SG_PATHNAME_NULLFREE(pCtx, pPath1);
	SG_PATHNAME_NULLFREE(pCtx, pPath2);
	SG_PATHNAME_NULLFREE(pCtx, pPath3);

	return;

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPath1);
	SG_PATHNAME_NULLFREE(pCtx, pPath2);
	SG_PATHNAME_NULLFREE(pCtx, pPath3);
}


void do_cmd_revert(SG_context * pCtx,
				   SG_option_state * pOptSt,
				   SG_uint32 count_args, const char** paszArgs)
{
	SG_vhash * pvhResults = NULL;
	SG_int64 status;

	SG_ERR_CHECK(  SG_vv_verbs__revert(pCtx,
									   NULL,	// assume cwd
									   pOptSt->bRecursive,
									   pOptSt->bNoSubmodules,
									   pOptSt->bVerbose,
									   pOptSt->bTest,
									   pOptSt->bNoBackups,
									   pOptSt->pvec_use_submodules,
									   pOptSt->pFilespec,
									   count_args, paszArgs,
									   &pvhResults)  );
	SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pvhResults, "status", &status)  );
	switch (status)
	{
	default:
	case SG_VV_VERBS__STATUS__OK:
		if (pOptSt->bVerbose || pOptSt->bTest)
		{
			SG_varray * pvaActionLog;	// we don't own this

			SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "log", &pvaActionLog)  );
			SG_ERR_IGNORE(  sg_report_action_log2(pCtx, pvaActionLog)  );
		}
		break;

	case SG_VV_VERBS__STATUS__PORTABILITY_WARNING:
		{
			SG_varray * pvaWarnings;	// we don't own this
			
			SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "port", &pvaWarnings)  );
			SG_ERR_IGNORE(  sg_report_portability_warnings2(pCtx, pvaWarnings)  );
			SG_ERR_THROW(  SG_ERR_PORTABILITY_WARNINGS  ); // throw so that exe exits with non-zero status
		}
		break;

	case SG_VV_VERBS__STATUS__SUBMODULE_PROBLEM:
		{
			SG_varray * pvaMissingDetails;	// we don't own this

			SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "sub", &pvaMissingDetails)  );

			SG_ERR_IGNORE(  sg_report_submodule_missing_details(pCtx, pvaMissingDetails)  );
			SG_ERR_THROW(  SG_ERR_SUBMODULE_NESTED_REVERT_PROBLEM  ); // throw so that exe exits with non-zero status
		}
		break;
	}

fail:
	SG_VHASH_NULLFREE(pCtx, pvhResults);
}

void do_cmd_scan(SG_context * pCtx)
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	SG_timestamp_cache * pTSC = NULL;		// we do not own this

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	// start scan with an empty timestamp_cache and rebuild it.

	SG_ERR_CHECK(  SG_pendingtree__get_timestamp_cache(pCtx, pPendingTree, &pTSC)  );
	SG_ERR_CHECK(  SG_timestamp_cache__remove_all(pCtx, pTSC)  );

	SG_ERR_CHECK(  SG_pendingtree__scan(pCtx, pPendingTree, SG_TRUE, SG_FALSE, SG_FALSE, gpBlankFilespec)  );			// This does scan and implicit save and close of vfile.

	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);

	return;

fail:
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
}

void do_cmd_addremove(SG_context * pCtx,
					  SG_uint32 count_args, const char** paszArgs,
					  SG_bool bRecursive,
					  const SG_file_spec* pFilespec,
					  SG_bool bTest)
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__addremove(pCtx, pPendingTree, count_args, paszArgs, bRecursive, pFilespec, bTest)  );			// This does scan and implicit save and close of vfile.

	if (bTest == SG_TRUE)
		sg_report_action_log(pCtx, pPendingTree);

	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);

	return;

fail:
	if (pPendingTree && SG_context__err_equals(pCtx,SG_ERR_PORTABILITY_WARNINGS))
		SG_ERR_IGNORE(  sg_report_portability_warnings(pCtx,pPendingTree)  );
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
}


void do_cmd_status(SG_context * pCtx,
				   SG_uint32 count_args, const char ** paszArgs,
				   SG_bool bRecursive,
				   SG_bool bVerbose,
				   const SG_file_spec* pFilespec,
				   const char* pszRev1,
				   const char* pszRev2)
{
	SG_treediff2 * pTreeDiff = NULL;
	SG_string * pStrStatusReport = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	SG_repo* pRepo = NULL;
	const SG_varray * pvaIssuesIn = NULL;
	SG_vhash * pVhash = NULL;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );

	//We no longer check that all of the argument paths exist before continuing on.  -Jeremy

	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );

	SG_ERR_CHECK(  SG_vv_verbs__status(pCtx, pPendingTree, pszRev1, pszRev2, count_args, paszArgs, pFilespec, bRecursive, bVerbose, 
		(count_args == 0) ? &pvaIssuesIn : NULL, 
		&pVhash)  );

	SG_ERR_CHECK(  SG_status_formatter__vhash_to_string(pCtx, pVhash, pvaIssuesIn, pPendingTree, NULL, "\n", &pStrStatusReport)  );

	SG_ERR_CHECK(  SG_console__raw(pCtx,SG_CS_STDOUT,SG_string__sz(pStrStatusReport))  );

	// fall thru to common cleanup.

fail:
	SG_VHASH_NULLFREE(pCtx, pVhash);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_STRING_NULLFREE(pCtx, pStrStatusReport);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
}

static void _get_file_blob_hid_for_cat(
	SG_context* pCtx, 
	SG_repo* pRepo,
	const char* pszRefHidCs,
	const char* pszRefPathGiven, 
	SG_pendingtree* pPendingTree, 
	char* pszBufHidFileBlob,
	SG_uint32 lenBuf)
{
	char* pszGid = NULL;
	const char* pszRefAbsoluteRepoPath = NULL;
	SG_pathname* pTempPathname = NULL;
	SG_treenode_entry* ptne = NULL;
	SG_treenode* pTreenodeRoot = NULL;
	char * pszRefHidTreeNode = NULL;

	/* Were we given an explicit full repository path? */
	if (pszRefPathGiven[0] == '@')
		pszRefAbsoluteRepoPath = pszRefPathGiven;

	if (!pszRefAbsoluteRepoPath && !pPendingTree)
	{
		/* We can't find anything but a full repo path without a pending tree. */
		/* TODO: Consider converting it to a full repo path and trying. */
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, 
			"When outside a working copy or when specifying --repo, an absolute repository path, starting with @/, must be specified.\n")  );
		pszBufHidFileBlob[0] = '\0';
		return;
	}

	if (!pszRefAbsoluteRepoPath)
	{
		/* We're in a working folder and no absolute repo path was given.
		 * Lookup the GID with PendingTree and get the TreeNodeEntry from treendx. */

		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pTempPathname, pszRefPathGiven)  );
		SG_ERR_CHECK(  SG_pendingtree__get_gid_from_local_path(pCtx, pPendingTree, pTempPathname, &pszGid)  );
		SG_ERR_CHECK(  SG_repo__treendx__get_path_in_dagnode(pCtx, pRepo, SG_DAGNUM__VERSION_CONTROL, 
			pszGid, pszRefHidCs, NULL, &ptne)  );
	}
	else
	{
		/* An absolute repo path was given. Look it up the the changeset. */
		SG_ERR_CHECK(  SG_changeset__tree__find_root_hid(pCtx, pRepo, pszRefHidCs, &pszRefHidTreeNode)  );
		SG_ERR_CHECK(  SG_treenode__load_from_repo(pCtx, pRepo, pszRefHidTreeNode, &pTreenodeRoot)  );
		SG_ERR_CHECK(  SG_treenode__find_treenodeentry_by_path(pCtx, pRepo, pTreenodeRoot, 
			pszRefAbsoluteRepoPath, NULL, &ptne)  );
	}

	if (ptne)
	{
		const char* pszRefHidFileBlob = NULL;
		SG_treenode_entry_type ptneType;
		SG_ERR_CHECK(  SG_treenode_entry__get_entry_type(pCtx, ptne, &ptneType)  );
		if (ptneType == SG_TREENODEENTRY_TYPE_REGULAR_FILE)
		{
			SG_ERR_CHECK(  SG_treenode_entry__get_hid_blob(pCtx, ptne, &pszRefHidFileBlob)  );
			SG_ERR_CHECK(  SG_strcpy(pCtx, pszBufHidFileBlob, lenBuf, pszRefHidFileBlob)  );
		}
		else
		{
			pszBufHidFileBlob[0] = '\0';
		}
	}
	else
	{
		pszBufHidFileBlob[0] = '\0';
	}

	/* Common cleanup */
fail:
	SG_NULLFREE(pCtx, pszGid);
	SG_PATHNAME_NULLFREE(pCtx, pTempPathname);
	SG_TREENODE_ENTRY_NULLFREE(pCtx, ptne);
	SG_NULLFREE(pCtx, pszRefHidTreeNode);
	SG_TREENODE_NULLFREE(pCtx, pTreenodeRoot);
}


void do_cmd_cat(SG_context * pCtx, SG_uint32 count_args, const char ** paszArgs, SG_option_state* pOptSt)
{
	SG_uint32 count_rev_specs = 0;
    SG_repo* pRepo = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	char* pszHidCs = NULL;			// will be freed
	const char* pszRefHidCs = NULL; // won't be freed
	SG_rev_spec* pTempRevSpec = NULL;
	SG_repo_fetch_blob_handle* pfh = NULL;
	SG_byte* pBuf = NULL;

	/* First, get the repo from either --repo or the current working directory. */
	SG_ERR_CHECK(  SG_cmd_util__get_repo_from_options_or_cwd(pCtx, pOptSt, &pRepo, &pPathCwd)  );

	/* If pPathCwd is non-null, we're in a working folder. Get the pendingtree. We'll use it
	 * to translate file paths. If no changeset was specified by the user, we'll also use it
	 * to get the working directory baseline. */
	if (pPathCwd)
	{
		SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

		if (pPendingTree)
		{
			const SG_varray * pvaRefWdParents;		// we do not own this
			SG_uint32 nrParents;

			SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pvaRefWdParents)  );
			SG_ERR_CHECK(  SG_varray__count(pCtx, pvaRefWdParents, &nrParents)  );
			if (nrParents > 1)
				SG_ERR_THROW(  SG_ERR_CANNOT_DO_WHILE_UNCOMMITTED_MERGE  );

			SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaRefWdParents, 0, &pszRefHidCs)  );
		}
	}

	/* Next, we need the changeset where we'll look for file GIDs. It'll end up in pszRefHidCs. */

	/* OPT_REV_SPEC options passed validation: we have either zero or one rev/tag/branch specified. 
	 * If we have one, get its hid. */
	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pOptSt->pRevSpec, &count_rev_specs)  );
	if (count_rev_specs)
	{
		SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pOptSt->pRevSpec, &pszHidCs, NULL)  );
		pszRefHidCs = pszHidCs;
	}

	if (!pszRefHidCs)
	{
		/* We're not in a working folder or --repo was specified.  */
		/* No specific changeset was given, so we'll use head of master. */

		SG_REV_SPEC__ALLOC(pCtx, &pTempRevSpec);
		SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pTempRevSpec, SG_VC_BRANCHES__DEFAULT)  );
		SG_ERR_CHECK(  SG_rev_spec__get_one__repo(pCtx, pRepo, pTempRevSpec, &pszHidCs, NULL)  ); // TODO: better error message when no master
		pszRefHidCs = pszHidCs;
	}

	/* Now we find the GIDs of any files given as arguments. */
	{
		char bufHid[SG_HID_MAX_BUFFER_LENGTH];
		SG_uint32 i;

		for (i = 0; i < count_args; i++)
		{
			SG_ERR_CHECK(  _get_file_blob_hid_for_cat(pCtx, pRepo, pszRefHidCs, paszArgs[i], pPendingTree, bufHid, SG_HID_MAX_BUFFER_LENGTH)  );

			if (bufHid[0] == '\0')
			{
				SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, 
					"%s not found.\n", paszArgs[i])  );
			}
			else
			{
				SG_bool done = SG_FALSE;
				SG_uint32 len_got = 0;
				SG_uint32 lenBuf;
				SG_uint64 lenFile;

				SG_ERR_CHECK(  SG_repo__fetch_blob__begin(pCtx, pRepo, bufHid, SG_TRUE, NULL, NULL, NULL, NULL, NULL, &lenFile, &pfh)  );

				/* Would it be better to just always allocate a big buffer one time? I don't think people typically cat dozens of files at once... */
				if (lenFile > SG_UINT16_MAX)
					lenBuf = SG_UINT16_MAX;
				else
					lenBuf = (SG_uint32)lenFile + 1;

				SG_ERR_CHECK(  SG_allocN(pCtx, lenBuf, pBuf)  );

				while (!done)
				{
					SG_ERR_CHECK(  SG_repo__fetch_blob__chunk(pCtx, pRepo, pfh, lenBuf-1, pBuf, &len_got, &done)  );
					SG_ASSERT(len_got < lenBuf);
					pBuf[len_got] = '\0';
					SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, (const char*)pBuf)  );
				}
				SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\n")  );

				SG_NULLFREE(pCtx, pBuf);
				SG_ERR_CHECK(  SG_repo__fetch_blob__end(pCtx, pRepo, &pfh)  );
			}
		}
	}

	/* Common cleanup */
fail:
	if (pfh)
		SG_ERR_IGNORE(  SG_repo__fetch_blob__abort(pCtx, pRepo, &pfh)  );
    SG_REPO_NULLFREE(pCtx, pRepo);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_NULLFREE(pCtx, pszHidCs);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_REV_SPEC_NULLFREE(pCtx, pTempRevSpec);
	SG_NULLFREE(pCtx, pBuf);
}

//////////////////////////////////////////////////////////////////

#define REPLACEINARG(_replaceMe_, _replaceWith_) SG_ERR_CHECK(  SG_string__replace_bytes(pCtx, pCurrentArg, (const SG_byte*)_replaceMe_, SG_STRLEN(_replaceMe_), (const SG_byte*)_replaceWith_, SG_STRLEN(_replaceWith_), 1, SG_TRUE, NULL)  );
/**
 * Perform a version control tree-diff using 2 changesets.
 */
void do_cmd_diff(SG_context * pCtx,
				 SG_uint32 count_args, const char ** paszArgs,
				 SG_bool bRecursive,
				 const SG_file_spec* pFilespec,
				 const char* psz_rev_1,
				 const char* psz_rev_2,
				 const char* pszTool)
{
	SG_treediff2 * pTreeDiff = NULL;
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	SG_repo* pRepo = NULL; // Repo pointer that we do not own.

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );
	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );

	SG_ERR_CHECK( SG_pendingtree__diff_or_status(pCtx, pPendingTree, psz_rev_1, psz_rev_2, &pTreeDiff)   );

	SG_ERR_CHECK(  SG_treediff2__file_spec_filter(pCtx, pTreeDiff, count_args, paszArgs, bRecursive, pFilespec)  );

	SG_ERR_CHECK(  SG_treediff2__diff(pCtx, pTreeDiff, pszTool)  );
	
	// fall thru to common cleanup.

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_TREEDIFF2_NULLFREE(pCtx, pTreeDiff);
}

void do_cmd_diffmerge(SG_context * pCtx,
					  const char* psz_rev_1,
					  const char* psz_rev_2,
					  SG_bool bNoSubmodules,
					  const SG_vector * pvec_use_submodules)	//<[in] hints for any referenced submodules within either cset
{
	SG_pathname* pPathCwd = NULL;
	SG_pendingtree* pPendingTree = NULL;
	SG_repo* pRepo = NULL; // Repo pointer that we do not own.
	const char * pszDescriptorName = NULL;
	SG_pathname * pPathTemp1 = NULL;
	SG_pathname * pPathTemp2 = NULL;
	SG_string * pLabel1 = NULL;
	SG_string * pLabel2 = NULL;
	SG_vhash * pvhResults = NULL;
	SG_int64 status;
	SG_bool bDeletePath2 = SG_FALSE;
	SG_bool bRightSideIsWritable = SG_FALSE;

	SG_ERR_CHECK(  SG_PATHNAME__ALLOC(pCtx, &pPathCwd)  );
	SG_ERR_CHECK(  SG_pathname__set__from_cwd(pCtx, pPathCwd)  );
	SG_ERR_CHECK(  SG_pendingtree__alloc(pCtx, pPathCwd, &pPendingTree)  );

	SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );
	SG_ERR_CHECK(  SG_repo__get_descriptor_name(pCtx, pRepo, &pszDescriptorName)  );

	//Now we need to fetch revisions to a temp folder.
	if (psz_rev_1 == NULL)
	{
		//Ask the pending tree what the baseline is.
		const SG_varray * pvaParents = NULL; //It's ok to declare this here, since we only get a reference from the pending tree.
		const char * pszParentZeroHID = NULL;
		SG_ERR_CHECK(  SG_pendingtree__get_wd_parents__ref(pCtx, pPendingTree, &pvaParents)  );
		SG_ERR_CHECK(  SG_varray__get__sz(pCtx, pvaParents, 0, &pszParentZeroHID)  );
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__USER_TEMP_DIRECTORY(pCtx, &pPathTemp1)  );
		SG_ERR_CHECK(  SG_pathname__append__force_nonexistant(pCtx, pPathTemp1, pszParentZeroHID)  );
		SG_ERR_CHECK(  SG_vv_verbs__export(pCtx,
										   pszDescriptorName,
										   pszParentZeroHID,
										   SG_pathname__sz(pPathTemp1),
										   bNoSubmodules,
										   pvec_use_submodules,
										   &pvhResults)  );
		SG_ERR_CHECK(  SG_STRING__ALLOC__SZ(pCtx, &pLabel1, "Baseline")  );
	}
	else
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__USER_TEMP_DIRECTORY(pCtx, &pPathTemp1)  );
		SG_ERR_CHECK(  SG_pathname__append__force_nonexistant(pCtx, pPathTemp1, psz_rev_1)  );
		SG_ERR_CHECK(  SG_vv_verbs__export(pCtx,
										   pszDescriptorName,
										   psz_rev_1,
										   SG_pathname__sz(pPathTemp1),
										   bNoSubmodules,
										   pvec_use_submodules,
										   &pvhResults)  );
		SG_ERR_CHECK(  SG_STRING__ALLOC__SZ(pCtx, &pLabel1, psz_rev_1)  );
	}

	SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pvhResults, "status", &status)  );
	switch (status)
	{
	default:
	case SG_VV_VERBS__STATUS__OK:
		break;

	case SG_VV_VERBS__STATUS__SUBMODULE_PROBLEM:
		{
			SG_varray * pvaMissingDetails;	// we don't own this

			SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "sub", &pvaMissingDetails)  );

			SG_ERR_IGNORE(  sg_report_submodule_missing_details(pCtx, pvaMissingDetails)  );
			// TODO 2011/02/01 Should we have a __NESTED_EXPORT_PROBLEM code?
			SG_ERR_THROW(  SG_ERR_SUBMODULE_NESTED_UPDATE_CHECKOUT_PROBLEM  ); // throw so that exe exits with non-zero status
		}
		break;
	}
	SG_VHASH_NULLFREE(pCtx, pvhResults);

	if (psz_rev_2 == NULL)
	{
		// They want to compare something to the current (possibly dirty)
		// working directory.
		// 
		// TODO 2011/02/02 if they said "--no-submodules" then the tree
		// TODO            we exported above into temp for rev_1 will
		// TODO            not have any contained submodules.  but the
		// TODO            live working directory will, so DiffMerge will
		// TODO            show everything in the submodules as peerless
		// TODO            when it shows the main folder window.  this
		// TODO            may cause a little confusion.

		//Get the top dir of the working folder.
		SG_ERR_CHECK(  SG_pendingtree__get_working_directory_top(pCtx, pPendingTree, &pPathTemp2)  );
		bDeletePath2 = SG_FALSE;

		bRightSideIsWritable = SG_TRUE;		// right-side refers to the WD so they can edit the contents.
	}
	else
	{
		//Fetch the changeset that we're diffing against.
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__USER_TEMP_DIRECTORY(pCtx, &pPathTemp2)  );
		SG_ERR_CHECK(  SG_pathname__append__force_nonexistant(pCtx, pPathTemp2, psz_rev_2)  );
		SG_ERR_CHECK(  SG_vv_verbs__export(pCtx,
										   pszDescriptorName,
										   psz_rev_2,
										   SG_pathname__sz(pPathTemp2),
										   bNoSubmodules,
										   pvec_use_submodules,
										   &pvhResults)  );
		bDeletePath2 = SG_TRUE;
		SG_ERR_CHECK(  SG_STRING__ALLOC__SZ(pCtx, &pLabel2, psz_rev_2)  );

		SG_ERR_CHECK(  SG_vhash__get__int64(pCtx, pvhResults, "status", &status)  );
		switch (status)
		{
		default:
		case SG_VV_VERBS__STATUS__OK:
			break;

		case SG_VV_VERBS__STATUS__SUBMODULE_PROBLEM:
			{
				SG_varray * pvaMissingDetails;	// we don't own this

				SG_ERR_CHECK(  SG_vhash__get__varray(pCtx, pvhResults, "sub", &pvaMissingDetails)  );

				SG_ERR_IGNORE(  sg_report_submodule_missing_details(pCtx, pvaMissingDetails)  );
				// TODO 2011/02/01 Should we have a __NESTED_EXPORT_PROBLEM code?
				SG_ERR_THROW(  SG_ERR_SUBMODULE_NESTED_UPDATE_CHECKOUT_PROBLEM  ); // throw so that exe exits with non-zero status
			}
			break;
		}
	}

	//Go ahead and release the pending tree lock.
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);

	SG_ERR_CHECK(  SG_difftool__built_in_tool__exec_diffmerge(pCtx, pPathTemp1, pPathTemp2, pLabel1, pLabel2, bRightSideIsWritable, NULL)  );

	if (bDeletePath2)
		SG_ERR_CHECK( SG_fsobj__rmdir_recursive__pathname(pCtx, pPathTemp2)  );
	SG_ERR_CHECK(  SG_fsobj__rmdir_recursive__pathname(pCtx, pPathTemp1)  );

	// fall thru to common cleanup.

fail:
	SG_PATHNAME_NULLFREE(pCtx, pPathTemp1);
	SG_PATHNAME_NULLFREE(pCtx, pPathTemp2);
	SG_STRING_NULLFREE(pCtx, pLabel1);
	SG_STRING_NULLFREE(pCtx, pLabel2);
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_VHASH_NULLFREE(pCtx, pvhResults);
}

typedef struct
{
	const char* pszFilePath;
	SG_vhash* pvhCleanPileOfBranches;
	SG_bool first;
	SG_bool verbose;
} parents_foreach_ctx;

static void _parents_foreach_rev_spec(
	SG_context* pCtx, 
	SG_repo* pRepo,
	SG_rev_spec_type specType,
	const char* pszGiven,
	const char* pszFullHid,
	void* vpCtx)
{
	SG_rev_spec* pTempRevSpec = NULL;
	SG_stringarray * pStringArrayParents = NULL;
	parents_foreach_ctx* pForeachCtx = (parents_foreach_ctx*)vpCtx;

	SG_UNUSED(specType);

	SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pTempRevSpec)  );
	SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pTempRevSpec, pszFullHid)  );

	SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, pTempRevSpec, pForeachCtx->pszFilePath, &pStringArrayParents)  );
	if (pStringArrayParents != NULL)
	{
		SG_uint32 i, countResults;
		if(pForeachCtx->first)
			pForeachCtx->first = SG_FALSE;
		else
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "\n")  );
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "\nParents of %s:\n", pszGiven)  );
		SG_ERR_CHECK(  SG_stringarray__count(pCtx, pStringArrayParents, &countResults)  );
		for (i = 0; i < countResults;  i++)
		{
			const char* pszRef;
			SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, pStringArrayParents, i, &pszRef)  );
			SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, pszRef, pForeachCtx->pvhCleanPileOfBranches, SG_FALSE, pForeachCtx->verbose)  );
		}
		SG_STRINGARRAY_NULLFREE(pCtx, pStringArrayParents);
	}

	/* common cleanup */
fail:
	SG_REV_SPEC_NULLFREE(pCtx, pTempRevSpec);
	SG_STRINGARRAY_NULLFREE(pCtx, pStringArrayParents);
}

/**
 * List the working folder's parent changesets
 */
void do_cmd_parents(SG_context * pCtx, SG_uint32 count_args, const char ** paszArgs, SG_rev_spec* pRevSpec, SG_bool bVerbose)
{
	SG_repo * pRepo = NULL;
	SG_uint32 countRevSpecs = 0;
	const char* pszArgumentPath = NULL;
	SG_stringarray * pStringArrayParents = NULL;
    SG_vhash* pvhPileOfCleanBranches = NULL;

	// use the CWD to get a descriptor to the REPO and the top of the Working Directory.
	SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );

    SG_ERR_CHECK(  SG_vc_branches__cleanup(pCtx, pRepo, &pvhPileOfCleanBranches)  );

	//If a file argument is given, the revision in which the file was last changed (before the working
	//directory revision or the argument to --rev if given) is printed
	if (count_args == 1)
	{
		pszArgumentPath = paszArgs[0];
	}

	// Make sure the spec is sane before embarking on the foreach.
	SG_ERR_CHECK(  SG_rev_spec__get_all__repo(pCtx, pRepo, pRevSpec, SG_TRUE, NULL)  );

	SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pRevSpec, &countRevSpecs)  );
	if (countRevSpecs)
	{
		parents_foreach_ctx foreachCtx;
		foreachCtx.pszFilePath = pszArgumentPath;
		foreachCtx.pvhCleanPileOfBranches = pvhPileOfCleanBranches;
		foreachCtx.first = SG_TRUE;
		foreachCtx.verbose = bVerbose;

		SG_ERR_CHECK(  SG_rev_spec__foreach__repo(pCtx, pRepo, pRevSpec, _parents_foreach_rev_spec, &foreachCtx)  );
	}
	else
	{
		SG_uint32 nResultCount = 0;
		SG_uint32 nParentIndex = 0;
		const char * pszCurrentParentHid = NULL;
		SG_ERR_CHECK(  SG_parents__list(pCtx, pRepo, NULL, pszArgumentPath, &pStringArrayParents)  );
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDOUT, "Parents of pending changes in working copy:\n" )  );
		SG_ERR_CHECK(  SG_stringarray__count(pCtx, pStringArrayParents, &nResultCount)  );
		for (nParentIndex = 0; nParentIndex < nResultCount;  nParentIndex++)
		{
			SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, pStringArrayParents, nParentIndex, &pszCurrentParentHid)  );
			SG_ERR_CHECK(  SG_cmd_util__dump_log(pCtx, SG_CS_STDOUT, pRepo, pszCurrentParentHid, pvhPileOfCleanBranches, SG_FALSE, bVerbose)  );
		}
	}

fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_STRINGARRAY_NULLFREE(pCtx, pStringArrayParents);
    SG_VHASH_NULLFREE(pCtx, pvhPileOfCleanBranches);
}

void do_cmd_incoming(
	SG_context* pCtx,
	const char* pszSrcRepoSpec,
	const char* pszDestRepoDescriptorName,
	SG_bool bVerbose,
	SG_bool bTrace)
{
	SG_history_result* pInfo = NULL;
	SG_vhash* pvhStats = NULL;
	
	SG_ERR_CHECK(  SG_vv_verbs__incoming(pCtx, pszSrcRepoSpec, pszDestRepoDescriptorName, &pInfo, bTrace ? &pvhStats : NULL)  );
	
	if (pInfo)
		SG_ERR_CHECK(  SG_cmd_util__dump_history_results(pCtx, SG_CS_STDOUT, pInfo, NULL, SG_FALSE, bVerbose, SG_TRUE)  );
	else
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "No incoming changes.\n")  );

#if defined(DEBUG)
	if (pvhStats)
		SG_ERR_CHECK(  SG_vhash_debug__dump_to_console__named(pCtx, pvhStats, "Incoming stats")  );
#endif

fail:
	SG_HISTORY_RESULT_NULLFREE(pCtx, pInfo);
	SG_VHASH_NULLFREE(pCtx, pvhStats);
}

void do_cmd_pull(
	SG_context * pCtx,
	SG_option_state * pOptSt,
	const char* pszSrcRepoSpec,
	const char* pszDestRepoDescriptorName,
	int * pExitStatus)
{
	SG_rev_spec* pMyRevSpec = NULL;
	SG_vhash* pvhStats = NULL;

	if (pOptSt->bUpdate && pOptSt->pRevSpec)
	{
		/* Pull will steal the rev spec. If we're going to use it again for update below,
		 * make our own copy. */
		SG_ERR_CHECK(  SG_REV_SPEC__ALLOC__COPY(pCtx, pOptSt->pRevSpec, &pMyRevSpec)  );
	}

	SG_vv_verbs__pull(pCtx, pszSrcRepoSpec, pszDestRepoDescriptorName, 
		pOptSt->pvh_area_names, &pOptSt->pRevSpec, 
		pOptSt->bTrace ? &pvhStats : NULL);
	if (SG_context__err_equals(pCtx, SG_ERR_REPO_ID_MISMATCH))
	{
		SG_context__err_reset(pCtx);
		SG_pull__admin(pCtx, pszDestRepoDescriptorName, pszSrcRepoSpec, pOptSt->bTrace ? &pvhStats : NULL);
		if (SG_context__err_equals(pCtx, SG_ERR_ADMIN_ID_MISMATCH))
		{
			SG_context__err_reset(pCtx);
			SG_ERR_THROW(SG_ERR_REPO_ID_MISMATCH);
		}
		else
			SG_ERR_CHECK_CURRENT;

		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\n"
			"WARNING: The repositories are not clones of one another,\n"
			"         but they belong to the same administrative group.\n"
			"         Only the user database was pulled.\n\n")  );
	}
	else
		SG_ERR_CHECK_CURRENT;

	// TODO: friendly error message when repo's not found (local or remote).

	if (pOptSt->bUpdate)
	{
		// TODO 2010/12/09 We have "vv pull --update" which causes an automatic UPDATE
		// TODO            after we do a PULL as a convenience feature.  This may cause
		// TODO            problems when we have submodules because a nested update may
		// TODO            require a sub to do a pull.  Think about this some more....

		SG_uint32 countRevSpecs;
		SG_ERR_CHECK(  SG_rev_spec__count(pCtx, pMyRevSpec, &countRevSpecs)  );

		if (countRevSpecs > 1)
		{
			SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, "Skipping update: multiple revisions were specified.\n")  );
		}
		else
		{
			SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, "Updating...")  );
			SG_ERR_CHECK(  do_cmd_update(pCtx,
				pMyRevSpec,	// update based on the rev spec provided to pull
				NULL,
				SG_FALSE,	// bVerbose
				SG_FALSE,	// bTest
				pOptSt->pvec_use_submodules,
				pExitStatus)  );
			SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, " Done.\n")  );
		}
	}

#if defined(DEBUG)
	if (pvhStats)
		SG_ERR_CHECK(  SG_vhash_debug__dump_to_console__named(pCtx, pvhStats, "Pull stats")  );
#endif
	/* Common cleanup */
fail:
	SG_REV_SPEC_NULLFREE(pCtx, pMyRevSpec);
	SG_VHASH_NULLFREE(pCtx, pvhStats);
}

void do_cmd_outgoing(
	SG_context* pCtx,
	const char* pszDestRepoSpec,
	const char* pszSrcRepoDescriptorName,
	SG_bool bVerbose,
	SG_bool bTrace)
{
	SG_history_result* pInfo = NULL;
	SG_vhash* pvhStats = NULL;

	SG_ERR_CHECK(  SG_vv_verbs__outgoing(pCtx, pszDestRepoSpec, pszSrcRepoDescriptorName, &pInfo, 
		bTrace ? &pvhStats : NULL)  );
	if (pInfo)
		SG_ERR_CHECK(  SG_cmd_util__dump_history_results(pCtx, SG_CS_STDOUT, pInfo, NULL, SG_FALSE, bVerbose, SG_FALSE)  );
	else
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "No outgoing changes.\n")  );

#if defined(DEBUG)
	if (pvhStats)
		SG_ERR_CHECK(  SG_vhash_debug__dump_to_console__named(pCtx, pvhStats, "Outgoing stats")  );
#endif

fail:
	SG_HISTORY_RESULT_NULLFREE(pCtx, pInfo);
	SG_VHASH_NULLFREE(pCtx, pvhStats);
}

void do_cmd_push(
	SG_context * pCtx,
	SG_option_state * pOptSt,
	const char* pszDestRepoSpec,
	const char* pszSrcRepoDescriptorName,
	int * pExitStatus)
{
	SG_vhash* pvhStats = NULL;
	SG_repo* pRepo = NULL;

	SG_ASSERT(pExitStatus!=NULL);

	SG_vv_verbs__push(pCtx, pszDestRepoSpec, pszSrcRepoDescriptorName, pOptSt->pvh_area_names, pOptSt->pRevSpec, pOptSt->bForce, 
		pOptSt->bTrace ? &pvhStats : NULL);
	if(SG_context__err_equals(pCtx, SG_ERR_SERVER_DOESNT_ACCEPT_PUSHES))
	{
		SG_context__err_reset(pCtx);
		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "The server does not accept pushes.\n")  );
		*pExitStatus = 1;
	}
	else if (SG_context__err_equals(pCtx, SG_ERR_REPO_ID_MISMATCH))
	{
		SG_context__err_reset(pCtx);
		SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pszSrcRepoDescriptorName, &pRepo)  );
		
		SG_push__admin(pCtx, pRepo, pszDestRepoSpec, pOptSt->bTrace ? &pvhStats : NULL);
		if (SG_context__err_equals(pCtx, SG_ERR_ADMIN_ID_MISMATCH))
		{
			SG_context__err_reset(pCtx);
			SG_ERR_THROW(SG_ERR_REPO_ID_MISMATCH);
		}
		else
			SG_ERR_CHECK_CURRENT;

		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "\n"
			"WARNING: The repositories are not clones of one another,\n"
			"         but they belong to the same administrative group.\n"
			"         Only the user database was pushed.\n\n")  );

		SG_REPO_NULLFREE(pCtx, pRepo);
	}
	else
		SG_ERR_CHECK_CURRENT;

	if (pvhStats)
	{
#ifdef DEBUG
		SG_ERR_CHECK(  SG_vhash_debug__dump_to_console__named(pCtx, pvhStats, "Push stats")  );
#endif
		SG_VHASH_NULLFREE(pCtx, pvhStats);
	}

	return;

fail:
	SG_VHASH_NULLFREE(pCtx, pvhStats);
	SG_REPO_NULLFREE(pCtx, pRepo);
}

/* ---------------------------------------------------------------- */
/* All the cmd funcs are below */

/*
 * If you are implementing a new cmd func, pick one of the existing
 * ones and copy it as a starting point.  All cmd funcs should follow
 * a similar structure:
 *
 * 1.  Check the flags.  Set bUsageError if anything is wrong.
 *
 * 2.  Check the number of args, set bUsageError if necessary.
 *
 * 3.  If no problems with flags/args, try to do the command.
 *     This will either succeed, throw a hard error, or throw a
 *     usage-error (when it is too complex to fully check here).
 *
 * 4.  if we have a usage-error, print some help and throw.
 */

/**
 * A private macro to run the command and handle hard- and usage-errors.
 * Invoke the command handler using pCtx as an argument.
 *
 * We Assume arg/flag checking code do a THROW for hard errors
 * and *only* set bUsageError when a bad arg/flag was seen.
 *
 * We do the THROW if bUsageError set.  (This allows *ALL*
 * flags/args to reported by the flag/arg checker, rather
 * than just the first.)
 *
 * Try to run the actual command.
 *
 * If it threw an error, we rethrow it.  If something within
 * the command threw Usage, we set our flag to be consistent.
 *
 * If something within the command-block set our flag (but
 * didn't throw), then we take care of it.
 */
#define INVOKE(_c_)	SG_STATEMENT(						\
	SG_ASSERT( (SG_context__has_err(pCtx)==SG_FALSE) );	\
	if (*pbUsageError)									\
		SG_ERR_THROW(  SG_ERR_USAGE  );					\
	_c_;												\
	if (SG_context__has_err(pCtx))						\
	{													\
		if (SG_context__err_equals(pCtx,SG_ERR_USAGE))	\
			*pbUsageError = SG_TRUE;						\
		SG_ERR_RETHROW;									\
	}													\
	else if (*pbUsageError)								\
	{													\
		SG_ERR_THROW(  SG_ERR_USAGE  );					\
	}													)


/* ---------------------------------------------------------------- */

DECLARE_CMD_FUNC(hid)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 1, pbUsageError)  );

	INVOKE(  do_cmd_hid(pCtx,paszArgs[0])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(dump_json)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_dump_json(pCtx, pOptSt->psz_repo, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(check_integrity)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 1, pbUsageError)  );

	INVOKE(  do_cmd_check_integrity(pCtx, paszArgs[0])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(walk_fulltree)
{
	SG_UNUSED(pOptSt);
	SG_UNUSED(pGetopt);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);
	INVOKE(  do_cmd_walk_fulltree(pCtx)  );

fail:
	return;
}

DECLARE_CMD_FUNC(manually_set_parents)
{
	const char** paszArgs= NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_manually_set_parents(pCtx, pOptSt->pRevSpec)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}


DECLARE_CMD_FUNC(repo)
{
	const char** paszArgs= NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_repo(
		pCtx,
		!strcmp(pszCommandName, "repos") || !strcmp(pszCommandName, "repositories"),
		count_args, paszArgs,
		pOptSt->bAllBut, pOptSt->bNoPrompt,
		pOptSt->bVerbose,
		pOptSt->psz_storage, pOptSt->psz_hash, pOptSt->psz_shared_users,
		pExitStatus)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(serve)
{
	const char** paszArgs= NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if(count_args>0)
		SG_ERR_THROW(SG_ERR_USAGE);

	INVOKE(  do_cmd_serve(pCtx, pOptSt, pExitStatus)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(graph_history)
{
	const char** paszArgs= NULL;
	const char * pszStamp=NULL;
	SG_uint32 count_args = 0;
	SG_uint32 count_stamps = 0;
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if (count_args > 1)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "History queries only support one file/folder argument at a time.\n")  );
		*pbUsageError = SG_TRUE;
		goto fail;
	}
	/* TODO Put check for valid args here */
	//History only supports one stamp argument at the moment.
	if (pOptSt->psa_stamps != NULL)
	{
		SG_ERR_CHECK(  SG_stringarray__count(pCtx, pOptSt->psa_stamps, &count_stamps)  );
		if (count_stamps > 1)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "History queries only support one stamp at a time.\n")  );
			*pbUsageError = SG_TRUE;
			goto fail;
		}
		else if (count_stamps == 1)
		{
			SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, pOptSt->psa_stamps, 0, &pszStamp)  );
		}
	}

	INVOKE(
		do_cmd_history(pCtx, SG_TRUE, pOptSt->psz_repo, count_args, paszArgs, pOptSt->pRevSpec, pOptSt->psz_username, pszStamp, pOptSt->psz_from_date, pOptSt->psz_to_date, pOptSt->bLeaves, pOptSt->iMaxHistoryResults, pOptSt->bHideMerges, pOptSt->bReverseOutput, pOptSt->bVerbose);
		);

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(history)
{
	const char** paszArgs= NULL;
	const char * pszStamp=NULL;
	SG_uint32 count_args = 0;
	SG_uint32 count_stamps = 0;
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (count_args > 1)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "History queries only support one file/folder argument at a time.\n")  );
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	/* TODO Put check for valid args here */
	//History only supports one stamp argument at the moment.
	if (pOptSt->psa_stamps != NULL)
	{
		SG_ERR_CHECK(  SG_stringarray__count(pCtx, pOptSt->psa_stamps, &count_stamps)  );
		if (count_stamps > 1)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "History queries only support one stamp at a time.\n")  );
			*pbUsageError = SG_TRUE;
			goto fail;
		}
		else if (count_stamps == 1)
		{
			SG_ERR_CHECK(  SG_stringarray__get_nth(pCtx, pOptSt->psa_stamps, 0, &pszStamp)  );
		}
	}

	INVOKE(
		do_cmd_history(pCtx, SG_FALSE, pOptSt->psz_repo, count_args, paszArgs, pOptSt->pRevSpec, pOptSt->psz_username, pszStamp, pOptSt->psz_from_date, pOptSt->psz_to_date, pOptSt->bLeaves, pOptSt->iMaxHistoryResults, pOptSt->bHideMerges, pOptSt->bReverseOutput, pOptSt->bVerbose);
		);

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(localsettings)
{
	const char** paszArgs= NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_localsettings(pCtx, pOptSt->bVerbose, pOptSt->bForce, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(dump_lca)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if (count_args)
		*pbUsageError = SG_TRUE;

#if defined(DEBUG)
	if (pOptSt->bGaveDagNum)
	{
		SG_uint32 tagCount = 0;
		SG_ERR_CHECK(  SG_rev_spec__count_tags(pCtx, pOptSt->pRevSpec, &tagCount)  );
		if (tagCount > 0)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "tags cannot be specified in combination with dagnums\n")  );
			*pbUsageError = SG_TRUE;
		}
	}
#endif

	if (pOptSt->bList && pOptSt->bVerbose)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Cannot use both --list and --verbose.\n")  );
		*pbUsageError = SG_TRUE;
	}

	// allow "vv dump_lca [--list | --verbose] [--rev=r1 | --tag=t1 [--rev=r2 | --tag=t2 ...]] [repo_desc_name dag_num]"
	
	INVOKE(  do_cmd_dump_lca(pCtx, pOptSt)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

#ifdef DEBUG
DECLARE_CMD_FUNC(dump_treenode)
{
	const char** paszArgs = NULL;
	const char* pszChangesetId = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (count_args > 0 )
		pszChangesetId = paszArgs[0];

	INVOKE(  do_cmd_dump_treenode(pCtx, pOptSt->psz_repo, pszChangesetId)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}
#endif

DECLARE_CMD_FUNC(commit)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;
    char* psz_tied_branch = NULL;
	SG_string *pstrBoilerplate = NULL;
	SG_pendingtree *pPendingTree = NULL;
	SG_vhash *pvh_user = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    SG_ERR_CHECK(  my_get_tied_branch(pCtx, &psz_tied_branch)  );

    if (psz_tied_branch && pOptSt->bAllowDetached)
    {
        SG_console(pCtx, SG_CS_STDERR, "--detached not allowed when the working dir is attached to a branch.\n");
		SG_ERR_THROW(SG_ERR_USAGE);
    }
    else if (!psz_tied_branch && !pOptSt->bAllowDetached)
    {
        SG_console(pCtx, SG_CS_STDERR, "The working copy is not attached to a branch.  Use --detached to commit anyway.\n");
		SG_ERR_THROW(SG_ERR_USAGE);
    }

	if (pOptSt->psz_username)
	{
		SG_repo *pRepo = NULL;

		SG_ERR_CHECK(  SG_PENDINGTREE__ALLOC_FROM_CWD(pCtx, &pPendingTree)  );
		SG_ERR_CHECK(  SG_pendingtree__get_repo(pCtx, pPendingTree, &pRepo)  );

		SG_ERR_CHECK(  SG_user__lookup_by_name(pCtx, pRepo, pOptSt->psz_username, &pvh_user)  );

		if (! pvh_user)
			SG_ERR_CHECK(  SG_user__lookup_by_userid(pCtx, pRepo, pOptSt->psz_username, &pvh_user)  );

		if (! pvh_user)
		{
			const char *descriptor = NULL;

			SG_ERR_CHECK(  SG_repo__get_descriptor_name(pCtx, pRepo, &descriptor)  );

			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "User %s does not exist in the %s repository\n", pOptSt->psz_username, descriptor)  );
			*pExitStatus = 1;
			goto fail;
		}

		SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
		SG_VHASH_NULLFREE(pCtx, pvh_user);
	}

	if (pOptSt->bPromptForDescription)
	{
		SG_ERR_CHECK(  SG_cmd_util__build_comment_boilerplate(pCtx, SG_TRUE, count_args, paszArgs, pOptSt, &pstrBoilerplate)  );
	}
	else
	{
		SG_bool   allWhitespace = SG_FALSE;
		
		if ((! pOptSt->psz_message) || (! *pOptSt->psz_message))
			SG_ERR_THROW(SG_ERR_EMPTYCOMMITMESSAGE);

		SG_ERR_CHECK(  SG_sz__is_all_whitespace(pCtx, pOptSt->psz_message, &allWhitespace)  );

		if (allWhitespace)
			SG_ERR_THROW(  SG_ERR_EMPTYCOMMITMESSAGE  );
	}
	
	{
		INVOKE(  do_cmd_commit
				 ( pCtx,
				   count_args, paszArgs,
				   pOptSt->psz_message,
				   pOptSt->psz_username,
				   pOptSt->psz_when,
				   pOptSt->psa_stamps,
				   pOptSt->bRecursive,
				   pOptSt->pFilespec,
				   pOptSt->ppAssocs, pOptSt->iCountAssocs,
				   pOptSt->bNoSubmodules,
				   pOptSt->bPromptForDescription ? SG_cmd_util__get_comment_from_editor : NULL,	
				   pstrBoilerplate
					 )  
			);
	}

fail:
	if (SG_context__has_err(pCtx) && SG_context__err_equals(pCtx, SG_ERR_EMPTYCOMMITMESSAGE))
	{
		char bufErrorMessage[SG_ERROR_BUFFER_SIZE+1];
		SG_error__get_message(SG_ERR_EMPTYCOMMITMESSAGE, SG_TRUE, bufErrorMessage, SG_ERROR_BUFFER_SIZE);
		SG_context__err_reset(pCtx);

		SG_ERR_IGNORE(  SG_log__report_verbose(pCtx,
									 "Command '%s' completed with error:\n%s\n\n",
											   SG_getopt__get_command_name(pGetopt), bufErrorMessage)  );

		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "%s: Commit aborted: no comment.\n",
								   SG_getopt__get_app_name(pGetopt))  );

		*pExitStatus = 1;
	}

	SG_PENDINGTREE_NULLFREE(pCtx, pPendingTree);
	SG_NULLFREE(pCtx, paszArgs);
	SG_NULLFREE(pCtx, psz_tied_branch);
	SG_STRING_NULLFREE(pCtx, pstrBoilerplate);
    SG_VHASH_NULLFREE(pCtx, pvh_user);
}

DECLARE_CMD_FUNC(revert)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pExitStatus);

	if (pOptSt->bTest && pOptSt->bVerbose)		// don't allow both --test and --verbose
	{
		_print_invalid_option_combination(pCtx, pszAppName, pszCommandName, (SG_int32)'T', (SG_int32)opt_verbose);
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	if (pOptSt->bAll)
	{
		SG_bool bHasIncludeExclude = SG_FALSE;

		if (!pOptSt->bRecursive)				// Don't allow revert if --all is specified with --nonrecursive
		{
			_print_invalid_option_combination(pCtx, pszAppName, pszCommandName, (SG_int32)'N', (SG_int32)opt_all);
			*pbUsageError = SG_TRUE;
			goto fail;
		}

		SG_ERR_CHECK(  SG_file_spec__has_pattern(pCtx, pOptSt->pFilespec, SG_FILE_SPEC__PATTERN__INCLUDE | SG_FILE_SPEC__PATTERN__EXCLUDE, &bHasIncludeExclude)  );
		if (bHasIncludeExclude == SG_TRUE)		// don't allow --include, --exclude, --include-from, or --exclude-from with --all
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Cannot use --include, --include-from, --exclude, or --exclude-from with --all\n")  );
			*pbUsageError = SG_TRUE;
			goto fail;
		}
	}

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if ((!paszArgs && !pOptSt->bAll))				// Don't allow revert if no paszArgs are specified (unless --all)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Missing pathnames or --all\n")  );
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	if (paszArgs && pOptSt->bAll)					// don't allow pathnames and --all
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Cannot give pathnames and --all\n")  );
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	INVOKE(  do_cmd_revert(pCtx,
						   pOptSt,
						   count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(lock)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pOptSt);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	/* Put check for valid args here */

	INVOKE(  do_cmd_lock(pCtx, pOptSt, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(unlock)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pOptSt);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	/* Put check for valid args here */

	INVOKE(  do_cmd_unlock(pCtx, pOptSt->bForce, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(remove)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	/* Put check for valid args here */

	INVOKE(  do_cmd_remove(pCtx, count_args, paszArgs, pOptSt->pFilespec, pOptSt->bTest, pOptSt->bForce, pOptSt->bNoBackups)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(add)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	/* Put check for valid args here */

	INVOKE(  do_cmd_add(pCtx, pOptSt->bTest, count_args, paszArgs, pOptSt->bRecursive, pOptSt->pFilespec)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(help)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (count_args > 2)
	{
		*pbUsageError = SG_TRUE;
	}

	INVOKE(
		if (0 == count_args)
			do_cmd_help__list_all_commands(pCtx, SG_CS_STDOUT, pszAppName, pOptSt);
		else
		{
			const char *cmd = paszArgs[0];
			const char *subcmd = (1 == count_args) ? NULL : paszArgs[1];
			SG_bool	listed = SG_FALSE;

			SG_ERR_CHECK(  do_cmd_help__list_bonus_help(pCtx, SG_CS_STDOUT, cmd, &listed) );

			if (! listed)
			{
				do_cmd_help__list_command_help(pCtx, SG_CS_STDOUT, pszAppName, cmd, subcmd, SG_FALSE);

				if (SG_context__err_equals(pCtx, SG_ERR_USAGE))
				{
					do_cmd_help__list_all_commands(pCtx, SG_CS_STDERR, pszAppName, pOptSt);
					SG_context__err_reset(pCtx);
				}
			}
		}
		);

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(version)
{

	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;
	
	const char * szVersion= NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	INVOKE(
		if(count_args==0)
		{
			SG_lib__version(pCtx, &szVersion);
			if(!SG_context__has_err(pCtx))
				SG_console(pCtx, SG_CS_STDOUT, "%s\n", szVersion);
		}
		else if(count_args==1)
		{
			do_cmd_version__remote(pCtx, paszArgs[0]);
		}
		else
			*pbUsageError = SG_TRUE;
	);

	// fall through
fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(blobcount)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 1, pbUsageError)  );

    INVOKE(  do_cmd_blobcount(pCtx, paszArgs[0])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

#if 0
DECLARE_CMD_FUNC(group)
{
	const char** paszArgs = NULL;
    SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    if (count_args < 1)
    {
        SG_ERR_THROW2(  SG_ERR_INVALIDARG,
                       (pCtx, "Must include group name")  );
    }
    else if (count_args == 1)
    {
        INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__LIST_SHALLOW, NULL, 0)  );
    }
    else
    {
        if (0 == strcmp(paszArgs[1], "addusers"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__ADD_USERS, paszArgs+2, count_args-2)  );
        }
        else if (0 == strcmp(paszArgs[1], "addsubgroups"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__ADD_SUBGROUPS, paszArgs+2, count_args-2)  );
        }
        else if (0 == strcmp(paszArgs[1], "removeusers"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__REMOVE_USERS, paszArgs+2, count_args-2)  );
        }
        else if (0 == strcmp(paszArgs[1], "removesubgroups"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__REMOVE_SUBGROUPS, paszArgs+2, count_args-2)  );
        }
        else if (0 == strcmp(paszArgs[1], "list"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__LIST_SHALLOW, paszArgs+2, count_args-2)  );
        }
        else if (0 == strcmp(paszArgs[1], "listdeep"))
        {
            INVOKE(  do_cmd_group(pCtx, paszArgs[0], pOptSt->psz_repo, MY_CMD_GROUP_OP__LIST_DEEP, paszArgs+2, count_args-2)  );
        }
        else
        {
            SG_ERR_THROW2(  SG_ERR_INVALIDARG,
                           (pCtx, "Invalid subcommand: %s", paszArgs[1])  );
        }
    }

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(creategroup)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 1, pbUsageError)  );

    INVOKE(  do_cmd_creategroup(pCtx, paszArgs[0], pOptSt->psz_repo)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(groups)
{
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pGetopt);
	SG_UNUSED(pExitStatus);

    INVOKE(  do_cmd_groups(pCtx, pOptSt->psz_repo)  );

fail:
    ;
}

#endif

DECLARE_CMD_FUNC(zingmerge)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    INVOKE(  do_zingmerge(pCtx, pOptSt->psz_repo)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(reindex)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    INVOKE(  do_reindex(pCtx, pOptSt->psz_repo)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(whoami)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;
	SG_string * pstr_path = NULL;
	char * psz_username = NULL;
	SG_repo * pRepo = NULL;
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

    // TODO what about scope of whoami information?

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
    if (1 == count_args)
    {
		INVOKE(  do_cmd_set_whoami(pCtx, paszArgs[0], pOptSt->psz_repo, pOptSt->bCreate)  );
    }
    else
    {
        if (pOptSt->psz_repo)
        {
            SG_ERR_CHECK(  SG_REPO__OPEN_REPO_INSTANCE(pCtx, pOptSt->psz_repo, &pRepo)  );
        }
        else
        {
            SG_ERR_CHECK(  SG_cmd_util__get_repo_from_cwd(pCtx, &pRepo, NULL)  );
        }
    	SG_user__get_username_for_repo(pCtx, pRepo, &psz_username);
		if (SG_context__err_equals(pCtx, SG_ERR_NO_CURRENT_WHOAMI)  )
    	{
    		SG_ERR_CHECK(  SG_context__err_reset(pCtx)  );
    		SG_ERR_IGNORE(  SG_console__raw(pCtx, SG_CS_STDOUT, "The user has not been set yet.\n")  );
    		*pbUsageError = SG_TRUE;
    	}
    	else if (SG_context__has_err(pCtx))
    	{
    		SG_ERR_CHECK_CURRENT;
    	}
    	else
        {
    		SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDOUT, "%s\n", psz_username)  );
        }
    }

fail:
	SG_REPO_NULLFREE(pCtx, pRepo);
	SG_NULLFREE(pCtx, psz_username);
	SG_STRING_NULLFREE(pCtx, pstr_path);
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(locks)
{
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pGetopt);
	SG_UNUSED(pExitStatus);

    INVOKE(  do_cmd_locks(pCtx, pOptSt)  );

fail:
    ;
}

DECLARE_CMD_FUNC(user)
{
	const char** paszArgs= NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pGetopt);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_user(pCtx, !strcmp(pszCommandName,"users"), pOptSt->psz_repo, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(zip)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 1, pbUsageError)  );

    INVOKE(  do_cmd_zip(pCtx, pOptSt, paszArgs[0])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(clone)
{
	const char** paszArgs = NULL;
    SG_uint32 count_mutually_exclusive_options = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 2, pbUsageError)  );

    if (pOptSt->bPack)
    {
        count_mutually_exclusive_options++;
    }
    if (pOptSt->b_all_full)
    {
        count_mutually_exclusive_options++;
    }
    if (pOptSt->b_all_zlib)
    {
        count_mutually_exclusive_options++;
    }

    if (count_mutually_exclusive_options > 1)
    {
        SG_console(pCtx, SG_CS_STDERR, "clone can accept only one of --pack, --all-full, or --all-zlib\n");
        SG_ERR_THROW(SG_ERR_USAGE);
    }

    if (!pOptSt->bPack)
    {
    }

    INVOKE(  do_cmd_clone(pCtx, paszArgs[0], paszArgs[1], pOptSt)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(checkout)
{
	SG_uint32 countBranchesSpecified;
	const char** paszArgs = NULL;
	SG_uint32 count_args;

	SG_UNUSED(pszAppName);

	SG_ERR_CHECK(  SG_rev_spec__count_branches(pCtx, pOptSt->pRevSpec, &countBranchesSpecified)  );
    if (countBranchesSpecified && pOptSt->psz_attach)
    {
        _print_invalid_option_combination(pCtx, pszAppName, pszCommandName, (SG_int32)'b', (SG_int32)opt_attach);
        *pbUsageError = SG_TRUE;
        goto fail;
    }

	// we have "vv checkout [<options>] <repository> [<pathname>]"
	// so we should have 1 or 2 non-option args.
	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if ((count_args < 1) || (count_args > 2))
	{
		*pbUsageError = SG_TRUE;
		goto fail;
	}
	
	INVOKE(  do_cmd_checkout(pCtx,
							 paszArgs[0],
							 pOptSt->pRevSpec,
							 ((count_args > 1) ? paszArgs[1] : NULL),
							 pOptSt->psz_attach,
							 pOptSt->bSubmodule,
							 pOptSt->pvec_use_submodules,
							 pExitStatus)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(fast_import)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 2, pbUsageError)  );

	INVOKE(  do_cmd_fast_import(pCtx, pOptSt, paszArgs[0], paszArgs[1])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(fast_export)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 2, pbUsageError)  );

	INVOKE(  do_cmd_fast_export(pCtx, pOptSt, paszArgs[0], paszArgs[1])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(export)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	if ((pOptSt->bNoSubmodules) && (pOptSt->pvec_use_submodules))
	{
		// --no-submodules says not to export any contained submodules.
		// --use-sub is one or more hints to help reverse-map submodule repo-ids to repo-names.
		// it doesn't make any sense to give both.  we could let the slide if we want.
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Cannot combine --no-submodules and --use-sub")  );
	}

	// we have "vv export [<options>] <repository> [<pathname>]"
	// so we should have 1 or 2 non-option args.
	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if ((count_args < 1) || (count_args > 2))
	{
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	INVOKE(  do_cmd_export(pCtx,
						   paszArgs[0],
						   pOptSt->pRevSpec,
						   ((count_args > 1) ? paszArgs[1] : NULL),
						   pOptSt->bNoSubmodules,
						   pOptSt->pvec_use_submodules)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(rename)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);
	SG_UNUSED(pOptSt);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 2, pbUsageError)  );

	INVOKE(  do_cmd_rename(pCtx, paszArgs[0], paszArgs[1])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(resolve2)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_resolve2(pCtx, pOptSt, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(move)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);
	SG_UNUSED(pOptSt);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (count_args < 2)
	{
		*pbUsageError = SG_TRUE;
	}
	else
		INVOKE(  do_cmd_move(pCtx, paszArgs, count_args - 1, paszArgs[count_args - 1])  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(init_new_repo)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	const char * pszRepoName = NULL;
	const char * pszFolder = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if ((count_args != 2))
	{
		*pbUsageError = SG_TRUE;				// we only allow: repo_name folder
	}
	else
	{
		pszRepoName = paszArgs[0];
		pszFolder = paszArgs[1];
	}

	INVOKE(  do_cmd_init_new_repo(pCtx,
								  pszRepoName,
								  pszFolder,
								  pOptSt->psz_storage,
								  pOptSt->psz_hash,
								  SG_FALSE,
								  pOptSt->bSubmodule,
                                  pOptSt->psz_shared_users)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(parents)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (count_args > 1)  //We support only one argument
		*pbUsageError = SG_TRUE;
	else
		INVOKE(  do_cmd_parents(pCtx, count_args, paszArgs, pOptSt->pRevSpec, pOptSt->bVerbose)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}


DECLARE_CMD_FUNC(scan)
{
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 0, pbUsageError)  );

	INVOKE(  do_cmd_scan(pCtx)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(addremove)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_addremove(pCtx,
							  count_args, paszArgs,
							  pOptSt->bRecursive,
							  pOptSt->pFilespec,
							  pOptSt->bTest)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(status)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_string* pstrRepoName = NULL;
	
	char* pszRev1 = NULL;
	char* pszRev2 = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_cwd(pCtx, &pstrRepoName, NULL)  );

	/* Get revision specifiers. We have 0, 1, or 2. */
	SG_ERR_CHECK(  _get_zero_one_or_two_revision_specifiers(pCtx, SG_string__sz(pstrRepoName), pOptSt->pRevSpec, &pszRev1, &pszRev2)  );

	INVOKE(  do_cmd_status(pCtx,
						   count_args,paszArgs,
						   pOptSt->bRecursive,
						   pOptSt->bVerbose,
						   pOptSt->pFilespec,
						   pszRev1, pszRev2)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
	SG_STRING_NULLFREE(pCtx, pstrRepoName);
	SG_NULLFREE(pCtx, pszRev1);
	SG_NULLFREE(pCtx, pszRev2);
}

DECLARE_CMD_FUNC(stamp)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    INVOKE(  do_cmd_stamp(pCtx, pOptSt->psz_repo, !strcmp(pszCommandName,"stamps"), count_args, paszArgs, pOptSt->pRevSpec, pOptSt->bAll)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

#if 0
DECLARE_CMD_FUNC(slurp)
{
	SG_bool bUsageError = SG_FALSE;
	SG_uint32 i;
	cmdinfo* thisCmd;
	const char** paszArgs;
	SG_uint32 count_args = 0;
	
	SG_UNUSED(pExitStatus);

	_get_canonical_command(pszCommandName, &thisCmd);


	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    if (2 != count_args)
    {
		bUsageError = SG_TRUE;
    }

    INVOKE(  do_cmd_slurp(pCtx, paszArgs[0], paszArgs[1])  );

fail:
	if (bUsageError)
	{
        SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Usage: %s %s", pszAppName, pszCommandName)  );
        SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, " descriptor_name filename.fragball\n")  );
	}

	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(fragball)
{
	SG_bool bUsageError = SG_FALSE;
	SG_uint32 i;
	cmdinfo* thisCmd;
	const char** paszArgs;
	SG_uint32 count_args = 0;
	
	SG_UNUSED(pExitStatus);

	_get_canonical_command(pszCommandName, &thisCmd);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

    if (2 != count_args)
    {
		bUsageError = SG_TRUE;
    }

    INVOKE(  do_cmd_fragball(pCtx, paszArgs[0], paszArgs[1])  );

fail:
	if (bUsageError)
	{
        SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "Usage: %s %s", pszAppName, pszCommandName)  );
        SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, " filename.fragball changesetid\n")  );
	}

	SG_NULLFREE(pCtx, paszArgs);
}
#endif

DECLARE_CMD_FUNC(comment)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;
	SG_string* pstrRepoName = NULL;
	char* pszChangesetId = NULL;
	SG_string *boilerplate = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (0 != count_args)
		*pbUsageError = SG_TRUE;

	SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_options_or_cwd(pCtx, pOptSt, &pstrRepoName, NULL)  );
	SG_ERR_CHECK(  SG_rev_spec__get_one(pCtx, SG_string__sz(pstrRepoName), pOptSt->pRevSpec, &pszChangesetId, NULL)  );

	if (pOptSt->bPromptForDescription)
	{
		SG_ERR_CHECK(  SG_cmd_util__build_comment_boilerplate(pCtx, SG_FALSE, count_args, paszArgs, pOptSt, &boilerplate)  );
		SG_ERR_CHECK(  SG_cmd_util__get_comment_from_editor(pCtx, boilerplate, &pOptSt->psz_message, NULL)  );
	}

	{
		SG_bool bNoComment = SG_FALSE;
		if (pOptSt->psz_message)
		{
			SG_ERR_CHECK(  SG_sz__is_all_whitespace(pCtx, pOptSt->psz_message, &bNoComment)  );
		}
		else
			bNoComment = SG_TRUE;

		if(bNoComment)
		{
			SG_ERR_CHECK(  SG_console__raw(pCtx, SG_CS_STDOUT, "Comment cancelled: empty comment.\n")  );
		}
		else
		{
			INVOKE(  do_cmd_comment(pCtx, pszChangesetId, pOptSt->psz_message)  );
		}
	}

fail:
	SG_NULLFREE(pCtx, paszArgs);
	SG_STRING_NULLFREE(pCtx, pstrRepoName);
	SG_NULLFREE(pCtx, pszChangesetId);
	SG_STRING_NULLFREE(pCtx, boilerplate);
}

DECLARE_CMD_FUNC(tag)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_tag(pCtx, pOptSt->psz_repo, !strcmp(pszCommandName,"tags") || !strcmp(pszCommandName,"labels"), count_args, paszArgs, pOptSt->pRevSpec)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}


DECLARE_CMD_FUNC(cat)
{
	const char** paszArgs = NULL;
	SG_uint32 countArgs = 0;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &countArgs)  );

	INVOKE(  do_cmd_cat(pCtx, countArgs, paszArgs, pOptSt)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(vcdiff)
{
	SG_uint32 count_args = 0;
	const char** paszArgs = NULL;

	SG_UNUSED(pOptSt);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	INVOKE(  do_cmd_vcdiff(pCtx, count_args, paszArgs)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}


DECLARE_CMD_FUNC(diff)
{
	SG_uint32 count_args = 0;
	const char** paszArgs = NULL;

	SG_string* pstrRepoName = NULL;

	char* pszRev1 = NULL;
	char* pszRev2 = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_cwd(pCtx, &pstrRepoName, NULL)  );
	SG_ERR_CHECK(  _get_zero_one_or_two_revision_specifiers(pCtx, SG_string__sz(pstrRepoName), pOptSt->pRevSpec, &pszRev1, &pszRev2)  );

	INVOKE(  do_cmd_diff(pCtx,
						 count_args, paszArgs, pOptSt->bRecursive,
						 pOptSt->pFilespec,
						 pszRev1, pszRev2,
						 pOptSt->psz_tool)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
	SG_STRING_NULLFREE(pCtx, pstrRepoName);
	SG_NULLFREE(pCtx, pszRev1);
	SG_NULLFREE(pCtx, pszRev2);
}
DECLARE_CMD_FUNC(diffmerge)
{
	SG_uint32 count_args = 0;
	const char** paszArgs = NULL;

	SG_string* pstrRepoName = NULL;

	char* pszRev1 = NULL;
	char* pszRev2 = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if(count_args>0)
	{
		*pbUsageError = SG_TRUE;
	}

	// get any revision specifiers
	else
	{
		SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_cwd(pCtx, &pstrRepoName, NULL)  );
		SG_ERR_CHECK(  _get_zero_one_or_two_revision_specifiers(pCtx, SG_string__sz(pstrRepoName), pOptSt->pRevSpec, &pszRev1, &pszRev2)  );
	}

	if ((pOptSt->bNoSubmodules) && (pOptSt->pvec_use_submodules))
	{
		// --no-submodules says not to export any contained submodules.
		// --use-sub is one or more hints to help reverse-map submodule repo-ids to repo-names.
		// it doesn't make any sense to give both.  we could let the slide if we want.
		SG_ERR_THROW2(  SG_ERR_INVALIDARG,
						(pCtx, "Cannot combine --no-submodules and --use-sub")  );
	}

	INVOKE(  do_cmd_diffmerge(pCtx, pszRev1, pszRev2, 
							  pOptSt->bNoSubmodules,
							  pOptSt->pvec_use_submodules)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
	SG_STRING_NULLFREE(pCtx, pstrRepoName);
	SG_NULLFREE(pCtx, pszRev1);
	SG_NULLFREE(pCtx, pszRev2);
}

DECLARE_CMD_FUNC(merge)
{
	const char** paszArgs = NULL;
	SG_string* pstrRepoName = NULL;
	char * pszHid = NULL;
	const char * pszToolOverride = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 0, pbUsageError)  );

	// --no-auto-merge or --tool or neither

	if (pOptSt->bNoAutoMerge)
	{
		if (pOptSt->psz_tool)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "--no-auto-merge and --tool are incompatible\n")  );
			*pbUsageError = SG_TRUE;
		}
		else
		{
			pszToolOverride = SG_MERGETOOL__INTERNAL__SKIP;
		}
	}
	else
	{
		pszToolOverride = pOptSt->psz_tool;	// a value or null
	}

	INVOKE(  do_cmd_merge(pCtx,
						  pOptSt->pRevSpec,
						  pszToolOverride,
						  pOptSt->bVerbose, pOptSt->bTest,
						  pOptSt->pvec_use_submodules)  );

fail:
	SG_STRING_NULLFREE(pCtx, pstrRepoName);
	SG_NULLFREE(pCtx, pszHid);
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(branch)
{
	const char** paszArgs = NULL;
	SG_uint32 count_args = 0;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	if (*pbUsageError)
	{
		goto fail;
	}

	INVOKE(  do_cmd_branch(pCtx, !strcmp(pszCommandName,"branches"), count_args, paszArgs, pOptSt)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(leaves)
{
	SG_uint32 count_args = 0;
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if (count_args)
		*pbUsageError = SG_TRUE;

	INVOKE(  do_cmd_leaves(pCtx, pOptSt)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(heads)
{
	SG_uint32 count_args = 0;
	const char** paszArgs = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );
	if (count_args)
		*pbUsageError = SG_TRUE;

	INVOKE(  do_cmd_heads(pCtx, pOptSt->psz_repo, pOptSt->psa_branch_names, pOptSt->bAll, pOptSt->bVerbose)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
}

DECLARE_CMD_FUNC(update)
{
	const char ** paszArgs = NULL;
	char* pszTargetHid = NULL;
	char* pszTargetBranch = NULL;

	if (pOptSt->bTest && pOptSt->bVerbose)		// don't allow both --test and --verbose
	{
		_print_invalid_option_combination(pCtx, pszAppName, pszCommandName, (SG_int32)'T', (SG_int32)opt_verbose);
		*pbUsageError = SG_TRUE;
		goto fail;
	}

	if (pszTargetBranch && pOptSt->psz_attach)
    {
        _print_invalid_option_combination(pCtx, pszAppName, pszCommandName, (SG_int32)'b', (SG_int32)opt_attach);
        *pbUsageError = SG_TRUE;
        goto fail;
    }

	// We DO NOT take any non-option args.  Verify that we don't have any.
	//
	// TODO currently SG_getop__parse_all_args() has to build an actual
	// TODO array of them.  we don't need that, we just want the lenBranchNames.
	// TODO see if there is a function in SG_getopt to return the number of non-option arguments
	// TODO rather than extracting an array of everything left over.  (Or hack __parse_all_args
	// TODO to take a null for the array pointer and have it just return the lenBranchNames.)
	SG_ERR_CHECK(  _parse_and_count_args(pCtx, pGetopt, &paszArgs, 0, pbUsageError)  );

	INVOKE(  do_cmd_update(pCtx,
						   pOptSt->pRevSpec,
                           pOptSt->psz_attach,
						   pOptSt->bVerbose, pOptSt->bTest,
						   pOptSt->pvec_use_submodules,
						   pExitStatus)  );

fail:
	SG_NULLFREE(pCtx, paszArgs);
	SG_NULLFREE(pCtx, pszTargetHid);
	SG_NULLFREE(pCtx, pszTargetBranch);
}

static void _parse_pull_args(
	SG_context* pCtx, 
	SG_option_state* pOptSt,
	SG_getopt* pGetopt,
	SG_pathname** ppPathCwd,
	const char** ppszDestRepoDescriptorName,
	const char** ppszSrcRepoSpec)
{
	const char ** paszArgs = NULL;
	SG_uint32 count_args;
	SG_pathname* pPathCwd = NULL;
	SG_string* pstrCwdDescriptorName = NULL;
	char* pszDestRepoDescriptorName = NULL;
	char* pszSrcRepoSpec = NULL;
	const char* pszRefSrcRepoSpec;
	char* pszProtocol = NULL;
	SG_string* pstrSavedPath = NULL;

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	// Determine destination repo
	if (pOptSt->psz_dest_repo)
	{
		SG_ERR_CHECK(  SG_STRDUP(pCtx, pOptSt->psz_dest_repo, &pszDestRepoDescriptorName)  );
		if (pOptSt->bUpdate)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nThe update and dest flags cannot be combined.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
	}
	else
	{
		SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_cwd(pCtx, &pstrCwdDescriptorName, &pPathCwd)  );
		SG_ERR_CHECK(  SG_string__sizzle(pCtx, &pstrCwdDescriptorName, (SG_byte**)&pszDestRepoDescriptorName, NULL)  );
	}

	// Determine source repo
	if (0 == count_args)
	{
		if (!pszDestRepoDescriptorName)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nNot in a working copy and no SOURCEREPO specified.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}

		SG_localsettings__descriptor__get__sz(pCtx, pszDestRepoDescriptorName, "paths/default", &pszSrcRepoSpec);
		if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
		{
			SG_ERR_DISCARD;
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nNo default path and no SOURCEREPO specified.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		if (SG_context__err_equals(pCtx, SG_ERR_JSONDB_INVALID_PATH))
		{
			SG_ERR_DISCARD;
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\ndest must be a local repository name.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		else
			SG_ERR_CHECK_CURRENT;
		SG_ERR_CHECK_CURRENT;

		SG_ERR_CHECK(  SG_STRDUP(pCtx, pszSrcRepoSpec, (char**)&pszRefSrcRepoSpec)  );
	}
	else if (1 == count_args)
	{
		// A source repo argument was provided.

		SG_bool bExists = SG_FALSE;

		// Is it an http url?
		if (strlen(paszArgs[0]) > 6)
		{
			SG_ERR_CHECK(  SG_ascii__substring(pCtx, paszArgs[0], 0, 7, &pszProtocol)  );
			if (SG_stricmp("http://", (const char*)pszProtocol) == 0)
				bExists = SG_TRUE; // remote repo, existence is determined elsewhere
			SG_NULLFREE(pCtx, pszProtocol);
		}

		// Is it a valid descriptor name?
		if (!bExists)
			SG_ERR_CHECK(  SG_closet__descriptors__exists(pCtx, paszArgs[0], &bExists)  );

		if (bExists)
		{
		  SG_ERR_CHECK(  SG_STRDUP(pCtx, paszArgs[0], (char**)&pszRefSrcRepoSpec)  );
		}
		else // Is it a saved alias?
		{
			if (!pszDestRepoDescriptorName)
			{
				SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
					"\nThere is no local repository instance matching %s.\n\n",
					paszArgs[0])  );
				SG_ERR_THROW(SG_ERR_USAGE);
			}

			SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstrSavedPath)  );
			SG_ERR_CHECK(  SG_string__sprintf(pCtx, pstrSavedPath, "paths/%s", paszArgs[0])  );

			SG_localsettings__descriptor__get__sz(pCtx, pszDestRepoDescriptorName,
				SG_string__sz(pstrSavedPath), &pszSrcRepoSpec);
			if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
			{
				SG_ERR_DISCARD;
				SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
					"\nThere is no local repository instance or saved alias matching %s.\n\n",
					paszArgs[0])  );
				SG_ERR_THROW(SG_ERR_USAGE);
			}
			SG_ERR_CHECK_CURRENT;

			SG_ERR_CHECK(  SG_STRDUP(pCtx, pszSrcRepoSpec, (char**)&pszRefSrcRepoSpec)  );
		}
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}

	SG_RETURN_AND_NULL(pPathCwd, ppPathCwd);
	SG_RETURN_AND_NULL(pszRefSrcRepoSpec, ppszSrcRepoSpec);
	SG_RETURN_AND_NULL(pszDestRepoDescriptorName, ppszDestRepoDescriptorName);

	/* fall through */
fail:
	SG_PATHNAME_NULLFREE(pCtx, pPathCwd);
	SG_STRING_NULLFREE(pCtx, pstrCwdDescriptorName);
	SG_NULLFREE(pCtx, pszSrcRepoSpec);
	SG_NULLFREE(pCtx, paszArgs);
	SG_NULLFREE(pCtx, pszProtocol);
	SG_STRING_NULLFREE(pCtx, pstrSavedPath);
	SG_NULLFREE(pCtx, pszDestRepoDescriptorName);
}

DECLARE_CMD_FUNC(incoming)
{
	const char* pszDestRepoDescriptorName = NULL;
	const char* pszSrcRepoSpec = NULL;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_pull_args(pCtx, pOptSt, pGetopt, NULL, &pszDestRepoDescriptorName, &pszSrcRepoSpec)  );
	INVOKE(  do_cmd_incoming(pCtx, pszSrcRepoSpec, pszDestRepoDescriptorName, pOptSt->bVerbose, pOptSt->bTrace)  );

	/* fall through */
fail:
	if (SG_context__has_err(pCtx))
		*pbUsageError = SG_TRUE;
	SG_NULLFREE(pCtx, pszDestRepoDescriptorName);
	SG_NULLFREE(pCtx, pszSrcRepoSpec);
}

DECLARE_CMD_FUNC(pull)
{
	const char* pszDestRepoDescriptorName = NULL;
	const char* pszSrcRepoSpec = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);

	SG_ERR_CHECK(  _parse_pull_args(pCtx, pOptSt, pGetopt, NULL, &pszDestRepoDescriptorName, &pszSrcRepoSpec)  );

	INVOKE(  do_cmd_pull(pCtx, pOptSt, pszSrcRepoSpec, pszDestRepoDescriptorName, pExitStatus)  );

	/* fall through */
fail:
	if (SG_context__has_err(pCtx))
		*pbUsageError = SG_TRUE;
	SG_NULLFREE(pCtx, pszDestRepoDescriptorName);
	SG_NULLFREE(pCtx, pszSrcRepoSpec);
}

static void _parse_push_args(
	SG_context* pCtx, 
	SG_option_state* pOptSt,
	SG_getopt* pGetopt,
	const char** ppszSrcRepoDescriptorName,
	const char** ppszDestRepoSpec)
{
	const char ** paszArgs = NULL;
	SG_uint32 count_args;
	SG_string* pstrCwdDescriptorName = NULL;
	const char* pszSrcRepoDescriptorName = NULL;
	char* pszDestRepoSpec = NULL;
	const char* pszRefDestRepoSpec;
	char* pszProtocol = NULL;
	SG_string* pstrSavedPath = NULL;

	SG_ERR_CHECK(  SG_getopt__parse_all_args(pCtx, pGetopt, &paszArgs, &count_args)  );

	// Determine source repo
	if (pOptSt->psz_src_repo)
	  SG_ERR_CHECK(  SG_STRDUP(pCtx, pOptSt->psz_src_repo, (char**)&pszSrcRepoDescriptorName)  );
	else
	{
		SG_ERR_CHECK(  SG_cmd_util__get_descriptor_name_from_cwd(pCtx, &pstrCwdDescriptorName, NULL)  );
		SG_ERR_CHECK(  SG_STRDUP(pCtx, SG_string__sz(pstrCwdDescriptorName), (char**)&pszSrcRepoDescriptorName)  );
	}

	// Determine dest repo
	if (0 == count_args)
	{
		if (!pszSrcRepoDescriptorName)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nNot in a working copy and no DESTREPO specified.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		SG_ERR_CHECK_CURRENT;

		SG_localsettings__descriptor__get__sz(pCtx, pszSrcRepoDescriptorName, "paths/default", &pszDestRepoSpec);
		if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
		{
			SG_ERR_DISCARD;
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nNo default path and no DESTREPO specified.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		if (SG_context__err_equals(pCtx, SG_ERR_JSONDB_INVALID_PATH))
		{
			SG_ERR_DISCARD;
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\nsrc must be a local repository name.\n\n")  );
			SG_ERR_THROW(SG_ERR_USAGE);
		}
		else
			SG_ERR_CHECK_CURRENT;

		SG_ERR_CHECK(  SG_STRDUP(pCtx, pszDestRepoSpec, (char**)&pszRefDestRepoSpec)  );
	}
	else if (1 == count_args)
	{
		// A dest repo argument was provided.

		SG_bool bExists = SG_FALSE;

		// Is it an http url?
		if (strlen(paszArgs[0]) > 6)
		{
			SG_ERR_CHECK(  SG_ascii__substring(pCtx, paszArgs[0], 0, 7, &pszProtocol)  );
			if (SG_stricmp("http://", (const char*)pszProtocol) == 0)
				bExists = SG_TRUE; // remote repo, existence is determined elsewhere
			SG_NULLFREE(pCtx, pszProtocol);
		}

		// Is it a valid descriptor name?
		if (!bExists)
			SG_ERR_CHECK(  SG_closet__descriptors__exists(pCtx, paszArgs[0], &bExists)  );

		if (bExists)
		{
		  SG_ERR_CHECK(  SG_STRDUP(pCtx, paszArgs[0], (char**)&pszRefDestRepoSpec)  );
		}
		else // Is it a saved alias?
		{
			if (!pszSrcRepoDescriptorName)
			{
				SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
					"\nThere is no local repository instance matching %s.\n\n",
					paszArgs[0])  );
				SG_ERR_THROW(SG_ERR_USAGE);
			}

			SG_ERR_CHECK(  SG_STRING__ALLOC(pCtx, &pstrSavedPath)  );
			SG_ERR_CHECK(  SG_string__sprintf(pCtx, pstrSavedPath, "paths/%s", paszArgs[0])  );

			SG_localsettings__descriptor__get__sz(pCtx, pszSrcRepoDescriptorName,
				SG_string__sz(pstrSavedPath), &pszDestRepoSpec);
			if (SG_context__err_equals(pCtx, SG_ERR_NOT_FOUND))
			{
				SG_ERR_DISCARD;
				SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR,
					"\nThere is no local repository instance or saved alias matching %s.\n\n",
					paszArgs[0])  );
				SG_ERR_THROW(SG_ERR_USAGE);
			}
			SG_ERR_CHECK_CURRENT;

			SG_ERR_CHECK(  SG_STRDUP(pCtx, pszDestRepoSpec, (char**)&pszRefDestRepoSpec)  );
		}
	}
	else
	{
		SG_ERR_THROW(SG_ERR_USAGE);
	}

	SG_RETURN_AND_NULL(pszRefDestRepoSpec, ppszDestRepoSpec);
	SG_RETURN_AND_NULL(pszSrcRepoDescriptorName, ppszSrcRepoDescriptorName);

	/* fall through */
fail:
	SG_STRING_NULLFREE(pCtx, pstrCwdDescriptorName);
	SG_NULLFREE(pCtx, pszDestRepoSpec);
	SG_NULLFREE(pCtx, pszSrcRepoDescriptorName);
	SG_NULLFREE(pCtx, paszArgs);
	SG_NULLFREE(pCtx, pszProtocol);
	SG_STRING_NULLFREE(pCtx, pstrSavedPath);
}

DECLARE_CMD_FUNC(outgoing)
{
	const char* pszSrcRepoDescriptorName = NULL;
	const char* pszDestRepoSpec = NULL;

	SG_UNUSED(pszAppName);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pExitStatus);

	SG_ERR_CHECK(  _parse_push_args(pCtx, pOptSt, pGetopt, &pszSrcRepoDescriptorName, &pszDestRepoSpec)  );
	INVOKE(  do_cmd_outgoing(pCtx, pszDestRepoSpec, pszSrcRepoDescriptorName, pOptSt->bVerbose, pOptSt->bTrace)  );

	/* fall through */
fail:
	if (SG_context__has_err(pCtx))
		*pbUsageError = SG_TRUE;
	SG_NULLFREE(pCtx, pszSrcRepoDescriptorName);
	SG_NULLFREE(pCtx, pszDestRepoSpec);
}

DECLARE_CMD_FUNC(push)
{
	const char* pszSrcRepoDescriptorName = NULL;
	const char* pszDestRepoSpec = NULL;

	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);

	SG_ERR_CHECK(  _parse_push_args(pCtx, pOptSt, pGetopt, &pszSrcRepoDescriptorName, &pszDestRepoSpec)  );
	INVOKE(  do_cmd_push(pCtx, pOptSt, pszDestRepoSpec, pszSrcRepoDescriptorName, pExitStatus)  );

	/* fall through */
fail:
	if (SG_context__has_err(pCtx))
		*pbUsageError = SG_TRUE;
	SG_NULLFREE(pCtx, pszSrcRepoDescriptorName);
	SG_NULLFREE(pCtx, pszDestRepoSpec);
}

#if defined(HAVE_TEST_CRASH)
DECLARE_CMD_FUNC(crash)
{
	SG_uint32 * p = NULL;

	SG_UNUSED(pCtx);
	SG_UNUSED(pGetopt);
	SG_UNUSED(pOptSt);
	SG_UNUSED(pbUsageError);
	SG_UNUSED(pszCommandName);
	SG_UNUSED(pszAppName);
	SG_UNUSED(pExitStatus);

	// Time to bite the dust...

	*p = 1;

}
#endif

//////////////////////////////////////////////////////////////////

/**
 * convert (argc,argv) into a SG_getopt.  This takes care of converting
 * platform strings into UTF-8.
 *
 */

static void _intern_argv(SG_context * pCtx, int argc, ARGV_CHAR_T ** argv, SG_getopt ** ppGetopt)
{
	SG_ASSERT_RELEASE_RETURN(argc > 0);

	SG_ERR_CHECK_RETURN(  SG_getopt__alloc(pCtx, argc, argv, ppGetopt)  );

	(*ppGetopt)->interleave = 1;
}

static void _parse_option_port(SG_context *pCtx, const char *portname, const char *optstr, SG_int32 *pval)
{
	SG_int64 val;

	SG_ASSERT(SG_INT32_MAX >= 65535);
	SG_NULLARGCHECK_RETURN(portname);
	SG_NULLARGCHECK_RETURN(optstr);
	SG_NULLARGCHECK_RETURN(pval);

	SG_ERR_CHECK(  SG_int64__parse(pCtx, &val, optstr)  );

	if ((val < 1) || (val > 65535))
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "invalid value for --%s: %li -- must be an integer from 1 to 65535", portname, (long)val)  );
		SG_context__err(pCtx, SG_ERR_USAGE, __FILE__, __LINE__, "invalid port value");
	}
	else
	{
		*pval = (SG_int32)val;
	}

fail:
	;
}

static void _parse_option_unified(SG_context *pCtx, const char *pszArgName, const char *optstr, SG_int32 *pval)
{
	SG_int64 val;

	SG_NULLARGCHECK_RETURN(pszArgName);
	SG_NULLARGCHECK_RETURN(optstr);
	SG_NULLARGCHECK_RETURN(pval);

	SG_ERR_CHECK(  SG_int64__parse(pCtx, &val, optstr)  );

	if (val < 1)
	{
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "invalid value for --%s: %li -- must be an integer greater than zero", pszArgName, (long)val)  );
		SG_context__err(pCtx, SG_ERR_USAGE, __FILE__, __LINE__, "invalid value");
	}
	else
	{
		*pval = (SG_int32)val;
	}

fail:
	;
}

static void _parse_options(SG_context * pCtx, SG_getopt ** ppGetopt, SG_option_state ** ppOptSt)
{
	SG_option_state* pOptSt = *ppOptSt;
	const char* pszOptArg = NULL;
	SG_uint32 iOptId;
	SG_bool bNoMoreOptions = SG_FALSE;
	SG_file_spec * pFilespec = NULL;

	SG_ERR_CHECK(  SG_alloc1(pCtx, pOptSt)  );
	SG_ERR_CHECK(  SG_alloc(pCtx, (*ppGetopt)->count_args, sizeof(SG_int32), &pOptSt->paRecvdOpts)  );

	SG_ERR_CHECK(  SG_REV_SPEC__ALLOC(pCtx, &pOptSt->pRevSpec)  );

	// init default values -- NOTE only need to set things that are non-zero (because of above calloc())
	pOptSt->bRecursive = SG_TRUE;
	pOptSt->bReverseOutput = SG_FALSE;
	pOptSt->bPromptForDescription = SG_TRUE;
	pOptSt->iPort = 8080;
	pOptSt->iSPort = 0;
	pOptSt->iMaxHistoryResults = SG_UINT32_MAX;

	while (1)
	{
		SG_getopt__long(pCtx, *ppGetopt, sg_cl_options, &iOptId, &pszOptArg, &bNoMoreOptions);

		if (bNoMoreOptions)
		{
			SG_context__err_reset(pCtx);
			break;
		}
		else if (SG_context__has_err(pCtx))
		{
			const char* pszErr;
			SG_context__err_get_description(pCtx, &pszErr);
			SG_context__push_level(pCtx);
			SG_console(pCtx, SG_CS_STDERR, "\n");
			SG_console__raw(pCtx, SG_CS_STDERR, pszErr);
			SG_console(pCtx, SG_CS_STDERR, "\n");
			SG_context__pop_level(pCtx);
			SG_ERR_RETHROW;
		}

		pOptSt->optCountArray[iOptId]++;

		pOptSt->paRecvdOpts[pOptSt->count_options] = iOptId;
		pOptSt->count_options++;

		// TODO:  we should probably do some checking per option here
		// i.e. do the strings being passed in look like they should
		// for things like opt_changeset

		switch(iOptId)
		{
		case opt_all:
			{
				pOptSt->bAll = SG_TRUE;
			}
			break;
		case opt_all_full:
			{
				pOptSt->b_all_full = SG_TRUE;
			}
			break;
		case opt_all_zlib:
			{
				pOptSt->b_all_zlib = SG_TRUE;
			}
			break;
		case opt_all_but:
			{
				pOptSt->bAllBut = SG_TRUE;
			}
			break;
		case opt_show_all:
			{
				pOptSt->bShowAll = SG_TRUE;
			}
			break;
		case 'a':
			{
				// if not alloc'ed, alloc
				// add assoc to psa_assocs/increment counter at the end
				if (!pOptSt->psa_assocs)
				{
					SG_ERR_CHECK( SG_STRINGARRAY__ALLOC(pCtx, &pOptSt->psa_assocs, pOptSt->count_options) );
				}
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pOptSt->psa_assocs, pszOptArg)  );
			}
			break;
		case opt_cert:
			{
				if (!pOptSt->psz_cert)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_cert)  );
			}
			break;
		case 'C':
			{
				pOptSt->bClean = SG_TRUE;
			}
			break;
		case opt_dest:
			{
				if (!pOptSt->psz_dest_repo)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_dest_repo)  );
			}
			break;
		case opt_src:
			{
				if (!pOptSt->psz_src_repo)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_src_repo)  );
			}
			break;
		case opt_server:
			{
				if (!pOptSt->psz_server)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_server)  );
			}
			break;
		case opt_verbose:
			{
				if (pOptSt->bQuiet)
					pOptSt->bQuiet = SG_FALSE;
				pOptSt->bVerbose = SG_TRUE;
			}
			break;
		case opt_quiet:
			{
				if (pOptSt->bVerbose)
					pOptSt->bVerbose = SG_FALSE;
				pOptSt->bQuiet = SG_TRUE;
			}
		case opt_pull:
			{
				pOptSt->bPull = SG_TRUE;
			}
			break;
		case 'F':
			{
				pOptSt->bForce = SG_TRUE;
			}
			break;
		case opt_global:
			{
				pOptSt->bGlobal = SG_TRUE;
			}
			break;
		case 'X':
			{
				if (pFilespec == NULL)
				{
					SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
				}
				SG_ERR_CHECK(  SG_file_spec__add_pattern__sz(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__EXCLUDE, pszOptArg, 0u)  );
			}
			break;
		case opt_exclude_from:
			{
				if (pFilespec == NULL)
				{
					SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
				}
				SG_ERR_CHECK(  SG_file_spec__add_patterns__file__sz(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__EXCLUDE, pszOptArg, 0u)  );
			}
			break;
		case opt_no_ignores:
			{
				if (pFilespec == NULL)
				{
					SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
				}
				SG_ERR_CHECK(  SG_file_spec__modify_flags(pCtx, pFilespec, SG_FILE_SPEC__NO_IGNORES, 0u, NULL, NULL)  );
			}
			break;
		case 'I':
			{
				if (pFilespec == NULL)
				{
					SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
				}
				SG_ERR_CHECK(  SG_file_spec__add_pattern__sz(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__INCLUDE, pszOptArg, 0u)  );
			}
			break;
		case opt_include_from:
			{
				if (pFilespec == NULL)
				{
					SG_ERR_CHECK(  SG_FILE_SPEC__ALLOC(pCtx, &pFilespec)  );
				}
				SG_ERR_CHECK(  SG_file_spec__add_patterns__file__sz(pCtx, pFilespec, SG_FILE_SPEC__PATTERN__INCLUDE, pszOptArg, 0u)  );
			}
			break;
		case 'm':
			{
				if (!pOptSt->psz_message)
				{
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_message)  );
					pOptSt->bPromptForDescription = SG_FALSE;
				}
			}
			break;
		case 'N':
			{
				pOptSt->bRecursive = SG_FALSE;
			}
			break;
		case opt_no_plain:
			{
				pOptSt->bNoPlain = SG_TRUE;
			}
			break;
		case opt_no_prompt:
			{
				pOptSt->bNoPrompt = SG_TRUE;
			}
			break;
		case 'P':
			{
				pOptSt->bPublic = SG_TRUE;
			}
			break;
		case opt_port:
			{
				SG_ERR_CHECK(  _parse_option_port(pCtx, "port", pszOptArg, &pOptSt->iPort)  );
			}
			break;
		case opt_create:
			{
				pOptSt->bCreate = SG_TRUE;
			}
			break;
		case opt_subgroups:
			{
				pOptSt->bSubgroups = SG_TRUE;
			}
			break;
		case opt_remove:
			{
				pOptSt->bRemove = SG_TRUE;
			}
			break;
		case opt_detached:
			{
				pOptSt->bAllowDetached = SG_TRUE;
			}
			break;
		case opt_repo:
			{
				if (pOptSt->psz_repo == NULL)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_repo)  );
			}
			break;
		case 'b': // --branch
            {
				/* Add the option for both 'b' and OPT_REV_SPEC. 
				 * Later _inspect_all_options will make sure only the correct one is set. */

				if (pOptSt->psa_branch_names == NULL)
					SG_ERR_CHECK(  SG_STRINGARRAY__ALLOC(pCtx, &pOptSt->psa_branch_names, 2)  );
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pOptSt->psa_branch_names, pszOptArg)  );

				SG_ERR_CHECK(  SG_rev_spec__add_branch(pCtx, pOptSt->pRevSpec, pszOptArg)  );
			}
			break;
		case opt_attach:
            {
				if (pOptSt->psz_attach == NULL)
                {
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_attach)  );
                }
            }
			break;
		case 'r':
			{
				/* Add the option for both 'r' and OPT_REV_SPEC. 
				 * Later _inspect_all_options will make sure only the correct one is set. */
				if (pOptSt->psa_revs == NULL)
					SG_ERR_CHECK(  SG_STRINGARRAY__ALLOC(pCtx, &pOptSt->psa_revs, 2)  );
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pOptSt->psa_revs, pszOptArg)  );

				SG_ERR_CHECK(  SG_rev_spec__add_rev(pCtx, pOptSt->pRevSpec, pszOptArg)  );
			}
			break;
		case opt_area:
			{
				if (!pOptSt->pvh_area_names)
                {
					SG_ERR_CHECK(  SG_VHASH__ALLOC(pCtx, &pOptSt->pvh_area_names)  );
                }
				SG_ERR_CHECK(  SG_vhash__update__null(pCtx, pOptSt->pvh_area_names, pszOptArg)  );
			}
			break;
		case opt_sport:
			{
				SG_ERR_CHECK(  _parse_option_port(pCtx, "sport", pszOptArg, &pOptSt->iSPort)  );
			}
			break;
		case opt_stamp:
			{
				// if not alloc'ed, alloc
				// add stamp to psa_stamps
				if (!pOptSt->psa_stamps)
				{
					SG_ERR_CHECK( SG_STRINGARRAY__ALLOC(pCtx, &pOptSt->psa_stamps, pOptSt->count_options) );
				}
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pOptSt->psa_stamps, pszOptArg)  );
			}
			break;
		case opt_tag:
			{
				/* Add the option for both opt_tag and OPT_REV_SPEC. 
				 * Later _inspect_all_options will make sure only the correct one is set. */
				if (pOptSt->psa_tags == NULL)
					SG_ERR_CHECK(  SG_STRINGARRAY__ALLOC(pCtx, &pOptSt->psa_tags, 2)  );
				SG_ERR_CHECK(  SG_stringarray__add(pCtx, pOptSt->psa_tags, pszOptArg)  );

				SG_ERR_CHECK(  SG_rev_spec__add_tag(pCtx, pOptSt->pRevSpec, pszOptArg)  );
			}
			break;
		case 'T':
			{
				pOptSt->bTest = SG_TRUE;
			}
			break;
		case 'U':
			{
				if (!pOptSt->psz_username)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_username)  );
			}
			break;
		case 'u':
			pOptSt->bUpdate = SG_TRUE;
			break;
		case opt_unified:
			{
				SG_ERR_CHECK(  _parse_option_unified(pCtx, "unified", pszOptArg, &pOptSt->iUnified)  );
			}
			break;
		case opt_to:
			{
				if (!pOptSt->psz_to_date)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_to_date)  );
			}
			break;
		case opt_from:
			{
				if (!pOptSt->psz_from_date)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_from_date)  );
			}
			break;
		case opt_when:
			{
				if (!pOptSt->psz_when)
				SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_when)  );
			}
			break;
		case opt_leaves:
			{
				pOptSt->bLeaves = SG_TRUE;
			}
			break;
		case opt_hide_merges:
			{
				pOptSt->bHideMerges = SG_TRUE;
			}
			break;
		case opt_reverse:
			{
				pOptSt->bReverseOutput = SG_TRUE;
			}
			break;

		case opt_max:
			{
				SG_uint32 parsed = SG_UINT32_MAX;
				// not sure the best way to do this
				SG_ERR_CHECK(SG_uint32__parse__strict(pCtx, &parsed, pszOptArg));
				pOptSt->iMaxHistoryResults = parsed;
			}
			break;

		case opt_list:
			{
				pOptSt->bList = SG_TRUE;
			}
			break;

		case opt_list_all:
			{
				pOptSt->bListAll = SG_TRUE;
			}
			break;

		case opt_mark:
			{
				pOptSt->bMark = SG_TRUE;
			}
			break;

		case opt_mark_all:
			{
				pOptSt->bMarkAll = SG_TRUE;
			}
			break;

		case opt_remerge:
			{
				pOptSt->bRemerge = SG_TRUE;
			}
			break;

		case opt_tool:
			{
				if (!pOptSt->psz_tool)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_tool)  );
			}
			break;

		case opt_no_auto_merge:
			{
				pOptSt->bNoAutoMerge = SG_TRUE;
			}
			break;

		case opt_storage:
			{
				if (!pOptSt->psz_storage)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_storage)  );
			}
			break;

		case opt_shared_users:
			{
				if (!pOptSt->psz_shared_users)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_shared_users)  );
			}
			break;

		case opt_hash:
			{
				if (!pOptSt->psz_hash)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_hash)  );
			}
			break;

		case opt_submodule:
			{
				pOptSt->bSubmodule = SG_TRUE;
			}
			break;
		case opt_no_submodules:
			{
				pOptSt->bNoSubmodules = SG_TRUE;
			}
			break;
		case opt_use_submodule:
			{
				char * pszUseSub = NULL;

				if (!pOptSt->pvec_use_submodules)
					SG_ERR_CHECK(  SG_VECTOR__ALLOC(pCtx, &pOptSt->pvec_use_submodules, pOptSt->count_options)  );
				
				SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pszUseSub)  );
				SG_ERR_CHECK(  SG_vector__append(pCtx, pOptSt->pvec_use_submodules, pszUseSub, NULL)  );
			}
			break;

		case opt_pack:
			{
				pOptSt->bPack = SG_TRUE;
			}
			break;

		case opt_no_backups:
			{
				pOptSt->bNoBackups = SG_TRUE;
			}
			break;

		case 'i':
			{
				pOptSt->bInteractive = SG_TRUE;
			}
			break;

		case opt_overwrite:
			{
				pOptSt->bOverwrite = SG_TRUE;
			}
			break;

		case opt_label:
			{
				if (!pOptSt->psz_label)
					SG_ERR_CHECK(  SG_STRDUP(pCtx, pszOptArg, &pOptSt->psz_label)  );
			}
			break;

#if defined(DEBUG)
		case opt_dagnum:
			{
				SG_ERR_CHECK(  SG_dagnum__from_sz__hex(pCtx, pszOptArg, &pOptSt->ui64DagNum)  );
				pOptSt->bGaveDagNum = SG_TRUE;
			}
			break;
#endif

		case opt_trace:
			{
				pOptSt->bTrace = SG_TRUE;
			}
			break;

		default:
			break;
		}

	}

    if(pOptSt->psa_assocs!=NULL)
        SG_ERR_CHECK(  SG_stringarray__sz_array_and_count(pCtx, pOptSt->psa_assocs, &pOptSt->ppAssocs, &pOptSt->iCountAssocs)  );

	if (pFilespec == NULL)
	{
		pOptSt->pFilespec = gpBlankFilespec;
	}
	else
	{
		pOptSt->pFilespec = pFilespec;
	}

	*ppOptSt = pOptSt;
	return;
fail:
	SG_FILE_SPEC_NULLFREE(pCtx, pFilespec);
	SG_ERR_IGNORE(  _sg_option_state__free(pCtx, pOptSt)  );
}

//////////////////////////////////////////////////////////////////

/**
 * Dispatch to appropriate command using the given SG_getopt.
 *
 */
static void _dispatch(SG_context * pCtx, const char *szCmd, const char *szAppName, const cmdinfo *pCmdInfo, SG_getopt * pGetopt, SG_option_state* pOptSt, SG_bool initialUsageError, int * pExitStatus)
{
	if (pCmdInfo)
	{
		SG_bool	bUsageError = initialUsageError;

		SG_ERR_IGNORE(  SG_log__report_verbose(pCtx, "Performing '%s' command.", szCmd)  );

		// TODO consider just passing pArgv to the command functions.
		if (! bUsageError)
			SG_ERR_CHECK(  _inspect_all_options(pCtx, szAppName, szCmd, pOptSt, &bUsageError)  );

		if (! bUsageError)
		{
			SG_ERR_CHECK(  pCmdInfo->fn(
							   pCtx,
							   szAppName,
							   szCmd,
							   pGetopt,
							   pOptSt,
							   &bUsageError,
							   pExitStatus)
				);
		}

		if (bUsageError)
		{
			SG_ERR_THROW(SG_ERR_USAGE);
		}

		return;
	}

	SG_ERR_CHECK(  SG_console(pCtx, SG_CS_STDERR, "\nInvalid command: %s\n\n", szCmd));

	do_cmd_help__list_all_commands(pCtx, SG_CS_STDERR, szAppName, pOptSt);

	SG_ERR_THROW_RETURN(  SG_ERR_USAGE  );

fail:
	if (SG_context__err_equals(pCtx, SG_ERR_USAGE))
	{
		const char *szMessage = NULL;
		SG_error err;
		const char *szSubcommand = NULL;

		err = SG_context__err_get_description(pCtx, &szMessage);

		if (SG_IS_OK(err) && szMessage && *szMessage)
		{
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "%s", szMessage)  );
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "\n")  );
		}

		// Figure out if they used a subcommand that we have specifc help text for:
		if(pGetopt && pGetopt->count_args>=3)
		{
			SG_uint32 j;
			const cmdinfo * pCmdCanonical = NULL;
			_get_canonical_command(szCmd, &pCmdCanonical);
			for(j=0;j<SUBCMDCOUNT && !szSubcommand;++j)
				if(!strcmp(pCmdCanonical->name, subcommands[j].cmdname) && !strcmp(pGetopt->paszArgs[2], subcommands[j].name))
					szSubcommand = pGetopt->paszArgs[2];
		}

		SG_ERR_IGNORE(  do_cmd_help__list_command_help(pCtx, SG_CS_STDERR, szAppName, szCmd, szSubcommand, SG_FALSE)  );
	}
}

//////////////////////////////////////////////////////////////////

/**
 * Print error message on STDERR for the given error and convert the
 * SG_ERR_ value into a proper ExitStatus for the platform.
 *
 * TODO investigate if (-1,errno) is appropriate for all 3 platforms.
 *
 * DO NOT WRAP THE CALL TO THIS FUNCTION WITH SG_ERR_IGNORE() (because we
 * need to be able to access the original error/status).
 */
static void _compute_exit_status_and_print_error_message(SG_context * pCtx, SG_getopt * pGetopt, int * pExitStatus)
{
	SG_error err;
	char bufErrorMessage[SG_ERROR_BUFFER_SIZE+1];

	SG_ASSERT(pExitStatus!=NULL);

	SG_context__get_err(pCtx,&err);

	if (err == SG_ERR_OK)
	{
		SG_int_to_string_buffer tmp;
		if(*pExitStatus==0)
			SG_ERR_IGNORE(  SG_log__report_verbose(pCtx, "Command '%s' completed normally."SG_PLATFORM_NATIVE_EOL_STR SG_PLATFORM_NATIVE_EOL_STR, SG_getopt__get_command_name(pGetopt))  );
		else
			SG_ERR_IGNORE(  SG_log__report_error(pCtx, "Command '%s' completed with exit status %s."SG_PLATFORM_NATIVE_EOL_STR SG_PLATFORM_NATIVE_EOL_STR, SG_getopt__get_command_name(pGetopt), SG_int64_to_sz(*pExitStatus, tmp))  );
	}
	else if (err == SG_ERR_USAGE)
	{
#if 1
		// TODO 2010/05/13 We need to define a set of exit codes
		// TODO            for all of the commands in vv.
		// TODO            Something like:
		// TODO            0 : ok
		// TODO            1 : fail
		// TODO            2 : usage or other error
		// TODO
		// TODO            Like how /bin/diff returns 0,1,2.
#endif
		// We special case SG_ERR_USAGE because the code that signalled that
		// error has already printed whatever context-specific usage info.

		*pExitStatus = -2;  //Don't return -1, as exec() assumes that -1 means the exe could not be found.
	}
	else
	{
		SG_string * pStrContextDump = NULL;

		SG_error__get_message(err, SG_TRUE, bufErrorMessage, SG_ERROR_BUFFER_SIZE);

		// try to write full stacktrace and everything from the context to the log.
		// if we can't get that, just write the short form.
		//
		// TODO consider having the log_debug level do the full dump and the
		// TODO terser levels just do the summary.

		SG_context__err_to_string(pCtx, SG_TRUE, &pStrContextDump);

		SG_ERR_IGNORE(  SG_log__report_error(pCtx,
			"Command '%s' completed with error:" SG_PLATFORM_NATIVE_EOL_STR "%s"SG_PLATFORM_NATIVE_EOL_STR SG_PLATFORM_NATIVE_EOL_STR,
									 SG_getopt__get_command_name(pGetopt),
									 (  (pStrContextDump)
										? SG_string__sz(pStrContextDump)
										: bufErrorMessage))  );

#if defined(DEBUG)
		/* Print all the gory details to the console in debug builds. */
		SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "%s: %s\n",
								   SG_getopt__get_app_name(pGetopt),
								   (  (pStrContextDump)
										? SG_string__sz(pStrContextDump)
										: bufErrorMessage))  );
#else
		/* Print strictly the error message in release builds. The details are in the log. */
		SG_STRING_NULLFREE(pCtx, pStrContextDump);
		SG_context__err_to_string(pCtx, SG_FALSE, &pStrContextDump);
		if (pStrContextDump)
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "%s\n", SG_string__sz(pStrContextDump))  );
		else
		{
			SG_error__get_message(err, SG_FALSE, bufErrorMessage, SG_ERROR_BUFFER_SIZE);
			SG_ERR_IGNORE(  SG_console(pCtx, SG_CS_STDERR, "%s\n", bufErrorMessage)  );
		}
#endif

		SG_STRING_NULLFREE(pCtx, pStrContextDump);

		// the shell and command tools in general only expect ERRNO-type errors.

		if (SG_ERR_TYPE(err) == __SG_ERR__ERRNO__)
			*pExitStatus = SG_ERR_ERRNO_VALUE(err);
		else
			*pExitStatus = 1;  // this means we threw an error, there's no unix error code that will
							 // cover all the possible errors that could come out of vv, so this
							 // one will have to do (EPERM)
	}
}

//////////////////////////////////////////////////////////////////

static int _my_main(SG_context * pCtx, SG_getopt * pGetopt, SG_option_state* pOptSt, SG_bool bUsageError)
{
	int exitStatus = 0;

	SG_pathname*                          pLogPath           = NULL;
	char*                                 szLogPath          = NULL;
	char*                                 szLogLevel         = NULL;
	SG_log_console__data                  cLogStdData;
	SG_log_text__data                     cLogFileData;
	SG_log_text__writer__daily_path__data cLogFileWriterData;
	SG_uint32							  logFileFlags = SG_LOG__FLAG__NONE;
	const char *szCmd = NULL;
	const char *szAppName = NULL;
	const cmdinfo * pCmdInfo = NULL;
	SG_bool bIsServer = SG_FALSE;
	SG_string * pLogFileHeader = NULL;
	char * szSetting = NULL;

	SG_zero(cLogStdData);
	SG_zero(cLogFileData);
	SG_zero(cLogFileWriterData);

	// find the appropriate log path
	SG_ERR_CHECK(  SG_localsettings__get__sz(pCtx, SG_LOCALSETTING__LOG_PATH, NULL, &szLogPath, NULL)  );
	if (szLogPath != NULL)
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__SZ(pCtx, &pLogPath, szLogPath)  );
	}
	else
	{
		SG_ERR_CHECK(  SG_PATHNAME__ALLOC__LOG_DIRECTORY(pCtx, &pLogPath)  );
	}

	// get the configured log level
	SG_ERR_CHECK(  SG_localsettings__get__sz(pCtx, SG_LOCALSETTING__LOG_LEVEL, NULL, &szLogLevel, NULL)  );

	// register the stdout console logger
	SG_ERR_CHECK(  SG_log_console__set_defaults(pCtx, &cLogStdData)  );
	SG_ERR_CHECK(  SG_log_console__register(pCtx, &cLogStdData, SG_LOG__FLAG__HANDLER_TYPE__NORMAL)  );

	//Get the canonical command.  We do this to check for vv serve before we configure the logger.
	szCmd = SG_getopt__get_command_name(pGetopt);
	szAppName = SG_getopt__get_app_name(pGetopt);

	_get_canonical_command(szCmd, &pCmdInfo);

	if (pCmdInfo && strcmp(pCmdInfo->name, "serve") == 0)
	{
		bIsServer = SG_TRUE;
	}
	// register the file logger
	SG_ERR_CHECK(  SG_log_text__set_defaults(pCtx, &cLogFileData)  );
	cLogFileData.fWriter             = SG_log_text__writer__daily_path;
	cLogFileData.pWriterData         = &cLogFileWriterData;
	cLogFileData.szRegisterMessage   = NULL;
	cLogFileData.szUnregisterMessage = NULL;
	if (szLogLevel != NULL)
	{
		if (SG_stricmp(szLogLevel, "quiet") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__QUIET;
			cLogFileData.bLogVerboseOperations = SG_FALSE;
			cLogFileData.bLogVerboseValues     = SG_FALSE;
		}
		else if (SG_stricmp(szLogLevel, "normal") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__NORMAL;
			cLogFileData.bLogVerboseOperations = SG_FALSE;
			cLogFileData.bLogVerboseValues     = SG_FALSE;
		}
		else if (SG_stricmp(szLogLevel, "verbose") == 0)
		{
			logFileFlags = SG_LOG__FLAG__HANDLER_TYPE__ALL;
			cLogFileData.szRegisterMessage   = "---- " WHATSMYNAME " started logging ----";
			cLogFileData.szUnregisterMessage = "---- " WHATSMYNAME " stopped logging ----";
		}
	}
	SG_ERR_CHECK(  SG_log_text__writer__daily_path__set_defaults(pCtx, &cLogFileWriterData)  );
	cLogFileWriterData.bReopen          = SG_FALSE;
	cLogFileWriterData.ePermissions     = 0666;
	cLogFileWriterData.pBasePath        = pLogPath;
	if (bIsServer != SG_TRUE)
		cLogFileWriterData.szFilenameFormat = "vv-%d-%02d-%02d.log";
	else
	{
		const char * szVersion = NULL;
		const char * settings[] = {
			SG_LOCALSETTING__SERVER_FILES,
			"server/cache",
			"server/session_storage",
			SG_LOCALSETTING__SERVER_READONLY,
			SG_LOCALSETTING__SERVER_HOSTNAME,
			SG_LOCALSETTING__SERVER_SCHEME,
			SG_LOCALSETTING__SERVER_DEBUG_DELAY,
			SG_LOCALSETTING__SERVER_ENABLE_DIAGNOSTICS,
			SG_LOCALSETTING__SERVER_SSJS_MUTABLE};
		SG_uint32 i;

		cLogFileWriterData.szFilenameFormat = "vv-serve-%d-%02d-%02d.log";

		SG_ERR_CHECK(  SG_lib__version(pCtx, &szVersion)  );
		SG_ERR_CHECK(  SG_string__alloc__format(pCtx, &pLogFileHeader,
			"SourceGear Veracity" SG_PLATFORM_NATIVE_EOL_STR
			"Version %s" SG_PLATFORM_NATIVE_EOL_STR,
			szVersion)  );
		for(i=0; i<SG_NrElements(settings); ++i)
		{
			SG_ERR_CHECK(  SG_localsettings__get__sz(pCtx, settings[i], NULL, &szSetting, NULL)  );
			if(szSetting!=NULL)
				SG_ERR_CHECK(  SG_string__append__format(pCtx, pLogFileHeader, "%s = \"%s\"" SG_PLATFORM_NATIVE_EOL_STR, settings[i], szSetting)  );
			else if(i<4)
				SG_ERR_CHECK(  SG_string__append__format(pCtx, pLogFileHeader, "%s = <default>" SG_PLATFORM_NATIVE_EOL_STR, settings[i])  );
			SG_NULLFREE(pCtx, szSetting);
		}
		SG_ERR_CHECK(  SG_string__append__format(pCtx, pLogFileHeader, SG_PLATFORM_NATIVE_EOL_STR)  );
		cLogFileWriterData.szHeader = SG_string__sz(pLogFileHeader);
	}

	SG_ERR_CHECK(  SG_log_text__register(pCtx, &cLogFileData, logFileFlags)  );

	// dispatch the command
	SG_ERR_CHECK(  _dispatch(pCtx, szCmd, szAppName, pCmdInfo, pGetopt, pOptSt, bUsageError, &exitStatus)  );

fail:
	_compute_exit_status_and_print_error_message(pCtx, pGetopt, &exitStatus); // DO NOT WRAP THIS WITH SG_ERR_IGNORE
	SG_ERR_IGNORE(  SG_log_text__unregister(pCtx, &cLogFileData)  );
	SG_ERR_IGNORE(  SG_log_console__unregister(pCtx, &cLogStdData)  );
	SG_NULLFREE(pCtx, szLogLevel);
	SG_NULLFREE(pCtx, szLogPath);
	SG_PATHNAME_NULLFREE(pCtx, pLogPath); // must come after cLogFileData is unregistered, because it uses pLogPath
	SG_STRING_NULLFREE(pCtx, pLogFileHeader);
	SG_NULLFREE(pCtx, szSetting);
	return exitStatus;
}

/**
 * A simple wrapper around SG_free() because it is a macro
 * rather than a function pointer and I can reference it from
 * SG_vector__free__with_assoc() call below.
 */
static SG_free_callback _my_sg_free;
static void _my_sg_free(SG_context * pCtx, void * pVoid)
{
	SG_free(pCtx, pVoid);
}

static void _sg_option_state__free(SG_context* pCtx, SG_option_state* pThis)
{
	if (!pThis)
		return;

	SG_NULLFREE(pCtx, pThis->psz_repo);
	SG_NULLFREE(pCtx, pThis->psz_cert);
	SG_NULLFREE(pCtx, pThis->psz_comment);
	SG_NULLFREE(pCtx, pThis->psz_dest_repo);
	SG_NULLFREE(pCtx, pThis->psz_src_repo);
	SG_NULLFREE(pCtx, pThis->psz_server);
	SG_NULLFREE(pCtx, pThis->psz_message);
	SG_NULLFREE(pCtx, pThis->psz_username);
	SG_NULLFREE(pCtx, pThis->psz_when);
	SG_NULLFREE(pCtx, pThis->psz_from_date);
	SG_NULLFREE(pCtx, pThis->psz_to_date);
	SG_NULLFREE(pCtx, pThis->psz_tool);
	SG_NULLFREE(pCtx, pThis->psz_storage);
	SG_NULLFREE(pCtx, pThis->psz_hash);
	SG_NULLFREE(pCtx, pThis->psz_shared_users);
	SG_NULLFREE(pCtx, pThis->psz_attach);
	SG_NULLFREE(pCtx, pThis->psz_label);

	SG_STRINGARRAY_NULLFREE(pCtx, pThis->psa_stamps);

	SG_VHASH_NULLFREE(pCtx, pThis->pvh_area_names);

	SG_NULLFREE(pCtx, pThis->paRecvdOpts);

	SG_STRINGARRAY_NULLFREE(pCtx, pThis->psa_revs);
	SG_STRINGARRAY_NULLFREE(pCtx, pThis->psa_tags);
	SG_STRINGARRAY_NULLFREE(pCtx, pThis->psa_branch_names);

	SG_REV_SPEC_NULLFREE(pCtx, pThis->pRevSpec);

	SG_vector__free__with_assoc(pCtx, pThis->pvec_use_submodules, _my_sg_free);

    SG_STRINGARRAY_NULLFREE(pCtx, pThis->psa_assocs);

	SG_FILE_SPEC_NULLFREE(pCtx, pThis->pFilespec);

	SG_NULLFREE(pCtx, pThis);
}

//////////////////////////////////////////////////////////////////

#if defined(WINDOWS)
int wmain(int argc, wchar_t ** argv)
#else
int main(int argc, char ** argv)
#endif
{
	SG_context * pCtx = NULL;
	SG_getopt * pGetopt = NULL;
	SG_option_state * pOptSt = NULL;
	int exitStatus = 0;
	SG_bool bUsageError = SG_FALSE;

	SG_error err = SG_context__alloc(&pCtx);
	if (SG_IS_ERROR(err))
		return -2;
	
	SG_ERR_CHECK(  SG_lib__global_initialize(pCtx)  );

	SG_ERR_CHECK(  _intern_argv(pCtx, argc, argv, &pGetopt)  );

	_parse_options(pCtx, &pGetopt, &pOptSt);

	if (SG_context__has_err(pCtx))
	{
		if (SG_context__err_equals(pCtx, SG_ERR_GETOPT_BAD_ARG))
		{
			bUsageError = SG_TRUE;
			SG_context__err_reset(pCtx);
		}
		else
		{
			_compute_exit_status_and_print_error_message(pCtx, pGetopt, &exitStatus); // DO NOT WRAP THIS WITH SG_ERR_IGNORE
			SG_ERR_RETHROW;
		}
	}

	SG_ERR_CHECK(  exitStatus = _my_main(pCtx, pGetopt, pOptSt, bUsageError)  );

fail:
	SG_GETOPT_NULLFREE(pCtx, pGetopt);
	SG_ERR_IGNORE(  _sg_option_state__free(pCtx, pOptSt)  );

	SG_ERR_IGNORE(  SG_lib__global_cleanup(pCtx)  );
	SG_CONTEXT_NULLFREE(pCtx);
	return exitStatus;
}

